/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Osamu Sunohara (SONY:Studio Profile)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "momusys.h"
#include "vm_common_defs.h"

Int
CalcFieldParity_Studio(Int vop_structure,
		       Int vop_counter,
		       Int vop_top_field_first)
{
  Int vop_field_parity = 0;

  if(vop_structure == FIELD_VOP && (
     (vop_top_field_first == 0 && vop_counter == 0) ||
     (vop_top_field_first == 1 && vop_counter == 1)))
    vop_field_parity = 1;

  return(vop_field_parity);
}

Void CalcFieldMemoryPosition_Studio(Int vop_structure, Int field_parity,
				    Int x_position, Int y_position, Int vop_width,
				    Int *memory_position) {
  if(vop_structure == FRAME_VOP ) {
    memory_position[0] = y_position%2;
    memory_position[1] = (y_position/2)*vop_width + x_position;
  } else {
    memory_position[0] = field_parity;
    memory_position[1] = y_position*vop_width + x_position;
  }
}

Void ChangeRectFrameVOPtoFieldVOPs_Studio(Vop * vop) {
  Int i;
  Int width, height;
  Int x, y;

  Image *tmp_image;

  Int j;
  
  if(GetVopVopStructure_st(vop)!=FRAME_VOP) {
    fprintf(stderr, "This function is for Frame VOP.\n");
    exit(3);
  }
  width  = GetVopWidth_st (vop,0);
  height = GetVopHeight_st(vop,0);
  if((height%2) == 1) {
    fprintf(stderr, "height isn't even value.\n");
    exit(3);
  }

  PutVopVopStructure_st(FIELD_VOP, vop);
  x = width;
  y = height/2;
  for(i=0;i<2;i++) {
    PutVopWidth_st (x, vop, i);
    PutVopHeight_st(y, vop, i);
    PutVopHorSpatRef_st(0, vop, i);
    PutVopVerSpatRef_st(0, vop, i);
    
    if((tmp_image=GetVopQP_st(vop,i)) != NULL)
      FreeImage(tmp_image);
    tmp_image = AllocImage(x/16+((x/16)?0:1),y/16+((y/16)?0:1),SHORT_TYPE);
    PutVopQP_st(tmp_image,vop,i);

    for(j=0;j<MAX_AUX_COMP_NUM;j++) {
      if((tmp_image=GetVopGLQP_st(vop,j,i)) != NULL)
	FreeImage(tmp_image);
      tmp_image = AllocImage(x/16+((x/16)?0:1),y/16+((y/16)?0:1),SHORT_TYPE);
      PutVopGLQP_st(tmp_image,vop,j,i);
    }
    
    if((tmp_image=GetVopShapeMode_st(vop,i)) != NULL)
      FreeImage(tmp_image);
    tmp_image = AllocImage(x/16+((x/16)?0:1),y/16+((y/16)?0:1),SHORT_TYPE);
    PutVopShapeMode_st(tmp_image,vop,i);
  }
}
  
Void ChangeRectFieldVOPstoFrameVOP_Studio(Vop * vop) {
  Int i;
  Int width, height;
  Int x, y;

  Image *tmp_image;

  Int j;
  
  if(GetVopVopStructure_st(vop)!=FIELD_VOP) {
    fprintf(stderr, "This function is for Field VOP.\n");
    exit(3);
  }
  
  if((width  = GetVopWidth_st (vop,0)) != GetVopWidth_st (vop,1) ||
     (height = GetVopHeight_st(vop,0)) != GetVopHeight_st(vop,1)) {
    fprintf(stderr, "both fields have the different size.");
    exit(3);
  }

  x = width;
  y = height * 2;

  PutVopWidth_st (x, vop, 0);
  PutVopHeight_st(y, vop, 0);
  PutVopWidth_st (0, vop, 1);
  PutVopHeight_st(0, vop, 1);
  PutVopHorSpatRef_st(0, vop, 0);
  PutVopVerSpatRef_st(0, vop, 0);
  PutVopHorSpatRef_st(0, vop, 1);
  PutVopVerSpatRef_st(0, vop, 1);

  PutVopVopStructure_st(FRAME_VOP, vop);

  for(i=0;i<2;i++) {
    if((tmp_image=GetVopQP_st(vop,i)) != NULL)
      FreeImage(tmp_image);

    for(j=0;j<MAX_AUX_COMP_NUM;j++) {
      if((tmp_image=GetVopGLQP_st(vop,j,i)) != NULL)
	FreeImage(tmp_image);
    }
    
    if((tmp_image=GetVopShapeMode_st(vop,i)) != NULL)
      FreeImage(tmp_image);
  }

  tmp_image = AllocImage(x/16+((x/16)?0:1),y/16+((y/16)?0:1),SHORT_TYPE);
  PutVopQP_st(tmp_image,vop,0);
  
  for(j=0;j<MAX_AUX_COMP_NUM;j++) {
    tmp_image = AllocImage(x/16+((x/16)?0:1),y/16+((y/16)?0:1),SHORT_TYPE);
    PutVopGLQP_st(tmp_image,vop,j,0);
  }
  
  tmp_image = AllocImage(x/16+((x/16)?0:1),y/16+((y/16)?0:1),SHORT_TYPE);
  PutVopShapeMode_st(tmp_image,vop,0);
}
  
SInt GetImageDataFromFieldMemory(SInt *memory[],
				   int x_pos, int y_pos, int width,
				   int vop_structure, int field_parity)
{
  if(vop_structure == FRAME_VOP)
    return memory[y_pos%2][(y_pos/2) * width + x_pos];
  else
    return memory[field_parity][y_pos * width + x_pos];
}

void PutImageDataToFieldMemory(SInt *memory[],
			       int x_pos, int y_pos, int width,
			       int vop_structure, int field_parity,
			       SInt val)
{
  if(vop_structure == FRAME_VOP)
    memory[y_pos%2][(y_pos/2) * width + x_pos] = val;
  else
    memory[field_parity][y_pos * width + x_pos] = val;
}
