/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Osamu Sunohara (SONY:Studio Profile)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "momusys.h"
#include "vm_common_defs.h"

Int CalcQuantizerScale_Studio(Int q_scale_type, Int quantizer_scale_code) {
  Int quantizer_scale;

  if(quantizer_scale_code<1 || quantizer_scale_code>31) {
    fprintf(stderr,"quantizer_scale_code has the wrong value.\n");
    exit(3);
  }
  
  if(q_scale_type == 0)
    quantizer_scale = 2 * quantizer_scale_code;
  else {
    if(quantizer_scale_code < 9)
      quantizer_scale = quantizer_scale_code;
    else if(quantizer_scale_code < 17)
      quantizer_scale = 2 * (quantizer_scale_code - 9) + 10;
    else if(quantizer_scale_code < 25)
      quantizer_scale = 4 * (quantizer_scale_code - 17) + 28;
    else
      quantizer_scale = 8 * (quantizer_scale_code - 25) + 64;
  }

  return(quantizer_scale);
}

Int GetMBDQuant_Studio(Int quantizer_scale_code, Int random) {
  Int DQ = 0;
  
  if(random == 1)
    DQ = GetRandomValue_Studio(-5, 5);
  if(quantizer_scale_code+DQ < 1 || quantizer_scale_code+DQ > 31)
    DQ = 0;
  return DQ;
}
    
Int GetRandomValue_Studio(Int min, Int max) {
  double rv = (double) ((double)rand() / (double)(RAND_MAX));

  /* modified at 020308 BUGFIX */
  Int range = max - min/* + 1*/;

  Int result;

  result = (Int) (range * rv + 0.5);
  result += min;

  if(result < min)
    result = min;
  else if(result > max)
    result = max;

  return(result);
}

Int GetMaxQDC_Studio(Int bits_per_pixel, Int intra_dc_precision) {
  return ((1 << (bits_per_pixel + intra_dc_precision)) - 1);
}

Int GetMaxQDC_DCText_Studio(Int bits_per_pixel, Int dct_precision, Int intra_dc_precision) {
  return ((1 << (bits_per_pixel + dct_precision + intra_dc_precision)) - 1);
}
