/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Osamu Sunohara (SONY:Studio Profile)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/
#include <stdio.h>
#include "studio_sprite_defocus_filter.h"

Void    defocus_filter_Studio( Vop *filter_vop ) 
{
  Int   cntl_param = GetVopDefocusingControlParameter_st( filter_vop ); 
  Int   chroma = GetVopChromaFormat_st( filter_vop ); 
  Vop*  tmp_vop; 
  SInt  *data = NULL; 
  SInt  *ptr[2] = {NULL,NULL}; 
  SInt  *a_ptr[2] = {NULL,NULL}; 
  Int   pels, lines; 
  Int   x, y; 
  Int   i, j, n_x, n_y; 
  Int   off_x; 
  Int   ext; 
  Int   cdivx = 0; 
  Int   cdivy = 0; 
  int binary_number;

  if ( cntl_param == 0 ) 
    return; 
  
  if ( chroma == CHROMA444 )       /* 4:4:4 format */
    {
      cdivx = 1;        cdivy = 1; 
    }
  else if ( chroma == CHROMA422 )  /* 4:2:2 format */
    {
      cdivx = 2;        cdivy = 1; 
    }
  else                          /* 4:2:0 format */
    {
      cdivx = 2;        cdivy = 2; 
    }

  binary_number = 0;
  if(GetVopShape(filter_vop) == GREY_SCALE) {
    for(i=0;i<MAX_AUX_COMP_NUM;i++) {
      if(GetVopAuxCompType_st(filter_vop,i) == AUX_COMP_ALPHA) {
        binary_number = i;
        break;
      }
    }
  }

  fprintf(stdout, "Defocus filter binary_number = %d\n", binary_number);
  fprintf(stdout, "DefocusingControlParameter = %d\n", cntl_param);
  tmp_vop = CloneVop_Studio( filter_vop,FRAME_VOP,0,0 ); 
  pels  = (Int)GetVopWidth_st(filter_vop,0); 
  lines = (Int)GetVopHeight_st(filter_vop,0); 
  x = pels + 2*cntl_param; 
  y = lines + 2*cntl_param; 

  ext = (cntl_param+15)/16*16; 
  DefocusPadding( tmp_vop, chroma, ext ); 
  WriteRectFrameVopGeneric_Studio(tmp_vop,
				  "defpad",
				  "defpad",
				  "defpad",
				  "aa",
				  "bb",
				  0,
				  " ",
				  "OVERWRITE",
				  0);
  data = (SInt *)malloc( x*y * sizeof(SInt)); 
  n_x = ext - cntl_param; 
  n_y = ext - cntl_param; 
fprintf(stdout,"Y n_x,n_y = %d %d\n", n_x, n_y);
  /* Luma */

  
  for(i=0;i<2;i++)
    ptr[i] = (SInt *)GetImageData(GetVopY_st(tmp_vop,i));

  for ( j=0 ; j<y ; j++ ) {
      for ( i=0 ; i<x ; i++ ) {
	data[j*x+i] = GetImageDataFromFieldMemory(ptr, n_x+i, n_y+j, pels+2*ext,
						  FRAME_VOP, 0);
      }
    }

  for(i=0;i<2;i++) {
    a_ptr[i] = (SInt *)GetImageData(GetVopA_st(filter_vop,binary_number,i)); 
    ptr[i]   = (SInt *)GetImageData(GetVopY_st(filter_vop,i));
  }
  for ( j=0 ; j<lines ; j++ ) 
    for ( i=0 ; i<pels ; i++ ) 
      {
	if ( GetImageDataFromFieldMemory(a_ptr, i, j, pels, FRAME_VOP, 0)/**a_ptr++*/ ) 
	  PutImageDataToFieldMemory(ptr, i, j, pels, FRAME_VOP, 0,
				    filter_core( data, i, j, x, cntl_param, cntl_param));
      }

  /* Chroma U */
  for(i=0;i<2;i++)
    a_ptr[i] = (SInt *)GetImageData(GetVopAuv_st(filter_vop,binary_number,i));
  
  pels /= cdivx;
  lines /= cdivy;
  x /= cdivx; 
  y /= cdivy;


  for(i=0;i<2;i++)
    ptr[i] = (SInt *)GetImageData(GetVopU_st(tmp_vop,i)); 
  n_x = (ext - cntl_param)/cdivx; 
  n_y = (ext - cntl_param)/cdivy; 
fprintf(stdout,"C n_x,n_y = %d %d\n", n_x, n_y);

  for ( j=0 ; j<y ; j++ ) {
    for ( i=0 ; i<x ; i++ ) {
      data[j*x+i] = GetImageDataFromFieldMemory(ptr, n_x+i, n_y+j, pels+2*ext/cdivx,
						  FRAME_VOP, 0);
    }
  }

  for(i=0;i<2;i++)
    ptr[i] = (SInt *)GetImageData(GetVopU_st(filter_vop,i)); 
  for ( j=0 ; j<lines ; j++ ) 
    for ( i=0 ; i<pels ; i++ ) {
      if ( GetImageDataFromFieldMemory(a_ptr, i, j, pels, FRAME_VOP, 0) ) 
	PutImageDataToFieldMemory(ptr, i, j, pels, FRAME_VOP, 0,
				    filter_core( data, i, j, x, cntl_param/cdivx, cntl_param/cdivy ));
    }
  
  /* Chroma V */
  for(i=0;i<2;i++)
    a_ptr[i] = (SInt *)GetImageData(GetVopAuv_st(filter_vop,binary_number,i));
  for(i=0;i<2;i++)
    ptr[i] = (SInt *)GetImageData(GetVopV_st(tmp_vop,i)); 

  for ( j=0 ; j<y ; j++ ) {
    for ( i=0 ; i<x ; i++ ) {
      data[j*x+i] = GetImageDataFromFieldMemory(ptr, n_x+i, n_y+j, pels+2*ext/cdivx,
						  FRAME_VOP, 0);
    }
  }

  for(i=0;i<2;i++)
    ptr[i] = (SInt *)GetImageData(GetVopV_st(filter_vop,i)); 
  for ( j=0 ; j<lines ; j++ ) 
    for ( i=0 ; i<pels ; i++ ) {
      if ( GetImageDataFromFieldMemory(a_ptr, i, j, pels, FRAME_VOP, 0) ) 
	PutImageDataToFieldMemory(ptr, i, j, pels, FRAME_VOP, 0,
				  filter_core( data, i, j, x, cntl_param/cdivx, cntl_param/cdivy ));
    }
  
  free(data);
  FreeVop( tmp_vop ); 
}

SInt    filter_core( SInt data[], Int x, Int y, Int xx, Int off_set_x, Int off_set_y ) 
{
        Int     x_size = 2*off_set_x+1;
        Int     y_size = 2*off_set_y+1;
        Int     mat_count = x_size*y_size;
        Int     sum = 0;
	int i, j;

        for( i=0 ; i<y_size ; i++ )
	  for( j=0 ; j<x_size ; j++ ) {
	    sum += data[(xx*y+x)+i*xx+j];
	  }
        return  (sum/mat_count);
}

Void DefocusPadding( Vop* curr_vop, Int  chroma_format, Int edge )
{
  Vop   *curr_vop_padded;
  Int   i,j;

  int vop_structure = GetVopVopStructure_st(curr_vop);
  int orig_width  = GetVopWidth_st (curr_vop,0);
  int orig_height = GetVopHeight_st(curr_vop,0);

  int cdivx = 1, cdivy = 1;

  if(chroma_format==CHROMA422)
    cdivx <<= 1;
  else if(chroma_format==CHROMA420) {
    cdivx <<= 1;
    cdivy <<= 1;
  }

  if(vop_structure != FRAME_VOP) {
    fprintf(stderr, "\nFrameVopPadding\n");
    exit(0);
  }

  curr_vop_padded = AllocFrameVop_Studio();
  AllocVopChannels_Studio(curr_vop_padded, FRAME_VOP, 0,
                          orig_width +2*edge,
                          orig_height+2*edge,
                          chroma_format);
  curr_vop_padded->hor_spat_ref=curr_vop->hor_spat_ref-edge;
  curr_vop_padded->ver_spat_ref=curr_vop->ver_spat_ref-edge;

  /* Store width and height as integer multiples of 16 */
  PutVopWidth_st (orig_width +2*edge, curr_vop_padded, 0);
  PutVopHeight_st(orig_height+2*edge, curr_vop_padded, 0);

  MakeImageEdge_Studio(GetVopY2_st(curr_vop),
                       FRAME_VOP, 0,
                       orig_width, orig_height, edge, edge,
                       GetVopY2_st(curr_vop_padded));
  if (GetVopShape(curr_vop) == GREY_SCALE) {
    for(i=0; i<MAX_AUX_COMP_NUM; i++)
      MakeImageEdge_Studio(GetVopG2_st(curr_vop,i),
                           FRAME_VOP, 0,
                           orig_width, orig_height, edge, edge,
                           GetVopG2_st(curr_vop_padded,i));
  }
  MakeImageEdge_Studio(GetVopU2_st(curr_vop),
                       FRAME_VOP, 0,
                       orig_width/cdivx, orig_height/cdivy, edge/cdivx, edge/cdivy,
                       GetVopU2_st(curr_vop_padded));
  MakeImageEdge_Studio(GetVopV2_st(curr_vop),
                       FRAME_VOP, 0,
                       orig_width/cdivx, orig_height/cdivy, edge/cdivx, edge/cdivy,
                       GetVopV2_st(curr_vop_padded));
  for(i=0; i<MAX_AUX_COMP_NUM; i++) {
    MakeImageEdge_Studio(GetVopA2_st(curr_vop,i),
                         FRAME_VOP, 0,
                         orig_width, orig_height, edge, edge,
                         GetVopA2_st(curr_vop_padded,i));
    MakeImageEdge_Studio(GetVopAuv2_st(curr_vop,i),
                         FRAME_VOP, 0,
                         orig_width/cdivx, orig_height/cdivy, edge/cdivx, edge/cdivy,
                         GetVopAuv2_st(curr_vop_padded,i));
  }

  curr_vop_padded->arbitrary_shape=GetVopShape(curr_vop);
  PutVopBitsPerPixel(GetVopBitsPerPixel(curr_vop),curr_vop_padded);
  for(i=0; i<MAX_AUX_COMP_NUM; i++)
    PutVopGrayBitsPerPixel_st(GetVopGrayBitsPerPixel_st(curr_vop,i),curr_vop_padded,i);  

/*  PutVopShapeExtension (GetVopShapeExtension (curr_vop), curr_vop_padded);*/

#if 0
  if (GetVopShape (curr_vop) != RECTANGULAR)
  {
    MacroblockBasedPadding_CHRext (GetVopA(curr_vop_padded),
                                   curr_vop_padded, chroma_format);
  }
#endif 0
  
  RecRepetitivePadding_Studio (curr_vop_padded, edge);

  for(i=0; i<2; i++) {
    for(j=0; j<MAX_AUX_COMP_NUM; j++) {
      PutVopA_st(GetVopA_st(curr_vop_padded,j,i),curr_vop,j,i);
      PutVopAuv_st(GetVopAuv_st(curr_vop_padded,j,i),curr_vop,j,i);
      PutVopG_st(GetVopG_st(curr_vop_padded,j,i),curr_vop,j,i);
    }
    PutVopY_st(GetVopY_st(curr_vop_padded,i),curr_vop,i);
    PutVopU_st(GetVopU_st(curr_vop_padded,i),curr_vop,i);
    PutVopV_st(GetVopV_st(curr_vop_padded,i),curr_vop,i);
    FreeImage(GetVopQP_st(curr_vop_padded,i));
    FreeImage(GetVopGLQP_st(curr_vop_padded,0,i));
    FreeImage(GetVopShapeMode_st(curr_vop_padded,i));
  }
  SfreeVop(curr_vop_padded);
  curr_vop->hor_spat_ref_st[0] -= edge;
  curr_vop->ver_spat_ref_st[0] -= edge;
  PutVopWidth_st (GetVopWidth_st (curr_vop,0)+2*edge,curr_vop,0);
  PutVopHeight_st(GetVopHeight_st(curr_vop,0)+2*edge,curr_vop,0);
  
}
