/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)
 *   Stefan Schultz (TUH / ACTS-MoMuSyS)
 *   Michael Wollborn (TUH / ACTS-MoMuSyS)
 *
 * and edited by
 * 
 *   Karl Lillevold (Telenor / ACTS-MoMuSyS)
 *   Cor Quist (KPN / ACTS-MoMuSyS)
 *   Jan De Lameillieure (HHI / ACTS-MoMuSyS)
 *   Gunn K Klungsoeyr (Telenor / ACTS-MoMuSyS)
 *   Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	text_quant.c
 *
 * Authors:	Robert Danielsen, Telenor R&D, Norway 
 *		Stefan Schultz, Michael Wollborn (TUH)
 * Created:	
 *                                                                         
 * Description: 
 *
 * Notes: 	
 *
 * Modified:	Karl Lillevold, Telenor (tmn)
 *		Cor Quist -- 12-Apr-96	
 * 		21.04.96 Robert Danielsen: Reformatted. New headers.
 *		Jan De Lameillieure (HHI), 19-AUG-96 : corrected rounding of 
 *			intra-DC-coefficient
 *	10.09.96 Robert Danielsen: Added BlockQuant(), BlockDequant()	       
 *	21-APR-97 Jan De Lameillieure : removed Quant()	
 *	22-APR-97 Michael Wollborn: Added the functions for MPEG-like
 *		  quantization from the former file "text_quant_mpeg.c",
 *		  renamed "BlockQuant()" -> "BlockQuantH263()" and the
 *		  dequantization respectively    
 *      23.06.98  G. Klungsoeyr: added support for short video header (H.263)   
 *      01.04.01  Osamu Sunohara(SONY): Functions for Studio Profile
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include <stdlib.h>
#include "text_defs.h"
#include "momusys.h"
#include "text_quant.h"
#include "text_util.h"		/* JDL 31-JUL-96 : because zz[] is needed */

/* 01.04.01: Studio Profile */
#include "vm_common_defs.h"

#ifdef _STUDIO_CONF_6_
#include <math.h>
#endif _STUDIO_CONF_6_

/***********************************************************CommentBegin******
 *
 * -- cal_dc_scaler -- calculation of DC quantization scale according 
                      to the incoming Q and type; 
 *
 * Author : Minhua Zhou 		
 *	
 *
 * Created : 04.11.97		
 *	
 *
 * Purpose :		
 *
 * 
 * Arguments in : 	
 *   Int Qp 
 * Arguments in/out :	
 *	
 * Arguments out :	
 *
 * Return values :	
 *	 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :		
 ***********************************************************CommentEnd********/


Int cal_dc_scaler (Int QP, Int type) {

 Int dc_scaler;
 if (type == 1)
    {
      if (QP > 0 && QP < 5) dc_scaler = 8;
      else if (QP > 4 && QP < 9) dc_scaler = 2 * QP;
      else if (QP > 8 && QP < 25) dc_scaler = QP + 8;
      else dc_scaler = 2 * QP - 16;
    }
  else
    {
      if (QP > 0 && QP < 5) dc_scaler = 8;
      else if (QP > 4 && QP < 25) dc_scaler = (QP + 13) / 2;
      else dc_scaler = QP - 6;
    }   
  return dc_scaler;
}


/***********************************************************CommentBegin******
 *
 * -- BlockQuantH263 -- 8x8 block level quantization
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	8x8 block level quantization
 * 
 * Arguments in : 	
 *	Int *coeff	non-quantized coefficients
 *	Int QP		quantization parameter
 *	Int Mode	Macroblock coding mode
 *
 * Arguments in/out :	
 *
 * Arguments out :	
 *	Int *qcoeff	quantized coefficients
 *
 * Return values :	
 *	Void
 *
 * Side effects :	
 *
 * Description :	
 *
 * See also :
 *
 * Modified :		
 *	27.01.97 Robert Danielsen: Included rounding-fix for DC-coeff. Did
 *			it as in Quant() above. From the Berkeley team.
 *	27.02.97 Robert Danielsen: Changed to nonlinear quantization of
 *			Intra DC coefficients. Accoring to VM6 from Sevilla.
 *	22.04.97 Michael Wollborn: renamed to "BlockQuantH263()"
 *      25.03.98 M. Wollborn: modifications due to N2171 Cl. 2.2.14
 *
 ***********************************************************CommentEnd********/
Void
BlockQuantH263 (Int *coeff, Int QP, Int mode, Int type, Int *qcoeff, Int maxDC, Int image_type)
{
  Int i;
  Int level;
  Int dc_scaler;

  MOMCHECK(QP > 0 && (QP < 32*image_type));
  MOMCHECK(type == 1 || type == 2);
  dc_scaler = cal_dc_scaler(QP,type);
   
  if (QP)
    {
      if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
	{ /* Intra */
	  qcoeff[0] = MAX(1,MIN(maxDC-1, (coeff[0] >= 0) ? (coeff[0] + dc_scaler/2)/dc_scaler
				      : (coeff[0] - dc_scaler/2) /dc_scaler)); /* rounding */

	  for (i = 1; i < 64; i++)
	    {
	      level = (ABS(coeff[i])) / (2*QP);
	      /* Modified due to N2171 Cl. 2.2.14 MW 25-MAR-1998 */
	      /* qcoeff[i] =  MIN(127,MAX(-127,SIGN(coeff[i]) * level)); */
	      /* qcoeff[i] =  MIN(2047,MAX(-2047,SIGN(coeff[i]) * level)); */
              /* 16.11.98 Sven Brandau: "correct saturation" due to N2470, Clause 2.1.6 */
              qcoeff[i] =  MIN(2047,MAX(-2048,SIGN(coeff[i]) * level));
	    }
	}
      else
	{ /* non Intra */
	  for (i = 0; i < 64; i++)
	    {
	      level = (ABS(coeff[i])-QP/2)  / (2*QP);
	      /* Modified due to N2171 Cl. 2.2.14 MW 25-MAR-1998 */
	      /* qcoeff[i] = MIN(127,MAX(-127,SIGN(coeff[i]) * level)); */
	      /*qcoeff[i] = MIN(2047,MAX(-2047,SIGN(coeff[i]) * level));*/
              /* 16.11.98 Sven Brandau: "correct saturation" due to N2470, Clause 2.1.6 */
              qcoeff[i] =  MIN(2047,MAX(-2048,SIGN(coeff[i]) * level));
	    }
	}
    }
  else
    {
      /* No quantizing.
	 Used only for testing. Bitstream will not be decodable 
	 whether clipping is performed or not */
      for (i = 0; i < 64; i++)
	{
	  qcoeff[i] = coeff[i];
	}
    }
  return;
}

/***********************************************************CommentBegin******
 *
 * -- BlockDequantH263 -- 8x8 block dequantization
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	8x8 block dequantization
 * 
 * Arguments in : 	
 *	Int *qcoeff	        quantized coefficients
 *	Int QP		        quantization parameter
 *	Int mode	        Macroblock coding mode
 *      Int short_video_header  Flag to signal short video header bitstreams (H.263)
 *
 * Arguments in/out :	
 *
 * Arguments out :	
 *	Int *rcoeff	reconstructed (dequantized) coefficients
 *
 * Return values :	
 *	Void
 *
 * Side effects :	
 *	
 * Description :	
 *	
 * See also :
 *	
 * Modified :		
 *	27.02.97 Robert Danielsen: Changed to nonlinear quantization of
 *			Intra DC coefficients. Accoring to VM6 from Sevilla.
 *	22.04.97 Michael Wollborn: renamed to "BlockDequantH263()"
 *      23.06.98 G. Klungsoeyr: added support for short video header
 *	
 ***********************************************************CommentEnd********/
Void
BlockDequantH263 (Int *qcoeff, Int QP, Int mode, Int type, Int *rcoeff, 
                  Int image_type, Int short_video_header, Int bits_per_pixel)
{
  Int i;
  Int dc_scaler;
  Int lim;

  lim = (1 << (bits_per_pixel + 3));
  
  if (QP)
    {
      for (i = 0; i < 64; i++)
	{
	  if (qcoeff[i])
	    {
              /* 16.11.98 Sven Brandau: "correct saturation" due to N2470, Clause 2.1.6 */
              qcoeff[i] =  MIN(2047, MAX(-2048, qcoeff[i] ));
	      if ((QP % 2) == 1)
		rcoeff[i] = QP * (2*ABS(qcoeff[i]) + 1);
	      else
		rcoeff[i] = QP * (2*ABS(qcoeff[i]) + 1) - 1;
	      rcoeff[i] = SIGN(qcoeff[i]) * rcoeff[i];
	    }
	  else
	    rcoeff[i] = 0;
	}
      if (mode == MODE_INTRA || mode == MODE_INTRA_Q) 
	{ /* Intra */

	    MOMCHECK(QP > 0 && (QP < 32*image_type));
	    MOMCHECK(type == 1 || type == 2);

	    if (short_video_header)
	      dc_scaler = 8;
	    else	    
	      dc_scaler = cal_dc_scaler(QP,type);
  
	    rcoeff[0] = qcoeff[0] * dc_scaler;
	}
    }
  else
    {
      /* No quantizing at all */
      for (i = 0; i < 64; i++)
	{
	  rcoeff[i] = qcoeff[i];
	}

      if (mode == MODE_INTRA || mode == MODE_INTRA_Q) 
      { /* Intra */
      	rcoeff[0] = qcoeff[0]*8;
      }
    }
  for (i=0;i<64;i++)
      if (rcoeff[i]>(lim-1)) rcoeff[i]=(lim-1);
       else if (rcoeff[i]<(-lim)) rcoeff[i]=(-lim);

  return;
}


/***********************************************************CommentBegin******
 *
 * -- BlockQuantMPEG -- Quantize DCT-coefficients MPEG-like
 *
 * Author :		
 *	Stefan Schultz, TUH
 *
 * Created :		
 *	
 *
 * Purpose :		
 *	Quantize DCT-coefficients using definable quant. matrix
 * 
 * Arguments in : 	
 *	Int *coeff : pointer to dct-coefficients
 * 	Int QP : quantizer step ( 1 - 31)
 * 	mode : Macroblock encoding mode
 *
 * Arguments in/out :	
 *	Int *qcoeff : pointer to quantized coefficients
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :
 *	31-JUL-96 Jan De Lameillieure (HHI) : corrected MPEG-1/-2 inverse 
 *	quantisationby doing zigzag of elements of weighting matrices
 *
 *	19-AUG-96 Jan De Lameillieure (HHI) : corrected rounding in 
 *	MPEG-1/-2 quantisation
 *
 *	14-NOV-1996, M.Wollborn: Removed zigzag-stuff, snce that is now
 *	from VM4.0 done directly before encoding and after decoding of
 *	the DCT coefficients
 *	27.02.97 Robert Danielsen: Changed to nonlinear quantization of
 *			Intra DC coefficients. Accoring to VM6 from Sevilla.
 *      25.03.98 M. Wollborn: modifications due to N2171 Cl. 2.2.14
 *
 ***********************************************************CommentEnd********/
 
Void
BlockQuantMPEG (Int *coeff, Int QP, Int mode, Int type, Int *qmat, Int *qcoeff, Int maxDC, Int image_type)
{
  Int i;
  Int ac1, ac2, ac3, p=3, q=4;
  Int dc_scaler;

  MOMCHECK(QP > 0 && (QP < 32*image_type));
  MOMCHECK(type == 1 || type == 2);

  dc_scaler = cal_dc_scaler(QP,type);  
  if (QP)
    {
      if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
	{ /* Intra */
	  /* corrected rounding ; JDL 19-AUG-96 */
	  qcoeff[0] = MAX(1,MIN(maxDC-1, (coeff[0] >= 0) ?
				(coeff[0]+dc_scaler/2)/dc_scaler : 
				(coeff[0]-dc_scaler/2)/dc_scaler));

	  for (i = 1; i < 64; i++)
	    {
	      /* added zz[] (JDL 31-JUL-96) */
	      /* added corrected rounding (JDL 19-AUG-96) */
	      /* removed zz[] (MW 14-NOV-1996 */
	      ac1 = (16*coeff[i] + ((coeff[i]>0) ? 1 : (-1))*qmat[i]/2) 
		     / qmat[i];

	      ac2 = MAX(-2048, MIN(2047, ac1));

	      /* corrected rounding ; JDL 19-AUG-96 */
	      ac3 = ( ac2 + SIGN0(ac2)*((p*QP + (q/2))/q) )/(2*QP);	

	      /* Modified due to N2171 Cl. 2.2.14 MW 25-MAR-1998 */
	      /* qcoeff[i] =  MIN(127, MAX(-127, ac3)); */
              /* 16.11.98 Sven Brandau: "correct saturation" due to N2470, Clause 2.1.6 */
	      /*qcoeff[i] =  MIN(2047, MAX(-2047, ac3));*/
              qcoeff[i] =  MIN(2047, MAX(-2048, ac3));
	    }
	}
      else
	{ /* non Intra */
	  for (i = 0; i < 64; i++)
	    {
	      /* added zz[] (JDL 31-JUL-96) */
	      /* added corrected rounding (JDL 19-AUG-96) */
	      /* removed zz[] (MW 14-NOV-1996 */
	      ac1 = (16*coeff[i] + ((coeff[i]>0) ? 1 : (-1))*qmat[i]/2) 
		     / qmat[i]; 

	      ac2 = ac1/(2*QP);

	      /* Modified due to N2171 Cl. 2.2.14 MW 25-MAR-1998 */
	      /* qcoeff[i] =  MIN(127, MAX(-127, ac2)); */
              /* 16.11.98 Sven Brandau: "correct saturation" due to N2470, Clause 2.1.6 */
	      /*qcoeff[i] =  MIN(2047, MAX(-2047, ac2));*/
              qcoeff[i] =  MIN(2047, MAX(-2048, ac2));
	    }
	}
    }
  else
    {
      /* No quantizing.
	 Used only for testing. Bitstream will not be decodable 
	 whether clipping is performed or not */
      for (i = 0; i < 64; i++)
	{
	  qcoeff[i] = coeff[i];
	}
    }

  return;
}

/***********************************************************CommentBegin******
 *
 * -- BlockDequantMPEG -- Dequantize DCT coefficients MPEG-like
 *
 * Author :		
 *	Stefan Schultz, TUH
 *
 * Created :		
 *	
 *
 * Purpose :		
 *	Dequant_mpegizer for quantized dct-coefficients
 * 
 * Arguments in : 	
 *	Int *qcoeff : pointer to quantized coefficients
 * 	Int QP : quantizer step 
 * 	Int mode : macroblock encoding mode
 *
 * Arguments in/out :	
 *	Int *rcoeff : pointer to reconstructed coefficients
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :		
 *	31-JUL-96 Jan De Lameillieure (HHI) : corrected MPEG-1/-2 inverse 
 *	quantisation by doing zigzag of elements of weighting matrices
 *
 *	14-NOV-1996, M.Wollborn: Removed zigzag-stuff, snce that is now
 *	from VM4.0 done directly before encoding and after decoding of
 *	the DCT coefficients
 *	27.02.97 Robert Danielsen: Changed to nonlinear quantization of
 *			Intra DC coefficients. Accoring to VM6 from Sevilla.
 *      04.08.97 Minhua Zhou: added "mismatch control"
 *	
 ***********************************************************CommentEnd********/

Void
BlockDequantMPEG (Int *qcoeff, Int QP, Int mode, Int type, Int *qmat, 
		  Int *rcoeff, Int image_type)
{
  Int i;
  Int dc_scaler,sum;
  
  if (QP)
    {
      if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
	{ /* Intra */
	  MOMCHECK(QP > 0 && (QP < 32*image_type));
	  MOMCHECK(type == 1 || type == 2);

	 dc_scaler = cal_dc_scaler(QP,type);
	  rcoeff[0] = qcoeff[0]*dc_scaler;

	  for (i = 1; i < 64; i++)
	    {
	      if (qcoeff[i]) {
                /* 16.11.98 Sven Brandau: "correct saturation" due to N2470, Clause 2.1.6 */
                qcoeff[i] =  MIN(2047, MAX(-2048, qcoeff[i] ));
		  /* added zz[] ; JDL 31-JUL-96 */
		  /* removed zz[], (14-NOV-1996, MW) */
	        rcoeff[i] = (qcoeff[i]*qmat[i]*QP*2)/16;
                if (rcoeff[i]>2047) rcoeff[i]=2047;
                  else if (rcoeff[i]<-2048) rcoeff[i]=-2048;
               }  	
	      else
	        rcoeff[i] = 0;
	    }
	}
      else
        { /* non intra */
	  for (i = 0; i < 64; i++)
            {
	      if (qcoeff[i]) {
                /* 16.11.98 Sven Brandau: "correct saturation" due to N2470, Clause 2.1.6 */
                qcoeff[i] =  MIN(2047, MAX(-2048, qcoeff[i] ));

	        /* added zz[] ; JDL 31-JUL-96 */
	        /* removed zz[], (14-NOV-1996, MW) */
	        rcoeff[i]=(((qcoeff[i]*2)+SIGN0(qcoeff[i]))
			   *qmat[i]*QP)/16; 
                if (rcoeff[i]>2047) rcoeff[i]=2047;
                  else if (rcoeff[i]<-2048) rcoeff[i]=-2048;
               }
	      else
	        rcoeff[i] = 0;
	    }
	}
    }
  else
    {
      /* No quantizing at all */
      for (i = 0; i < 64; i++)
	{
	  rcoeff[i] = qcoeff[i];
	}
    }




/* added by Minhua Zhou, 04.08.1997 Mismatch Control */
  sum =0;
  for (i=0;i<64;i++) sum+=abs(rcoeff[i]);
  if (sum) {
    sum=0;
    for (i=0;i<64;i++) sum+=rcoeff[i];
     if ((sum&0x1)==0) rcoeff[63]+=((rcoeff[63]&0x1)?-1:1); 
  }
    

  return;
}

/* 01.04.01: Studio Profile */
/***********************************************************CommentBegin******
 *
 * -- cal_dc_scaler_Studio -- calculation of DC quantization scale according 
 *                     to the incoming Q and type; 
 *
 * Author : Minhua Zhou 		
 *	
 *
 * Created : 04.11.97		
 *	
 *
 * Purpose :		
 *
 * 
 * Arguments in : 	
 *   Int Qp 
 * Arguments in/out :	
 *	
 * Arguments out :	
 *
 * Return values :	
 *	 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :
 *    01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/

Int cal_dc_scaler_Studio (Int intra_dc_precision) {

  Int dc_scaler;

  if(intra_dc_precision == INTRA_DC_PRECISION8)
    dc_scaler = 8;
  else if(intra_dc_precision == INTRA_DC_PRECISION4)
    dc_scaler = 4;
  else if(intra_dc_precision == INTRA_DC_PRECISION2)
    dc_scaler = 2;
  else/* if(intra_dc_precision == INTRA_DC_PRECISION1)*/
    dc_scaler = 1;
  
  return dc_scaler;
}

/***********************************************************CommentBegin******
 *
 * -- BlockQuantMPEG_DCText_Studio -- Quantize DCT-coefficients MPEG-like
 *
 * Author :		
 *	Stefan Schultz, TUH
 *
 * Created :		
 *	
 *
 * Purpose :		
 *	Quantize DCT-coefficients using definable quant. matrix
 * 
 * Arguments in : 	
 *	Int *coeff : pointer to dct-coefficients
 * 	Int QP : quantizer step ( 1 - 31)
 * 	mode : Macroblock encoding mode
 *
 * Arguments in/out :	
 *	Int *qcoeff : pointer to quantized coefficients
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :
 *	31-JUL-96 Jan De Lameillieure (HHI) : corrected MPEG-1/-2 inverse 
 *	quantisationby doing zigzag of elements of weighting matrices
 *
 *	19-AUG-96 Jan De Lameillieure (HHI) : corrected rounding in 
 *	MPEG-1/-2 quantisation
 *
 *	14-NOV-1996, M.Wollborn: Removed zigzag-stuff, snce that is now
 *	from VM4.0 done directly before encoding and after decoding of
 *	the DCT coefficients
 *	27.02.97 Robert Danielsen: Changed to nonlinear quantization of
 *			Intra DC coefficients. Accoring to VM6 from Sevilla.
 *      25.03.98 M. Wollborn: modifications due to N2171 Cl. 2.2.14
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/
Void
BlockQuantMPEG_DCText_Studio (Int *coeff, Int QP, Int mode, Int type,
			      Int intra_dc_precision, Int q_scale_type, Int *qmat,
			      Int dct_precision,
			      Int bits_per_pixel,
			      Int *qcoeff, Int maxDC, Int image_type)
{
  Int i;
  Int ac1, ac2, ac3, p=3, q=4;
  Int dc_scaler;

  Int quantizer_scale;

  Double base_quantiser = 1.0/pow(2.0, dct_precision);
  Int range = (Int)pow(2.0, bits_per_pixel+dct_precision+3);
  
  MOMCHECK(QP > 0 && (QP < 32*image_type));
  MOMCHECK(type == 1 || type == 2);

#ifdef _STUDIO_CONF_5_
  ;
#elif  _STUDIO_CONF_6_
  for(i=0;i<64;i++) {
    qcoeff[i] = (int)( sqrt( ((double)rand())/((double)RAND_MAX)*((double)67092481L) ) + 0.5);
    if ( rand() < RAND_MAX/2 ) {
      qcoeff[i] ++;
      qcoeff[i] *= -1;
    }
    
    if ( qcoeff[i] == -1 * (1<<13) )
      qcoeff[i] += 1;
    
/*    qcoeff[i] = (int)sqrt( ((double)rand())/((double)RAND_MAX)*((double)268402689L) );
    qcoeff[i] -= 1<<13;
*/    
  }
  if (mode == MODE_INTRA || mode == MODE_INTRA_Q) {
    if ( qcoeff[0] == -1 * (1<<13) )
      qcoeff[0] += 1;
    if ( qcoeff[0] < 0 )
      qcoeff[0] *= -1;
  }
#else
  for(i=0;i<64;i++)
    coeff[i] = (Int)((Double)coeff[i]/base_quantiser);
#endif _STUDIO_CONF_5_
  
  dc_scaler = cal_dc_scaler_Studio(intra_dc_precision);
  if (QP)
    {
#ifdef _STUDIO_CONF_6_
      ;
#else
      if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
	{ /* Intra */
	  /* corrected rounding ; JDL 19-AUG-96 */
#ifdef _STUDIO_CONF_5_
          qcoeff[0] = MAX(0,MIN(maxDC, coeff[0]));
#else
	  qcoeff[0] = MAX(1,MIN(maxDC-1, (coeff[0] >= 0) ?
				(coeff[0]+dc_scaler/2)/dc_scaler : 
				(coeff[0]-dc_scaler/2)/dc_scaler));
#endif _STUDIO_CONF_5_

	  for (i = 1; i < 64; i++)
	    {
#ifdef _STUDIO_CONF_5_
              qcoeff[i] = MIN(range-1, MAX(-(range-1), coeff[i]));
#else
	      ac1 = (16*coeff[i] + ((coeff[i]>0) ? 1 : (-1))*qmat[i]/2) 
		     / qmat[i];

	      ac2 = MAX(-range,MIN(range-1, ac1));

	      /* corrected rounding ; JDL 19-AUG-96 */
	      quantizer_scale = CalcQuantizerScale_Studio(q_scale_type,QP);
	      ac3 = ( 2*ac2 + SIGN0(ac2)*((p*quantizer_scale + (q/2))/q) )/(2*quantizer_scale);
	      /* MoMuSys-2000FDAM1COR-020612 */
	      qcoeff[i] = MIN(range-1, MAX(-(range-1), ac3));
	      /*qcoeff[i] = MIN(range-1, MAX(-range, ac3));*/
	      /**/
#endif _STUDIO_CONF_5_
	    }
	}
      else
	{ /* non Intra */
	  for (i = 0; i < 64; i++)
	    {
	      ac1 = (16*coeff[i] + ((coeff[i]>0) ? 1 : (-1))*qmat[i]/2) 
		     / qmat[i];

	      quantizer_scale = CalcQuantizerScale_Studio(q_scale_type,QP);
	      ac2 = ac1/(quantizer_scale);
	      /* MoMuSys-2000FDAM1COR-020612 */
	      qcoeff[i] = MIN(range-1, MAX(-(range-1), ac2));
	      /*qcoeff[i] = MIN(range-1, MAX(-range, ac2));*/
	      /**/
	    }
	}
#endif _STUDIO_CONF_6_      
    }
  else
    {
      /* No quantizing.
	 Used only for testing. Bitstream will not be decodable 
	 whether clipping is performed or not */
      for (i = 0; i < 64; i++)
	{
	  qcoeff[i] = coeff[i];
	}
    }

  return;
}

/***********************************************************CommentBegin******
 *
 * -- BlockDequantMPEG_DCText_DeadZone_Studio -- Dequantize DCT coefficients MPEG-like
 *
 * Author :		
 *	Stefan Schultz, TUH
 *
 * Created :		
 *	
 *
 * Purpose :		
 *	Dequant_mpegizer for quantized dct-coefficients
 * 
 * Arguments in : 	
 *	Int *qcoeff : pointer to quantized coefficients
 * 	Int QP : quantizer step 
 * 	Int mode : macroblock encoding mode
 *
 * Arguments in/out :	
 *	Int *rcoeff : pointer to reconstructed coefficients
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :		
 *	31-JUL-96 Jan De Lameillieure (HHI) : corrected MPEG-1/-2 inverse 
 *	quantisation by doing zigzag of elements of weighting matrices
 *
 *	14-NOV-1996, M.Wollborn: Removed zigzag-stuff, snce that is now
 *	from VM4.0 done directly before encoding and after decoding of
 *	the DCT coefficients
 *	27.02.97 Robert Danielsen: Changed to nonlinear quantization of
 *			Intra DC coefficients. Accoring to VM6 from Sevilla.
 *      04.08.97 Minhua Zhou: added "mismatch control"
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *	
 ***********************************************************CommentEnd********/
Void
BlockDequantMPEG_DCText_DeadZone_Studio(Int *qcoeff, Int QP, Int mode, Int type,
					Int intra_dc_precision, Int q_scale_type, Int *qmat,
					Int dct_precision, Int bits_per_pixel,
					Int dead_zone_disable,
					Int *rcoeff, Int image_type,
					Int mpeg2)
{
  Int i;
  Int dc_scaler,sum;

  Int quantizer_scale;

  Double base_quantiser; /* = 1.0/pow(2.0, dct_precision);*/
  Int range; /*= (Int)pow(2.0, bits_per_pixel+6);*/

  if(mpeg2==0) {
    base_quantiser = 1.0/pow(2.0, dct_precision);
    range = (Int)pow(2.0, bits_per_pixel+6);
  } else {
    base_quantiser = 0.125;
    range = (Int)pow(2.0, bits_per_pixel+3);
  }
    
  if (QP)
    {
      if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
	{ /* Intra */
	  MOMCHECK(QP > 0 && (QP < 32*image_type));
	  MOMCHECK(type == 1 || type == 2);

#ifdef _STUDIO_CONF_6_
          if ( qcoeff[0] < 0 || qcoeff[0] > (1 << dct_precision+bits_per_pixel+intra_dc_precision ) -1 )
            exit(-2);
#endif _STUDIO_CONF_6_
          
	  dc_scaler = cal_dc_scaler_Studio(intra_dc_precision);
	  rcoeff[0] = qcoeff[0]*dc_scaler*base_quantiser*8;

	  for (i = 1; i < 64; i++)
	    {
	      if (qcoeff[i]) {
		quantizer_scale = CalcQuantizerScale_Studio(q_scale_type,QP);
	        rcoeff[i] = (2*qcoeff[i]*qmat[i]*quantizer_scale*base_quantiser*8)/32;

#ifdef _STUDIO_CONF_6_
                if (rcoeff[i]>range-1) {
                  rcoeff[i]=range-1;
                  fprintf(stdout, "satured i+!!\n");
                } else if (rcoeff[i]<-range) {
                  rcoeff[i]=-range;
                  fprintf(stdout, "satured i-!!\n");
                }
#else
                if (rcoeff[i]>range-1) rcoeff[i]=range-1;
		else if (rcoeff[i]<-range) rcoeff[i]=-range;
#endif _STUDIO_CONF_6_
               }  	
	      else
	        rcoeff[i] = 0;
	    }
	}
      else
        { /* non intra */
	  for (i = 0; i < 64; i++)
            {
	      if (qcoeff[i]) {
		quantizer_scale = CalcQuantizerScale_Studio(q_scale_type,QP);
		if(!dead_zone_disable) {
		  rcoeff[i]=(((qcoeff[i]*2)+SIGN0(qcoeff[i]))*qmat[i]*quantizer_scale*base_quantiser*8)/32;
		} else {
		  rcoeff[i] = (2*qcoeff[i]*qmat[i]*quantizer_scale*base_quantiser*8)/32;
		}
		
#ifdef _STUDIO_CONF_6_
                if (rcoeff[i]>range-1) {
                  rcoeff[i]=range-1;
                  fprintf(stdout, "satured n+!!\n");
                } else if (rcoeff[i]<-range) {
                  rcoeff[i]=-range;
                  fprintf(stdout, "satured n-!!\n");
                }
#else
                if (rcoeff[i]>range-1) rcoeff[i]=range-1;
		else if (rcoeff[i]<-range) rcoeff[i]=-range;
#endif _STUDIO_CONF_6_

               }
	      else
	        rcoeff[i] = 0;
	    }
	}
    }
  else
    {
      /* No quantizing at all */
      for (i = 0; i < 64; i++)
	{
	  rcoeff[i] = qcoeff[i];
	}
    }




/* added by Minhua Zhou, 04.08.1997 Mismatch Control */
  sum =0;
  for (i=0;i<64;i++) sum+=abs(rcoeff[i]);
  if (sum) {
    sum=0;
    for (i=0;i<64;i++) sum+=rcoeff[i];
     if ((sum&0x1)==0) rcoeff[63]+=((rcoeff[63]&0x1)?-1:1); 
  }
  
  return;
}
/* 01.04.01: Studio Profile */




