/******************************************************************************
 *
 * This software module was originally developed by
 *
 *      Ji Heon Kweon (HYUNDAI)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1998
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *
 * File:        alp_dec_grey.c
 *
 * Author:      HYUNDAI - Ji Hweon Kweon
 * Created:     21-05-98
 *
 * Description:
 *
 * Notes:
 *
 * Modified: 
 * 18.06.98 A. Sandvand: Modified function calls to VlcGetBlock and BlockDequantH263
 * 16.08.99 Shigeru Fukunaga (Oki): added newpred_d.h for NEWPRED
 * 01.04.01 Osamu Sunohara (SONY): Functions for Studio Profile
 *
 ***********************************************************HeaderEnd*********/

#include <sys/stat.h>
#include <stdio.h>
#include "mom_bitstream_d.h"
#include "newpred_d.h"	/** added for NEWPRED (Oki) 16-AUG-1999 **/
#include "combined_decode.h"
#include "momusys.h"
#include "mom_access.h"
#include "mom_image.h"
#include "vm_enc_defs.h"
#include "vm_common_defs.h"
#include "mot_est.h"            /* because of MBM_INTRA */
#include "mom_bitstream_i.h"
#include "text_util.h"
#include "text_bits.h"
#include "text_quant.h"
#include "putvlc.h"
#include "text_code_mb.h"
#include "alp_code_grey.h"
#include "zigzag.h"             /* added 14-NOV-1996 MW */
/* 09.02.99 HHI Schueuer */
#include "sadct_bsnrmem.h"
#include "sadct.h"
#include "sadct_s_k.h"
#include "sadct_momusys.h"
#include "sadct_momusys_kaup.h"
#include "sadct_momusys_s_k.h"
/* end HHI */

#define OPAQUE 255
#define MIDOPAQUE 128

Void GetMBgraydata(Int transp_pattern[],
                    Int intra_dcpred_disable,
                    Int quant_step,
                    Bitstream *texture_bits,
                    Int mbnum,
                    Trace *trace,
                    SInt *g_mblock,
                    Int MB_width,
                    Int ***DC_store,
                    SInt *QP_store,
                    Int *error_flag,
                    UInt mode,
                    Vop *rec_vop,
		    Short *alpha)
{
  Int  comp;
  Int  DCT_block[B_SIZE * B_SIZE];
  Int  q_block[B_SIZE * B_SIZE];
  Int  r_block[B_SIZE * B_SIZE];
  Int  r_block_2D[B_SIZE][B_SIZE];
  Int  pcoeff[B_SIZE * B_SIZE];
  Int  pred_DC_coeff;
  Int  x_pos = mbnum % MB_width;
  Int  y_pos = mbnum / MB_width;
  Int  m, n;
  Int  CBPA;
  Int ACpred_flag = -1;
  Int QP;
  /* 09.02.99 HHI Schueuer */
  Int sadct_used = 0;
  UChar disable_sadct = GetVopSADCTDisable(rec_vop);
  /* variables because of contour-adaptive DCT */
  Int i, j;
  Short alpha_block[4][8][8];
  Short nrpix = MB_width * MB_SIZE;
  /* SInt mb_shape[6][64],mb_dct_bf_iq[6][64],mb_dct_iq[6][64],mb_idct[6][64]; */
  /* end HHI */

  /* 09.02.99 HHI Schueuer */
  if (disable_sadct==0) {
    for (i = 0; i < 8; i++) {
      for (j = 0; j < 8; j++) {
	alpha_block[0][i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + j)];
	alpha_block[1][i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
	alpha_block[2][i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + j)];
	alpha_block[3][i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
      }
    }
  }
  
	
  /* end HHI */
      
  

  if ((mode == MODE_INTRA || mode == MODE_INTRA_Q) && (intra_dcpred_disable == 0))
    {
       ACpred_flag = BitstreamReadBits(texture_bits,1,"ACpred_flag",trace,FLAG);
    }


  QP = quant_step;

  g_MBvalue (g_mblock, 0);

  CBPA = VlcDecCBPY (texture_bits, 0, transp_pattern,error_flag, trace);
  if (*error_flag == RETURN)
    {
       printf ("Error decoding CBPA of grayscale alpha block %d\n", (Int)mbnum);
       return;
    }

  /*****
   *
   *      Decoding of the 4 GREY_SCALE blocks (depending on transparent pattern)
   *
   *****/

  if (mode == MODE_INTRA || mode == MODE_INTRA_Q) {
    for (comp=0; comp<4; comp++)
     {   
        Bzero (q_block);
	/* 10.02.99 HHI Schueuer */
	if (!disable_sadct) {
	  sadct_rowlength_out(comp, alpha_block[comp], 8, 8);
	  SADCT_Used_Decision_d (comp, &sadct_used);
	}
	/* end HHI */

        if(transp_pattern[comp]!=1) /* No decoding of transparent Y blocks */
        {

                /* This must be Int not UInt MW 15-NOV-1996 */
                  pred_DC_coeff = DecodePredictedIntraDC(comp,
                                                         texture_bits,
                                                         trace,
                                                         NUM);
                  q_block[0] = pred_DC_coeff;
 
          if ((CBPA>>(3-comp)) & 1) {
 
            VlcGetBlock(texture_bits, q_block, INTRA,
                      trace, error_flag, 1, GetVopReverseVlc(rec_vop),0,0);
          }
 
          /* Do the DC/AC prediction, changing the qcoeff values as
             appropriate */
          if (intra_dcpred_disable == 0)
              doDCACrecon(q_block, 64, DC_store, QP_store, QP,
                          x_pos, y_pos, MB_width, ACpred_flag, 
			  /*GetVopAlternateScan(rec_vop)*/ 0, comp, MIDOPAQUE, sadct_used); /* SB, 08.10.98 */
	  /* 29.01.99 HHI Schueuer */
          else {
	    
	    /* Ordinary inverse zigzag-scanning */
	    /* 10.02.99 HHI Schueuer */
	    if (sadct_used == 0) {
	      for (m = 0; m < 64; m++)
		pcoeff[m] = q_block[zigzag[m]];
	    }
	    else {
	      SADCT_Inverse_Scan(q_block, pcoeff, comp, 0);
	    }
	    for (m = 0; m < 8; m++)
	      for (n = 0; n < 8; n++)
		q_block[m*8+n] = pcoeff[m*8+n];
 
          }
 
          /* Store the qcoeff-values needed later for prediction */
          DC_store[mbnum][comp][0] = q_block[0]*cal_dc_scaler(QP,1);
 
          for (m = 1; m < 8; m++)
            DC_store[mbnum][comp][m] = q_block[m];

          for (m = 0; m < 7; m++)
            DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
   
 
              /* MW 22-APR-1997 Changes for MPEG dequantization */
          if(GetVopQuantType(rec_vop))
            BlockDequantMPEG(q_block,quant_step,INTRA,1,
                             GetVopIntraQuantMat(rec_vop),DCT_block,2);
          else
            BlockDequantH263(q_block,quant_step,INTRA,1,DCT_block,2,0, GetVopBitsPerPixel(rec_vop));

	  /* 10.02.99 HHI Schueuer */
	  if ( disable_sadct || (sadct_used == 0))
	    BlockIDCT(DCT_block, r_block_2D, OPAQUE);
	  else
	    BlockSAIDCT_s_k(DCT_block, r_block_2D, alpha_block[comp]);
 
          for (m = 0; m < B_SIZE; m++)
            for (n = 0; n < B_SIZE; n++)
              r_block[m*B_SIZE+n] = r_block_2D[m][n];
 
        }
      else /* clear r_block if transparent block */
        {
          Bzero(r_block);
 
          /* Fill DC_store with default coeff values */
          DC_store[mbnum][comp][0] = MIDOPAQUE*8;
          for(n=1; n<15; n++)
            DC_store[mbnum][comp][n] = 0;
        }
         
      PutBlockG (comp, r_block, g_mblock);
    }
  }  
  else /* MODE INTER */
    {
      for (comp=0; comp<4; comp++) {
 
	Bzero (q_block);

	/* 10.02.99 HHI Schueuer */
	if (!disable_sadct) {
	  sadct_rowlength_out(comp, alpha_block[comp], 8, 8);
	  SADCT_Used_Decision_d (comp, &sadct_used);
	}
	/* end HHI */
 
	if(transp_pattern[comp]!=1) { /* No decoding of transparent Y blocks */
	  if ((CBPA>>(3-comp)) & 1) {
	    VlcGetBlock (texture_bits, q_block, INTER, trace,
			 error_flag, 0, GetVopReverseVlc(rec_vop),0,0);
	  }
 
	  /* Inverse zigzag-scanning also here 12-11-96 MW */
	  /* 10.02.99 HHI Schueuer */
	  if (sadct_used == 0) {
	    for (m = 0; m < 64; m++)
	      pcoeff[m] = q_block[zigzag[m]];
	  }
	  else {
	    SADCT_Inverse_Scan(q_block, pcoeff, comp, 0);
	  }
 
	  for (m = 0; m < 8; m++)
	    for (n = 0; n < 8; n++)
	      q_block[m*8+n] = pcoeff[m*8+n];
 
	  /* Fill DC_store with default coeff values */
	  DC_store[mbnum][comp][0] = MIDOPAQUE*8;
	  for (n = 1; n < 15; n++)
	    DC_store[mbnum][comp][n] = 0;
 
	  /* MW 22-APR-1997 Changes for MPEG dequantization */
	  if(GetVopQuantType(rec_vop))
	    BlockDequantMPEG(q_block,quant_step,INTER,1,
                       GetVopNonintraQuantMat(rec_vop),DCT_block,2);
	  else
	    BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,2,0, GetVopBitsPerPixel(rec_vop));

	  /* 10.02.99 HHI Berlin */
	  if (disable_sadct || (sadct_used == 0))
	    BlockIDCT(DCT_block, r_block_2D, OPAQUE);
	  else
	    BlockSAIDCT_Kaup (DCT_block, r_block_2D, alpha_block[comp]);
	  /* end HHI */
 
	  for (m = 0; m < B_SIZE; m++)
	    for (n = 0; n < B_SIZE; n++)
	      r_block[m*B_SIZE+n] = r_block_2D[m][n];
	}
	else /* clear r_block if transparent block */
	  {  
	    Bzero(r_block);
 
	    /* Fill DC_store with default coeff values */
	    DC_store[mbnum][comp][0] = MIDOPAQUE*8;
	    for(n=1; n<15; n++)
	      DC_store[mbnum][comp][n] = 0;
	  }
       
	PutBlockG (comp, r_block, g_mblock);
      }
    }
 
  if ((mode == MODE_INTRA)||(mode == MODE_INTRA_Q))
      g_MB_clip (g_mblock);
}                               /* GetMBgraydata */
 
 
Void B_GetMBgraydata(Int transp_pattern[],
                    Int quant_step,
                    Bitstream *texture_bits,
                    Int mbnum,
                    Trace *trace,
                    SInt *g_mblock,
                    Int MB_width,
                    Int *error_flag,
                    Vop *rec_vop,
		    Short *alpha)
{
  Int  comp;
  Int  DCT_block[B_SIZE * B_SIZE];
  Int  q_block[B_SIZE * B_SIZE];
  Int  r_block[B_SIZE * B_SIZE];
  Int  r_block_2D[B_SIZE][B_SIZE];
  Int  pcoeff[B_SIZE * B_SIZE];
  Int  m, n;
  Int  CBPA;
  /* 10.02.99 HHI Schueuer */
  Int sadct_used = 0;
  UChar disable_sadct = GetVopSADCTDisable(rec_vop);
  /* variables because of SA-DCT */
  Int  x_pos = mbnum % MB_width;
  Int  y_pos = mbnum / MB_width;
  Int i, j;
  Short alpha_block[4][8][8];
  Short nrpix = MB_width * MB_SIZE;
  /* SInt mb_shape[6][64],mb_dct_bf_iq[6][64],mb_dct_iq[6][64],mb_idct[6][64]; */
  /* end HHI */

  /* 09.02.99 HHI Schueuer */
  if (disable_sadct==0) {
    for (i = 0; i < 8; i++) {
      for (j = 0; j < 8; j++) {
	alpha_block[0][i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + j)];
	alpha_block[1][i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
	alpha_block[2][i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + j)];
	alpha_block[3][i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
      }
    }
  }
  /* end HHI */  
  
  g_MBvalue (g_mblock, 0);
 
  CBPA = VlcDecCBPY (texture_bits, 0, transp_pattern,error_flag, trace);
  if (*error_flag == RETURN)
    {
       printf ("Error decoding CBPA of grayscale alpha block %d\n", (Int)mbnum);
       return;
    }
 
  /*****
   *
   *      Decoding of the 4 GREY_SCALE blocks (depending on transparent pattern)
   *
   *****/
 
 
  for (comp=0; comp<4; comp++) {
 
      Bzero (q_block);

      /* 10.02.99 HHI Schueuer */
      if (!disable_sadct) {
	sadct_rowlength_out(comp, alpha_block[comp], 8, 8);
	SADCT_Used_Decision_d (comp, &sadct_used);
      }
      /* end HHI */
 
      if(transp_pattern[comp]!=1) { /* No decoding of transparent Y blocks */
        if ((CBPA>>(3-comp)) & 1) {
           VlcGetBlock (texture_bits, q_block, INTER, trace,
                        error_flag, 0, GetVopReverseVlc(rec_vop),0,0);
        }
 
      /* Inverse zigzag-scanning also here 12-11-96 MW */
	/* 10.02.99 HHI Schueuer */
	if (sadct_used == 0) {
	  for (m = 0; m < 64; m++)
	    pcoeff[m] = q_block[zigzag[m]];
	}
	else {
	  SADCT_Inverse_Scan(q_block, pcoeff, comp, 0);
	}
 
	for (m = 0; m < 8; m++)
	  for (n = 0; n < 8; n++)
	    q_block[m*8+n] = pcoeff[m*8+n];
 
	/* MW 22-APR-1997 Changes for MPEG dequantization */
	if(GetVopQuantType(rec_vop))
	  BlockDequantMPEG(q_block,quant_step,INTER,1,
			   GetVopNonintraQuantMat(rec_vop),DCT_block,2);
	else
	  BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,2,0, GetVopBitsPerPixel(rec_vop));

	/* 10.02.99 HHI Berlin */
	if (disable_sadct || (sadct_used == 0))
	  BlockIDCT(DCT_block, r_block_2D, OPAQUE);
	else
	  BlockSAIDCT_Kaup (DCT_block, r_block_2D, alpha_block[comp]);
	/* end HHI */
 
	for (m = 0; m < B_SIZE; m++)
	  for (n = 0; n < B_SIZE; n++)
	    r_block[m*B_SIZE+n] = r_block_2D[m][n];
      }
      else /* clear r_block if transparent block */
	Bzero(r_block);
      
      
      PutBlockG (comp, r_block, g_mblock);
  }
}                               /* GetMBgraydata */

/* 01.04.01: Studio Profile */
Void GetMBgraydata_DCT_StudioPDAM(Int aux_comp_num,
			      Int transp_pattern[],
			      Int intra_dcpred_disable,
			      Int *quant_step,
			      Int intra_dc_precision,
			      Int q_scale_type,
			      Int interlaced,
			      Int frame_pred_frame_dct,
			      Int field_dct,
			      Bitstream *texture_bits,
			      Int mbnum,
			      Trace *trace,
			      SInt *g_mblock,
			      Int MB_width,
			      Int ***DC_store,
			      SInt *QP_store,
			      Int *error_flag,
			      UInt mode,
			      Int *DC_prev,
			      Vop *rec_vop)
{
  Int  comp;
  Int  DCT_block[B_SIZE * B_SIZE];
  Int  q_block[B_SIZE * B_SIZE];
  Int  r_block[B_SIZE * B_SIZE];
  Int  r_block_2D[B_SIZE][B_SIZE];
  Int  pcoeff[B_SIZE * B_SIZE];
  Int  pred_DC_coeff;
  Int  x_pos = mbnum % MB_width;
  Int  y_pos = mbnum / MB_width;
  Int  m, n;
  Int  CBPA;
  Int ACpred_flag = -1;
  Int QP;

  Int vop_structure = GetVopVopStructure_st(rec_vop);
  Int field_parity = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(rec_vop), GetVopTopFieldFirst(rec_vop));
  Int tmpvar;

  Int a_uncompress_flag, a_ucbp_code;
  Int tmp_dc;
  Int k, i;
  Int bits_per_pixel = GetVopGrayBitsPerPixel_st(rec_vop,aux_comp_num);
  int ii, jj;

  
  QP = *quant_step;

  g_MBvalue (g_mblock, 0);

  if(mode == MODE_INTRA || mode == MODE_INTRA_Q) {
    CBPA = 15;
  } else {/*inter*/
    /* Decode Alpha MB header */
    CBPA = VlcDecCBPY (texture_bits, 0, transp_pattern,error_flag, trace);
    if (*error_flag == RETURN)
      {
	printf ("Error decoding CBPA of grayscale alpha block %d\n", (Int)mbnum);
	return;
      }
  }

  tmpvar = (Int) BitstreamReadBits(texture_bits,1,"load_alpha_quantizer_scale_code",trace,CODE);
  if(tmpvar == 1) {
    tmpvar = (Int) BitstreamReadBits(texture_bits,5,"alpha_quantizer_scale_code",trace,CODE);
    QP = *quant_step = tmpvar;
  }
  
  /*****
   *
   *      Decoding of the 4 GREY_SCALE blocks (depending on transparent pattern)
   *
   *****/

  if (mode == MODE_INTRA || mode == MODE_INTRA_Q) {
    for (comp=0; comp<4; comp++)
      {   
        Bzero (q_block);

        if(transp_pattern[comp]!=1) /* No decoding of transparent Y blocks */
	  {
	    
	    /* This must be Int not UInt MW 15-NOV-1996 */
	    pred_DC_coeff = DecodePredictedIntraDC_bySVLC_Studio(comp,
							  texture_bits,
							  trace,
							  NUM);

	    q_block[0] = pred_DC_coeff;

	      VlcGetBlock_bySVLC_Studio(texture_bits, q_block, INTRA,
					trace, error_flag, 
					bits_per_pixel+GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity)+4);

	    q_block[0] += *DC_prev;
	    
	    *DC_prev = q_block[0];

	    /* Zigzag scanning */
	    if (GetVopGrayAlternateScan_st(rec_vop,aux_comp_num,field_parity)) {
	      for (m = 0; m < 64; m ++)
		pcoeff[m] = q_block[zigzag_v[m]];
	    } else {
	      /* Ordinary inverse zigzag-scanning */
	      for (m = 0; m < 64; m++)
		pcoeff[m] = q_block[zigzag[m]];
	    }
	    for (m = 0; m < 8; m++)
	      for (n = 0; n < 8; n++)
		q_block[m*8+n] = pcoeff[m*8+n];

	    
	    /* Store the qcoeff-values needed later for prediction */
	    DC_store[mbnum][comp][0] = q_block[0] * cal_dc_scaler_Studio(intra_dc_precision);
	    
	    for (m = 1; m < 8; m++)
	      DC_store[mbnum][comp][m] = q_block[m];
	    
	    for (m = 0; m < 7; m++)
	      DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];

	    
	    /* MW 22-APR-1997 Changes for MPEG dequantization */
	    if(1/*GetVopQuantType(rec_vop)*/)
	      BlockDequantMPEG_DCText_DeadZone_Studio(q_block,*quant_step,INTRA,1,intra_dc_precision,q_scale_type,
						      GetVopGrayIntraQuantMat_st(rec_vop,aux_comp_num),
						      GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity),
						      bits_per_pixel,
						      GetVopDeadZoneDisable_st(rec_vop,field_parity),
						      DCT_block,1/*2*/,
						      0);

	    BlockIDCT_DCText_Studio(DCT_block, r_block_2D, GetVopGrayOpaque_Studio(rec_vop,aux_comp_num),0);
	    
	    for (m = 0; m < B_SIZE; m++)
	      for (n = 0; n < B_SIZE; n++)
		r_block[m*B_SIZE+n] = r_block_2D[m][n];


	      }
      else /* clear r_block if transparent block */
        {
	  *DC_prev = pow(2,GetVopGrayBitsPerPixel_st(rec_vop,aux_comp_num)+GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity)+GetVopGrayIntraDCPrecision_st(rec_vop,aux_comp_num,field_parity)-1);
	  
	  Bzero(r_block);
 
          /* Fill DC_store with default coeff values */
          DC_store[mbnum][comp][0] = GetVopGrayMidValue_Studio(rec_vop,aux_comp_num)*8;	  
          for(n=1; n<15; n++)
            DC_store[mbnum][comp][n] = 0;
        }
         
      PutBlockG (comp, r_block, g_mblock);
      } /* block roop */

  } /* intra mb */

  if (field_dct) {
    /* Convert luma from field DCT order to frame order */
    static Char inv_shuffle[] = {
      1,  0,  8,  1,  4,  8,  2,  4,  0,  2,  3,  0,  9,  3, 12,  9,  6, 12,
      0,  6,  5,  0, 10,  5,  0, 10,  7,  0, 11,  7, 13, 11, 14, 13,  0, 14
    };
    Int tmp[MB_SIZE];
    Int ii, jj;
    
    for (n = 0; n < sizeof(inv_shuffle); n += 2)
      memcpy(inv_shuffle[n+1] ? &g_mblock[inv_shuffle[n+1]*MB_SIZE] : &tmp[0],/*Check warning*/
	     inv_shuffle[n+0] ? &g_mblock[inv_shuffle[n+0]*MB_SIZE] : &tmp[0],/*Check warning*/
	     MB_SIZE * sizeof(SInt));
    

  }

  if ((mode == MODE_INTRA)||(mode == MODE_INTRA_Q))
    g_MB_clip_Studio (g_mblock,
		      GetVopMinimumAlphaLevel_st(rec_vop,aux_comp_num),
		      GetVopMaximumAlphaLevel_st(rec_vop,aux_comp_num));
/*    g_MB_clip_Studio (g_mblock, GetVopGrayOpaque_Studio(rec_vop,aux_comp_num));*/
}                               /* GetMBgraydata */

Void GetMBgraydata_DCT_StudioFDAM(Int aux_comp_num,
			      Int transp_pattern[],
			      Int intra_dcpred_disable,
			      Int *quant_step,
			      Int intra_dc_precision,
			      Int q_scale_type,
			      Int interlaced,
			      Int frame_pred_frame_dct,
			      Int field_dct,
			      Bitstream *texture_bits,
			      Int mbnum,
			      Trace *trace,
			      SInt *g_mblock,
			      Int MB_width,
			      Int ***DC_store,
			      SInt *QP_store,
			      Int *error_flag,
			      UInt mode,
			      Int *DC_prev,
			      Vop *rec_vop)
{
  Int  comp;
  Int  DCT_block[B_SIZE * B_SIZE];
  Int  q_block[B_SIZE * B_SIZE];
  Int  r_block[B_SIZE * B_SIZE];
  Int  r_block_2D[B_SIZE][B_SIZE];
  Int  pcoeff[B_SIZE * B_SIZE];
  Int  pred_DC_coeff;
  Int  x_pos = mbnum % MB_width;
  Int  y_pos = mbnum / MB_width;
  Int  m, n;
  Int  CBPA;
  Int ACpred_flag = -1;
  Int QP;

  Int vop_structure = GetVopVopStructure_st(rec_vop);
  Int field_parity = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(rec_vop), GetVopTopFieldFirst(rec_vop));
  Int tmpvar;

  Int a_uncompress_flag, a_ucbp_code;
  Int tmp_dc;
  Int k, i;
  Int bits_per_pixel = GetVopGrayBitsPerPixel_st(rec_vop,aux_comp_num);
  int ii, jj;

  Int dc_reset_value = pow(2,GetVopGrayBitsPerPixel_st(rec_vop,aux_comp_num)+GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity)+GetVopGrayIntraDCPrecision_st(rec_vop,aux_comp_num,field_parity)-1);

  QP = *quant_step;

  g_MBvalue (g_mblock, 0);

  if(mode == MODE_INTRA || mode == MODE_INTRA_Q) {
    CBPA = 15;
  } else {/*inter*/
    /* Decode Alpha MB header */
    CBPA = VlcDecCBPY (texture_bits, 1, transp_pattern,error_flag, trace);
    if (*error_flag == RETURN)
      {
	printf ("Error decoding CBPA of grayscale alpha block %d\n", (Int)mbnum);
	return;
      }
fprintf(stdout, "mb[%d] : cbpa = %d\n", mbnum,CBPA);
  }

  if(CBPA) {
    tmpvar = (Int) BitstreamReadBits(texture_bits,1,"load_alpha_quantizer_scale_code",trace,CODE);
    if(tmpvar == 1) {
      tmpvar = (Int) BitstreamReadBits(texture_bits,5,"alpha_quantizer_scale_code",trace,CODE);
      QP = *quant_step = tmpvar;
    }
  }
  
  /*****
   *
   *      Decoding of the 4 GREY_SCALE blocks (depending on transparent pattern)
   *
   *****/

  if (mode == MODE_INTRA || mode == MODE_INTRA_Q) {
    for (comp=0; comp<4; comp++)
      {   
        Bzero (q_block);

        if(transp_pattern[comp]!=1) /* No decoding of transparent Y blocks */
	  {
	    
	    /* This must be Int not UInt MW 15-NOV-1996 */
	    pred_DC_coeff = DecodePredictedIntraDC_bySVLC_Studio(comp,
							  texture_bits,
							  trace,
							  NUM);

	    q_block[0] = pred_DC_coeff;

	      VlcGetBlock_bySVLC_Studio(texture_bits, q_block, INTRA,
					trace, error_flag,
					bits_per_pixel+GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity)+4);

	    q_block[0] += *DC_prev;
	    *DC_prev = q_block[0];

	    /* Zigzag scanning */
	    if (GetVopGrayAlternateScan_st(rec_vop,aux_comp_num,field_parity)) {
	      for (m = 0; m < 64; m ++)
		pcoeff[m] = q_block[zigzag_v[m]];
	    } else {
	      /* Ordinary inverse zigzag-scanning */
	      for (m = 0; m < 64; m++)
		pcoeff[m] = q_block[zigzag[m]];
	    }
	    for (m = 0; m < 8; m++)
	      for (n = 0; n < 8; n++)
		q_block[m*8+n] = pcoeff[m*8+n];

	    
	    /* Store the qcoeff-values needed later for prediction */
	    DC_store[mbnum][comp][0] = q_block[0] * cal_dc_scaler_Studio(intra_dc_precision);
	    
	    for (m = 1; m < 8; m++)
	      DC_store[mbnum][comp][m] = q_block[m];
	    
	    for (m = 0; m < 7; m++)
	      DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
	    
	    /* MW 22-APR-1997 Changes for MPEG dequantization */
	    if(1/*GetVopQuantType(rec_vop)*/)
	      BlockDequantMPEG_DCText_DeadZone_Studio(q_block,*quant_step,INTRA,1,intra_dc_precision,q_scale_type,
						      GetVopGrayIntraQuantMat_st(rec_vop,aux_comp_num),
						      GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity),
						      bits_per_pixel,
						      GetVopDeadZoneDisable_st(rec_vop,field_parity),
						      DCT_block,1/*2*/,
						      0);

	    BlockIDCT_DCText_Studio(DCT_block, r_block_2D, GetVopGrayOpaque_Studio(rec_vop,aux_comp_num),0);
	    
	    for (m = 0; m < B_SIZE; m++)
	      for (n = 0; n < B_SIZE; n++)
		r_block[m*B_SIZE+n] = r_block_2D[m][n];


	      }
      else /* clear r_block if transparent block */
        {

	  *DC_prev = dc_reset_value;
	  
	  Bzero(r_block);
 
          /* Fill DC_store with default coeff values */
          DC_store[mbnum][comp][0] = GetVopGrayMidValue_Studio(rec_vop,aux_comp_num)*8;	  
          for(n=1; n<15; n++)
            DC_store[mbnum][comp][n] = 0;
        }
         
      PutBlockG (comp, r_block, g_mblock);
      } /* block roop */

  } /* intra mb */
  else /* MODE INTER */
  {
    
    *DC_prev = GetVopGrayMidValue_Studio(rec_vop,aux_comp_num)*8/cal_dc_scaler_Studio(intra_dc_precision);
      
    for (comp=0; comp<4; comp++) {
 
      Bzero (q_block);
 
      if(transp_pattern[comp]!=1) { /* No decoding of transparent Y blocks */

	if ((CBPA>>(3-comp)) & 1) {
           VlcGetBlock_bySVLC_Studio (texture_bits, q_block, INTER, trace,
				      error_flag,
				      bits_per_pixel+GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity)+4);
        }

	/* Zigzag scanning */
	if (GetVopGrayAlternateScan_st(rec_vop,aux_comp_num,field_parity)) {
	  for (m = 0; m < 64; m ++)
	    pcoeff[m] = q_block[zigzag_v[m]];
	} else {
	  /* Ordinary inverse zigzag-scanning */
	  for (m = 0; m < 64; m++)
	    pcoeff[m] = q_block[zigzag[m]];
	}
	for (m = 0; m < 8; m++)
	  for (n = 0; n < 8; n++)
	    q_block[m*8+n] = pcoeff[m*8+n];
	
	/* Fill DC_store with default coeff values */
	DC_store[mbnum][comp][0] = GetVopGrayMidValue_Studio(rec_vop,aux_comp_num)*8;
	for (n = 1; n < 15; n++)
	  DC_store[mbnum][comp][n] = 0;

	/* MW 22-APR-1997 Changes for MPEG dequantization */
	if(1/*GetVopQuantType(rec_vop)*/)
	  BlockDequantMPEG_DCText_DeadZone_Studio(q_block,*quant_step,INTER,1,intra_dc_precision,q_scale_type,
						  GetVopGrayNonintraQuantMat_st(rec_vop,aux_comp_num),
						  GetVopGrayDCTPrecision_st(rec_vop,aux_comp_num,field_parity),
						  bits_per_pixel,
						  GetVopDeadZoneDisable_st(rec_vop,field_parity),
						  DCT_block,2,
						  0);
	else
	  BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,2,0, GetVopBitsPerPixel(rec_vop));
	
	BlockIDCT_DCText_Studio(DCT_block, r_block_2D, GetVopGrayOpaque_Studio(rec_vop,aux_comp_num),0);
	
	for (m = 0; m < B_SIZE; m++)
	  for (n = 0; n < B_SIZE; n++)
	    r_block[m*B_SIZE+n] = r_block_2D[m][n];
	
	*DC_prev = dc_reset_value;
      }
    else /* clear r_block if transparent block */
      {  
          *DC_prev = dc_reset_value;
	
          Bzero(r_block);
 
          /* Fill DC_store with default coeff values */
          DC_store[mbnum][comp][0] = GetVopGrayMidValue_Studio(rec_vop,aux_comp_num)*8;
          for(n=1; n<15; n++)
            DC_store[mbnum][comp][n] = 0;
	}
      
      PutBlockG (comp, r_block, g_mblock);
    } /* block roop */
  } /* inter mb */

  if (field_dct) {
    /* Convert luma from field DCT order to frame order */
    static Char inv_shuffle[] = {
      1,  0,  8,  1,  4,  8,  2,  4,  0,  2,  3,  0,  9,  3, 12,  9,  6, 12,
      0,  6,  5,  0, 10,  5,  0, 10,  7,  0, 11,  7, 13, 11, 14, 13,  0, 14
    };
    Int tmp[MB_SIZE];
    Int ii, jj;
    
    for (n = 0; n < sizeof(inv_shuffle); n += 2)
      memcpy(inv_shuffle[n+1] ? &g_mblock[inv_shuffle[n+1]*MB_SIZE] : &tmp[0],/*Check warning*/
	     inv_shuffle[n+0] ? &g_mblock[inv_shuffle[n+0]*MB_SIZE] : &tmp[0],/*Check warning*/
	     MB_SIZE * sizeof(SInt));
  }

  if ((mode == MODE_INTRA)||(mode == MODE_INTRA_Q))
    g_MB_clip_Studio (g_mblock,
		      GetVopMinimumAlphaLevel_st(rec_vop,aux_comp_num),
		      GetVopMaximumAlphaLevel_st(rec_vop,aux_comp_num));
/*    g_MB_clip_Studio (g_mblock, GetVopGrayOpaque_Studio(rec_vop,aux_comp_num));*/
}                               /* GetMBgraydata */

#if 0
Void GetMBgraydata_Uncompress_Studio(Int aux_comp_num,
				  Int transp_pattern[],
				  Int intra_dcpred_disable,
				  Int *quant_step,
				  Int intra_dc_precision,
				  Int q_scale_type,
				  Int interlaced,
				  Int frame_pred_frame_dct,
				  Bitstream *texture_bits,
				  Int mbnum,
				  Trace *trace,
				  SInt *g_mblock,
				  Int MB_width,
				  Int ***DC_store,
				  SInt *QP_store,
				  Int *error_flag,
				  UInt mode,
				  Int *DC_prev,
				  Vop *rec_vop)
{
  Int  comp;
  Int  DCT_block[B_SIZE * B_SIZE];
  Int  q_block[B_SIZE * B_SIZE];
  Int  r_block[B_SIZE * B_SIZE];
  Int  r_block_2D[B_SIZE][B_SIZE];
  Int  pcoeff[B_SIZE * B_SIZE];
  Int  pred_DC_coeff;
  Int  x_pos = mbnum % MB_width;
  Int  y_pos = mbnum / MB_width;
  Int  m, n;
  Int  CBPA;
  Int ACpred_flag = -1;
  Int QP;

  Int field_dct=0;
  Int vop_structure = GetVopVopStructure_st(rec_vop);
  Int field_parity = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(rec_vop), GetVopTopFieldFirst(rec_vop));
  Int tmpvar;

  /*WD2.1_2*/
  Int a_uncompress_flag, a_ucbp_code;
  Int tmp_dc;
  Int k, i;
  Int bits_per_pixel = GetVopGrayBitsPerPixel_st(rec_vop,aux_comp_num);
  /*WD2.1_2*/

  
  g_MBvalue (g_mblock, 0);

  /*****
   *
   *      Decoding of the 4 GREY_SCALE blocks (depending on transparent pattern)
   *
   *****/

  if (mode == MODE_INTRA || mode == MODE_INTRA_Q) {
    for (comp=0; comp<4; comp++)
      {   
        Bzero (q_block);

        if(transp_pattern[comp]!=1) /* No decoding of transparent Y blocks */
	  {

                  /* Read Raw Block data from bitstream */
                  for(i=0;i<64;i++) {
                    r_block[i] = BitstreamReadBits(texture_bits, (long)bits_per_pixel, "alpha_pixel_data", trace, NUM);
                    if(BitstreamReadBits(texture_bits, 1L, "marker_bit", trace, NUM)==0) {/*marker_bits*/
                      fprintf(stderr, "marker_bit shall be 1\n");
                      exit(-1);
                    }
                  }
                  tmp_dc = 0;
                  for(i=0;i<64;i++)
                    tmp_dc += r_block[i];
                  *DC_prev = tmp_dc/(8*cal_dc_scaler_Studio(intra_dc_precision));


	      }
      else /* clear r_block if transparent block */
        {

	  *DC_prev = GetVopGrayMidValue_Studio(rec_vop,aux_comp_num)*8/cal_dc_scaler_Studio(intra_dc_precision);
	  
	  Bzero(r_block);
 
          /* Fill DC_store with default coeff values */
/*          DC_store[mbnum][comp][0] = MIDOPAQUE*8;*/
          DC_store[mbnum][comp][0] = GetVopGrayMidValue_Studio(rec_vop,aux_comp_num)*8;	  
          for(n=1; n<15; n++)
            DC_store[mbnum][comp][n] = 0;
        }
         
      PutBlockG (comp, r_block, g_mblock);
      } /* block roop */
  } /* intra mb */
#if 0
  else /* MODE INTER */
  {
    for (comp=0; comp<4; comp++) {
 
      Bzero (q_block);
 
      if(transp_pattern[comp]!=1) { /* No decoding of transparent Y blocks */
        if ((CBPA>>(3-comp)) & 1) {
           VlcGetBlock (texture_bits, q_block, INTER, trace,
                        error_flag, 0, GetVopReverseVlc(rec_vop),0,0);
        }
 
      /* Inverse zigzag-scanning also here 12-11-96 MW */
      for (m = 0; m < 64; m++)
          pcoeff[m] = q_block[zigzag[m]];
 
      for (m = 0; m < 8; m++)
        for (n = 0; n < 8; n++)
          q_block[m*8+n] = pcoeff[m*8+n];
 
      /* Fill DC_store with default coeff values */
      DC_store[mbnum][comp][0] = MIDOPAQUE*8;
      for (n = 1; n < 15; n++)
        DC_store[mbnum][comp][n] = 0;
 
      /* MW 22-APR-1997 Changes for MPEG dequantization */
      if(GetVopQuantType(rec_vop))
         BlockDequantMPEG(q_block,quant_step,INTER,1,
                       GetVopNonintraQuantMat(rec_vop),DCT_block,2);
      else
        BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,2,0, GetVopBitsPerPixel(rec_vop));
 
      BlockIDCT(DCT_block, r_block_2D, OPAQUE);
 
      for (m = 0; m < B_SIZE; m++)
        for (n = 0; n < B_SIZE; n++)
          r_block[m*B_SIZE+n] = r_block_2D[m][n];
    }
    else /* clear r_block if transparent block */
      {  
          Bzero(r_block);
 
          /* Fill DC_store with default coeff values */
          DC_store[mbnum][comp][0] = MIDOPAQUE*8;
          for(n=1; n<15; n++)
            DC_store[mbnum][comp][n] = 0;
      }
       
      PutBlockG (comp, r_block, g_mblock);
    } /* block roop */
  } /* inter mb */
#endif 0

  if ((mode == MODE_INTRA)||(mode == MODE_INTRA_Q))
    g_MB_clip_Studio (g_mblock, GetVopGrayOpaque_Studio(rec_vop,aux_comp_num));
}                               /* GetMBgraydata */
#endif 0
/* 01.04.01: Studio Profile */
