/*****************************************************************************
 *
 * This software module was originally developed by
 *
 * Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/
/***********************************************************HeaderBegin*******
 *                                                                         
 * File:        studio_function_dec.c
 *
 * Author:      Osamu Sunohara (SONY)
 * Created:     01.04.01
 *                                                                         
 * Description:
 *
 * Notes:       
 *
 * Modified:
 *
 ***********************************************************HeaderEnd*********/
#include "momusys.h"
#include "mom_bitstream_d.h"
#include "mom_bitstream_i.h"
#include "vm_common_defs.h"
#include "studio_function_dec.h"

Void GetExtendedCBPC_dec_Studio(Int CBPC, Int *cbpcb_ext, Int *cbpcr_ext,
		     Bitstream * texture_bits, Trace *trace,
		     Int *MB_transp_pattern_uv, Int chroma_format,Int method)
{
  Int coded_block=0, read_bits;
  Int cbpcb = (CBPC >> 1) & 1;
  Int cbpcr = CBPC & 1;
  Int i;
  
  if(chroma_format == CHROMA444)
    {
	for(i=0;i<4;i++)
	  if(MB_transp_pattern_uv[i]==0)
	    coded_block++;
    }
  else if(chroma_format == CHROMA422)
    {
      if(MB_transp_pattern_uv[0]==0)
	coded_block++;
      if(MB_transp_pattern_uv[2]==0)
	coded_block++;
    }
  else
    return;

  if(method == 1) {
    if(cbpcb)
      *cbpcb_ext = BitstreamReadBits(texture_bits, coded_block, "CBPCb", trace, FLAG);
    if(cbpcr)
      *cbpcr_ext = BitstreamReadBits(texture_bits, coded_block, "CBPCr", trace, FLAG);
  } else if(method == 2) {
    Int cbpc_ext=0;
    read_bits = 2*(coded_block-1);
    if(read_bits<0) {
      fprintf(stderr, "This MB is all_0 alpha map, But in this function the MB like this is not supported.(%d)\n",read_bits);
      exit(3);
    }

    if(read_bits>0)
      cbpc_ext = BitstreamReadBits(texture_bits, read_bits, "cbpc_extension", trace, FLAG);

    cbpc_ext = (CBPC<<read_bits) | cbpc_ext;

    *cbpcb_ext = (((cbpc_ext>>4)&8)|((cbpc_ext>>3)&4)|((cbpc_ext>>2)&2)|((cbpc_ext>>1)&1));
    *cbpcr_ext = (((cbpc_ext>>3)&8)|((cbpc_ext>>2)&4)|((cbpc_ext>>1)&2)|(cbpc_ext&1));
  }
  
  return;
}

Int CalcCBP_dec_Studio(Int CBPY, Int CBPC, Int cbpcb_ext, Int cbpcr_ext,
	     Int *MB_transp_pattern_uv, Int chroma_format)
{
  
  Int coded_block=0;
  Int CBP=0;
  Int cbpc_ext[8]={0,0,0,0,0,0,0,0}; /* coded block pattern for block 4,5,...,11 */
  Int cnv[4]={0,2,1,3};
  Int block_roop, i;
  
  if(chroma_format == CHROMA444 || chroma_format == CHROMA422)
    {
      if(chroma_format == CHROMA444)
	{
	  for(i=0;i<4;i++)
	    if(MB_transp_pattern_uv[i]==0)
	      coded_block++;
	  block_roop = 4;
	}
      else if(chroma_format == CHROMA422)
	{
	  if(MB_transp_pattern_uv[0]==0)
	    coded_block++;
	  if(MB_transp_pattern_uv[2]==0)
	    coded_block++;
	  block_roop = 2;
	}
      
      for(i=0;i<block_roop;i++)
	{
	  if(MB_transp_pattern_uv[cnv[i]]==0)
	    {
	      cbpc_ext[2*i]=(cbpcb_ext>>(coded_block-1))&1;
	      cbpc_ext[2*i+1]=(cbpcr_ext>>(coded_block-1))&1;
	      coded_block--;
	    }
	}
  
      CBP = cbpc_ext[4]<<11 | cbpc_ext[5]<<10 | cbpc_ext[6]<<9 | cbpc_ext[7]<<8
	| cbpc_ext[2]<<7 | cbpc_ext[3]<<6 | CBPY << 2 | cbpc_ext[0]<<1 | cbpc_ext[1];
    }
  else if(chroma_format == CHROMA420)
    CBP = CBPY << 2 | CBPC;

  return CBP;
}
