/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Karl Lillevold (Telenor / ACTS-MoMuSyS)
 *
 * and edited by
 * 
 *   Paulo Nunes (IST / ACTS-MoMuSyS)
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)
 *   Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	vm_get_blk_studio.c
 *
 * Author:	
 * Created:	
 *                                                                         
 * Description: 
 *
 * Notes: 	The functions contained in this file were adapted from
 *   		tmndecode 
 * 		Written by Karl Olav Lillevold <kol@nta.no>,
 *		1995 Telenor R&D.
 * 		Donated to the Momusys-project as background code by
 *		Telenor. 
 *   		
 *   		based on mpeg2decode, (C) 1994, MPEG Software Simulation Group
 *   		and mpeg2play, (C) 1994 Stefan Eckart
 *   					<stefan@lis.e-technik.tu-muenchen.de>
 *
 *
 * Modified:	7-May-96 Paulo Nunes: Reformatted. New headers.
 *              02.05.97 Luis Ducla-Soares: enabled the use of Reversible VLCs.
 *              23.06.98 G. Klungsoeyr/A. Sandvand: added support for 
 *                                                  short video header
 *              01.04.01 Osamu Sunohara(SONY): Functions for Studio Profile
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include "vm_get_blk_studio.h"
#include "vm_vlc_dec_studio.h"

/***********************************************************CommentBegin******
 *
 * -- UInt DecodePredictedIntraDC_bySVLC_Studio --
 *
 * Author :     Osamu Sunohara (SONY)
 *
 * Created :    01.04.01
 *
 *
 * Purpose : To decode the predicted intra dc for intra macroblocks
 *
 * Arguments in : Int mbnum : macroblock number
 *                Int compnum : block number (0..5)
 *                Bitstream *stream : a pointer to the bitstream
 *                Trace *trace : trace info
 *                Int x_width : horizontal dimension of the VOP
 *                UInt *new_vop : indicates if this MB is the first MB for
 *                                this VOP
 *                Int trace_mode : mode info. for trace
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *
 *
 * Return values : UInt DC_coeff : the dc coefficient for one block
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also : This algorithm is based upon VM2.0.
 *            Not compatible with VM2.1
 *
 *
 * Modified :
 *
 ***********************************************************CommentEnd********/
Int DecodePredictedIntraDC_bySVLC_Studio(Int compnum,
					 Bitstream *stream,
					 Trace *trace,
					 Int trace_mode)
{
  UInt DC_size;
  UInt code;
  Int INTRADC_delta;
  Int error_flag=0;
  Int first_bit;
  
  DC_size = VlcDecIntraDCPredSize_bySVLC_Studio(stream,
						&error_flag,compnum,trace);
  
  if ( error_flag == RETURN) {
    fprintf(stderr, "Error decoding INTRADC pred. size\n");
    return -1;
  }
  
  if (DC_size == 0) {
    INTRADC_delta = 0;
  } else {
    if(compnum<4)
      code = BitstreamReadBits(stream,DC_size,"DC coeff(Y)",trace,trace_mode);
    else
      code = BitstreamReadBits(stream,DC_size,"DC coeff(C)",trace,trace_mode);
      
    first_bit = code >> (DC_size-1);

    if (first_bit == 0 ) { /* negative delta INTRA DC */
      INTRADC_delta  = -1 * (code ^( (Int)pow(2.0,(Double)DC_size)-1) );
    } else { /* positive delta INTRA DC */
      INTRADC_delta = code;
    }
    if (DC_size > 8)
      (Void) BitstreamReadBits(stream,1,"Marker bit",trace,trace_mode);
  }

  return INTRADC_delta;
}


/***********************************************************CommentBegin******
 *
 * -- VlcGetBlock_bySVLC_Studio -- Decodes the DCT coefficients of one 8x8 block
 *
 * Author :	Osamu Sunohara(SONY)
 * Created :	01.04.01
 *	
 * Purpose :		
 *	To decode the DCT coefficients of one 8x8 block.
 * 
 * Arguments in : 	
 *	Bitstream *texture_bits,	the input bitstream.
 *	SInt      inter_flag,		Inter coded mode flag. 
 *	Int        trace,		Flag to help trace.
 *	
 * Arguments in/out :	
 *      Int short_video_header          Flag to signal short video header 
 *                                      bitstreams (H.263)
 *	
 * Arguments out :	
 *	Int       *coeff_block,		block of DCT coefficients.
 *	Int       *error_flag,		error flag.
 *	
 * Return values :	
 *	
 * Side effects :	
 *	
 * Description :	
 *	
 * See also :
 *	
 * Modified :		
 *
 ***********************************************************CommentEnd********/

Void
VlcGetBlock_bySVLC_Studio (Bitstream *texture_bits,
			   Int *coeff_block,
			   SInt inter_flag,
			   Trace *trace,
			   Int *error_flag,
			   Int flc_length
			   )
{
  Int vlc_table;
  Int symbol_group, symbol_size, add_code;
  Int run_length, level_value;
  
  Int i = (inter_flag == INTRA);

  vlc_table = 0;
  i = (inter_flag == INTRA);
  /* 01.04.01: Studio Profile: BUGFIX */
  while(1/*i<64*/) {

    /* Decode symbol group VLC */
    symbol_group = VlcDecSymbolGroup_bySVLC_Studio(vlc_table,texture_bits,
						   error_flag, trace, inter_flag==INTRA);
/*if(vlc_table==16)fprintf(stderr, "symbol_group=%d\n", symbol_group);*/
    if(*error_flag)
      return;

    if(symbol_group==0) { /* EOB Symbol */
      vlc_table = 0;
      break;
    } else {
      /* 01.04.01: Studio Profile: BUGFIX */
      if(i==64) {
	fprintf(stderr, "EOB code shall come at the last of the block,prev_gp = %d\n",vlc_table);
	exit(-1);
      }

      
      if(symbol_group>0 && symbol_group<7) { /* zeroone group */
	symbol_size = symbol_group;
	vlc_table = 16;
      } else if(symbol_group>7 && symbol_group<14) { /* zerorun group */
	symbol_size = symbol_group - 8;
	vlc_table = 8;
      } else if(symbol_group>16 && symbol_group<25) { /* level group */
	symbol_size = symbol_group - 16;
	vlc_table = symbol_group;
      } else if(symbol_group==25) { /* ESCAPE mode */
	symbol_size = flc_length;
	vlc_table = symbol_group;
      }
      
      /* Read an additional code */
      add_code = BitstreamReadBits(texture_bits,symbol_size,"additional code",trace,CODE);
      if(symbol_size==0 && add_code!=0) {
	fprintf(stderr, "WRONG!!\n");
	exit(-5);
      }
      
      if(symbol_group>0 && symbol_group<7) { /* zeroone group */

#if 0 /***BUGFIX***/
	/* Decode run_length and level_value */
	if(add_code>>(symbol_size-1)) {
	  run_length = add_code;
	  level_value = 1;
	} else {
	  run_length = add_code ^( (Int)pow(2.0,(Double)symbol_size)-1 );
	  level_value = -1;
	}
#endif 0 /***BUGFIX***/
	
	run_length = (1<<symbol_size-1) | (add_code>>1);
	if(add_code&1)
	  level_value = 1;
	else
	  level_value = -1;


	/* Update coeff values */
	i += run_length;
	if (i >= 64) {
	  *error_flag = 1;
          printf ("TOO MUCH COEFF !!");
	  return;
	}

	coeff_block[i] = level_value;
	
      } else if(symbol_group>7 && symbol_group<14) { 

	/* Decode run_length */
	run_length = (1<<symbol_size) | add_code;

	/* Update coeff values */
	i += run_length - 1;
	if (i >= 64) { /*...*/
	  *error_flag = 1;
          printf ("TOO MUCH COEFF !!");
	  return;
	}
	
      } else if(symbol_group>16 && symbol_group<25) { /* level group */

	/* Decode level_value */
	if(add_code>>(symbol_size-1)) {
	  level_value = add_code;
	} else {
	  level_value = -1 * (add_code ^( (Int)pow(2.0,(Double)symbol_size)-1) );
	}

	/* Update coeff values */
	coeff_block[i] = level_value;
	
      } else if(symbol_group==25) { /* ESCAPE mode */
	
	/* Decode level_value */
	if(add_code>>(flc_length-1)) { /* a negative level */
	  level_value = -1 * ( ( add_code^((Int)pow(2.0,(Double)flc_length)-1) ) + 1 );
	} else { /* a positive value */
	  level_value = add_code;
	}

	/* MoMuSys-2000FDAM1COR-020612_1 */
	if ( add_code >> flc_length - 1  && ! ( add_code & ( 1 << flc_length -1 ) - 1 ) ) {
	  fprintf ( stderr, "Sequence is 10...0, reserved\n" );
	  exit(-3);
	}
	if ( level_value > ((Int)pow(2.0,(Double)(flc_length-1))-1) ||
	     ( level_value > -256 && level_value < 256 ) ||
	     level_value < -((Int)pow(2.0,(Double)(flc_length-1))-1) ) {
	  fprintf ( stderr, "escape value = %d\n", level_value );
	  exit(-3);
	}
	/**/

	/* Update coeff values */
	coeff_block[i] = level_value;
      }
    }

    /* Update the position counter */
    i++;
  }
  
}
