/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Paulo Nunes (IST / ACTS-MoMuSyS)
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)
 *
 *   and edited by
 *
 *   Sven Brandau (HHI)
 *   Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:        vm_vlc_dec_studio.c
 *
 * Author:      Osamu Sunohara(SONY)
 * Created:     01.04.01
 *                                                                         
 * Description: This file contains the VLC functions needed to decode a
 *              bitstream conforming studio profile syntax. 
 *
 * Notes:       
 *              The functions contained in this file were adapted from
 *              tmndecode
 *              Written by Karl Olav Lillevold <kol@nta.no>,
 *              1995 Telenor R&D.
 *              Donated to the Momusys-project as background code by
 *              Telenor. 
 *
 *              based on mpeg2decode, (C) 1994, MPEG Software Simulation Group
 *              and mpeg2play, (C) 1994 Stefan Eckart
 *                                      <stefan@lis.e-technik.tu-muenchen.de>
 *
 *
 * Modified:
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include "vm_vlc_dec_studio.h"
/* 01.04.01: Studio Profile: BUGFIX */
/*
#include "vm_vlc_dec_tab_studio.h"
*/

/***********************************************************CommentBegin******
 *
 * -- UInt VlcDecIntraDCPredSize_bySVLC_Studio --
 *
 * Author : Cor Quist, KPN Research - The Netherlands.
 *
 *
 * Created : 23-May-1996
 *
 *
 * Purpose : VLC decoding of the SIZE field off the predicted INTRA DC
 *           coefficient.
 *
 *
 * Arguments in : Bitstream *stream : a pointer to the bitstream
 *                Int *error_flag : indicates the occurence of an error
 *                Int compnum : the block number (0..5)
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *
 *
 * Return values : UInt DC_size : The number of bits of the INTRA DC
 *                                difference
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also : This algorithm is based upon VM2.0.
 *            Not compatible with VM2.1
 *
 *
 * Modified :
 *  01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 *
 ***********************************************************CommentEnd********/

UInt
VlcDecIntraDCPredSize_bySVLC_Studio (Bitstream *stream, Int *error_flag,
			      Int compnum, Trace *trace) {
  Int code;
  Int value;
  Int length;
  Int dec_group_no;
  Int i;
  
  code = BitstreamShowBits (stream, 16);/*max length must be equal or less than 16*/
  dec_group_no = -1;
  
  for(i=0;i<19;i++) {
/* 01.04.01: Studio Profile: BUGFIX */
    if(compnum<4) {
      value = GetStudioDCLumVLCCode_Studio(i);
      length = GetStudioDCLumVLCLength_Studio(i);
    } else {
      value = GetStudioDCChromVLCCode_Studio(i);
      length = GetStudioDCChromVLCLength_Studio(i);
    }
/*   
    if(compnum<4) {
      value = studio_dc_lum_table_dec[i].val;
      length = studio_dc_lum_table_dec[i].len;
    } else {
      value = studio_dc_chrom_table_dec[i].val;
      length = studio_dc_chrom_table_dec[i].len;
    }
*/

    if(length) {
      if( value == code>>(16-length) ) {

	/* VLC match!! */
	dec_group_no = i;
	
	BitstreamFlushBits(stream, length);
/* 01.04.01: Studio Profile: TEMPORALLY removed */
/*	
    if(compnum<4) {
      BitstreamTraceDCSize_Studio(studio_dc_lum_table_dec[dec_group_no],
				  "DCT DC size VLC(Y)", trace);
    } else {
      BitstreamTraceDCSize_Studio(studio_dc_chrom_table_dec[dec_group_no],
				  "DCT DC size VLC(C)", trace);
    }
*/    
	break;
      }
    }

  }

  if(dec_group_no==-1) {
    *error_flag = 1;
    fprintf(stderr, "NO group was found.\n");
    return -1;
  }

 return dec_group_no;
}

/***********************************************************CommentBegin******
 *
 * -- VlcDecSymbolGroup_bySVLC_Studio -- Decodes a VLC coded symbol group
 *
 * Author :      Osamu Sunohara(SONY Corp.)
 *
 * Created :     01.04.01
 *
 * Purpose :            
 *      To decode a VLC coded symbol group.
 * 
 * Arguments in :       
 *      Bitstream * stream,             input bitstream.
 *      Int intra_luma                  flag to indicate table to use
 *
 *
 * Arguments in/out :   
 *      
 *
 * Arguments out :      
 *      Int * error_flag,               flag to signal an invalid group
 *                                      code.
 *
 * Return values :      
 *      A value of a symbol group.
 *
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *
 ***********************************************************CommentEnd********/
Int
VlcDecSymbolGroup_bySVLC_Studio(Int vlc_table_no, Bitstream *stream,
				Int *error_flag, Trace *trace, Int intra)
{
  Int code;
  Int value;
  Int length;
  Int dec_group_no;
  Int i;

  VLCtab tmpv;
  
  code = BitstreamShowBits (stream, 16);/*max length must be equal or less than 16*/
  dec_group_no = -1;
  
  for(i=0;i<32;i++) {

    if(intra) {
      
/* 01.04.01: Studio Profile: BUGFIX */
      value = GetStudioGroupVLCCodeIntra_Studio(vlc_table_no,i);
      length = GetStudioGroupVLCLengthIntra_Studio(vlc_table_no,i);
/*      
      value = studio_group_table_dec[vlc_table_no][i].val;
      length = studio_group_table_dec[vlc_table_no][i].len;
*/      
      
      if(length) {
	if( value == code>>(16-length) ) {
	  
	  /* VLC match!! */
	  dec_group_no = i;
	  
	  BitstreamFlushBits(stream, length);
/* 01.04.01: Studio Profile: TEMPORALLY removed */
/**/
	  tmpv.val = value;
	  tmpv.len = length;
	  BitstreamTraceSymbolGroup_Studio(vlc_table_no,
					   tmpv,/*studio_group_table_dec[vlc_table_no][dec_group_no],*/
					   "Symbol Group VLC Intra", trace);
/**/	  
	  break;
	}
      }
    } else {

/* 01.04.01: Studio Profile: BUGFIX */
      value = GetStudioGroupVLCCodeInter_Studio(vlc_table_no,i);
      length = GetStudioGroupVLCLengthInter_Studio(vlc_table_no,i);
/*      
      value = studio_group_table_dec_inter[vlc_table_no][i].val;
      length = studio_group_table_dec_inter[vlc_table_no][i].len;
*/
      
      if(length) {
	if( value == code>>(16-length) ) {
	  
	  /* VLC match!! */
	  dec_group_no = i;
	  
	  BitstreamFlushBits(stream, length);
/* 01.04.01: Studio Profile: TEMPORALLY removed */
/**/	  
	  tmpv.val = value;
	  tmpv.len = length;
	  BitstreamTraceSymbolGroup_Studio(vlc_table_no,
					   tmpv,/*studio_group_table_dec_inter[vlc_table_no][dec_group_no],*/
					   "Symbol Group VLC Inter", trace);
/**/					   
	  break;
	}
      }
    }
    
  }

  if(dec_group_no==-1) {
    *error_flag = 1;
    fprintf(stderr, "NO group was found.\n");
    return -1;
  }

  return dec_group_no;
}

/* For tracing: subroutines are defined in mom_trace.c  */
Void
BitstreamTraceSymbolGroup_Studio(Int vlc_table_no, VLCtab vlc, Char *s, Trace *trace) {
#ifdef _DEBUG_
  if(trace->trace) {
    fprintf(trace->fp_trace, "***  VLC table NO.%2d  ***\n", vlc_table_no);
    trace_routine(s, vlc.val, btos(vlc.val, vlc.len), trace, CODE);
  }
#endif
}

Void
BitstreamTraceDCSize_Studio(VLCtab vlc, Char *s, Trace *trace) {
#ifdef _DEBUG_
  if(trace->trace) {
    trace_routine(s, vlc.val, btos(vlc.val, vlc.len), trace, CODE);
  }
#endif
}



  
