/******************************************************************************
 *
 * This software module was originally developed by
 *
 *      Ji Heon Kweon (HYUNDAI)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1998
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *
 * File:        alp_code_grey.c
 *
 * Author:      HYUNDAI - Ji Hweon Kweon
 * Created:     21-05-98
 *
 * Description:
 *
 * Notes:
 *
 * Modified:
 *    Osamu Sunohara (SONY): Functions for Studio Profile
 *
 ***********************************************************HeaderEnd*********/

#include <stdio.h>
#include <sys/stat.h>
#include "momusys.h"
#include "mom_access.h"
#include "mom_image.h"
#include "vm_enc_defs.h"
#include "vm_common_defs.h"
#include "mot_est.h"            /* because of MBM_INTRA */
#include "mom_bitstream_i.h"
#include "text_util.h"
#include "text_bits.h"
#include "text_quant.h"
#include "putvlc.h"
#include "text_code_mb.h"
#include "alp_code_grey.h"
#include "zigzag.h"             /* added 14-NOV-1996 MW */

#include "vm_config.h"
#include "text_code.h"


/* 08.02.99 HHI Schueuer */
#include "sadct_bsnrmem.h"
#include "sadct.h"
#include "sadct_s_k.h"
#include "sadct_momusys.h"
#include "sadct_momusys_kaup.h"
#include "sadct_momusys_s_k.h"
/* end HHI */

#define OPAQUE 255
#define MIDOPAQUE 128


void
ObtainSupport(Image *AlphaPlain)
{
  Int i;
  Short *alpha = (Short *) GetImageData(AlphaPlain);
 
  for (i=0;i<GetImageSizeY(AlphaPlain)*GetImageSizeX(AlphaPlain);i++)
    alpha[i] = (alpha[i]>0) ? BINARY_ALPHA : 0;
}

void
InvertAlpha(Image *AlphaPlain)
{
  Int i;
  Short *alpha = (Short *) GetImageData(AlphaPlain);
 
  for (i=0;i<GetImageSizeY(AlphaPlain)*GetImageSizeX(AlphaPlain);i++)
    alpha[i] = (alpha[i]>0) ? 0 : BINARY_ALPHA;
}

void
CodeGrayscaleAlpha(
        Bits *bits,
        Vop *curr,
        Vop *rec_curr,
        Int MB_width,
        Int mb_x,
        Int mb_y,
        Int ***DC_store,
        SInt *QP_store,
        Int QP,
        Int Mode,
        Int MB_transp_pattern[4],
        Image *shape_bitstream,
        Int aux_comp )
{
  Int i,j,k,l,m,n;
  Image *mblock;
  Short *pixel;
  Int alpha_mb_all_opaque=0;
  Int alpha_residue_all_zero=0;
  Int fblock[8][8];
  Int coeff[64];
  Int *coeff_ind = coeff;
  Int qcoeff[256];
  Int *qcoeff_ind;
  Int rcoeff[256];
  Int *rcoeff_ind;
  Image *currentImage;
  SInt *recon;
  Int CBPA=0;                   /* JDL 21-APR-97 : initialised CBPA */
  Int block_nr;
  Int *qmat;
  Int direction[4];
  Int ACpred_flag = -1;
  Int dc_scaler;

  /* 05.02.99 HHI Schueuer */
  /* variables for the sadct */
  Short *alpha_pntr = (Short *)(GetImageData(GetVopA(rec_curr)));
  Short alpha_block[4][8][8];
  Short alpha_buf[4][8][8];
  Short nrpix = GetImageSizeX(GetVopA(rec_curr));
  Int active_pels[4];
  Int x1, y1;
  Int nr;
  /* end HHI */

  currentImage = GetVopG(aux_comp,curr);
  recon = (SInt *) GetImageData(GetVopG(aux_comp,rec_curr));

  
 
  mblock = AllocImage(16,16,SHORT_TYPE);
  GetSubImage(currentImage,mblock,mb_x*MB_SIZE,mb_y*MB_SIZE);
  pixel = (Short *)GetImageData(mblock);
 
  QP_store[mb_y*MB_width+mb_x] = (SInt) QP;

  /* Ref(start) : MS TestMemo 3 */
  dc_scaler = cal_dc_scaler(QP,1);
 
  alpha_mb_all_opaque = 1;
  for (i=0; i<MB_SIZE; i++)
    for (j=0; j<MB_SIZE; j++)
      if(pixel[i*MB_SIZE+j] != OPAQUE)   /* Ref(insert) : MS TestMemo 4 */
	alpha_mb_all_opaque = 0;

  /* 05.02.99 HHI Schueuer */
  for (k = 0; k < 4; k++)
    for (i = 0; i < 8; i++)
      for (j = 0; j < 8; j++)
	alpha_buf[k][i][j]=alpha_block[k][i][j]=BINARY_ALPHA;

  
  if (GetVopSADCTDisable(curr) == 0) {
      for (i = 0; i < 8; i++) {
	for (j = 0; j < 8; j++) {
	  alpha_block[0][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i) * nrpix + ((mb_x * MB_SIZE) + j)];
	  alpha_block[1][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i) * nrpix + ((mb_x * MB_SIZE) + j + 8)];
	  alpha_block[2][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i + 8) * nrpix + ((mb_x * MB_SIZE) + j)];
	  alpha_block[3][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i + 8) * nrpix + ((mb_x * MB_SIZE) + j + 8)];
	}
      }
  }

  for (k = 0; k < 4; k++) {
    sadct_rowlength_out(k, alpha_block[k], 8, 8);
    active_pels[k] = 0;
    for (x1 = 0; x1 < 8; x1++)
      for (y1 = 0; y1 < 8; y1++)
	if(alpha_block[k][x1][y1] == BINARY_ALPHA)
	  active_pels[k]++;
  }

  if ( (active_pels[0] != 64) || (active_pels[1] != 64) || (active_pels[2] != 64) || (active_pels[3] != 64))
    alpha_mb_all_opaque = 0;
  

  /* end HHI */
 
  if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {
    /* writing CODA */
    BitstreamPutBits(shape_bitstream, alpha_mb_all_opaque, 1);
    bits->CODA += 1;
  } else {
      alpha_residue_all_zero = 1;
        for (i=0; i<MB_SIZE; i++)
          for (j=0; j<MB_SIZE; j++)
            if(pixel[i*MB_SIZE+j] != 0)
              alpha_residue_all_zero = 0;
 
        if(alpha_residue_all_zero) {
          BitstreamPutBits(shape_bitstream, 1, 1);
          bits->CODA += 1;
        } else if(alpha_mb_all_opaque) {
            BitstreamPutBits(shape_bitstream, 1, 2);
            bits->CODA += 2;
        } else {
            BitstreamPutBits(shape_bitstream, 0, 2);
            bits->CODA += 2;
        }

  }
 
  if ((!alpha_mb_all_opaque  ) && !alpha_residue_all_zero) {
 
  if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {
 
      block_nr=0;
      /* 08.02.99 HHI Schueuer: TODO : ersetzen durch block_nr */
      nr = 0;
      /* end HHI */
      for (i=0; i<MB_SIZE; i+=8)
        for (j=0; j<MB_SIZE; j+=8) {	  

          qcoeff_ind = &(qcoeff[block_nr*64]);
          rcoeff_ind = &(rcoeff[block_nr*64]);
 
          if(MB_transp_pattern[block_nr]!=1) {
            for (k=0; k<8; k++) 
              for (l=0; l<8; l++)
                fblock[k][l] = pixel[(i+k)*MB_SIZE + (j+l)];

	    /* 08.02.99 HHI Schueuer */
	    if ((GetVopSADCTDisable(curr)!=0) || (active_pels[nr] == 64) ) {
	      BlockDCT (fblock, coeff_ind);
	    }
	    else {
	      BlockSADCT_s_k (fblock, coeff_ind, alpha_block[nr]);
	    }
	    /* end HHI */
	      
 
            /* MW 22-APR-1997 Changes for MPEG quantization */
 
            if(GetVopQuantType(curr))
              {
                if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q)
                  qmat = GetVopGrayIntraQuantMat(aux_comp,curr);
                else
                  qmat = GetVopGrayNonintraQuantMat(aux_comp,curr);
 
                /* Ref(start) : MS TestMemo 1 */
                BlockQuantMPEG(coeff_ind,QP,Mode,1,qmat,qcoeff_ind,OPAQUE,2);
                BlockDequantMPEG(qcoeff_ind,QP,Mode,1,qmat,rcoeff_ind,2);
                /* Ref(end) : MS TestMemo 1 */
              }
            else {
              /* Ref(start) : MS TestMemo 1 */
              BlockQuantH263(coeff_ind,QP,Mode,1,qcoeff_ind,OPAQUE,2);
              BlockDequantH263(qcoeff_ind,QP,Mode,1,rcoeff_ind,2, 0, GetVopBitsPerPixel(curr));
              /* Ref(end) : MS TestMemo 1 */
            }

	    /* 08.02.99 HHI Schueuer */
	    if ((GetVopSADCTDisable(curr)!=0) || (active_pels[nr] == 64) ) 
	      BlockIDCT (rcoeff_ind, fblock, OPAQUE);
	    else
	      BlockSAIDCT_s_k (rcoeff_ind, fblock, alpha_block[nr]);
	    /* end HHI */
 
            for (k=0;k<8;k++)
              for (l=0;l<8;l++)
                fblock[k][l] =  MIN(OPAQUE,MAX(0,fblock[k][l]));  /* intra clipping */
 
            for(k=1; k<64; k++)
              if (*(qcoeff_ind+k))
                  CBPA |= 1<<(3-block_nr);
 
            /* Store the qcoeff-values needed later for prediction */
            if(GetVopIntraACDCPredDisable(curr) == 0) {
              DC_store[mb_y*MB_width+mb_x][block_nr][0] = *(qcoeff_ind) * cal_dc_scaler(QP,1);              

              for (m = 1; m < 8; m++)
                DC_store[mb_y*MB_width+mb_x][block_nr][m] = *(qcoeff_ind+m);
 
              for (m = 0; m < 7; m++)
                DC_store[mb_y*MB_width+mb_x][block_nr][m+8] = *(qcoeff_ind+(m+1)*8);
            }

	  }
        else {
          for(k=0; k<64; k++)
            *(qcoeff_ind+k) = 0;
          for (k=0; k<8; k++)
            for (l=0; l<8; l++)
              fblock[k][l] = 0;
 
          /* overwrite the DC_store with QDC = MIDOPAQUE for transparent blocks */
          if(GetVopIntraACDCPredDisable(curr) == 0)
            DC_store[mb_y*MB_width+mb_x][block_nr][0] = MIDOPAQUE*8;
        }
 
        for (k=0; k<8; k++)
          for (l=0; l<8; l++)
            recon[(mb_y*MB_SIZE+i+k)*GetImageSizeX(GetVopG(aux_comp,rec_curr)) + (mb_x*MB_SIZE+j+l)]
              = (SInt)fblock[k][l];
 
        block_nr++;
	nr++;
      }  
 
      if(GetVopIntraACDCPredDisable(curr) == 0)
         {
           ACpred_flag = doDCACpred_grey(
                                qcoeff,
                                &CBPA,
                                64,
                                mb_x,
                                mb_y,
                                DC_store,
                                QP_store,
                                QP,
                                MB_width,
                                direction,
                                MIDOPAQUE,
                                GetVopSADCTDisable(curr) || (GetVopShape(curr)==RECTANGULAR));
           BitstreamPutBits(shape_bitstream, (long)ACpred_flag, 1L);
           bits->g_ACpred_flag += 1;
         }
      else
        ACpred_flag = -1;
 
      /* CBPA */
      bits->CBPA += PutCBPY(CBPA,0,MB_transp_pattern,shape_bitstream); /* Ref(insert) : MS TestMemo 5 */
 
      CodeAlphaBlockData(
                                bits,
                                MB_transp_pattern,
                                qcoeff,
                                Mode,
                                CBPA,
                                64,
                                GetVopIntraACDCPredDisable(curr),
                                shape_bitstream,
                                direction,
				GetVopSADCTDisable(curr));
 
    }
    else {      /* P-VOP and not an intra-MB */
 
      block_nr=0;

      /* 08.02.99 HHI Schueuer */
      nr = 0;
      /* end HHI */
      
      for (i=0; i<MB_SIZE; i+=8)
        for (j=0; j<MB_SIZE; j+=8) {
          qcoeff_ind = &(qcoeff[block_nr*64]);
          rcoeff_ind = &(rcoeff[block_nr*64]);
 
          if(MB_transp_pattern[block_nr]!=1) {
            for (k=0; k<8; k++)
              for (l=0; l<8; l++)
                fblock[k][l] = pixel[(i+k)*MB_SIZE + (j+l)];

	    /* 08.02.99 HHI Schueuer */
	    if ((GetVopSADCTDisable(curr)!=0) || (active_pels[nr] == 64) ) 
	      BlockDCT (fblock, coeff_ind);
	    else
	      BlockSADCT_Kaup (fblock, coeff_ind, alpha_block[nr]);
	    /* end HHI */
	      
            if(GetVopQuantType(curr))
              {
                if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q)
                  qmat = GetVopGrayIntraQuantMat(aux_comp,curr);
                else
                  qmat = GetVopGrayNonintraQuantMat(aux_comp,curr);
 
                /* Ref(start) : MS TestMemo 1 */
                BlockQuantMPEG(coeff_ind,QP,Mode,1,qmat,qcoeff_ind,OPAQUE,2);
                BlockDequantMPEG(qcoeff_ind,QP,Mode,1,qmat,rcoeff_ind,2);
                /* Ref(end) : MS TestMemo 1 */
              }
            else {
              /* Ref(start) : MS TestMemo 1 */
              BlockQuantH263(coeff_ind,QP,Mode,1,qcoeff_ind,OPAQUE,2);
              BlockDequantH263(qcoeff_ind,QP,Mode,1,rcoeff_ind,2, 0, GetVopBitsPerPixel(curr));
              /* Ref(end) : MS TestMemo 1 */
            }

	    /* 08.02.99 HHI Schueuer */
	    if ((GetVopSADCTDisable(curr)!=0) || (active_pels[nr] == 64) ) 
	      BlockIDCT (rcoeff_ind, fblock,OPAQUE);
	    else
	      BlockSAIDCT_Kaup (rcoeff_ind, fblock, alpha_block[nr]);
	    /* end HHI */
	      
            for(k=0; k<64; k++)
                if (*(qcoeff_ind+k))
                  CBPA |= 1<<(3-block_nr);
      } else {
          for(k=0; k<64; k++)
            *(qcoeff_ind+k) = 0;
          for (k=0; k<8; k++)
            for (l=0; l<8; l++)
              fblock[k][l] = 0;
      }  

	  
      for (k=0; k<8; k++)
        for (l=0; l<8; l++)
          recon[(mb_y*MB_SIZE+i+k)*GetImageSizeX(GetVopG(aux_comp,rec_curr)) + (mb_x*MB_SIZE+j+l)]
            = (SInt)fblock[k][l];
 
      DC_store[mb_y*MB_width+mb_x][block_nr][0] = MIDOPAQUE*8;
 
      block_nr++;
      /* 08.02.99 HHI Schueuer */
      nr++;
      /* end HHI */
    }
 
    /* CBPA */
    bits->CBPA += PutCBPY(CBPA,0,MB_transp_pattern,shape_bitstream);

    CodeAlphaBlockData(
                                bits,
                                MB_transp_pattern,
                                qcoeff,
                                Mode,
                                CBPA,
                                64,
                                GetVopIntraACDCPredDisable(curr),
                                shape_bitstream,
                                direction,
				GetVopSADCTDisable(curr));
 
    }
  }
  else {
    if(alpha_mb_all_opaque) {
      for (i=0; i<MB_SIZE; i++)
        for (j=0; j<MB_SIZE; j++)
          recon[(mb_y*MB_SIZE+i)*GetImageSizeX(GetVopG(aux_comp,rec_curr))
               + (mb_x*MB_SIZE+j)] = (SInt)OPAQUE;
      if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {
      /* Ref(start) : MS TestMemo 3 */
        for (m = 0; m < 4; m++) {
           DC_store[mb_y*MB_width+mb_x][m][0]
                = (((OPAQUE*8) + (dc_scaler>>1)) / dc_scaler) * dc_scaler;
           for(n=1; n<15; n++)
             DC_store[mb_y*MB_width+mb_x][m][n] = 0;
        /* Ref(end) : MS TestMemo 3 */
        }
      } else {
          for (m = 0; m < 4; m++) {
             DC_store[mb_y*MB_width+mb_x][m][0] = MIDOPAQUE*8;
             for(n=1; n<15; n++)
               DC_store[mb_y*MB_width+mb_x][m][n] = 0;
          }
      }  
    } else {
      for (m = 0; m < 4; m++) {
         DC_store[mb_y*MB_width+mb_x][m][0] = MIDOPAQUE*8;
         for(n=1; n<15; n++)
           DC_store[mb_y*MB_width+mb_x][m][n] = 0;
      }  
 
      for (i=0; i<MB_SIZE; i++)
        for (j=0; j<MB_SIZE; j++)
          recon[(mb_y*MB_SIZE+i)*GetImageSizeX(GetVopG(aux_comp,rec_curr)) + (mb_x*MB_SIZE+j)] = (SInt)0;
    }
  }
 
  FreeImage(mblock);
}
 
 
void
B_CodeGrayscaleAlpha(
        Bits *bits,
        Vop *curr,
        Vop *rec_curr,
        Int MB_width,
        Int mb_x,
        Int mb_y,
        Int QP,
        Int MB_transp_pattern[4],
        Image *shape_bitstream,
        Int aux_comp )
{
  Int i,j,k,l;
  Image *mblock;
  Short *pixel;
  Int alpha_mb_all_opaque=0;
  Int alpha_residue_all_zero=0;
  Int fblock[8][8];
  Int coeff[64];
  Int *coeff_ind = coeff;
  Int qcoeff[256];
  Int *qcoeff_ind;
  Int rcoeff[256];
  Int *rcoeff_ind;
  Image *currentImage;
  SInt *recon;
  Int CBPA=0;                   /* JDL 21-APR-97 : initialised CBPA */
  Int block_nr;
  Int *qmat;
  Int direction[4];
  /* 10.02.99 HHI Schueuer */
  /* variables for the sadct */
  Short *alpha_pntr = (Short *)(GetImageData(GetVopA(rec_curr)));
  Short alpha_block[4][8][8];
  Short alpha_buf[4][8][8];
  Short nrpix = GetImageSizeX(GetVopA(rec_curr));
  Int active_pels[4];
  Int x1, y1;
  /* end HHI */  
 
 
  currentImage = GetVopG(aux_comp,curr);
  recon = (SInt *) GetImageData(GetVopG(aux_comp,rec_curr));
 
  mblock = AllocImage(16,16,SHORT_TYPE);
  GetSubImage(currentImage,mblock,mb_x*MB_SIZE,mb_y*MB_SIZE);
  pixel = (Short *)GetImageData(mblock);

  /* 10.02.99 HHI Schueuer */
  for (k = 0; k < 4; k++)
    for (i = 0; i < 8; i++)
      for (j = 0; j < 8; j++)
	alpha_buf[k][i][j]=alpha_block[k][i][j]=BINARY_ALPHA;

  
  if (GetVopSADCTDisable(curr) == 0) {
      for (i = 0; i < 8; i++) {
	for (j = 0; j < 8; j++) {
	  alpha_block[0][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i) * nrpix + ((mb_x * MB_SIZE) + j)];
	  alpha_block[1][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i) * nrpix + ((mb_x * MB_SIZE) + j + 8)];
	  alpha_block[2][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i + 8) * nrpix + ((mb_x * MB_SIZE) + j)];
	  alpha_block[3][i][j] = alpha_pntr[((mb_y * MB_SIZE) + i + 8) * nrpix + ((mb_x * MB_SIZE) + j + 8)];
	}
      }
  }

  for (k = 0; k < 4; k++) {
    sadct_rowlength_out(k, alpha_block[k], 8, 8);
    active_pels[k] = 0;
    for (x1 = 0; x1 < 8; x1++)
      for (y1 = 0; y1 < 8; y1++)
	if(alpha_block[k][x1][y1] == BINARY_ALPHA)
	  active_pels[k]++;
  }

  /* end HHI */
  
  alpha_mb_all_opaque = 1;
    for (i=0; i<MB_SIZE; i++)
      for (j=0; j<MB_SIZE; j++)
        if(pixel[i*MB_SIZE+j] != OPAQUE)   /* Ref(insert) : MS TestMemo 4 */
          alpha_mb_all_opaque = 0;
 
  alpha_residue_all_zero = 1;
    for (i=0; i<MB_SIZE; i++)
      for (j=0; j<MB_SIZE; j++)
        if(pixel[i*MB_SIZE+j] != 0)
          alpha_residue_all_zero = 0;
 
    if(alpha_residue_all_zero) {
        BitstreamPutBits(shape_bitstream, 1, 1);
        bits->CODA += 1;
    } else if(alpha_mb_all_opaque) {
        BitstreamPutBits(shape_bitstream, 1, 2);
        bits->CODA += 2;
    } else {
        BitstreamPutBits(shape_bitstream, 0, 2);
        bits->CODA += 2;
    }
 
  if (!alpha_mb_all_opaque && !alpha_residue_all_zero) {
 
      block_nr=0;
      for (i=0; i<MB_SIZE; i+=8)
        for (j=0; j<MB_SIZE; j+=8) {
          qcoeff_ind = &(qcoeff[block_nr*64]);
          rcoeff_ind = &(rcoeff[block_nr*64]);
 
          if(MB_transp_pattern[block_nr]!=1) {
            for (k=0; k<8; k++)
              for (l=0; l<8; l++)
                fblock[k][l] = pixel[(i+k)*MB_SIZE + (j+l)];

	    /* 10.02.99 HHI Schueuer */
	    if ((GetVopSADCTDisable(curr)!=0) || (active_pels[block_nr] == 64) )
	      BlockDCT (fblock, coeff_ind);
	    else
	      BlockSADCT_Kaup (fblock, coeff_ind, alpha_block[block_nr]);
	    /* end HHI */
 
            if(GetVopQuantType(curr))
              {
                qmat = GetVopGrayNonintraQuantMat(aux_comp,curr);
 
                /* Ref(start) : MS TestMemo 1 */
                BlockQuantMPEG(coeff_ind,QP,MODE_INTER,1,qmat,qcoeff_ind,OPAQUE,2);
                BlockDequantMPEG(qcoeff_ind,QP,MODE_INTER,1,qmat,rcoeff_ind,2);
                /* Ref(end) : MS TestMemo 1 */
              }
            else {
              /* Ref(start) : MS TestMemo 1 */
              BlockQuantH263(coeff_ind,QP,MODE_INTER,1,qcoeff_ind,OPAQUE,2);
              BlockDequantH263(qcoeff_ind,QP,MODE_INTER,1,rcoeff_ind,2, 0, GetVopBitsPerPixel(curr));
              /* Ref(end) : MS TestMemo 1 */
            }

	    /* 10.02.99 HHI Schueuer */
	    if ((GetVopSADCTDisable(curr)!=0) || (active_pels[block_nr] == 64) ) 
	      BlockIDCT (rcoeff_ind, fblock,OPAQUE);
	    else
	      BlockSAIDCT_Kaup (rcoeff_ind, fblock, alpha_block[block_nr]);
	    /* end HHI */
 
            for(k=0; k<64; k++)
                if (*(qcoeff_ind+k))
                  CBPA |= 1<<(3-block_nr);
      } else {
          for(k=0; k<64; k++)
            *(qcoeff_ind+k) = 0;
          for (k=0; k<8; k++)
            for (l=0; l<8; l++)
              fblock[k][l] = 0;
      }  
 
      for (k=0; k<8; k++)
        for (l=0; l<8; l++)
          recon[(mb_y*MB_SIZE+i+k)*GetImageSizeX(GetVopG(aux_comp,rec_curr)) + (mb_x*MB_SIZE+j+l)]
            = (SInt)fblock[k][l];
 
        block_nr++;
    }
 
    /* CBPA */
    bits->CBPA += PutCBPY(CBPA,0,MB_transp_pattern,shape_bitstream);
 
    CodeAlphaBlockData(
                                bits,
                                MB_transp_pattern,
                                qcoeff,
                                MODE_INTER,
                                CBPA,
                                64,
                                GetVopIntraACDCPredDisable(curr),
                                shape_bitstream,
                                direction,
				GetVopSADCTDisable(curr));
 
  }
  else {
    if(alpha_mb_all_opaque) {
      for (i=0; i<MB_SIZE; i++)
        for (j=0; j<MB_SIZE; j++)
          recon[(mb_y*MB_SIZE+i)*GetImageSizeX(GetVopG(aux_comp,rec_curr))
               + (mb_x*MB_SIZE+j)] = (SInt)OPAQUE;
    } else {
      for (i=0; i<MB_SIZE; i++)
        for (j=0; j<MB_SIZE; j++)
          recon[(mb_y*MB_SIZE+i)*GetImageSizeX(GetVopG(aux_comp,rec_curr)) + (mb_x*MB_SIZE+j)] = (SInt)0;
    }
  }
 
  FreeImage(mblock);
}




Void CodeAlphaBlockData (Bits *bits, Int transp_pattern[], Int *qcoeff, Int Mode, Int
			 CBPA, Int ncoeffs,
                         Int intra_dcpred_disable, Image *bitstream, Int direction[],
			 Int sadct_disable)
{
    Int i, m, coeff[64];
    Int switched = 0; /* Ref : MS TestMemo 2 */
    Int reverse_vlc = 0; /* Error Resilient Enable Mode is not supported in this s/w version */

    /* 08.02.99 HHI Schueuer */
    Int sadct_used_alpha[4];
    for (i = 0; i < 4; i++)
      sadct_used_alpha[i] = 0;
    if (sadct_disable == 0) 
      SADCT_Used_Decision_alpha(sadct_used_alpha);
    /* end HHI */
    
    if (Mode == MODE_INTRA || Mode == MODE_INTRA_Q)
      {  
            for (i = 0; i < 4; i++)
              {  
                if (transp_pattern[i]!=1) /* Not transparent */
                  {
                    /* Do the zigzag scanning of coefficients */
		    /* 08.02.99 HHI Schueuer */
		    if (sadct_used_alpha[i]) { 
		      SADCT_Scan(&qcoeff[i*ncoeffs], coeff, i, direction[i]);
		    }
		    else { /* end HHI */
		      for (m = 0; m < 64; m++)
			{
			  if (direction[i] == 0)
                            *(coeff + zigzag[m]) = qcoeff[i*ncoeffs+m];
			  else if (direction[i] == 1)
                            *(coeff + zigzag_v[m]) = qcoeff[i*ncoeffs+m];
			  else if (direction[i] == 2)
                            *(coeff + zigzag_h[m]) = qcoeff[i*ncoeffs+m];
			  else
			    {
			      fprintf(stderr, "MB_CodeCoeff(): Error in zigzag direction\n");
			      exit(-1);
			    }
			}
                    }
 
                if (switched==0)
                      bits->G += IntraDC_dpcm(coeff[0],1,bitstream);
 
                    /* Code AC coeffs. dep. on block pattern MW 15-NOV-1996 */
                    if ((i==0 && CBPA&8) ||
                        (i==1 && CBPA&4) ||
                        (i==2 && CBPA&2)  ||
                        (i==3 && CBPA&1))
                      {  
                        if (!reverse_vlc)
                              bits->G += CodeCoeff(1-switched,Mode, coeff,i,ncoeffs,bitstream);
                        else
                              bits->G += CodeCoeff_RVLC(1-switched,Mode, coeff, i,
                                                        ncoeffs, bitstream);
                      }  
                  }
              }
      }
      else        /* inter block encoding */
      {
        for (i = 0; i < 4; i++)
          {
 
           if (transp_pattern[i]!=1) { /* Not transparent */
	     /* Do the zigzag scanning of coefficients */
	     /* 08.02.99 HHI Schueuer */
	     if (sadct_used_alpha[i]) {
	       SADCT_Scan(&qcoeff[i*ncoeffs], coeff, i, 0);
	     }       
	     else { /* end HHI */
	       for (m = 0; m < 64; m++)
		 *(coeff + zigzag[m]) = qcoeff[i*ncoeffs+m];
	     }
 
              if ((i==0 && CBPA&8) ||
                  (i==1 && CBPA&4) ||
                  (i==2 && CBPA&2) ||
                  (i==3 && CBPA&1))
                {
                  if (!reverse_vlc)
                        bits->G += CodeCoeff(0,Mode, coeff, i, ncoeffs, bitstream);
                  else
                        bits->G += CodeCoeff_RVLC(0,Mode, coeff, i, ncoeffs,
                                                bitstream);
         
                }
            }
         }
      }     
}        
 
/* 01.04.01: Studio Profile */
void
CodeGrayscaleAlpha_StudioPDAM(Bits *bits,
			      Vop *curr,
			      Vop *rec_curr,
			      Int MB_width,
			      Int mb_x,
			      Int mb_y,
			      /*AM2PDAM*/
			      Int ***DC_store[],
			      SInt *QP_store[],
			      Int intra_dc_precision[],
			      Int q_scale_type[],
			      Int /***/QP[],
			      Int QP_last_coded[],
			      /*AM2PDAM*/
			      Int Mode,
			      Int fieldDCT,/*AM2PDAM*/
			      Int MB_transp_pattern[4],
			      Image *shape_bitstream,
			      VolConfig *vol_config,
			      Int is_slice_start_mb)
{
  Int vop_structure = GetVopVopStructure_st(curr);
  Int field_parity = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(curr), GetVopTopFieldFirst(curr));
  
  Int i,j,k,l,m,n;
  Image *mblock;
  Short *pixel;
  Int alpha_mb_all_opaque=0;
  Int alpha_residue_all_zero=0;
  Int fblock[4][8][8];
  Int coeff[64];
  Int *coeff_ind = coeff;
  Int qcoeff[256];
  Int *qcoeff_ind;
  Int rcoeff[256];
  Int *rcoeff_ind;
  Image **currentImage;
  SInt *recon[2];
  Int CBPA=0;                   /* JDL 21-APR-97 : initialised CBPA */
  Int block_nr;
  Int *qmat;
  Int direction[4];
  Int ACpred_flag = -1;
  Int dc_scaler;
  
#if 0 /*AM2PDAM*/
  Image *alpha_block;
  SInt *alpha_pixel;
  Int num_transp=0;
  Int fieldDCT=0;
#endif 0
  
  Int DQUANT=0;
  Int DC_prev;

  /*WD2.1_2*/
/*  Int coded_fblock[4][8][8];*/
  /*
  Int *raw_blocks;
  Int a_uncompress_flag, a_ucbp_code;
  */
  Int tmp_dc;
  Int ii, jj, kk;
  Int tmp_dc_prev;
  /*WD2.1_2*/
  
  Int block_opaque[MAX_AUX_COMP_NUM][8][8], block_transp[MAX_AUX_COMP_NUM][8][8];

  /*AM2PDAM*/
  Int *raw_blocks;
  Image *mb_vlc_bitstream;
  Image *mb_raw_bitstream;
  Bits bits_mbvlc;
  Int bits_mbraw;
  /*AM2PDAM*/
  
  /*v001225*/
  Int bits_per_pixel;
  Int dct_precision;
  Int dc_reset_value;
  Int intra_dc_mult;
  Int dpcm_block_mean;
  /*v001225*/
  
  if(Mode==MODE_UNCOMPRESS) {
    Mode = MODE_INTRA;
    fieldDCT = 0;
  }
  
  /*AM2PDAM*/
  for(kk=0;kk<GetVopAuxCompCount_st(curr);kk++) {

    /*v001225*/
    bits_per_pixel = GetVopGrayBitsPerPixel_st(curr,kk);
    dct_precision = GetVopGrayDCTPrecision_st(curr,kk,field_parity);
    dc_reset_value = 1 << bits_per_pixel + 6/*BUGFIX*//*dct_precision + intra_dc_precision[kk]*/ - 1;
    intra_dc_mult = 1 << 6 - dct_precision - intra_dc_precision[kk];
    /*v001225*/
    
    for(i=0;i<8;i++) {
      for(j=0;j<8;j++) {
	block_opaque[kk][i][j] = GetVopGrayOpaque_Studio(curr,kk);
	block_transp[kk][i][j] = 0;
      }
    }
    
    /* initializing */
    CBPA=0;

#if 0
    if(Mode==MODE_UNCOMPRESS) {
      Mode = MODE_INTRA;
      fieldDCT = 0;
    }
#endif 0
    
    /* Get alpha block data */
    for(i=0;i<2;i++)
      recon[i] = (SInt *) GetImageData(GetVopG_st(rec_curr,kk,i));
    mblock = AllocImage(16,16,SHORT_TYPE);
    GetSubImage_Studio(GetVopG2_st(curr,kk), vop_structure, field_parity,
		       mblock, mb_x*MB_SIZE, mb_y*MB_SIZE);
    pixel = (Short *)GetImageData(mblock);
    
#if 0 /*AM2PDAM*/
    alpha_block = AllocImage(16,16,SHORT_TYPE);
    GetSubImage_Studio(GetVopA2_st(rec_curr), vop_structure, field_parity,
		       alpha_block, mb_x*MB_SIZE, mb_y*MB_SIZE);
    alpha_pixel = (SInt *)GetImageData(alpha_block);
#endif 0 /*AM2PDAM*/
    
    QP_store[kk][mb_y*MB_width+mb_x] = (SInt) /***/QP[kk];
    
    /* Ref(start) : MS TestMemo 3 */
    dc_scaler = cal_dc_scaler_Studio(intra_dc_precision[kk]/*GetVopGrayIntraDCPrecision_st(curr,field_parity)*/);
    
    alpha_mb_all_opaque = 1;
    for (i=0; i<MB_SIZE; i++)
      for (j=0; j<MB_SIZE; j++)
        if(pixel[i*MB_SIZE+j] != GetVopGrayOpaque_Studio(curr,kk)/*OPAQUE*/)   /* Ref(insert) : MS TestMemo 4 */
          alpha_mb_all_opaque = 0;
    /* In the above routine, is it necessary to check binary transparency?? */
    
    if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {
      /* writing CODA */
      BitstreamPutBits(shape_bitstream, alpha_mb_all_opaque, 1);
      bits->CODA += 1;
    } else {
      alpha_residue_all_zero = 1;
      for (i=0; i<MB_SIZE; i++)
	for (j=0; j<MB_SIZE; j++)
	  if(pixel[i*MB_SIZE+j] != 0)
	    alpha_residue_all_zero = 0;
      
      if(alpha_residue_all_zero) {
	BitstreamPutBits(shape_bitstream, 1, 1);
	bits->CODA += 1;
      } else if(alpha_mb_all_opaque) {
	BitstreamPutBits(shape_bitstream, 1, 2);
	bits->CODA += 2;
      } else {
	BitstreamPutBits(shape_bitstream, 0, 2);
	bits->CODA += 2;
      }
    }

    
    if (!alpha_mb_all_opaque && !alpha_residue_all_zero) {
      
      if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {


	/*Start to Code MB by using Uncompressed MB syntax*//*AM2PDAM*/
	{
	  
	  mb_raw_bitstream = BitstreamInit();
	  bits_mbraw = 0;
	  
	  raw_blocks = CodeAlphaMB_DPCM_Studio(curr, kk,
					       mb_x*MB_SIZE, mb_y*MB_SIZE, 
					       mb_raw_bitstream,
					       &bits_mbraw,
					       &dpcm_block_mean);
#if 0	  
	  raw_blocks = CodeAlphaMB_Uncompress_Studio(curr, kk,
						     mb_x*MB_SIZE, mb_y*MB_SIZE, MB_width*MB_SIZE,
						     MB_transp_pattern, GetVopGrayBitsPerPixel_st(curr,kk),
						     mb_raw_bitstream, &bits_mbraw);
#endif 0	    
	}
	/*End of Coding MB by using Uncompressed MB syntax*/
	  
	/*Start to Code MB by using DCT*/
	{
	  mb_vlc_bitstream = BitstreamInit();
	  Bits_Reset(&bits_mbvlc);

	
	/*Update quantizer*/
	DQUANT = GetMBDQuant_Studio(/***/QP[kk], vol_config->gray_mb_quantizer_st[kk]);
#if 0
	/***/QP[kk] += DQUANT;
	QP_store[kk][mb_y*MB_width+mb_x] = (SInt) /***/QP[kk];
#endif 0
	  
	/* Set temporal blocks */
	block_nr=0;
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    for (k=0; k<8; k++)
	      for (l=0; l<8; l++)
		fblock[block_nr][k][l] = pixel[(i+k)*MB_SIZE + (j+l)];
	    block_nr++;
	  }
	
	/*AM2PDAM*/
	if(fieldDCT)
	  GrayConvForFieldDCT_Studio(&fblock[0][0][0]);
	
	/* process for each block */
	block_nr=0;
	/*a_ucbp_code=0;*/ /*WD2.1_2*/
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    /*a_ucbp_code<<=1;*//*WD2.1_2*/
	    if (MB_transp_pattern[block_nr] != 1) { /* non transparent block */
	      /* DCT block */
	      BlockDCT (fblock[block_nr], coeff_ind);
	      
	      qcoeff_ind = &(qcoeff[block_nr*64]);
	      rcoeff_ind = &(rcoeff[block_nr*64]);
	      
	      /* Quantize & Dequantize DCTed block */
	      if(1/*GetVopQuantType(curr)*/)
		{
		  if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q)
		    qmat = GetVopGrayIntraQuantMat_st(curr,kk);
		  else
		    qmat = GetVopGrayNonintraQuantMat_st(curr,kk);
		  
		  /* Ref(start) : MS TestMemo 1 */
		  /*v001225*/
		  BlockQuantMPEG_DCText_Studio(coeff_ind,/***/QP[kk]+DQUANT,Mode,1,
					       intra_dc_precision[kk], q_scale_type[kk], qmat,
					       dct_precision, bits_per_pixel,/*v001225*/
					       qcoeff_ind,
					       GetMaxQDC_DCText_Studio(bits_per_pixel,dct_precision,intra_dc_precision[kk]),/*v001225*/
					       1);
		  /*v010113_0*/
		  BlockDequantMPEG_DCText_DeadZone_Studio(qcoeff_ind,/***/QP[kk]+DQUANT,Mode,1,
							  intra_dc_precision[kk], q_scale_type[kk], qmat,
							  dct_precision, bits_per_pixel,/*v001225*/
							  GetVopDeadZoneDisable_st(curr,field_parity),
							  rcoeff_ind,1,0);
#if 0
		  BlockDequantMPEG_DCText_Studio(qcoeff_ind,/***/QP[kk]+DQUANT,Mode,1,
						 intra_dc_precision[kk], q_scale_type[kk], qmat,
						 dct_precision, bits_per_pixel,/*v001225*/
						 rcoeff_ind,1);
#endif 0
#if 0
		  BlockQuantMPEG_Studio(coeff_ind,/***/QP[kk]+DQUANT,Mode,1,
					intra_dc_precision[kk], q_scale_type[kk], qmat,
					qcoeff_ind,
					GetMaxQDC_Studio(GetVopGrayBitsPerPixel_st(curr,kk),intra_dc_precision[kk]),1);
		  BlockDequantMPEG_Studio(qcoeff_ind,/***/QP[kk]+DQUANT,Mode,1,
					  intra_dc_precision[kk], q_scale_type[kk], qmat,
					  rcoeff_ind,1);
#endif 0
		}
	      else {
		fprintf(stderr, "H263 quant is not used for Studio Profile.\n");
		exit(1);
		/* Ref(start) : MS TestMemo 1 */
		BlockQuantH263(coeff_ind,*QP,Mode,1,qcoeff_ind,OPAQUE,2);
		BlockDequantH263(qcoeff_ind,*QP,Mode,1,rcoeff_ind,2, 0, GetVopBitsPerPixel(curr));
		/* Ref(end) : MS TestMemo 1 */
	      }
	      
	      /*v001225*/
	      BlockIDCT_DCText_Studio (rcoeff_ind, fblock[block_nr], GetVopGrayOpaque_Studio(curr,kk),0);
	      /*BlockIDCT (rcoeff_ind, fblock[block_nr], GetVopGrayOpaque_Studio(curr,kk));*/
	      /* original fblock is updated to reconstructed fblock. */

	      for (k=0;k<8;k++)
		for (l=0;l<8;l++)
		  fblock[block_nr][k][l] =  MIN(GetVopGrayOpaque_Studio(curr,kk),MAX(0,fblock[block_nr][k][l]));  /* intra clipping */

	      /* Here, get a qcoeffed and a reconstructed macroblock */

	      /* Calclate coded_block_pattern_alpha */
/*	      
	      for(k=1; k<64; k++)
		if (*(qcoeff_ind+k))
*//*AM2PDAM*/
	      CBPA |= 1<<(3-block_nr);/* for transparent block, this value isn't updated. */

	      /* Store the qcoeff-values needed later for prediction */
	      DC_store[kk][mb_y*MB_width+mb_x][block_nr][0] = *(qcoeff_ind) * intra_dc_mult;/* * dc_scaler;*//*v001225*/
	      for (m = 1; m < 8; m++)
		DC_store[kk][mb_y*MB_width+mb_x][block_nr][m] = *(qcoeff_ind+m);
	      for (m = 0; m < 7; m++)
		DC_store[kk][mb_y*MB_width+mb_x][block_nr][m+8] = *(qcoeff_ind+(m+1)*8);
	      
	      /* DC prediction */
	      if(is_slice_start_mb && block_nr==0)
		DC_prev = dc_reset_value;/*GetVopGrayMidValue_Studio(curr,kk)*8;*//*v001225*/
	      else
		DC_prev = DC_store[kk][mb_y*MB_width+(4*mb_x+block_nr-1)/4][(4*mb_x+block_nr-1)%4][0];
	      *(qcoeff_ind) -= DC_prev / intra_dc_mult; /*dc_scaler;*//*v001225*/

	    }/* non transparent block */
	    else {/*transparent block*/
	      /* Get a quantized and dequantized block for a transparent block */
	      qcoeff_ind = &(qcoeff[block_nr*64]);/*WD1.3*/
	      for(k=0; k<64; k++)
		*(qcoeff_ind+k) = 0;
	      for (k=0; k<8; k++)
		for (l=0; l<8; l++)
		  fblock[block_nr][k][l] = 0;
	      
	      /* overwrite the DC_store with QDC = MIDOPAQUE for transparent blocks */
	      DC_store[kk][mb_y*MB_width+mb_x][block_nr][0] = dc_reset_value;/*GetVopGrayMidValue_Studio(curr,kk)*8;*//*v001225*/
	    }/*transparent block*/
	    
	    block_nr++;
	  }/* 4blocks roop */

#if 0	
	/* Buck up coded fblock */
	for(k=0;k<4;k++)
	  for(j=0;j<8;j++)
	    for(i=0;i<8;i++)
	      coded_fblock[k][j][i] = fblock[k][j][i];
#endif 0

	/* Deinterlace */
	/*AM2PDAM*/
	if (/*curr->interlaced && (vop_structure == FRAME_VOP)
	      && (GetVopGrayFramePredFrameDCT_st(curr) == 0)
	      && */fieldDCT)
	  fieldDCTtoFrame(&fblock[0][0][0]);

	/* Put the reconstructed fblock to memory at the position of the current block */
	block_nr=0;
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    BlockRebuild_Studio(recon, vop_structure, field_parity,
				mb_x*MB_SIZE+j, mb_y*MB_SIZE+i, MB_width*16, fblock[block_nr]);
	    block_nr++;
	  }

#if 0
	if(GetVopIntraACDCPredDisable(curr) == 0)
	  {
	    ACpred_flag = doDCACpred_grey(
					  qcoeff,
					  &CBPA,
					  64,
					  mb_x,
					  mb_y,
					  DC_store,
					  QP_store,
					  QP,
					  MB_width,
					  direction,
					  MIDOPAQUE);
	    BitstreamPutBits(shape_bitstream, (long)ACpred_flag, 1L);
	    bits->g_ACpred_flag += 1;
	  }
	else
#endif 0
	  
	  ACpred_flag = -1;
	
	/* BitstreamPut CBPA */
/*	  
	bits_mbvlc.mbheader += (bits_mbvlc.CBPA += PutCBPY(CBPA,0,MB_transp_pattern,mb_vlc_bitstream) );
*//*AM2PDAM*/	
	
	/* BitstreamPut gray_quantizer */
	if(DQUANT)
	  BitstreamPutBits(mb_vlc_bitstream, 1L, 1);
	else
	  BitstreamPutBits(mb_vlc_bitstream, 0L, 1);
	bits_mbvlc.g_QUANT += 1;
	bits_mbvlc.mbheader += 1;
	  
	if(DQUANT) {
	  BitstreamPutBits(mb_vlc_bitstream, /***/QP[kk]+DQUANT, 5L);
	  bits_mbvlc.g_QUANT += 5;
	  bits_mbvlc.mbheader += 5;
	}

#if 0
	/* BitstreamPut gray_dct_type */
	if (curr->interlaced) {
	  if(vop_structure == FRAME_VOP && GetVopFramePredFrameDCT_st(curr)/*AM2PDAM*/ == 0) {
	    bits->g_fieldDCT += fieldDCT;
	    BitstreamPutBits(shape_bitstream, fieldDCT, 1);
	    bits->g_interlaced++;
	  }
	}

	/*WD2.1_2*/
	if(a_ucbp_code)
	  a_uncompress_flag = 1;
	else
	  a_uncompress_flag = 0;
	
	/* Put a_uncompress_flag and a_ucbp_code to shape_bitstream */
	BitstreamPutBits(shape_bitstream, a_uncompress_flag, 1L);
	if(a_uncompress_flag) {
	  for(k=0;k<4;k++) {
	    if(MB_transp_pattern[k]!=1) {
	      BitstreamPutBits(shape_bitstream, ((a_ucbp_code)>>(4-k-1))&1, 1L);
	    }
	  }
	}
	/*WD2.1_2*/
#endif 0
	
	/* Code gray block */
	CodeAlphaBlockData_DCT_StudioPDAM(&bits_mbvlc,
					  MB_transp_pattern,
					  GetVopAlternateScan_st(curr, field_parity)/*AM2PDAM*/,
					  qcoeff,
					  Mode,
					  CBPA,
					  64,
					  mb_vlc_bitstream,
					  GetVopGrayBitsPerPixel_st(curr,kk),
					  GetVopGrayBitsPerPixel_st(curr,kk)+GetVopGrayDCTPrecision_st(curr,kk,field_parity)+4/*v001225*/);
#if 0
	/*WD2.1_2*/
	CodeAlphaBlockData_DCT_Studio(&bits_mbvlc,
				      MB_transp_pattern,
				      GetVopAlternateScan_st(curr, field_parity)/*AM2PDAM*/,
				      qcoeff,
				      Mode,
				      CBPA,
				      64,
				      mb_vlc_bitstream,
				      GetVopGrayBitsPerPixel_st(curr,kk));
#endif 0
#if 0
	CodeAlphaBlockData_Through_Studio(&bits_mbvlc,
					  /*WD2.1_2*/
					  a_ucbp_code,
					  coded_fblock,
					  /*WD2.1_2*/
					  MB_transp_pattern,
					  GetVopAlternateScan_st(curr, field_parity)/*AM2PDAM*/,
					  qcoeff,
					  Mode,
					  CBPA,
					  64,
					  mb_vlc_bitstream,
					  GetVopGrayBitsPerPixel_st(curr,kk));
#endif 0
	  }
	  /*End of Coding MB by using DCT*/

	/* Decide which coding scheme is used to encode the macroblock */
	/* Append the decided bitstream to slice_bitstream */
	/* Append the bitcount data to bits structure */
	/* Put the coded macroblock to the memory if the mode is the uncompressed */
	/* Update DC_store if the uncompressed */
	{
	  Int *raw_blocks_ind;
	  Int tmp_dc;
	  Int compression_mode;
	  
	  Int vlcbit;
	  vlcbit = bits_mbvlc.mbheader + bits_mbvlc.G;
	  
	  /* 1. Mode Decision */
	  if(GetVolConfigThroughMode_st(vol_config)==0) /*only VLC coding*/
	    compression_mode = 0;
	  else if(GetVolConfigThroughMode_st(vol_config)==1) /*only Uncompress coding*/
	    compression_mode = 1;
	  else if(GetVolConfigThroughMode_st(vol_config)==2) { /*RANDOM*/
	    compression_mode = GetRandomValue_Studio(0,1);
#if 0
	    Int vlcbit, rawbit;
	    vlcbit = bits_mbvlc.mbheader + bits_mbvlc.G;
	    rawbit = bits_mbraw;
	    
	    if( vlcbit < rawbit ) /*DCT coding*/
	      compression_mode = 0;
	    else /*Uncompress coding*/
	      compression_mode = 1;
#endif 0    
	  } else if(GetVolConfigThroughMode_st(vol_config)==3) { /*The DCT coding is selected if less than the threshold*/
	    Int vlcbit;
	    vlcbit = bits_mbvlc.mbheader + bits_mbvlc.G;
	    
	    if( vlcbit < GetVolConfigUncompressThreshold_st(vol_config) )
	      compression_mode = 0;
	    else
	      compression_mode = 1;
	    
	  } else {
	    fprintf(stderr, "this compression mode is not supported[%d].\n", GetVolConfigThroughMode_st(vol_config));
	    exit(-1);
	  }
	  
	  /*2. Put compression_mode to slice_stream*/
	  /* 011210_1 Syntax */
	  BitstreamPutBits(shape_bitstream, !compression_mode /*compression_mode*/, 1);
	  bits->mbheader ++;
	  
	  /*3. Put a selected MB data to slice_stream*/
	  if(!compression_mode) { /*VLC mode*/
	    
	    BitstreamAppend(mb_vlc_bitstream,shape_bitstream);
	    AddBits_Studio(&bits_mbvlc, bits);
	    
	    /*Update coded QP*//*moved*/
	    QP[kk]+=DQUANT;
	    QP_last_coded[kk]=QP[kk];
	    
	  } else { /*Uncompress mode*/
	    /* MoMuSys-2000FDAM1COR-020612 */
	    /* marker_bit */
	    bits->mbheader ++;
	    /**/
	    
	    bits->mbheader ++; /*alpha_dpcm_scan_order*/
	    
	    BitstreamAppend(mb_raw_bitstream,shape_bitstream);
	    
	    bits->G_raw  += bits_mbraw;
	    
	    raw_blocks_ind = raw_blocks;
	    
	    for(k=0;k<4;k++) {
	      if(MB_transp_pattern[k]!=1) {
		
		/* Update texture */
		BlockRebuild_Studio (recon, vop_structure, field_parity, mb_x*MB_SIZE+(k%2)*8, mb_y*MB_SIZE+(k/2)*8, MB_width*MB_SIZE, raw_blocks_ind);
		
		/*v001225*/
		DC_store[kk][mb_y*MB_width+mb_x][k][0] = dpcm_block_mean*(1<< 6/*dct_precision+intra_dc_precision[kk]*/);
#if 0
		/* Update DC_store */
		tmp_dc = 0;
		for(ii=0;ii<64;ii++) {
		  tmp_dc += *raw_blocks_ind;
		  raw_blocks_ind++;
		}
		tmp_dc /= 8;
		DC_store[kk][mb_y*MB_width+mb_x][k][0] = tmp_dc;
#endif 0
		
	      } else { /*transp bl.*/
		/** raw_blocks_ind += 64; **/
		
		/*v001225*/
		DC_store[kk][mb_y*MB_width+mb_x][k][0] = dpcm_block_mean*(1<< 6/*dct_precision+intra_dc_precision[kk]*/);
		
		/* Texture has already been filled with 0 in CodeMB_DCT_Studio() */
		/* The DC_store for transp bl. has been already updated in VLC routine with the value, GetVopMidGrey(curr)*8 */
	      }                 
	      raw_blocks_ind += 64;
	    }
	    
	    }
	  
	  /*Free pointers*/
	  free ((Char*)raw_blocks);
	  BitstreamFree(mb_vlc_bitstream);
	  BitstreamFree(mb_raw_bitstream);
	} /*Mode decision, and the following procedures*/
	
                

	
      }/*Intra MB*/
      else {      /* P-VOP and not an intra-MB */
#if 0
	block_nr=0;
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    qcoeff_ind = &(qcoeff[block_nr*64]);
	    rcoeff_ind = &(rcoeff[block_nr*64]);
	    
	    if(MB_transp_pattern[block_nr]!=1) {
	      for (k=0; k<8; k++)
		for (l=0; l<8; l++)
		  fblock[k][l] = pixel[(i+k)*MB_SIZE + (j+l)];
	      
	      BlockDCT (fblock, coeff_ind);
	      
	      if(GetVopQuantType(curr))
		{
		  if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q)
		    qmat = GetVopGrayIntraQuantMat(curr);
		  else
		    qmat = GetVopGrayNonintraQuantMat(curr);
		  
		  /* Ref(start) : MS TestMemo 1 */
		  BlockQuantMPEG(coeff_ind,QP,Mode,1,qmat,qcoeff_ind,OPAQUE,2);
		  BlockDequantMPEG(qcoeff_ind,QP,Mode,1,qmat,rcoeff_ind,2);
		  /* Ref(end) : MS TestMemo 1 */
		}
	      else {
		/* Ref(start) : MS TestMemo 1 */
		BlockQuantH263(coeff_ind,QP,Mode,1,qcoeff_ind,OPAQUE,2);
		BlockDequantH263(qcoeff_ind,QP,Mode,1,rcoeff_ind,2, 0, GetVopBitsPerPixel(curr));
		/* Ref(end) : MS TestMemo 1 */
	      }
	      
	      BlockIDCT (rcoeff_ind, fblock,OPAQUE);
	      
	      for(k=0; k<64; k++)
                if (*(qcoeff_ind+k))
                  CBPA |= 1<<(3-block_nr);
	    } else {
	      for(k=0; k<64; k++)
		*(qcoeff_ind+k) = 0;
	      for (k=0; k<8; k++)
		for (l=0; l<8; l++)
		  fblock[k][l] = 0;
	    }  
	    
	    for (k=0; k<8; k++)
	      for (l=0; l<8; l++)
		recon[(mb_y*MB_SIZE+i+k)*GetImageSizeX(GetVopG(rec_curr)) + (mb_x*MB_SIZE+j+l)]
		  = (SInt)fblock[k][l];
	    
	    DC_store[mb_y*MB_width+mb_x][block_nr][0] = MIDOPAQUE*8;
	    
	    block_nr++;
	  }
	
	/* CBPA */
	bits->CBPA += PutCBPY(CBPA,0,MB_transp_pattern,shape_bitstream);
	
	CodeAlphaBlockData(
			   bits,
			   MB_transp_pattern,
			   qcoeff,
			   Mode,
			   CBPA,
			   64,
			   GetVopIntraACDCPredDisable(curr),
			   shape_bitstream,
			   direction);
#endif 0 
      }/*Inter MB*/
    }
    else {
      if(alpha_mb_all_opaque) {/* Both for Intra */
	for(i=0;i<2;i++)
	  for(j=0;j<2;j++)
	    BlockRebuild_Studio(recon, vop_structure, field_parity,
				mb_x*MB_SIZE+j*8, mb_y*MB_SIZE+i*8, MB_width*16, block_opaque[kk]);
	/*      for (i=0; i<MB_SIZE; i++)
		for (j=0; j<MB_SIZE; j++)
		recon[(mb_y*MB_SIZE+i)*GetImageSizeX(GetVopG(rec_curr)) + (mb_x*MB_SIZE+j)] = (SInt)OPAQUE;*/
	
	if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {
	  
	  
	  /* Ref(start) : MS TestMemo 3 */
	  for (m = 0; m < 4; m++) {
	    DC_store[kk][mb_y*MB_width+mb_x][m][0]
	      /*v001225_1 FIX*/
	      = GetVopGrayOpaque_Studio(curr,kk) * (1 << GetVopGrayDCTPrecision_st(curr,kk,field_parity)+GetVopGrayIntraDCPrecision_st(curr,kk,field_parity));
	    
#if 0
	      /*v001225*/
	      = (1 << bits_per_pixel + 6/*BUGFIX*//*dct_precision + intra_dc_precision[kk]*/) - 1;
	      /*=  ((Int)pow(2, GetVopGrayBitsPerPixel_st(curr,kk)+GetVopGrayIntraDCPrecision_st(curr,kk,field_parity))-1)*dc_scaler;*//*WD2.1*/
#endif 0
	    /*GetVopGrayOpaque_Studio(curr)*8;*//*GetVopGrayMidValue_Studio(curr)*8;*//*WD1.3*/
	    /*                = (((OPAQUE*8) + (dc_scaler>>1)) / dc_scaler) * dc_scaler;*/
	    for(n=1; n<15; n++)
	      DC_store[kk][mb_y*MB_width+mb_x][m][n] = 0;
	    /* Ref(end) : MS TestMemo 3 */
	  }
	} else {
          for (m = 0; m < 4; m++) {
	    /*             DC_store[mb_y*MB_width+mb_x][m][0] = MIDOPAQUE*8;*/
	    DC_store[kk][mb_y*MB_width+mb_x][m][0]
	      /*v001225_1 FIX*/
	      = GetVopGrayOpaque_Studio(curr,kk) * (1 << GetVopGrayDCTPrecision_st(curr,kk,field_parity)+GetVopGrayIntraDCPrecision_st(curr,kk,field_parity));
#if 0
	      = dc_reset_value;/*GetVopGrayMidValue_Studio(curr,kk)*8;*//*v001225*/
#endif 0
	    for(n=1; n<15; n++)
	      DC_store[kk][mb_y*MB_width+mb_x][m][n] = 0;
          }
	}  
      } else {/* Only for Inter */
#if 0 /*inter*/
	for (m = 0; m < 4; m++) {
	  /*         DC_store[mb_y*MB_width+mb_x][m][0] = MIDOPAQUE*8;*/
	  DC_store[mb_y*MB_width+mb_x][m][0] = GetVopGrayMidValue_Studio(curr)*8;
	  for(n=1; n<15; n++)
	    DC_store[mb_y*MB_width+mb_x][m][n] = 0;
	}  
	
	for(i=0;i<2;i++)
	for(j=0;j<2;j++)
	  BlockRebuild_Studio(recon, vop_structure, field_parity,
			      mb_x*MB_SIZE+j*8, mb_y*MB_SIZE+i*8, MB_width*16, block_transp[kk]);
	/*      for (i=0; i<MB_SIZE; i++)
		for (j=0; j<MB_SIZE; j++)
		recon[(mb_y*MB_SIZE+i)*GetImageSizeX(GetVopG(rec_curr)) + (mb_x*MB_SIZE+j)] = (SInt)0;*/
#endif 0 /*inter*/
      }
    }
    
    FreeImage(mblock);
#if 0 /*AM2PDAM*/
    FreeImage(alpha_block);
#endif 0
    
  } /* -roop- aux_comp */
}


/*v001215*/
void
CodeGrayscaleAlpha_StudioFDAM(Bits *bits,
			      Vop *curr,
			      Vop *rec_curr,
			      Vop *orig_curr,
			      Vop *comp,
			      Int MB_width,
			      Int mb_x,
			      Int mb_y,
			      /*AM2PDAM*/
			      Int ***DC_store[],
			      SInt *QP_store[],
			      Int intra_dc_precision[],
			      Int q_scale_type[],
			      Int /***/QP[],
			      Int QP_last_coded[],
			      /*AM2PDAM*/
			      Int Mode,
			      Int fieldDCT,/*AM2PDAM*/
			      Int MB_transp_pattern[4],
			      Image *shape_bitstream,
			      VolConfig *vol_config,
			      Int is_slice_start_mb)
{
  Int vop_structure = GetVopVopStructure_st(curr);
  Int field_parity = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(curr), GetVopTopFieldFirst(curr));
  
  Int i,j,k,l,m,n;
  Image *mblock;
  Short *pixel;
  Int alpha_mb_all_opaque=0;
  Int alpha_residue_all_zero=0;
  Int fblock[4][8][8];
  Int coeff[64];
  Int *coeff_ind = coeff;
  Int qcoeff[256];
  Int *qcoeff_ind;
  Int rcoeff[256];
  Int *rcoeff_ind;
  Image **currentImage;
  SInt *recon[2];
  Int CBPA=0;                   /* JDL 21-APR-97 : initialised CBPA */
  Int block_nr;
  Int *qmat;
  Int direction[4];
  Int ACpred_flag = -1;
  Int dc_scaler;
  
#if 0 /*AM2PDAM*/
  Image *alpha_block;
  SInt *alpha_pixel;
  Int num_transp=0;
  Int fieldDCT=0;
#endif 0
  
  Int DQUANT=0;
  Int DC_prev;

  /*WD2.1_2*/
/*  Int coded_fblock[4][8][8];*/
  /*
  Int *raw_blocks;
  Int a_uncompress_flag, a_ucbp_code;
  */
  Int tmp_dc;
  Int ii, jj, kk;
  Int tmp_dc_prev;
  /*WD2.1_2*/
  
  Int block_opaque[MAX_AUX_COMP_NUM][8][8], block_transp[MAX_AUX_COMP_NUM][8][8];

  /*AM2PDAM*/
  Int *raw_blocks;
  Image *mb_vlc_bitstream;
  Image *mb_raw_bitstream;
  Bits bits_mbvlc;
  Int bits_mbraw;
  /*AM2PDAM*/
  
  /*v001225*/
  Int bits_per_pixel;
  Int dct_precision;
  Int dc_reset_value;
  Int intra_dc_mult;
  Int dpcm_block_mean;
  /*v001225*/

  Image *orig_mblock;
  Short *orig_pixel;

  if(Mode==MODE_UNCOMPRESS) {
    Mode = MODE_INTRA;
    fieldDCT = 0;
  }
  
  /*AM2PDAM*/
  for(kk=0;kk<GetVopAuxCompCount_st(curr);kk++) {

    /*v001225*/
    bits_per_pixel = GetVopGrayBitsPerPixel_st(curr,kk);
    dct_precision = GetVopGrayDCTPrecision_st(curr,kk,field_parity);
    dc_reset_value = 1 << bits_per_pixel + 6/*BUGFIX*//*dct_precision + intra_dc_precision[kk]*/ - 1;
    intra_dc_mult = 1 << 6 - dct_precision - intra_dc_precision[kk];
    /*v001225*/
    
    for(i=0;i<8;i++) {
      for(j=0;j<8;j++) {
	block_opaque[kk][i][j] = GetVopGrayOpaque_Studio(curr,kk);
	block_transp[kk][i][j] = 0;
      }
    }
    
    /* initializing */
    CBPA=0;

#if 0    
    if(Mode==MODE_UNCOMPRESS) {
      Mode = MODE_INTRA;
      fieldDCT = 0;
    }
#endif 0
    
    /* Get alpha block data */
    for(i=0;i<2;i++)
      recon[i] = (SInt *) GetImageData(GetVopG_st(rec_curr,kk,i));
    mblock = AllocImage(16,16,SHORT_TYPE);
    GetSubImage_Studio(GetVopG2_st(curr,kk), vop_structure, field_parity,
		       mblock, mb_x*MB_SIZE, mb_y*MB_SIZE);
    pixel = (Short *)GetImageData(mblock);

    /****/
    orig_mblock = AllocImage(16,16,SHORT_TYPE);
    GetSubImage_Studio(GetVopG2_st(orig_curr,kk), vop_structure, field_parity,
		       orig_mblock, mb_x*MB_SIZE, mb_y*MB_SIZE);
    orig_pixel = (Short *)GetImageData(orig_mblock);
    
#if 0 /*AM2PDAM*/
    alpha_block = AllocImage(16,16,SHORT_TYPE);
    GetSubImage_Studio(GetVopA2_st(rec_curr), vop_structure, field_parity,
		       alpha_block, mb_x*MB_SIZE, mb_y*MB_SIZE);
    alpha_pixel = (SInt *)GetImageData(alpha_block);
#endif 0 /*AM2PDAM*/
    
    QP_store[kk][mb_y*MB_width+mb_x] = (SInt) /***/QP[kk];
    
    /* Ref(start) : MS TestMemo 3 */
    dc_scaler = cal_dc_scaler_Studio(intra_dc_precision[kk]/*GetVopGrayIntraDCPrecision_st(curr,field_parity)*/);

/***    if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {***/
      alpha_mb_all_opaque = 1;
      for (i=0; i<MB_SIZE; i++)
	for (j=0; j<MB_SIZE; j++)
	  if(orig_pixel[i*MB_SIZE+j] != GetVopGrayOpaque_Studio(curr,kk)/*OPAQUE*/)   /* Ref(insert) : MS TestMemo 4 */
	    alpha_mb_all_opaque = 0;
/***    }***/
fprintf(stdout, "mb[%d] alpha_mode = %d\n",     mb_y*MB_width+mb_x, Mode);
    if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {
      /* writing CODA */
      BitstreamPutBits(shape_bitstream, alpha_mb_all_opaque, 1);
      bits->CODA += 1;
fprintf(stdout,"bf: intra: mb[%d] amao = %d\n",mb_y*MB_width+mb_x,alpha_mb_all_opaque);
    } else {
      alpha_residue_all_zero = 1;
      for (i=0; i<MB_SIZE; i++)
	for (j=0; j<MB_SIZE; j++)
	  if(pixel[i*MB_SIZE+j] != 0)
	    alpha_residue_all_zero = 0;
fprintf(stdout,"bf: inter: mb[%d] amao = %d araz = %d\n",mb_y*MB_width+mb_x,alpha_mb_all_opaque,alpha_residue_all_zero);
      
      if(alpha_residue_all_zero) {
	BitstreamPutBits(shape_bitstream, 1, 1);
	bits->CODA += 1;
      } else if(alpha_mb_all_opaque) {
	BitstreamPutBits(shape_bitstream, 1, 2);
	bits->CODA += 2;
      } else {
	BitstreamPutBits(shape_bitstream, 0, 2);
	bits->CODA += 2;
      }
    }

    
    if (!alpha_mb_all_opaque && !alpha_residue_all_zero) {
      
      if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {


	/*Start to Code MB by using Uncompressed MB syntax*//*AM2PDAM*/
	{
	  
          mb_raw_bitstream = BitstreamInit();
          bits_mbraw = 0;
          
          raw_blocks = CodeAlphaMB_DPCM_Studio(curr, kk,
                                               mb_x*MB_SIZE, mb_y*MB_SIZE, 
                                               mb_raw_bitstream,
                                               &bits_mbraw,
                                               &dpcm_block_mean);
	}
	/*End of Coding MB by using Uncompressed MB syntax*/
	  
	/*Start to Code MB by using DCT*/
	{
	  mb_vlc_bitstream = BitstreamInit();
	  Bits_Reset(&bits_mbvlc);

	
	/*Update quantizer*/
	DQUANT = GetMBDQuant_Studio(/***/QP[kk], vol_config->gray_mb_quantizer_st[kk]);
#if 0
	/***/QP[kk] += DQUANT;
	QP_store[kk][mb_y*MB_width+mb_x] = (SInt) /***/QP[kk];
#endif 0
	  
	/* Set temporal blocks */
	block_nr=0;
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    for (k=0; k<8; k++)
	      for (l=0; l<8; l++)
		fblock[block_nr][k][l] = pixel[(i+k)*MB_SIZE + (j+l)];
	    block_nr++;
	  }
	
	/*AM2PDAM*/
	if(fieldDCT)
	  GrayConvForFieldDCT_Studio(&fblock[0][0][0]);
	
	/* process for each block */
	block_nr=0;
	/*a_ucbp_code=0;*/ /*WD2.1_2*/
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    /*a_ucbp_code<<=1;*//*WD2.1_2*/
	    if (MB_transp_pattern[block_nr] != 1) { /* non transparent block */
	      /* DCT block */
	      BlockDCT (fblock[block_nr], coeff_ind);
	      
	      qcoeff_ind = &(qcoeff[block_nr*64]);
	      rcoeff_ind = &(rcoeff[block_nr*64]);
	      
	      /* Quantize & Dequantize DCTed block */
	      if(1/*GetVopQuantType(curr)*/)
		{
		  if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q)
		    qmat = GetVopGrayIntraQuantMat_st(curr,kk);
		  else
		    qmat = GetVopGrayNonintraQuantMat_st(curr,kk);
		  
		  /* Ref(start) : MS TestMemo 1 */
		  /*v001225*/
		  BlockQuantMPEG_DCText_Studio(coeff_ind,/***/QP[kk]+DQUANT,Mode,1,
					       intra_dc_precision[kk], q_scale_type[kk], qmat,
					       dct_precision, bits_per_pixel,/*v001225*/
					       qcoeff_ind,
					       GetMaxQDC_DCText_Studio(bits_per_pixel,dct_precision,intra_dc_precision[kk]),/*v001225*/
					       1);
		  /*v010113_0*/
		  BlockDequantMPEG_DCText_DeadZone_Studio(qcoeff_ind,/***/QP[kk]+DQUANT,Mode,1,
							  intra_dc_precision[kk], q_scale_type[kk], qmat,
							  dct_precision, bits_per_pixel,/*v001225*/
							  GetVopDeadZoneDisable_st(curr,field_parity),
							  rcoeff_ind,1,0);
#if 0
		  BlockDequantMPEG_DCText_Studio(qcoeff_ind,/***/QP[kk]+DQUANT,Mode,1,
						 intra_dc_precision[kk], q_scale_type[kk], qmat,
						 dct_precision, bits_per_pixel,/*v001225*/
						 rcoeff_ind,1);
#endif 0
#if 0
		  BlockQuantMPEG_Studio(coeff_ind,/***/QP[kk]+DQUANT,Mode,1,
					intra_dc_precision[kk], q_scale_type[kk], qmat,
					qcoeff_ind,
					GetMaxQDC_Studio(GetVopGrayBitsPerPixel_st(curr,kk),intra_dc_precision[kk]),1);
		  BlockDequantMPEG_Studio(qcoeff_ind,/***/QP[kk]+DQUANT,Mode,1,
					  intra_dc_precision[kk], q_scale_type[kk], qmat,
					  rcoeff_ind,1);
#endif 0
		}
	      else {
		fprintf(stderr, "H263 quant is not used for Studio Profile.\n");
		exit(1);
		/* Ref(start) : MS TestMemo 1 */
		BlockQuantH263(coeff_ind,*QP,Mode,1,qcoeff_ind,OPAQUE,2);
		BlockDequantH263(qcoeff_ind,*QP,Mode,1,rcoeff_ind,2, 0, GetVopBitsPerPixel(curr));
		/* Ref(end) : MS TestMemo 1 */
	      }
	      
	      /*v001225*/
	      BlockIDCT_DCText_Studio (rcoeff_ind, fblock[block_nr], GetVopGrayOpaque_Studio(curr,kk),0);
	      /*BlockIDCT (rcoeff_ind, fblock[block_nr], GetVopGrayOpaque_Studio(curr,kk));*/
	      /* original fblock is updated to reconstructed fblock. */

	      for (k=0;k<8;k++)
		for (l=0;l<8;l++)
		  fblock[block_nr][k][l] =  MIN(GetVopGrayOpaque_Studio(curr,kk),MAX(0,fblock[block_nr][k][l]));  /* intra clipping */

	      /* Here, get a qcoeffed and a reconstructed macroblock */

	      /* Calclate coded_block_pattern_alpha */
/*	      
	      for(k=1; k<64; k++)
		if (*(qcoeff_ind+k))
*//*AM2PDAM*/
	      CBPA |= 1<<(3-block_nr);/* for transparent block, this value isn't updated. */

	      /* Store the qcoeff-values needed later for prediction */
	      DC_store[kk][mb_y*MB_width+mb_x][block_nr][0] = *(qcoeff_ind) * intra_dc_mult;/* * dc_scaler;*//*v001225*/
	      for (m = 1; m < 8; m++)
		DC_store[kk][mb_y*MB_width+mb_x][block_nr][m] = *(qcoeff_ind+m);
	      for (m = 0; m < 7; m++)
		DC_store[kk][mb_y*MB_width+mb_x][block_nr][m+8] = *(qcoeff_ind+(m+1)*8);
	      
	      /* DC prediction */
	      if(is_slice_start_mb && block_nr==0)
		DC_prev = dc_reset_value;/*GetVopGrayMidValue_Studio(curr,kk)*8;*//*v001225*/
	      else
		DC_prev = DC_store[kk][mb_y*MB_width+(4*mb_x+block_nr-1)/4][(4*mb_x+block_nr-1)%4][0];
	      *(qcoeff_ind) -= DC_prev / intra_dc_mult; /*dc_scaler;*//*v001225*/

	    }/* non transparent block */
	    else {/*transparent block*/
	      /* Get a quantized and dequantized block for a transparent block */
	      qcoeff_ind = &(qcoeff[block_nr*64]);/*WD1.3*/
	      for(k=0; k<64; k++)
		*(qcoeff_ind+k) = 0;
	      for (k=0; k<8; k++)
		for (l=0; l<8; l++)
		  fblock[block_nr][k][l] = 0;
	      
	      /* overwrite the DC_store with QDC = MIDOPAQUE for transparent blocks */
	      DC_store[kk][mb_y*MB_width+mb_x][block_nr][0] = dc_reset_value;/*GetVopGrayMidValue_Studio(curr,kk)*8;*//*v001225*/
	    }/*transparent block*/
	    
	    block_nr++;
	  }/* 4blocks roop */

#if 0	
	/* Buck up coded fblock */
	for(k=0;k<4;k++)
	  for(j=0;j<8;j++)
	    for(i=0;i<8;i++)
	      coded_fblock[k][j][i] = fblock[k][j][i];
#endif 0

	/* Deinterlace */
	/*AM2PDAM*/
	if (/*curr->interlaced && (vop_structure == FRAME_VOP)
	      && (GetVopGrayFramePredFrameDCT_st(curr) == 0)
	      && */fieldDCT)
	  fieldDCTtoFrame(&fblock[0][0][0]);

	/* Put the reconstructed fblock to memory at the position of the current block */
	block_nr=0;
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    BlockRebuild_Studio(recon, vop_structure, field_parity,
				mb_x*MB_SIZE+j, mb_y*MB_SIZE+i, MB_width*16, fblock[block_nr]);
	    block_nr++;
	  }

#if 0
	if(GetVopIntraACDCPredDisable(curr) == 0)
	  {
	    ACpred_flag = doDCACpred_grey(
					  qcoeff,
					  &CBPA,
					  64,
					  mb_x,
					  mb_y,
					  DC_store,
					  QP_store,
					  QP,
					  MB_width,
					  direction,
					  MIDOPAQUE);
	    BitstreamPutBits(shape_bitstream, (long)ACpred_flag, 1L);
	    bits->g_ACpred_flag += 1;
	  }
	else
#endif 0
	  
	  ACpred_flag = -1;
	
	/* BitstreamPut CBPA */
/*	  
	bits_mbvlc.mbheader += (bits_mbvlc.CBPA += PutCBPY(CBPA,0,MB_transp_pattern,mb_vlc_bitstream) );
*//*AM2PDAM*/	
	
	/* BitstreamPut gray_quantizer */
	if(DQUANT)
	  BitstreamPutBits(mb_vlc_bitstream, 1L, 1);
	else
	  BitstreamPutBits(mb_vlc_bitstream, 0L, 1);
	bits_mbvlc.g_QUANT += 1;
	bits_mbvlc.mbheader += 1;
	  
	if(DQUANT) {
	  BitstreamPutBits(mb_vlc_bitstream, /***/QP[kk]+DQUANT, 5L);
	  bits_mbvlc.g_QUANT += 5;
	  bits_mbvlc.mbheader += 5;
	}

#if 0
	/* BitstreamPut gray_dct_type */
	if (curr->interlaced) {
	  if(vop_structure == FRAME_VOP && GetVopFramePredFrameDCT_st(curr)/*AM2PDAM*/ == 0) {
	    bits->g_fieldDCT += fieldDCT;
	    BitstreamPutBits(shape_bitstream, fieldDCT, 1);
	    bits->g_interlaced++;
	  }
	}

	/*WD2.1_2*/
	if(a_ucbp_code)
	  a_uncompress_flag = 1;
	else
	  a_uncompress_flag = 0;
	
	/* Put a_uncompress_flag and a_ucbp_code to shape_bitstream */
	BitstreamPutBits(shape_bitstream, a_uncompress_flag, 1L);
	if(a_uncompress_flag) {
	  for(k=0;k<4;k++) {
	    if(MB_transp_pattern[k]!=1) {
	      BitstreamPutBits(shape_bitstream, ((a_ucbp_code)>>(4-k-1))&1, 1L);
	    }
	  }
	}
	/*WD2.1_2*/
#endif 0
	
	/* Code gray block */
	CodeAlphaBlockData_DCT_StudioPDAM(&bits_mbvlc,
				      MB_transp_pattern,
				      GetVopAlternateScan_st(curr, field_parity)/*AM2PDAM*/,
				      qcoeff,
				      Mode,
				      CBPA,
				      64,
				      mb_vlc_bitstream,
				      GetVopGrayBitsPerPixel_st(curr,kk),
					  GetVopGrayBitsPerPixel_st(curr,kk)+GetVopGrayDCTPrecision_st(curr,kk,field_parity)+4/*v001225*/);
#if 0
	/*WD2.1_2*/
	CodeAlphaBlockData_DCT_Studio(&bits_mbvlc,
				      MB_transp_pattern,
				      GetVopAlternateScan_st(curr, field_parity)/*AM2PDAM*/,
				      qcoeff,
				      Mode,
				      CBPA,
				      64,
				      mb_vlc_bitstream,
				      GetVopGrayBitsPerPixel_st(curr,kk));
#endif 0
#if 0
	CodeAlphaBlockData_Through_Studio(&bits_mbvlc,
					  /*WD2.1_2*/
					  a_ucbp_code,
					  coded_fblock,
					  /*WD2.1_2*/
					  MB_transp_pattern,
					  GetVopAlternateScan_st(curr, field_parity)/*AM2PDAM*/,
					  qcoeff,
					  Mode,
					  CBPA,
					  64,
					  mb_vlc_bitstream,
					  GetVopGrayBitsPerPixel_st(curr,kk));
#endif 0
	  }
	  /*End of Coding MB by using DCT*/

	/* Decide which coding scheme is used to encode the macroblock */
	/* Append the decided bitstream to slice_bitstream */
	/* Append the bitcount data to bits structure */
	/* Put the coded macroblock to the memory if the mode is the uncompressed */
	/* Update DC_store if the uncompressed */
	{
	  Int *raw_blocks_ind;
	  Int tmp_dc;
	  Int compression_mode;
	  
	  Int vlcbit;
	  vlcbit = bits_mbvlc.mbheader + bits_mbvlc.G;
	  
	  /* 1. Mode Decision */
	  if(GetVolConfigThroughMode_st(vol_config)==0) /*only VLC coding*/
	    compression_mode = 0;
	  else if(GetVolConfigThroughMode_st(vol_config)==1) /*only Uncompress coding*/
	    compression_mode = 1;
	  else if(GetVolConfigThroughMode_st(vol_config)==2) { /*A coding scheme which has generated less number of bits.*/
            compression_mode = GetRandomValue_Studio(0,1);
#if 0	    
	    Int vlcbit, rawbit;
	    vlcbit = bits_mbvlc.mbheader + bits_mbvlc.G;
	    rawbit = bits_mbraw;
	    
	    if( vlcbit < rawbit ) /*DCT coding*/
	      compression_mode = 0;
	    else /*Uncompress coding*/
	      compression_mode = 1;
#endif 0    
	  } else if(GetVolConfigThroughMode_st(vol_config)==3) { /*The DCT coding is selected if less than the threshold*/
	    Int vlcbit;
	    vlcbit = bits_mbvlc.mbheader + bits_mbvlc.G;
	    
	    if( vlcbit < GetVolConfigUncompressThreshold_st(vol_config) )
	      compression_mode = 0;
	    else
	      compression_mode = 1;
	    
	  } else {
	    fprintf(stderr, "this compression mode is not supported[%d].\n", GetVolConfigThroughMode_st(vol_config));
	    exit(-1);
	  }
	  
	  /*2. Put compression_mode to slice_stream*/
	  /* 011210_1 Syntax */
	  BitstreamPutBits(shape_bitstream, !compression_mode /*compression_mode*/, 1);
          bits->mbheader ++;
	  
	  /*3. Put a selected MB data to slice_stream*/
	  if(!compression_mode) { /*VLC mode*/
	    
	    BitstreamAppend(mb_vlc_bitstream,shape_bitstream);
	    AddBits_Studio(&bits_mbvlc, bits);
	    
	    /*Update coded QP*//*moved*/
	    QP[kk]+=DQUANT;
	    QP_last_coded[kk]=QP[kk];
	    
	  } else { /*Uncompress mode*/
	    /* MoMuSys-2000FDAM1COR-020612 */
	    /* marker_bit */
	    bits->mbheader ++;
	    /**/
	    
            bits->mbheader ++; /*alpha_dpcm_scan_order*/
	    
	    BitstreamAppend(mb_raw_bitstream,shape_bitstream);
	    
	    bits->G_raw  += bits_mbraw;
	    
	    raw_blocks_ind = raw_blocks;
	    
	    for(k=0;k<4;k++) {
	      if(MB_transp_pattern[k]!=1) {
		
		/* Update texture */
		BlockRebuild_Studio (recon, vop_structure, field_parity, mb_x*MB_SIZE+(k%2)*8, mb_y*MB_SIZE+(k/2)*8, MB_width*MB_SIZE, raw_blocks_ind);
		
		/*v001225*/
		DC_store[kk][mb_y*MB_width+mb_x][k][0] = dpcm_block_mean*(1<<6/*dct_precision+intra_dc_precision[kk]*/);
#if 0
		/* Update DC_store */
		tmp_dc = 0;
		for(ii=0;ii<64;ii++) {
		  tmp_dc += *raw_blocks_ind;
		  raw_blocks_ind++;
		}
		tmp_dc /= 8;
		DC_store[kk][mb_y*MB_width+mb_x][k][0] = tmp_dc;
#endif 0
		
	      } else { /*transp bl.*/
		/***raw_blocks_ind += 64;***/
		
		/*v001225*/
		DC_store[kk][mb_y*MB_width+mb_x][k][0] = dpcm_block_mean*(1<<6/*dct_precision+intra_dc_precision[kk]*/);
		/* Texture has already been filled with 0 in CodeMB_DCT_Studio() */
		/* The DC_store for transp bl. has been already updated in VLC routine with the value, GetVopMidGrey(curr)*8 */
	      }
              raw_blocks_ind += 64;	      
	    }
	    
	  }
	  
	  /*Free pointers*/
	  free ((Char*)raw_blocks);
	  BitstreamFree(mb_vlc_bitstream);
	  BitstreamFree(mb_raw_bitstream);
	} /*Mode decision, and the following procedures*/
	
                

	
      }/*Intra MB*/
      else {      /* P-VOP and not an intra-MB */

	{ /*** DPCM ***/
          mb_raw_bitstream = BitstreamInit();
          bits_mbraw = 0;
          
          raw_blocks = CodeAlphaMB_DPCM_Studio(orig_curr, kk,
                                               mb_x*MB_SIZE, mb_y*MB_SIZE, 
                                               mb_raw_bitstream,
                                               &bits_mbraw,
                                               &dpcm_block_mean);
	} /*** DPCM ***/

	{ /*** DCT ***/
        mb_vlc_bitstream = BitstreamInit();
        Bits_Reset(&bits_mbvlc);
	
        /*Update quantizer*/
        DQUANT = GetMBDQuant_Studio(QP[kk], vol_config->gray_mb_quantizer_st[kk]);

        /* Set temporal blocks */
        block_nr=0;
        for (i=0; i<MB_SIZE; i+=8)
          for (j=0; j<MB_SIZE; j+=8) {
            for (k=0; k<8; k++)
              for (l=0; l<8; l++)
                fblock[block_nr][k][l] = pixel[(i+k)*MB_SIZE + (j+l)];
            block_nr++;
          }
	
        if(fieldDCT)
          GrayConvForFieldDCT_Studio(&fblock[0][0][0]);
        
	block_nr=0;
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    qcoeff_ind = &(qcoeff[block_nr*64]);
	    rcoeff_ind = &(rcoeff[block_nr*64]);
	    
	    if(MB_transp_pattern[block_nr]!=1) {
	      
	      BlockDCT (fblock[block_nr], coeff_ind);
	      
	      if(1/*GetVopQuantType(curr)*/)
		{
                  if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q)
                    qmat = GetVopGrayIntraQuantMat_st(curr,kk);
                  else
                    qmat = GetVopGrayNonintraQuantMat_st(curr,kk);

		  BlockQuantMPEG_DCText_Studio(coeff_ind,QP[kk]+DQUANT,Mode,1,
					       intra_dc_precision[kk], q_scale_type[kk], qmat,
					       dct_precision, bits_per_pixel,/*v001225*/
					       qcoeff_ind,
					       GetMaxQDC_DCText_Studio(bits_per_pixel,dct_precision,intra_dc_precision[kk]),/*v001225*/
					       2);

		  /*v010113_0*/
		  BlockDequantMPEG_DCText_DeadZone_Studio(qcoeff_ind,QP[kk]+DQUANT,Mode,1,
							  intra_dc_precision[kk], q_scale_type[kk], qmat,
							  dct_precision,bits_per_pixel,/*v001225*/
							  GetVopDeadZoneDisable_st(curr,field_parity),
							  rcoeff_ind,2,0);
#if 0
		  BlockDequantMPEG_DCText_Studio(qcoeff_ind,QP[kk],Mode,1,
						 intra_dc_precision[kk], q_scale_type[kk], qmat,
						 dct_precision,bits_per_pixel,/*v001225*/
						 rcoeff_ind,2);
#endif 0      
#if 0	      
                  BlockQuantMPEG_Studio(coeff_ind,QP[kk]+DQUANT,Mode,1,
                                        intra_dc_precision[kk], q_scale_type[kk], qmat,
                                        qcoeff_ind,
                                        GetMaxQDC_Studio(GetVopGrayBitsPerPixel_st(curr,kk),intra_dc_precision[kk]),2);
                  BlockDequantMPEG_Studio(qcoeff_ind,QP[kk]+DQUANT,Mode,1,
                                          intra_dc_precision[kk], q_scale_type[kk], qmat,
                                          rcoeff_ind,2);
#endif 0
		}
	      else {
		fprintf(stderr, "WRONG\n");
		exit(-1);
		/* Ref(start) : MS TestMemo 1 */
		BlockQuantH263(coeff_ind,QP,Mode,1,qcoeff_ind,OPAQUE,2);
		BlockDequantH263(qcoeff_ind,QP,Mode,1,rcoeff_ind,2, 0, GetVopBitsPerPixel(curr));
		/* Ref(end) : MS TestMemo 1 */
	      }
	      
	      BlockIDCT_DCText_Studio(rcoeff_ind, fblock[block_nr],GetVopGrayOpaque_Studio(curr,kk),0);/*v001225*/
              /*BlockIDCT (rcoeff_ind, fblock[block_nr], GetVopGrayOpaque_Studio(curr,kk));*/

	      for(k=0; k<64; k++)
                if (*(qcoeff_ind+k))
                  CBPA |= 1<<(3-block_nr);
	    } else {
	      for(k=0; k<64; k++)
		*(qcoeff_ind+k) = 0;
	      for (k=0; k<8; k++)
		for (l=0; l<8; l++)
		  fblock[block_nr][k][l] = 0;
	    }  

	    DC_store[kk][mb_y*MB_width+mb_x][block_nr][0] = dc_reset_value;/*GetVopGrayMidValue_Studio(curr,kk)*8;*//*v001225*/
	    
	    block_nr++;
	  }
	
	if (fieldDCT)
	  fieldDCTtoFrame(&fblock[0][0][0]);
	
	block_nr=0;
	for (i=0; i<MB_SIZE; i+=8)
	  for (j=0; j<MB_SIZE; j+=8) {
	    BlockRebuild_Studio(recon, vop_structure, field_parity,
				mb_x*MB_SIZE+j, mb_y*MB_SIZE+i, MB_width*16, fblock[block_nr]);
	    block_nr++;
	  }

#if 0 /****/	
	/*alpha_compression_mode*/
	BitstreamPutBits(shape_bitstream, 0, 1);
	/*bits->XXX += 1;*/
#endif 0 /****/	
	
	/* CBPA */
	bits_mbvlc.CBPA += PutCBPY(CBPA,1,MB_transp_pattern,mb_vlc_bitstream);
fprintf(stdout, "mb[%d] : cbpa = %d\n", mb_y*MB_width+mb_x,CBPA);
	if(CBPA) {
	  if(DQUANT)
	    BitstreamPutBits(mb_vlc_bitstream, 1L, 1);
	  else
	    BitstreamPutBits(mb_vlc_bitstream, 0L, 1);
	  bits_mbvlc.g_QUANT += 1;
	  bits_mbvlc.mbheader += 1;
	    
	  if(DQUANT) {
	    BitstreamPutBits(mb_vlc_bitstream, QP[kk]+DQUANT, 5L);
	    bits_mbvlc.g_QUANT += 5;
	    bits_mbvlc.mbheader += 5;
	  }
	}

	CodeAlphaBlockData_DCT_StudioFDAM(&bits_mbvlc,
					  MB_transp_pattern,
					  GetVopAlternateScan_st(curr, field_parity),
					  qcoeff,
					  Mode,
					  CBPA,
					  64,
					  mb_vlc_bitstream,
					  GetVopGrayBitsPerPixel_st(curr,kk),
					  GetVopGrayBitsPerPixel_st(curr,kk)+GetVopGrayDCTPrecision_st(curr,kk,field_parity)+4/*v001225*/);

      } /*** DCT ***/

	{
          Int *raw_blocks_ind;
          Int compression_mode;
          
          /* 1. Mode Decision */
          if(GetVolConfigThroughMode_st(vol_config)==0) /*only VLC coding*/
            compression_mode = 0;
          else if(GetVolConfigThroughMode_st(vol_config)==1) /*only Uncompress coding*/
            compression_mode = 1;
          else if(GetVolConfigThroughMode_st(vol_config)==2) { /*RANDOM*/
            compression_mode = GetRandomValue_Studio(0,1);
          } else if(GetVolConfigThroughMode_st(vol_config)==3) { /*The DCT coding is selected if less than the threshold*/
            Int vlcbit;
            vlcbit = bits_mbvlc.mbheader + bits_mbvlc.G;
            
            if( vlcbit < GetVolConfigUncompressThreshold_st(vol_config) )
              compression_mode = 0;
            else
              compression_mode = 1;
            
          } else {
            fprintf(stderr, "this compression mode is not supported[%d].\n", GetVolConfigThroughMode_st(vol_config));
            exit(-1);
          }
          
          /*2. Put compression_mode to slice_stream*/
          /* 011210_1 Syntax */
	  BitstreamPutBits(shape_bitstream, !compression_mode /*compression_mode*/, 1);
          bits->mbheader ++;
          
          /*3. Put a selected MB data to slice_stream*/
          if(!compression_mode) { /*VLC mode*/
            
            BitstreamAppend(mb_vlc_bitstream,shape_bitstream);
            AddBits_Studio(&bits_mbvlc, bits);
            
            /*Update coded QP*//*moved*/
            QP[kk]+=DQUANT;
            QP_last_coded[kk]=QP[kk];
            
          } else { /*Uncompress mode*/
	    /* MoMuSys-2000FDAM1COR-020612 */
	    /* marker_bit */
	    bits->mbheader ++;
	    /**/
	    
            bits->mbheader ++; /*alpha_dpcm_scan_order*/
            
            BitstreamAppend(mb_raw_bitstream,shape_bitstream);
            
            bits->G_raw  += bits_mbraw;
            
            raw_blocks_ind = raw_blocks;
            
            for(k=0;k<4;k++) {
              if(MB_transp_pattern[k]!=1) {
                
                /* Update texture */
                BlockRebuild_Studio (recon, vop_structure, field_parity, mb_x*MB_SIZE+(k%2)*8, mb_y*MB_SIZE+(k/2)*8, MB_width*MB_SIZE,
 raw_blocks_ind);
                
                /*v001225*/
                DC_store[kk][mb_y*MB_width+mb_x][k][0] = dpcm_block_mean*(1<< 6/*dct_precision+intra_dc_precision[kk]*/);
                
              } else { /*transp bl.*/
                /** raw_blocks_ind += 64; **/
                
                /*v001225*/
                DC_store[kk][mb_y*MB_width+mb_x][k][0] = dpcm_block_mean*(1<< 6/*dct_precision+intra_dc_precision[kk]*/);
                
                /* Texture has already been filled with 0 in CodeMB_DCT_Studio() */
                /* The DC_store for transp bl. has been already updated in VLC routine with the value, GetVopMidGrey(curr)*8 */
              }                 
              raw_blocks_ind += 64;
            }
            
	  }
          
          /*Free pointers*/
          free ((Char*)raw_blocks);
          BitstreamFree(mb_vlc_bitstream);
          BitstreamFree(mb_raw_bitstream);
        } /*Mode decision, and the following procedures*/

	


     	
	
      }/*Inter MB*/
    }
    else {
      if(alpha_mb_all_opaque) {/* Both for Intra */
	for(i=0;i<2;i++)
	  for(j=0;j<2;j++)
	    BlockRebuild_Studio(recon, vop_structure, field_parity,
				mb_x*MB_SIZE+j*8, mb_y*MB_SIZE+i*8, MB_width*16, block_opaque[kk]);
	/*      for (i=0; i<MB_SIZE; i++)
		for (j=0; j<MB_SIZE; j++)
		recon[(mb_y*MB_SIZE+i)*GetImageSizeX(GetVopG(rec_curr)) + (mb_x*MB_SIZE+j)] = (SInt)OPAQUE;*/
	
	if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) {
	  
	  /* Ref(start) : MS TestMemo 3 */
	  for (m = 0; m < 4; m++) {
	    DC_store[kk][mb_y*MB_width+mb_x][m][0]
	      /*v001225_1 FIX*/
	      = GetVopGrayOpaque_Studio(curr,kk) * (1 << GetVopGrayDCTPrecision_st(curr,kk,field_parity)+GetVopGrayIntraDCPrecision_st(curr,kk,field_parity));
#if 0
	      /*v001225*/
	      = (1 << bits_per_pixel + 6/*BUGFIX*//*dct_precision + intra_dc_precision[kk]*/) - 1;
	      /*=  ((Int)pow(2, GetVopGrayBitsPerPixel_st(curr,kk)+GetVopGrayIntraDCPrecision_st(curr,kk,field_parity))-1)*dc_scaler;*//*WD2.1*/
#endif 0
	    /*GetVopGrayOpaque_Studio(curr)*8;*//*GetVopGrayMidValue_Studio(curr)*8;*//*WD1.3*/
	    /*                = (((OPAQUE*8) + (dc_scaler>>1)) / dc_scaler) * dc_scaler;*/
	    for(n=1; n<15; n++)
	      DC_store[kk][mb_y*MB_width+mb_x][m][n] = 0;
	    /* Ref(end) : MS TestMemo 3 */
	  }
	} else {
          for (m = 0; m < 4; m++) {
	    /*             DC_store[mb_y*MB_width+mb_x][m][0] = MIDOPAQUE*8;*/
	    DC_store[kk][mb_y*MB_width+mb_x][m][0]
	      /*v001225_1 FIX*/
	      = GetVopGrayOpaque_Studio(curr,kk) * (1 << GetVopGrayDCTPrecision_st(curr,kk,field_parity)+GetVopGrayIntraDCPrecision_st(curr,kk,field_parity));
 
#if 0
	      = GetVopGrayMidValue_Studio(curr,kk)*8;
#endif 0
	    for(n=1; n<15; n++)
	      DC_store[kk][mb_y*MB_width+mb_x][m][n] = 0;
          }
	}  
      } else {/* Only for Inter */

	for (m = 0; m < 4; m++) {
	  DC_store[kk][mb_y*MB_width+mb_x][m][0] = dc_reset_value;/*GetVopGrayMidValue_Studio(curr,kk)*8;*//*v001225*/
	  for(n=1; n<15; n++)
	    DC_store[kk][mb_y*MB_width+mb_x][m][n] = 0;
	}  
	
	for(i=0;i<2;i++)
	  for(j=0;j<2;j++)
	    BlockRebuild_Studio(recon, vop_structure, field_parity,
				mb_x*MB_SIZE+j*8, mb_y*MB_SIZE+i*8, MB_width*16, block_transp[kk]);
	
      }
    }
    
    FreeImage(mblock);
#if 0 /*AM2PDAM*/
    FreeImage(alpha_block);
#endif 0
    
  } /* -roop- aux_comp */
}
/*v001215*/

Int *
CodeAlphaMB_Uncompress_Studio(Vop *curr, Int aux_comp_num,
			      Int x_pos, Int y_pos, UInt width,
			      Int MB_transp_pattern[],
			      Int bits_per_pixel,
			      Image *mb_raw_bitstream, Int *bits_mbraw) {
  
  Int vop_structure = GetVopVopStructure_st(curr);
  Int fip = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(curr),
				   GetVopTopFieldFirst(curr));
  
  Int fblock[4][8][8];
  SInt *current[2], *recon[2] = {NULL,NULL};
  Int x, y;
  UInt xwidth;
  Int i, j, k;
  Int kk;

  Int num_transp = 0;
  
  Int *rawmb, *rawmb_ind;/*AM2PDAM*/
  
  Int block_count;
  Int cdivx, cdivy;
  
  rawmb_ind = rawmb = (Int *)malloc(sizeof(Int)*4*64);
  
  /* Get fblock(inputed block) */
  for (k = 0; k < 4; k++) {
    switch (k) {
    case 0:
      x = x_pos;
      y = y_pos;
      xwidth = width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    case 1:
      x = x_pos + 8;
      y = y_pos;
      xwidth = width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    case 2:
      x = x_pos;
      y = y_pos + 8;
      xwidth = width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    case 3:
      x = x_pos + 8;
      y = y_pos + 8;
      xwidth = width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    default:
      fprintf (stdout, "CodeMB (): Non-existing case in switch statement\n");
      exit (-1);
      break;
    }
    BlockPredict_Studio(current, vop_structure, fip, recon, x, y, xwidth, fblock[k]);
  }/* for (k = 0; k < 4; k++) */

  /*Get rawmb */
  for(k=0;k<4;k++)
    for(i=0;i<8;i++)
      for(j=0;j<8;j++) {
	*rawmb_ind = fblock[k][i][j];
	rawmb_ind++;
      }

  rawmb_ind = rawmb;
  *bits_mbraw = 0;
  
  /* Process each block */
  for(k=0; k<4; k++) {

    if(MB_transp_pattern[k]!=1) { /* Nontransparent block */
      *bits_mbraw +=
	BitstreamPutUncompressedBlock(rawmb_ind, bits_per_pixel, mb_raw_bitstream);
    }
    
    rawmb_ind += 64;
    
  } /*for(k=0; k<4; k++)*/

  return rawmb;
}

Int *
CodeAlphaMB_DPCM_Studio(Vop *curr, Int aux_comp_num,
		   Int x_pos, Int y_pos,
		   Image *mb_dpcm_bitstream,
		   Int *bits_mbraw,
		   Int *block_mean) {
  
  Int vop_structure = GetVopVopStructure_st(curr);
  Int fip = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(curr),
				   GetVopTopFieldFirst(curr));
  Int bits_per_pixel = GetVopGrayBitsPerPixel_st(curr,aux_comp_num);
  Int vop_width = GetVopWidth_st(curr, fip);
  Int vop_height = GetVopHeight_st(curr, fip);
  
  Int *rawmb, *rawmb_ind;/*AM2PDAM*/
  
  Int fblock[4][8][8];
  SInt *current[2], *recon[2] = {NULL,NULL};
  Int x, y;
  UInt xwidth;
  Int i, j, k;
  Int kk;

  Int ii, jj;
  
  SInt compblock[256];

  Int dpcm_scan_order = 0;
  
#if 0
  Image *Sub_Alpha= AllocImage(MB_SIZE, MB_SIZE, SHORT_TYPE);
  SInt  *sub_alpha_ptr;
#endif
  
  Int num_transp = 0;
  
/*  Int binary_number;*/

  Int block_count;
  Int cdivx, cdivy;
  
  rawmb_ind = rawmb = (Int *)malloc(sizeof(Int)*4*64);

  /* Get fblock(inputed block) */
  for (k = 0; k < 4; k++) {
    switch (k) {
    case 0:
      x = x_pos;
      y = y_pos;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    case 1:
      x = x_pos + 8;
      y = y_pos;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    case 2:
      x = x_pos;
      y = y_pos + 8;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    case 3:
      x = x_pos + 8;
      y = y_pos + 8;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopG_st (curr,aux_comp_num,kk));
      break;
    default:
      fprintf (stdout, "CodeMB (): Non-existing case in switch statement\n");
      exit (-1);
      break;
    }
    BlockPredict_Studio(current, vop_structure, fip, recon, x, y, xwidth, fblock[k]);
  }/* for (k = 0; k < 4; k++) */

  /*Get rawmb */
  for(k=0;k<4;k++)
    for(i=0;i<8;i++)
      for(j=0;j<8;j++) {
	*rawmb_ind = fblock[k][i][j];
	rawmb_ind++;
      }

  /*Get compblock*/
  for(k=0;k<4;k++)
    for(j=0;j<8;j++)
      for(i=0;i<8;i++)
	compblock[((k/2)*8+j)*16+((k%2)*8+i)] = fblock[k][j][i];

  /* MoMuSys-2000FDAM1COR-020612 */
  /* marker_bit */
  BitstreamPutBits(mb_dpcm_bitstream,1,1);
  /**/
  
  BitstreamPutBits(mb_dpcm_bitstream,dpcm_scan_order,1);
  /*bits_dpcm_header = 1;*/

  *bits_mbraw = BitstreamPutDPCMBlock(compblock, 16, 16, bits_per_pixel,
				      dpcm_scan_order,
				      mb_dpcm_bitstream,
				      block_mean);

  return rawmb;
}


Int
GrayConvForFieldDCT_Studio(Int *fblock)
/*decision:0-decision algorithm*/
/*decision:1-force to perform FieldDCT*/
{
    static unsigned char mbline[] = {
         0*BLOCK_SIZE,  2*BLOCK_SIZE,  4*BLOCK_SIZE,  6*BLOCK_SIZE,
        16*BLOCK_SIZE, 18*BLOCK_SIZE, 20*BLOCK_SIZE, 22*BLOCK_SIZE,
         8*BLOCK_SIZE, 10*BLOCK_SIZE, 12*BLOCK_SIZE, 14*BLOCK_SIZE,
        24*BLOCK_SIZE, 26*BLOCK_SIZE, 28*BLOCK_SIZE, 30*BLOCK_SIZE,  };
    static unsigned char fwd_shuffle[] = {
         1*BLOCK_SIZE, 0*BLOCK_SIZE,    2*BLOCK_SIZE, 1*BLOCK_SIZE,    4*BLOCK_SIZE, 2*BLOCK_SIZE,
        16*BLOCK_SIZE, 4*BLOCK_SIZE,    0*BLOCK_SIZE,16*BLOCK_SIZE,    3*BLOCK_SIZE, 0*BLOCK_SIZE,
         6*BLOCK_SIZE, 3*BLOCK_SIZE,   20*BLOCK_SIZE, 6*BLOCK_SIZE,   17*BLOCK_SIZE,20*BLOCK_SIZE,
         0*BLOCK_SIZE,17*BLOCK_SIZE,    5*BLOCK_SIZE, 0*BLOCK_SIZE,   18*BLOCK_SIZE, 5*BLOCK_SIZE,
         0*BLOCK_SIZE,18*BLOCK_SIZE,    7*BLOCK_SIZE, 0*BLOCK_SIZE,   22*BLOCK_SIZE, 7*BLOCK_SIZE,
        21*BLOCK_SIZE,22*BLOCK_SIZE,   19*BLOCK_SIZE,21*BLOCK_SIZE,    0*BLOCK_SIZE,19*BLOCK_SIZE,
         9*BLOCK_SIZE, 0*BLOCK_SIZE,   10*BLOCK_SIZE, 9*BLOCK_SIZE,   12*BLOCK_SIZE,10*BLOCK_SIZE,
        24*BLOCK_SIZE,12*BLOCK_SIZE,    0*BLOCK_SIZE,24*BLOCK_SIZE,   11*BLOCK_SIZE, 0*BLOCK_SIZE,
        14*BLOCK_SIZE,11*BLOCK_SIZE,   28*BLOCK_SIZE,14*BLOCK_SIZE,   25*BLOCK_SIZE,28*BLOCK_SIZE,
         0*BLOCK_SIZE,25*BLOCK_SIZE,   13*BLOCK_SIZE, 0*BLOCK_SIZE,   26*BLOCK_SIZE,13*BLOCK_SIZE,
         0*BLOCK_SIZE,26*BLOCK_SIZE,   15*BLOCK_SIZE, 0*BLOCK_SIZE,   30*BLOCK_SIZE,15*BLOCK_SIZE,
        29*BLOCK_SIZE,30*BLOCK_SIZE,   27*BLOCK_SIZE,29*BLOCK_SIZE,    0*BLOCK_SIZE,27*BLOCK_SIZE, };
    
    Double frame, field, d;
    Int tmp[BLOCK_SIZE], *p, *q, *plast;
    Int i;

    /*
     * Re-order luminance macroblock lines
     */
    for (i = 0; i < sizeof(fwd_shuffle); i += 2)
      memcpy(fwd_shuffle[i+1] ? &fblock[fwd_shuffle[i+1]] : tmp,
	     fwd_shuffle[i+0] ? &fblock[fwd_shuffle[i+0]] : tmp,
	     BLOCK_SIZE * sizeof(Int));
    
    return(1);
  }

Void CodeAlphaBlockData_DCT_StudioPDAM (Bits *bits,
					Int transp_pattern[], Int alternate_scan,
					Int *qcoeff, Int Mode, Int CBPA, Int ncoeffs,
					Image *bitstream, Int bits_per_pixel,
					Int flc_length/*v001225*/)
{
    Int i, m, coeff[64];
    Int *zz = alternate_scan ? zigzag_v : zigzag;
    Int tmp_coef;

    Int j, k;    /*WD2.1_2*/
    
    if (Mode == MODE_INTRA || Mode == MODE_INTRA_Q)
      {  
	for (i = 0; i < 4; i++)
	  {  
	    if (transp_pattern[i]!=1) /* Not transparent */
	      {
		  
		/* Do the zigzag scanning of coefficients */
		for (m = 0; m < 64; m++)
		  *(coeff + zz[m]) = qcoeff[i*ncoeffs+m];

		tmp_coef = IntraDC_dpcm_Studio_bySVLC(coeff[0],1,bitstream);

		bits->G += tmp_coef;
		bits->Gdc += tmp_coef;

		/* Code AC coeffs. dep. on block pattern MW 15-NOV-1996 */
/*
                if ((i==0 && CBPA&8) ||
		    (i==1 && CBPA&4) ||
		    (i==2 && CBPA&2)  ||
		    (i==3 && CBPA&1))
		  {
*//*AM2PDAM*/		  
		    tmp_coef = CodeCoeff_Studio_bySVLC2(1,Mode, coeff,i,ncoeffs,bitstream,flc_length/*v001225*/);
		    bits->G += tmp_coef;
		    bits->Gac += tmp_coef;
/*		
		  }
*//*AM2PDAM*/			  
		
	      }
	
	  }/*4blocks*/
      }/*INTRA MB*/
#if 0
      else        /* inter block encoding */
      {
        for (i = 0; i < 4; i++)
          {
 
           if (transp_pattern[i]!=1) { /* Not transparent */
              /* Do the zigzag scanning of coefficients */
              for (m = 0; m < 64; m++)
                  *(coeff + zigzag[m]) = qcoeff[i*ncoeffs+m];
 
              if ((i==0 && CBPA&8) ||
                  (i==1 && CBPA&4) ||
                  (i==2 && CBPA&2) ||
                  (i==3 && CBPA&1))
                {
                  if (!reverse_vlc)
                        bits->G += CodeCoeff(0,Mode, coeff, i, ncoeffs, bitstream);
                  else
                        bits->G += CodeCoeff_RVLC(0,Mode, coeff, i, ncoeffs,
                                                bitstream);
         
                }
            }
         }
      }/*INTER MB*/
#endif 0

}        


/*AM2PDAM*/

/*v001215*/
Void CodeAlphaBlockData_DCT_StudioFDAM (Bits *bits,
					Int transp_pattern[], Int alternate_scan,
					Int *qcoeff, Int Mode, Int CBPA, Int ncoeffs,
					Image *bitstream, Int bits_per_pixel,
					Int flc_length/*v001225*/)
{
    Int i, m, coeff[64];
    Int *zz = alternate_scan ? zigzag_v : zigzag;
    Int tmp_coef;

    Int j, k;    /*WD2.1_2*/
    
    if (Mode == MODE_INTRA || Mode == MODE_INTRA_Q)
      {  
	for (i = 0; i < 4; i++)
	  {  
	    if (transp_pattern[i]!=1) /* Not transparent */
	      {
		  
		/* Do the zigzag scanning of coefficients */
		for (m = 0; m < 64; m++)
		  *(coeff + zz[m]) = qcoeff[i*ncoeffs+m];

		tmp_coef = IntraDC_dpcm_Studio_bySVLC(coeff[0],1,bitstream);

		bits->G += tmp_coef;
		bits->Gdc += tmp_coef;

		/* Code AC coeffs. dep. on block pattern MW 15-NOV-1996 */
/*
                if ((i==0 && CBPA&8) ||
		    (i==1 && CBPA&4) ||
		    (i==2 && CBPA&2)  ||
		    (i==3 && CBPA&1))
		  {
*//*AM2PDAM*/		  
		    tmp_coef = CodeCoeff_Studio_bySVLC2(1,Mode, coeff,i,ncoeffs,bitstream,flc_length/*v001225*/);
		    bits->G += tmp_coef;
		    bits->Gac += tmp_coef;
/*		
		  }
*//*AM2PDAM*/			  
		
	      }
	
	  }/*4blocks*/
      }/*INTRA MB*/
      else        /* inter block encoding */
      {
        for (i = 0; i < 4; i++)
          {
 
           if (transp_pattern[i]!=1) { /* Not transparent */
	     
              /* Do the zigzag scanning of coefficients */
              for (m = 0; m < 64; m++)
                  *(coeff + zz[m]) = qcoeff[i*ncoeffs+m];
 
              if ((i==0 && CBPA&8) ||
                  (i==1 && CBPA&4) ||
                  (i==2 && CBPA&2) ||
                  (i==3 && CBPA&1))
                {
		  tmp_coef = CodeCoeff_Studio_bySVLC2(0,Mode, coeff,i,ncoeffs,bitstream,flc_length/*v001225*/);
		  bits->G += tmp_coef;
		  bits->Gac += tmp_coef;
                }
            }
         }
      }/*INTER MB*/

}        

/* 01.04.01: Studio Profile */
