/*****************************************************************************
 "This software module was originally developed by:

	Noboru Yamaguchi (TOSHIBA CORPORATION), 
	Takashi Ida (TOSHIBA CORPORATION) 

	and edited by:

 	Toshiaki Watanabe (TOSHIBA CORPORATION), 
	Yoshihiro Kikuchi (TOSHIBA CORPORATION), 
	Noel Brady (TELTEC IRELAND),
        Minhua Zhou (HHI Berlin),
	Dae-Sung Cho (Samsung AIT)
	Takefumi Nagumo (SONY Corporation)
	Osamu Sunohara (SONY)

	in the course of development of the <MPEG-4 Video(ISO/IEC 14496-2)>. This
  software module is an implementation of a part of one or more <MPEG-4 Video
  (ISO/IEC 14496-2)> tools as specified by the <MPEG-4 Video(ISO/IEC 14496-2)
  >. ISO/IEC gives users of the <MPEG-4 Video(ISO/IEC 14496-2)> free license
  to this software module or modifications thereof for use in hardware or
  software products claiming conformance to the <MPEG-4 Video(ISO/IEC 14496-2
  )>. Those intending to use this software module in hardware or software
  products are advised that its use may infringe existing patents. The
  original developer of this software module and his/her company, the
  subsequent editors and their companies, and ISO/IEC have no liability for
  use of this software module or modifications thereof in an implementation.
  Copyright is not released for non <MPEG-4 Video(ISO/IEC 14496-2)>
  conforming products. TOSHIBA CORPORATION retains full right to use the code
  for his/her own purpose, assign or donate the code to a third party and to
  inhibit third parties from using the code for non <MPEG-4 Video(ISO/IEC
  14496-2)> conforming products. This copyright notice must be included in
  all copies or derivative works. Copyright (c)1996".
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	alp_code_header.c
 *
 * Author:	TOSHIBA + Noel Brady Teltec Irl.
 * Created:	11-04-97
 *                                                                         
 * Description: Contains functions for encoding the first_shape_code in 
 *							I/P/B Vops
 *
 *                                 
 ***********************************************************HeaderEnd*********/


#include<stdio.h>
#ifndef WIN32
#include<values.h>
#endif
#include <string.h>
#include "momusys.h"
#include "alp_common_def.h"
#include "alp_mode_tables.h"
#include "vm_common_defs.h"
#include "alp_common_util.h"
#include "alp_code_header.h"
/*OBSSFIX_MODE3-start-*/
#include "vm_enc_defs.h"
/*OBSSFIX_MODE3-end-*/

#define _ENCODE_DEBUG_FIRST
#define _MB_TYPE_DEBUG

/* 01.04.01: Studio Profile */
#include "vm_enc_defs.h"

/* ######### VM7 Shape Mode Encoding (I-VOP) ########## */

Int CodeIntraMBtype( UChar *stream, Int mb_type, Int i,
	Int j, Vop *vop)
{
  Int 	context;
  Int  total, max_stream;
  UChar tmp[6];
  Int		a=3,b=3,c=3,d=3;
  Int last_col = GetVopWidth(vop)/MB_SIZE - 1;
  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode(vop));
  Int  hsize = GetImageSizeX(GetVopShapeMode(vop));

  max_stream = MAX_STREAM;
 
  total = 0;
	if (i>0)
		{
			if (j>0) a = cur_shape_mode[(i-1)*hsize+j-1]+1;
			b = cur_shape_mode[(i-1)*hsize+j]+1;
			if (j<last_col)	c = cur_shape_mode[(i-1)*hsize+j+1]+1;
		}
	if (j>0) d = cur_shape_mode[i*hsize+j-1]+1;

	context = 27*(a-3) +	9*(b-3) +	3*(c-3) + d-3;

 	strcpy(tmp, IMB_TYPE[context][cur_shape_mode[i*hsize+j]-2]);
  StreamOut(stream, &total, &max_stream, tmp);
  return total;
}


Int CodeIntraMBtype_ER( UChar *stream, Int mb_type, Int i,
	Int j, Vop *vop, Image *alpha_mode_packet)
{
  Int 	context;
  Int  total, max_stream;
  UChar tmp[6];
  Int		a=3,b=3,c=3,d=3;
	Int width = GetImageSizeX(alpha_mode_packet);
  Int last_col = GetVopWidth(vop)/MB_SIZE - 1;
	SInt *p = (SInt *) GetImageData(alpha_mode_packet);
  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode(vop));
  Int  hsize = GetImageSizeX(GetVopShapeMode(vop));
  max_stream = MAX_STREAM;

  total = 0;
	if (i>0)
		{
			if (j>0 && p[(i-1)*width+j-1]) a = cur_shape_mode[(i-1)*hsize+j-1]+1;
			if (p[(i-1)*width+j])  b = cur_shape_mode[(i-1)*hsize+j]+1;
			if (j<last_col && p[(i-1)*width+j+1])	c = cur_shape_mode[(i-1)*hsize+j+1]+1;
		}
	if (j>0 && p[i*width+j-1]) d = cur_shape_mode[i*hsize+j-1]+1;

	p[i*width+j] = 1;

	context = 27*(a-3) +	9*(b-3) +	3*(c-3) + d-3;

 	strcpy(tmp, IMB_TYPE[context][cur_shape_mode[i*hsize+j]-2]);
  StreamOut(stream, &total, &max_stream, tmp);
  return total;
}

/* ######### VM7 Shape Mode Encoding (INTER) ########## */

Int
CodeInterMBtype(
				Int i,
				Int j,
				UChar *stream,
                                Vop *vop,
                                Vop *ref_vop
		   )
{
  UChar
    tmp[9];

  Int
    max_stream=7,
    prei,prej,
    bits=0,
    curr_mode,
    prev_mode;
  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode(vop));
  Int  hsize = GetImageSizeX(GetVopShapeMode(vop));
  SInt *ref_shape_mode = (SInt *)GetImageData(GetVopShapeMode(ref_vop));
  Int  hsize_ref = GetImageSizeX(GetVopShapeMode(ref_vop));
  Int  vsize_ref = GetImageSizeY(GetVopShapeMode(ref_vop));


  if(i<vsize_ref) prei=i;   else prei=vsize_ref-1;

  if(j<hsize_ref) prej=j;  else prej=hsize_ref-1;

/*OBSSFIX_MODE3-start-*/
  if(GetVopScalability(vop)==1 && GetVopHierarchyType(vop)==0 &&
     GetVopEnhanceType(vop)==1 && GetVopUseRefShape(vop)==1   &&
     GetVopPredictionType (vop)==P_VOP){
    prev_mode = 3; /* 3 means all opaque*/
  } else{
    prev_mode = ref_shape_mode[prei*hsize_ref+prej];
  }
/*	prev_mode = ref_shape_mode[prei*hsize_ref+prej];*/
/*OBSSFIX_MODE3-end-*/  

	curr_mode = cur_shape_mode[i*hsize+j];

	switch (prev_mode)
		{
			case 0: strcpy( tmp, MMB_TYPE0[curr_mode]);	
							break;
			case 1: strcpy( tmp, MMB_TYPE1[curr_mode]);	
							break;
			case 2: strcpy( tmp, MMB_TYPE2[curr_mode]);	
							break;
			case 3: strcpy( tmp, MMB_TYPE3[curr_mode]);	
							break;
			case 4: strcpy( tmp, MMB_TYPE4[curr_mode]);	
							break;
			case 5: strcpy( tmp, MMB_TYPE5[curr_mode]);	
							break;
			case 6: strcpy( tmp, MMB_TYPE6[curr_mode]);	
							break;
			default: printf("Error in PRE_MB_TYPE\n");
							 exit(-1);
	}

  StreamOut(stream, &bits, &max_stream, tmp);

  return bits;
}

/* begin: added for OBSS by Samsung AIT (1999-09-29) */

/* ######### Shape Mode Encoding of P-VOP In Enhanced Layer ########## */
Int 
CodeIntraMBtypeEnh(Int i, Int j, 
		UChar *stream, 
		Vop *vop)
{
  Char tmp[9];
 
  Int
    max_stream,
    bits=0,
    curr_mode;

  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode(vop));
  Int  hsize = GetImageSizeX(GetVopShapeMode(vop));
 
  max_stream = MAX_STREAM;

  curr_mode = cur_shape_mode[i*hsize+j];

#ifdef  _DEBUG_OBSS_
  fprintf(stderr,">>>CodeIntraMBtypeEnh()==>\n");
  fprintf(stderr,">>>>>curr_mode[%d,%d]=%d\n",i,j,curr_mode);
  /*  getchar();  */
#endif
        
  if(curr_mode > 0 && curr_mode <= 2)
  {
  	strcpy( tmp, IMB_ENH_TYPE[curr_mode-1]);
  } else
  {
        printf("Error in P-VOP mode coding in the enhancement layer\n");      
	exit(-1);
  }
 
  StreamOut(stream, &bits, &max_stream, tmp);
 
  return bits;
}

/* ######### Shape Mode Encoding of B-VOP In Enhanced Layer ########## */
Int 
CodeInterMBtypeEnh(Int i, Int j, 
		UChar *stream, 
		Vop *vop,
		Vop *ref_vop)  
{
  UChar tmp[9];
 
  Int
    max_stream,
    ref_i,ref_j,
    bits=0,
    curr_mode,
    ref_mode;

  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode(vop));
  Int  hsize = GetImageSizeX(GetVopShapeMode(vop));
  SInt *ref_shape_mode = (SInt *)GetImageData(GetVopShapeMode(ref_vop));
  Int  hsize_ref = GetImageSizeX(GetVopShapeMode(ref_vop));
  Int  vsize_ref = GetImageSizeY(GetVopShapeMode(ref_vop));
 
  max_stream = MAX_STREAM;

  if(i<vsize_ref) ref_i=i;   
  else		  ref_i=vsize_ref-1;
  if(j<hsize_ref) ref_j=j;   
  else		  ref_j=hsize_ref-1;
 
  ref_mode = ref_shape_mode[ref_i*hsize_ref+ref_j];
  curr_mode = cur_shape_mode[i*hsize+j];

#ifdef  _DEBUG_OBSS_
  fprintf(stderr,">>>CodeInterMBtypeEnh()==>\n");
  fprintf(stderr,">>>>>ref_mode[%d,%d]=%d, curr_mode=%d\n", ref_i,ref_j,ref_mode,curr_mode);
  /*  getchar();  */
#endif
 
  switch (ref_mode)
  {
        case 0: strcpy( tmp, MMB_ENH_TYPE3[curr_mode-1]);       break;
        case 1: strcpy( tmp, MMB_ENH_TYPE3[curr_mode-1]);       break;
        case 2: strcpy( tmp, MMB_ENH_TYPE1[curr_mode-1]);       break;
        case 3: strcpy( tmp, MMB_ENH_TYPE1[curr_mode-1]);       break;
        case 4: strcpy( tmp, MMB_ENH_TYPE2[curr_mode-1]);       break;
        case 5: strcpy( tmp, MMB_ENH_TYPE4[curr_mode-1]);       break;
        case 6: strcpy( tmp, MMB_ENH_TYPE4[curr_mode-1]);       break;
        default: printf("Error in PRE_MB_TYPE\n");      exit(-1);
  }
 
  StreamOut(stream, &bits, &max_stream, tmp);
 
  return bits;
}
/* end: added for OBSS by Samsung AIT (1999-09-29) */

void
MakeMBtype(
	   Int mb_type,
	   Int mvda,
	   Int i,
	   Int j,
           Vop *vop
	   )
{

  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode(vop));
  Int  hsize = GetImageSizeX(GetVopShapeMode(vop));


  if(mb_type==-1) 
		{
    	if(mvda==1) {
      	/* MVDs==0 */
      	cur_shape_mode[i*hsize+j]=0;
    	}else {
      	/* MVDs!=0 */
      	cur_shape_mode[i*hsize+j]=1;
    	}
		}
	else if(mb_type==0){
    /* all255 */
    cur_shape_mode[i*hsize+j]=3;
  }else if(mb_type==1){
    /* all0  */
    cur_shape_mode[i*hsize+j]=2;
  }else if(mb_type>=2 && mb_type<=6){
    /* MMR */
    if (mb_type<=4) cur_shape_mode[i*hsize+j]=4;
		else 
			{
				if (mvda==1) cur_shape_mode[i*hsize+j]=5;
				else cur_shape_mode[i*hsize+j]=6;
			}
  }
	else{
    printf("wrong mb_type=%d: i j=%d %d\n",mb_type,i,j);
    exit(0);
  }

}


/* begin: added for OBSS by Samsung AIT (1999-09-29) */
Void 
MakeMBtypeEnh(Int mb_type, 
		Int i, 
		Int j, 
		Vop *vop)
{
  /*
    MB_TYPE:
    1: intra_not_coded
    2: intra_coded
    3: inter_not_coded
    4: inter_coded
  */
  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode(vop));
  Int  hsize = GetImageSizeX(GetVopShapeMode(vop));

  if(mb_type==-1)
  {
      	cur_shape_mode[i*hsize+j]=3; /* inter not coded */
  }
  else if(mb_type==0 || mb_type==1)
  {
        /* all255 or all0 or intra_not_coded */
      	cur_shape_mode[i*hsize+j]=1; /* intra not coded */
  }
  else if(mb_type>=2 && mb_type<=6)
  {
        if (mb_type<=4) cur_shape_mode[i*hsize+j]=2; /* intra coded */
        else            cur_shape_mode[i*hsize+j]=4; /* inter coded */

  }
  else
  {
        printf("wrong mb_type=%d: i j=%d %d\n",mb_type,i,j);
        exit(0);
  }
#ifdef  _DEBUG_OBSS
  fprintf(stderr,">>>MakeMBtypeEnh()==>!\n");
  fprintf(stderr,">>>>>cur_shape_mode[%d][%d] <- %d (1-intra_not 2-intra 3-inter-not 4-inter)\n", 
		i,j,cur_shape_mode[i*hsize+j]);
/*  getchar();  */
#endif

}
/* end: added for OBSS by Samsung AIT (1999-09-29) */

Int GetModeCodeLength(
		      Int mode,
		      Int j,
		      Int i,
                      Vop *ref_vop
		   )
{
  Int
    prei,prej,
		curr_mode=4,
		prev_mode;
  SInt *ref_shape_mode = (SInt *)GetImageData(GetVopShapeMode(ref_vop));
  Int  hsize_ref = GetImageSizeX(GetVopShapeMode(ref_vop));
  Int  vsize_ref = GetImageSizeY(GetVopShapeMode(ref_vop));


 if(i<vsize_ref) prei=i;   else prei=vsize_ref-1;

  if(j<hsize_ref) prej=j;  else prej=hsize_ref-1;

	prev_mode = ref_shape_mode[prei*hsize_ref+prej];
	
	if (mode==2) curr_mode = 4;
	if (mode==5) curr_mode = 6;
	if (mode==6) curr_mode = 5;

	switch (prev_mode)
		{
			case 0: return strlen(MMB_TYPE0[curr_mode]);	
							break;
			case 1: return strlen(MMB_TYPE1[curr_mode]);	
							break;
			case 2: return strlen(MMB_TYPE2[curr_mode]);	
							break;
			case 3: return strlen(MMB_TYPE3[curr_mode]);	
							break;
			case 4: return strlen(MMB_TYPE4[curr_mode]);	
							break;
			case 5: return strlen(MMB_TYPE5[curr_mode]);	
							break;
			case 6: return strlen(MMB_TYPE6[curr_mode]);	
							break;
			default: printf("Error in PRE_MB_TYPE\n");
							 exit(-1);
	}
}


/* begin: added for OBSS by Samsung AIT (1999-09-29) */
Int GetModeCodeLength_Enh(
		      Int mode,
		      Int j,
		      Int i,
                      Vop *ref_vop
		   )
{
  Int ref_i,ref_j, curr_mode, ref_mode;
  SInt *ref_shape_mode = (SInt *)GetImageData(GetVopShapeMode(ref_vop));
  Int hsize_ref = GetImageSizeX(GetVopShapeMode(ref_vop));
  Int vsize_ref = GetImageSizeY(GetVopShapeMode(ref_vop));

  if(i<vsize_ref) ref_i=i;   
  else		  ref_i=vsize_ref-1;
  if(j<hsize_ref) ref_j=j;   
  else		  ref_j=hsize_ref-1;

  ref_mode = ref_shape_mode[ref_i*hsize_ref+ref_j];
 
  if(mode==-1){
	curr_mode = 3; 			/* inter_not_coded */
  } 
  else if(mode==0 || mode==1) {
	curr_mode = 1;			/* intra_not_coded */
  } 
  else if(mode>=2 && mode<=6) {
        if (mode<=4)	curr_mode = 2;	/* intra_coded */
	else		curr_mode = 4;	/* inter_coded */
  } 
  else {
        printf("wrong mode=%d: i j=%d %d\n",mode,i,j);
        exit(0);
  }
 
  switch (ref_mode)
  {
        case 0: return strlen( MMB_ENH_TYPE3[curr_mode-1]);       break;
        case 1: return strlen( MMB_ENH_TYPE3[curr_mode-1]);       break;
        case 2: return strlen( MMB_ENH_TYPE1[curr_mode-1]);       break;
        case 3: return strlen( MMB_ENH_TYPE1[curr_mode-1]);       break;
        case 4: return strlen( MMB_ENH_TYPE2[curr_mode-1]);       break;
        case 5: return strlen( MMB_ENH_TYPE4[curr_mode-1]);       break;
        case 6: return strlen( MMB_ENH_TYPE4[curr_mode-1]);       break;
        default: printf("Error in PRE_MB_TYPE\n");      exit(-1);
  }

}
/* end: added for OBSS by Samsung AIT (1999-09-29) */

/* 01.04.01: Studio Profile */
Int CodeIntraMBtype_IntraReset_Studio(UChar *stream, Int mb_type, Int i,
				      Int j, Vop *vop, Int vop_structure, Int field_parity,
				      Int reset)
{
  Int 	context;
  Int  total, max_stream;
  Char tmp[6];
  Int		a=3,b=3,c=3,d=3;
  Int last_col = GetVopWidth_st(vop,field_parity)/MB_SIZE - 1;/*STP*/
  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode_st(vop,field_parity));
  Int  hsize = GetImageSizeX(GetVopShapeMode_st(vop,field_parity));

  max_stream = MAX_STREAM;
 
  total = 0;
  if (j>0 && !reset) d = cur_shape_mode[i*hsize+j-1]+1;
  
	context = 27*(a-3) +	9*(b-3) +	3*(c-3) + d-3;

 	strcpy(tmp, IMB_TYPE[context][cur_shape_mode[i*hsize+j]-2]);
  StreamOut(stream, &total, &max_stream, tmp);
  return total;
}

Int
CodeInterMBtype_HHC_Studio(Int i,
			   Int j,
			   UChar *stream,
			   Vop *vop,
			   Vop *ref_vop)
{
  Int vopstr = GetVopVopStructure_st(vop);
  Int fip = CalcFieldParity_Studio(vopstr,
				   GetVopVopCounter_st(vop),
				   GetVopTopFieldFirst(vop));
  Char
    tmp[9];

  Int
    max_stream=7,
    prei,prej,
    bits=0,
    curr_mode,
    prev_mode;
  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode_st(vop,fip));
  Int  hsize = GetImageSizeX(GetVopShapeMode_st(vop,fip));
  SInt *ref_shape_mode;
  Int  hsize_ref;
  Int  vsize_ref;

  if(GetVopVopCounter_st(vop)==1&&GetVopPredictionType_st(vop,!fip)==I_VOP) {
    ref_shape_mode = (SInt *)GetImageData(GetVopShapeMode_st(vop,!fip));
    hsize_ref = GetImageSizeX(GetVopShapeMode_st(vop,!fip));
    vsize_ref = GetImageSizeY(GetVopShapeMode_st(vop,!fip));
  } else {
    ref_shape_mode = (SInt *)GetImageData(GetVopShapeMode_st(ref_vop,fip));
    hsize_ref = GetImageSizeX(GetVopShapeMode_st(ref_vop,fip));
    vsize_ref = GetImageSizeY(GetVopShapeMode_st(ref_vop,fip));
  }    
  
  
  if(i<vsize_ref) prei=i;   else prei=vsize_ref-1;

  if(j<hsize_ref) prej=j;  else prej=hsize_ref-1;
  
  prev_mode = ref_shape_mode[prei*hsize_ref+prej];
  
  curr_mode = cur_shape_mode[i*hsize+j];

  
	switch (prev_mode)
		{
			case 0: strcpy( tmp, MMB_TYPE0[curr_mode]);	
							break;
			case 1: strcpy( tmp, MMB_TYPE1[curr_mode]);	
							break;
			case 2: strcpy( tmp, MMB_TYPE2[curr_mode]);	
							break;
			case 3: strcpy( tmp, MMB_TYPE3[curr_mode]);	
							break;
			case 4: strcpy( tmp, MMB_TYPE4[curr_mode]);	
							break;
			case 5: strcpy( tmp, MMB_TYPE5[curr_mode]);	
							break;
			case 6: strcpy( tmp, MMB_TYPE6[curr_mode]);	
							break;
			default: printf("Error in PRE_MB_TYPE\n");
							 exit(-1);
	}

  StreamOut(stream, &bits, &max_stream, tmp);

  return bits;
}

void
MakeMBtype_HHC_Studio(
		      Int mb_type,
		      Int mvda,
		      Int i,
		      Int j,
		      Vop *vop,
		      Int vop_structure,
		      Int field_parity
		      )
{
  SInt *cur_shape_mode = (SInt *)GetImageData(GetVopShapeMode_st(vop,field_parity));
  Int  hsize = GetImageSizeX(GetVopShapeMode_st(vop,field_parity));

  if(mb_type==-1) 
    {
      if(mvda==1) {
      	/* MVDs==0 */
      	cur_shape_mode[i*hsize+j]=0;
      }else {
      	/* MVDs!=0 */
      	cur_shape_mode[i*hsize+j]=1;
      }
  } else if(mb_type==0){
    /* all255 */
    cur_shape_mode[i*hsize+j]=3;
  }else if(mb_type==1){
    /* all0  */
    cur_shape_mode[i*hsize+j]=2;
  }else if(mb_type==2){
    cur_shape_mode[i*hsize+j]=4;
  } else{
    printf("wrong mb_type=%d: i j=%d %d\n",mb_type,i,j);
    exit(0);
  }
  
}
/* 01.04.01: Studio Profile */




