/*****************************************************************************
 "This software module was originally developed by:
	
	Noboru Yamaguchi (TOSHIBA CORPORATION), Takashi Ida (TOSHIBA CORPORATION) 

	and edited by:

	 	Toshiaki Watanabe (TOSHIBA CORPORATION), 
		Yoshihiro Kikuchi (TOSHIBA CORPORATION), 
		Noel Brady (TELTEC IRELAND),
		Dae-Sung Cho (Samsung AIT)
		Osamu Sunohara (SONY)
	
	in the course of development of the <MPEG-4 Video(ISO/IEC 14496-2)>. This
  software module is an implementation of a part of one or more <MPEG-4 Video
  (ISO/IEC 14496-2)> tools as specified by the <MPEG-4 Video(ISO/IEC 14496-2)
  >. ISO/IEC gives users of the <MPEG-4 Video(ISO/IEC 14496-2)> free license
  to this software module or modifications thereof for use in hardware or
  software products claiming conformance to the <MPEG-4 Video(ISO/IEC 14496-2
  )>. Those intending to use this software module in hardware or software
  products are advised that its use may infringe existing patents. The
  original developer of this software module and his/her company, the
  subsequent editors and their companies, and ISO/IEC have no liability for
  use of this software module or modifications thereof in an implementation.
  Copyright is not released for non <MPEG-4 Video(ISO/IEC 14496-2)>
  conforming products. TOSHIBA CORPORATION retains full right to use the code
  for his/her own purpose, assign or donate the code to a third party and to
  inhibit third parties from using the code for non <MPEG-4 Video(ISO/IEC
  14496-2)> conforming products. This copyright notice must be included in
  all copies or derivative works. Copyright (c)1996".
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	alp_code_mom.c
 *
 * Author:	TOSHIBA + Noel Brady Teltec Irl. + Dae-Sung Cho (Samsung AIT)
 * Created:	11-04-97
 *                                                                         
 * Description: Contains high level functions implementing binary shape coding
 *							as per VM7
 *
 * Modified:
 *
 *    16.06.97 Angel Pacheco: unified the TRANSPARENT modes.
 *		06.08.97 Noel Brady: Added mods for BINARY_SHAPE_ONLY
 *       09.29.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 *       991110   U. Benzler, University of Hannover - correct the placement of FreeImage(bordered_smb_data)
 *       01.04.01 Osamu Sunohara (SONY): Functions for Studio Profile
 *                                 
 ***********************************************************HeaderEnd*********/


/********* header ********/

#include<stdio.h>
#ifndef WIN32
#include<values.h>
#endif
#include <string.h>
#include "momusys.h"
#include "mom_structs.h"
#include "mom_access.h"
#include "vm_enc_defs.h"
#include "vm_config.h"					/* BSO_NOEL */
#include "alp_common_def.h"
#include "mot_est.h"
#include "alp_common_cae.h"
#include "alp_common_si.h"	/* added for OBSS by Samsung AIT (1999-09-29) */
#include "alp_common_util.h"
#include "alp_code_mom.h"
#include "alp_code_header.h"
#include "alp_code_decisions.h"
#include "alp_code_cae.h"
#include "alp_code_si.h"	/* added for OBSS by Samsung AIT (1999-09-29) */
#include "alp_code_mc.h"

/* 01.04.01: Studio Profile */
#include "alp_common_shape_vlc.h"

#ifdef  _DEBUG_OBSS_
#include "alp_common_mc.h"
#endif

extern Int 	total_first_bits[3],
						total_second_bits[3],
						total_amv_bits[3],
						total_cae_bits[3],
						total_cae_intra_bits[3],
						total_cae_inter_bits[3],
						total_coded_mbs[3],
						total_coded_intra_mbs[3],
						total_coded_inter_mbs[3],
						curr_first_bits,
						curr_second_bits,
						curr_amv_bits,
						curr_cae_bits,
						curr_cae_intra_bits,
						curr_cae_inter_bits,
						curr_coded_mbs,
						curr_coded_intra_mbs,
						curr_coded_inter_mbs;

/* 01.04.01: Studio Profile */
extern Int total_simple_bits[3], curr_simple_bits;

Image *alpha_rec_packet = NULL,
      *alpha_mode_packet = NULL;

Int   MBnum=0; 			/* bug fix by Noboru Yamaguchi, 16.09.98 SB */

/***********************************************************CommentBegin******
 *
 * -- VopShapeCode -- Produces all the bitstream elements for all
 *											BABs in a given Vop and produces the reconstructed
 *											alpha map..
 *
 * Author :		
 *	Noel Brady Teltec Irl.
 *
 * Created :		
 *	11-04-97
 * 
 * Arguments In: 	
 *					curr - the input vop containing the binary
 *									alpha map
 *					rec_prev - the previously reconstructed VOP
 *					x,y - the top-left pixel coordinate of the BAB
 *					thresh - alpha threshold
 *					mot_x,mot_y - motion vector field used for texture MC
 *					MB_decisions - the coding modes for all texture MBs
 *
 * Arguments In/Out:
 *
 * Arguments Out:
 *					rec_curr - the reconstructed VOP with coded alpha map.
 *					alpha_decisions - the blockwise transparancy decisons 
 *					First_stream - contains all first_shape_codes
 *					Shape_stream - contains all the rest of the shape related codes
 *
 * Return values :
 *					the number of bits generated.
 *
 * Side effects :	
 *	-
 *
 * Description : As per VM7
 *
 * See also :
 * Modified: 
 *     06.08.97 Minhua Zhou: set the motion vectors to zero 
 *                           for MBs changing from non-transparent
 *                           to transparent after shape coding. This setting
 *                           is essential
 *                           for B-VOPs which utilize vectors of I-, P-VOPs
 *                           during coding
 *		19.11.97 Noel Brady: ShapeCodeMB computes the reconstructed values for
 *													the Auv blocks also.
 *		19.11.97 Noel Brady: conditions fro use of ModifyModesandMVs() is changed
 *      09.29.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 *      991110   U. Benzler, University of Hannover - correct the placement of FreeImage(bordered_smb_data) 
 *
 ***********************************************************CommentEnd********/
Int
VopShapeCode(Vop *curr,
						Vop *rec_curr,
						Vop *rec_prev,
						Vop *rec_up,    /* added for OBSS by Samsung AIT (1999-09-29) */
						Int vo_id, 
						Int thresh,
						Image *mot_x,
						Image *mot_y,
						Image *MB_decisions,
						Image *alpha_decisions,
						Image **First_stream,
						Image **Shape_stream
	     )
{
  Image *modeA; /* the flag of current alpha block is inter coded or not */
  Image *motA_x; /* the x compornent of shape MV */
  Image *motA_y; /* the y compornent of shape MV */
  Int first_bits, shape_bits, total_shape_bits = 0;
  Int i, j, k;
  UChar first_stream[7], shape_stream[4096];

#ifdef  _DEBUG_OBSS_
  Int x, y;
#endif

  /* Allocate shape ME/MC */
  modeA = AllocImage(GetVopWidth(curr)/MB_SIZE,
		     GetVopHeight(curr)/MB_SIZE,SHORT_TYPE);
  motA_x = AllocImage(GetVopWidth(curr)/MB_SIZE,
		      GetVopHeight(curr)/MB_SIZE,SHORT_TYPE);
  motA_y = AllocImage(GetVopWidth(curr)/MB_SIZE,
		      GetVopHeight(curr)/MB_SIZE,SHORT_TYPE);

#ifdef	_DEBUG_OBSS_
/*if (GetVopScalability(curr) && 
        (!GetVopScalability(curr)||
          (GetVopPredictionType(curr)==P_VOP&&GetVopRefSelCode(curr)==3)||
          (GetVopPredictionType(curr)==B_VOP&&GetVopRefSelCode(curr)==0)) &&
	(!GetVopScalability(curr)||
	   (!GetVopEnhanceType(curr))||(GetVopArbitraryShape(curr)==BINARY_SHAPE_ONLY))) */
if (GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr)))
{
   fprintf(stderr,">>>VopShapeCode()==>!\n");
   fprintf(stderr,">>>>>>Shape mode in reference vop!\n");
   for(i=0; i<GetVopHeight(curr)/MB_SIZE; i++) {
     for(j=0; j<GetVopWidth(curr)/MB_SIZE; j++) {
	fprintf(stderr,"%d ", GetBabType(i,j,rec_up));
     }  fprintf(stderr,"\n");
   }
   fprintf(stderr,">>>>>>Alpha motion in reference vop!\n");
   for(i=0; i<GetVopHeight(curr)/MB_SIZE; i++) {
     for(j=0; j<GetVopWidth(curr)/MB_SIZE; j++) {
	GetBabMV(j,i,&x,&y,rec_up);
	fprintf(stderr,"(%d,%d)",x,y);
     }  fprintf(stderr,"\n");
   }
   fprintf(stderr,">>>>>>Alpha motion (only INTER MODE) in reference vop!\n");
   for(i=0; i<GetVopHeight(curr)/MB_SIZE; i++) {
     for(j=0; j<GetVopWidth(curr)/MB_SIZE; j++) {
        k = GetBabType(i,j,rec_up);
	if(k==0 || k==1 || k==5 || k==6){
		GetBabMV(j,i,&x,&y,rec_up);
		fprintf(stderr,"(%d,%d)",x,y);
	} else {
        	fprintf(stderr,"(-,-)");
	}
     }  fprintf(stderr,"\n");
   }
   getchar();
}
#endif

  for(i=0; i<GetVopHeight(curr)/MB_SIZE; i++) {
    for(j=0; j<GetVopWidth(curr)/MB_SIZE; j++) {
      k = i*(GetVopWidth(curr)/MB_SIZE)+j;

#ifdef  _DEBUG_OBSS_
      if (GetVopScalability(curr) && GetVopHierarchyType(curr)==0 &&
	(GetVopEnhanceType(curr)==0 || GetVopUseRefShape(curr)==0)) 
        fprintf(stderr,">>>>>BAB[%d,%d]>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n", i,j);
#endif

      shape_bits = ShapeCodeMB(curr,
								rec_curr,
								rec_prev,
								rec_up, /* added for OBSS by Samsung AIT (1999-09-29) */
								vo_id, 
								i,
								j,
								thresh,
								modeA,
								mot_x,
								mot_y,
								motA_x,
								motA_y,
								MB_decisions,
								alpha_decisions,
								&first_bits,
								first_stream,
								shape_stream,
								0
			       );
      BitstreamAppendTtoM(First_stream[k], first_stream, first_bits);
      BitstreamAppendTtoM(Shape_stream[k], shape_stream, shape_bits);
			total_shape_bits += shape_bits + first_bits;
#ifdef  _DEBUG_OBSS_
  if (GetVopScalability(curr) && GetVopHierarchyType(curr)== 0 &&
	(GetVopEnhanceType(curr)==0 || GetVopUseRefShape(curr)==0))
  	getchar();
#endif

    }
  }
				
  /* Free shape ME/MC */
  FreeImage(modeA);
  FreeImage(motA_x);
  FreeImage(motA_y);
	return total_shape_bits;
}

Int
ShapeCodeMB(Vop *curr,
	    Vop *rec_curr,
	    Vop *rec_prev,
	    Vop *rec_up, /* added for OBSS by Samsung AIT (1999-09-29) */
			Int vo_id, 
	    Int adr_i,
	    Int adr_j,
	    Int thresh,
	    Image *modeA,
	    Image *mot_x,
	    Image *mot_y,
	    Image *motA_x,
	    Image *motA_y,
	    Image *MB_decisions,
			Image *alpha_decisions,
	    Int *first_bits,
	    UChar *first_stream,
	    UChar *shape_stream,
			Int start_of_packet
	    )
{
  /* SAIT begin */
  Image
    *tmp,
    *bordered_smb_data,
    *top_border_data = AllocImage(MB_SIZE+4,2,SHORT_TYPE),
    *left_border_data = AllocImage(2,MB_SIZE,SHORT_TYPE),
    *bottom_border_data = AllocImage(MB_SIZE+4,2,SHORT_TYPE),	/* added for OBSS by Samsung AIT (1999-09-29) */
    *right_border_data = AllocImage(2,MB_SIZE,SHORT_TYPE);	/* added for OBSS by Samsung AIT (1999-09-29) */
 
  SInt *p;
  Int   j;
  /* SAIT end */


  Int vol_id = GetVopId(curr);

  Image 
    *mblock,
    *mblock_rec,
			*mblock_up,     /* added for OBSS by Samsung AIT (1999-09-29) */
    			*mblock_spos,	/* added for OBSS by Samsung AIT (1999-09-29) */
		*block_rec,
		*rec_prev_alpha,
		*rec_alpha;

  SInt	pred[MB_SIZE][MB_SIZE],
				*a_mb,
				*a_mb_rec,
					*a_mb_up,       /* added for OBSS by Samsung AIT (1999-09-29) */
    					*a_mb_spos,	/* added for OBSS by Samsung AIT (1999-09-29) */
				*a_b_rec;

  Int
		error_res_disable,
    width=GetVopWidth(curr),
    second_bits=0,
    cae_bits=0,
    amvbits=0,
    mb_bits=0,
    mvda,
    shape_inter,
    mb_type,
    i,
    x,y,
    h,v;

  Int flag,
			send_cr;

  UChar
    alpha_mb[MB_SIZE*MB_SIZE],
    alpha_mb_comp[MB_SIZE*MB_SIZE],
	alpha_mb_up[MB_SIZE*MB_SIZE],   /* added for OBSS by Samsung AIT (1999-09-29) */
		alpha_mb_spos[MB_SIZE*MB_SIZE],	/* added for OBSS by Samsung AIT (1999-09-29) */
    alpha_smb[MB_SIZE*MB_SIZE],
    first_shape_code[7],
    second_code[2],
    cae_stream[4096],
    mv_stream[32];

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
    UChar  alpha_mb_spos_h[MB_SIZE], 
		alpha_mb_spos_v[MB_SIZE];
  Int offsetx=16,offsety=16;
/* end: added for OBSS by Samsung AIT (1999-09-29) */
  Int 	ox=0,oy=0;

  for (x=0;x<MB_SIZE;x++)
    for (y=0;y<MB_SIZE;y++) 
       pred[x][y]=0;

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) &&
	!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY) ) 
  {
/* end: added for OBSS by Samsung AIT (1999-09-29) */
	if (GetVopPredictionType(curr) != I_VOP 
			&& GetVopShapeCodingType(curr) ) flag = 1;
  	else flag = 0;
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  } else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) 
  {
	if (GetVopPredictionType(curr) != I_VOP
	      && GetVopPredictionType(curr) != P_VOP
		&& GetVopShapeCodingType(curr) ) flag = 1;
     	else flag = 0;
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  error_res_disable = GetVopErrorResDisable(curr);

  if (adr_i==0 && adr_j==0) MBnum = 0; 	     /* bug fix by Tomoko Aono, 07.10.98 SB */
  if ( (!error_res_disable) && (start_of_packet == 1)) 
  {
	SetConstantImage(alpha_rec_packet,0);
	if (!flag) SetConstantImage(alpha_mode_packet,0);
	else SetConstantImage(modeA,0);
	MBnum = adr_i*(width/MB_SIZE)+adr_j; /* bug fix by Noboru Yamaguchi, 16.09.98 SB */
  }

  mblock           = AllocImage(16,16,SHORT_TYPE);
  mblock_rec       = AllocImage(16,16,SHORT_TYPE);

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) &&
	!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY) ) 
  {
	mblock_up = NULL;
	mblock_spos = NULL;	
  } else if ( GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) 
  {
	mblock_up = AllocImage(16,16,SHORT_TYPE);
	mblock_spos = AllocImage(16,16,SHORT_TYPE);;
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  a_mb = (SInt *) GetImageData(mblock);
  a_mb_rec = (SInt *) GetImageData(mblock_rec);

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) &&
	!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY) ) 
  {
	a_mb_up = NULL;
	a_mb_spos = NULL;
  } else if ( GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) 
  {
	a_mb_up = (SInt *) GetImageData(mblock_up);
	a_mb_spos = (SInt *) GetImageData(mblock_spos);
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  x = adr_j * MB_SIZE;
  y = adr_i * MB_SIZE;	

  amvbits=ShapeInterMB(curr,
						rec_curr, /* added for OBSS by Samsung AIT (1999-09-29) */
						rec_prev,
						rec_up, /* added for OBSS by Samsung AIT (1999-09-29) */
						pred,
						modeA,
						mot_x,
						mot_y,
						motA_x,
						motA_y,
						MB_decisions,
						alpha_decisions,
						&mvda,
						&shape_inter,
						mv_stream,
						thresh,
						width,
						adr_i,
						adr_j
		       );
			
	/* mvda = 1 means mvds=0 */

  GetSubImage(GetVopA(curr),mblock,x,y);
	  
  for( v=0; v<MB_SIZE; v++) for( h=0; h<MB_SIZE; h++ ) {
    alpha_mb[v*MB_SIZE+h] = BINARISE((UChar)a_mb[v*MB_SIZE+h]);
    alpha_mb_comp[v*MB_SIZE+h] = (UChar)pred[v][h];
  }

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) && 
	!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY) ) 
  {
	for( v=0; v<MB_SIZE; v++) for( h=0; h<MB_SIZE; h++ ) {
	   alpha_mb_up[v*MB_SIZE+h] = 0;
	}
  } else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) 
  {
	GetSubImage(GetVopA(rec_up),mblock_up,x+offsetx,y+offsety);
	for( v=0; v<MB_SIZE; v++) for( h=0; h<MB_SIZE; h++ ) {
	   alpha_mb_up[v*MB_SIZE+h] = BINARISE((UChar)a_mb_up[v*MB_SIZE+h]);
	}

	GetSubImage(GetVopASamplePos(rec_up),mblock_spos,x,y);
	for( v=0; v<MB_SIZE; v++) for( h=0; h<MB_SIZE; h++ ) {
	   alpha_mb_spos[v*MB_SIZE+h] = (UChar)a_mb_spos[v*MB_SIZE+h];
	}
        /* reconstructing vertical sampling position */
	for( v=0; v<MB_SIZE; v++) {
	   i = 0;
	   for (h=0; h<MB_SIZE; h++) {
		if( alpha_mb_spos[v*MB_SIZE+h] > 0 ) i = 1;	
		if( i==1 ) break;
	   }
	   alpha_mb_spos_v[v] = i;
	}
        /* reconstructing vertical sampling position */
	for( h=0; h<MB_SIZE; h++) {
	   j = 0;
	   for (v=0; v<MB_SIZE; v++) {
		if( alpha_mb_spos[v*MB_SIZE+h] > 0 ) j = 1;	
		if( j==1 )  break;
	   }
	   alpha_mb_spos_h[h] = j;
	}

  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */
			
  /* Detect shape_mode */

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) && 
        !(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY) )
  {
/* end: added for OBSS by Samsung AIT (1999-09-29) */
  	mb_type=AlphaModeDecision( alpha_mb, alpha_mb_comp, shape_inter, mvda, thresh);
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  } else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr))	
  {
	mb_type=AlphaModeDecisionEnh(alpha_mb,
					alpha_mb_comp,
					alpha_mb_up,
					alpha_mb_spos,
					rec_up,
					adr_i,
					adr_j,
					shape_inter,
					thresh);
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */


#ifdef  _DEBUG_OBSS_
if (GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr)))
{
   fprintf(stderr,">>>ShapeCodeMB() ==>\n");
   fprintf(stderr,">>>>>mb_type:%d (-1: inter_not_coded, 0: all_255, 1: all_0, 2: multi)\n", mb_type);
}
#endif

  if( mb_type == 2) 	/* in the case of a coded BAB */
  {
    	for( v=0; v<MB_SIZE; v++ ) 
	for( h=0; h<MB_SIZE; h++ )
      		alpha_mb[h*MB_SIZE+v] = f1bit(alpha_mb[h*MB_SIZE+v]);	

	/* get the subsampled and reconstructed MBs */

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  	if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
		(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) && 
         	!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY)) 
	{
/* end: added for OBSS by Samsung AIT (1999-09-29) */

      	   /*** SAIT begin ***/
      	   if (error_res_disable)
      	   {
     	      if (y>1) GetSubImage(GetVopA(rec_curr),top_border_data,x-2,y-2);
      	      else SetConstantImage(top_border_data,0);
      	      if (x>1) GetSubImage(GetVopA(rec_curr),left_border_data,x-2,y);
      	      else SetConstantImage(left_border_data,0);

      	   } else 
      	   {
     	      if (y>1) GetSubImage(alpha_rec_packet,top_border_data,x-2,y-2);
      	      else SetConstantImage(top_border_data,0);
      	      if (x>1) GetSubImage(alpha_rec_packet,left_border_data,x-2,y);
      	      else SetConstantImage(left_border_data,0);
      	   }
      	   BinariseImage(top_border_data);
      	   BinariseImage(left_border_data);
 
      	   bordered_smb_data = AllocImage(MB_SIZE+4,MB_SIZE+4,SHORT_TYPE);
      	   /*** SAIT end ***/

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
	} else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) 
	{
     	   if (y>1) GetSubImage(GetVopA(rec_curr),top_border_data,x-2,y-2);
      	   else SetConstantImage(top_border_data,0);
      	   if (x>1) GetSubImage(GetVopA(rec_curr),left_border_data,x-2,y);
      	   else SetConstantImage(left_border_data,0);

	   if (y+MB_SIZE<GetVopHeight(rec_curr)-1) 
		GetSubImage(GetVopA(rec_up),bottom_border_data,x+offsetx-2,y+offsety+MB_SIZE);
	   else SetConstantImage(bottom_border_data,0);
	   if (x+MB_SIZE<GetVopWidth(rec_curr)-1)
		GetSubImage(GetVopA(rec_up),right_border_data,x+offsetx+MB_SIZE,y+offsety);
	   else SetConstantImage(right_border_data,0);

      	   BinariseImage(top_border_data);
      	   BinariseImage(left_border_data);
      	   BinariseImage(bottom_border_data);
      	   BinariseImage(right_border_data);

      	   bordered_smb_data = AllocImage(MB_SIZE+4,MB_SIZE+4,SHORT_TYPE);
	}
/* end: added for OBSS by Samsung AIT (1999-09-29) */

	if (GetVopChangeCRDisable(curr)) 
	{
	  send_cr = 0;
   	  for(i=0;i<MB_SIZE*MB_SIZE;i++) alpha_smb[i]=alpha_mb[i]; 

          /*** SAIT begin ***/
          PutSubImage(bordered_smb_data,top_border_data,0,0);
          PutSubImage(bordered_smb_data,left_border_data,0,2);
/* Free displaced to the end of the file : for leakage problem: Dufour 15.04.98 */
/*
          FreeImage(top_border_data);
          FreeImage(left_border_data);
*/
          p = (SInt *) GetImageData(bordered_smb_data)
            + 2*GetImageSizeX(bordered_smb_data) + 2;
          for( i=0; i<MB_SIZE; i++)
            for( j=0; j<MB_SIZE; j++) {
              p[GetImageSizeX(bordered_smb_data)*i+j]
                = (SInt) alpha_smb[i*MB_SIZE+j];
            }
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  	  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
		(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) &&
		!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY))  
	  {
/* end: added for OBSS by Samsung AIT (1999-09-29) */
             tmp = AllocImage(2,1,SHORT_TYPE);
             GetSubImage(bordered_smb_data,tmp,0,MB_SIZE+1);
             PutSubImage(bordered_smb_data,tmp,0,MB_SIZE+2);
             PutSubImage(bordered_smb_data,tmp,0,MB_SIZE+3);
             FreeImage(tmp);
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
	  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */
          /*** SAIT end ***/

	}
	else 
	{
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  	  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
		(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) && 
         	!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY)) 
	  {
/* end: added for OBSS by Samsung AIT (1999-09-29) */

	  	/* SAIT begin */
	  	if (error_res_disable) rec_alpha = GetVopA(rec_curr);
	  	else rec_alpha = alpha_rec_packet;

         	mb_type = DetectCR( rec_alpha, x, y, alpha_mb, alpha_smb, thresh,
                             &bordered_smb_data, /* ch. SB */
                             top_border_data, left_border_data);

/* Free displaced to the end of the file : for leakage problem: Dufour 15.04.98 */
/*
          	FreeImage(top_border_data);
          	FreeImage(left_border_data);
*/
	  	/* SAIT end */

		send_cr = 1;

           
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
	   } else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) {	
		for(i=0;i<MB_SIZE*MB_SIZE;i++) alpha_smb[i]=alpha_mb[i];
		send_cr = 0;
		mb_type = 2;
                
		fprintf(stderr,"EORROR: Currently CR is not supported for scalable shape coding !\n");
		exit(0);
           
	   }
/* end: added for OBSS by Samsung AIT (1999-09-29) */
	}
 
 	/* encode the BAB using CAE*/

   	if( mb_type >= 2 && mb_type <= 4 )
	{
		if (flag == 1) 
		{
			rec_prev_alpha = GetVopA(rec_prev);
  			ox = GetVopHorSpatRef(curr) - GetVopHorSpatRef(rec_prev);
 			oy = GetVopVerSpatRef(curr) - GetVopVerSpatRef(rec_prev);
		}
		else rec_prev_alpha = NULL;

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  	  	if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
			(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) && 
         		!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY)) 
		{
/* end: added for OBSS by Samsung AIT (1999-09-29) */
			cae_bits=CAE_MB(rec_prev,
					bordered_smb_data, /*SAIT*/
						alpha_mb_comp,
						x,
						y,
						ox,
						oy,
						GetVopA(rec_curr),
						rec_prev_alpha,
						alpha_rec_packet,
						motA_x,
						motA_y,
						shape_inter,
						mvda,
						amvbits,
						send_cr,
						&mb_type,
						cae_stream,	
						vo_id,
						vol_id,
						error_res_disable);

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
		} else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) {	

#ifdef  _DEBUG_OBSS_
fprintf(stderr,">>>>>Scalable Shape Encoding (to SI_MB())!\n");
fprintf(stderr,">>>>>Input bordered_smb>>>>>>>>>>>>>>>>>>\n");
p = (SInt *)GetImageData(bordered_smb_data);
for (i=0;i<MB_SIZE+4;i++)
{
        for (j=0;j<MB_SIZE+4;j++) fprintf(stderr,"%d", (*(p++)>0) );
        fprintf(stderr,"\n");
}
fprintf(stderr,"\n");
#endif
                        cae_bits=SI_MB(rec_up,	
						bordered_smb_data,
						bottom_border_data,
						right_border_data,
						alpha_mb_comp,
						alpha_mb_spos_h,
						alpha_mb_spos_v,
						x,
						y,
						ox,
						oy,
						GetVopA(rec_curr),
						rec_prev_alpha,
						motA_x,
						motA_y,
						shape_inter,
						amvbits,
						send_cr,
						&mb_type,
						cae_stream,
						rec_curr,
						error_res_disable);

#ifdef  _DEBUG_OBSS_
fprintf(stdout,">>>>>Output bordered_smb>>>>>>>>>>>>>>>>>\n");
p = (SInt *)GetImageData(bordered_smb_data);
for (i=0;i<MB_SIZE+4;i++)
{
    for (j=0;j<MB_SIZE+4;j++) fprintf(stderr,"%d", (*(p++)>0) );
    fprintf(stderr,"\n");
}
fprintf(stdout,"\n");
#endif
      FreeImage (bordered_smb_data); /* added for OBSS by Samsung AIT (1999-09-29) ; moved to the inside of the if-clause to prevent multiple FreeImage calls in case of no OBSS - U. Benzler, UoH 991110*/

		}
/* end: added for OBSS by Samsung AIT (1999-09-29) */

    	}
	else
	{
      		cae_bits=0;
    	}

	if (mb_type==5) mb_type+=mvda; /* inter and coded */

  }
  else cae_bits=0;

  if( (mb_type == -1) || (mb_type >= 5)) 
  {
	*((SInt *)GetImageData(modeA)+adr_i*(width/MB_SIZE)+adr_j) = 1;
  }

  if( mb_type >= 0 && mb_type <= 4 ) amvbits = 0;

  if( mb_type == -1 )
   	   for( v=0; v<MB_SIZE; v++ ) for( h=0; h<MB_SIZE; h++ )
      		alpha_mb[v*MB_SIZE+h] = BINARISE(alpha_mb_comp[v*MB_SIZE+h]);
  else
  {
   	   for( v=0; v<MB_SIZE; v++ ) for( h=0; h<MB_SIZE; h++ )
      		alpha_mb[v*MB_SIZE+h] = f8bit(alpha_mb[v*MB_SIZE+h]);
  }

  /* Encode the first_shape_code */

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) &&
        !(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY)) 
  {
/* end: added for OBSS by Samsung AIT (1999-09-29) */
	MakeMBtype( mb_type, mvda, adr_i, adr_j, rec_curr);
	
	if( flag == 1 ) 
	{
		*first_bits = CodeInterMBtype( adr_i, adr_j, first_shape_code,rec_curr,rec_prev);
	}
	else
	{
		if (!error_res_disable)
			*first_bits = CodeIntraMBtype_ER( first_shape_code, mb_type, 
							adr_i, adr_j, rec_curr, 
 							alpha_mode_packet);
		else
			*first_bits = CodeIntraMBtype( first_shape_code, mb_type, 
							adr_i, adr_j, rec_curr);
	}
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  } else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr))	
  {
	MakeMBtypeEnh( mb_type, adr_i, adr_j, rec_curr );

	if( flag == 1) {
		*first_bits = CodeInterMBtypeEnh(adr_i,
					adr_j,
					first_shape_code,
					rec_curr,
					rec_up);
	} else {
		if (error_res_disable)
		{
		   *first_bits = CodeIntraMBtypeEnh(adr_i,
					adr_j,
					first_shape_code,
					rec_curr);
		} else {
		   fprintf(stderr,"Error: There is no error resil. mode in scalable shape !\n");
		   exit(0);
		}
	}
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  /* update the decoded alpha image */
  for( v=0; v<MB_SIZE; v++ ) for( h=0; h<MB_SIZE; h++ ) {
    a_mb_rec[v*MB_SIZE+h] = (SInt)alpha_mb[v*MB_SIZE+h];
  }

#ifdef  _DEBUG_OBSS_
if (GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) 
{
  fprintf(stdout,">>>>>Encoded BAB Map>>>>>>>>>>>>>>>>>\n");
  for (v=0;v<MB_SIZE;v++)
  {
       for (h=0;h<MB_SIZE;h++) 
           fprintf(stderr,"%d", ((SInt)alpha_mb[v*MB_SIZE+h]>0) );
       fprintf(stderr,"\n");
  }
  fprintf(stdout,"\n");
}
#endif

  if(shape_inter==0 && mb_type==-1){
    printf(" Error: shape_inter=%d however mb_type=%d\n",
	    shape_inter,mb_type);
    exit(0);
  }

	/* do all bitstream book-keeping */

  /* first_shape_code bit stream */
  for(i=0; i<*first_bits; i++)
    first_stream[i] = first_shape_code[i];

	total_first_bits[vol_id] += *first_bits;
	curr_first_bits += *first_bits;

  /* second_code bit stream */
  mb_bits+=second_bits;
  for(i=0; i<second_bits; i++)
    shape_stream[i] = second_code[i];
  shape_stream+=second_bits;

	total_second_bits[vol_id] += second_bits;

  /* mv bit stream */
  mb_bits+=amvbits;
  for(i=0; i<amvbits; i++)
    shape_stream[i] = mv_stream[i];
  shape_stream+=amvbits;

	total_amv_bits[vol_id] += amvbits;
	curr_amv_bits += amvbits;

  /* cae bit stream */
  mb_bits+=cae_bits;

  for(i=0; i<cae_bits; i++)
    shape_stream[i] = cae_stream[i];
  shape_stream+=cae_bits;

  if (cae_bits>0) 
  {
	total_coded_mbs[vol_id]++;
	curr_coded_mbs++;
	total_cae_bits[vol_id] += cae_bits;
	curr_cae_bits += cae_bits;
	if (mb_type>=2 && mb_type<=4)
	{
		total_coded_intra_mbs[vol_id]++;
		curr_coded_intra_mbs++;
		total_cae_intra_bits[vol_id] += cae_bits;
		curr_cae_intra_bits += cae_bits;
	}
	else
	{
		total_coded_inter_mbs[vol_id]++;
		curr_coded_inter_mbs++;
		total_cae_inter_bits[vol_id] += cae_bits;
		curr_cae_inter_bits += cae_bits;
	}
   }

   /* write the coded BAB to the reconstructed VOP */

  PutSubImage(GetVopA(rec_curr),mblock_rec,x,y);
	if (!error_res_disable)
  	PutSubImage(alpha_rec_packet,mblock_rec,x,y);

  block_rec = AllocImage(8,8,SHORT_TYPE);				
  a_b_rec = (SInt *) GetImageData(block_rec);
  for( v=0; v<8; v++ ) for( h=0; h<8; h++ )
  {
	Int index = (v+v)*MB_SIZE+h+h;   
	a_b_rec[v*8+h] = (a_mb_rec[index] ||
				a_mb_rec[index+1] ||
				a_mb_rec[index+MB_SIZE] ||
				a_mb_rec[index+MB_SIZE+1])?255:0;
  }

  PutSubImage(GetVopAuv(rec_curr),block_rec,x/2,y/2);
  FreeImage (block_rec);

  /* modify the modes and MVs based on coded shape */
	
  if ((GetVopPredictionType(curr) != I_VOP) && (GetVopShape(curr)!=BINARY_SHAPE_ONLY))  /* BSO_NOEL */
		ModifyModesAndMVs(GetVopPredictionType(curr),
                                  mblock_rec,MB_decisions,alpha_decisions,
				  mot_x,mot_y,adr_j,adr_i);

  FreeImage (mblock);
  FreeImage (mblock_rec);

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if ( (GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) ||
        (GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY))
  {
		if(!GetVopUseRefShape(curr) || GetVopShape(curr)==BINARY_SHAPE_ONLY ) {	
        FreeImage(mblock_up);
        FreeImage(mblock_spos);
		}	
  } 
/* end: added for OBSS by Samsung AIT (1999-09-29) */

/* Free displaced here : for leakage problem: Dufour 15.04.98 */
FreeImage(top_border_data);
FreeImage(left_border_data);

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
   FreeImage(bottom_border_data);
   FreeImage(right_border_data);
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  return mb_bits;
}

Void ModifyModesAndMVs(Int vop_type,Image *mblock_rec ,Image *MB_decisions, 
			Image *alpha_decisions, Image *mot_x, Image *mot_y, 
			Int mb_x, Int mb_y)
{
	Int	x,y;

	SInt 	*ps = (SInt *) GetImageData(mblock_rec),
				*p;

	Int opaque1,
			opaque2,
			opaque3,
			opaque4,
			opaque,
			pos1,
			pos2,
			width = GetImageSizeX(alpha_decisions);

	SInt *mode_ptr = (SInt *) GetImageData(MB_decisions),
				mode;

	SInt *alpdec_ptr = (SInt *) GetImageData(alpha_decisions);

	Float *px = (Float *) GetImageData(mot_x),
				*py = (Float *) GetImageData(mot_y);

	pos1 = mb_y * GetImageSizeX(MB_decisions) + mb_x;
	pos2 = 2*mb_y * GetImageSizeX(alpha_decisions) + 2*mb_x;

	mode = mode_ptr[pos1];

	/* Block 1 */
	opaque1 = 0;
	for (y=0,p=ps;y<MB_SIZE/2;y++,p+=MB_SIZE/2)
		for (x=0;x<MB_SIZE/2;x++,p++)
			opaque1 |= (*p)&1;
	
	if (opaque1) alpdec_ptr[pos2] = 1;
	else alpdec_ptr[pos2] = MBM_TRANSPARENT;


	/* Block 2 */
	opaque2 = 0;
	for (y=0,p=ps+8;y<MB_SIZE/2;y++,p+=MB_SIZE/2)
		for (x=0;x<MB_SIZE/2;x++,p++)
			opaque2 |= (*p)&1;

	if (opaque2) alpdec_ptr[pos2+1] = 1;
	else alpdec_ptr[pos2+1] = MBM_TRANSPARENT;


       /* Block 3 */
	opaque3 = 0;
	for (y=0,p=ps+8*MB_SIZE;y<MB_SIZE/2;y++,p+=MB_SIZE/2)
		for (x=0;x<MB_SIZE/2;x++,p++)
			opaque3 |= (*p)&1;
	
	if (opaque3) alpdec_ptr[pos2+width] = 1;
	else alpdec_ptr[pos2+width] = MBM_TRANSPARENT;


      	/* Block 4 */
	opaque4 = 0;
	for (y=0,p=ps+8*MB_SIZE+8;y<MB_SIZE/2;y++,p+=MB_SIZE/2)
		for (x=0;x<MB_SIZE/2;x++,p++)
			opaque4 |= (*p)&1;
   
	if (opaque4) alpdec_ptr[pos2+width+1] = 1;
	else alpdec_ptr[pos2+width+1] = MBM_TRANSPARENT;

         opaque = opaque1 || opaque2 || opaque3 || opaque4;	

        if (vop_type==B_VOP) {
            if (!opaque) mode_ptr[pos1] = -1; /* transparent */
        } else {

          if (!opaque) {
		mode_ptr[pos1] = MBM_TRANSPARENT;
                px[pos2] = px[pos2+1]=px[pos2+width]=px[pos2+width+1]=0.0; 	
                py[pos2] = py[pos2+1]=py[pos2+width]=py[pos2+width+1]=0.0; 		
	   } else {	
                 if ((opaque1+opaque2+opaque3+opaque4==1)&&(mode == MODE_INTER4V)) {
                    mode_ptr[pos1] = MODE_INTER; 
                 }
             if (mode !=MODE_INTRA) {
                    /* vector padding Minhua Zhou 08.10.1997*/
                 if (!opaque1) {
                    px[pos2]=(opaque2)?px[pos2+1]:((opaque3)?px[pos2+width]:px[pos2+width+1]);
                    py[pos2]=(opaque2)?py[pos2+1]:((opaque3)?py[pos2+width]:py[pos2+width+1]);
                 }
                if (!opaque2) {
                    px[pos2+1]=(opaque1)?px[pos2]:((opaque4)?px[pos2+width+1]:px[pos2+width]);
                    py[pos2+1]=(opaque1)?py[pos2]:((opaque4)?py[pos2+width+1]:py[pos2+width]);
                 }

               if (!opaque3) {
                    px[pos2+width]=(opaque4)?px[pos2+width+1]:((opaque1)?px[pos2]:px[pos2+1]);
                    py[pos2+width]=(opaque4)?py[pos2+width+1]:((opaque1)?py[pos2]:py[pos2+1]);
                 }
              if (!opaque4) {
                    px[pos2+width+1]=(opaque3)?px[pos2+width]:((opaque2)?px[pos2+1]:px[pos2]);
                    py[pos2+width+1]=(opaque3)?py[pos2+width]:((opaque2)?py[pos2+1]:py[pos2]);
                 }
             } /* MODE_INTRA */
          }
      }

		
}

Void CopyVopABlock(Vop *vop1, Vop *vop2, Int x, Int y)  
{
	Image *block = AllocImage(16,16,SHORT_TYPE);

	GetSubImage(GetVopA(vop1), block, x, y);
	PutSubImage(GetVopA(vop2), block, x, y);
	FreeImage(block);
}

Void CopyVopAuvBlock(Vop *vop1, Vop *vop2, Int x, Int y)
{
	Image *block = AllocImage(8,8,SHORT_TYPE);

	GetSubImage(GetVopAuv(vop1), block, x, y);
	PutSubImage(GetVopAuv(vop2), block, x, y);
	FreeImage(block);
}



Int
ShapeInterMB(Vop *curr,
	     Vop *rec_curr, /* added for OBSS by Samsung AIT (1999-09-29) */
	     Vop *rec_prev,
	     Vop *rec_up, /* added for OBSS by Samsung AIT (1999-09-29) */
	     SInt pred[MB_SIZE][MB_SIZE],
	     Image *modeA,
	     Image *mot_x,
	     Image *mot_y,
	     Image *motA_x,
	     Image *motA_y,
	     Image *MB_decisions,
	     Image *alpha_decisions,
	     Int *mvda,
	     Int *shape_inter,
	     UChar *alphaMV_stream,
	     Int thresh,
	     Int width,
	     Int i,
	     Int j )
{
  Image
    *mblock;

  SInt
    *a_mb;

  Int
    ox,oy,
    x, y,
    mb_size=MB_SIZE,
    amvbits=0;
  
  Int
    motDA_x[4], motDA_y[4];
  
  mblock = AllocImage(16,16,SHORT_TYPE);
  a_mb = (SInt *) GetImageData(mblock);

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if (!(GetVopScalability(curr) && !GetVopHierarchyType(curr) &&
	(!GetVopEnhanceType(curr) || !GetVopUseRefShape(curr))) &&
       	!(GetVopScalability(curr) && GetVopShape(curr)==BINARY_SHAPE_ONLY)) 
  {
/* end: added for OBSS by Samsung AIT (1999-09-29) */
  	if (GetVopPredictionType(curr) == I_VOP || (GetVopShapeCodingType(curr)==0)) {
    		*shape_inter = 0;
  	} else {
    		*shape_inter = 1;
  	}
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  } else if (GetVopShape(curr)==BINARY_SHAPE_ONLY || !GetVopUseRefShape(curr)) 
  {
#ifdef  _DEBUG_OBSS_
	fprintf(stderr,">>>ShapeInterMB()==>\n");
#endif
	if ((GetVopPredictionType(curr) == I_VOP) ||
	    	(GetVopPredictionType(curr) == P_VOP) ||
			(GetVopShapeCodingType(curr)==0) )
	{
		*shape_inter = 0;
#ifdef  _DEBUG_OBSS_
fprintf(stderr,">>>>>(a)mb[%d %d], shape_inter=%d\n", i, j, *shape_inter);
#endif
	}
	else
	{
		*shape_inter = 1;
	}
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  x = j * mb_size;
  y = i * mb_size;
  *mvda = 1;

  if (*shape_inter ) {

    GetSubImage(GetVopA(curr),mblock,x,y);
    ox = GetVopHorSpatRef(curr) - GetVopHorSpatRef(rec_prev);
    oy = GetVopVerSpatRef(curr) - GetVopVerSpatRef(rec_prev);
    amvbits = FindPredAlphaAndMVmei (curr,
					rec_curr, /* added for OBSS by Samsung AIT (1999-09-29) */
					rec_up, /* added for OBSS by Samsung AIT (1999-09-29) */
				     x,y,
				     ox,oy,
				     mot_x,mot_y,
				     motA_x,motA_y,
				     MB_decisions,
				     alpha_decisions,
				     modeA,
				     (SInt *)GetImageData(GetVopA(rec_prev)),
				     (SInt *)pred,
				     a_mb,
				     width,
				     GetImageSizeX(GetVopA(rec_prev)),
				     GetImageSizeY(GetVopA(rec_prev)),
				     mb_size,
				     thresh,
				     motDA_x,motDA_y,
				     mvda,
				     alphaMV_stream,
				     MBnum ); /* bug fix by Noboru Yamaguchi, 16.09.98 SB */
  }
  
  FreeImage (mblock);
  return amvbits;
}


void
BitstreamAppendTtoM(Image *bitstream, UChar *stream, UInt nbits)
{
  UInt i;
  SInt bits;

  /* it should be 1 <= nbits <= 32 */

  if (nbits > 0)
    {
      for ( i=0; i<nbits; i++ ) {
	bits = (SInt)(*(stream++));
        BitstreamPutBit(bitstream, bits);
      }
    }
}

void
PutFirstCodes(
	      Image **First_stream,
	      Image *first_shape_code_bitstream,
	      Int width,
	      Int height
	      )
{
  Int i,j;

  for(i=0; i<height; i++)
    for(j=0; j<width; j++)
      BitstreamAppend(First_stream[i*width+j], first_shape_code_bitstream);
}
				
void
PutShapeCodes(
	      Image **Shape_stream,
	      Image *shape_bitstream,
	      Int width,
	      Int height
	      )
{
  Int i,j;

  for(i=0; i<height; i++)
    for(j=0; j<width; j++)
      BitstreamAppend(Shape_stream[i*width+j], shape_bitstream);
}


Void AllocShapePacket(Vop *vop)
{
	Int 	width = GetVopWidth(vop)/MB_SIZE,
				height = GetVopHeight(vop)/MB_SIZE;

	alpha_rec_packet = AllocSameImage(GetVopA(vop));
	alpha_mode_packet = AllocImage(width,height,SHORT_TYPE);
}

Void FreeShapePacket()
{
	FreeImage(alpha_rec_packet);
	FreeImage(alpha_mode_packet);
}

/* 01.04.01: Studio Profile */
Int
VopShapeCode_HHC_Studio(Vop *curr,
			Vop *rec_curr,
			Vop *rec_prev,
			Int vo_id,
			Int binum,/*001229*/
			Int thresh,
			Image **First_stream,
			Image **Shape_stream)
{
  Image *modeA; /* the flag of current alpha block is inter coded or not */
  Image *motA_x; /* the x compornent of shape MV */
  Image *motA_y; /* the y compornent of shape MV */
  Int first_bits, shape_bits, total_shape_bits = 0;
  Int i, j, k;
  UChar first_stream[7], shape_stream[4096];

  Int vop_structure = GetVopVopStructure_st(curr);
  Int fip = CalcFieldParity_Studio(vop_structure,
				   GetVopVopCounter_st(curr),
				   GetVopTopFieldFirst(curr));
  
  /* Allocate shape ME/MC */
  modeA = AllocImage(GetVopWidth_st(curr,fip)/MB_SIZE,
		     GetVopHeight_st(curr,fip)/MB_SIZE,SHORT_TYPE);
  motA_x = AllocImage(GetVopWidth_st(curr,fip)/MB_SIZE,
		      GetVopHeight_st(curr,fip)/MB_SIZE,SHORT_TYPE);
  motA_y = AllocImage(GetVopWidth_st(curr,fip)/MB_SIZE,
		      GetVopHeight_st(curr,fip)/MB_SIZE,SHORT_TYPE);

  for(i=0; i<GetVopHeight_st(curr,fip)/MB_SIZE; i++) {
    for(j=0; j<GetVopWidth_st(curr,fip)/MB_SIZE; j++) {
      k = i*(GetVopWidth_st(curr,fip)/MB_SIZE)+j;
      shape_bits = ShapeCodeMB_HHC_Studio(curr,
					  rec_curr,
					  rec_prev,
					  vo_id,
					  binum,/*001229*/
					  i,
					  j,
					  thresh,
					  modeA,
					  motA_x,
					  motA_y,
					  &first_bits,
					  first_stream,
					  shape_stream,
					  0);
      BitstreamAppendTtoM(First_stream[k], first_stream, first_bits);
      BitstreamAppendTtoM(Shape_stream[k], shape_stream, shape_bits);
      total_shape_bits += shape_bits + first_bits;
    }
  }
				
  /* Free shape ME/MC */
  FreeImage(modeA);
  FreeImage(motA_x);
  FreeImage(motA_y);
  return total_shape_bits;
}

Int
ShapeCodeMB_HHC_Studio(Vop *curr,
		       Vop *rec_curr,
		       Vop *rec_prev,
		       Int vo_id,
		       Int binum,/*001229*/
		       Int adr_i,
		       Int adr_j,
		       Int thresh,
		       Image *modeA,
		       Image *motA_x,
		       Image *motA_y,
		       Int *first_bits,
		       UChar *first_stream,
		       UChar *shape_stream,
		       Int start_of_packet)
{
  Int vopstr = GetVopVopStructure_st(curr);
  Int fip = CalcFieldParity_Studio(vopstr,
				   GetVopVopCounter_st(curr),
				   GetVopTopFieldFirst(curr));
  
  /* SAIT begin */
  Image
    *tmp,
    *bordered_smb_data,
    *top_border_data = AllocImage(MB_SIZE+4,2,SHORT_TYPE),
    *left_border_data = AllocImage(2,MB_SIZE,SHORT_TYPE);
 
  SInt *p;
  Int   j;
  /* SAIT end */

  Int vol_id = GetVopId(curr);

  Image 
    *mblock,
    *mblock_rec,
    *block_rec,
    *rec_prev_alpha,
    *rec_alpha;
  
  SInt
    pred[MB_SIZE][MB_SIZE],
    *a_mb,
    *a_mb_rec,
    *a_b_rec;
  
  Int
    error_res_disable,
    width=GetVopWidth_st(curr,fip),
    second_bits=0,
    cae_bits=0,
    amvbits=0,
    mb_bits=0,
    mvda,
    shape_inter,
    mb_type,
    i,
    x,y,
    h,v;

  Int
    flag,
    send_cr;

  UChar
    alpha_mb[MB_SIZE*MB_SIZE],
    alpha_mb_comp[MB_SIZE*MB_SIZE],
    alpha_smb[MB_SIZE*MB_SIZE],
    first_shape_code[7],
    second_code[2],
    cae_stream[4096],
    mv_stream[32];

  Int 	ox=0,oy=0;

  Int k;

  for (x=0;x<MB_SIZE;x++)
    for (y=0;y<MB_SIZE;y++) 
       pred[x][y]=0;

  if (GetVopPredictionType_st(curr,fip)!=I_VOP && GetVopShapeCodingType_st(curr,fip) )
    flag = 1;
  else
    flag = 0;

  error_res_disable = GetVopErrorResDisable(curr);

  if (adr_i==0 && adr_j==0) MBnum = 0; 	     /* bug fix by Tomoko Aono, 07.10.98 SB */
  if ( (!error_res_disable) && (start_of_packet == 1)) 
  {
	SetConstantImage(alpha_rec_packet,0);
	if (!flag) SetConstantImage(alpha_mode_packet,0);
	else SetConstantImage(modeA,0);
	MBnum = adr_i*(width/MB_SIZE)+adr_j; /* bug fix by Noboru Yamaguchi, 16.09.98 SB */
  }

  mblock           = AllocImage(16,16,SHORT_TYPE);
  mblock_rec       = AllocImage(16,16,SHORT_TYPE);

  a_mb = (SInt *) GetImageData(mblock);
  a_mb_rec = (SInt *) GetImageData(mblock_rec);

  x = adr_j * MB_SIZE;
  y = adr_i * MB_SIZE;	

  amvbits=ShapeInterMB_HHC_Studio(curr,
				  rec_curr,/*001229*/
				  rec_prev,
				  pred,
				  modeA,
				  motA_x,
				  motA_y,
				  &mvda,
				  &shape_inter,
				  mv_stream,
				  thresh,
				  width,
				  adr_i,
				  adr_j,
				  binum/*001229*/);
			
  /* mvda = 1 means mvds=0 */
  GetSubImage_Studio(GetVopA2_st(curr,binum),vopstr,fip,
		     mblock,x,y);
  /*GetSubImage(GetVopA(curr),mblock,x,y);*/
	  
  for( v=0; v<MB_SIZE; v++) for( h=0; h<MB_SIZE; h++ ) {
    alpha_mb[v*MB_SIZE+h] = BINARISE((UChar)a_mb[v*MB_SIZE+h]);
    alpha_mb_comp[v*MB_SIZE+h] = (UChar)pred[v][h];
  }
			
  /* Detect shape_mode */

  mb_type=AlphaModeDecision_Studio( alpha_mb, alpha_mb_comp, shape_inter, mvda, thresh);


  if( mb_type == 2) 	/* in the case of HHC */
  {
/**/
    Int inferior = InferiorSymbolDecision_Studio( alpha_mb, MB_SIZE );
    Int CBBP = CodedBoundaryBlockPatternDecision_Studio( inferior, alpha_mb );

    Int mask;
    UChar alpha_block[64];
    Int inferiorBlock;
    Int line_direction;
    Int line_reverse;
    Int CLP;
    UInt mask1, mask3;
    Int mask2;
    Int LBP;
    
    /* inferior symbol output */
    if ( inferior )     cae_stream[cae_bits++] = 0x1;
    else                cae_stream[cae_bits++] = 0x0;

    mask = 0x1<<(cbbp[CBBP].len-1);
    
    /* coded boundary block pattern output */
    for( i=0 ; i<cbbp[CBBP].len ; i++ )
      {
        if ( cbbp[CBBP].code & mask )  cae_stream[cae_bits++] = 0x1;
        else                           cae_stream[cae_bits++] = 0x0;
        mask >>= 1;
      }

    mask = 0x8;
    for( i=0 ; i<4 ; i++ )
      {
        if ( CBBP&mask )
	  {
	    SetToBlock_Studio( i, alpha_mb, alpha_block );
	    inferiorBlock = InferiorSymbolDecision_Studio( alpha_block, 8 );
	    line_direction = LineScanDirection_Studio( inferiorBlock, alpha_block );
	    line_reverse = LineReverseFlagDecision_Studio( inferiorBlock, line_direction, alpha_block );
	    /* block inferior symbol output */
	    if ( inferiorBlock )    cae_stream[cae_bits++] = 0x1;
	    else            cae_stream[cae_bits++] = 0x0;
	    /* line scan direction output */
	    if ( line_direction )   cae_stream[cae_bits++] = 0x1;
	    else                    cae_stream[cae_bits++] = 0x0;
	    /* line reverse flag output */
	    if ( line_reverse )     cae_stream[cae_bits++] = 0x1;
	    else            cae_stream[cae_bits++] = 0x0;
	    CLP = CodedLinePattern_Studio( i, line_direction, inferiorBlock, alpha_mb );
	    mask1 = 0x1<<(clp_tbl[CLP].len-1);
	    /* coded boundary block pattern output */
	    for( j=0 ; j<clp_tbl[CLP].len ; j++ )
	      {
		if ( clp_tbl[CLP].code&mask1 )        cae_stream[cae_bits++] = 0x1;
		else                          cae_stream[cae_bits++] = 0x0;
		mask1 >>= 1;
	      }

	    /* MoMuSys-2000FDAM1COR-020615_0 *//* marker_bit */
	    cae_stream[cae_bits++] = 0x1;
	    /**/
	    
	    mask2 = 0x80;
	    for ( j=0 ; j<8 ; j++ )
	      {
		if ( CLP&mask2 )
		  {
		    LBP = LineBitPattern_Studio( i, j, inferiorBlock, line_direction, line_reverse, alpha_mb );
		    mask3 = 0x1<<(lbp_tbl[LBP].len-1);
		    /* coded boundary block pattern output */
		    for( k=0 ; k<lbp_tbl[LBP].len ; k++ )
		      {
			if ( lbp_tbl[LBP].code&mask3 )    cae_stream[cae_bits++] = 0x1;
			else                              cae_stream[cae_bits++] = 0x0;
			mask3 >>= 1;
		      }
		    
		    /* MoMuSys-2000FDAM1COR-020615_0 *//* marker_bit */
		    cae_stream[cae_bits++] = 0x1;
		    /**/

		  }
		mask2 >>= 1;
	      }
	  }
        mask >>= 1;
      }
    /**/
    


    
    	for( v=0; v<MB_SIZE; v++ ) 
	for( h=0; h<MB_SIZE; h++ )
      		alpha_mb[h*MB_SIZE+v] = f1bit(alpha_mb[h*MB_SIZE+v]);	
#if 0
	/* get the subsampled and reconstructed MBs */

      /*** SAIT begin ***/
      if (error_res_disable)
      {
     	if (y>1) GetSubImage(GetVopA(rec_curr),top_border_data,x-2,y-2);
      	else SetConstantImage(top_border_data,0);
      	if (x>1) GetSubImage(GetVopA(rec_curr),left_border_data,x-2,y);
      	else SetConstantImage(left_border_data,0);
      } else 
      {
     	if (y>1) GetSubImage(alpha_rec_packet,top_border_data,x-2,y-2);
      	else SetConstantImage(top_border_data,0);
      	if (x>1) GetSubImage(alpha_rec_packet,left_border_data,x-2,y);
      	else SetConstantImage(left_border_data,0);
      }

      BinariseImage(top_border_data);
      BinariseImage(left_border_data);
 
      bordered_smb_data = AllocImage(MB_SIZE+4,MB_SIZE+4,SHORT_TYPE);
      /*** SAIT end ***/


	if (GetVopChangeCRDisable(curr)) 
	{
		send_cr = 0;
#ifndef _VM7_FILTER_
   		for(i=0;i<MB_SIZE*MB_SIZE;i++) alpha_smb[i]=alpha_mb[i];
#else
    		ChangeSamplingRate( alpha_mb, alpha_smb, MB_SIZE, MB_SIZE, MB_SIZE, MB_SIZE );
#endif

          /*** SAIT begin ***/
          PutSubImage(bordered_smb_data,top_border_data,0,0);
          PutSubImage(bordered_smb_data,left_border_data,0,2);
/* Free displaced to the end of the file : for leakage problem: Dufour 15.04.98 */
/*
          FreeImage(top_border_data);
          FreeImage(left_border_data);
*/
          p = (SInt *) GetImageData(bordered_smb_data)
            + 2*GetImageSizeX(bordered_smb_data) + 2;
          for( i=0; i<MB_SIZE; i++)
            for( j=0; j<MB_SIZE; j++) {
              p[GetImageSizeX(bordered_smb_data)*i+j]
                = (SInt) alpha_smb[i*MB_SIZE+j];
            }
          tmp = AllocImage(2,1,SHORT_TYPE);
          GetSubImage(bordered_smb_data,tmp,0,MB_SIZE+1);
          PutSubImage(bordered_smb_data,tmp,0,MB_SIZE+2);
          PutSubImage(bordered_smb_data,tmp,0,MB_SIZE+3);
          FreeImage(tmp);
          /*** SAIT end ***/

	}
	else 
	{
	  /* SAIT begin */
	  	/******* comment out
		if (error_res_disable) rec_alpha = GetVopA(rec_curr);
		else rec_alpha = alpha_rec_packet;
		mb_type = DetectCR( rec_alpha, x, y, alpha_mb, alpha_smb, thresh );
	  	*********************/
	  if (error_res_disable) rec_alpha = GetVopA(rec_curr);
	  else rec_alpha = alpha_rec_packet;

         mb_type = DetectCR( rec_alpha, x, y, alpha_mb, alpha_smb, thresh,
                             &bordered_smb_data, /* ch. SB */
                             top_border_data, left_border_data);

/* Free displaced to the end of the file : for leakage problem: Dufour 15.04.98 */
/*
          FreeImage(top_border_data);
          FreeImage(left_border_data);
*/
	  /* SAIT end */

		send_cr = 1;

	}
 
 	/* encode the BAB using CAE*/

   	if( mb_type >= 2 && mb_type <= 4 )
	{
		if (flag == 1) 
		{
			rec_prev_alpha = GetVopA(rec_prev);
  			ox = GetVopHorSpatRef(curr) - GetVopHorSpatRef(rec_prev);
 			oy = GetVopVerSpatRef(curr) - GetVopVerSpatRef(rec_prev);
		}
		else rec_prev_alpha = NULL;

					cae_bits=CAE_MB(rec_prev,
							bordered_smb_data, /*SAIT*/
								alpha_mb_comp,
								x,
								y,
								ox,
								oy,
								GetVopA(rec_curr),
								rec_prev_alpha,
								alpha_rec_packet,
								motA_x,
								motA_y,
								shape_inter,
								mvda,
								amvbits,
								send_cr,
								&mb_type,
								cae_stream,	
								vo_id,
								vol_id,
								error_res_disable);

	      }
	else
	{
      		cae_bits=0;
    	}

#endif 0
	if (mb_type==5) mb_type+=mvda; /* inter and coded */

      }/*in case of HHC*/
	else cae_bits=0;

	if( (mb_type == -1) || (mb_type >= 5)) 
		{
			*((SInt *)GetImageData(modeA)+adr_i*(width/MB_SIZE)+adr_j) = 1;
		}

	if( mb_type >= 0 && mb_type <= 4 ) amvbits = 0;

  if( mb_type == -1 )
    for( v=0; v<MB_SIZE; v++ ) for( h=0; h<MB_SIZE; h++ )
      alpha_mb[v*MB_SIZE+h] = BINARISE(alpha_mb_comp[v*MB_SIZE+h]);
  else
		{
    	for( v=0; v<MB_SIZE; v++ ) for( h=0; h<MB_SIZE; h++ )
      	alpha_mb[v*MB_SIZE+h] = f8bit(alpha_mb[v*MB_SIZE+h]);
		}

	/* Encode the first_shape_code */

	MakeMBtype_HHC_Studio( mb_type, mvda, adr_i, adr_j, rec_curr,
			      vopstr, fip);
	
	if( flag == 1 ) 
		{
			*first_bits = CodeInterMBtype_HHC_Studio( adr_i, adr_j, first_shape_code,rec_curr,rec_prev);
		}
	else
		{
			if (!error_res_disable)
				*first_bits = CodeIntraMBtype_ER( first_shape_code, mb_type, 
												adr_i, adr_j, rec_curr, 
 												alpha_mode_packet);
			else
			  /*v010113_0*/
			  *first_bits = CodeIntraMBtype_IntraReset_Studio(first_shape_code, mb_type, 
									  adr_i, adr_j, rec_curr,
									  vopstr, fip,
									  GetVopPredictionType_st(curr,fip)==I_VOP&&
									  GetVopIntraPredictorsReset_st(curr,fip));
#if 0
			  *first_bits = CodeIntraMBtype_Studio( first_shape_code, mb_type, 
							       adr_i, adr_j, rec_curr,
							       vopstr, fip);
#endif 0
		}

  /* update the decoded alpha image */
  for( v=0; v<MB_SIZE; v++ ) for( h=0; h<MB_SIZE; h++ ) {
    a_mb_rec[v*MB_SIZE+h] = (SInt)alpha_mb[v*MB_SIZE+h];
  }

  if(shape_inter==0 && mb_type==-1){
    printf(" Error: shape_inter=%d however mb_type=%d\n",
	    shape_inter,mb_type);
    exit(0);
  }

	/* do all bitstream book-keeping */

  /* first_shape_code bit stream */
  for(i=0; i<*first_bits; i++)
    first_stream[i] = first_shape_code[i];

	total_first_bits[vol_id] += *first_bits;
	curr_first_bits += *first_bits;

  /* second_code bit stream */
  mb_bits+=second_bits;
  for(i=0; i<second_bits; i++)
    shape_stream[i] = second_code[i];
  shape_stream+=second_bits;

	total_second_bits[vol_id] += second_bits;

  /* mv bit stream */
  mb_bits+=amvbits;
  for(i=0; i<amvbits; i++)
    shape_stream[i] = mv_stream[i];
  shape_stream+=amvbits;

	total_amv_bits[vol_id] += amvbits;
	curr_amv_bits += amvbits;

  /* cae bit stream */
  mb_bits+=cae_bits;

  for(i=0; i<cae_bits; i++)
    shape_stream[i] = cae_stream[i];
  shape_stream+=cae_bits;

	if (cae_bits>0) 
		{
			total_coded_mbs[vol_id]++;
			curr_coded_mbs++;
			total_cae_bits[vol_id] += cae_bits;
			curr_cae_bits += cae_bits;
			if (mb_type>=2 && mb_type<=4)
				{
					total_coded_intra_mbs[vol_id]++;
					curr_coded_intra_mbs++;
					total_cae_intra_bits[vol_id] += cae_bits;
					curr_cae_intra_bits += cae_bits;
				}
			else
				{
					total_coded_inter_mbs[vol_id]++;
					curr_coded_inter_mbs++;
					total_cae_inter_bits[vol_id] += cae_bits;
					curr_cae_inter_bits += cae_bits;
				}
		}

	/* write the coded BAB to the reconstructed VOP */

  PutSubImage_Studio(GetVopA2_st(rec_curr,binum),vopstr,fip,mblock_rec,x,y);
	if (!error_res_disable)
  	PutSubImage(alpha_rec_packet,mblock_rec,x,y);

#if 0 /*Is the following routine necessary???*/  
  block_rec = AllocImage(8,8,SHORT_TYPE);				
  a_b_rec = (SInt *) GetImageData(block_rec);
  for( v=0; v<8; v++ ) for( h=0; h<8; h++ )
		{
			Int index = (v+v)*MB_SIZE+h+h;   
			a_b_rec[v*8+h] = (a_mb_rec[index]
					  ||a_mb_rec[index+1]
					  ||a_mb_rec[index+MB_SIZE]
					  ||a_mb_rec[index+MB_SIZE+1])?255:0;
		}

  PutSubImage(GetVopAuv(rec_curr),block_rec,x/2,y/2);
  FreeImage (block_rec);
#endif 0

	/* modify the modes and MVs based on coded shape */

#if 0
	if ((GetVopPredictionType(curr) != I_VOP) && (GetVopShape(curr)!=BINARY_SHAPE_ONLY))  /* BSO_NOEL */
		ModifyModesAndMVs(GetVopPredictionType(curr),
                                  mblock_rec,MB_decisions,alpha_decisions,
				  mot_x,mot_y,adr_j,adr_i);
#endif 0
  FreeImage (mblock);
  FreeImage (mblock_rec);
/* Free displaced here : for leakage problem: Dufour 15.04.98 */
FreeImage(top_border_data);
FreeImage(left_border_data);

  return mb_bits;
}

Int
ShapeInterMB_HHC_Studio(Vop *curr,
			Vop *rec_curr,/*001229*/
			Vop *rec_prev,
			SInt pred[MB_SIZE][MB_SIZE],
			Image *modeA,
			Image *motA_x,
			Image *motA_y,
			Int *mvda,
			Int *shape_inter,
			UChar *alphaMV_stream,
			Int thresh,
			Int width,
			Int i,
			Int j,
			Int binum)
{
  Image
    *mblock;

  SInt
    *a_mb;

  Int
    ox,oy,
    x, y,
    mb_size=MB_SIZE,
    amvbits=0;
  
  Int
    motDA_x[4], motDA_y[4];

  Int vopstr = GetVopVopStructure_st(curr);
  Int fip = CalcFieldParity_Studio(vopstr,
				   GetVopVopCounter_st(curr),
				   GetVopTopFieldFirst(curr));
  SInt *a_ref[2];
  Int ref_vopstr, ref_fip;
  Int ref_width, ref_height;
  Int k;
  
  mblock = AllocImage(16,16,SHORT_TYPE);
  a_mb = (SInt *) GetImageData(mblock);

  if (GetVopPredictionType_st(curr,fip) == I_VOP || (GetVopShapeCodingType_st(curr,fip)==0)) {
    *shape_inter = 0;
  } else {
    *shape_inter = 1;
  }
  x = j * mb_size;
  y = i * mb_size;
  *mvda = 1;

  if (*shape_inter ) {

    GetSubImage_Studio(GetVopA2_st(curr,binum),vopstr,fip,
		       mblock,x,y);
    /*GetSubImage(GetVopA(curr),mblock,x,y);*/

    if(GetVopVopStructure_st(curr)==FIELD_VOP &&
       GetVopVopCounter_st(curr) &&
       GetVopPredictionType_st(curr,!fip)==I_VOP) {

      ox = GetVopHorSpatRef_st(curr,fip) - GetVopHorSpatRef_st(curr,!fip);
      oy = GetVopVerSpatRef_st(curr,fip) - GetVopVerSpatRef_st(curr,!fip);

      for(k=0;k<2;k++)
	a_ref[k] = (SInt *)GetImageData(GetVopA_st(curr,binum,k));

      ref_vopstr = FIELD_VOP;
      ref_fip = !fip;
      ref_width = GetVopWidth_st(curr,!fip);
      ref_height = GetVopHeight_st(curr,!fip);
      
    } else {
      
      ox = GetVopHorSpatRef_st(curr,fip) - GetVopHorSpatRef_st(rec_prev,fip);
      oy = GetVopVerSpatRef_st(curr,fip) - GetVopVerSpatRef_st(rec_prev,fip);

      for(k=0;k<2;k++)
	a_ref[k] = (SInt *)GetImageData(GetVopA_st(rec_prev,binum,k));
      
      ref_vopstr = GetVopVopStructure_st(rec_prev);
      ref_fip = fip;
      ref_width = GetVopWidth_st(rec_prev,fip);
      ref_height = GetVopHeight_st(rec_prev,fip);
      
    }

    amvbits = FindPredAlphaAndMVmei_HHC_Studio (curr,
						x,y,
						ox,oy,
						motA_x,motA_y,
						modeA,
						a_ref,/*(SInt *)GetImageData(GetVopA(rec_prev)),*//*v001229*/
						(SInt *)pred,
						a_mb,
						width,
						ref_vopstr,/*v001229*/
						ref_fip,/*v001229*/
						ref_width,/*GetImageSizeX(GetVopA(rec_prev)),*//*v001229*/
						ref_height,/*GetImageSizeY(GetVopA(rec_prev)),*//*v001229*/
						mb_size,
						thresh,
						motDA_x,motDA_y,
						mvda,
						alphaMV_stream,
						MBnum ); /* bug fix by Noboru Yamaguchi, 16.09.98 SB */
  }
  
  FreeImage (mblock);
  return amvbits;
}

Int	InferiorSymbolDecision_Studio( UChar alpha_mb[], Int size )
{
	Int	obj_count = 0;
	Int	zero_count = 0;
	Int	loop_count = size*size;
	Int	ret;

	Int i;

	for( i=0 ; i<loop_count ; i++ )
		if ( alpha_mb[i] == 255 )
			obj_count++;
	for( i=0 ; i<loop_count ; i++ )
		if ( alpha_mb[i] == 0 )
			zero_count++;

	if ( obj_count < zero_count )
		ret = 1;
	else
		ret = 0;

	return	ret;
}

Int	LineScanDirection_Studio( Int inferior, UChar alpha_block[] )
{
	Int	symbol;
	Int	hori_cnt, vert_cnt, ret;

	Int i, j;
	
	if ( inferior )		symbol = 255;
	else			symbol = 0;

	/* horizontal scan */
	hori_cnt = 0;
	for( i=0 ; i<8 ; i++ )
		for( j=0 ; j<8 ; j++ )
			if ( alpha_block[i*8+j] == symbol )
			{
				hori_cnt++;
				break;
			}

	/* vertical scan */
	vert_cnt = 0;
	for( i=0 ; i<8 ; i++ )
		for( j=0 ; j<8 ; j++ )
			if ( alpha_block[i+8*j] == symbol )
			{
				vert_cnt++;
				break;
			}

	if ( hori_cnt <= vert_cnt )	ret = 0;	/* horizontal */
	else				ret = 1;	/* vertical */

	return	ret;
}

Int	LineReverseCheck_Studio( Int symbol, Int direction, UChar alpha[] )
{
	Int	ret = 0;
	Int	forward, backward;

	Int i;
	
	forward = backward = 0;
	if ( direction )	/* vertical */
	{
		for( i=0 ; i<8 ; i++ )
			if ( alpha[i*8] == symbol )
			{
				if ( i<4 )	forward++;
				else		backward++;
			}
	}
	else			/* horizontal */
	{
		for( i=0 ; i<8 ; i++ )
			if ( alpha[i] == symbol )
			{
				if ( i<4 )	forward++;
				else		backward++;
			}
	}

	if ( backward > forward )	ret = 1;
	if ( backward == 0 && forward == 0 )	ret = 2;
	if ( backward == 4 && forward == 4 )	ret = 2;

	return	ret;
}

Int	LineReverseFlagDecision_Studio( Int inferior, Int linedirection, UChar alpha_block[] )
{
	Int	symbol;
	Int	line_reverse_cnt = 0;
	Int	nouse_cnt = 0;
	Int	ret = 0;
	Int	lrc_ret = 0;

	Int i;
	
	if ( inferior )		symbol = 255;
	else			symbol = 0;

	if ( linedirection )	/* vertical */
	{
		for( i=0 ; i<8 ; i++ )
		{
			lrc_ret = LineReverseCheck_Studio( symbol, 1, alpha_block+i );
			if ( lrc_ret == 1 )
				line_reverse_cnt++;
			if ( lrc_ret == 2 )
				nouse_cnt++;
		}
	}
	else			/* horizontal */
	{
		for( i=0 ; i<8 ; i++ )
		{
			lrc_ret =  LineReverseCheck_Studio( symbol, 0, alpha_block+i*8 );
			if ( lrc_ret == 1 )
				line_reverse_cnt++;
			if ( lrc_ret == 2 )
				nouse_cnt++;
		}
	}

	if ( line_reverse_cnt > ((8-nouse_cnt)/2) )	ret = 1;

	return	ret;
}

Int	CodedBoundaryBlockPatternDecision_Studio( Int inferior, UChar alpha[] )
{
	Int	symbol;
	Int	CBBP = 0;
	Int	alpha_block[4];

	Int i, j;
	
	if ( inferior )		symbol = 255;
	else			symbol = 0;

	alpha_block[0] = 0;
	for( i=0 ; i<8 ; i++ )
	for( j=0 ; j<8 ; j++ )
		if ( alpha[i*MB_SIZE+j] == symbol )
			alpha_block[0] += 1;

	alpha_block[1] = 0;
	for( i=0 ; i<8 ; i++ )
	for( j=0 ; j<8 ; j++ )
		if ( alpha[i*MB_SIZE+j+8] == symbol )
			alpha_block[1] += 1;

	alpha_block[2] = 0;
	for( i=0 ; i<8 ; i++ )
	for( j=0 ; j<8 ; j++ )
		if ( alpha[i*MB_SIZE+j+128] == symbol )
			alpha_block[2] += 1;

	alpha_block[3] = 0;
	for( i=0 ; i<8 ; i++ )
	for( j=0 ; j<8 ; j++ )
		if ( alpha[i*MB_SIZE+j+136] == symbol )
			alpha_block[3] += 1;

	for( i=0 ; i<4 ; i++ )
	{
		CBBP <<= 1;
		if ( alpha_block[i] )
			CBBP |= 1;
	}

	return	CBBP;
}

Int	CodedLinePattern_Studio( Int no, Int line_direction, Int inferior, UChar alpha_mb[] )
{
	Int	symbol;
	UChar*	alpha_ptr;
	Int	step1, step2;
	Int	CLP = 0;

	Int i, j;
	
	if ( inferior )		symbol = 255;
	else			symbol = 0;
	if ( line_direction )
	{
		step1 = MB_SIZE;
		step2 = 1;
	}
	else
	{
		step1 = 1;
		step2 = MB_SIZE;
	}
	alpha_ptr = alpha_mb+(128*(no/2)+8*(no%2));
	for( i=0 ; i<8 ; i++ )
	{
		CLP <<= 1;
		for( j=0 ; j<8 ; j++ )
			if ( alpha_ptr[j*step1] == symbol )
			{
				CLP |= 1;
				break;
			}
		alpha_ptr += step2;
	}

	return	CLP;
}

Int	LineBitPattern_Studio( Int no, Int lno, Int inferior, Int line_direction,
			Int line_reverse, UChar alpha_mb[] )
{
	Int	symbol;
	UChar*	alpha_ptr;
	Int	LBP = 0;
	Int	tmpLBP = 0;
	Int	step;

	Int i;
	
	if ( inferior )		symbol = 255;
	else			symbol = 0;
	alpha_ptr = alpha_mb+(128*(no/2)+8*(no%2));
	if ( line_direction )
	{
		alpha_ptr += lno;
		step = MB_SIZE;
	}
	else
	{
		alpha_ptr += lno*MB_SIZE;
		step = 1;
	}
	for ( i=0 ; i<8 ; i++ )
	{
		LBP <<= 1;
		if ( alpha_ptr[i*step] == symbol )
			LBP |= 1;
	}
	if ( line_reverse )
	{
		for ( i=0 ; i<8 ; i++ )
		{
			tmpLBP <<= 1;
			if ( LBP & (0x1<<i) )
				tmpLBP |= 1;
		}
		LBP = tmpLBP;
	}

	return	LBP;
}

Void	SetToBlock_Studio( Int no, UChar alpha_mb[], UChar alpha_block[] )
{
	Int	offset=0;

	Int i, j;
	
	if ( no == 0 )	offset = 0;
	else if ( no == 1 )	offset = 8;
	else if ( no == 2 )	offset = 128;
	else if ( no == 3 )	offset = 136;

	for( i=0 ; i<8 ; i++ )
		for( j=0 ; j<8 ; j++ )
			alpha_block[i*8+j] = alpha_mb[offset+i*16+j];

}
/* 01.04.01: Studio Profile */
