/******************************************************************************
 *                                                                          
 * This software module was originally developed by 
 *
 * Robert Danielsen (Telenor / ACTS-MoMuSys). 	    	              	
 *
 * and edited by 
 *
 * Luis Ducla-Soares (IST / ACTS-MoMuSys).     	           	   	
 * Cor Quist (KPN / ACTS-MoMuSys).
 * Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4 
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free 
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original 
 * developer of this software module and his/her company, the subsequent 
 * editors and their companies, and ISO/IEC have no liability for use of this 
 * software module or modifications thereof in an implementation. Copyright is 
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) standard conforming 
 * products. 
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own 
 * purpose, assign or donate the code to a third party and to inhibit third 
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works. 
 *
 * Copyright (c) 1997
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	putvlc_studio.c
 *
 * Author:	Osamu Sunohara(SONY)
 * Created:	01.04.01
 *                                                                         
 * Description: Functions for writing to bitstream
 *
 * Notes: 	Same kind of tables as in the MPEG-2 software simulation
 *		group software.
 *
 * Modified:
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include "momusys.h"
#include "vlc_studio.h"
#include "putvlc_studio.h"
#include "mom_bitstream_i.h"

/***********************************************************CommentBegin******
 *
 * -- Putxxx -- Write bits from huffman tables to file
 *
 *	Int PutDCsize_chrom_Studio_bySVLC (Int size, Image *bitstream)
 *	Int PutDCsize_lum_Studio_bySVLC (Int size, Image *bitstream)
 *      Int PutGroupVLC_Studio_bySVLC (Int vlc_table, Int group, Image *bitstream)
 *
 * Author :	Osamu Sunohara(SONY)
 * Created :	01.04.01
 *
 * Purpose :		
 *	Writes bits from huffman tables to bitstream
 * 
 * Arguments in : 	
 *	various, see prototypes above
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	Number of bits written
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :
 *
 ***********************************************************CommentEnd********/

Int
PutDCsize_lum_Studio_bySVLC (Int size, Image *bitstream) {
  MOMCHECK(size >= 0 && size <= 18/*13*/);/*10+3*//*v001225*/

  BitstreamPutBits (bitstream, DCtab_lum_studio[size].code, DCtab_lum_studio[size].len);

   return DCtab_lum_studio[size].len; 
}

Int
PutDCsize_chrom_Studio_bySVLC (Int size, Image *bitstream) {
  MOMCHECK (size >= 0 && size <= 18/*13*/);/*v001225*/

  BitstreamPutBits (bitstream, DCtab_chrom_studio[size].code, DCtab_chrom_studio[size].len);

  return DCtab_chrom_studio[size].len;
}

Int 
PutGroupVLC_Studio_bySVLC2(Int intra, Int vlc_table, Int group, Image *bitstream) {
  Int length = 0;
  
  MOMCHECK (vlc_table==0 || vlc_table==8 || (vlc_table>=16&&vlc_table<=25));
  MOMCHECK ((group>=0&&group<=6) || (group>=8&&group<=13) || (group>=17&&group<=25));

  if(intra) {    
    if(studio_coeff[vlc_table][group].len==0) {
      fprintf(stderr, "0 byte vlc\n");
      exit(-6);
    }
    
    BitstreamPutBits (bitstream,
		      studio_coeff[vlc_table][group].code,
		      studio_coeff[vlc_table][group].len);
    length = studio_coeff[vlc_table][group].len;
    
  } else {
    if(studio_coeff_inter[vlc_table][group].len==0) {
      fprintf(stderr, "0 byte vlc inter\n");
      exit(-6);
    }
    
    BitstreamPutBits (bitstream,
		      studio_coeff_inter[vlc_table][group].code,
		      studio_coeff_inter[vlc_table][group].len);
    length = studio_coeff_inter[vlc_table][group].len;
    
  }

  return length;  
}


