/******************************************************************************
 *                                                                          
 * This software module was originally developed by 
 *
 * Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4 
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free 
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original 
 * developer of this software module and his/her company, the subsequent 
 * editors and their companies, and ISO/IEC have no liability for use of this 
 * software module or modifications thereof in an implementation. Copyright is 
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) standard conforming 
 * products. 
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own 
 * purpose, assign or donate the code to a third party and to inhibit third 
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works. 
 *
 * Copyright (c) 1997
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	studio_function_enc.c
 *
 * Author:	Osamu Sunohara(SONY)
 * Created:	01.04.01
 *                                                                         
 * Description:
 *
 * Notes: 	
 *
 * Modified:
 *
 ***********************************************************HeaderEnd*********/
#include "momusys.h"

#include "mom_bitstream_i.h"
#include "vm_common_defs.h"
#include "studio_function_enc.h"
Void CalcCBPC_Studio(Int CBP, Int *pcbpc, Int *pcbpcb, Int *pcbpcr,
	      Int chroma_format, Int *transp_pattern_uv, Int method)
{
  Int cbpcm, cbpc_bl[12], cbpc45, cbpc67, cbpc8901;
  Int i, ii;
  Int conv[4] = {0, 2, 1, 3};
  
  if(chroma_format == CHROMA444)
    {
      cbpc45 = CBP & 3;
      cbpc45 = cbpc45 << 6;
      cbpc67 = CBP >> 2;
      cbpc67 = cbpc67 & 48;
      cbpc8901 = CBP >> 8;
      cbpc8901 = cbpc8901 & 15;
      cbpcm = cbpc45 | cbpc67 | cbpc8901;
      
      for(i=4;i<12;i++)
	{
	  cbpc_bl[i] = cbpcm >> (11 - i);
	  cbpc_bl[i] = cbpc_bl[i] & 1;
	}

      if(method == 1)
	{
	  *pcbpcb = (cbpc_bl[4]<<3)|(cbpc_bl[6]<<2)|(cbpc_bl[8]<<1)|cbpc_bl[10];
	  *pcbpcr = (cbpc_bl[5]<<3)|(cbpc_bl[7]<<2)|(cbpc_bl[9]<<1)|cbpc_bl[11];
	  
	  *pcbpc = ( ((*pcbpcb ? 1:0)<<1) | (*pcbpcr ? 1:0) ) & 3;
	}
      else if(method == 2)
	{
	  *pcbpcb = (cbpc_bl[4]<<3)|(cbpc_bl[6]<<2)|(cbpc_bl[8]<<1)|cbpc_bl[10];
	  *pcbpcr = (cbpc_bl[5]<<3)|(cbpc_bl[7]<<2)|(cbpc_bl[9]<<1)|cbpc_bl[11];

	  for(ii=0;ii<4;ii++) {
	    if(transp_pattern_uv[conv[ii]]!=1) {
	      *pcbpc = ((cbpc_bl[4+2*ii]<<1) | (cbpc_bl[5+2*ii]));
	      break;
	    }
	  }
	}
      else
	{
	  fprintf(stderr,"method has the wrong value[%d].(CalcCBPC)\n",method);
	  exit(-1);
	}
    }
  else if(chroma_format == CHROMA422)
    {
      cbpc45 = CBP & 3;
      cbpc45 = cbpc45 << 2;
      cbpc67 = CBP >> 6;
      cbpc67 = cbpc67 & 3;
      cbpcm = cbpc45 | cbpc67;
      
      for(i=4;i<8;i++)
	{
	  cbpc_bl[i] = cbpcm >> (7 - i);
	  cbpc_bl[i] = cbpc_bl[i] & 1;
	}

      if(method == 1)
	{
	  *pcbpcb = (cbpc_bl[4]<<1)|cbpc_bl[6];
	  *pcbpcr = (cbpc_bl[5]<<1)|cbpc_bl[7];
	  
	  *pcbpc =  ( ((*pcbpcb ? 1:0)<<1) | (*pcbpcr ? 1:0) ) & 3;
	}
      else if(method == 2)
	{
	  *pcbpcb = (cbpc_bl[4]<<1)|cbpc_bl[6];
	  *pcbpcr = (cbpc_bl[5]<<1)|cbpc_bl[7];
	  
	  for(ii=0;ii<2;ii++) {
	    if(transp_pattern_uv[conv[ii]]==0) {
	      *pcbpc = ((cbpc_bl[4+2*ii]<<1) | (cbpc_bl[5+2*ii]));
	      break;
	    }
	  }

	}
      else
	{
	  fprintf(stderr,"method has the wrong value[%d].(CalcCBPC)\n",method);
	  exit(-1);
	}
    }
  else if(chroma_format == CHROMA420)
    {
      *pcbpc = CBP & 3; /* last 2 bits */
      *pcbpcb = 0;
      *pcbpcr = 0;
    }
  else
    {
      fprintf(stderr,"chroma_format has the wrong value[%d].(CalcCBPC)\n",chroma_format);
      exit(-1);
    }
}

Int PutExtendedCBPC_Studio(Int cbpcb, Int cbpcr, Image *mottext_bitstream,
		     Int *MB_transp_pattern_uv, Int chroma_format, Int method)
{

  Int conv[4] = {0,2,1,3};
  Int bit_cb[4] = {0,0,0,0}, bit_cr[4] = {0,0,0,0};
  Int cbpcb_st=0, cbpcr_st=0;
  Int inc=0;
  Int i;

  if(chroma_format == CHROMA444)
    {
      if(method==1)
	{
	  for(i=0;i<4;i++)
	    {
	      if(MB_transp_pattern_uv[conv[i]] != 1)
		{
		  inc++;
		  bit_cb[inc-1] = (cbpcb>>(3-i))&1;
		  bit_cr[inc-1] = (cbpcr>>(3-i))&1;
		}
	    }
	  for(i=0;i<inc;i++)
	    {
	      cbpcb_st = cbpcb_st | (bit_cb[i]<<(inc-i-1));
	      cbpcr_st = cbpcr_st | (bit_cr[i]<<(inc-i-1));
	    }

	  if((cbpcb==0&&cbpcb_st!=0)||(cbpcb!=0&&cbpcb_st==0))
	    {
	      fprintf(stderr,"this calclation has error!(PutExtendedCBPC().CHROMA444)\n");
	      exit(-1);
	    }

	  if(cbpcb)
	    BitstreamPutBits(mottext_bitstream, (long) cbpcb_st, inc);
	  if(cbpcr)
	    BitstreamPutBits(mottext_bitstream, (long) cbpcr_st, inc);
	} else if(method == 2) {
	  Int first = 1;

	  for(i=0;i<4;i++)
	    {
	      if(MB_transp_pattern_uv[conv[i]] != 1)
		{
		  if(first==1)
		    first=0;
		  else {
		    BitstreamPutBits(mottext_bitstream, (long) ((cbpcb>>(3-i))&1), 1);
		    BitstreamPutBits(mottext_bitstream, (long) ((cbpcr>>(3-i))&1), 1);
		    inc++;
		  }
		}
	    }
	}
    }
  else if(chroma_format == CHROMA422)
    {
      if(method==1)
	{
	  for(i=0;i<2;i++)
	    {
	      if(MB_transp_pattern_uv[conv[i]] != 1)
		{
		  inc++;
		  bit_cb[inc-1] = (cbpcb>>(1-i))&1;
		  bit_cr[inc-1] = (cbpcr>>(1-i))&1;
		}
	    }

	  for(i=0;i<inc;i++)
	    {
	      cbpcb_st = cbpcb_st | (bit_cb[i]<<(inc-i-1));
	      cbpcr_st = cbpcr_st | (bit_cr[i]<<(inc-i-1));
	    }

	  if((cbpcb==0&&cbpcb_st!=0)||(cbpcb!=0&&cbpcb_st==0))
	    {
	      fprintf(stderr,"this calclation has error!(PutExtendedCBPC().CHROMA422)\n");
	      exit(-1);
	    }
	  if(cbpcb)
	    BitstreamPutBits(mottext_bitstream, (long) cbpcb_st, inc);
	  if(cbpcr)
	    BitstreamPutBits(mottext_bitstream, (long) cbpcr_st, inc);
	} else if(method == 2) {
	  Int first = 1;

	  for(i=0;i<2;i++)
	    {
	      if(MB_transp_pattern_uv[conv[i]] != 1)
		{
		  if(first==1)
		    first=0;
		  else {
		    BitstreamPutBits(mottext_bitstream, (long) ((cbpcb>>(1-i))&1), 1);
		    BitstreamPutBits(mottext_bitstream, (long) ((cbpcr>>(1-i))&1), 1);
		    inc++;

		  }
		}
	    }
	}
    }
  else if(chroma_format == CHROMA420)
    ;
  else
    {
      fprintf(stderr,"chroma_format has the wrong value[%d].(PutExtendedCBPC)\n",chroma_format);
      exit(-1);
    }

  return inc;
}
