/******************************************************************************
 *                                                                          
 * This software module was originally developed by 
 *
 * Robert Danielsen (Telenor / ACTS-MoMuSys). 	    	              	
 *
 * and edited by 
 *
 * Cor Quist (KPN / ACTS-MoMuSys).     			              	
 * Minhua Zhou (HHI / ACTS-MoMuSys).   	  	 	              	
 * Luis Ducla-Soares (IST / ACTS-MoMuSys)
 * Minhua Zhou (HHI / ACTS-MoMuSys)
 * Bob Eifrig (NextLevel Systems).     	           	   	
 * Michael Frater (UNSW)
 * Sven Brandau (HHI)
 * Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4 
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free 
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original 
 * developer of this software module and his/her company, the subsequent 
 * editors and their companies, and ISO/IEC have no liability for use of this 
 * software module or modifications thereof in an implementation. Copyright is 
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) standard conforming 
 * products. 
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own 
 * purpose, assign or donate the code to a third party and to inhibit third 
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works. 
 *
 * Copyright (c) 1997
 *
 *****************************************************************************/


/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	text_bits_studio.c
 *
 * Author:	Osamu Sunohara(SONY)
 * Created:	01.04.01
 *                                                                         
 * Description:	texture encoding
 *
 * Notes: 	
 *
 * Modified:  	
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include "momusys.h"
#include "text_defs.h"
#include "mom_bitstream_i.h"
#include "text_bits.h"
#include "text_bits_studio.h"
#include "putvlc_studio.h"
#include "text_util.h"
#include "zigzag.h"		/* added, 14-NOV-1996 MW */
#include "max_level.h"          /* 3-mode esc */
#include "startcode.h"		/* added by SB 10.08.98 */

/***********************************************************CommentBegin******
 *
 * -- MB_CodeCoeff_Studio_bySVLC -- Codes coefficients by Switching VLC tables
 *
 * Author :	Osamu Sunohara(SONY)
 * Created :	01.04.01
 *
 * Purpose :		
 *	Codes coefficients
 * 
 * Arguments in :
 *	Int *qcoeff : quantized dct-coefficients
 * 	Int Mode : encoding mode
 * 	Int CBP : coded block pattern
 * 	Int ncoeffs : number of coefficients per block
 * 	Int x_pos : the horizontal position of the macroblock in the vop
 *      Int intra_dcpred_disable : disable the intradc prediction
 *	Int transp_pattern[]:	Describes which blocks are transparent
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	Bits *bits : struct to count the number of texture bits 
 * 	Image *bitstream : output bitstream
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :
 *	The intradc prediction can be switched off by setting the variable
 *	intradc_pred_disable to '1'.
 *
 * See also :
 *	
 *
 * Modified :
 *
 ***********************************************************CommentEnd********/
Void MB_CodeCoeff_Studio_bySVLC(Bits* bits, Int *qcoeff,
				Int Mode, Int CBP, Int ncoeffs, 
				Image *bitstream,
				Int transp_pattern[],
				Int transp_pattern_uv[],
				Int alternate_scan,
				Int chroma_format,
				Int flc_length/*v001225*/)
{
    Int coeff[64];
    Int *zz = alternate_scan ? zigzag_v : zigzag;
    Int i, m;
   	
    Int block_count;
    Int cnv[4]={0,2,1,3};
    Int val_tmp;
    
    if(chroma_format==CHROMA444)
      block_count = 12;
    else if(chroma_format==CHROMA422)
      block_count = 8;
    else
      block_count = 6;

    if (Mode == MODE_INTRA || Mode == MODE_INTRA_Q) {
      for (i = 0; i < block_count; i++) {
	if(((i<4)&&(transp_pattern[i]!=1))
	   ||((i>3)&&(transp_pattern_uv[cnv[i/2-2]]!=1))) { /* Not transparent */
	  
	  /* Do the zigzag scanning of coefficients */
	  for (m=0; m<64; m++) {
	      *(coeff + zz[m]) = qcoeff[i*ncoeffs+m];
	    }

	  /* Code an intra DC coeff */
	  if (i < 4) {
	    bits->Y += (val_tmp = IntraDC_dpcm_Studio_bySVLC(coeff[0],1,bitstream));
	    bits->Ydc += val_tmp;
	  } else {
	    bits->C += (val_tmp = IntraDC_dpcm_Studio_bySVLC(coeff[0],0,bitstream));
	    bits->Cdc += val_tmp;
	      if(i%2==0)
		bits->Cb_dc += val_tmp;
	      else
		bits->Cr_dc += val_tmp;
	  }
	  
	  /* Code AC coeffs */
	  if (i < 4) {
	    bits->Y += (val_tmp = CodeCoeff_Studio_bySVLC2(1,Mode,coeff,i,ncoeffs,bitstream,flc_length/*v001225*/));
	    bits->Yac += val_tmp;
	  } else {
	    bits->C += (val_tmp = CodeCoeff_Studio_bySVLC2(1,Mode, coeff,i,ncoeffs,bitstream,flc_length/*v001225*/));
	    bits->Cac += val_tmp;
	    if(i%2==0)
	      bits->Cb_ac += val_tmp;
	    else
	      bits->Cr_ac += val_tmp;
	  }
	} /*transp_pattern*/
      } /*for (i = 0; i < block_count; i++)*/
    } else { 	/* inter block encoding */
      
      for (i = 0; i < block_count; i++) 
	{
	  /* Do the zigzag scanning of coefficients */
	  for (m = 0; m < 64; m++)
	    *(coeff + zz[m]) = qcoeff[i*ncoeffs+m];
	  if ((i==0 && CBP&32) || 
	      (i==1 && CBP&16) ||
	      (i==2 && CBP&8) || 
	      (i==3 && CBP&4) ||
	      (i==4 && CBP&2) ||
	      (i==5 && CBP&1) ||
	      (i==6 && CBP&128) || 
	      (i==7 && CBP&64) ||
	      (i==8 && CBP&2048)  || 
	      (i==9 && CBP&1024)  ||
	      (i==10 && CBP&512)  ||
	      (i==11 && CBP&256)) {
	    
	    if (i < 4) {
/*	      bits->Y += (*/val_tmp = CodeCoeff_Studio_bySVLC2(0,Mode,coeff,i,ncoeffs,bitstream,flc_length/*v001225*/)/*)*/;
	      bits->Yac_inter += val_tmp;
	    } else {
/*	      bits->C += (*/val_tmp = CodeCoeff_Studio_bySVLC2(0,Mode, coeff,i,ncoeffs,bitstream,flc_length/*v001225*/)/*)*/;
	      bits->Cac_inter += val_tmp;
	      if(i%2==0)
		bits->Cb_ac_inter += val_tmp;
	      else
		bits->Cr_ac_inter += val_tmp;
	    }

	  }
	}
    }
  }

/***********************************************************CommentBegin******
 *
 * -- IntraDC_dpcm_Studio_bySVLC -- DPCM Encoding of INTRADC in case of Intra macroblocks
 *
 * Author : Cor Quist (KPN)		
 *	
 *
 * Created :		
 *	
 *
 * Purpose :		
 *	DPCM Encoding of INTRADC in case of Intra macroblocks
 * 
 * Arguments in : 	
 *	Int val : the difference value with respect to the previous
 *		INTRADC value
 * 	Int lum : indicates whether the block is a luminance block (lum=1) or 
 * 		a chrominance block ( lum = 0)
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	Image* bitstream  : a pointer to the output bitstream
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :		
 *	07.06.96: Robert Danielsen: Changed from using Encode to Putxxx
 *		instead, based on new VLC encoding scheme.
 *      01.04.01 Osamu Sunohara (SONY) : For DC size extension
 *	
 *
 ***********************************************************CommentEnd********/

Int 
IntraDC_dpcm_Studio_bySVLC(Int val, Int lum, Image *bitstream)
{
    Int n_bits;
    Int absval, size = 0;
	
    absval = (val<0) ? -val : val;
    
    /* compute dct_dc_size */
    size = 0;
    while(absval)
    {
	absval>>=1;
	size++;
    }
	
    if (lum) {	/* luminance */
      n_bits = PutDCsize_lum_Studio_bySVLC (size, bitstream);
    }
    else {	/* chrominance */
      n_bits = PutDCsize_chrom_Studio_bySVLC (size, bitstream);
    }
    
    if ( size != 0 ) {
      if (val>=0) {
	;
      } else {
	absval = -val;
	val = (absval ^( (int)pow(2.0,(double)size)-1) );
      }
      BitstreamPutBits(bitstream, (long)val, (long)size);
      n_bits += size;
      
      if (size > 8) {
	BitstreamPutBits(bitstream, (long)1, (long)1);
	n_bits++;
      }
    }
    
    return n_bits;	/* # bits for intra_dc dpcm */
	
}

/***********************************************************CommentBegin******
 *
 * -- CodeCoeff_Studio_bySVLC -- VLC encoding of quantized DCT coefficients
 *                               by using a switching VLC scheme
 *
 * Author :		
 *	
 *
 * Created :		
 *	
 *
 * Purpose :		
 *	VLC encoding of quantized DCT coefficients, except for 
 *      INTRADC in case of Intra macroblocks
 *	Used by Bits_CountCoeff
 * 
 * Arguments in : 	
 *	Int Mode : encoding mode
 * 	Int *qcoeff: pointer to quantized dct coefficients
 * 	Int block : number of the block in the macroblock (0-5)
 * 	Int ncoeffs : the number of coefficients per block
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	Image *bitstream : pointer to the output bitstream
 *
 * Return values :	
 *	Int bits : number of bits added to the bitstream (?)
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :
 *     01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/
Int CodeCoeff_Studio_bySVLC2(Int j_start, Int Mode, Int qcoeff[], Int block, Int ncoeffs, Image *bitstream, Int flc_length/*v001225*/)
{
  Int bits;
  Int next_vlc_table;
  Int next_coef;
  Int next_coef_sign, next_coef_abs;
  Int level_group, level_size;
  Int eob;
  Int zerorun_length, tmp_zerorun_length;
  Int zerorun_group, zerorun_size;
  Int level_following_zero;
  Int i, j;

  bits = 0;
  next_vlc_table = 0;

  /* 01.04.01: Studio Profile: BUGFIX */
  j = j_start;
  while (1) {

    if ( j == ncoeffs ) {
      /* Put Group VLC */
      bits += PutGroupVLC_Studio_bySVLC2(Mode == MODE_INTRA || Mode == MODE_INTRA_Q, next_vlc_table, 0, bitstream);
      next_vlc_table = 0;
      break;
    }
/*    
  for (j = j_start; j< ncoeffs; j++) {
*/

    
    next_coef = qcoeff[j];
    if(next_coef) { /* processing a non zero coef */

      next_coef_sign = (next_coef>0) ? 1 : -1;
      
      next_coef_abs = abs(next_coef);
      level_group = 0;
      while(next_coef_abs) {
	next_coef_abs>>=1;
	level_group++;
      }
      next_coef_abs = abs(next_coef);

      level_size = level_group;
      level_group += 16;

      if(level_group<=24) {
	/* Put Group VLC */
	bits += PutGroupVLC_Studio_bySVLC2(Mode == MODE_INTRA || Mode == MODE_INTRA_Q, next_vlc_table, level_group, bitstream);
	next_vlc_table = level_group;
	
	if(next_coef_sign<0)
	  next_coef_abs = (next_coef_abs^( (int)pow(2.0,(double)level_size)-1));
	
	/* Put Level FLC */
	BitstreamPutBits(bitstream, (long)next_coef_abs, (long)level_size);
	bits += level_size;
      } else { /* ESCAPE */
	bits += PutGroupVLC_Studio_bySVLC2(Mode == MODE_INTRA || Mode == MODE_INTRA_Q, next_vlc_table, 25, bitstream);
	next_vlc_table = 25;

	/* MoMuSys-2000FDAM1COR-020612 */
	if( next_coef_abs < 256 || next_coef_abs > (int)pow(2.0,(double)(flc_length-1))-1 ) {
	  fprintf(stderr, "next_coef = %d is not allowed.\n", next_coef);
	  exit(-4);
	}
	/**/
	
	/*v001225*/
/*	
	if(next_coef==0 || next_coef==-2048) {
	  fprintf(stderr, "next_coef = %d is not allowed.\n", next_coef);
	  exit(-4);
	}
*/	
	if(next_coef_sign<0)
	  next_coef_abs = (next_coef_abs^( (int)pow(2.0,(Double)flc_length/*v001225*//*14.0*/)-1)) + 1;
	
	/* Put Level FLC */
	BitstreamPutBits(bitstream, (long)next_coef_abs, (long)flc_length/*v001225*//*14*/);
	bits += flc_length/*v001225*//*14*/;
      }	
    } else { /* processing a zero run or EOB */

      if(next_vlc_table==8) {
	fprintf(stderr, "A Zero Coef must not appear when next_vlc_table is T8\n");
	exit(-1);
      }

      /* Checking the EOB */
      eob=1;
      zerorun_length=1;
      level_following_zero=1;
      if(j!=ncoeffs-1) {
	for(i=j+1;i<ncoeffs;i++) {
	  if(qcoeff[i]) {
	    eob=0;
	    level_following_zero = qcoeff[i];
	    break;
	  }
	  zerorun_length++;
	}
      }

      if(eob==0) { /* processing zero run */

	tmp_zerorun_length = zerorun_length;
	zerorun_group = 0;
	while(tmp_zerorun_length) {
	  tmp_zerorun_length>>=1;
	  zerorun_group++;
	}
	zerorun_size = zerorun_group;
	
	if(abs(level_following_zero)==1) { /* processing zerorun+1 */

	  zerorun_group += 0;
	  
	  /* Put Group VLC */
	  bits += PutGroupVLC_Studio_bySVLC2(Mode == MODE_INTRA || Mode == MODE_INTRA_Q, next_vlc_table, zerorun_group, bitstream);
	  next_vlc_table = 16;

#if 0 /***BUGFIX***/
	  tmp_zerorun_length = zerorun_length;
	  if(level_following_zero<0)
	    tmp_zerorun_length = (tmp_zerorun_length^( (int)pow(2.0,(double)zerorun_size)-1));

	  /* Put Runlength FLC */
	  BitstreamPutBits(bitstream, (long)tmp_zerorun_length, (long)zerorun_size);
#endif 0 /***BUGFIX***/

	  BitstreamPutBits(bitstream, (long)zerorun_length, (long)(zerorun_size-1));
	  if(level_following_zero>0)
	    BitstreamPutBits(bitstream, 1, 1);
	  else
	    BitstreamPutBits(bitstream, 0, 1);	  

	  
	  bits += zerorun_size;

	  j += zerorun_length;
	} else { /* processing zerorun */

	  zerorun_group += 7;
	  
	  /* Put Group VLC */
	  bits += PutGroupVLC_Studio_bySVLC2(Mode == MODE_INTRA || Mode == MODE_INTRA_Q, next_vlc_table, zerorun_group, bitstream);
	  next_vlc_table = 8;
      
	  /* Put Runlength FLC */
	  BitstreamPutBits(bitstream, (long)zerorun_length, (long)(zerorun_size-1));
	  bits += zerorun_size-1;

	  j += zerorun_length-1;
	}
      } else { /* EOB */
	
	/* Put Group VLC */
	bits += PutGroupVLC_Studio_bySVLC2(Mode == MODE_INTRA || Mode == MODE_INTRA_Q, next_vlc_table, 0, bitstream);
	next_vlc_table = 0;
	break;
      }
    }

    /* 01.04.01: Studio Profile: BUGFIX */
    j++;
  }/*j roop*/
  
    return bits;
}



