/******************************************************************************
 *                                                                          
 * This software module was originally developed by 
 *
 * Cor Quist (KPN / ACTS-MoMuSys).     			              	
 *
 * and edited by 
 *
 * Robert Danielsen (Telenor / ACTS-MoMuSys). 	    	              	
 * Noel Brady (TELTEC IRELAND / ACTS-MoMuSys). 
 * Jan De Lameillieure (HHI / ACTS-MoMuSys).     	              	
 * Michael Wollborn (TUH / ACTS-MoMuSys). 	    	              	
 * Minhua Zhou (HHI / ACTS-MoMuSys)
 * Bob Eifrig (NextLevel Systems)
 * Michael Frater (UNSW)
 * Claudia Billotet-Hoffmann (T-Berkom / ACTS-MoMuSys)
 * Jong Deuk Kim (HYUNDAI)
 * Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4 
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free 
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original 
 * developer of this software module and his/her company, the subsequent 
 * editors and their companies, and ISO/IEC have no liability for use of this 
 * software module or modifications thereof in an implementation. Copyright is 
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) standard conforming 
 * products. 
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own 
 * purpose, assign or donate the code to a third party and to inhibit third 
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works. 
 *
 * Copyright (c) 1997
 *
 *****************************************************************************/


/***********************************************************HeaderBegin*******
 *                                                                         
 * File: text_code_mb.c
 *
 * Author: Telenor, TMN
 * Created: 
 *                                                                         
 * Description: texture encoding  
 *
 * Notes:  
 *
 * Modified:
 *	Robert Danielsen, Feb 14, 1996
 *  	Cor Quist -- 26-Feb-96
 *  	Cor Quist -- 11-Apr-96
 *  	Cor Quist -- 16 April 96
 *   	21.04.96 Robert Danielsen: Reformatted. New headers.
 *   	23.09.96 Cor Quist: Added MPEG-2 DCT-function
 *	23.10.96 Robert Danielsen: Removed Dct(), also did some other
 *			cleaning up.
 *		04.02.97 Noel O'Connor: removed all calls to assert()
 *      20-MAR-97 Jan De Lameillieure : added "conservative" subsampling of chroma
 *      04.01.98 Michael Frater: support for non-8-bit video
 *      7. 3. 98 C.Billotet-Hoffmann: cleaning
 *	07.05.98 Jong Deuk Kim (jdkim97@hei.co.kr, HYUNDAI) : interlaced tools
 *      01.04.01 Osamu Sunohara (SONY): Functions for Studio Profile
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include <stdio.h>
#include <stdlib.h>
#include "text_util.h"		/* Added 19-JUN-1996, IC */
#include "text_defs.h"
#include "text_code_mb.h"
#include "mom_access.h"
#include "vm_config.h"		/* Added 20-MAR-97, JDL, because of "GRAY_SCALE" */
#include "text_bits.h"
#include "drc_util_enc.h"

/* 01.02.99 HHI Schueuer */
/* include files necessary for the SA-DCT */
#include "sadct_bsnrmem.h"
#include "sadct.h"
#include "sadct_s_k.h"
#include "sadct_momusys.h"
#include "sadct_momusys_kaup.h"
#include "sadct_momusys_s_k.h"
/* end HHI */

#ifdef _STUDIO_CONF_3_
#include "zigzag.h"
#endif _STUDIO_CONF_3_

/*#define DISPLAY_ENC_TRACE*/ /* deleted for OBSS by Samsung AIT (1999-09-29) */

#define BLOCK_SIZE 8

#ifdef _STUDIO_CONF_5_
#include <math.h>

long g_pixel[1024], g_coeff[4096];

long ie_L = 256, ie_H = 255;
int ie_sign = 1;

long ie_rand(L,H)
long L,H;
{
  static long   randx = 1;
  static double z = (double) 0x7fffffff;

  long          i, j;
  double        x;

  randx = (randx * 1103515245) + 12345;
  i = randx & 0x7ffffffe;
  x = ((double)i)/z;
  x *= (L+H+1);
  j = x;
  return(j-L);
} 
#endif _STUDIO_CONF_5_

#ifdef _STUDIO_CONF_6_
FILE *t6_fp_qcoef;
int  t6_v_qcoef[18000];
#endif _STUDIO_CONF_6_

/***********************************************************CommentBegin******
 *
 * -- CodeMB -- Code, decode and reconstruct Macroblock
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	Code, decode and reconstruct Macroblock
 * 
 * Arguments in : 	
 *	Int x_pos	x_position of Macroblock
 *	Int y_pos	y_position of Macroblock
 *	UInt width	width of Vop bounding box
 *	Int QP		Quantization parameter
 *	Int Mode	Macroblock coding mode
 *	Int transp_pattern[] Pattern of transparent blocks in MB
 *
 * Arguments in/out :	
 *	Vop *curr	current Vop, uncoded
 *	Vop *rec_curr	current Vop, decoded and reconstructed
 *
 * Arguments out :	
 *  Int *fieldDCT : field DCT permutation (if non-zero)
 *
 * Return values :	
 *	Int *		array of quantized coefficients
 *
 * Side effects :	
 *
 * Description :	
 *
 * See also :
 *
 * Modified :		
 *	27.02.97  Robert Danielsen: Added type parameter to signal luma or
 *			  chroma to the nonlinear quantization.
 *  	20-MAR-97 Jan De Lameillieure : added "conservative" subsampling
 *			  of chroma
 *	22-APR-97 Michael Wollborn: changed name of BlockQuant to 
 *			  BlockQuantH263 (and for dequant. also) and changed
 *			  stuff for MPEG-like quantization
 *  	11-08-97  Minhua Zhou: added clipping for MODE_INTRA_Q mode 
 *  	11.12.97  Bob Eifrig: Field DCT test & permutation
 *      07.05.98  Jong Deuk Kim (HYUNDAI) : all boundary MBs are coded
 *                                          as Frame DCT mode.
 *
 ***********************************************************CommentEnd********/

Int *CodeMB(Vop *curr, Vop *rec_curr, Int x_pos, Int y_pos, UInt width,
            Int QP, Int Mode, Int MB_transp_pattern[], Int *fieldDCT)
{
    Int k;
    Int fblock[6][8][8];
    Int coeff[384];
    Int *coeff_ind;
    Int *qcoeff;
    Int *qcoeff_ind;
    Int* rcoeff_ind;
    Int x, y;
    SInt *current, *recon = NULL;
    UInt xwidth;
    Int iblock[6][8][8];
    Int rcoeff[6*64];
    Int i, j;
    Int type;                   /* luma = 1, chroma = 2 */
    Int *qmat;

    /* 01.02.99 HHI Schueuer */
    /* variables for the sadct */
    Short *alpha_pntr = (Short *)(GetImageData(GetVopA(rec_curr)));
    Short alpha_block[6][BLOCK_SIZE][BLOCK_SIZE];
    Short alpha_buf[6][BLOCK_SIZE][BLOCK_SIZE];
    Short nrpix = GetImageSizeX(GetVopA(rec_curr));
    Int active_pels[6];
    Int x1, y1;
    /* end HHI */

    /* HYUNDAI 980507 : start */
    Image *Sub_Alpha= AllocImage(MB_SIZE, MB_SIZE, SHORT_TYPE);
    SInt  *sub_alpha_ptr;
    Int num_transp = 0;
 
    GetSubImage(GetVopA(rec_curr), Sub_Alpha, x_pos, y_pos);
    sub_alpha_ptr = (SInt *)GetImageData(Sub_Alpha);
    /* HYUNDAI 980507 : end */

    qcoeff = (Int *) malloc (sizeof (Int) * 384);
    coeff_ind = coeff;
    qcoeff_ind = qcoeff;
    rcoeff_ind = rcoeff;

    /* 01.02.99 HHI Schueuer */
    for (k=0;k<6;k++)
      for (i=0; i<BLOCK_SIZE; i++)
	for (j=0; j<BLOCK_SIZE; j++)
	  alpha_buf[k][i][j]=alpha_block[k][i][j]=BINARY_ALPHA;
    /* end HHI */

    for (k = 0; k < 6; k++) {
        switch (k) {
        case 0:
            x = x_pos;
            y = y_pos;
            xwidth = width;
            current = (SInt *) GetImageData (GetVopY (curr));
            break;
        case 1:
            x = x_pos + 8;
            y = y_pos;
            xwidth = width;
            current = (SInt *) GetImageData (GetVopY (curr));
            break;
        case 2:
            x = x_pos;
            y = y_pos + 8;
            xwidth = width;
            current = (SInt *) GetImageData (GetVopY (curr));
            break;
        case 3:
            x = x_pos + 8;
            y = y_pos + 8;
            xwidth = width;
            current = (SInt *) GetImageData (GetVopY (curr));
            break;
        case 4:
            x = x_pos / 2;
            y = y_pos / 2;
            xwidth = width / 2;
            current = (SInt *) GetImageData (GetVopU (curr));
            break;
        case 5:
            x = x_pos / 2;
            y = y_pos / 2;
            xwidth = width / 2;
            current = (SInt *) GetImageData (GetVopV (curr));
            break;
        default:
            fprintf (stdout, "CodeMB (): Non-existing case in switch statement\n");
            exit (-1);
            break;
        }
        BlockPredict (current, recon, x, y, xwidth, fblock[k]);
    }

    for (k = 0; k < 5; k++) {
      /* 11-02-98 Klaas Schueuer: changed to compute PSNR for Padding */
      if ((GetVopSADCTDisable(curr)==0)&&(GetVopShape(curr)!=0)) { 
	/* if (GetVopShape(curr)!=0) { */
        switch(k) {
	case 0 :
	  for (i=0; i<BLOCK_SIZE; i++)
	    for (j=0; j<BLOCK_SIZE; j++)
	      alpha_block[0][i][j] = alpha_pntr[(y_pos+i)*nrpix + (x_pos+j)];
	  break;
	case 1 :
	  for (i=0; i<BLOCK_SIZE; i++)
	    for (j=0; j<BLOCK_SIZE; j++)
	      alpha_block[1][i][j] = alpha_pntr[(y_pos+i)*nrpix + (x_pos+8+j)];
	  break;
	case 2 :
	  for (i=0; i<BLOCK_SIZE; i++)
	    for (j=0; j<BLOCK_SIZE; j++)
	      alpha_block[2][i][j] = alpha_pntr[(y_pos+8+i)*nrpix + (x_pos+j)];
	  break;
	case 3 :
	  for (i=0; i<BLOCK_SIZE; i++)
	    for (j=0; j<BLOCK_SIZE; j++)
	      alpha_block[3][i][j] = alpha_pntr[(y_pos+8+i)*nrpix + (x_pos+8+j)];
	  break;
	case 4 :
    /* Extended for Interlaced */
    /* 03.09.99 TUM Mooshofer */
    if (GetVopInterlaced(curr)) {
      for (i=0; i<BLOCK_SIZE; i++)
        for (j=0; j<BLOCK_SIZE; j++) {
          if (i%2==0) {
            alpha_block[4][i][j] = 
              ((alpha_pntr[(y_pos+2*i)*nrpix + (x_pos+2*j)] == BINARY_ALPHA) ||
               (alpha_pntr[(y_pos+2*i)*nrpix + (x_pos+2*j+1)] == BINARY_ALPHA) ||
               (alpha_pntr[(y_pos+2*i+2)*nrpix + (x_pos+2*j)] == BINARY_ALPHA) ||
               (alpha_pntr[(y_pos+2*i+2)*nrpix + (x_pos+2*j+1)] == BINARY_ALPHA))
              ? BINARY_ALPHA : 0 ;
          }
          else {
            alpha_block[4][i][j] = 
              ((alpha_pntr[(y_pos+2*i-1)*nrpix + (x_pos+2*j)] == BINARY_ALPHA) ||
               (alpha_pntr[(y_pos+2*i-1)*nrpix + (x_pos+2*j+1)] == BINARY_ALPHA) ||
               (alpha_pntr[(y_pos+2*i+1)*nrpix + (x_pos+2*j)] == BINARY_ALPHA) ||
                 (alpha_pntr[(y_pos+2*i+1)*nrpix + (x_pos+2*j+1)] == BINARY_ALPHA))
              ? BINARY_ALPHA : 0 ;
          }
        }
    }
    else {
      for (i=0; i<BLOCK_SIZE; i++)
        for (j=0; j<BLOCK_SIZE; j++) {
          alpha_block[4][i][j] = 
            ((alpha_pntr[(y_pos+2*i)*nrpix + (x_pos+2*j)] == BINARY_ALPHA) ||
             (alpha_pntr[(y_pos+2*i)*nrpix + (x_pos+2*j+1)] == BINARY_ALPHA) ||
             (alpha_pntr[(y_pos+2*i+1)*nrpix + (x_pos+2*j)] == BINARY_ALPHA) ||
             (alpha_pntr[(y_pos+2*i+1)*nrpix + (x_pos+2*j+1)] == BINARY_ALPHA))
            ? BINARY_ALPHA : 0 ; 
        }
    }
    break;
	default: break;
	}                     
      }
    }

    for (i=0; i<BLOCK_SIZE; i++)
      for (j=0; j<BLOCK_SIZE; j++)
        alpha_block[5][i][j]=alpha_block[4][i][j];
    
#ifdef DISPLAY_ENC_TRACE
    printf("============================================================\n\n");

    printf("Original block of MB %d (%d,%d)\n",
           (x_pos/MB_SIZE)+(y_pos/MB_SIZE)*(width/MB_SIZE), 
           (x_pos/MB_SIZE), (y_pos/MB_SIZE));
    for (k=0;k<6;k+=2) {
      for(i=0; i<8; i++) {
        for(j=0; j<8; j++) {
          if (alpha_block[k][i][j]==0)
            printf("   -");
          else
            printf(" %3d", fblock[k][i][j]);
		    }
        printf("    ");
        for(j=0; j<8; j++) {
          if (alpha_block[k+1][i][j]==0)
            printf("   -");
          else
            printf(" %3d", fblock[k+1][i][j]);
		    }
		    printf("\n");
		  }
      printf("\n");
    }
#endif
    
/* HYUNDAI 980507 : start */

     for(i=0; i<MB_SIZE; i++)
        for(j=0; j<MB_SIZE; j++)
        {
            if(sub_alpha_ptr[i*MB_SIZE+j])
                num_transp ++;
        }

    if (curr->interlaced && (fieldDCT != NULL))
    {
       if ( num_transp == 0 || num_transp == MB_SIZE*MB_SIZE)
            *fieldDCT = FrameFieldDCTDecide(&fblock[0][0][0]);
       else
            *fieldDCT = 0;
    }

/* HYUNDAI 980507 : end */
    

    /* 01.02.99 HHI Schueuer */

    /* for (k=0;k<4;k++) MB_trans_out[k] = MB_transp_pattern[k]; */	
    
    if (GetVopShape(curr)!=0) {
      for (k=0;k<6;k++)
	for (i=0; i<BLOCK_SIZE; i++)
	  for (j=0; j<BLOCK_SIZE; j++)
	    alpha_buf[k][i][j]=alpha_block[k][i][j]; 
  
      /* BBM_Merging(alpha_block,fblock,MB_transp_pattern,MB_trans_out); */
      for (k=0;k<6;k++)
	sadct_rowlength_out(k, alpha_block[k], BLOCK_SIZE, BLOCK_SIZE);
	
    } 
     
    /* end HHI */

    
    for (k = 0; k < 6; k++) {

      /* 01.02.99 HHI Schueuer */
      active_pels[k] = 0;
      for (x1 = 0; x1 < 8; x1++)
	for (y1 = 0; y1 < 8; y1++)
	  if(alpha_block[k][x1][y1] == BINARY_ALPHA)
	    active_pels[k]++;
      /* end HHI */
      
      
      if ((k < 4 && MB_transp_pattern[k] != 1) || (k > 3))
	{
	  /* 01.02.99 HHI Schueuer */
	  if ((GetVopSADCTDisable(curr)!=0)||(GetVopShape(curr)==0) || (active_pels[k] == 64) ) {
	    /* end HHI */
	    
	    BlockDCT (fblock[k], coeff_ind);
	  
	    /* 01.02.99 HHI Schueuer */
	  }
	  else {
	    if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q)) 
	      BlockSADCT_s_k (fblock[k], coeff_ind, alpha_block[k]);
	    else
	      BlockSADCT_Kaup (fblock[k], coeff_ind, alpha_block[k]);
	  }
	  /* end HHI */
	      

/* >>> added for DRC by Fujitsu (top)    <<< */
    	    if ((GetVopReducedResolution(curr)==0) && (GetVopDctCoefMask(curr)!=8));
	      MaskDctCoeff(coeff_ind, GetVopDctCoefMask(curr));
/* >>> added for DRC by Fujitsu (bottom) <<< */

            if (k < 4) type = 1;
            else type = 2;

            /* MW 22-APR-1997 Changes for MPEG quantization */
            if(GetVopQuantType(curr))
              {
                if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q)
                  qmat = GetVopIntraQuantMat(curr);
                else
                  qmat = GetVopNonintraQuantMat(curr);

                BlockQuantMPEG(coeff_ind,QP,Mode,type,qmat,qcoeff_ind,
			       GetVopBrightWhite(curr),1);
                BlockDequantMPEG(qcoeff_ind,QP,Mode,type,qmat,rcoeff_ind,1);
              }
            else
              {
                BlockQuantH263(coeff_ind,QP,Mode,type,qcoeff_ind,
			       GetVopBrightWhite(curr),1); 
                BlockDequantH263(qcoeff_ind,QP,Mode,type,rcoeff_ind,1, 0, GetVopBitsPerPixel(curr)); 
              }

	    /* 01.02.99 HHI Schueuer */
	    if ((GetVopSADCTDisable(curr)!=0)||(GetVopShape(curr)==0) || (active_pels[k] == 64) )
	      /* end HHI */
              BlockIDCT (rcoeff_ind, iblock[k],GetVopBrightWhite(curr));
	    /* 01.02.99 HHI Schueuer */
	    else
	      if ((Mode == MODE_INTRA) || (Mode == MODE_INTRA_Q))
		BlockSAIDCT_s_k (rcoeff_ind, iblock[k], alpha_block[k]); 
	      else
		BlockSAIDCT_Kaup (rcoeff_ind, iblock[k], alpha_block[k]);
	    /* end HHI */

        }
          else                  /* Transparent block */
        {
            for (i = 0; i < 64; i++)
                qcoeff_ind[i] = 0;
            for (i = 0; i < 8; i++)
                for (j = 0; j < 8; j++)
                    iblock[k][i][j] = 0;
        }

        coeff_ind += 64;
        qcoeff_ind += 64;
        rcoeff_ind += 64;

        if (Mode == MODE_INTRA||Mode==MODE_INTRA_Q)
            for (i = 0; i < 8; i++)
                for (j = 0; j < 8; j ++)
                    iblock[k][i][j] = MIN (GetVopBrightWhite(curr), MAX (0, iblock[k][i][j]));
        if ((GetVopPredictionType(curr)==B_VOP)&&(Mode==4))
           for (i = 0; i < 8; i++)
                for (j = 0; j < 8; j ++)
                    iblock[k][i][j] = 0;

        switch (k) {
        case 0:
        case 1:
        case 2:
            continue;

        case 3:
            if (curr->interlaced && (fieldDCT != NULL) && *fieldDCT)
               fieldDCTtoFrame(&iblock[0][0][0]);
            recon = (SInt *) GetImageData (GetVopY (rec_curr));
            BlockRebuild (recon, x_pos,     y_pos,     width, iblock[0]);
            BlockRebuild (recon, x_pos + 8, y_pos,     width, iblock[1]);
            BlockRebuild (recon, x_pos,     y_pos + 8, width, iblock[2]);
            BlockRebuild (recon, x_pos + 8, y_pos + 8, width, iblock[3]);
            continue;

        case 4:
            BlockRebuild ((SInt *) GetImageData (GetVopU (rec_curr)),
                x_pos/2, y_pos/2, width/2, iblock[4]);
            continue;

        case 5:
            BlockRebuild ((SInt *) GetImageData (GetVopV (rec_curr)),
                x_pos/2, y_pos/2, width/2, iblock[5]);
            continue;
        }
    }

/* HYUNDAI 980507 */

#ifdef DISPLAY_ENC_TRACE
    printf("Reconstructed block of MB %d (%d,%d)\n",
           (x_pos/MB_SIZE)+(y_pos/MB_SIZE)*(width/MB_SIZE), 
           (x_pos/MB_SIZE), (y_pos/MB_SIZE));
    for (k=0;k<6;k+=2) {
      for(i=0; i<8; i++) {
        for(j=0; j<8; j++) {
          if (alpha_block[k][i][j]==0)
            printf("   -");
          else
            printf(" %3d", iblock[k][i][j]);
		    }
        printf("   ");
        for(j=0; j<8; j++) {
          if (alpha_block[k+1][i][j]==0)
            printf("   -");
          else
            printf(" %3d", iblock[k+1][i][j]);
		    }
		    printf("\n");
		  }
      printf("\n");
    }
#endif
    
    FreeImage(Sub_Alpha);

    return qcoeff;
}


/***********************************************************CommentBegin******
 *
 * -- BlockPredict -- Get prediction for an Intra block
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	Get prediction for an Intra block
 * 
 * Arguments in : 	
 *	Int x_pos	x_position of Macroblock
 *	Int y_pos	y_position of Macroblock
 *	UInt width	width of Vop bounding box
 *
 * Arguments in/out :	
 *	SInt *curr	current uncoded Vop data
 *	SInt *rec_curr	reconstructed Vop data area
 *
 * Arguments out :	
 *	Int fblock[][8]	the prediction block to be coded for bitstream
 *
 * Return values :	
 *	Void
 *
 * Side effects :	
 *
 * Description :	
 *	
 * See also :
 *
 * Modified :		
 *
 ***********************************************************CommentEnd********/
Void
BlockPredict (SInt *curr, SInt *rec_curr, Int x_pos, Int y_pos,
	      UInt width, Int fblock[][8])
{
    Int i, j;
  
    for (i = 0; i < 8; i++)
    {
	for (j = 0; j < 8; j++)
	{
	    fblock[i][j] = curr[(y_pos+i)*width + x_pos+j];
	}
    }
}

/***********************************************************CommentBegin******
 *
 * -- BlockRebuild -- Reconstructs a block into data area of Vop
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	Reconstructs a block into data area of Vop
 * 
 * Arguments in : 	
 *	Int x_pos	x_position of Macroblock
 *	Int y_pos	y_position of Macroblock
 *	UInt width	width of Vop bounding box
 *
 * Arguments in/out :	
 *	SInt *rec_curr	current Vop data area to be reconstructed
 *
 * Arguments out :	
 *
 * Return values :	
 *	Void
 *
 * Side effects :	
 *
 * Description :	
 *	Does reconstruction for Intra predicted blocks also
 *
 * See also :
 *
 * Modified :		
 *
 ***********************************************************CommentEnd********/
  
Void
BlockRebuild (SInt *rec_curr, Int x_pos, Int y_pos, UInt width,
	      Int fblock[][8])
{
    Int i, j;

    for (i = 0; i < 8; i++)
    {
	for (j = 0; j < 8; j++)
	{
	    rec_curr[(y_pos+i)*width + x_pos+j] =
		fblock[i][j];
	}
    }
}
 

/***********************************************************CommentBegin******
 *
 * -- BlockDCT -- 8x8 block DCT
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	8x8 block DCT
 * 
 * Arguments in : 	
 *	Int block[][8]	8x8 data block
 *
 * Arguments in/out :	
 *
 * Arguments out :	
 *	Int *coeff	transform coefficients
 *
 * Return values :	
 *	Int		status (0 for OK only)
 *
 * Side effects :	
 *
 * Description :	
 *	This is an implementation originally done by Gisle Bjontegaard
 *	in FORTRAN.
 *
 * See also :
 *
 * Modified :
 *	27.02.97 Robert Danielsen: Changed from truncating to rounding of
 *			coefficient values. This was decided in the
 *			38.MPEG-meeting in Sevilla.
 *
 ***********************************************************CommentEnd********/
Int
BlockDCT (Int block[][8], Int *coeff)
{
#ifdef _STUDIO_CONF_5_
  /* declaration */
  double   trans[64];
  double   pi = acos( -1.0 );

  int i, j;
  unsigned int hexVal;
  double coeff1[64];
  double coeff2[64];
  double tmp;

  /* initialization */
  trans[ 0 * 8 + 0 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 1 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 2 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 3 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 4 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 5 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 6 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 7 ] =   cos( pi       / 4.0 ) ;
  
  trans[ 1 * 8 + 0 ] =   cos( pi       / 16.0 ) ;
  trans[ 1 * 8 + 1 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 2 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 3 ] =   sin( pi       / 16.0 ) ;
  trans[ 1 * 8 + 4 ] = - sin( pi       / 16.0 ) ;
  trans[ 1 * 8 + 5 ] = - sin( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 6 ] = - cos( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 7 ] = - cos( pi       / 16.0 ) ;
  
  trans[ 2 * 8 + 0 ] =   cos( pi       / 8.0 ) ;
  trans[ 2 * 8 + 1 ] =   cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 2 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 3 ] = - cos( pi       / 8.0 ) ;
  trans[ 2 * 8 + 4 ] = - cos( pi       / 8.0 ) ;
  trans[ 2 * 8 + 5 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 6 ] =   cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 7 ] =   cos( pi       / 8.0 ) ;
  
  trans[ 3 * 8 + 0 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 3 * 8 + 1 ] = - sin( pi       / 16.0 ) ;
  trans[ 3 * 8 + 2 ] = - cos( pi       / 16.0 ) ;
  trans[ 3 * 8 + 3 ] = - sin( pi * 3.0 / 16.0 ) ;
  trans[ 3 * 8 + 4 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 3 * 8 + 5 ] =   cos( pi       / 16.0 ) ;
  trans[ 3 * 8 + 6 ] =   sin( pi       / 16.0 ) ;
  trans[ 3 * 8 + 7 ] = - cos( pi * 3.0 / 16.0 ) ;
  
  trans[ 4 * 8 + 0 ] =   cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 1 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 2 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 3 ] =   cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 4 ] =   cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 5 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 6 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 7 ] =   cos( pi       / 4.0 ) ;
  
  trans[ 5 * 8 + 0 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 5 * 8 + 1 ] = - cos( pi       / 16.0 ) ;
  trans[ 5 * 8 + 2 ] =   sin( pi       / 16.0 ) ;
  trans[ 5 * 8 + 3 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 5 * 8 + 4 ] = - cos( pi * 3.0 / 16.0 ) ;
  trans[ 5 * 8 + 5 ] = - sin( pi       / 16.0 ) ;
  trans[ 5 * 8 + 6 ] =   cos( pi       / 16.0 ) ;
  trans[ 5 * 8 + 7 ] = - sin( pi * 3.0 / 16.0 ) ;
  
  trans[ 6 * 8 + 0 ] =   cos( pi * 3.0 / 8.0 ) ;
  trans[ 6 * 8 + 1 ] = - cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 2 ] =   cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 3 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 6 * 8 + 4 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 6 * 8 + 5 ] =   cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 6 ] = - cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 7 ] =   cos( pi * 3.0 / 8.0 ) ;
    
  trans[ 7 * 8 + 0 ] =   sin( pi       / 16.0 ) ;
  trans[ 7 * 8 + 1 ] = - sin( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 2 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 3 ] = - cos( pi       / 16.0 ) ;
  trans[ 7 * 8 + 4 ] =   cos( pi       / 16.0 ) ;
  trans[ 7 * 8 + 5 ] = - cos( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 6 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 7 ] = - sin( pi       / 16.0 ) ;

  /* transformation */
  for( i = 0; i < 8; i++ ){
    for( j = 0 ; j < 8; j++ ){      
      coeff1[i*8+j] = (double) (
				trans[j*8+0] * block[i][0] +
				trans[j*8+1] * block[i][1] +
				trans[j*8+2] * block[i][2] +
				trans[j*8+3] * block[i][3] +
				trans[j*8+4] * block[i][4] +
				trans[j*8+5] * block[i][5] +
				trans[j*8+6] * block[i][6] +
				trans[j*8+7] * block[i][7]);
    }
  }
  
  for( i = 0; i < 8; i++ ){
    for( j = 0 ; j < 8; j++ ){ 
      coeff2[i*8+j] = (double) (
				trans[i*8+0] * coeff1[0*8+j] +
				trans[i*8+1] * coeff1[1*8+j] +
				trans[i*8+2] * coeff1[2*8+j] +
				trans[i*8+3] * coeff1[3*8+j] +
				trans[i*8+4] * coeff1[4*8+j] +
				trans[i*8+5] * coeff1[5*8+j] +
				trans[i*8+6] * coeff1[6*8+j] +
				trans[i*8+7] * coeff1[7*8+j]);

      if( coeff2[i*8+j] >= 0.0 ){
	tmp = (double)
	  ( (int) (coeff2[i*8+j] / 4.0  + 0.5 ) );
      }
      else{
	tmp = (double)
	  ( (int) (coeff2[i*8+j] / 4.0  - 0.5 ) );
	/*
	   tmp = (double)
	   ( (int) (coeff2[i*8+j] / 4.0  - 0.49999999999 ) );
	   */
      }
      
      if( tmp > 2047.0 ){
	tmp = 2047.0 ;
      }
      else if ( tmp < - 2048.0 ){
	tmp = -2048.0 ;
      }
      
      coeff[i*8+j] =(int) tmp;
    }
  }

  return 0;
  /* end */
#elif _STUDIO_CONF_10_
  /* declaration */
  double   trans[64];
  double   pi = acos( -1.0 );

  int i, j;
  unsigned int hexVal;
  double coeff1[64];
  double coeff2[64];
  double tmp;

  /* initialization */
  trans[ 0 * 8 + 0 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 1 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 2 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 3 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 4 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 5 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 6 ] =   cos( pi       / 4.0 ) ;
  trans[ 0 * 8 + 7 ] =   cos( pi       / 4.0 ) ;
  
  trans[ 1 * 8 + 0 ] =   cos( pi       / 16.0 ) ;
  trans[ 1 * 8 + 1 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 2 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 3 ] =   sin( pi       / 16.0 ) ;
  trans[ 1 * 8 + 4 ] = - sin( pi       / 16.0 ) ;
  trans[ 1 * 8 + 5 ] = - sin( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 6 ] = - cos( pi * 3.0 / 16.0 ) ;
  trans[ 1 * 8 + 7 ] = - cos( pi       / 16.0 ) ;
  
  trans[ 2 * 8 + 0 ] =   cos( pi       / 8.0 ) ;
  trans[ 2 * 8 + 1 ] =   cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 2 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 3 ] = - cos( pi       / 8.0 ) ;
  trans[ 2 * 8 + 4 ] = - cos( pi       / 8.0 ) ;
  trans[ 2 * 8 + 5 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 6 ] =   cos( pi * 3.0 / 8.0 ) ;
  trans[ 2 * 8 + 7 ] =   cos( pi       / 8.0 ) ;
  
  trans[ 3 * 8 + 0 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 3 * 8 + 1 ] = - sin( pi       / 16.0 ) ;
  trans[ 3 * 8 + 2 ] = - cos( pi       / 16.0 ) ;
  trans[ 3 * 8 + 3 ] = - sin( pi * 3.0 / 16.0 ) ;
  trans[ 3 * 8 + 4 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 3 * 8 + 5 ] =   cos( pi       / 16.0 ) ;
  trans[ 3 * 8 + 6 ] =   sin( pi       / 16.0 ) ;
  trans[ 3 * 8 + 7 ] = - cos( pi * 3.0 / 16.0 ) ;
  
  trans[ 4 * 8 + 0 ] =   cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 1 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 2 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 3 ] =   cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 4 ] =   cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 5 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 6 ] = - cos( pi       / 4.0 ) ;
  trans[ 4 * 8 + 7 ] =   cos( pi       / 4.0 ) ;
  
  trans[ 5 * 8 + 0 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 5 * 8 + 1 ] = - cos( pi       / 16.0 ) ;
  trans[ 5 * 8 + 2 ] =   sin( pi       / 16.0 ) ;
  trans[ 5 * 8 + 3 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 5 * 8 + 4 ] = - cos( pi * 3.0 / 16.0 ) ;
  trans[ 5 * 8 + 5 ] = - sin( pi       / 16.0 ) ;
  trans[ 5 * 8 + 6 ] =   cos( pi       / 16.0 ) ;
  trans[ 5 * 8 + 7 ] = - sin( pi * 3.0 / 16.0 ) ;
  
  trans[ 6 * 8 + 0 ] =   cos( pi * 3.0 / 8.0 ) ;
  trans[ 6 * 8 + 1 ] = - cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 2 ] =   cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 3 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 6 * 8 + 4 ] = - cos( pi * 3.0 / 8.0 ) ;
  trans[ 6 * 8 + 5 ] =   cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 6 ] = - cos( pi       / 8.0 ) ;
  trans[ 6 * 8 + 7 ] =   cos( pi * 3.0 / 8.0 ) ;
    
  trans[ 7 * 8 + 0 ] =   sin( pi       / 16.0 ) ;
  trans[ 7 * 8 + 1 ] = - sin( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 2 ] =   cos( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 3 ] = - cos( pi       / 16.0 ) ;
  trans[ 7 * 8 + 4 ] =   cos( pi       / 16.0 ) ;
  trans[ 7 * 8 + 5 ] = - cos( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 6 ] =   sin( pi * 3.0 / 16.0 ) ;
  trans[ 7 * 8 + 7 ] = - sin( pi       / 16.0 ) ;

  /* transformation */
  for( i = 0; i < 8; i++ ){
    for( j = 0 ; j < 8; j++ ){      
      coeff1[i*8+j] = (double) (
                                trans[j*8+0] * block[i][0] +
                                trans[j*8+1] * block[i][1] +
                                trans[j*8+2] * block[i][2] +
                                trans[j*8+3] * block[i][3] +
                                trans[j*8+4] * block[i][4] +
                                trans[j*8+5] * block[i][5] +
                                trans[j*8+6] * block[i][6] +
                                trans[j*8+7] * block[i][7]);
    }
  }
  
  for( i = 0; i < 8; i++ ){
    for( j = 0 ; j < 8; j++ ){ 
      coeff2[i*8+j] = (double) (
                                trans[i*8+0] * coeff1[0*8+j] +
                                trans[i*8+1] * coeff1[1*8+j] +
                                trans[i*8+2] * coeff1[2*8+j] +
                                trans[i*8+3] * coeff1[3*8+j] +
                                trans[i*8+4] * coeff1[4*8+j] +
                                trans[i*8+5] * coeff1[5*8+j] +
                                trans[i*8+6] * coeff1[6*8+j] +
                                trans[i*8+7] * coeff1[7*8+j]);

      if( coeff2[i*8+j] >= 0.0 ){
        tmp = (double)
          ( (int) (coeff2[i*8+j] / 4.0  + 0.5 ) );
      }
      else{
        tmp = (double)
          ( (int) (coeff2[i*8+j] / 4.0  - 0.5 ) );
        /*
           tmp = (double)
           ( (int) (coeff2[i*8+j] / 4.0  - 0.49999999999 ) );
           */
      }

/*      
      if( tmp > 2047.0 ){
        tmp = 2047.0 ;
      }
      else if ( tmp < - 2048.0 ){
        tmp = -2048.0 ;
      }
*/      
      
      coeff[i*8+j] =(int) tmp;
    }
  }

  return 0;
  /* end */
#else
    Int   j1, i, j;
    Double b[8];
    Double b1[8];
    Double d[8][8];
    Double f0=0.7071068,f1=0.4903926,f2=0.4619398,f3=0.4157348,f4=0.3535534;
    Double f5=0.2777851,f6=0.1913417,f7=0.0975452;

    for (i = 0; i < 8; i++)
    {
	for (j = 0; j < 8; j++)
	{
	    b[j] = block[i][j];
	}
    
	/* Horizontal transform */
	for (j = 0; j < 4; j++)
	{
	    j1 = 7 - j;
	    b1[j] = b[j] + b[j1];
	    b1[j1] = b[j] - b[j1];
	}
    
	b[0] = b1[0] + b1[3];
	b[1] = b1[1] + b1[2];
	b[2] = b1[1] - b1[2];
	b[3] = b1[0] - b1[3];
	b[4] = b1[4];
	b[5] = (b1[6] - b1[5]) * f0;
	b[6] = (b1[6] + b1[5]) * f0;
	b[7] = b1[7];
	d[i][0] = (b[0] + b[1]) * f4;
	d[i][4] = (b[0] - b[1]) * f4;
	d[i][2] = b[2] * f6 + b[3] * f2;
	d[i][6] = b[3] * f6 - b[2] * f2;
	b1[4] = b[4] + b[5];
	b1[7] = b[7] + b[6];
	b1[5] = b[4] - b[5];
	b1[6] = b[7] - b[6];
	d[i][1] = b1[4] * f7 + b1[7] * f1;
	d[i][5] = b1[5] * f3 + b1[6] * f5;
	d[i][7] = b1[7] * f7 - b1[4] * f1;
	d[i][3] = b1[6] * f3 - b1[5] * f5;
    }
  
    /* Vertical transform */

    for (i = 0; i < 8; i++)
    {
	for (j = 0; j < 4; j++)
	{
	    j1 = 7 - j;
	    b1[j] = d[j][i] + d[j1][i];
	    b1[j1] = d[j][i] - d[j1][i];
	}
    
	b[0] = b1[0] + b1[3];
	b[1] = b1[1] + b1[2];
	b[2] = b1[1] - b1[2];
	b[3] = b1[0] - b1[3];
	b[4] = b1[4];
	b[5] = (b1[6] - b1[5]) * f0;
	b[6] = (b1[6] + b1[5]) * f0;
	b[7] = b1[7];
	d[0][i] = (b[0] + b[1]) * f4;
	d[4][i] = (b[0] - b[1]) * f4;
	d[2][i] = b[2] * f6 + b[3] * f2;
	d[6][i] = b[3] * f6 - b[2] * f2;
	b1[4] = b[4] + b[5];
	b1[7] = b[7] + b[6];
	b1[5] = b[4] - b[5];
	b1[6] = b[7] - b[6];
	d[1][i] = b1[4] * f7 + b1[7] * f1;
	d[5][i] = b1[5] * f3 + b1[6] * f5;
	d[7][i] = b1[7] * f7 - b1[4] * f1;
	d[3][i] = b1[6] * f3 - b1[5] * f5;
    }
  
    /* Do rounding instead of just truncating.
       Decided in 38.MPEG-meeting in Sevilla */
    
    for (i = 0; i < 8; i++)
	for (j = 0; j < 8; j++)
	    coeff[i*8+j] = (int)(floor((d[i][j]) + 0.499999));
   
    return 0;
#endif _STUDIO_CONF_5_
}

/***********************************************************CommentBegin******
 *
 * -- FindCBP -- Find the CBP for a macroblock 
 *
 * Author :  
 * 
 *
 * Created :  
 * 
 *
 * Purpose :  
 * Find the CBP for a macroblock
 * 
 * Arguments in :  
 * Int *qcoeff : pointer to quantized coefficients
 *  Int Mode : macroblock encoding mode information
 *  Int ncoeffs : the number of coefficients
 *
 * Arguments in/out : 
 * 
 *
 * Arguments out : 
 * 
 *
 * Return values : 
 * Int CBP : The coded block pattern for a macroblock
 *
 * Side effects : 
 * 
 *
 * Description : 
 * 
 *
 * See also :
 * 
 *
 * Modified :  
 * 
 *
 ***********************************************************CommentEnd********/

Int 
FindCBP (Int* qcoeff, Int Mode, Int ncoeffs)
{
    Int i,j;
    Int CBP = 0;
    Int intra = (Mode == MODE_INTRA || Mode == MODE_INTRA_Q);

    /* Set CBP for this Macroblock */
   
    for (i = 0; i < 6; i++) 
    {
	for (j = i*ncoeffs + intra; j < (i+1)*ncoeffs; j++) 
	{
        
	    if (qcoeff[j]) 
	    {
		if (i == 0) {CBP |= 32;}
		else if (i == 1) {CBP |= 16;}
		else if (i == 2) {CBP |= 8;}
		else if (i == 3) {CBP |= 4;}
		else if (i == 4) {CBP |= 2;}
		else if (i == 5) {CBP |= 1;}
		else 
		{
		    fprintf (stderr, "Error in CBP assignment\n");
		    exit(-1);
		}
    
		break;
	    }
	}
    }

    return CBP;
}

/***********************************************************CommentBegin******
 *
 * -- FrameFieldDCTDecide -- Determine if field DCT is better & swizzle
 *
 * Author :
 *                      Bob Eifrig
 *
 * Created :
 *                      24-jan-97
 *
 * Purpose :
 *                      Determine if the macroblock should be field DCT code
 *
 * Arguments in :
 *
 * Arguments in/out :
 *                      fblock[][][]    An array of 6 8x8 blocks
 *
 * Arguments out :
 *
 * Return values :
 *                      Int FieldDCT : 0 if frame DCT, 1 if field DCT
 *
 * Side effects :
 *                      fblock[][][] is redefined if field DCT is indicated
 *
 * Description :
 *                      fblock can be indexed as fblock[6][8][8].  That is, flbock
 *                      consists of 64 elements of Y1, then 64 elements of Y2, ...
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/
 
Int
FrameFieldDCTDecide(Int *fblock)
{
    static unsigned char mbline[] = {
         0*BLOCK_SIZE,  2*BLOCK_SIZE,  4*BLOCK_SIZE,  6*BLOCK_SIZE,
        16*BLOCK_SIZE, 18*BLOCK_SIZE, 20*BLOCK_SIZE, 22*BLOCK_SIZE,
         8*BLOCK_SIZE, 10*BLOCK_SIZE, 12*BLOCK_SIZE, 14*BLOCK_SIZE,
        24*BLOCK_SIZE, 26*BLOCK_SIZE, 28*BLOCK_SIZE, 30*BLOCK_SIZE,  };
    static unsigned char fwd_shuffle[] = {
         1*BLOCK_SIZE, 0*BLOCK_SIZE,    2*BLOCK_SIZE, 1*BLOCK_SIZE,    4*BLOCK_SIZE, 2*BLOCK_SIZE,
        16*BLOCK_SIZE, 4*BLOCK_SIZE,    0*BLOCK_SIZE,16*BLOCK_SIZE,    3*BLOCK_SIZE, 0*BLOCK_SIZE,
         6*BLOCK_SIZE, 3*BLOCK_SIZE,   20*BLOCK_SIZE, 6*BLOCK_SIZE,   17*BLOCK_SIZE,20*BLOCK_SIZE,
         0*BLOCK_SIZE,17*BLOCK_SIZE,    5*BLOCK_SIZE, 0*BLOCK_SIZE,   18*BLOCK_SIZE, 5*BLOCK_SIZE,
         0*BLOCK_SIZE,18*BLOCK_SIZE,    7*BLOCK_SIZE, 0*BLOCK_SIZE,   22*BLOCK_SIZE, 7*BLOCK_SIZE,
        21*BLOCK_SIZE,22*BLOCK_SIZE,   19*BLOCK_SIZE,21*BLOCK_SIZE,    0*BLOCK_SIZE,19*BLOCK_SIZE,
         9*BLOCK_SIZE, 0*BLOCK_SIZE,   10*BLOCK_SIZE, 9*BLOCK_SIZE,   12*BLOCK_SIZE,10*BLOCK_SIZE,
        24*BLOCK_SIZE,12*BLOCK_SIZE,    0*BLOCK_SIZE,24*BLOCK_SIZE,   11*BLOCK_SIZE, 0*BLOCK_SIZE,
        14*BLOCK_SIZE,11*BLOCK_SIZE,   28*BLOCK_SIZE,14*BLOCK_SIZE,   25*BLOCK_SIZE,28*BLOCK_SIZE,
         0*BLOCK_SIZE,25*BLOCK_SIZE,   13*BLOCK_SIZE, 0*BLOCK_SIZE,   26*BLOCK_SIZE,13*BLOCK_SIZE,
         0*BLOCK_SIZE,26*BLOCK_SIZE,   15*BLOCK_SIZE, 0*BLOCK_SIZE,   30*BLOCK_SIZE,15*BLOCK_SIZE,
        29*BLOCK_SIZE,30*BLOCK_SIZE,   27*BLOCK_SIZE,29*BLOCK_SIZE,    0*BLOCK_SIZE,27*BLOCK_SIZE, };
    Double frame, field, d;
    Int tmp[BLOCK_SIZE], *p, *q, *plast;
    Int i;
 
    frame = field = 0.0;
    for (i = 0; i < sizeof(mbline)-1; i++) {
        if (i == (sizeof(mbline)/2 - 1))
            continue;
        p = &fblock[mbline[i + 0]];
        q = &fblock[mbline[i + 1]];
        for (plast = p + BLOCK_SIZE; p < plast; p++, q++) {
            d = p[0] - p[BLOCK_SIZE];
            frame += d * d;
            d = p[BLOCK_SIZE] - q[0];
            frame += d * d;
 
            d = p[0] - q[0];
            field += d * d;
            d = p[BLOCK_SIZE] - q[BLOCK_SIZE];
            field += d * d;
        }
    }
    if (frame <= field)
      return(0);
    /*
     * Re-order luminance macroblock lines
     */
    for (i = 0; i < sizeof(fwd_shuffle); i += 2)
        memcpy(fwd_shuffle[i+1] ? &fblock[fwd_shuffle[i+1]] : tmp,
               fwd_shuffle[i+0] ? &fblock[fwd_shuffle[i+0]] : tmp,
               BLOCK_SIZE * sizeof(Int));
    return(1);
}



/***********************************************************CommentBegin******
 *
 * -- fieldDCTtoFrame -- permute the field DCT block back to frame layout
 *
 * Author :
 *                      Bob Eifrig
 *
 * Created :
 *                      24-jan-97
 *
 * Purpose :
 *                      Undo the field DCT permutation
 *
 * Arguments in :
 *
 * Arguments in/out :
 *                      iblock[][][]    An array of 6 8x8 blocks
 *
 * Arguments out :
 *
 * Return values :
 *
 * Side effects :
 *                      iblock[][][] is re-defined
 *
 * Description :
 *                      iblock can be indexed as fblock[6][8][8].  That is, flbock
 *                      consists of 64 elements of Y1, then 64 elements of Y2, ...
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/
 
Void
fieldDCTtoFrame(Int *iblock)
{
    static unsigned char inv_shuffle[] = {
         1*BLOCK_SIZE, 0*BLOCK_SIZE, 16*BLOCK_SIZE, 1*BLOCK_SIZE,  4*BLOCK_SIZE,16*BLOCK_SIZE,
         2*BLOCK_SIZE, 4*BLOCK_SIZE,  0*BLOCK_SIZE, 2*BLOCK_SIZE,  3*BLOCK_SIZE, 0*BLOCK_SIZE,
        17*BLOCK_SIZE, 3*BLOCK_SIZE, 20*BLOCK_SIZE,17*BLOCK_SIZE,  6*BLOCK_SIZE,20*BLOCK_SIZE,
         0*BLOCK_SIZE, 6*BLOCK_SIZE,  5*BLOCK_SIZE, 0*BLOCK_SIZE, 18*BLOCK_SIZE, 5*BLOCK_SIZE,
         0*BLOCK_SIZE,18*BLOCK_SIZE,  7*BLOCK_SIZE, 0*BLOCK_SIZE, 19*BLOCK_SIZE, 7*BLOCK_SIZE,
        21*BLOCK_SIZE,19*BLOCK_SIZE, 22*BLOCK_SIZE,21*BLOCK_SIZE,  0*BLOCK_SIZE,22*BLOCK_SIZE,
         9*BLOCK_SIZE, 0*BLOCK_SIZE, 24*BLOCK_SIZE, 9*BLOCK_SIZE, 12*BLOCK_SIZE,24*BLOCK_SIZE,
        10*BLOCK_SIZE,12*BLOCK_SIZE,  0*BLOCK_SIZE,10*BLOCK_SIZE, 11*BLOCK_SIZE, 0*BLOCK_SIZE,
        25*BLOCK_SIZE,11*BLOCK_SIZE, 28*BLOCK_SIZE,25*BLOCK_SIZE, 14*BLOCK_SIZE,28*BLOCK_SIZE,
         0*BLOCK_SIZE,14*BLOCK_SIZE, 13*BLOCK_SIZE, 0*BLOCK_SIZE, 26*BLOCK_SIZE,13*BLOCK_SIZE,
         0*BLOCK_SIZE,26*BLOCK_SIZE, 15*BLOCK_SIZE, 0*BLOCK_SIZE, 27*BLOCK_SIZE,15*BLOCK_SIZE,
        29*BLOCK_SIZE,27*BLOCK_SIZE, 30*BLOCK_SIZE,29*BLOCK_SIZE,  0*BLOCK_SIZE,30*BLOCK_SIZE,  };
    Int tmp[BLOCK_SIZE];
    Int i;
  
    for (i = 0; i < sizeof(inv_shuffle); i += 2)
        memcpy(inv_shuffle[i+1] ? &iblock[inv_shuffle[i+1]] : tmp,
               inv_shuffle[i+0] ? &iblock[inv_shuffle[i+0]] : tmp,
                BLOCK_SIZE*sizeof(Int));
}

/* 01.04.01: Studio Profile */
Int *CodeMB_DCT_Studio(Vop *curr, Vop *rec_curr, Int x_pos, Int y_pos, UInt width,
		       Int QP, Int Mode,
		       Int MB_transp_pattern[], Int MB_transp_pattern_uv[],
		       Int *fieldDCT, Int chroma_format, VolConfig *vol_config)
{
  Int vop_structure = GetVopVopStructure_st(curr);
  Int fip = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(curr), GetVopTopFieldFirst(curr));
  Int intra_dc_precision = GetVopIntraDCPrecision_st(curr,fip),
      q_scale_type = GetVopQScaleType_st(curr,fip);
  
  Int kk;
  Int ii,jj;  
    Int k;
    Int fblock[12][8][8];
    Int coeff[12*64];
    Int *coeff_ind;
    Int *qcoeff;
    Int *qcoeff_ind;
    Int *rcoeff_ind;
    Int x, y;
/*    SInt *current, *recon = NULL;*/
    SInt *current[2], *recon[2] = {NULL,NULL};  
    UInt xwidth;
    Int iblock[12][8][8];
    Int rcoeff[12*64];
    Int i, j;
    Int type;                   /* luma = 1, chroma = 2 */
    Int *qmat;

    /* HYUNDAI 980507 : start */
    Image *Sub_Alpha= AllocImage(MB_SIZE, MB_SIZE, SHORT_TYPE);
    SInt  *sub_alpha_ptr;
    Int num_transp = 0;

    Int binary_number;
  
    Int block_count;
    Int cdivx, cdivy;
    Int iii,jjj,kkk;

#ifdef _STUDIO_CONF_1_
    int flag_r = 0;
    if ( rand() < RAND_MAX * 5 / 6 )
      flag_r = 1;
#endif _STUDIO_CONF_1_
  
#ifdef _STUDIO_CONF_3_
  Int maxvalue, coefstate, kk2;
#endif _STUDIO_CONF_3_
  
    if(chroma_format == CHROMA444)
      {
        block_count = 12;
        cdivx = 1;
        cdivy = 1;
      }
    else if(chroma_format == CHROMA422)
      {
        block_count = 8;
        cdivx = 2;
        cdivy = 1;
      }
    else
      {
        block_count = 6;
        cdivx = 2;
        cdivy = 2;
      }

  /*AM2PDAM*/
  binary_number = 0;
  if(GetVopShape(curr) == GREY_SCALE) {
    for(i=0;i<MAX_AUX_COMP_NUM;i++) {
      if(GetVopAuxCompType_st(curr,i) == AUX_COMP_ALPHA) {
        binary_number = i;
        break;
      }
    }
  }

  
    GetSubImage_Studio(GetVopA2_st(rec_curr,binary_number), vop_structure, fip,
		 Sub_Alpha, x_pos, y_pos);
    sub_alpha_ptr = (SInt *)GetImageData(Sub_Alpha);
    /* HYUNDAI 980507 : end */

    qcoeff = (Int *) malloc (sizeof (Int) * 12*64);
    coeff_ind = coeff;
    qcoeff_ind = qcoeff;
    rcoeff_ind = rcoeff;

  /* Get fblock(inputed block) */
    for (k = 0; k < block_count; k++) {
        switch (k) {
        case 0:
            x = x_pos;
            y = y_pos;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
            break;
        case 1:
            x = x_pos + 8;
            y = y_pos;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
            break;
        case 2:
            x = x_pos;
            y = y_pos + 8;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
            break;
        case 3:
            x = x_pos + 8;
            y = y_pos + 8;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
            break;
        case 4:
            x = x_pos / cdivx;
            y = y_pos / cdivy;
            xwidth = width / cdivx;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
            break;
        case 5:
            x = x_pos / cdivx;
            y = y_pos / cdivy;
            xwidth = width / cdivx;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
            break;
        case 6:
            x = x_pos / cdivx;
            y = y_pos + 8;
            xwidth = width / cdivx;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
            break;
        case 7:
            x = x_pos / cdivx;
            y = y_pos + 8;
            xwidth = width / cdivx;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
            break;
        case 8:
            x = x_pos + 8;
            y = y_pos;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
            break;
        case 9:
            x = x_pos + 8;
            y = y_pos;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
            break;
        case 10:
            x = x_pos + 8;
            y = y_pos + 8;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
            break;
        case 11:
            x = x_pos + 8;
            y = y_pos + 8;
            xwidth = width;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
            break;
/*        case 4:
            x = x_pos / 2;
            y = y_pos / 2;
            xwidth = width / 2;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
            break;
        case 5:
            x = x_pos / 2;
            y = y_pos / 2;
            xwidth = width / 2;
	    for(kk=0;kk<2;kk++)
	      current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
            break;
*/
        default:
            fprintf (stdout, "CodeMB (): Non-existing case in switch statement\n");
            exit (-1);
            break;
        }
        BlockPredict_Studio(current, vop_structure, fip, recon, x, y, xwidth, fblock[k]);
      }/* for (k = 0; k < block_count; k++) */

#ifdef _STUDIO_CONF_5_
  for ( kk=0; kk<block_count; kk++ )
    for ( jj=0; jj<8; jj++ )
      for (ii=0; ii<8; ii++ ) {
        fblock[kk][jj][ii] = (int)ie_sign * ie_rand(ie_L, ie_H);
        g_pixel[fblock[kk][jj][ii]+512] ++;
      }
#endif  _STUDIO_CONF_5_

#ifdef _STUDIO_CONF_7_
  {
    int tmp_v;

    for ( kk=0; kk<block_count; kk++ ) {
      /* decide the value of the current block */
      tmp_v = (int)( (double)rand() / (double)RAND_MAX * 1023.0 + 0.5 );
      if ( Mode == MODE_INTER || Mode == MODE_INTER_Q )
        if ( rand() < RAND_MAX / 2 ) {
          tmp_v ++;
          tmp_v *= -1;
        }

      /* fill the current block with tmp_v */
      for ( jj=0; jj<8; jj++ )
        for (ii=0; ii<8; ii++ )
          fblock[kk][jj][ii] = tmp_v;
    }
  }
#endif _STUDIO_CONF_7_
  
#ifdef _STUDIO_CONF_8_
  {
    int tmp_v;

    for ( kk=0; kk<block_count; kk++ ) {
      /* decide the value of the current block */
      tmp_v = (int)( (double)rand() / (double)RAND_MAX * 1023.0 + 0.5 );
      if ( Mode == MODE_INTER || Mode == MODE_INTER_Q )
        tmp_v = 0;

      /* fill the current block with tmp_v */
      for ( jj=0; jj<8; jj++ )
        for (ii=0; ii<8; ii++ )
          fblock[kk][jj][ii] = tmp_v;
    }
  }
#endif _STUDIO_CONF_8_
  
#ifdef _STUDIO_CONF_10_
  {
    for ( kk=0; kk<block_count; kk++ )
      for ( jj=0; jj<8; jj++ )
        for (ii=0; ii<8; ii++ ) {
          fblock[kk][jj][ii] = (int)( (double)rand() / (double)RAND_MAX * 1500.0/*1535.0*/ + 0.5 );
          if ( Mode == MODE_INTER || Mode == MODE_INTER_Q )
            if ( rand() < RAND_MAX / 2 ) {
              fblock[kk][jj][ii] ++;
              fblock[kk][jj][ii] *= -1;
            }
        }
  }
#endif _STUDIO_CONF_10_

/* HYUNDAI 980507 : start */

     for(i=0; i<MB_SIZE; i++)
        for(j=0; j<MB_SIZE; j++)
        {
            if(sub_alpha_ptr[i*MB_SIZE+j])
                num_transp ++;
        }

    if (curr->interlaced && (fieldDCT != NULL) && vop_structure == FRAME_VOP && GetVopFramePredFrameDCT_st(curr) == 0)
      {
	if ( num_transp == 0 || num_transp == MB_SIZE*MB_SIZE) {
	  if(vol_config->mb_dct_type_st == 1 || (vol_config->mb_dct_type_st==3 && GetRandomValue_Studio(0,1)))
	    *fieldDCT = FrameFieldDCTDecide_Studio(&fblock[0][0][0],chroma_format,1)/*1*/;
	  else if(vol_config->mb_dct_type_st == 2)
	    *fieldDCT = FrameFieldDCTDecide_Studio(&fblock[0][0][0],chroma_format,0);
	  else
	    *fieldDCT = 0;
	}
	else
	  *fieldDCT = 0;
      } /*else if(curr->interlaced && fieldDCT != NULL)
	*fieldDCT = 1;*/
      else
	*fieldDCT = 0;
  
/* HYUNDAI 980507 : end */

    for (k = 0; k < block_count; k++) {

/*      if ((k < 4 && MB_transp_pattern[k] != 1) || (k > 3))*/
      if ((k < 4 && MB_transp_pattern[k] != 1)
          ||(((((k==4)||(k==5))&&MB_transp_pattern_uv[0] != 1)
              || (((k==6)||(k==7))&&MB_transp_pattern_uv[2] != 1)
              || (((k==8)||(k==9))&&MB_transp_pattern_uv[1] != 1)
              || (((k==10)||(k==11))&&MB_transp_pattern_uv[3] != 1))
             &&(chroma_format!=CHROMA420))
          ||((k > 3)&&(chroma_format==CHROMA420)))
      {
	    BlockDCT (fblock[k], coeff_ind);

#ifdef _STUDIO_CONF_3_
            
            if ( Mode == MODE_INTRA || Mode == MODE_INTRA_Q ) {
              maxvalue = 4095;
              coeff_ind[0] = GetRandomValue_Studio( 0, 8191);
            } else
              maxvalue = 8191;

            coefstate = 0;
            for ( kk2 = (Mode == MODE_INTRA || Mode == MODE_INTRA_Q); kk2 < 64; kk2 ++ ) {
              kk = zigzag_i[kk2];
              
              if ( coefstate == 0 ) {
                if ( GetRandomValue_Studio( 0, 3) < 3 )
                  coeff_ind[kk] = GetRandomValue_Studio( 0, maxvalue);
                else {
                  coeff_ind[kk] = 0;
                  if ( GetRandomValue_Studio( 0, 1) )
                    coefstate = 1;
                  else
                    coefstate = 2;
                }
              } else if ( coefstate == 1 ) {
                if ( GetRandomValue_Studio( 0, 8) < 8 )
                  coeff_ind[kk] = 0;
                else {
                  if ( GetRandomValue_Studio( 0, 1) )
                    coeff_ind[kk] = 1;
                  else
                    coeff_ind[kk] = GetRandomValue_Studio( 0, maxvalue);
                  coefstate = 0;
                }
              } else {
                if ( GetRandomValue_Studio( 0, 2) < 1 )
                  coeff_ind[kk] = 0;
                else {
                  if ( GetRandomValue_Studio( 0, 1) )
                    coeff_ind[kk] = 1;
                  else
                    coeff_ind[kk] = GetRandomValue_Studio( 0, maxvalue);
                  coefstate = 0;
                }
              }
              if ( GetRandomValue_Studio( 0, 1) )
                coeff_ind[kk] *= -1;
            }
              
#endif _STUDIO_CONF_3_
      
#ifdef _STUDIO_CONF_4_
            if ( Mode == MODE_INTER || Mode == MODE_INTER_Q )
              coeff_ind[0] = 0;
            for ( ii=1; ii<64; ii++ )
              coeff_ind[ii] = 0;
#endif _STUDIO_CONF_4_
	    
            if (k < 4) type = 1;
            else type = 2;
	    
            /* MW 22-APR-1997 Changes for MPEG quantization */
            if(GetVopQuantType(curr))
              {
                if(Mode==MODE_INTRA || Mode==MODE_INTRA_Q) {
		  if(k<4)
		    qmat = GetVopIntraQuantMat(curr);
		  else
		    qmat = GetVopChromaIntraQuantMat_st(curr);
                } else {
		  if(k<4)
		    qmat = GetVopNonintraQuantMat(curr);
		  else
		    qmat = GetVopChromaNonintraQuantMat_st(curr);
		}
/*
		fprintf(stdout, "block_count = %d\n", k);
		for(ii=0;ii<8;ii++) {
		  for(jj=0;jj<8;jj++) {
		    fprintf(stdout, " %4d", qmat[ii*8+jj]);
		  }
		  fprintf(stdout, "\n");
		}
*/
		
		/*v001225*/
                BlockQuantMPEG_DCText_Studio(coeff_ind,QP,Mode,type,
					     intra_dc_precision, q_scale_type,qmat,
					     GetVopDCTPrecision_st(curr,fip), GetVopBitsPerPixel(curr),/*v001225*/
					     qcoeff_ind,
					     GetMaxQDC_DCText_Studio(GetVopBitsPerPixel(curr),GetVopDCTPrecision_st(curr,fip),intra_dc_precision),/*v001225*/
					     /*GetVopBrightWhite(curr),*/1);

#ifdef _STUDIO_CONF_5_
                if ( qcoeff_ind[0]<0 || qcoeff_ind[0]>2047 )
                  exit(-5);
                for ( ii=1; ii<64; ii++ )
                  if ( qcoeff_ind[ii]<-2048 || qcoeff_ind[ii]>2047 )
                    exit(-4);

                for ( ii=0; ii<64; ii++ )
                  g_coeff[qcoeff_ind[ii]+2048] ++;
#endif _STUDIO_CONF_5_
		
#ifdef _STUDIO_CONF_6_
                for ( ii = (Mode == MODE_INTRA || Mode == MODE_INTRA_Q); ii<64; ii++ )
                  t6_v_qcoef[qcoeff_ind[ii]+9000] ++;
#endif _STUDIO_CONF_6_

#ifdef _STUDIO_CONF_1_
                if ( Mode == MODE_INTER || Mode == MODE_INTER_Q )
                  if ( flag_r && rand() < RAND_MAX * 0.840909/*0.840909-->1/4allzero*//*0.871688-->1/3allzero*//*0.917011-->1/2allzero*/ )
                    for ( ii = 0; ii < 64; ii++ )
                      qcoeff_ind[ii] = 0;
#endif _STUDIO_CONF_1_

		/*v010113_0*/
#ifdef _STUDIO_CONF_5_
                BlockDequantMPEG_DCText_DeadZone_Studio(qcoeff_ind,QP,Mode,type,
							intra_dc_precision, q_scale_type, qmat,
							GetVopDCTPrecision_st(curr,fip), GetVopBitsPerPixel(curr),/*v001225*/
							GetVopDeadZoneDisable_st(curr,fip),
							rcoeff_ind,1,1);
#else
                BlockDequantMPEG_DCText_DeadZone_Studio(qcoeff_ind,QP,Mode,type,
							intra_dc_precision, q_scale_type, qmat,
							GetVopDCTPrecision_st(curr,fip), GetVopBitsPerPixel(curr),/*v001225*/
							GetVopDeadZoneDisable_st(curr,fip),
							rcoeff_ind,1,0);
#endif _STUDIO_CONF_5_
		
#if 0
                BlockDequantMPEG_DCText_Studio(qcoeff_ind,QP,Mode,type,
					       intra_dc_precision, q_scale_type, qmat,
					       GetVopDCTPrecision_st(curr,fip), GetVopBitsPerPixel(curr),/*v001225*/
					       rcoeff_ind,1);
#endif 0	
#if 0
                BlockQuantMPEG_Studio(coeff_ind,QP,Mode,type,
				intra_dc_precision, q_scale_type,qmat,
				qcoeff_ind,
				GetMaxQDC_Studio(GetVopBitsPerPixel(curr),intra_dc_precision),
				/*GetVopBrightWhite(curr),*/1);

                BlockDequantMPEG_Studio(qcoeff_ind,QP,Mode,type,
				  intra_dc_precision, q_scale_type, qmat,
				  rcoeff_ind,1);
#endif 0
	      }
            else
              {
                BlockQuantH263(coeff_ind,QP,Mode,type,qcoeff_ind,
			       GetVopBrightWhite(curr),1); 
                BlockDequantH263(qcoeff_ind,QP,Mode,type,rcoeff_ind,1, 0, GetVopBitsPerPixel(curr)); 
              }

	    /*v001225*/
	    BlockIDCT_DCText_Studio (rcoeff_ind, iblock[k],GetVopBrightWhite(curr),0);

	  }
          else                  /* Transparent block */
        {
            for (i = 0; i < 64; i++)
                qcoeff_ind[i] = 0;
            for (i = 0; i < 8; i++)
                for (j = 0; j < 8; j++)
                    iblock[k][i][j] = 0;
        }

        coeff_ind += 64;
        qcoeff_ind += 64;
        rcoeff_ind += 64;

        if (Mode == MODE_INTRA||Mode==MODE_INTRA_Q)
            for (i = 0; i < 8; i++)
                for (j = 0; j < 8; j ++)
                    iblock[k][i][j] = MIN (GetVopBrightWhite(curr), MAX (0, iblock[k][i][j]));
        if ((GetVopPredictionType(curr)==B_VOP)&&(Mode==4))
           for (i = 0; i < 8; i++)
                for (j = 0; j < 8; j ++)
                    iblock[k][i][j] = 0;

        switch (k) {
        case 0:
        case 1:
        case 2:
            continue;

        case 3:
            if (curr->interlaced && (vop_structure == FRAME_VOP)
		&& (GetVopFramePredFrameDCT_st(curr) == 0)
		&&(fieldDCT != NULL) && *fieldDCT)
               fieldDCTtoFrame(&iblock[0][0][0]);
	    for(kk=0;kk<2;kk++)
	      recon[kk] = (SInt *) GetImageData (GetVopY_st (rec_curr,kk));
            BlockRebuild_Studio (recon, vop_structure, fip, x_pos,     y_pos,     width, iblock[0]);
            BlockRebuild_Studio (recon, vop_structure, fip, x_pos + 8, y_pos,     width, iblock[1]);
            BlockRebuild_Studio (recon, vop_structure, fip, x_pos,     y_pos + 8, width, iblock[2]);
            BlockRebuild_Studio (recon, vop_structure, fip, x_pos + 8, y_pos + 8, width, iblock[3]);

            continue;

	  case 4:
#if 0
	    for(kk=0;kk<2;kk++)
	      recon[kk] = (SInt *) GetImageData (GetVopU2 (rec_curr,kk));	      
            BlockRebuild2 (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			   vop_structure, fip, 
			   x_pos/2, y_pos/2, width/2, iblock[4]);
#endif
            continue;

	  case 5:
            if(chroma_format == CHROMA420)
              {

		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopU_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos/2, y_pos/2, width/2, iblock[4]);
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopV_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopV (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos/2, y_pos/2, width/2, iblock[5]);
	      }
	    continue;
	    
	  case 6:
	    continue;
	  case 7:
	    if(chroma_format == CHROMA422) {
	      if (curr->interlaced && (vop_structure == FRAME_VOP)
		  && (GetVopFramePredFrameDCT_st(curr) == 0)
		  &&(fieldDCT != NULL) && *fieldDCT)
		fieldDCTtoFrameChroma_Studio(&iblock[0][0][0], chroma_format);
	      
	      for(kk=0;kk<2;kk++)
		recon[kk] = (SInt *) GetImageData (GetVopU_st (rec_curr,kk));	      
	      BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			     vop_structure, fip, 
			     x_pos/cdivx, y_pos/cdivy, width/cdivx, iblock[4]);
	      for(kk=0;kk<2;kk++)
		recon[kk] = (SInt *) GetImageData (GetVopV_st (rec_curr,kk));	      
	      BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopV (rec_curr))*/,
			     vop_structure, fip, 
			     x_pos/cdivx, y_pos/cdivy, width/cdivx, iblock[5]);
	      for(kk=0;kk<2;kk++)
		recon[kk] = (SInt *) GetImageData (GetVopU_st (rec_curr,kk));	      
	      BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			     vop_structure, fip, 
			     x_pos/cdivx, y_pos+8, width/cdivx, iblock[6]);
	      for(kk=0;kk<2;kk++)
		recon[kk] = (SInt *) GetImageData (GetVopV_st (rec_curr,kk));	      
	      BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopV (rec_curr))*/,
			     vop_structure, fip, 
			     x_pos/cdivx, y_pos+8, width/cdivx, iblock[7]);
	    }
	    continue;
	  case 8:
	  case 9:
	  case 10:
	    continue;
	  case 11:
            if(chroma_format == CHROMA444)
              {
		if (curr->interlaced && (vop_structure == FRAME_VOP)
		    && (GetVopFramePredFrameDCT_st(curr) == 0)
		    &&(fieldDCT != NULL) && *fieldDCT)
                  fieldDCTtoFrameChroma_Studio(&iblock[0][0][0], chroma_format);
		
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopU_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos, y_pos, width, iblock[4]);
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopV_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopV (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos, y_pos, width, iblock[5]);
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopU_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos, y_pos+8, width, iblock[6]);
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopV_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopV (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos, y_pos+8, width, iblock[7]);

		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopU_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos+8, y_pos, width, iblock[8]);
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopV_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopV (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos+8, y_pos, width, iblock[9]);
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopU_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopU (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos+8, y_pos+8, width, iblock[10]);
		for(kk=0;kk<2;kk++)
		  recon[kk] = (SInt *) GetImageData (GetVopV_st (rec_curr,kk));	      
		BlockRebuild_Studio (recon/*(SInt *) GetImageData (GetVopV (rec_curr))*/,
			       vop_structure, fip, 
			       x_pos+8, y_pos+8, width, iblock[11]);
              }
            continue;
	    
	      
      
	  }/*switch(k)*/
    }/*for (k = 0; k < block_count; k++)*/

/* HYUNDAI 980507 */
    FreeImage(Sub_Alpha);

    return qcoeff;
}

Int
BitstreamPutDPCMBlock(SInt *block,
		      Int width, Int height,
		      Int bits_per_pixel,
		      Int dpcm_scan_order,
		      Image *bitstream,
		      Int *block_mean) {
  int i, j;
  int bitcount=0;

  int tab[256];

  int sum = 0;
  int sumx[12];
  int rice_parameter;

  int current, top, left, topleft;
  int pred, pred2, mi, ma, mi2, ma2, residual;

  int v;
  int k;
  
  for (i=0; i<height; i++)
    for (j=0; j<width; j++)
      sum += block[width*i+j];

  *block_mean = (sum+width*height/2)/(width*height);
  /* MoMuSys-2000FDAM1COR-020612 */
  if ( *block_mean == 0 )
    *block_mean = 1;
  /**/

  fprintf ( stderr, "020711 : bpp = %d\n", 1 << (bits_per_pixel-1) );
  
  for (i=0; i<height; i++) {

    /* MoMuSys-2000FDAM1COR-020711_2_BugFix */
    current = 1 << (bits_per_pixel-1); /**block_mean;*/
    top = 1 << (bits_per_pixel-1); /**block_mean;*/
    /**/
    
    for (j=0; j<width; j++) {
      
      left = current;
      topleft = top;
      
      if (dpcm_scan_order == 0) {
	current = block[width*i+j];
	if (i!=0)
	  top = block[width*i+j-width];
      }
      else {
	current = block[width*(height-1-i)+j];
	if (i!=0)
	  top = block[width*(height-1-i)+j+width];
      }
      
      pred = top+left-topleft;
      mi = (top < left) ? top : left;
      ma = top+left-mi;
      
      if (pred < mi)
	pred = mi;
      if (pred > ma)
	pred = ma;
      
      mi2 = (topleft < mi) ? topleft : mi;
      ma2 = (topleft > ma) ? topleft : ma;
      
      pred2 = (mi2+ma2)>>1;

      if (pred == pred2)
	pred2 = *block_mean;
      
      residual = current - pred;
      
      if (pred2 > pred)
	residual = -residual;
      
      residual &= (1<<bits_per_pixel)-1;
      
      residual += 1<<bits_per_pixel-1;
      residual &= (1<<bits_per_pixel)-1;
      residual -= 1<<bits_per_pixel-1;
      
      if (residual < 0)
	residual = -2*residual-1;
      else
	residual = 2*residual;
      
      tab[width*i+j] = residual;
    }
  }

  for (i=0; i<12; i++)
    sumx[i] = 0;
  for (i=0; i<height; i++)
    for (j=0; j<width; j++) {
      v = tab[width*i+j];
      for (k=0; k<12; k++)
	if ((v>>k) >= 11)
	  sumx[k] += 12 + bits_per_pixel;
	else
	  sumx[k] += (v>>k) + 1 + k;
    }
  
  rice_parameter = 0;
  for (i=1; i<12; i++)
    if (sumx[i] < sumx[rice_parameter])
      rice_parameter = i;

  BitstreamPutBits(bitstream,*block_mean,bits_per_pixel);
  bitcount += bits_per_pixel;

  /* MoMuSys-2000FDAM1COR-020612 */
  if ( rice_parameter > 11 ) {
    fprintf(stderr, "rice_paremeter = %d\n", rice_parameter);
    exit(-2);
  }
  /**/
  
  if (rice_parameter == 0) {
    BitstreamPutBits(bitstream,15,4);
    bitcount += 4;
  } else {
    BitstreamPutBits(bitstream,rice_parameter,4);
    bitcount += 4;
  }
    
  for (i=0; i<height; i++)
    for (j=0; j<width; j++) {
      v = tab[width*i+j];
      if ((v>>rice_parameter) >= 11) {
	BitstreamPutBits(bitstream,1,12);
	bitcount += 12;
	BitstreamPutBits(bitstream,v,bits_per_pixel);
	bitcount += bits_per_pixel;
      }
      else {
	BitstreamPutBits(bitstream,1,(v>>rice_parameter)+1);
	bitcount += ((v>>rice_parameter)+1);
	BitstreamPutBits(bitstream,v,rice_parameter);
	bitcount += rice_parameter;
      }
    }

  return bitcount;
}

/***********************************************************CommentBegin******
 *
 * -- BlockPredict_Studio -- Get prediction for an Intra block
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	Get prediction for an Intra block
 * 
 * Arguments in : 	
 *	Int x_pos	x_position of Macroblock
 *	Int y_pos	y_position of Macroblock
 *	UInt width	width of Vop bounding box
 *
 * Arguments in/out :	
 *	SInt *curr	current uncoded Vop data
 *	SInt *rec_curr	reconstructed Vop data area
 *
 * Arguments out :	
 *	Int fblock[][8]	the prediction block to be coded for bitstream
 *
 * Return values :	
 *	Void
 *
 * Side effects :	
 *
 * Description :	
 *	
 * See also :
 *
 * Modified :		
 * 26.07.99 Osamu Sunohara (SONY): For Studio Profile
 *
 ***********************************************************CommentEnd********/
Void
BlockPredict_Studio(SInt **curr, Int vop_structure, Int field_parity,
	      SInt **rec_curr, Int x_pos, Int y_pos,
	      UInt width, Int fblock[][8])
{
    Int i, j;

    Int mem_pos[2];
    
    for (i = 0; i < 8; i++)
    {
	for (j = 0; j < 8; j++)
	{
	  CalcFieldMemoryPosition_Studio(vop_structure, field_parity,
				   x_pos+j, y_pos+i, width,
				   mem_pos);
	  fblock[i][j] = curr[mem_pos[0]][mem_pos[1]];
/*          fblock[i][j] = curr[(y_pos+i)*width + x_pos+j];*/
	}
    }
}

/***********************************************************CommentBegin******
 *
 * -- BlockRebuild_Studio -- Reconstructs a block into data area of Vop
 *
 * Author :		
 *	Robert Danielsen, Telenor R&D, <Robert.Danielsen@nta.no>
 *
 * Created :		
 *	28.06.96
 *
 * Purpose :		
 *	Reconstructs a block into data area of Vop
 * 
 * Arguments in : 	
 *	Int x_pos	x_position of Macroblock
 *	Int y_pos	y_position of Macroblock
 *	UInt width	width of Vop bounding box
 *
 * Arguments in/out :	
 *	SInt *rec_curr	current Vop data area to be reconstructed
 *
 * Arguments out :	
 *
 * Return values :	
 *	Void
 *
 * Side effects :	
 *
 * Description :	
 *	Does reconstruction for Intra predicted blocks also
 *
 * See also :
 *
 * Modified :
 * 26.07.99 Osamu Sunohara (SONY): For Studio Profile
 *
 ***********************************************************CommentEnd********/
  
Void
BlockRebuild_Studio (SInt **rec_curr, Int vop_structure, Int field_parity, Int x_pos, Int y_pos, UInt width,
	      Int fblock[][8])
{
    Int i, j;

    Int mem_pos[2];
    
    for (i = 0; i < 8; i++)
      {
	for (j = 0; j < 8; j++)
	  {
	    CalcFieldMemoryPosition_Studio(vop_structure, field_parity,
				     x_pos+j, y_pos+i, width,
				     mem_pos);
	    rec_curr[mem_pos[0]][mem_pos[1]] =
	    /*rec_curr[(y_pos+i)*width + x_pos+j] =*/
		fblock[i][j];
	}
    }
}

/***********************************************************CommentBegin******
 *
 * -- FindCBP_Studio -- Find the CBP for a macroblock 
 *
 * Author :  
 * 
 *
 * Created :  
 * 
 *
 * Purpose :  
 * Find the CBP for a macroblock
 * 
 * Arguments in :  
 * Int *qcoeff : pointer to quantized coefficients
 *  Int Mode : macroblock encoding mode information
 *  Int ncoeffs : the number of coefficients
 *
 * Arguments in/out : 
 * 
 *
 * Arguments out : 
 * 
 *
 * Return values : 
 * Int CBP : The coded block pattern for a macroblock
 *
 * Side effects : 
 * 
 *
 * Description : 
 * 
 *
 * See also :
 * 
 *
 * Modified :  
 * 26.07.99 Osamu Sunohara (SONY): For Studio Profile
 * 
 *
 ***********************************************************CommentEnd********/
Int 
FindCBP_Studio (Int* qcoeff, Int Mode, Int ncoeffs, Int chroma_format)
{
    Int i,j;
    Int CBP = 0;
    Int intra = (Mode == MODE_INTRA || Mode == MODE_INTRA_Q);

    Int block_count;
    if(chroma_format == CHROMA444)
      block_count = 12;
    else if(chroma_format == CHROMA422)
      block_count = 8;
    else if(chroma_format == CHROMA420)
      block_count = 6;
    else
      {
        fprintf(stderr,"chroma_format has the wrong value[%d].\n",chroma_format);
        exit(-1);
      }
    
    /* Set CBP for this Macroblock */

    for (i = 0; i < block_count; i++)
      {
        for (j = i*ncoeffs + intra; j < (i+1)*ncoeffs; j++) 
        {
        
            if (qcoeff[j]) 
            {
                if (i == 0) {CBP |= 32;}
                else if (i == 1) {CBP |= 16;}
                else if (i == 2) {CBP |= 8;}
                else if (i == 3) {CBP |= 4;}
                else if (i == 4) {CBP |= 2;}
                else if (i == 5) {CBP |= 1;}
                else if (i == 6) {CBP |= 128;}
                else if (i == 7) {CBP |= 64;}
                else if (i == 8) {CBP |= 2048;}
                else if (i == 9) {CBP |= 1024;}
                else if (i == 10) {CBP |= 512;}
                else if (i == 11) {CBP |= 256;}
                else 
                {
                    fprintf (stderr, "Error in CBP assignment\n");
                    exit(-1);
                }
    
                break;
            }
        }
    }

    return CBP;
}

/***********************************************************CommentBegin******
 *
 * -- FrameFieldDCTDecide_Studio -- Determine if field DCT is better & swizzle
 *
 * Author :
 *                      Bob Eifrig
 *
 * Created :
 *                      24-jan-97
 *
 * Purpose :
 *                      Determine if the macroblock should be field DCT code
 *
 * Arguments in :
 *
 * Arguments in/out :
 *                      fblock[][][]    An array of 6 8x8 blocks
 *
 * Arguments out :
 *
 * Return values :
 *                      Int FieldDCT : 0 if frame DCT, 1 if field DCT
 *
 * Side effects :
 *                      fblock[][][] is redefined if field DCT is indicated
 *
 * Description :
 *                      fblock can be indexed as fblock[6][8][8].  That is, flbock
 *                      consists of 64 elements of Y1, then 64 elements of Y2, ...
 *
 * See also :
 *
 *
 * Modified :
 * 26.07.99 Osamu Sunohara (SONY): For Studio Profile
 *
 ***********************************************************CommentEnd********/
Int
FrameFieldDCTDecide_Studio(Int *fblock, Int chroma_format, Int decision)
/*decision:0-decision algorithm*/
/*decision:1-force to perform FieldDCT*/
{
    static unsigned /*char*/int mbline[] = {/*AM2PDAM*/
         0*BLOCK_SIZE,  2*BLOCK_SIZE,  4*BLOCK_SIZE,  6*BLOCK_SIZE,
        16*BLOCK_SIZE, 18*BLOCK_SIZE, 20*BLOCK_SIZE, 22*BLOCK_SIZE,
         8*BLOCK_SIZE, 10*BLOCK_SIZE, 12*BLOCK_SIZE, 14*BLOCK_SIZE,
        24*BLOCK_SIZE, 26*BLOCK_SIZE, 28*BLOCK_SIZE, 30*BLOCK_SIZE,  };
    static unsigned /*char*/int fwd_shuffle[] = {/*AM2PDAM*/
         1*BLOCK_SIZE, 0*BLOCK_SIZE,    2*BLOCK_SIZE, 1*BLOCK_SIZE,    4*BLOCK_SIZE, 2*BLOCK_SIZE,
        16*BLOCK_SIZE, 4*BLOCK_SIZE,    0*BLOCK_SIZE,16*BLOCK_SIZE,    3*BLOCK_SIZE, 0*BLOCK_SIZE,
         6*BLOCK_SIZE, 3*BLOCK_SIZE,   20*BLOCK_SIZE, 6*BLOCK_SIZE,   17*BLOCK_SIZE,20*BLOCK_SIZE,
         0*BLOCK_SIZE,17*BLOCK_SIZE,    5*BLOCK_SIZE, 0*BLOCK_SIZE,   18*BLOCK_SIZE, 5*BLOCK_SIZE,
         0*BLOCK_SIZE,18*BLOCK_SIZE,    7*BLOCK_SIZE, 0*BLOCK_SIZE,   22*BLOCK_SIZE, 7*BLOCK_SIZE,
        21*BLOCK_SIZE,22*BLOCK_SIZE,   19*BLOCK_SIZE,21*BLOCK_SIZE,    0*BLOCK_SIZE,19*BLOCK_SIZE,
         9*BLOCK_SIZE, 0*BLOCK_SIZE,   10*BLOCK_SIZE, 9*BLOCK_SIZE,   12*BLOCK_SIZE,10*BLOCK_SIZE,
        24*BLOCK_SIZE,12*BLOCK_SIZE,    0*BLOCK_SIZE,24*BLOCK_SIZE,   11*BLOCK_SIZE, 0*BLOCK_SIZE,
        14*BLOCK_SIZE,11*BLOCK_SIZE,   28*BLOCK_SIZE,14*BLOCK_SIZE,   25*BLOCK_SIZE,28*BLOCK_SIZE,
         0*BLOCK_SIZE,25*BLOCK_SIZE,   13*BLOCK_SIZE, 0*BLOCK_SIZE,   26*BLOCK_SIZE,13*BLOCK_SIZE,
         0*BLOCK_SIZE,26*BLOCK_SIZE,   15*BLOCK_SIZE, 0*BLOCK_SIZE,   30*BLOCK_SIZE,15*BLOCK_SIZE,
        29*BLOCK_SIZE,30*BLOCK_SIZE,   27*BLOCK_SIZE,29*BLOCK_SIZE,    0*BLOCK_SIZE,27*BLOCK_SIZE, };

    static unsigned /*char*/int fwd_shuffle_chr422[] = {/*AM2PDAM*/
        33*BLOCK_SIZE, 0*BLOCK_SIZE,   34*BLOCK_SIZE,33*BLOCK_SIZE,   36*BLOCK_SIZE,34*BLOCK_SIZE,
        48*BLOCK_SIZE,36*BLOCK_SIZE,    0*BLOCK_SIZE,48*BLOCK_SIZE,   35*BLOCK_SIZE, 0*BLOCK_SIZE,
        38*BLOCK_SIZE,35*BLOCK_SIZE,   52*BLOCK_SIZE,38*BLOCK_SIZE,   49*BLOCK_SIZE,52*BLOCK_SIZE,
         0*BLOCK_SIZE,49*BLOCK_SIZE,   37*BLOCK_SIZE, 0*BLOCK_SIZE,   50*BLOCK_SIZE,37*BLOCK_SIZE,
         0*BLOCK_SIZE,50*BLOCK_SIZE,   39*BLOCK_SIZE, 0*BLOCK_SIZE,   54*BLOCK_SIZE,39*BLOCK_SIZE,
        53*BLOCK_SIZE,54*BLOCK_SIZE,   51*BLOCK_SIZE,53*BLOCK_SIZE,    0*BLOCK_SIZE,51*BLOCK_SIZE,

        41*BLOCK_SIZE, 0*BLOCK_SIZE,   42*BLOCK_SIZE,41*BLOCK_SIZE,   44*BLOCK_SIZE,42*BLOCK_SIZE,
        56*BLOCK_SIZE,44*BLOCK_SIZE,    0*BLOCK_SIZE,56*BLOCK_SIZE,   43*BLOCK_SIZE, 0*BLOCK_SIZE,
        46*BLOCK_SIZE,43*BLOCK_SIZE,   60*BLOCK_SIZE,46*BLOCK_SIZE,   57*BLOCK_SIZE,60*BLOCK_SIZE,
         0*BLOCK_SIZE,57*BLOCK_SIZE,   45*BLOCK_SIZE, 0*BLOCK_SIZE,   58*BLOCK_SIZE,45*BLOCK_SIZE,
         0*BLOCK_SIZE,58*BLOCK_SIZE,   47*BLOCK_SIZE, 0*BLOCK_SIZE,   62*BLOCK_SIZE,47*BLOCK_SIZE,
        61*BLOCK_SIZE,62*BLOCK_SIZE,   59*BLOCK_SIZE,61*BLOCK_SIZE,    0*BLOCK_SIZE,59*BLOCK_SIZE, };
    static unsigned /*char*/int fwd_shuffle_chr444[] = {/*AM2PDAM*/
        33*BLOCK_SIZE, 0*BLOCK_SIZE,   34*BLOCK_SIZE,33*BLOCK_SIZE,   36*BLOCK_SIZE,34*BLOCK_SIZE,
        48*BLOCK_SIZE,36*BLOCK_SIZE,    0*BLOCK_SIZE,48*BLOCK_SIZE,   35*BLOCK_SIZE, 0*BLOCK_SIZE,
        38*BLOCK_SIZE,35*BLOCK_SIZE,   52*BLOCK_SIZE,38*BLOCK_SIZE,   49*BLOCK_SIZE,52*BLOCK_SIZE,
         0*BLOCK_SIZE,49*BLOCK_SIZE,   37*BLOCK_SIZE, 0*BLOCK_SIZE,   50*BLOCK_SIZE,37*BLOCK_SIZE,
         0*BLOCK_SIZE,50*BLOCK_SIZE,   39*BLOCK_SIZE, 0*BLOCK_SIZE,   54*BLOCK_SIZE,39*BLOCK_SIZE,
        53*BLOCK_SIZE,54*BLOCK_SIZE,   51*BLOCK_SIZE,53*BLOCK_SIZE,    0*BLOCK_SIZE,51*BLOCK_SIZE,

        41*BLOCK_SIZE, 0*BLOCK_SIZE,   42*BLOCK_SIZE,41*BLOCK_SIZE,   44*BLOCK_SIZE,42*BLOCK_SIZE,
        56*BLOCK_SIZE,44*BLOCK_SIZE,    0*BLOCK_SIZE,56*BLOCK_SIZE,   43*BLOCK_SIZE, 0*BLOCK_SIZE,
        46*BLOCK_SIZE,43*BLOCK_SIZE,   60*BLOCK_SIZE,46*BLOCK_SIZE,   57*BLOCK_SIZE,60*BLOCK_SIZE,
         0*BLOCK_SIZE,57*BLOCK_SIZE,   45*BLOCK_SIZE, 0*BLOCK_SIZE,   58*BLOCK_SIZE,45*BLOCK_SIZE,
         0*BLOCK_SIZE,58*BLOCK_SIZE,   47*BLOCK_SIZE, 0*BLOCK_SIZE,   62*BLOCK_SIZE,47*BLOCK_SIZE,
        61*BLOCK_SIZE,62*BLOCK_SIZE,   59*BLOCK_SIZE,61*BLOCK_SIZE,    0*BLOCK_SIZE,59*BLOCK_SIZE,

        65*BLOCK_SIZE, 0*BLOCK_SIZE,   66*BLOCK_SIZE,65*BLOCK_SIZE,   68*BLOCK_SIZE,66*BLOCK_SIZE,
        80*BLOCK_SIZE,68*BLOCK_SIZE,    0*BLOCK_SIZE,80*BLOCK_SIZE,   67*BLOCK_SIZE, 0*BLOCK_SIZE,
        70*BLOCK_SIZE,67*BLOCK_SIZE,   84*BLOCK_SIZE,70*BLOCK_SIZE,   81*BLOCK_SIZE,84*BLOCK_SIZE,
         0*BLOCK_SIZE,81*BLOCK_SIZE,   69*BLOCK_SIZE, 0*BLOCK_SIZE,   82*BLOCK_SIZE,69*BLOCK_SIZE,
         0*BLOCK_SIZE,82*BLOCK_SIZE,   71*BLOCK_SIZE, 0*BLOCK_SIZE,   86*BLOCK_SIZE,71*BLOCK_SIZE,
        85*BLOCK_SIZE,86*BLOCK_SIZE,   83*BLOCK_SIZE,85*BLOCK_SIZE,    0*BLOCK_SIZE,83*BLOCK_SIZE,

        73*BLOCK_SIZE, 0*BLOCK_SIZE,   74*BLOCK_SIZE,73*BLOCK_SIZE,   76*BLOCK_SIZE,74*BLOCK_SIZE,
        88*BLOCK_SIZE,76*BLOCK_SIZE,    0*BLOCK_SIZE,88*BLOCK_SIZE,   75*BLOCK_SIZE, 0*BLOCK_SIZE,
        78*BLOCK_SIZE,75*BLOCK_SIZE,   92*BLOCK_SIZE,78*BLOCK_SIZE,   89*BLOCK_SIZE,92*BLOCK_SIZE,
         0*BLOCK_SIZE,89*BLOCK_SIZE,   77*BLOCK_SIZE, 0*BLOCK_SIZE,   90*BLOCK_SIZE,77*BLOCK_SIZE,
         0*BLOCK_SIZE,90*BLOCK_SIZE,   79*BLOCK_SIZE, 0*BLOCK_SIZE,   94*BLOCK_SIZE,79*BLOCK_SIZE,
        93*BLOCK_SIZE,94*BLOCK_SIZE,   91*BLOCK_SIZE,93*BLOCK_SIZE,    0*BLOCK_SIZE,91*BLOCK_SIZE, };
    
    Double frame, field, d;
    Int tmp[BLOCK_SIZE], *p, *q, *plast;
    Int i;

    if(decision == 0) {
       
    frame = field = 0.0;
    for (i = 0; i < sizeof(mbline)/sizeof(unsigned int)-1; i++) {
        if (i == (sizeof(mbline)/sizeof(unsigned int)/2 - 1))
            continue;
        p = &fblock[mbline[i + 0]];
        q = &fblock[mbline[i + 1]];
        for (plast = p + BLOCK_SIZE; p < plast; p++, q++) {
            d = p[0] - p[BLOCK_SIZE];
            frame += d * d;
            d = p[BLOCK_SIZE] - q[0];
            frame += d * d;
 
            d = p[0] - q[0];
            field += d * d;
            d = p[BLOCK_SIZE] - q[BLOCK_SIZE];
            field += d * d;
        }
    }
    if (frame <= field)
        return(0);

  }
    

    /*
     * Re-order luminance macroblock lines
     */
    for (i = 0; i < sizeof(fwd_shuffle)/sizeof(unsigned int); i += 2)
        memcpy(fwd_shuffle[i+1] ? &fblock[fwd_shuffle[i+1]] : tmp,
               fwd_shuffle[i+0] ? &fblock[fwd_shuffle[i+0]] : tmp,
               BLOCK_SIZE * sizeof(Int));

    /* Re-order chrominance macroblock lines */
    if(chroma_format==CHROMA444)
        {
            for (i = 0; i < sizeof(fwd_shuffle)/sizeof(unsigned int); i += 2)
                memcpy(fwd_shuffle[i+1] ? &fblock[fwd_shuffle[i+1]+32*BLOCK_SIZE] : tmp,
                       fwd_shuffle[i+0] ? &fblock[fwd_shuffle[i+0]+32*BLOCK_SIZE] : tmp,
                       BLOCK_SIZE * sizeof(Int));
            for (i = 0; i < sizeof(fwd_shuffle)/sizeof(unsigned int); i += 2)
                memcpy(fwd_shuffle[i+1] ? &fblock[fwd_shuffle[i+1]+64*BLOCK_SIZE] : tmp,
                       fwd_shuffle[i+0] ? &fblock[fwd_shuffle[i+0]+64*BLOCK_SIZE] : tmp,
                       BLOCK_SIZE * sizeof(Int));
        }
    else if(chroma_format==CHROMA422)
        {
          for (i = 0; i < sizeof(fwd_shuffle)/sizeof(unsigned int); i += 2)
                memcpy(fwd_shuffle[i+1] ? &fblock[fwd_shuffle[i+1]+32*BLOCK_SIZE] : tmp,
                       fwd_shuffle[i+0] ? &fblock[fwd_shuffle[i+0]+32*BLOCK_SIZE] : tmp,
                       BLOCK_SIZE * sizeof(Int));
/*              memcpy(fwd_shuffle_chr422[i+1] ? &fblock[fwd_shuffle_chr422[i+1]] : tmp,
                       fwd_shuffle_chr422[i+0] ? &fblock[fwd_shuffle_chr422[i+0]] : tmp,
                       BLOCK_SIZE * sizeof(Int));
*/      }
    
    return(1);
}

/***********************************************************CommentBegin******
 *
 * -- fieldDCTtoFrameChroma_Studio -- permute the field DCT block back to frame layout
 *
 * Author :
 *                      Bob Eifrig
 *
 * Created :
 *                      24-jan-97
 *
 * Purpose :
 *                      Undo the field DCT permutation
 *
 * Arguments in :
 *
 * Arguments in/out :
 *                      iblock[][][]    An array of 6 8x8 blocks
 *
 * Arguments out :
 *
 * Return values :
 *
 * Side effects :
 *                      iblock[][][] is re-defined
 *
 * Description :
 *                      iblock can be indexed as fblock[6][8][8].  That is, flbock
 *                      consists of 64 elements of Y1, then 64 elements of Y2, ...
 *
 * See also :
 *
 *
 * Modified :
 * 26.07.99 Osamu Sunohara (SONY): For Studio Profile
 *
 ***********************************************************CommentEnd********/
Void
fieldDCTtoFrameChroma_Studio(Int *iblock, Int chroma_format)
{
    static unsigned /*char*/int inv_shuffle[] = {/*AM2PDAM*/
         1*BLOCK_SIZE, 0*BLOCK_SIZE, 16*BLOCK_SIZE, 1*BLOCK_SIZE,  4*BLOCK_SIZE,16*BLOCK_SIZE,
         2*BLOCK_SIZE, 4*BLOCK_SIZE,  0*BLOCK_SIZE, 2*BLOCK_SIZE,  3*BLOCK_SIZE, 0*BLOCK_SIZE,
        17*BLOCK_SIZE, 3*BLOCK_SIZE, 20*BLOCK_SIZE,17*BLOCK_SIZE,  6*BLOCK_SIZE,20*BLOCK_SIZE,
         0*BLOCK_SIZE, 6*BLOCK_SIZE,  5*BLOCK_SIZE, 0*BLOCK_SIZE, 18*BLOCK_SIZE, 5*BLOCK_SIZE,
         0*BLOCK_SIZE,18*BLOCK_SIZE,  7*BLOCK_SIZE, 0*BLOCK_SIZE, 19*BLOCK_SIZE, 7*BLOCK_SIZE,
        21*BLOCK_SIZE,19*BLOCK_SIZE, 22*BLOCK_SIZE,21*BLOCK_SIZE,  0*BLOCK_SIZE,22*BLOCK_SIZE,
         9*BLOCK_SIZE, 0*BLOCK_SIZE, 24*BLOCK_SIZE, 9*BLOCK_SIZE, 12*BLOCK_SIZE,24*BLOCK_SIZE,
        10*BLOCK_SIZE,12*BLOCK_SIZE,  0*BLOCK_SIZE,10*BLOCK_SIZE, 11*BLOCK_SIZE, 0*BLOCK_SIZE,
        25*BLOCK_SIZE,11*BLOCK_SIZE, 28*BLOCK_SIZE,25*BLOCK_SIZE, 14*BLOCK_SIZE,28*BLOCK_SIZE,
         0*BLOCK_SIZE,14*BLOCK_SIZE, 13*BLOCK_SIZE, 0*BLOCK_SIZE, 26*BLOCK_SIZE,13*BLOCK_SIZE,
         0*BLOCK_SIZE,26*BLOCK_SIZE, 15*BLOCK_SIZE, 0*BLOCK_SIZE, 27*BLOCK_SIZE,15*BLOCK_SIZE,
        29*BLOCK_SIZE,27*BLOCK_SIZE, 30*BLOCK_SIZE,29*BLOCK_SIZE,  0*BLOCK_SIZE,30*BLOCK_SIZE,  };

    static unsigned /*char*/int inv_shuffle_chr422[] = {/*AM2PDAM*/
        33*BLOCK_SIZE, 0*BLOCK_SIZE, 48*BLOCK_SIZE,33*BLOCK_SIZE, 36*BLOCK_SIZE,48*BLOCK_SIZE,
        34*BLOCK_SIZE,36*BLOCK_SIZE,  0*BLOCK_SIZE,34*BLOCK_SIZE, 35*BLOCK_SIZE, 0*BLOCK_SIZE,
        49*BLOCK_SIZE,35*BLOCK_SIZE, 52*BLOCK_SIZE,49*BLOCK_SIZE, 38*BLOCK_SIZE,52*BLOCK_SIZE,
         0*BLOCK_SIZE,38*BLOCK_SIZE, 37*BLOCK_SIZE, 0*BLOCK_SIZE, 50*BLOCK_SIZE,37*BLOCK_SIZE,
         0*BLOCK_SIZE,50*BLOCK_SIZE, 39*BLOCK_SIZE, 0*BLOCK_SIZE, 51*BLOCK_SIZE,39*BLOCK_SIZE,
        53*BLOCK_SIZE,51*BLOCK_SIZE, 54*BLOCK_SIZE,53*BLOCK_SIZE,  0*BLOCK_SIZE,54*BLOCK_SIZE,

        41*BLOCK_SIZE, 0*BLOCK_SIZE, 56*BLOCK_SIZE,41*BLOCK_SIZE, 44*BLOCK_SIZE,56*BLOCK_SIZE,
        42*BLOCK_SIZE,44*BLOCK_SIZE,  0*BLOCK_SIZE,42*BLOCK_SIZE, 43*BLOCK_SIZE, 0*BLOCK_SIZE,
        57*BLOCK_SIZE,43*BLOCK_SIZE, 60*BLOCK_SIZE,57*BLOCK_SIZE, 46*BLOCK_SIZE,60*BLOCK_SIZE,
         0*BLOCK_SIZE,46*BLOCK_SIZE, 45*BLOCK_SIZE, 0*BLOCK_SIZE, 58*BLOCK_SIZE,45*BLOCK_SIZE,
         0*BLOCK_SIZE,58*BLOCK_SIZE, 47*BLOCK_SIZE, 0*BLOCK_SIZE, 59*BLOCK_SIZE,47*BLOCK_SIZE,
        61*BLOCK_SIZE,59*BLOCK_SIZE, 62*BLOCK_SIZE,61*BLOCK_SIZE,  0*BLOCK_SIZE,62*BLOCK_SIZE,  };
    
    static unsigned /*char*/int inv_shuffle_chr444[] = {/*AM2PDAM*/
        33*BLOCK_SIZE, 0*BLOCK_SIZE, 48*BLOCK_SIZE,33*BLOCK_SIZE, 36*BLOCK_SIZE,48*BLOCK_SIZE,
        34*BLOCK_SIZE,36*BLOCK_SIZE,  0*BLOCK_SIZE,34*BLOCK_SIZE, 35*BLOCK_SIZE, 0*BLOCK_SIZE,
        49*BLOCK_SIZE,35*BLOCK_SIZE, 52*BLOCK_SIZE,49*BLOCK_SIZE, 38*BLOCK_SIZE,52*BLOCK_SIZE,
         0*BLOCK_SIZE,38*BLOCK_SIZE, 37*BLOCK_SIZE, 0*BLOCK_SIZE, 50*BLOCK_SIZE,37*BLOCK_SIZE,
         0*BLOCK_SIZE,50*BLOCK_SIZE, 39*BLOCK_SIZE, 0*BLOCK_SIZE, 51*BLOCK_SIZE,39*BLOCK_SIZE,
        53*BLOCK_SIZE,51*BLOCK_SIZE, 54*BLOCK_SIZE,53*BLOCK_SIZE,  0*BLOCK_SIZE,54*BLOCK_SIZE,

        41*BLOCK_SIZE, 0*BLOCK_SIZE, 56*BLOCK_SIZE,41*BLOCK_SIZE, 44*BLOCK_SIZE,56*BLOCK_SIZE,
        42*BLOCK_SIZE,44*BLOCK_SIZE,  0*BLOCK_SIZE,42*BLOCK_SIZE, 43*BLOCK_SIZE, 0*BLOCK_SIZE,
        57*BLOCK_SIZE,43*BLOCK_SIZE, 60*BLOCK_SIZE,57*BLOCK_SIZE, 46*BLOCK_SIZE,60*BLOCK_SIZE,
         0*BLOCK_SIZE,46*BLOCK_SIZE, 45*BLOCK_SIZE, 0*BLOCK_SIZE, 58*BLOCK_SIZE,45*BLOCK_SIZE,
         0*BLOCK_SIZE,58*BLOCK_SIZE, 47*BLOCK_SIZE, 0*BLOCK_SIZE, 59*BLOCK_SIZE,47*BLOCK_SIZE,
        61*BLOCK_SIZE,59*BLOCK_SIZE, 62*BLOCK_SIZE,61*BLOCK_SIZE,  0*BLOCK_SIZE,62*BLOCK_SIZE,

        65*BLOCK_SIZE, 0*BLOCK_SIZE, 48*BLOCK_SIZE,33*BLOCK_SIZE, 36*BLOCK_SIZE,48*BLOCK_SIZE,
        66*BLOCK_SIZE,68*BLOCK_SIZE,  0*BLOCK_SIZE,34*BLOCK_SIZE, 35*BLOCK_SIZE, 0*BLOCK_SIZE,
        81*BLOCK_SIZE,67*BLOCK_SIZE, 52*BLOCK_SIZE,49*BLOCK_SIZE, 38*BLOCK_SIZE,52*BLOCK_SIZE,
         0*BLOCK_SIZE,70*BLOCK_SIZE, 37*BLOCK_SIZE, 0*BLOCK_SIZE, 50*BLOCK_SIZE,37*BLOCK_SIZE,
         0*BLOCK_SIZE,82*BLOCK_SIZE, 39*BLOCK_SIZE, 0*BLOCK_SIZE, 51*BLOCK_SIZE,39*BLOCK_SIZE,
        85*BLOCK_SIZE,83*BLOCK_SIZE, 54*BLOCK_SIZE,53*BLOCK_SIZE,  0*BLOCK_SIZE,54*BLOCK_SIZE,

        73*BLOCK_SIZE, 0*BLOCK_SIZE, 88*BLOCK_SIZE,73*BLOCK_SIZE, 76*BLOCK_SIZE,88*BLOCK_SIZE,
        74*BLOCK_SIZE,76*BLOCK_SIZE,  0*BLOCK_SIZE,74*BLOCK_SIZE, 75*BLOCK_SIZE, 0*BLOCK_SIZE,
        89*BLOCK_SIZE,75*BLOCK_SIZE, 92*BLOCK_SIZE,89*BLOCK_SIZE, 78*BLOCK_SIZE,92*BLOCK_SIZE,
         0*BLOCK_SIZE,78*BLOCK_SIZE, 77*BLOCK_SIZE, 0*BLOCK_SIZE, 90*BLOCK_SIZE,77*BLOCK_SIZE,
         0*BLOCK_SIZE,90*BLOCK_SIZE, 79*BLOCK_SIZE, 0*BLOCK_SIZE, 91*BLOCK_SIZE,79*BLOCK_SIZE,
        93*BLOCK_SIZE,91*BLOCK_SIZE, 94*BLOCK_SIZE,93*BLOCK_SIZE,  0*BLOCK_SIZE,94*BLOCK_SIZE,  };

    Int tmp[BLOCK_SIZE];
    Int i;
  
/*    for (i = 0; i < sizeof(inv_shuffle); i += 2)
        memcpy(inv_shuffle[i+1] ? &iblock[inv_shuffle[i+1]] : tmp,
               inv_shuffle[i+0] ? &iblock[inv_shuffle[i+0]] : tmp,
                BLOCK_SIZE*sizeof(Int));
*/
    if(chroma_format == CHROMA444)
        {
            for (i = 0; i < sizeof(inv_shuffle)/sizeof(unsigned int); i += 2)
	    /*for (i = 0; i < sizeof(inv_shuffle); i += 2)*//*BugFix_011203*/
	        memcpy(inv_shuffle[i+1] ? &iblock[inv_shuffle[i+1]+32*BLOCK_SIZE] : tmp,
		       inv_shuffle[i+0] ? &iblock[inv_shuffle[i+0]+32*BLOCK_SIZE] : tmp,
		       BLOCK_SIZE*sizeof(Int));
            for (i = 0; i < sizeof(inv_shuffle)/sizeof(unsigned int); i += 2)
	    /*for (i = 0; i < sizeof(inv_shuffle); i += 2)*//*BugFix_011203*/
	        memcpy(inv_shuffle[i+1] ? &iblock[inv_shuffle[i+1]+64*BLOCK_SIZE] : tmp,
		       inv_shuffle[i+0] ? &iblock[inv_shuffle[i+0]+64*BLOCK_SIZE] : tmp,
		       BLOCK_SIZE*sizeof(Int));
	}
    else if(chroma_format == CHROMA422)
        {
	  for (i = 0; i < sizeof(inv_shuffle); i += 2)
	        memcpy(inv_shuffle[i+1] ? &iblock[inv_shuffle[i+1]+32*BLOCK_SIZE] : tmp,
		       inv_shuffle[i+0] ? &iblock[inv_shuffle[i+0]+32*BLOCK_SIZE] : tmp,
		       BLOCK_SIZE*sizeof(Int));
/*	        memcpy(inv_shuffle_chr422[i+1] ? &iblock[inv_shuffle_chr422[i+1]] : tmp,
		       inv_shuffle_chr422[i+0] ? &iblock[inv_shuffle_chr422[i+0]] : tmp,
		       BLOCK_SIZE*sizeof(Int));
*/	}

}

Int
BitstreamPutUncompressedBlock(Int *rawblock,
			      Int bits_per_pixel,
			      Image *mb_raw_bitstream) {
  Int i, j;
  Int bitcount=0;

  Int *rawblock_ind = rawblock;
  
  for(j=0;j<8;j++)
    for(i=0;i<8;i++) {
      BitstreamPutBits(mb_raw_bitstream,*rawblock_ind,bits_per_pixel);
      bitcount += bits_per_pixel;
      BitstreamPutBits(mb_raw_bitstream,1,1);
      bitcount += 1;
      rawblock_ind++;
    }

  return bitcount;
}

Int *
CodeMB_DPCM_Studio(Vop *curr,
		   Int x_pos, Int y_pos,
		   Image *mb_dpcm_bitstream,
		   Int bits_mbraw[],
		   Int block_mean[]) {
  
  Int vop_structure = GetVopVopStructure_st(curr);
  Int fip = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(curr),
				   GetVopTopFieldFirst(curr));
  Int chroma_format = GetVopChromaFormat_st(curr);
  Int bits_per_pixel = GetVopBitsPerPixel(curr);
  Int vop_width = GetVopWidth_st(curr, fip);
  Int vop_height = GetVopHeight_st(curr, fip);
  
  Int *rawmb, *rawmb_ind;/*AM2PDAM*/
  
  Int fblock[12][8][8];
  SInt *current[2], *recon[2] = {NULL,NULL};
  Int x, y;
  UInt xwidth;
  Int i, j, k;
  Int kk;

  Int ii, jj;
  
  SInt compblock[3][256];

  Int dpcm_scan_order = 0;
  
#if 0
  Image *Sub_Alpha= AllocImage(MB_SIZE, MB_SIZE, SHORT_TYPE);
  SInt  *sub_alpha_ptr;
#endif
  
  Int num_transp = 0;
  
/*  Int binary_number;*/

  Int block_count;
  Int cdivx, cdivy;
  
  if(chroma_format == CHROMA444) {
    block_count = 12;
    cdivx = 1;
    cdivy = 1;
  } else if(chroma_format == CHROMA422) {
    block_count = 8;
    cdivx = 2;
    cdivy = 1;
  } else {
    block_count = 6;
    cdivx = 2;
    cdivy = 2;
  }

#if 0
  /*AM2PDAM*/
  binary_number = 0;
  if(GetVopShape(curr) == GREY_SCALE) {
    for(i=0;i<MAX_AUX_COMP_NUM;i++) {
      if(GetVopAuxCompType_st(curr,i) == AUX_COMP_ALPHA) {
        binary_number = i;
        break;
      }
    }
  }

  /* Reconstructed BinaryAlpha Macroblock */
  GetSubImage_Studio(GetVopA2_st(rec_curr,binary_number), vop_structure, fip,
		     Sub_Alpha, x_pos, y_pos);
  sub_alpha_ptr = (SInt *)GetImageData(Sub_Alpha);
  /*AM2PDAM*/
#endif 0
  
  rawmb_ind = rawmb = (Int *)malloc(sizeof(Int)*12*64);
  
  /* Get fblock(inputed block) */
  for (k = 0; k < block_count; k++) {
    switch (k) {
    case 0:
      x = x_pos;
      y = y_pos;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
      break;
    case 1:
      x = x_pos + 8;
      y = y_pos;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
      break;
    case 2:
      x = x_pos;
      y = y_pos + 8;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
      break;
    case 3:
      x = x_pos + 8;
      y = y_pos + 8;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopY_st (curr,kk));
      break;
    case 4:
      x = x_pos / cdivx;
      y = y_pos / cdivy;
      xwidth = vop_width / cdivx;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
      break;
    case 5:
      x = x_pos / cdivx;
      y = y_pos / cdivy;
      xwidth = vop_width / cdivx;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
      break;
    case 6:
      x = x_pos / cdivx;
      y = y_pos + 8;
      xwidth = vop_width / cdivx;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
      break;
    case 7:
      x = x_pos / cdivx;
      y = y_pos + 8;
      xwidth = vop_width / cdivx;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
      break;
    case 8:
      x = x_pos + 8;
      y = y_pos;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
      break;
    case 9:
      x = x_pos + 8;
      y = y_pos;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
      break;
    case 10:
      x = x_pos + 8;
      y = y_pos + 8;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopU_st (curr,kk));
      break;
    case 11:
      x = x_pos + 8;
      y = y_pos + 8;
      xwidth = vop_width;
      for(kk=0;kk<2;kk++)
	current[kk] = (SInt *) GetImageData (GetVopV_st (curr,kk));
      break;
    default:
      fprintf (stdout, "CodeMB (): Non-existing case in switch statement\n");
      exit (-1);
      break;
    }
    BlockPredict_Studio(current, vop_structure, fip, recon, x, y, xwidth, fblock[k]);
  }/* for (k = 0; k < block_count; k++) */

  /*Get rawmb*/
  for(k=0;k<block_count;k++)
    for(i=0;i<8;i++)
      for(j=0;j<8;j++) {
	*rawmb_ind = fblock[k][i][j];
	rawmb_ind++;
      }

  /*Get compblock*/
  for(k=0;k<4;k++)
    for(j=0;j<8;j++)
      for(i=0;i<8;i++)
	compblock[0][((k/2)*8+j)*16+((k%2)*8+i)] = fblock[k][j][i];
  for(k=4;k<block_count;k++)
    for(j=0;j<8;j++)
      for(i=0;i<8;i++)
	compblock[k%2+1][(((k/2-2)%2)*8+j)*16/cdivx+(((k/2-2)/2)*8+i)] = fblock[k][j][i];

  /* MoMuSys-2000FDAM1COR-020612 */
  /* marker_bit */
  BitstreamPutBits(mb_dpcm_bitstream,1,1);
  /**/
  
  BitstreamPutBits(mb_dpcm_bitstream,dpcm_scan_order,1);
  /*bits_dpcm_header = 1;*/

  bits_mbraw[0] = BitstreamPutDPCMBlock(compblock[0], 16, 16, bits_per_pixel,
					dpcm_scan_order,
					mb_dpcm_bitstream,
					&block_mean[0]);
  for(i=1;i<3;i++)
    bits_mbraw[i] = BitstreamPutDPCMBlock(compblock[i], 16/cdivx, 16/cdivy, bits_per_pixel,
					  dpcm_scan_order,
					  mb_dpcm_bitstream,
					  &block_mean[i]);
  return rawmb;
}

/* 01.04.01: Studio Profile */
