/*****************************************************************************
 *
 * This software module was originally developed by
 * 
 *   Michael Wollborn (TUH / ACTS-MoMuSyS)
 *   Sylvie Jeannin (Philips / ACTS-MoMuSyS)
 *
 * and edited by
 * 
 *   Ferran Marques (UPC / ACTS-MoMuSyS)
 *   Noel Brady (Teltec DCU / ACTS-MoMuSyS) 
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS) 
 *   Aasmund Sandvand (Telenor / ACTS-MoMuSyS)     	                    
 *   Gunn Klungsoeyr (Telenor / ACTS-MoMuSyS)     	                    
 *   Frederic Dufaux (Digital Equipment Corp.)        	                  
 *   Cecile Dufour (Philips LEP / ACTS-MoMuSys)        	                   
 *   Michael Frater (UNSW)
 *   Ji Heon Kweon (HYUNDAI)
 *   Jong Deuk Kim (HYUNDAI)
 *   Marc Mongenet (EPFL)
 *   Sven Brandau, Guido Heising (HHI)
 *   Seishi TAKAMURA (NTT)
 *   Oki Electric Industry Co., Ltd. (contact: Shigeru Fukunaga)
 *   Fujitsu Laboratories Ltd. (contact: Eishi Morimatsu)
 *   Dae-Sung Cho (Samsung AIT)
 *   Massimo Ravasi (EPFL)
 *   Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1997
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                          
 * File:	vm_dec_main.c
 *
 * Author:	Michael Wollborn (TUH)
 * Created:	08-AUG-1996
 *                                                                         
 * Description: 
 *	Main modules necessary for decoding an MPEG-4 Video (ISO/IEC 14496-2)
 *      bitstream.
 *
 * Notes: 	
 *
 * Modified : 17-SEP-1996, M.Wollborn, included changes for the new MMR shape
 *			   decoding 
 *	      12-OCT-1996, F. Marques, to use the new names for access
 *			   functions introduced by N. O'connor 
 *            14-Jan-1997, Luis Ducla-Soares added byte alignment for
 *			   VOP_START_CODE
 *                         in function DecodeVol() and for VOL_START_CODE in
 *                         function DecodeVolHeader().
 *            02-Feb-1997, Aa. Sandvand, moved deblocking filter out of loop
 *	      04-Feb-1997, Noel O'Connor : removed all calls to assert()
 *	      07-Feb-1997, M.Wollborn: changed in order to treat skipped MBs
 *			   correctly
 *	      12.02.97 M.Wollborn: Changed to avoid double writing of last VOP
 *	23.04.97 Michael Wollborn: Added MPEG quantization stuff completely
 *      26.04.97 Luis Ducla Soares: Changes to allow error resilient
 *                                  decoding in the combined mode with
 *                                  data partitioning.      
 *	09.05.97 Minhua Zhou: added "DecodeGOV"
 *      14.05.97 F. Jaureguizar: Modification of number of bits of fcodes in
 *               DecodeVolHeader(). From 2 bits to 3 bits.
 *      20.06.97 M.Wollborn Extended trace mechanism for bitstream exchange
 *      07.07.97 A. Sandvand: Added support for random access
 *      24.07.97 Minhua Zhou: added B-VOPs in the decoder
 *	30.07.97 F. Jaureguizar: Integrated spatial scalability (SpSc)
 *      04.08.97 Minhua Zhou : renamed advanced_prediction_disable
 *      28.08.97 Osamu Sunohara(Sony): modified for spatial scalable coding
 *      15.09.97 A. Sandvand: Updated random access
 *      23.10.97 Minhua Zhou: changed deblock_filter_disable to 
 *                            post_filter_type
 *      03.01.97 Michael Frater: Support for non-8-bit video
 *	07.05.98 Jong Deuk Kim (jdkim97@hei.co.kr, HYUNDAI) : interlaced tools
 *	21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 *      09.06.98 Marc Mongenet (EPFL): added Complexity Estimation syntax support
 *      23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header
 *      11.08.98 Sven Brandau: changes due to N2339
 *      31.08.98 Guido Heising: non version 1 items removed 
 *      11.09.98 Guido Heising: -temporal references for temporal scalab. changed
 *                              -changes concerning error res. and scalability 
 *      15.02.99 U. Benzler : added quarter pel support
 *	03.03.99 Seishi TAKAMURA (NTT): added GMC coding
 *	16.08.99 Shigeru Fukunaga (Oki): added and modified modules for NEWPRED
 *	18.08.99 Seishi TAKAMURA (NTT): modify GMC coding  (warping type check)
 *	06.09.99 Eishi Morimatsu (Fujitsu Labs.): added DRC support
 *      26.10.99 Massimo Ravasi (EPFL): added Complexity Estimation syntax support
 *                                  Update version 2
 *      11.11.99 U. Benzler (UoH): corrected input of SADCT flag to be done only for Version2
 *      01.04.01 Osamu Sunohara (SONY): Functions for Studio Profile
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include <string.h>
#include <sys/stat.h>
#include "momusys.h"
#include "mom_bitstream_d.h"
#include "vm_compos.h"
#include "text_decode.h"
#include "mot_padding.h"
#include "io_generic.h"
#include "newpred_d.h"	/** added for NEWPRED (Oki) 16-AUG-1999 **/
#include "vm_dec_main.h"
#include "mom_vol.h"
#include "mom_access.h"
#include "mot_comp.h"
#include "zigzag.h"
#include "post_filter.h"
#include "mom_vo.h"
#include "combined_decode.h"
#include "vm_common_defs.h"
#include "sprite_util.h"
#include "sprite_dec_util.h"
#include "sprite_dec_piece.h"
#include "alp_dec_mom.h"
#include "alp_dec_header.h"
#include "alp_dec_mc.h" /* added for OBSS by Samsung AIT (1999-09-29) */
#include "text_quant_mat_def.h"

#include "dwt.h"
#include "dataStruct.h"
#include "interface.h"

#include "do_bgc.h" 
#include "scal_dec.h"
#include "startcode.h"
#include "sadct.h"
#include "sadct_kaup.h"
#include "sadct_s_k.h"

/* 01.04.01: Studio Profile */
#include "studio_sprite_defocus_filter.h"

#ifndef SEEK_SET
#define SEEK_SET 0
#endif


/*****
 *
 *    Scalability settings
 *
 *****/

Vop *base_output_vop[3];
Float display_framerate_TPS    = 0;
Int temporal_scalability       = 0,    /* 0 is off 1 is onn */
    TPS_base_count             = 0,
    last_base_decoding_time[3] = {-1, -1, -1},
    last_enh_decoding_time     = 0;

Int not_scalability = 1;

/*****
 *
 *    Set control for full-trace frames and default trace levels 
 *    for the other frames (1 - trace enabled, 0 - trace disabled)
 *
 *    Refer to the README file for more details.
 *
 *****/

/* Control for full-trace-frames */
#define FULL_TRACE_START	0
#define FULL_TRACE_END		300
#define FULL_TRACE_PERIOD	1

/* Default trace level settings */

/* Level 1 */
#define TRACE_LEVEL_1		1
#define	TRACE_VO_HEADER		1
#define	TRACE_VOL_HEADER	1
#define	TRACE_VOP_HEADER	1

/* Level 2 */
#define TRACE_LEVEL_2		1
#define	TRACE_MB_HEADER		1
#define	TRACE_MB_MOTION		1
#define	TRACE_MB_SHAPE		1
#define	TRACE_MB_TEXTURE        1

/* Level 3 */
#define TRACE_LEVEL_3			1
#define	TRACE_MB_SDATA		        1
#define	TRACE_MB_TDATA_BEFORE_IQ	1
#define	TRACE_MB_TDATA_AFTER_IQ		1
#define	TRACE_MB_TDATA_AFTER_IDCT	1
#define	TRACE_MB_TDATA_PREDICTED	1
#define	TRACE_MB_TDATA_RECONSTRUCTED	1


/***********************************************************CommentBegin******
 *
 * -- main -- Decodes an MPEG4 bitstream and composes the decoded VOPs
 *
 * Author :		
 *	Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :		
 *	Handle the decoding of the MPEG-4 bitstream, and compose the decoded
 *	VOPs into a sequence with a given display rate, width and height.
 * 
 *
 * Arguments in : 	
 *	
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	27.11.96 - Noel O'Connor : Changed the variables "next_display_time" 
 *	and "time_between_two_frames" to Floats. Also added "intelligent" 
 *	rounding
 *	to avoid accumulation of Floating point errors.
 *	15.01.97 Robert Danielsen: Changed return type, added exit().
 *			Solution given by Paulo Nunes.
 *	12.02.97 M.Wollborn: Changed to avoid double writing of last VOP
 *	15-APR-97 Jan De Lameillieure : added initialisation of SA-DCT
 *      18.06.97 M.Wollborn: Added correct tracing mechanism
 *      07.07.97 A. Sandvand: Added support for random access
 *      24.07.97 Minhua Zhou: added B-VOPs in the decoder
 *      29.08.97 Osamu Sunohara: modified to make composit image exactly
 *                               in spatial scalable coding.
 *      15.09.97 A. Sandvand: Updated random access
 *      25.03.98 M.Wollborn: changes due to N2171, Cl. 2.1.9 (user data)
 *      29.03.98 M.Wollborn: cleaning of code, just editorial
 *      23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *	16.08.99 Shigeru Fukunaga (Oki): added and modified modules for NEWPRED
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/

Int main (Int argc, Char *argv[])
{
  Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS];
  Trace *trace[MAX_NUM_VOS][MAX_NUM_VOLS];
  Vol *display_vol_list[MAX_NUM_VOS];
  Vop *display_vop=NULL;
  
  Char bitstream_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300], 
       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
       vol_user_data_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300], /* N2171 */
       gov_user_data_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300], /* N2171 */
       output_filename[4][300];

  Float display_framerate;         /* frames/s for display given by the user */

  Int display_width,		   /* width of images to be written to disk */
      display_height,		   /* height of images to be written to disk */
      number_of_vos,	           /* number of video objects */
      number_of_vols[MAX_NUM_VOS], /* number of layers for each VO*/  
      image_counter,		   /* Counter for output images */
      post_filter_type,            /* status of deblocking filter */
      bits_per_pixel,              /* Pixel depth of display */
      random_access_start_time,    /* starttime for decoding */
      short_video_header[MAX_NUM_VOS], /* type of video object */
      stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
      readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS];	

  Float time_between_two_frames,   /* number of ms between 2 displ. frames */
	next_display_time;	   /* next absolute time for displ. a frame */

  Int i,k;

  Int erd_flag[MAX_NUM_VOS][MAX_NUM_VOLS]; /* MW 14-MAY-1998 */


  /* VTC Begin -----------------------------------------------*/
 /*  Int NoVOLflag = TRUE;  :unused SB */
  /*  Int num_VOLs=0;   :unused SB */

  File *VTCfptr;
  Int  number_of_vtcs,
       vo,shape_width,shape_height,
       target_bitrate, 
			 target_shape_level,     /* SAIT_V2: added by Samsung AIT (99/02/23) */
       target_spatial_level, 
       target_snr_level;
	Int  target_tile_id_from, target_tile_id_to; /* added by sharp (99/2/16) */
  Char VTC_bitstream_filename[300], 
       VTC_output_filename[300], 
       VTC_shape_filename[300];
  /* VTC End -----------------------------------------------*/

  /* 1197-3 SONY */
  /* added by Sony 010997 */
  /*  Int not_scalability = 1;*/
  Vol *next_vol = NULL;
  /* 010997 */
  /* 1197-3 */
 
  /*included for combination of error res. and scalability (Guido Heising, HHI, 11.09.98)*/
  static ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS], *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS];
  static Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS]; 

  /** added for NEWPRED (Oki) 16-AUG-1999 **/
  NpConfig	**np_cfg;	/* config structure for NEWPRED */
  /** end of NEWPRED (Oki) 16-AUG-1999 **/

/* 01.04.01: Studio Profile */
  Int studio_profile;
  Char output_filename_aux_comp[MAX_AUX_COMP_NUM][100];
  Int display_chroma_format; /*chroma_format*/
  Int gray_bits_per_pixel[MAX_AUX_COMP_NUM]; /*gray_bits_per_pixel*/
/* 01.04.01: Studio Profile */

  /*****
   *
   *	Get the arguments from the user:
   *
   *	Number of VOs, layers for each VO and the filenames for the
   *	respective bitstreams (each VOL is stored in a separate bitstream).
   *
   *	Name and path of the output image sequence file(s) and display
   *	frame rate
   *
   *****/

  /* VTC Begin -----------------------------------------------*/
#if 0
  VTCfptr = GetDecoderArguments_Studio (argc, argv, &number_of_vos, number_of_vols, 
					&number_of_vtcs, /* VTC */
					bitstream_filename, vol_user_data_filename,
					gov_user_data_filename, output_filename,
					&display_framerate,
					&display_width,
					&display_height,
					&post_filter_type,
					&bits_per_pixel,
					&random_access_start_time,
					&studio_profile,
					output_filename_aux_comp,
					&display_chroma_format,
					gray_bits_per_pixel,
					erd_flag);

#endif 0
  VTCfptr = GetDecoderArguments (argc, argv, &number_of_vos, number_of_vols, 
                                 &number_of_vtcs, /* VTC */
                                 bitstream_filename, vol_user_data_filename,
				 gov_user_data_filename, output_filename, 
                                 &display_framerate,&display_width,
				 &display_height,
                                 &post_filter_type,&bits_per_pixel,
                                 &random_access_start_time,erd_flag);
  
  for (vo=0; vo<number_of_vtcs; vo++) 
    {
      GetVTCDecoderArguments (VTCfptr,
			      VTC_bitstream_filename, 
			      VTC_output_filename, 
			      VTC_shape_filename, 
			      &shape_width,
			      &shape_height,
			      &target_bitrate, 
						&target_shape_level,    /* SAIT_V2: added by Samsung AIT (99/02/23) */
			      &target_spatial_level,
			      &target_snr_level,
						&target_tile_id_from,/* modified by Sharp (99/2/16) */
            &target_tile_id_to /* modified by Sharp (99/2/16) */
						);
      VTC_Decode(VTC_bitstream_filename, 
					 VTC_output_filename,
					 VTC_shape_filename,
					 shape_width,
					 shape_height,
					 target_bitrate, 
					 target_shape_level,     /* SAIT_V2: added by Samsung AIT (99/02/23) */
					 target_spatial_level, 
					 target_snr_level,
					 target_tile_id_from,/* modified by Sharp (99/2/16) */
					 target_tile_id_to /* modified by Sharp (99/2/16) */
					 );
    }

  /* 01.04.01: Studio Profile */
  GetStudioDecoderArguments_Studio (VTCfptr,
				    &studio_profile,
				    &display_chroma_format,
				    output_filename_aux_comp,
				    gray_bits_per_pixel);

  
  fclose(VTCfptr);
	if ( number_of_vtcs != 0 )
		exit(0);
  /* VTC End -----------------------------------------------*/

  
  /*****
   *
   *	Allocate memory for the display_vop which is used for composition
   *	purposes
   *
   *****/

/* 01.04.01: Studio Profile */
  if(studio_profile) {
    display_vop = AllocFrameVop_Studio();/*As FRAME_VOP*/
    AllocVopChannels_Studio(display_vop, FRAME_VOP, 0,
			    display_width,
			    display_height,
			    display_chroma_format);
    PutVopBitsPerPixel(bits_per_pixel, display_vop);
    for(i=0;i<MAX_AUX_COMP_NUM;i++)
      PutVopGrayBitsPerPixel_st(gray_bits_per_pixel[i], display_vop, i);
    PutVopShape(GREY_SCALE,display_vop);

    /* Initialize Default VLC tables */
    InitializeStdioGroupTable_Studio();
  }
/* 01.04.01: Studio Profile */  
  
        /* display_vop = AllocVop(display_width,display_height,0);  - at this point, aux_comp_count is unknown! (SB) */
/*  PutVopBitsPerPixel(bits_per_pixel, display_vop); */

  /*****
   *
   *	Calculate the time between two frames to be displayed in ms
   *	(from display_frame_rate) and allocate the necessary VOL, bitstream
   *	and trace structures
   *
   *****/
  time_between_two_frames = (Float)1000 / display_framerate;

  /* for TPS */
  for (i=0; i<MAX_NUM_VOS; i++) 
    {
      bs_vo[i] = -1;
      for (k=0; k<MAX_NUM_VOLS; k++) 
	bs_vol[i][k] = -1;
    }
  /**/

  for(i=0; i<number_of_vos; i++)
    {
      /* for TPS */
      bs_vo[i] = i; /* This part needs to change when VO header is decoded. */
      /**/
      short_video_header[i] = 0;

/* 01.04.01: Studio Profile */
      if(studio_profile)
	display_vol_list[i] = InitVolList_Studio(number_of_vols[i]);/*vop->images aren't allocated.*/
      else
/* 01.04.01: Studio Profile */      
      display_vol_list[i] = InitVolList(number_of_vols[i]);
      
      for(k=0; k<number_of_vols[i]; k++)
	{
	  stream[i][k] = BitstreamOpen(bitstream_filename[i][k]);
	  readen_bits[i][k] = 0;
	  stop_decoding_vol[i][k] = 0;

	  strcpy(trace_filename[i][k], bitstream_filename[i][k]);
	  strcat(trace_filename[i][k], ".dec_log");
	  
	  /*****
	   *
	   *    Trace capabilities. The settings by the users should be
	   *    done at the top of this file!
	   *
	   *    Please DO NOT MODIFY any values below here !
	   *
	   *****/
	  
#ifdef _DEBUG_
	  
	  trace[i][k] = allocate_trace_file(trace_filename[i][k], "w");
	  
	  trace[i][k]->counter              = 0; 
	  trace[i][k]->trace                = 0; 

	  /* Set frames for full trace (see README for details) */
	  trace[i][k]->full_trace_start     = FULL_TRACE_START; 
	  trace[i][k]->full_trace_end       = FULL_TRACE_END; 
	  trace[i][k]->full_trace_period    = FULL_TRACE_PERIOD; 

	  /* Set default trace levels (see README for details) */
	  
	  /* Level 1 data */
	  if(TRACE_LEVEL_1)
	    {
	      trace[i][k]->VO_header_def  = TRACE_VO_HEADER;
	      trace[i][k]->VOL_header_def = TRACE_VOL_HEADER;
	      trace[i][k]->VOP_header_def = TRACE_VOP_HEADER;
	    }
	  else
	    {
	      trace[i][k]->VO_header_def  = 0;
	      trace[i][k]->VOL_header_def = 0;
	      trace[i][k]->VOP_header_def = 0;
	    }
	  
	  /* Level 2 data */
	  if(TRACE_LEVEL_2)
	    {
	      trace[i][k]->MB_header_def  = TRACE_MB_HEADER;
	      trace[i][k]->MB_motion_def  = TRACE_MB_MOTION;
	      trace[i][k]->MB_shape_def   = TRACE_MB_SHAPE;
	      trace[i][k]->MB_texture_def = TRACE_MB_TEXTURE;
	    }
	  else
	    {
	      trace[i][k]->MB_header_def  = 0;
	      trace[i][k]->MB_motion_def  = 0;
	      trace[i][k]->MB_shape_def   = 0;
	      trace[i][k]->MB_texture_def = 0;
	    }
	  
	  /* Level 3 data */
	  if(TRACE_LEVEL_3)
	    {
	      trace[i][k]->MB_sdata_def               = TRACE_MB_SDATA;
	      trace[i][k]->MB_tdata_before_IQ_def     = TRACE_MB_TDATA_BEFORE_IQ;
	      trace[i][k]->MB_tdata_after_IQ_def      = TRACE_MB_TDATA_AFTER_IQ;
	      trace[i][k]->MB_tdata_after_IDCT_def    = TRACE_MB_TDATA_AFTER_IDCT;
	      trace[i][k]->MB_tdata_predicted_def     = TRACE_MB_TDATA_PREDICTED;
	      trace[i][k]->MB_tdata_reconstructed_def = TRACE_MB_TDATA_RECONSTRUCTED;
	    }
	  else
	    {
	      trace[i][k]->MB_sdata_def               = 0;
	      trace[i][k]->MB_tdata_before_IQ_def     = 0;
	      trace[i][k]->MB_tdata_after_IQ_def      = 0;
	      trace[i][k]->MB_tdata_after_IDCT_def    = 0;
	      trace[i][k]->MB_tdata_predicted_def     = 0;
	      trace[i][k]->MB_tdata_reconstructed_def = 0;
	    }
	  
#else
	  trace[i][k] = (Trace *) malloc( sizeof(Trace) );
	  
	  trace[i][k]->counter              = 0; 
	  trace[i][k]->trace                = 0; 
	  
	  /* Set frames for full trace (see README for details) */
	  trace[i][k]->full_trace_start     = 0; 
	  trace[i][k]->full_trace_end       = 0; 
	  trace[i][k]->full_trace_period    = 0; 
	  
	  /* Set default trace levels (see README for details) */
	  
	  /* Level 1 data */
	  trace[i][k]->VO_header_def  = 0;
	  trace[i][k]->VOL_header_def = 0;
	  trace[i][k]->VOP_header_def = 0;
	  
	  /* Level 2 data */
	  trace[i][k]->MB_header_def  = 0;
	  trace[i][k]->MB_motion_def  = 0;
	  trace[i][k]->MB_shape_def   = 0;
	  trace[i][k]->MB_texture_def = 0;
	  
	  /* Level 3 data */
	  trace[i][k]->MB_sdata_def               = 0;
	  trace[i][k]->MB_tdata_before_IQ_def     = 0;
	  trace[i][k]->MB_tdata_after_IQ_def      = 0;
	  trace[i][k]->MB_tdata_after_IDCT_def    = 0;
	  trace[i][k]->MB_tdata_predicted_def     = 0;
	  trace[i][k]->MB_tdata_reconstructed_def = 0;
	  
#endif

	}

/* 01.04.01: Studio Profile */
      if(studio_profile)
	/* This function includes DecodeVolHeader_Studio() which outputs the total bits for VOL */
	DecodeVolHeaderList_Studio(number_of_vols[i],stream[i],trace[i],
				   display_vol_list[i],readen_bits[i],i,
				   vol_user_data_filename[i],erd_flag[i], /* N2171 user data */ /* erd_flag MW 14-MAY-1998 */
				   &short_video_header[i],
				   mot_x_P,mot_y_P,MB_decisions_P);
      else
/* 01.04.01: Studio Profile */            
      DecodeVolHeaderList(number_of_vols[i],stream[i],trace[i],
                          display_vol_list[i],readen_bits[i],i,
                          vol_user_data_filename[i],erd_flag[i], /* N2171 user data */ /* erd_flag MW 14-MAY-1998 */
                          &short_video_header[i],
                          mot_x_P,mot_y_P,MB_decisions_P);

     }

  /*****
   *
   *	Initializations
   *
   *****/

  /* Random access */
  if (random_access_start_time != 0)
    random_access_start_time = 
      ((Int)(random_access_start_time + 0.5*time_between_two_frames) / 
       (Int) time_between_two_frames) * time_between_two_frames;
  if (random_access_start_time != 0)
    printf("Adjusted random_access_start_time=%i\n",random_access_start_time);
  next_display_time = 0;
  
  /* Scalability */
  image_counter = 0;
  for(i = 0; i < 3 ; i++) 
    base_output_vop[i] = NULL;

  display_framerate_TPS = display_framerate;
  image_width  = display_width;
  image_height = display_height;

  strcpy( &comp_output_filename[ 0 ][ 0 ] , &output_filename[ 0 ][ 0 ] );
  strcpy( &comp_output_filename[ 1 ][ 0 ] , &output_filename[ 1 ][ 0 ] );
  strcpy( &comp_output_filename[ 2 ][ 0 ] , &output_filename[ 2 ][ 0 ] );
  strcpy( &comp_output_filename[ 3 ][ 0 ] , &output_filename[ 3 ][ 0 ] );

  /* 29.01.99 HHI Schueuer */
  /* Initialise SA-IDCT */
  /*010725
  Initialisations_for_SADCT();
  Initialisations_for_SADCT_s_k();
  Initialisations_for_SADCT_Kaup();
  */
  /* end HHI */

  
  /* if random_access, find start GOV */
  if (random_access_start_time != 0)
    {
      next_display_time = random_access_start_time;
      for(i=0; i<number_of_vos; i++)
	FindStartGOV(number_of_vols[i],stream[i],trace[i],display_vol_list[i],
		     readen_bits[i],random_access_start_time);
    }

  /* added by Sony 010997 */
  for(i=0; i<number_of_vos; i++) 
    {
      next_vol = display_vol_list[i];
      for(k=0; k<number_of_vols[i]; k++) 
	{
	  if(GetVolScalability(next_vol))
	    not_scalability=0;
	  next_vol = GetVolNext(next_vol);
	}
      next_vol = NULL;
    }
  next_vol = NULL;
  /* 010997 */
  
  /*****
   *
   *	Process all bitstreams in parallel, i.e. up to the display time
   *	decode each VOL of each VO and compose then the respective
   *	image to be displayed out of all layers of all VOs. Then again
   *	decode each VOL of each VO up to the next display time etc.
   *
   *****/

/* 01.04.01: Studio Profile */  
  if(!studio_profile) {
/* 01.04.01: Studio Profile */
    
  for(i=0; i<number_of_vos; i++)
    	  if(short_video_header[i]==0)
         Check_TPS(number_of_vols[i],stream[i],trace[i],random_access_start_time,
	      display_vol_list[i]);
  

  /** added for NEWPRED (Oki) 16-AUG-1999 **/
  if(GetVolNewpredEnable(display_vol_list[0])){
    /* initialize NEWPRED mode */
    InitNewpred_d(&np_cfg, 		/* config structure for NEWPRED */
		  number_of_vos, 	/* number of VOs */
		  display_width, 	/* width */
		  display_height);	/* height */
  }
  /** end of NEWPRED (Oki) 16-AUG-1999 **/

  }/* 01.04.01: Studio Profile */	
  
  while(!StopDecoding(number_of_vos,number_of_vols,stop_decoding_vol))
    {
       if (temporal_scalability)
	fprintf(stderr,"===( next [%d] )===\n",(Int)next_display_time);
      for(i=0; i<number_of_vos; i++) { /** added for NEWPRED (Oki) 31-AUG-1999 **/
	/* Modified due to N2171 Cl. 2.1.x (user data) MW 26-MAR-1998 */
	/* DecodeVolList(number_of_vols[i],stream[i],trace[i], */
	/* 	      next_display_time, random_access_start_time, */
	/* 	      display_vol_list[i],readen_bits[i], */
	/*            stop_decoding_vol[i], */
	/* 	      i,post_filter_type,); */

/* 01.04.01: Studio Profile */
	if(studio_profile)
	  DecodeVolList_Studio(number_of_vols[i],stream[i],trace[i],
			       next_display_time, random_access_start_time,
			       display_vol_list[i],readen_bits[i],stop_decoding_vol[i],
			       i,post_filter_type, gov_user_data_filename[i],short_video_header[i],
			       mot_x_P,mot_y_P,MB_decisions_P);
	else {
/* 01.04.01: Studio Profile */	
	
	/** added for NEWPRED (Oki) 16-AUG-1999 **/
	if(GetVolNewpredEnable(display_vol_list[0])){
	  DecodeVolList(number_of_vols[i],stream[i],trace[i],
			next_display_time, random_access_start_time,
			display_vol_list[i],readen_bits[i],stop_decoding_vol[i],
			i,post_filter_type, gov_user_data_filename[i],short_video_header[i],
			mot_x_P,mot_y_P,MB_decisions_P,
			np_cfg[i],	  /* config structure for NEWPRED */
			output_filename); /** Oki 31-AUG-1999 **/
	}
        else{
	/** end of NEWPRED (Oki) 16-AUG-1999 **/

	  DecodeVolList(number_of_vols[i],stream[i],trace[i],
			next_display_time, random_access_start_time,
			display_vol_list[i],readen_bits[i],stop_decoding_vol[i],
			i,post_filter_type, gov_user_data_filename[i],short_video_header[i],
			mot_x_P,mot_y_P,MB_decisions_P,
			NULL, NULL);	/** added for NEWPRED (Oki) 31-AUG-1999 **/
	} /** added for NEWPRED (Oki) 16-AUG-1999 **/

      } /* 01.04.01: Studio Profile */	
	
      } /** added for NEWPRED (Oki) 31-AUG-1999 **/

/* 01.04.01: Studio Profile */
       if(studio_profile) {
	 WriteOutputImage_Studio(number_of_vos,
				 display_vol_list,next_display_time,
				 random_access_start_time,display_vop,image_counter,
				 output_filename,
				 output_filename_aux_comp,
				 post_filter_type,stop_decoding_vol);
	 image_counter++;
       } else {
/* 01.04.01: Studio Profile */
       
      if(!GetVolNewpredEnable(display_vol_list[0])){ /** added for NEWPRED (Oki) 31-AUG-1999 **/
	/* In the NEWPRED mode, decoded VOPs are writen by WriteOutputImageNewpred_d() in
	   DecodeVol() */
	if (not_scalability) 
	  {  
            display_vop = AllocVop(display_width,display_height,GetVolAuxCompCount(display_vol_list[0]));
            PutVopBitsPerPixel(bits_per_pixel, display_vop);
	    WriteOutputImage(number_of_vos,
			     display_vol_list,next_display_time,
			     random_access_start_time,display_vop,image_counter,
			     output_filename,post_filter_type,stop_decoding_vol);
            FreeVop(display_vop);
	    image_counter++;
	  }
       } /** added for NEWPRED (Oki) 31-AUG-1999 **/

    } /* 01.04.01: Studio Profile */
      
      printf( "======================================\n");
      printf( "[%d]{curr_time}:%f ->" , image_counter, next_display_time );
      
      next_display_time += time_between_two_frames;
      /* next_display_time = RoundTime(next_display_time); */
    
      printf( "{next_time} = %f\n" , next_display_time );
    }
  
  /*****
   *
   *	Freeing of display_vop, VOLs, bitstream structures and trace files
   *
   *****/

/* 01.04.01: Studio Profile */
  if(studio_profile)
    FreeVop_Studio(display_vop);/*010725*/
/* 01.04.01: Studio Profile */    
  
  /*FreeVop(display_vop); */
  
  for(i=0; i<number_of_vos; i++)
    {
      /*010725*/
      if(studio_profile)
	FreeVolList_Studio(display_vol_list[i]);
      else
      /*010725*/
	FreeVolList(display_vol_list[i]);
      
      for(k=0; k<number_of_vols[i]; k++)
	{
	  BitstreamClose(stream[i][k]);
#ifdef _DEBUG_
	  free_trace_file(trace[i][k]);
#endif
	}
    }

  /*010725*/
  if(studio_profile) {
    Int j;
    for(i=0;i<MAX_NUM_VOS;i++)
      for(j=0;j<MAX_NUM_VOLS;j++) {
	if (mot_x_P[i][j]!=NULL) FreeImage(mot_x_P[i][j]);
	if (mot_y_P[i][j]!=NULL) FreeImage(mot_y_P[i][j]);
	if (MB_decisions_P[i][j]!=NULL) FreeImage(MB_decisions_P[i][j]);
      }
  }
  /*010725*/

  
  for (i=0;i<3;i++) 
    if (base_output_vop[i]!=NULL) FreeVop(base_output_vop[i]);
  if (temporal_scalability) 
    FreeShapeVop(); 

  exit(0);
}				/* main */





/***********************************************************CommentBegin******
 *
 * -- GetDecoderArguments -- Gets from the user needed arguments for 
 *			     decoding & display  
 *
 * Author :		
 *	Sylvie Jeannin (LEP)
 *
 * Created :		
 *	11-Mar-96
 *
 * Purpose :		
 *	Get from the user needed arguments for decoding & display.
 *
 * 
 * Arguments in : 	
 *	Int   argc,		 as usual (coming from main).
 *	Char *argv[],		 as usual (coming from main).
 *
 *
 * Arguments in/out :	
 * 	Int *number_of_vos		number of video objects
 *	Int number_of_vols[]   		number of layers per video objects
 *	Char bitstream_filename[][][]	filenames for the respective bitstreams
 *	Char output_filename[][]	names of output files for Y, U and V
 *	Char user_data_filename [][]    names of files to write received user 
 *                                      data (N2171)
 *	Float *display_frame_rate		frame rate for output sequence
 *      Int *post_filter_type     status of deblock filter
 *      Int *random_access_start_time
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 08.08.96 - M. Wollborn: changed for high level syntax in order
 *				      to be compliant with VM3.0
 *            31.01.97 Aa. Sandvand: Added post_filter_type
 *            07.07.97 A. Sandvand: Added random_access_start_time
 *            03.01.98 M. Frater: Support for non-8-bit video
 *	      25.03.98 M.Wollborn: included argument for user data files 
 *                                 (N2171)
 *
 ***********************************************************CommentEnd********/
File *GetDecoderArguments (Int argc, Char *argv[], 
			   Int *number_of_vos, 
			   Int number_of_vols[], 
                           Int *number_of_vtcs,  /* VTC */
			   Char bitstream_filename[][MAX_NUM_VOLS][300], 
			   Char vol_user_data_filename[][MAX_NUM_VOLS][300], /* N2171 */
			   Char gov_user_data_filename[][MAX_NUM_VOLS][300], /* N2171 */
			   Char output_filename[][300],
			   Float *display_framerate,
			   Int *display_width,
			   Int *display_height,
			   Int *post_filter_type,
			   Int *bits_per_pixel,
			   Int *random_access_start_time,
			   Int erd_flag[][MAX_NUM_VOLS] /* MW 14-MAY-1998 */
                           )
{
  Char config_filename[300],garbage[80];
  File *fd;
  Int i,k;
  
  /*****
   *
   *	The parameters have to be read in by a configuration file
   *
   *****/
  if(argc == 2)
    strcpy(config_filename,argv[1]);
  else
    {
      (Void) printf("Usage: vm_dec.exe control_file\n");
      exit(1);
    }
  
  /*****
   *
   *	Read the decoder configuration file
   *
   *****/
  if((fd = fopen(config_filename,"r")) == NULL)
    {
      printf("Error: Control file <<%s>> not found\n",config_filename);
      exit(1);
    }
  
  /* Number of video objects */
  fscanf(fd,"%d",number_of_vos);
  fgets(garbage,80,fd);
  
  /* Number of video object layers per VO */
  for(i=0; i<*number_of_vos; i++)
    {
      fscanf(fd,"%d",&number_of_vols[i]);
      fgets(garbage,80,fd);
      
      /* Name of bitstream files for each VOL of each VO */
      for(k=0; k<number_of_vols[i]; k++)
	{
	  fscanf(fd,"%s", (Char *)&bitstream_filename[i][k]);
	  fgets(garbage,80,fd);
	}
      
      /* Name of output file for user data  on VOL level for each VOL of each VO */
      for(k=0; k<number_of_vols[i]; k++)
	{
	  fscanf(fd,"%s", (Char *)&vol_user_data_filename[i][k]);
	  fgets(garbage,80,fd);
	} 
      
      /* Name of output file for user data  on GOV level for each VOL of each VO */
      for(k=0; k<number_of_vols[i]; k++)
	{
	  fscanf(fd,"%s", (Char *)&gov_user_data_filename[i][k]);
	  fgets(garbage,80,fd);
	} 
      
      /* Usage of error-resilience tools (for case data-part.==0, i.e. resync markers) MW 14-MAY-1998 */
      for(k=0; k<number_of_vols[i]; k++)
	{
	  fscanf(fd,"%d",&erd_flag[i][k]);
	  fgets(garbage,80,fd);
	} 
    } 
  
  /* Name of output filenames for Y, U and V components */
  for(i=0; i<4; i++)
    {
      fscanf(fd,"%s", (Char *)&output_filename[i]);
      fgets(garbage,80,fd);
    }
  
  /* Display framerate */
  fscanf(fd,"%f",display_framerate);
  fgets(garbage,80,fd);
  
  /* Display width */
  fscanf(fd,"%d",display_width);
  fgets(garbage,80,fd);

  /* Display height */
  fscanf(fd,"%d",display_height);
  fgets(garbage,80,fd);

  /* post_filter_type */
  fscanf(fd,"%d",post_filter_type);
  fgets(garbage,80,fd);

  /* display pixel depth */
  fscanf(fd,"%d",bits_per_pixel);
  fgets(garbage,80,fd);

  /* random_access_start_time */
  fscanf(fd,"%d",random_access_start_time);
  fgets(garbage,80,fd);

  /* Number of VTC objects */
  fscanf(fd,"%d",number_of_vtcs);
  fgets(garbage,80,fd);

  return fd;  /* VTC */
} 			/* GetDecoderArguments */

/* 01.04.01: Studio Profile */
Void GetStudioDecoderArguments_Studio (File *fd,
				       Int *studio_profile,
				       Int *display_chroma_format,
				       Char output_filename_aux_comp[][100],
				       Int gray_bits_per_pixel[])
{
  Char garbage[80];
  Int i;
  
  /*****
   *
   *	Read the decoder configuration file
   *
   *****/

  /* studio profile */
  fscanf(fd,"%d",studio_profile);
  fgets(garbage,80,fd);

  /* display chroma format */
  fscanf(fd,"%d",display_chroma_format);
  fgets(garbage,80,fd);

  /* display gray file name */
  for(i=0;i<MAX_AUX_COMP_NUM;i++) {
    fscanf(fd,"%s", (Char *)&output_filename_aux_comp[i]);
    fgets(garbage,80,fd);
  }
  
  /* display gray pixel depth */
  for(i=0;i<MAX_AUX_COMP_NUM;i++) {
    fscanf(fd,"%d",&gray_bits_per_pixel[i]);
    fgets(garbage,80,fd);
  }

  if(!studio_profile) {
    for(i=0;i<MAX_AUX_COMP_NUM;i++)
      strcpy((Char *)&output_filename_aux_comp[i], "unloaded_aux_comp_file");
    *display_chroma_format = CHROMA420;
    for(i=0;i<MAX_AUX_COMP_NUM;i++)
      gray_bits_per_pixel[i] = 8;
  }
} 			/* GetStudioDecoderArguments_Studio */
/* 01.04.01: Studio Profile */



/* VTC Begin -----------------------------------------------*/
Void GetVTCDecoderArguments (File *fd,
                              Char VTC_bitstream_filename[], 
                              Char VTC_output_filename[], 
                              Char VTC_shape_filename[], 
                              Int *shape_width,
                              Int *shape_height,
                              Int *target_bitrate, 
															Int *target_shape_level,    /* SAIT_V2: added by Samsung AIT (99/02/23) */
                              Int *target_spatial_level,
                              Int *target_snr_level,
															Int *target_tile_id_from,/* modified by Sharp (99/2/16) */
                              Int *target_tile_id_to /* modified by Sharp (99/2/16) */
															)
{
  Char garbage[80];
                                          
  fscanf(fd,"%s", VTC_bitstream_filename);
  fgets(garbage,80,fd);

  fscanf(fd,"%s", VTC_output_filename);
  fgets(garbage,80,fd);

  fscanf(fd,"%d",shape_width);
  fgets(garbage,80,fd);

  fscanf(fd,"%d",shape_height);
  fgets(garbage,80,fd);

  fscanf(fd,"%d",target_bitrate);
  fgets(garbage,80,fd);

  fscanf(fd,"%d",target_spatial_level);
  fgets(garbage,80,fd);

  fscanf(fd,"%d",target_snr_level);
  fgets(garbage,80,fd);

/* SAIT_V2 begin: added by Samsung AIT (99/02/23) */
  fscanf(fd,"%d",target_shape_level);
  fgets(garbage,80,fd);
/* SAIT_V2 end */

/*begin: added by Sharp (99/2/16)*/
  fscanf(fd,"%d %d",target_tile_id_from, target_tile_id_to);
  fgets(garbage,80,fd);
/*end: added by Sharp (99/2/16)*/


}
/* VTC End -----------------------------------------------*/





/***********************************************************CommentBegin******
 *
 * -- InitVolList -- Initialize a list of video object layers for one VO 
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Initialize a list of layers for a specific video object. Within
 *	each VOL two VOPs are "salloced" in temporal list, one for the
 *	previous reconstructed VOP which is required for decoding of
 *	P-VOPs and one for the current VOP to be decoded.
 *
 * 
 * Arguments in : 	
 *	Int number_of_vols	number of layers for the video object
 *
 *
 * Arguments in/out :	
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :
 *	Vol *vol_list		pointer to list of layers for video object
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :
 *	
 *
 ***********************************************************CommentEnd********/

Vol *InitVolList(Int number_of_vols)
{
  Vol *vol_list,*next_vol;
  Vop *vop1,*vop2;
  Int i;

  /*****
   *
   *	Initialize as many VOLs as necessary
   *
   *****/
  if(number_of_vols > 0)
    {
    vol_list = SallocVol();
    
    vop1 = SallocVop();
    PutVolVop(vop1,vol_list);
    
    vop2 = SallocVop();
    PutVopWidth(0,vop2);
    vop2->mod_time_base=-1;
    PutVopNextTemp(vop2,vop1);

    vop2 = SallocVop();
    PutVopWidth(0,vop2);
    vop2->mod_time_base=-1;
    PutVopPrevTemp(vop2,vop1);

    /* 1197-3 SONY */
    PutVopBvopTemp( NULL, vop1);
    /* 1197-3 */

    next_vol = vol_list;
    
    for(i=1; i<number_of_vols; i++)
      {
	PutVolNext(SallocVol(),next_vol);
	next_vol = GetVolNext(next_vol);
	
	vop1 = SallocVop();
	PutVolVop(vop1,next_vol);
	
	vop2 = SallocVop();
	vop2->mod_time_base=-1;
	PutVopWidth(0,vop2);
	
	PutVopNextTemp(vop2,vop1);
	
	vop2 = SallocVop();
	vop2->mod_time_base=-1;
	PutVopWidth(0,vop2);
	
	PutVopPrevTemp(vop2,vop1);
	
	/* 1197-3 SONY */
	PutVopBvopTemp( NULL, vop1);
	/* 1197-3 */
	
      }
    
    return(vol_list);
    }
  else
    return(NULL);
}





/***********************************************************CommentBegin******
 *
 * -- DecodeVolHeaderList -- Decode the headers for a list of VOLs for one VO 
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode the headers for a list of VOLs for a specific VO from given
 *	bitstreams. The bitstream for each layer is given in a separate file
 *
 * 
 * Arguments in : 	
 *	Int number_of_vols	number of layers for the video object
 *	Bitstream *stream[]	Pointer list to the bitstreams
 *	Trace *trace[]		Trace information for each layer
 *
 *
 * Arguments in/out :	
 *	Vol *vol_list	concat. list of layers for video object
 *	Int readen_bits[]	Readen bits for each layer
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :	25.03.98 M.Wollborn: N2171 user data
 *              11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *	
 *
 ***********************************************************CommentEnd********/
Void DecodeVolHeaderList(Int number_of_vols, Bitstream *stream[], 
                         Trace *trace[], Vol *vol_list, Int readen_bits[],
                         Int vo_id, Char vol_user_data_filename[][300],
                         Int erd_flag[], Int *short_video_header, /* MW 14-MAY-1998 */
                         ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                         ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                         Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])    

{
  Vol *next_vol;
  Int i;
  
  /*****
   *
   *	Decode the header for each VOL for the current VO
   *	(allocate first dummy VOP if non-rectangular shape)
   *
   *****/
  next_vol = vol_list;
  
  for(i=0; i<number_of_vols; i++)
    {
      DecodeVolHeader(stream[i],trace[i],next_vol,&readen_bits[i],vo_id,i,
                      vol_user_data_filename[i],erd_flag[i],short_video_header,
                      mot_x_P,mot_y_P,MB_decisions_P) ;
                      /* N2171 user data */ /* erd_flag MW 14-MAY-1998 */
      if(GetVolShape(next_vol) == 0)
	AllocVopChannels(GetVolVop(next_vol),GetVolWidth(next_vol),
			 GetVolHeight(next_vol),GetVolAuxCompCount(next_vol));
      else
	AllocVopChannels(GetVolVop(next_vol),2,2,0);
      
      next_vol = GetVolNext(next_vol);
    }
  
  return;
}





/***********************************************************CommentBegin******
 *
 * -- DecodeVolHeader -- Decode the header of a VOL
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode the header of a VOL
 *
 * 
 * Arguments in : 	
 *	Bitstream *stream	Pointer to the bitstream
 *	Trace *trace		Trace information for VOL
 *
 *
 * Arguments in/out :	
 *	Vol *next_vol		video object layer to be decoded
 *	Int *readen_bits	Readen bits for VOL
 *      Int *short_video_header Flag to signal short video header bitstreams (H.263)
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : Noel Brady 14-10-96
 *			Code added to read 
 *			error_resilient_disable,
 *			intra_acdc_disable,
 *			deblocking_filter_disable.
 *            14.01.97 Luis Ducla-Soares: added byte alignment for
 *			VOL_START_CODE
 *                      (call to function BitstreamByteAlign() )	
 *	28.01.97 Robert Danielsen: Small change to zigzag-scan
 *	23.04.97 Michael Wollborn: Added MPEG quantization stuff completely
 *      26.04.97 Luis Ducla Soares: Changes to allow error resilient
 *                                  decoding in the combined mode with
 *                                  data partitioning.
 * 12.05.97 Minhua Zhou: added "not_8_bit", "advanced_prediction_disable", etc.
 * 14.05.97 F. Jaureguizar: Modification of number of bits of fcodes, from
 *          2 bits to 3 bits (according to VM7.0).
 *    18.08.97 Minhua Zhou: added video_object_start_code
 * 18.06.97 M.Wollborn: Added correct trace mechanism
 * 09.07.97 A. Sandvand added storing of VO Id in VOL structure
 * 04.08.97 Minhua Zhou: removed f_codes in the VOL  
 * 06.08.97 Noel Brady: added mods for BINARY_SHAPE_ONLY mode
 * 07.08.97 Minhua Zhou: changed quant. matrix decoding
 * 08.09.97 Cecile Dufour: added 
 * 23.03.98 Michael Wollborn: moved "random_accessible_vol" due to N2171, 
 *                            Clause 2.1.7
 * 21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 * 05.06.98 Marc Mongenet (EPFL): added Complexity Estimation syntax support
 * 23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header 
 * 11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 * 15.02.99 U. Benzler : added quarter pel support
 * 03.03.99 Seishi TAKAMURA (NTT): added GMC coding, version2 syntax
 * 16.08.99 Shigeru Fukunaga (Oki): added modules for NEWPRED
 * 18.08.99 Seishi TAKAMURA (NTT): modify GMC coding  (warping type check)
 * 06.09.99 Eishi Morimatsu (Fujitsu Labs.): added DRC support
 * 09.29.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 * 11.11.99 U. Benzler (UoH): corrected input of SADCT flag to be done only for Version2
 *
 ***********************************************************CommentEnd********/
Void DecodeVolHeader(Bitstream *stream, Trace *trace, Vol *curr_vol, 
                     Int *readen_bits, Int vo_id, Int file_vol_id,
                     Char vol_user_data_filename[], Int erd_flag,
		     Int *short_video_header,      
                     ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                     ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                     Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])
                             /* N2171 */ /* erd_flag MW 14-MAY-1998 */
{
  Int tmpvar,i,*qmat,j, res, bits, aux;
  Vop	*sprite=NULL;
  TrajPoint *tmp_ref_point_coord=NULL;
  
  /*****
   *
   *    Set trace level for VO header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VO_header_def;

  TraceBreakLine(trace);

  /*****
   *
   *	Read the VOL header entries from the bitstream
   *
   *****/

  /* vol_start_code (VOL_START_CODE_LENGTH bits) */

  tmpvar = (Int) BitstreamShowBits(stream, 27);

  if (tmpvar == VO_START_CODE) 
    {
      *short_video_header = 0;
      
  tmpvar = (Int) BitstreamReadBits(stream,27,
				   "vo_start_code",trace,CODE);
  tmpvar = (Int) BitstreamReadBits(stream,5,
				   "vo_id",trace,CODE);
  
  *readen_bits +=32;
  PutVolVOId(tmpvar,curr_vol);

  
  /*****
   *
   *    Set trace level for VOL header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VOL_header_def;

  TraceBreakLine(trace);
  
  
  tmpvar = (Int) BitstreamReadBits(stream,VOL_START_CODE_LENGTH,
				   "vol_start_code",trace,CODE);
  *readen_bits += VOL_START_CODE_LENGTH;
  
  if(tmpvar != VOL_START_CODE)
    {
      printf ("Bitstream does not start with VOL_START_CODE\n");
      exit (1);
    }
  
  /* vol_id (4 bits) */
  tmpvar = (Int) BitstreamReadBits(stream,4,"vol_id",trace,CODE);
  *readen_bits += 4;
  PutVolId(tmpvar,curr_vol);
  
  /* for TPS */
  bs_vol[vo_id][tmpvar] = file_vol_id;
  
  /* Included due to N2171, Clause 2.1.7 MW 23-MAR-1998 */
  tmpvar = (Int) BitstreamReadBits(stream,1,"random_accessible_vol",
				   trace,FLAG);
  *readen_bits += 1;
  PutVolRandomAccessibleVol(tmpvar,curr_vol);
  
  /* 15.12.98 Sven Brandau: add video_object_type_indication, see N2502(a), Clause 6.2.4 */
  tmpvar = BitstreamReadBits(stream,8,"video_object_type_indication",trace,CODE);
  *readen_bits += 8;
  PutVolVisualObjectTypeIndication(tmpvar,curr_vol);
  
  tmpvar = (Int) BitstreamReadBits(stream,1,"is_object_layer_identifier",trace,CODE);
  *readen_bits +=1;
  PutVolIsObjectLayerIdentifier(tmpvar,curr_vol);
  if (GetVolIsObjectLayerIdentifier(curr_vol)) 
    {
      tmpvar = (Int) BitstreamReadBits(stream,4,"visual_object_layer_verid",
				       trace,CODE);
      PutVolVisualObjectLayerVerid(tmpvar,curr_vol);
      *readen_bits +=4;
      tmpvar = (Int) BitstreamReadBits(stream,3,"visual_object_layer_priority"
				       ,trace,CODE);
      PutVolVisualObjectLayerPriority(tmpvar,curr_vol);
      *readen_bits +=3;
    } 
/* modified NTT for version2 syntax : start */
  else {
      PutVolVisualObjectLayerVerid(1,curr_vol);		/* version1 syntax changed back for bw comp test, U. Benzler*/
      PutVolVisualObjectLayerPriority(1,curr_vol);
  }
/* modified NTT for version2 syntax : end */
  

  /* 11.08.98 Sven Brandau:  "insert aspect_ratio_info" due to  N2339, Clause 2.1.17 */
  tmpvar = (Int) BitstreamReadBits( stream, 4, "aspect_ratio_info", trace, CODE );
  PutVolAspectRatioInfo( tmpvar, curr_vol );
  *readen_bits += 4;
  if (GetVolAspectRatioInfo(curr_vol)==EXTENDED_PAR) {
    tmpvar = (Int) BitstreamReadBits( stream, 8, "par_width", trace, CODE );
    PutVolAspectRatioInfoParWidth( tmpvar, curr_vol );
    tmpvar = (Int) BitstreamReadBits( stream, 8, "par_height", trace, CODE );
    PutVolAspectRatioInfoParHeight( tmpvar, curr_vol );
     *readen_bits += 16;
  }



  tmpvar = (Int) BitstreamReadBits(stream,1,"vol_control_parameters", trace, CODE );
  PutVolVolControlParameters(tmpvar,curr_vol);
  *readen_bits +=1;

  /* 16.11.98 Sven Brandau:  "change vol_control_parameters syntax" due to N2470b, Clause 2.1.38 */
  /* 08.06.99 Karsten Suehring: added vol_control_parameter processing */
  if (GetVolVolControlParameters(curr_vol)) {
    tmpvar = (Int) BitstreamReadBits( stream,  2, "chroma_format", trace, CODE );
    PutVolChromaFormat(tmpvar,curr_vol);
    tmpvar = (Int) BitstreamReadBits( stream,  1, "low_delay", trace, CODE );
    PutVolLowDelay(tmpvar,curr_vol);
  #ifdef PRINT_VOL_CONTROL_PARAMETERS
    printf("VOL Control Parameters enabled:\n");
    printf("    Chroma Format: %i\n",GetVolChromaFormat(curr_vol));
    printf("    LowDelay     :");
    if (GetVolLowDelay(curr_vol)) printf(" yes\n");
    else  printf(" no\n");
  #endif
    tmpvar = (Int) BitstreamReadBits( stream, 1, "vbv_parameters", trace, CODE );
    PutVolVbvParameters(tmpvar,curr_vol);
    *readen_bits += 4;

    if (tmpvar) {
      tmpvar = (Int) BitstreamReadBits( stream, 15, "first_half_bit_rate", trace, CODE );
      PutVolFirstHalfBitRate(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );
      tmpvar = (Int) BitstreamReadBits( stream, 15, "latter_half_bit_rate", trace, CODE );
      PutVolLatterHalfBitRate(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );
      tmpvar = (Int) BitstreamReadBits( stream, 15, "first_half_vbv_buffer_size", trace, CODE );
      PutVolFirstHalfVbvBufferSize(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );   
      tmpvar = (Int) BitstreamReadBits( stream,  3, "last_half_vbv_buffer_size", trace, CODE );
      PutVolLatterHalfVbvBufferSize(tmpvar,curr_vol);
      tmpvar = (Int) BitstreamReadBits( stream, 11, "first_half_vbv_occupancy", trace, CODE );
      PutVolFirstHalfVbvOccupancy(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );
      tmpvar = (Int) BitstreamReadBits( stream, 15, "last_half_vbv_occupancy", trace, CODE );
      PutVolLatterHalfVbvOccupancy(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );  
      *readen_bits += 15 + 1 + 15 + 1 + 15 + 1 + 3 + 11 + 1 + 15 + 1;
    #ifdef PRINT_VOL_CONTROL_PARAMETERS
      printf("    VBV Parameters enabled:\n");
      printf("        Bit Rate       : %ld\n", ((LInt)GetVolFirstHalfBitRate(curr_vol)<<15)+GetVolLatterHalfBitRate(curr_vol));
      printf("        VBV Buffer Size: %ld\n", ((LInt)GetVolFirstHalfVbvBufferSize(curr_vol)<<3)+GetVolLatterHalfVbvBufferSize(curr_vol));
      printf("        VBV Occupancy  : %ld\n", ((LInt)GetVolFirstHalfVbvOccupancy(curr_vol)<<15)+GetVolLatterHalfVbvOccupancy(curr_vol));
    #endif
    }
  }
  
  /* vol_shape (2 bits) */
  tmpvar = (Int) BitstreamReadBits(stream,2,"vol_shape",trace,CODE);
  *readen_bits += 2;
  PutVolShape(tmpvar,curr_vol);
  
  /* begin MAC extensions - Sven Brandau 16-Nov-99 */
  if (GetVolShape(curr_vol)==GREY_SCALE) {
    if (GetVolVisualObjectLayerVerid(curr_vol)!=1) { /* version 2 */
      tmpvar = (Int) BitstreamReadBits(stream,4,"vol_shape_extension",trace,CODE);
      *readen_bits += 4;
      PutVolShapeExtension(tmpvar,curr_vol);
      PutVolAuxCompCount( GetAuxCompCount( tmpvar ), curr_vol );
    } else { /* version 1 */
      PutVolAuxCompCount( 1, curr_vol );
    }
  } else {
    PutVolAuxCompCount( 0, curr_vol );
  }
  if (GetVolAuxCompCount(curr_vol)>MAX_MAC) {
    fprintf(stderr,"Can't code more than %d auxillary components! ... exit\n", MAX_MAC );
    exit(-1);
  }
  /* end MAC extensions - Sven Brandau 16-Nov-99 */

  /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.5 */
  BitstreamReadBits(stream, 1, "marker_bit", trace, CODE);
  *readen_bits += 1;

  /* 10.08.98 Sven Brandau:  "change time_increment_resolution to 16 Bit" due to N2339, Clause 2.1.2 */
  tmpvar = (Int) BitstreamReadBits(stream,16,"time_increment_resolution",
				   trace,NUM);
  PutVolTimeIncrementResolution(tmpvar,curr_vol);
  *readen_bits +=16;

  /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.5 */
  BitstreamReadBits(stream, 1, "marker_bit", trace, CODE);
  *readen_bits += 1;

  tmpvar = (Int) BitstreamReadBits(stream,1,"fixed_vop_rate",trace,CODE);
  PutVolFixedVopRate(tmpvar,curr_vol);
  *readen_bits +=1;


  /* 11.08.98 Sven Brandau:  "read fixed_vop_time_increment (1-16)" due to N2339, Clause 2.1.18 */
  if (GetVolFixedVopRate( curr_vol )) {
    bits = ceil(log((double)GetVolTimeIncrementResolution(curr_vol))/log(2.0));
    if (bits<1) bits=1;
    BitstreamReadBits( stream, bits,"fixed_vop_time_increment", trace, CODE);
    *readen_bits += bits;
  }

  
  if (GetVolShape(curr_vol) != BINARY_SHAPE_ONLY)  /* BSO_NOEL */ 
    {
      if(GetVolShape(curr_vol) == 0)
	{
	  BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  *readen_bits += 1;
	  
	  /* vol_width (13 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,13,"vol_width",trace,NUM);
	  *readen_bits += 13;
	  PutVolWidth(tmpvar,curr_vol);
	  
	  BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  *readen_bits += 1;
	  
	  /* vol_height (13 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,13,"vol_height",trace,NUM);
	  *readen_bits += 13;
	  PutVolHeight(tmpvar,curr_vol);

	  /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.6 */
	  BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  *readen_bits += 1;	  
	}

      /* 19.08.98 Sven Brandau: "move interlaced bit from VOP to VOL", due to N2339, Clause 2.1.3 */
      tmpvar = (Int) BitstreamReadBits(stream,1,"interlaced",trace,FLAG);
      *readen_bits += 1;
      PutVolInterlaced(tmpvar,curr_vol); 

      
      tmpvar = (Int) BitstreamReadBits(stream,1,"OBMC_disable",trace,NUM);
      *readen_bits += 1;
      PutVolOBMCDisable(tmpvar,curr_vol); 
      
      /* vol_sprite_usage (1 bits) */
/* modified by NTT for GMC coding : start
      tmpvar = (Int) BitstreamReadBits(stream,1,"vol_sprite_usage",trace,CODE);
      *readen_bits += 1;
*/
      if (GetVolVisualObjectLayerVerid(curr_vol) == 1) {
	  tmpvar = (Int) BitstreamReadBits(stream,1,"vol_sprite_usage",trace,CODE);
	  *readen_bits += 1;
      } else {
	  tmpvar = (Int) BitstreamReadBits(stream,2,"vol_sprite_usage",trace,CODE); /* test for bw comp, u. Benzler */
	  *readen_bits += 2; /* test for bw comp., U. Benzler */
      }
/* modified by NTT for GMC coding : end */

      PutVolSpriteUsage(tmpvar,curr_vol);
      
      if (GetVolSpriteUsage(curr_vol) != SPRITE_NOT_USED)
	{
	  if( GetVolSpriteUsage(curr_vol) == STATIC_SPRITE)
	    {
	      /* sprite_hdim (13 bits) */
	      tmpvar = (Int) BitstreamReadBits(stream,13,"vol_sprite_hdim",
					       trace,NUM);
	      *readen_bits += 13;
	      PutVolSpriteHdim(tmpvar,curr_vol);
	      
	      BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	      *readen_bits += 1;
	      
	      /* sprite_vdim (13 bits) */
	      tmpvar = (Int) BitstreamReadBits(stream,13,"vol_sprite_vdim",
					       trace,NUM);
	      *readen_bits += 13;
	      PutVolSpriteVdim(tmpvar,curr_vol);
	      
	      BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	      *readen_bits += 1;
	      
	      /* sprite_left_edge (13 bits) */
	      tmpvar = (Int) BitstreamReadBits(stream,13,
					       "vol_sprite_left_edge",
					       trace,NUM);
	      *readen_bits += 13;
	      if (tmpvar>4095) tmpvar -= 8192; 
	      PutVolSpriteLeftEdge(tmpvar,curr_vol);
	      
	      BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	      *readen_bits += 1;
	      
	      /* sprite_top_edge (13 bits) */
	      tmpvar = (Int) BitstreamReadBits(stream,13,"vol_sprite_top_edge",
					       trace,NUM);
	      *readen_bits += 13;
	      if (tmpvar>4095) tmpvar -= 8192; 
	      PutVolSpriteTopEdge(tmpvar,curr_vol);
	      
	      BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	      *readen_bits += 1;
	    }
/* modified by NTT for GMC coding : start */
	  else {
	      PutVolSpriteHdim(352,curr_vol);		/* sprite_hdim  */
	      PutVolSpriteVdim(288,curr_vol);		/* sprite_vdim */
	      PutVolSpriteLeftEdge(0,curr_vol);		/* sprite_left_edge */
	      PutVolSpriteTopEdge(0,curr_vol);		/* sprite_top_edge */
	  }
/* modified by NTT for GMC coding : end */
	
	  
	  /* no_of_sprite_points (6 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,6,"vol_no_of_sprite_points",
					   trace,NUM);
	  *readen_bits += 6;
	  PutVolNoOfSpritePoints(tmpvar,curr_vol);
	  
	  if (GetVolNoOfSpritePoints(curr_vol))
	    {
	      tmp_ref_point_coord = (TrajPoint *) emalloc(GetVolNoOfSpritePoints(curr_vol)*sizeof(TrajPoint)); 
	      PutVolRefPointCoord(tmp_ref_point_coord,curr_vol);
	    }
      
	  /* warping type check for GMC		add NTT (990818) */
	  if (GetVolSpriteUsage(curr_vol) == GMC_SPRITE) {
	      if (! (GetVolNoOfSpritePoints(curr_vol) == 0 ||
		     GetVolNoOfSpritePoints(curr_vol) == 1 ||
		     GetVolNoOfSpritePoints(curr_vol) == 2 ||
		     GetVolNoOfSpritePoints(curr_vol) == 3)) {
		  /* perspective warping(=4) is not use. */
		  fprintf(stderr, "error: no_of_sprite_points error: %d\n",
			  GetVolNoOfSpritePoints(curr_vol));
		  exit(-1);
	      }
	  }

	  /* warping accuracy (2 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,2,
					   "warping_accuracy_in_sprite",
					   trace,NUM);
	  *readen_bits += 2;
	  PutVolWarpingAccuracy(tmpvar,curr_vol);
	  
	  /* brightness_change_in_sprite (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,
					   "vol_brightness_change_in_sprite",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVolBrightnessChangeInSprite(tmpvar,curr_vol);
	  
	  /* Basic or low latency sprite (1 bit) */
/* modified by NTT for GMC coding : start
	  tmpvar = (Int) BitstreamReadBits(stream,1,
					   "vol_low_latency_sprite_enable",
					   trace,FLAG);
	  *readen_bits += 1;
*/
	  if (GetVolSpriteUsage(curr_vol) == STATIC_SPRITE) {
	      tmpvar = (Int) BitstreamReadBits(stream,1,
					       "vol_low_latency_sprite_enable",
					       trace,FLAG);
	      *readen_bits += 1;
	  } else {        
	      tmpvar = 0;
	  }
/* modified by NTT for GMC coding : end */
	  PutVolLowLatencySpriteEnable(tmpvar,curr_vol);
	  
	  res = GetVolSpriteHdim(curr_vol)%16;
	  if (res>0) PutVolSpriteHdim(GetVolSpriteHdim(curr_vol)+
				      16-res,curr_vol);    
	  res = GetVolSpriteVdim(curr_vol)%16;
	  if (res>0) PutVolSpriteVdim(GetVolSpriteVdim(curr_vol)+
				      16-res,curr_vol); 
	  
	  sprite = AllocVop(GetVolSpriteHdim(curr_vol),
			    GetVolSpriteVdim(curr_vol),
                            GetVolAuxCompCount(curr_vol));
	  PutVolSprite(sprite,curr_vol);
	  PutVopSpriteUsage(GetVolSpriteUsage(curr_vol), sprite);
	  PutVopShape(GetVolShape(curr_vol),sprite);
	  SetConstantImage(GetVopY(sprite),128); 
	  SetConstantImage(GetVopU(sprite),0); 
	  SetConstantImage(GetVopV(sprite),0); 
	  SetConstantImage(GetVopA(sprite),0);    
	  if(GetVopShape(sprite)==GREY_SCALE)
            for(aux=0;aux<GetVopAuxCompCount(sprite);aux++)
              SetConstantImage(GetVopG(aux,sprite),0);
	  PutVolSpriteTransmitMode(PIECE, curr_vol);
	  if (GetVolLowLatencySpriteEnable(curr_vol))
	  	InitForDecodeLowLatencyStaticSprite(curr_vol);
	  
	} /* Sprite_usage != SPRITE_NOT_USED */

      /* 29.01.99 HHI Schueuer - U. Benzler UoH 991111 added (GetVolVisualObjectLayerVerid(curr_vol) != 1)*/
      if ((GetVolShape(curr_vol)!=0) && (GetVolVisualObjectLayerVerid(curr_vol) != 1) )
	{
	  tmpvar = (Int) BitstreamReadBits(stream,1,"vol_sadct_disable",trace,FLAG);
	  *readen_bits +=1;
	  PutVolSADCTDisable(tmpvar,curr_vol);
	} 
      else  
	PutVolSADCTDisable(1,curr_vol);
      /* end HHI */      
      
      
      /* not_8_bit */
      tmpvar = (Int) BitstreamReadBits(stream,1,"not_8_bit",trace,FLAG);
      *readen_bits += 1;
      if (tmpvar == 1) 
	{
	  tmpvar = (Int) BitstreamReadBits(stream,4,"quant_precision",
					   trace,CODE);
	  *readen_bits += 4;
	  PutVolQuantPrecision(tmpvar,curr_vol);
	  tmpvar = (Int) BitstreamReadBits(stream,4,"bits_per_pixel",
					   trace,CODE);
	  *readen_bits += 4;
	  PutVolBitsPerPixel(tmpvar,curr_vol);
	}
      else 
	{
	  PutVolQuantPrecision(5,curr_vol);
	  PutVolBitsPerPixel(8,curr_vol);
	}
      
      /* HYUNDAI (Grayscale) */ 
      if(GetVolShape(curr_vol) == GREY_SCALE) { /* GRAYLEVEL SHAPE */
         tmpvar = (Int) BitstreamReadBits(stream,1,"disable_gray_quant_update",
                                        trace,FLAG);
         *readen_bits += 1;
         PutVolDisableGrayQuantUpdate(tmpvar,curr_vol);

         /* composition_method : Not Supported */
         tmpvar = (Int) BitstreamReadBits(stream,1,"composition_method",
                                        trace,FLAG);

         /* composition_method : Not Supported */
         tmpvar = (Int) BitstreamReadBits(stream,1,"linear_composition",
                                        trace,FLAG);
      }
   
      /* vol_quant_type (1 bit) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"vol_quant_type",trace,FLAG);
      *readen_bits += 1;
      PutVolQuantType(tmpvar,curr_vol);
      
      /* Added MPEG quantization stuff completely, 23-APR-1997 MW */
      if(GetVolQuantType(curr_vol))
	{
	  /* load_intra_quant_mat (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"load_intra_quant_mat",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVolLoadIntraQuantMat(tmpvar,curr_vol);
	  
	  qmat = GetVolIntraQuantMat(curr_vol);
	  
	  if(GetVolLoadIntraQuantMat(curr_vol))
	    {
	      /* intra_quant_mat (8*64 bits) */
	      i=0;
	      do {
		tmpvar = BitstreamReadBits(stream,8,"intra_quant_mat",
					   trace,NUM);
		qmat[*(zigzag_i+i)] = tmpvar;
		*readen_bits +=8;
	      } while ((qmat[*(zigzag_i+i)]!=0)&&(++i<64));
	     
	      for (j=i;j<64;j++)
	        qmat[*(zigzag_i+j)]=qmat[*(zigzag_i+i-1)];
	    }
	  else
	    {
	      for(i=0; i<64; i++)
	        qmat[i] = mpeg_iqmat_def[i];
	    }
	  
	  /* load_nonintra_quant_mat (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"load_nonintra_quant_mat",
					   trace,FLAG);
	  *readen_bits += 1;
	  
	  PutVolLoadNonintraQuantMat(tmpvar,curr_vol);
	  
	  qmat = GetVolNonintraQuantMat(curr_vol);
	  
	  if(GetVolLoadNonintraQuantMat(curr_vol))
	    {
	      /* nonintra_quant_mat (8*64 bits) */
	      i=0;
	      do {
		tmpvar = BitstreamReadBits(stream,8,"nonintra_quant_mat",trace,NUM);
		qmat[*(zigzag_i+i)] = tmpvar;
		*readen_bits +=8;
	      } while ((qmat[*(zigzag_i+i)]!=0)&&(++i<64));
	      
	      for (j=i;j<64;j++)
	        qmat[*(zigzag_i+j)]=qmat[*(zigzag_i+i-1)];
	    }
	  else
	    {
	      for(i=0; i<64; i++)
	        qmat[i] = mpeg_nqmat_def[i];
	    }
	  

	  /* HYUNDAI (Grayscale) */
	  if(GetVolShape(curr_vol) == GREY_SCALE) {	/* GRAYLEVEL SHAPE */
	    
            for(aux=0;aux<GetVolAuxCompCount(curr_vol);aux++) { /* MAC (SB) 17-Nov-99 */
	      /* load_gray_intra_quant_mat (1 bit) */
	      tmpvar = (Int) BitstreamReadBits(stream,1,
					       "load_gray_intra_quant_mat",
					       trace,FLAG);
	      *readen_bits += 1;
	      PutVolLoadGrayIntraQuantMat(tmpvar,aux,curr_vol);
	      
	      qmat = GetVolGrayIntraQuantMat(aux,curr_vol);
	      
	      if(GetVolLoadGrayIntraQuantMat(aux,curr_vol))
		{
		  /* gray_intra_quant_mat (8*64 bits) */
		  i=0;
		  do {
		    tmpvar = BitstreamReadBits(stream,8,
					       "gray_intra_quant_mat",
					       trace,NUM);
		    qmat[*(zigzag_i+i)] = tmpvar;
		    *readen_bits +=8;
		  } while ((qmat[*(zigzag_i+i)]!=0)&&(++i<64));
		  
		  for (j=i;j<64;j++)
		    qmat[*(zigzag_i+j)]=qmat[*(zigzag_i+i-1)];
		}
	      
	      else
		{
		  for(i=0; i<64; i++)
		    qmat[i] = mpeg_giqmat_def[i];
		}
	      
	      /* load_gray_nonintra_quant_mat (1 bit) */
	      tmpvar = (Int) BitstreamReadBits(stream,1,
					       "load_gray_nonintra_quant_mat",
					       trace,FLAG);
	      *readen_bits += 1;
	      
	      PutVolLoadGrayNonintraQuantMat(tmpvar,aux,curr_vol);
	      
	      qmat = GetVolGrayNonintraQuantMat(aux,curr_vol);
	      
	      if(GetVolLoadGrayNonintraQuantMat(aux,curr_vol))
		{
		  /* gray_nonintra_quant_mat (8*64 bits) */
		  i=0;
		  do {
		    tmpvar = BitstreamReadBits(stream,8,
					       "gray_nonintra_quant_mat",
					       trace,NUM);
		    qmat[*(zigzag_i+i)] = tmpvar;
		    *readen_bits +=8;
		  } while ((qmat[*(zigzag_i+i)]!=0)&&(++i<64));
		  
		  for (j=i;j<64;j++)
		    qmat[*(zigzag_i+j)]=qmat[*(zigzag_i+i-1)];
		}
	      else
		{
		  for(i=0; i<64; i++)
		    qmat[i] = mpeg_gnqmat_def[i];
		}
              }
	    }
	}


/* modified NTT for version2 syntax : start */
      if (GetVolVisualObjectLayerVerid(curr_vol) != 1) {
/* modified NTT for version2 syntax : end */

      /* MW QPEL 07-JUL-1998 >> */
      /* vol_quarter_pel (1 bit) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"vol_quarter_pel",
				       trace,FLAG);
      *readen_bits += 1;
      PutVolQuarterPel(tmpvar,curr_vol);
      if (tmpvar)
        printf("\nQuarter Pel mode on !\n");
      else
        printf("\nQuarter Pel mode off!\n");
      /* << MW QPEL 07-JUL-1998 */

/* modified NTT for version2 syntax : start */
      } else {
	  PutVolQuarterPel(0,curr_vol);
      }
/* modified NTT for version2 syntax : end */

      /* START: Complexity Estimation syntax support - Marc Mongenet (EPFL) - 5 Jun 1998 */
      tmpvar = (Int) BitstreamReadBits(stream, 1,
				       "complexity_estimation_disable",
				       trace, FLAG);
      *readen_bits += 1;
      PutVolComplexityEstimationDisable(tmpvar, curr_vol);
      if (!GetVolComplexityEstimationDisable(curr_vol))
	*readen_bits += BitstreamReadComplexityEstimationHeader(stream, curr_vol, trace);
      else
	printf("Complexity Estimation is DISABLED\n");
      /* END: Complexity Estimation syntax support */

      /* Modified due to N2171 Cl. 2.5.1/14 MW 27-MAR-1998 */
      /* error_resilient_disable (1 bit) */
      /* tmpvar = (Int) BitstreamReadBits(stream,1,"error_resilient_disable", */
      /* 				       trace,FLAG); */
      /* *readen_bits += 1; */
      /* PutVolErrorResDisable(tmpvar,curr_vol); */
      
      /* if (!tmpvar)  */
      /* 	{ */
      /* 	  printf("Decoding in Error Resilient Mode\n"); */


      /* 24.08.98 Sven Brandau: "insert resync_marker_disable bit" due to N2339, Clause 3.5.1 */
      tmpvar = (Int) BitstreamReadBits(stream,1,"resync_marker_disable", trace, FLAG);
      *readen_bits += 1; 
      PutVolErrorResDisable(tmpvar,curr_vol);
      
      if (!GetVolErrorResDisable(curr_vol)) 
	printf("Using Resync Marker.\n");
     
      tmpvar = (Int) BitstreamReadBits(stream,1,"data_partitioning_enable",
				       trace,FLAG);
      *readen_bits += 1;
      PutVolDataPartEnable(tmpvar,curr_vol);
      
      if (GetVolDataPartEnable(curr_vol))
	{
	  printf("Using Data Partitioning.\n");
	  tmpvar = (Int) BitstreamReadBits(stream,1,"reversible_vlc_enable",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVolReverseVlc(tmpvar,curr_vol);
	  
	  if (GetVolReverseVlc(curr_vol))
	    printf("Using Reversible VLC.\n");
	  
	  /* Enable error resilience only if data partitioning is used */
	  /* for the time being MW 28-MAR-1998                         */    
	  PutVolErrorResDisable(0,curr_vol);
	}	  
      else
	{
	  printf("Using the neither Data Partitioning nor RVLC.\n");
	  /* Disable error resilience if no data partitioning is used */
	  /* for the time being MW 28-MAR-1998                         */ 

	  /* Now make it better and look into the configuration file */
	  /* if for the VOL the error resilience tools have been used */
	  /* This should only be temporal and be removed after hopefully */
	  /* re-inclusion of that flag into the bitstream */
	  /* MW 14-MAY-1998 */
	  /* PutVolErrorResDisable(1,curr_vol); */
	  PutVolErrorResDisable(erd_flag,curr_vol);
	}

      /*     } */
      /*   else  */
      /* printf("Decoding in Non-Error Resilient Mode\n"); */
    
      /** added for NEWPRED (Oki) 16-AUG-1999 **/
      if(GetVolVisualObjectLayerVerid(curr_vol) != 1) { /* version ID == version 2 */
	/* NEWPRED enable flag */
	tmpvar = (Int) BitstreamReadBits(stream,1,"nerpred_enable",
					 trace,FLAG);
	*readen_bits += 1;
	PutVolNewpredEnable(tmpvar,curr_vol);
     
	if(GetVolNewpredEnable(curr_vol)){
	  printf("NEWPRED: ON");
	  /* requested backward message type for NEWPRED */
	  tmpvar = (Int) BitstreamReadBits(stream,2,"newpred_backward_message_type",
					   trace,FLAG);
	  *readen_bits += 2;
	  PutVolNewpredBackwardMessageType(tmpvar,curr_vol);
	  switch(tmpvar){
	  case 0:
	    printf(" (ERROR : requested backward message type (%d) is not defined.)\n", tmpvar);
	    exit(1);
	    break;
	  case 1:
	    printf(" (ACK message is required)");
	    break;
	  case 2:
	    printf(" (NACK message is required)");
	    break;
	  case 3:
	    printf(" (Both ACK and NACK message are required)");
	    break;
	  }

	  /* segment type for NEWPRED */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"newpred_segment_type",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVolNewpredSegmentType(tmpvar,curr_vol);
	  switch(tmpvar){
	  case 0:
	    printf(" per Video Packet.\n");
	    break;
	  case 1:
	    printf(" per VOP.\n");
	    break;
	  }
	}
	else{
	  PutVolNewpredSegmentType(0,curr_vol); /** Oki 26-AUG-1999 **/
	  printf("NEWPRED: OFF\n");
	}

/* >>> added for DRC by Fujitsu (top)    <<< */
      /* reduced_resolution_vop_enable (1 bit) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"reduced_resolution_vop_enable",
				       trace,FLAG);
      *readen_bits += 1;
      PutVolReducedResolutionVopEnable(tmpvar,curr_vol);

      if (GetVolReducedResolutionVopEnable(curr_vol)) 
        printf("Reduced Resolution Vop ENABLED\n");
      else 
        printf("Reduced Resolution Vop DISABLED\n");
/* >>> added for DRC by Fujitsu (bottom)    <<< */

      }
      else{ /** Oki 26-AUG-1999 **/
	PutVolNewpredEnable(0,curr_vol);
	PutVolNewpredSegmentType(0,curr_vol);
	printf("NEWPRED: OFF\n");
      }
       /** end of NEWPRED (Oki) 16-AUG-1999 **/

      PutVolACDCPredDisable(0,curr_vol);
  
      if (!GetVolACDCPredDisable(curr_vol)) 
        printf("ACDC Prediction ENABLED\n");
      else 
        printf("INTRA ACDC Prediction Turned OFF\n");

      /* scalability (1 bit) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"scalability",
				       trace,FLAG);
      *readen_bits += 1;
      PutVolScalability(tmpvar,curr_vol);
      
      if(GetVolScalability(curr_vol))
	{
	  /*hierarchy type*/
	  tmpvar = (Int) BitstreamReadBits(stream,1,"hierachy_type",trace,FLAG);
	  *readen_bits += 1;
	  PutVolHierarchyType(tmpvar,curr_vol);
	  if(tmpvar == 0)
	    printf("Spatial scalability Turned ON\n");
	  else if (tmpvar == 1)
	    printf("Temporal scalability Turned ON\n");
	  else{
	    printf("Wrong paramter are detected\n");
	    exit(0);
	  }
	  /* ref_layer_id (4 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,4,"ref_layer_id",
					   trace,CODE);
	  *readen_bits += 4;
	  PutVolRefId(tmpvar,curr_vol);
	  
	  /* ref_layer_sampling_direc (1 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"ref_layer_sampling_direc",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVolRefSampDir(tmpvar,curr_vol);
	  
	  /* hor_sampling_factor_n (5 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,5,"hor_sampling_factor_n",
					   trace,NUM);
	  *readen_bits += 5;
	  PutVolHorSampN(tmpvar,curr_vol);
	  
	  /* hor_sampling_factor_m (5 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,5,"hor_sampling_factor_m",
					   trace,NUM);
	  *readen_bits += 5;
	  PutVolHorSampM(tmpvar,curr_vol);
	  
	  /* ver_sampling_factor_n (5 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,5,"ver_sampling_factor_n",
					   trace,NUM);
	  *readen_bits += 5;
	  PutVolVerSampN(tmpvar,curr_vol);
	  
	  /* ver_sampling_factor_m (5 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,5,"ver_sampling_factor_m",
					   trace,NUM);
	  *readen_bits += 5;
	  PutVolVerSampM(tmpvar,curr_vol);
	  
	  /* enhancement_type (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"enhancement_type",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVolEnhanceType(tmpvar,curr_vol);

/* begin : added for OBSS by Samsung AIT (1999-09-29)*/
          if(GetVolShape(curr_vol) && !GetVolHierarchyType(curr_vol))
          {
                /* use_ref_shape (1 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,1,"use_ref_shape",
                                     trace,NUM);
                *readen_bits += 1;
                PutVolUseRefShape(tmpvar,curr_vol);
 
                /* use_ref_texture (1 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,1,"use_ref_texture",
                                     trace,NUM);
                *readen_bits += 1;
                PutVolUseRefTexture(tmpvar,curr_vol);
 
                /* shape_hor_sampling_factor_n (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_hor_sampling_factor_n",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeHorSampN(tmpvar,curr_vol);
 
                /* shape_hor_sampling_factor_m (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_hor_sampling_factor_m",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeHorSampM(tmpvar,curr_vol);
 
                /* shape_ver_sampling_factor_n (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_ver_sampling_factor_n",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeVerSampN(tmpvar,curr_vol);
 
                /* shape_ver_sampling_factor_m (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_ver_sampling_factor_m",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeVerSampM(tmpvar,curr_vol);
          }
/* end : added for OBSS by Samsung AIT (1999-09-29)*/
	}

      /* Removed due to N2171, Clause 2.1.7 MW 23-MAR-1998 */
      /* tmpvar = (Int) BitstreamReadBits(stream,1,"random_accessible_vol", */
      /* 				     trace,FLAG); */
      /* *readen_bits += 1; */
      /* PutVolRandomAccessibleVol(tmpvar,curr_vol); */
      BitstreamByteAlign(stream);
      
      /* Included due to N2171, Cl. 2.1.9 MW 25-MAR-1998 */
      tmpvar = (Int) BitstreamShowBits(stream,USER_DATA_START_CODE_LENGTH);
      if (tmpvar == USER_DATA_START_CODE) 
        *readen_bits += GetUserDataFromBitstream(stream,trace,
						 vol_user_data_filename);
      
      if( GetVolSpriteUsage(curr_vol) == STATIC_SPRITE )
  	{
	  PutVolHeaderToVop(curr_vol, sprite);
	  PutVopWidth(GetVolSpriteHdim(curr_vol),sprite);
	  PutVopHeight(GetVolSpriteVdim(curr_vol),sprite);			
	  if (GetVolLowLatencySpriteEnable(curr_vol)==0)
	  	{
                DecodeBasicSprite(stream, curr_vol, vo_id, trace, readen_bits,
                                    mot_x_P,mot_y_P,MB_decisions_P); 		
	  	PutVolSpriteTransmitMode(STOP, curr_vol);
	  	}
	}	/* CASE STATIC_SPRITE */
    } /* BSO_NOEL */
  else  
    { 
      /* Modified due to N2171 Cl. 2.5.1/14 MW 27-MAR-1998 */
      /* error_resilient_disable (1 bit) */
      /* tmpvar = (Int) BitstreamReadBits(stream,1,"error_resilient_disable", */
      /* 				  trace,FLAG); */
      /* *readen_bits += 1; */
      /* PutVolErrorResDisable(tmpvar,curr_vol); */  /* BSO_NOEL */

/* begin : added for OBSS by Samsung AIT (1999-09-29)*/
      if(GetVolVisualObjectLayerVerid(curr_vol) != 1) { /* version ID == version 2 */
        /* scalability (1 bit) */
        tmpvar = (Int) BitstreamReadBits(stream,1,"scalability",
                                       trace,FLAG);
        *readen_bits += 1;
        PutVolScalability(tmpvar,curr_vol);
 
        if(GetVolScalability(curr_vol))
        {
                /* shape_hor_sampling_factor_n (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_hor_sampling_factor_n",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeHorSampN(tmpvar,curr_vol);

                /* shape_hor_sampling_factor_m (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_hor_sampling_factor_m",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeHorSampM(tmpvar,curr_vol);
 
                /* shape_ver_sampling_factor_n (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_ver_sampling_factor_n",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeVerSampN(tmpvar,curr_vol);
 
                /* shape_ver_sampling_factor_m (5 bits) */
                tmpvar = (Int) BitstreamReadBits(stream,5,"shape_ver_sampling_factor_m",
                                     trace,NUM);
                *readen_bits += 5;
                PutVolShapeVerSampM(tmpvar,curr_vol);
        }
      } else {
	fprintf(stderr,"Spatial scalable shape is not defined in version 1\n");
	exit(1);
      }
/* end : added for OBSS by Samsung AIT (1999-09-29)*/

      /* 24.08.98 Sven Brandau: "insert resync_marker_disable bit" due to N2339, Clause 3.5.1 */
      tmpvar = (Int) BitstreamReadBits(stream,1,"resync_marker_disable", trace, FLAG);
      *readen_bits += 1; 
      PutVolErrorResDisable(tmpvar,curr_vol);

      BitstreamByteAlign(stream);

      /* Included due to N2171, Cl. 2.1.9 MW 25-MAR-1998 */
      tmpvar = (Int) BitstreamShowBits(stream,USER_DATA_START_CODE_LENGTH);
      if (tmpvar == USER_DATA_START_CODE) 
        *readen_bits += GetUserDataFromBitstream(stream,trace,
						 vol_user_data_filename);
    }
  }
  else{ /*short_video_header*/
	  tmpvar = (Int) BitstreamShowBits(stream, SHORT_VIDEO_START_MARKER_LENGTH);
	  if(tmpvar == SHORT_VIDEO_START_MARKER){
		  printf("**short_video_header object detected **\n");
		  *short_video_header = 1;
	      InitShortVideoHeader(curr_vol);
			bs_vol[vo_id][0] = file_vol_id;
	  }


  }
  
  return;
}



/***********************************************************CommentBegin******
 *
 * -- DecodeVolList -- Decode a list of video object layers for one VO 
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode a list of layers for a specific video object from given
 *	bitstreams. The bitstream for each layer is given in a separate file
 *
 * 
 * Arguments in : 	
 *	Int number_of_vols	number of layers for the video object
 *	Bitstream *stream[]	 Pointer list to the bitstreams
 *	Trace *trace[]		 Trace information for each layer
 *	Int next_display_time	next time a frame is displayed
 *  Int random_access_start_time
 *
 *
 * Arguments in/out :	
 *	Vol *vol_list	         concatenated list of layers for video object
 *	Int readen_bits[]	 Readen bits for each layer
 *	Int stop_decoding_vol[]  Flag to indicate end of all bitstreams
 *	NpConfig *np_cfg	 config structure for NEWPRED (Oki)
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 07.07.97: A. Sandvand: Added support for Random Access
 *            11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 * 	      16.08.99 Shigeru Fukunaga (Oki): modified modules for NEWPRED
 *	
 *
 ***********************************************************CommentEnd********/
Void DecodeVolList(Int number_of_vols, Bitstream *stream[], Trace *trace[], 
                   Float next_display_time, Int random_access_start_time,
                   Vol *vol_list, Int readen_bits[], 
                   Int stop_decoding_vol[], Int vo_id, Int post_filter_type,
                   Char gov_user_data_filename[][300], Int short_video_header,
                   ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                   ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                   Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS],  
		   NpConfig *np_cfg,	         /** added for NEWPRED (Oki) 16-AUG-1999 **/
		   Char output_filename[][300])  /** added for NEWPRED (Oki) 31-AUG-1999 **/
{
  Vol *next_vol;
  Int k;

  /*****
   *
   *	Decode each layer for the current object until the time is
   *	come to display the next frame
   *
   *****/
  next_vol = vol_list;
  
  for(k=0; k<number_of_vols; k++)
    {
      if(stop_decoding_vol[k] == 0)
        DecodeVol(stream,trace[k],next_display_time,random_access_start_time,
                  next_vol,&readen_bits[k],
                  &stop_decoding_vol[k],vo_id,vol_list,post_filter_type,
                  gov_user_data_filename[k],short_video_header,
                  mot_x_P,mot_y_P,MB_decisions_P,                  
                  np_cfg,	    /** added for NEWPRED (Oki) 16-AUG-1999 **/
                  output_filename); /** added for NEWPRED (Oki) 31-AUG-1999 **/
      if(temporal_scalability)
	  WriteOutputImage_TPS_for_base(base_output_vop,
					next_display_time,
					post_filter_type,
					!GetVolScalability(next_vol),
					stop_decoding_vol,
					number_of_vols);
      next_vol = GetVolNext(next_vol);
    }
}





/***********************************************************CommentBegin******
 *
 * -- DecodeVol -- Decode a video object layer
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode a video object layer
 *
 * 
 * Arguments in : 	
 *	Bitstream *stream	Pointer to the bitstream
 *	Trace *trace		Trace information for VOL
 *	Int next_display_time	next time a frame is displayed
 *      Int random_access_start_time
 *
 *
 * Arguments in/out :	
 *	Vol *next_vol		video object layer to be decoded
 *	Int *readen_bits	Readen bits for VOL
 *	Int *stop_decoding_vol	Flag to indicate end of all bitstreams
 *      Int short_video_header  Flag to signal short video header bitstreams (H.263)
 *	NpConfig *np_cfg	 config structure for NEWPRED (Oki)
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *  	14.01.97 Luis Ducla-Soares: added byte alignment for VOP_START_CODE 
 *	                   (call to function BitstreamByteAlign() )
 *
 *	10.02.97 M.Wollborn: changed code in order to avoid problems with
 *		             different VO and display frame rates
 *	09.05.97 Minhua Zhou: added "DecodeGOV"
 *      07.07.97: A. Sandvand: Added support for Random Access
 *      24.07.97 Minhua Zhou:  Modified decoding control due to  B-VOPs 
 *      15.9.97 A. Sandvand: Updated Random Access implementation
 *      25.03.98 M.Wollborn: Modifications due to N2171 Cl. 2.1.9 user data
 *      23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 * 	16.08.99 Shigeru Fukunaga (Oki): modified modules for NEWPRED
 *
 ***********************************************************CommentEnd********/
Void DecodeVol(Bitstream **stream, Trace *trace, Float next_display_time_in, 
               Int random_access_start_time,
               Vol *curr_vol, Int *readen_bits, Int *stop_decoding_vol,
               Int vo_id, Vol *vol_list, Int post_filter_type, 
               Char gov_user_data_filename[], Int short_video_header,
               ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS],
	       NpConfig *np_cfg,	     /** added for NEWPRED (Oki) 16-AUG-1999 **/ 
	       Char output_filename[][300])  /** added for NEWPRED (Oki) 31-AUG-1999 **/
{
  Int tmpvar,vop_display_time;
  Int prev_vop_display_time;
  Int next_display_time;
  Vop *curr_vop;
  Int vol_id, nvol_id;
  Int enh_decoding_flag = 0;
  
  nvol_id = GetVolId( curr_vol );
  vol_id  = bs_vol[vo_id][nvol_id];

  /*****
   *
   *	Decode all VOPs from the current VOL until the next frame
   *	has to be displayed
   *
   *****/
  curr_vop = GetVolVop(curr_vol);
  PutVolHeaderToVop(curr_vol,curr_vop);
  next_display_time=(Int)(0.001+next_display_time_in/1000.0
                     *(Double)GetVopTimeIncrementResolution(curr_vop));
  if(next_display_time_in == random_access_start_time)
    vop_display_time = prev_vop_display_time=-1;
  else 
    {
      vop_display_time = CalcVopDisplayTime(curr_vop);
      prev_vop_display_time=CalcVopDisplayTime(GetVopPrevTemp(curr_vop));
    }
          
  while (DecodeTimeControl(stream[vol_id],
			   vop_display_time,
			   prev_vop_display_time,
			   next_display_time,
			   *stop_decoding_vol,
			   curr_vop,
			   random_access_start_time,
			   next_display_time_in,
			   &enh_decoding_flag,
			   trace))
    {
      printf("display_time=%d,prev_time=%d,vop_time=%d\n",next_display_time,
	     prev_vop_display_time,vop_display_time);
      
	  if (short_video_header) 
		/* If decoding a short_video_header bitstream, DecodeVop() is to be called next */
		tmpvar = VOP_START_CODE;
	  else
      tmpvar = (Int) BitstreamShowBits(stream[ vol_id ],VOP_START_CODE_LENGTH);
      if (tmpvar == GROUP_START_CODE) 
	
	/* Modified due to N2171, Cl. 2.1.9 MW 25-MAR-1998 */
        DecodeGOV(stream[ vol_id ],trace,curr_vop,readen_bits,
		  gov_user_data_filename);
      
      else if(tmpvar == VOP_START_CODE)
	{
          DecodeVop(stream,     vo_id,      trace,
                    curr_vop,   readen_bits,
                    vol_list,   next_display_time,
                    post_filter_type, short_video_header, stop_decoding_vol,
                    mot_x_P,mot_y_P,MB_decisions_P,
		    np_cfg);	/** added for NEWPRED (Oki) 16-AUG-1999 **/

	  if (GetVolSpriteUsage(curr_vol) == STATIC_SPRITE) 
	    PutVolSpriteTransmitMode(GetVopSpriteTransmitMode(curr_vop),
				     curr_vol);
	  
	  if(temporal_scalability && CheckBaseVop(curr_vop))
	    BASEImageInput(curr_vop, base_output_vop); /* Input base's Image to buffer */
	  
	  vop_display_time      = CalcVopDisplayTime(curr_vop);
	  prev_vop_display_time = CalcVopDisplayTime(GetVopPrevTemp(curr_vop));

	  /** added for NEWPRED (Oki) 31-AUG-1999 **/
	  if(GetVopNewpredEnable(curr_vop))
	     WriteOutputImageNewpred_d(curr_vop, output_filename, post_filter_type);
	  /** end of NEWPRED (Oki) 31-AUG-1999 **/
	}   
      else
	*stop_decoding_vol = 1;
    }
  
  return;
}





/***********************************************************CommentBegin******
 *
 * -- StopDecoding -- Checks if all bitstreams are decoded
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Check if all bitstreams of all VOLs are fully decoded
 *
 * 
 * Arguments in : 
 *	Int number_of_vos		Number of video objects
 *	Int number_of_vols[]		List with number of VOLs per VO	
 *	Int stop_decoding_vol[][] 	List with end info for all bitstreams
 *
 *
 * Arguments in/out :	
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	Int stop_decoding	flag which indicates if the decoding should
 *				stop (1) or not (0)
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	
 *
 ***********************************************************CommentEnd********/

Int StopDecoding(Int number_of_vos, Int number_of_vols[], 
		 Int stop_decoding_vol[][MAX_NUM_VOLS])
{
  Int i,k;

  /*****
   *
   *	Check all entries in the field for stopped bitstreams
   *
   *****/
  for(i=0; i<number_of_vos; i++)
    for(k=0; k<number_of_vols[i]; k++)
      if(stop_decoding_vol[i][k] == 0)
	return(0);
  
  return(1);
}





/***********************************************************CommentBegin******
 *
 * -- DecodeGOV -- Decodes the Group of VOPs from bitstream
 *
 * Author :		
 *	Minhua Zhou (HHI)
 *
 * Created :		
 *	05-05-1997
 *
 * Purpose :		

 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *	Int *readen_bits	pointer to value of up to now readen_bits
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	

 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 07.07.97 A. Sandvand: Added printing of GOV contents
 *            24.07.97 Minhua Zhou: modified the time base
 *	      25.03.98 M.Wollborn: Included reception of user data due to N2171
 *
 ***********************************************************CommentEnd********/


Void DecodeGOV(Bitstream *stream, Trace *trace, Vop *curr_vop, 
	       Int *readen_bits,
	       Char gov_user_data_filename[]) 
{
  Int tmpvar,time_s;
  Int time, closed_gov, broken_link;
  fprintf(stdout,"VO_%i, VOL_%i: Receiving GOV: {",
	  GetVopVOId(curr_vop),GetVopId(curr_vop));
  
  /* group_start_code (32 bits) */
  tmpvar = (Int) BitstreamReadBits(stream,32,
				   "group_start_code",trace,CODE);
  
  if(tmpvar != GROUP_START_CODE)
    {
      printf ("\nBitstream does not start with GROUP_START_CODE\n");
      exit (1);
    }
  tmpvar = (Int) BitstreamReadBits(stream,5,
				   "Hour",trace,NUM);
  time_s= tmpvar*3600;
  fprintf(stdout,"%i:",tmpvar);
  
  tmpvar = (Int) BitstreamReadBits(stream,6,
				   "Minute",trace,NUM);
  time_s += tmpvar*60;
  fprintf(stdout,"%i:",tmpvar);
  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "Marker_bit",trace,NUM);
  
  tmpvar = (Int) BitstreamReadBits(stream,6,
				   "Second",trace,NUM);
  time_s += tmpvar;
  fprintf(stdout,"%i}",tmpvar);
  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "closed_gov",trace,NUM);
  closed_gov= tmpvar;
  
  if (tmpvar == 1)
    fprintf(stdout,", closed_gov");
  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "broken_link",trace,NUM);
  broken_link = tmpvar;   
  
  if (tmpvar == 1)
    fprintf(stdout,", broken_link");
  *readen_bits += 52;
  fprintf(stdout,"\n");
  
  /* added by SONY 980212 */ 
  if ((closed_gov == 0)&&(broken_link == 1))
    {
      fprintf(stderr,"closed_gov = 0\nbroken_link = 1\n");
      exit(0);
    }
  /* 980212 */
  
  BitstreamByteAlign(stream);
  
  /* Included due to N2171, Cl. 2.1.9 MW 25-MAR-1998 */
  tmpvar = (Int) BitstreamShowBits(stream,USER_DATA_START_CODE_LENGTH);
  if (tmpvar == USER_DATA_START_CODE) 
    *readen_bits += GetUserDataFromBitstream(stream,trace,
					     gov_user_data_filename);
  
  if (GetVopPrevTemp(curr_vop)!=NULL) 
    {
      time=CalcVopDisplayTime(GetVopPrevTemp(curr_vop));
      PutVopModTimeBase(time_s, GetVopPrevTemp(curr_vop));
      time-=time_s*GetVopTimeIncrementResolution(curr_vop);
      PutVopTimeInc(time,GetVopPrevTemp(curr_vop));
    } 
  
  if (GetVopNextTemp(curr_vop)!=NULL) 
    {
      time=CalcVopDisplayTime(GetVopNextTemp(curr_vop));
      PutVopModTimeBase(time_s, GetVopNextTemp(curr_vop));
      time-=time_s*GetVopTimeIncrementResolution(curr_vop);
      PutVopTimeInc(time,GetVopNextTemp(curr_vop));
    }
  
} 





/***********************************************************CommentBegin******
 *
 * -- DecodeVopHeader -- Decodes the VOPheader information from the bitstream
 *
 * Author :		
 *	Cecile Dufour (Philips LEP/ ACTS MoMuSys)
 *
 * Created :		
 *	14-Nov-1997
 *
 * Purpose :		
 *	 Decode the vop headerinformation from the bitstream.
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *	Int *readen_bits	pointer to value of up to now readen_bits
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	This routine has been created by the splitting of the previous routine 
 *	DecodeVop: this previous routine contained both the reading of the header
 *	for each vop, and the call for further decoding.
 *	DecodeVopHeader has be created becasue it is called in the routine
 *	DecodeBasicSprite used in the STATIC SPRITE case
 *
 *      28.03.98 M.Wollborn: Modifications due to N2171 Cl. 2.5.14
 *      21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 *	08.06.98 Marc Mongenet (EPFL): added Complexity Estimation syntax support
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *      15.02.99 U. Benzler : added quarter pel support
 * 	16.08.99 Shigeru Fukunaga (Oki): added modules for NEWPRED
 *	06.09.99 Eishi Morimatsu (Fujitsu Labs.): added DRC support
 *	09.29.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 *
 ***********************************************************CommentEnd********/
   


/* CLEANING UNTIL HERE ON 29-MAR-1998 MW */

Int DecodeVopHeader(Bitstream *stream, Int vo_id, Trace *trace, Vop *curr_vop, 
               Int *readen_bits,
               ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])    

{
  Vop *tmp_vop;
  Int tmpvar;
  Int i,bits,aux;
  Int time_base0;
  Int time_base=0; 
  TrajPoint	*traj=NULL, *difftraj=NULL;
  Float	brightness_factor;
  /*  Int res;  :unused SB */
  TrajPoint	*tmp_refpoint=NULL;
  
#ifdef _DEBUG_SPSC_
  static Int count;
  Char name[81];
#endif
  
  Int prev_prediction_mode = -1;
  static Int prev_time_base=0;
  
  /** added for NEWPRED (Oki) 16-AUG-1999 **/
  static Int vop_id_indent = 0;
  Int vop_id_modulo;
  /** end of NEWPRED (Oki) 16-AUG-1999 **/
  
  /*****
   *
   *    Disable tracing for time preview 
   *
   *****/
  trace->trace = 0;
  
  /*****
   *
   *    Set trace level for VOP header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VOP_header_def;

  TraceBreakLine(trace);

  /*****
   *
   *	Free the forelast VOP and copy the current (which is now the previous
   *	VOP) into the right place of the structure
   *
   *****/
  

  /*****
   *
   *	Read the VOP header from the bitstream
   *
   *****/
  /* vop_start_code (VOP_START_CODE_LENGTH bits) */
  tmpvar = (Int) BitstreamReadBits(stream,VOP_START_CODE_LENGTH,
				   "vop_start_code",trace,CODE);

  *readen_bits += VOP_START_CODE_LENGTH;

  if(tmpvar != VOP_START_CODE)
    {
    printf ("Bitstream does not start with VOP_START_CODE\n");
    exit (1);
    }

  prev_prediction_mode = GetVopPredictionType(curr_vop);
 
  /* vop_prediction_type (2 bits) */
  tmpvar = (Int) BitstreamReadBits(stream,2,"vop_prediction_type",trace,CODE);
  *readen_bits += 2;
  PutVopPredictionType(tmpvar,curr_vop);

  /* modulo_time_base (? bits) */
  tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
  *readen_bits += 1;
  
  while(tmpvar == 1)
    {
      tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
      *readen_bits += 1;
      time_base++;
    }
  
  /* marker_bit (1 bit) */
  tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
  *readen_bits += 1;

  /* vop_time_increment (1-16 bits) */
  bits = ceil(log((double)GetVopTimeIncrementResolution(curr_vop))/log(2.0));
  if (bits<1) 
    bits=1;
  
  tmpvar = (Int) BitstreamReadBits(stream,bits,"vop_time_increment",trace,NUM); 
  *readen_bits += bits;
  PutVopTimeInc(tmpvar, curr_vop );
  
  if (GetVopSpriteUsage(curr_vop)!= STATIC_SPRITE||
      GetVopPredictionType(curr_vop)==SPRITE_VOP) {
    if (GetVopPredictionType(curr_vop)==B_VOP) 
      {
	if (temporal_scalability && GetVopScalability( curr_vop ))
	  time_base0=prev_time_base;
/* SONY 070498 for Spatial Scalability */
	else if (GetVopScalability( curr_vop ))
/* SONY 070498 */
	  time_base0=prev_time_base;
	else
	  time_base0=GetVopModTimeBase(GetVopPrevTemp(curr_vop));
	
	if (time_base0<0) 
	  time_base0=0;  
       
	PutVopModTimeBase(time_base0+time_base,curr_vop);

	/* 21.08.98 Sven Brandau: changes due to N2339, Clause 2.1.11 
	   from Takefumi Nagumo, eMail 16.08.98 */
        /*2.1.11*/
        if (GetVopScalability( curr_vop ))
          prev_time_base=GetVopModTimeBase(curr_vop);
        /*2.1.11*/

      }
    else 
      {
        if(temporal_scalability && GetVopScalability( curr_vop ) &&
	   prev_prediction_mode == B_VOP)
/* Changed form time_base0 = 0; by Guido Heising (HHI) 11.9.98*/
	  time_base0 = prev_time_base; 
/* SONY 070498 for Spatial Scalability */
	else if(GetVopScalability(curr_vop)) 
/* SONY 070498 */
	  time_base0 = prev_time_base;
	else
	  time_base0=GetVopModTimeBase(GetVopNextTemp(curr_vop));
        
	if (time_base0<0) 
	  time_base0=0;  
        
	PutVopModTimeBase(time_base0+time_base,curr_vop);
	
/* SONY 070498 for Spatial Scalability */
	if (GetVopScalability( curr_vop ))
/* SONY 070498 */
	  prev_time_base=GetVopModTimeBase(curr_vop);
      }
  }
  
  if(temporal_scalability) LastTimeInput(curr_vop);
  
  /* BSO_NOEL */
  
  /* marker_bit (1 bit) */
  tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
  *readen_bits += 1;
  
  
  tmpvar = (Int) BitstreamReadBits(stream,1,"vop_coded",trace,NUM);
  if (tmpvar==0) 
    {
      PutVopWidth(0,curr_vop);
      PutVopHeight(0,curr_vop);
      PutVopCoded(0,curr_vop);
      BitstreamByteAlign(stream);
      printf("Empty Vop decoded\n");
      
      if (GetVopSpriteUsage(curr_vop)!= STATIC_SPRITE||
	  GetVopPredictionType(curr_vop)==SPRITE_VOP)
        if (GetVopPredictionType(curr_vop)!=B_VOP) 
	  {
	    if (GetVopPrevTemp(curr_vop)!=NULL)
	    FreeVop(GetVopPrevTemp(curr_vop));
	    PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
	    PutVopNextTemp(NULL,curr_vop);

	    if (GetVopWidth(GetVopPrevTemp(curr_vop))!=0) 
	      {
		tmp_vop = CloneVop(GetVopPrevTemp(curr_vop)); 
		
		PutVopNextTemp(tmp_vop,curr_vop);
		PutVopModTimeBase(GetVopModTimeBase(curr_vop),GetVopNextTemp(curr_vop));
		PutVopTimeInc(GetVopTimeInc(curr_vop),GetVopNextTemp(curr_vop));
		PutVopCoded(GetVopCoded(curr_vop),GetVopNextTemp(curr_vop));
		
	      } 
	    else 
	      {
		tmp_vop = SallocVop(); 
		CopyVopNonImageField(curr_vop,tmp_vop);
		PutVopNextTemp(tmp_vop,curr_vop);
	      }
	  }
      
      return(0);
    } 

  PutVopCoded(1,curr_vop);

  /** added for NEWPRED (Oki) 16-AUG-1999 **/
  if(GetVopNewpredEnable(curr_vop)){
    /* read additional syntax for NEWPRED */
    bits = ceil(log((double)GetVopTimeIncrementResolution(curr_vop))/log(2.0));
    if (bits<1) bits=1;
    bits += 3;
    bits = (bits > 15) ? 15 : bits;
    vop_id_modulo = pow(2, bits);

    /* vop_id (4-15 bits) */
    tmpvar = (Int) BitstreamReadBits(stream,bits,"vop_id",trace,NUM); 
    *readen_bits += bits;
#ifdef _NP_DEBUG_
#if 1
    fprintf(stderr, "tmpvar(vop_id)=%d vop_id_indent=%d prev_vop_id=%d\n",
	    tmpvar, vop_id_indent, GetVopNewpredVopId(curr_vop));
#endif
#endif
    while(tmpvar + vop_id_indent < GetVopNewpredVopId(curr_vop))
      vop_id_indent += (vop_id_modulo - 1);
#ifdef _NP_DEBUG
#if 1
    fprintf(stderr, "tmpvar(vop_id)=%d vop_id_indent=%d\n",
	    tmpvar + vop_id_indent, vop_id_indent);
#endif
#endif
    PutVopNewpredVopId(tmpvar + vop_id_indent, curr_vop);

    /* vop_id_for_prediction_indication (1 bit) */
    tmpvar = (Int) BitstreamReadBits(stream,1,"vop_id_for_prediction_indication",trace,FLAG); 
    *readen_bits += 1;

    /* vop_id_for_prediction (4-15 bits) */
    if(tmpvar){
      tmpvar = (Int) BitstreamReadBits(stream,bits,"vop_id_for_prediction",trace,NUM); 
      *readen_bits += bits;
      if(tmpvar + vop_id_indent >= GetVopNewpredVopId(curr_vop))
	PutVopNewpredVopIdPred(tmpvar + vop_id_indent - (vop_id_modulo - 1), 
			       curr_vop, 
			       0);	/* np_seg_id */
      else
	PutVopNewpredVopIdPred(tmpvar + vop_id_indent, 
			       curr_vop, 
			       0);	/* np_seg_id */
    }
    else
      PutVopNewpredVopIdPred(0, curr_vop, 
			     0);	/* np_seg_id */

    /* marker_bit (1 bit) */
    tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
    *readen_bits += 1;
#ifdef _NP_DEBUG_
#if 1
    fprintf(stdout, "[%d] (0) vop_id_pred=%d\n", 
	    GetVopNewpredVopId(curr_vop),
	    GetVopNewpredVopIdPred(curr_vop, 0));
#endif
#endif
  }
  /** end of NEWPRED (Oki) 16-AUG-1999 **/

  /* set default value for shape_coding_type */
/* begin : added for OBSS by Samsung AIT (1999-09-29)*/
  if(!(GetVopScalability(curr_vop) && !GetVopHierarchyType(curr_vop) &&
        (!GetVopEnhanceType(curr_vop) || !GetVopUseRefShape(curr_vop))) &&
        !(GetVopScalability(curr_vop) && GetVopShape(curr_vop)==BINARY_SHAPE_ONLY) )
  {
/* end : added for OBSS by Samsung AIT (1999-09-29)*/
  if (GetVopPredictionType(curr_vop) == I_VOP) 		
    PutVopShapeCodingType(0,curr_vop);
  else 
    PutVopShapeCodingType(1,curr_vop);
/* begin : added for OBSS by Samsung AIT (1999-09-29)*/
  } else if(GetVopShape(curr_vop)==BINARY_SHAPE_ONLY|| !GetVopUseRefShape(curr_vop)) 
  {    
    if (GetVopPredictionType(curr_vop) == I_VOP
          || GetVopPredictionType(curr_vop) == P_VOP)
      PutVopShapeCodingType(0,curr_vop);
    else
      PutVopShapeCodingType(1,curr_vop);
  }
/* end : added for OBSS by Samsung AIT (1999-09-29)*/

  if (GetVopShape(curr_vop) != BINARY_SHAPE_ONLY) /* BSO_NOEL */
    {
  
      /* BEGINNING OF ROUNDING TYPE */
      /* get vop_rounding_type */
      if ( GetVopPredictionType(curr_vop)==P_VOP ||
	   ( GetVopSpriteUsage(curr_vop) != STATIC_SPRITE &&
	     GetVopPredictionType(curr_vop)==SPRITE_VOP))
	{
	  tmpvar = (Int) BitstreamReadBits(stream,1,
					   "vop_rounding_type",trace,NUM);
	  *readen_bits += 1;
	  PutVopRoundingType(tmpvar,curr_vop);
	}
      else
        PutVopRoundingType(0,curr_vop);
      /* END OF  ROUNDING TYPE */
      
/* >>> added for DRC by Fujitsu (top)    <<< */
  /* get vop_reduced_resolution */
  if ( ( GetVopSwitchingFlag(curr_vop) ) &&
      ( GetVopPredictionType(curr_vop)==P_VOP ||
         GetVopPredictionType(curr_vop)==I_VOP ) &&
        ( GetVopShape(curr_vop)==RECTANGULAR ) ) /* SwitchingFlag is used for reduced_resolution_vop_enable */
    {
      tmpvar = (Int) BitstreamReadBits(stream,1,
                                       "vop_reduced_resolution",trace,NUM);
      *readen_bits += 1;
      PutVopReducedResolution(tmpvar,curr_vop);
#ifdef _DRC_DEBUG_
      printf(" vop_r_r =%d (Start)\n",GetVopReducedResolution(curr_vop));
#endif
    }
  else
      PutVopReducedResolution(0,curr_vop);
/* >>> added for DRC by Fujitsu (bottom) <<< */

    } /* BSO_NOEL */
  
  if(GetVopShape(curr_vop) != 0)
    {
  if (!(GetVopSpriteUsage(curr_vop)==STATIC_SPRITE&&GetVopPredictionType(curr_vop)==I_VOP))
    {
      /* vop_width (13 bits) */
      tmpvar = (Int) BitstreamReadBits(stream,13,"vop_width",trace,NUM);
      *readen_bits += 13;
      PutVopWidth(tmpvar,curr_vop);
    
      /* marker_bit (1 bit) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
      *readen_bits += 1;
    
      /* vop_height (13 bits) */
      tmpvar = (Int) BitstreamReadBits(stream,13,"vop_height",trace,NUM);
      *readen_bits += 13;
      PutVopHeight(tmpvar,curr_vop);

      /* marker_bit (1 bit) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
      *readen_bits += 1;
    
      /* vop_horizontal_mc_spatial_ref (13 bits) */
      tmpvar = (Int) BitstreamReadBits(stream,13,"vop_horizontal_mc_spatial_ref",
				       trace,NUM);
      *readen_bits += 13;

      /* 16.11.98 - Sven Brandau:  "check even value for vop_horizontal_mc_spatial_ref"
                                                         due to N2470b, Clause 2.1.34 */
      if (tmpvar%2) {
        fprintf( stderr, "vop_horizontal_mc_spatial_ref is not divisible by 2!" );
        exit(-1);
      }

      /* Revised by C. Dufour 12/12/97 for low-latency*/
      if (GetVopSpriteUsage(curr_vop)== STATIC_SPRITE &&
	  GetVopPredictionType(curr_vop)==I_VOP)
	{
          if (tmpvar>4095) tmpvar -= 8192;
          tmpvar -= GetVopSpriteLeftEdge(curr_vop);
	}
      /* End Revision */

      PutVopHorSpatRef(tmpvar,curr_vop);
      
      /* marker_bit (1 bit) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
      *readen_bits += 1;

      /* vop_vertical_mc_spatial_ref (13 bits) */
      tmpvar = (Int) BitstreamReadBits(stream,13,"vop_vertical_mc_spatial_ref",
				       trace,NUM);
      *readen_bits += 13;

      /* 16.11.98 - Sven Brandau:  "check even value for vop_vertical_mc_spatial_ref"
                                                         due to N2470b, Clause 2.1.34 */
      if (GetVopInterlaced(curr_vop) && (tmpvar%4)) {
        fprintf( stderr, "vop_vertical_mc_spatial_ref is not divisible by 4!" );
        exit(-1);
      } else if (tmpvar%2) {
        fprintf( stderr, "vop_vertical_mc_spatial_ref is not divisible by 2!" );
        exit(-1);
      }
       
      /* Revised by C. Dufour 12/12/97 for low-latency*/
      if (GetVopSpriteUsage(curr_vop)== STATIC_SPRITE &&
	  GetVopPredictionType(curr_vop)==I_VOP)
	{
          if (tmpvar>4095) tmpvar -= 8192;
          tmpvar -= GetVopSpriteTopEdge(curr_vop);
	}
      /* End Revision */

      PutVopVerSpatRef(tmpvar,curr_vop);
      }
      else
      {
      PutVopWidth(GetVopSpriteHdim(curr_vop),curr_vop);
      PutVopHeight(GetVopSpriteVdim(curr_vop),curr_vop);
      PutVopHorSpatRef(0,curr_vop);
      PutVopVerSpatRef(0,curr_vop);
      }/* of STATIC_SPRITE and first I_VOP */

      /* marker_bit (1 bit) - Karsten Suehring */
      tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
      *readen_bits += 1;

      
/* begin : modified for OBSS by Samsung AIT (1999-09-29)*/
      /* if(GetVopScalability(curr_vop)==1 && GetVopEnhanceType(curr_vop)==1)*/
      if (GetVopShape(curr_vop)!=BINARY_SHAPE_ONLY &&
      	  GetVopScalability(curr_vop)==1 && GetVopEnhanceType(curr_vop)==1)
/* end : modified for OBSS by Samsung AIT (1999-09-29)*/
	{
	  /* background_composition (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"background_composition",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVopBackComp(tmpvar,curr_vop);
	}
      else 
        PutVopBackComp(0,curr_vop);

      /* change_CR_disable */
      tmpvar = (Int) BitstreamReadBits(stream,1,"change_CR_disable",trace,FLAG);
      PutVopChangeCRDisable(tmpvar,curr_vop);
      *readen_bits += 1;

      /* constant_alpha */
      tmpvar = (Int) BitstreamReadBits(stream,1,"vop_constant_alpha",trace,FLAG);
      PutVopConstantAlpha(tmpvar,curr_vop);
      *readen_bits += 1;
      if (GetVopConstantAlpha(curr_vop)) 
	{
	  tmpvar = (Int) BitstreamReadBits(stream,8,"vop_constant_alpha_value",trace,NUM);
	  PutVopConstantAlphaValue(tmpvar,curr_vop);
	  *readen_bits += 8;
	}
    }
  

  /* Complexity Estimation syntax support - Marc Mongenet (EPFL) - 8 Jun 1998 */
  if (!GetVopComplexityEstimationDisable(curr_vop))
    readen_bits += BitstreamReadDCECS(stream, curr_vop, trace);


  if (GetVopShape(curr_vop) != BINARY_SHAPE_ONLY)  /* BSO_NOEL */
    { 
      tmpvar = (Int) BitstreamReadBits(stream,3,"intra_dc_vlc_thr",trace,CODE);
      *readen_bits += 3;
      PutVopIntraDCVlcThr(tmpvar,curr_vop);

      /* 19.08.98 Sven brandau: "move interlaced bit from VOL to VOP" due to N2339, Clause 2.1.9 */
      /*
      tmpvar = (Int) BitstreamReadBits(stream,1,"interlaced",trace,CODE);
      *readen_bits += 1;
      PutVopInterlaced(tmpvar,curr_vop);
      */
      
      if (GetVopInterlaced(curr_vop)) 
	{
	  tmpvar = (Int) BitstreamReadBits(stream,1,"top_field_first",trace,CODE);
	  *readen_bits += 1;
	  PutVopTopFieldFirst(tmpvar,curr_vop);

	  tmpvar = (Int) BitstreamReadBits(stream,1,"alternate_scan",trace,CODE);
	  *readen_bits += 1;
	  PutVopAlternateScan(tmpvar,curr_vop);
	}
    }

  if ( (GetVopSpriteUsage(curr_vop) != SPRITE_NOT_USED)&&
       (GetVopPredictionType(curr_vop)==SPRITE_VOP) )
    {
      if(GetVopNoOfSpritePoints(curr_vop) > 0) 
	{
	  /* decode the differentially encoded trajectories */
	  difftraj = DecodeSpriteTraj(stream,trace, GetVopNoOfSpritePoints(curr_vop),
				      GetVopSpriteUsage(curr_vop)); 
	  traj = AddTraj( GetVopNoOfSpritePoints(curr_vop), difftraj);
	  PutVopTrajPointCoord(traj,curr_vop);
	  PutVopTrajPointCoord(traj,GetVopSprite(curr_vop));
	  
	  tmp_refpoint = GetVopRefPointCoord(curr_vop);
	  for (i=0 ; i<GetVopNoOfSpritePoints(curr_vop); i++)
	    {
	      tmp_refpoint[i].x = 
		GetVopHorSpatRef(curr_vop)+(i%2)*GetVopWidth(curr_vop);
	      tmp_refpoint[i].y = 
		GetVopVerSpatRef(curr_vop)+(i>>1)*GetVopHeight(curr_vop);
	    }
	  free(difftraj);
	}
      
      if(GetVopBrightnessChangeInSprite(curr_vop)) 
        brightness_factor = decode_brightness_change_factor(stream, trace, readen_bits);
      else
        brightness_factor = 1;

      PutVopBrightnessChangeFactor(brightness_factor, curr_vop);
      if(GetVopSpriteUsage(curr_vop) == STATIC_SPRITE)
	{
	  printf("Start SPRITE VOP (STATIC OFF-LINE CASE) decoding\n");
	  if (GetVopSpriteTransmitMode(curr_vop)!=STOP)
	    do
	      {
		/* sprite transmit mode (2 bits) */
		tmpvar = (Int) BitstreamReadBits(stream,2,"sprite_transmit_mode",trace,CODE);
		*readen_bits += 2;
		PutVopSpriteTransmitMode(tmpvar, curr_vop);
		
		if (GetVopSpriteTransmitMode(curr_vop)==PIECE ||
		    GetVopSpriteTransmitMode(curr_vop)==UPDATE)
                DecodeSpritePiece(stream, curr_vop, vo_id, trace, readen_bits,
                                    mot_x_P,mot_y_P,MB_decisions_P); 
	      } while (GetVopSpriteTransmitMode(curr_vop)!=STOP &&
		       GetVopSpriteTransmitMode(curr_vop)!=PAUSE);
				    
	  FreeVopChannels(curr_vop);
	  
/*
	  res = GetVopWidth(curr_vop)%16;
	  if (res>0) 
	    PutVopWidth(GetVopWidth(curr_vop)+16-res,curr_vop);    

	  res = GetVopHeight(curr_vop)%16;
	  if (res>0) 
	    PutVopHeight(GetVopHeight(curr_vop)+16-res,curr_vop);    
*/
 
	  AllocVopChannels(curr_vop,
			   GetVopWidth(curr_vop), GetVopHeight(curr_vop), GetVopAuxCompCount(curr_vop));
	  Compute_SpriteVop(curr_vop);

	  if(GetVopNoOfSpritePoints(curr_vop) > 0)
	    free(traj);
		
	  if (GetVopPrevTemp(curr_vop)!=NULL) 
	    FreeVop(GetVopPrevTemp(curr_vop));
		
	  PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
	  PutVopNextTemp(NULL,curr_vop);

	  tmp_vop =CloneVop(curr_vop);
	  if (GetVopPrevTemp(tmp_vop)!=NULL)
	    FreeVop(GetVopPrevTemp(tmp_vop));
		
	  PutVopPrevTemp(NULL,tmp_vop);  
	  PutVopNextTemp(tmp_vop,curr_vop);
	  
	  BitstreamByteAlign(stream); 
	  return(0);
	} /* END STATIC CASE */
    }  /* SPRITE_USAGE!=SPRITE_NOT_USED */

  if (GetVopShape(curr_vop) != BINARY_SHAPE_ONLY)  /* BSO_NOEL */
    { 
      /* vop_quant (5 bits) */
      tmpvar = (Int) BitstreamReadBits(stream,GetVopQuantPrecision(curr_vop),
				       "vop_quant",trace,NUM);
      *readen_bits += GetVopQuantPrecision(curr_vop);
      PutVopQuantizer(tmpvar,curr_vop);
  
      /* HYUNDAI (Grayscale) */ 
      if(GetVopShape(curr_vop) == GREY_SCALE) {
        for(aux=0;aux<GetVopAuxCompCount(curr_vop);aux++) {  /* MAC (SB) 17-Nov-99 */
          tmpvar = (Int) BitstreamReadBits(stream, 6, "vop_alpha_quant", trace, NUM);
          *readen_bits += 6;
          PutVopGLQuantizer(tmpvar,aux,curr_vop);
        }
      }
 
      /* added by Minhua Zhou: 04.08.97 */
      if (GetVopPredictionType(curr_vop)!=I_VOP) 
	{
	  tmpvar = (Int) BitstreamReadBits(stream,3,"vop_fcode_for",trace,NUM); 
	  *readen_bits += 3;
	  if (GetVopQuarterPel(curr_vop))
	      PutVopFCodeFor(tmpvar-1,curr_vop); /* UB 990215 added quarter pel support */
	  else
	      PutVopFCodeFor(tmpvar,curr_vop);
	}

      if (GetVopPredictionType(curr_vop)==B_VOP) 
	{
	  tmpvar = (Int) BitstreamReadBits(stream,3,"vop_fcode_back",trace,NUM); 
	  *readen_bits += 3;
	  if (GetVopQuarterPel(curr_vop))
	    PutVopFCodeBack(tmpvar-1,curr_vop); /* UB 990215 added quarter pel support */
	  else
	    PutVopFCodeBack(tmpvar,curr_vop);
	}
	
      if (!GetVopScalability(curr_vop))
      /* Modified due to N2171 Cl. 2.5.14 MW 28-MAR-1998 */
      /*   if (!GetVopErrorResDisable(curr_vop)) */
        if (GetVopShape(curr_vop) && GetVopPredictionType(curr_vop)!=I_VOP)
	  {
	    tmpvar = (Int) BitstreamReadBits(stream,1,
					     "vop_shape_coding_type",trace,FLAG);
	    PutVopShapeCodingType(tmpvar,curr_vop);
	  }
    } /* BSO_NOEL */
  
  return(1);
}





	
/***********************************************************CommentBegin******
 *
 * -- DecodeVop -- Decodes the VOP information from the bitstream
 *
 * Author :		
 *	Michael Wollborn (TUH)
 *
 * Created :		
 *	12-AUG-1996
 *
 * Purpose :		
 *	 Decode the vop information from the bitstream.
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *	Int *readen_bits	pointer to value of up to now readen_bits     
 *      Int short_video_header  Flag to signal short video header bitstreams (H.263)
 *	NpConfig *np_cfg	config structure for NEWPRED (Oki)
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	At the start of the module, the previous VOP is the current VOP.
 *	It is then copied to the next temporal one, which is the forelast
 *	VOP and which is freed. Then the VOP to be decoded is written into
 *	the current VOP.
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 28.10.96 Aasmund.Sandvand@fou.telenor.no: added support for
 *                     deblock filter
 * 	12.02.97 Aasmund Sanvand: Removed deblocking filter. Now done
 *			as post-processing.       
 *      07.07.97 Noel Brady: Added decoding of VOP_CR and change_CR_disable
 *      15-05-97 Minhua Zhou: added GOV and syntax changes
 *      18.06.97 M.Wollborn: Added correct trace mechanism
 *      24.07.97 Minhua Zhou: decoded non-B-VOPs are first padded,
 *                            then stored.
 *      04.08.97 Minhua Zhou: added f_code in the VOP layer
 *			      added intra_dc_vlc_thr
 *      05.08.97 Minhua Zhou added vop_coded
 *			06.08.97 Noel Brady: added mods for BINARY_SHAPE_ONLY
 *      28.08.97 Osamu Sunohara: modified for spatial scalable coding
 *      09.10.97 Minhua Zhou: added Vop_coded to deal with empty VOPs
 *      04.05.98 Jong Deuk Kim (HYUNDAI): field-baced ME/MC padding for interlaced
 *      23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 * 	16.08.99 Shigeru Fukunaga (Oki): modified modules for NEWPRED
 *
 ***********************************************************CommentEnd********/
Void DecodeVop(Bitstream **stream, Int vo_id, Trace *trace, Vop *curr_vop, 
               Int *readen_bits, Vol *vol_list, Int next_display_time,
               Int post_filter_type, Int short_video_header, Int *stop_decoding,
               ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS],
	       NpConfig	*np_cfg)	/** added for NEWPRED (Oki) 16-AUG-1999 **/ 
{
  Vop *tmp_vop;
  Int num_pixels;
  Int num_lines;
  Int MB_width;
  Int MB_height;
  TrajPoint	*traj=NULL;
  Int ok;

#ifdef _DEBUG_SPSC_
  static Int count;
  Char name[81];
#endif
 

  Int vol_id, nvol_id;

  /*****
   *
   *    Disable tracing for time preview 
   *
   *****/
  trace->trace = 0;

  nvol_id = GetVopId( curr_vop );
  vol_id = bs_vol[vo_id][nvol_id];
 
  /*****
   *
   *    Set trace level for VOP header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VOP_header_def;

  TraceBreakLine(trace);

  /*****
   *
   *	Free the forelast VOP and copy the current (which is now the previous
   *	VOP) into the right place of the structure
   *
   *****/
  if (short_video_header) {
      ok = VideoPlaneWithShortHeader(stream[vol_id], vo_id, trace, curr_vop, readen_bits, stop_decoding);
		PutVolWidth(GetVopWidth(curr_vop),vol_list);
		PutVolHeight(GetVopHeight(curr_vop), vol_list);
  }
  else 
    ok = DecodeVopHeader(stream[vol_id], vo_id, trace, curr_vop, readen_bits,
                         mot_x_P,mot_y_P,MB_decisions_P);    
   
  if (ok==0) 
    {
    if(temporal_scalability && GetVopScalability(curr_vop)==1)
	SetBGCImage(curr_vop,post_filter_type);
    return;
    }
 
  num_pixels = GetVopWidth(curr_vop);
  num_lines = GetVopHeight(curr_vop);


  MB_width = num_pixels / 16;
  MB_height = num_lines /16;

  /*****
   *
   *	Set the trace levels to the required values (either full or
   *    user-defined, see README for details)
   *
   *****/
  if(trace->full_trace_period < 0)
    {
      SetTraceLevelsFull(trace);
    }
  else if(trace->full_trace_period > 0 &&
	  trace->counter>=trace->full_trace_start &&
	  trace->counter<=trace->full_trace_end)
    {
      if((trace->counter - trace->full_trace_start) % trace->full_trace_period)
        SetTraceLevelsDefault(trace);
      else
        SetTraceLevelsFull(trace);
    }
  else
    {
      SetTraceLevelsDefaultZero(trace);
    }
  trace->counter++;

  TraceBreakLine(trace);

#ifdef _DEBUG_SPSC_
  PrintVopInfo(curr_vop,"currA_vop");
  PrintVopInfo(GetVopNextTemp(curr_vop),"currA_vop->N");
  PrintVopInfo(GetVopPrevTemp(curr_vop),"currA_vop->P");
  sprintf(name,"A_vop%d",count);
  WriteVopDebug(curr_vop,name,name,name,0);
#endif

  /*****
   *
   *	Do scalable or non-scalable decoding of the current VOP
   *
   *****/
  if(GetVopScalability(curr_vop) == 0) 
    DecodeVopNonScalable(stream[ vol_id ],vo_id,trace,curr_vop,short_video_header,
			 mot_x_P,mot_y_P,MB_decisions_P,
			 np_cfg);	/** added for NEWPRED (Oki) 16-AUG-1999 **/
  else 
    DecodeVopScalable(stream,vo_id,trace,curr_vop,vol_list,post_filter_type,
			mot_x_P,mot_y_P,MB_decisions_P);

  if(temporal_scalability && CheckBaseVop(curr_vop))
    fprintf(stderr,"[B]frame[%d]\n",CalcFrameNumber(curr_vop));

  if ( (GetVopSpriteUsage(curr_vop) != SPRITE_NOT_USED)&&
  			(GetVopPredictionType(curr_vop)==SPRITE_VOP) )
	{
	if(GetVopNoOfSpritePoints(curr_vop) > 0)
		free(traj);
	}

#ifdef _DEBUG_SPSC_
  PrintVopInfo(curr_vop,"currB_vop");
  PrintVopInfo(GetVopNextTemp(curr_vop),"currB_vop->N");
  PrintVopInfo(GetVopPrevTemp(curr_vop),"currB_vop->P");
  sprintf(name,"B_vop%d",count);
  WriteVopDebug(curr_vop,name,name,name,0);
#endif

/* added by Minhua Zhou */

  if (GetVopPredictionType(curr_vop)!=B_VOP
/* added for SpSc by Sony 280897 */
    ||(GetVopPredictionType(curr_vop)==B_VOP&&
       GetVopScalability(curr_vop)&&
       GetVopRefSelCode(curr_vop)==0)
/* 280897 */
    ) { 
    if (GetVopPrevTemp(curr_vop)!=NULL) 
      FreeVop(GetVopPrevTemp(curr_vop));
    PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
    PutVopNextTemp(NULL,curr_vop);

    tmp_vop =CloneVop(curr_vop);
    if (GetVopPrevTemp(tmp_vop)!=NULL)
      FreeVop(GetVopPrevTemp(tmp_vop));
    PutVopPrevTemp(NULL,tmp_vop);  

/* 1197-3 SONY */
    if(GetVopBvopTemp(tmp_vop)!=NULL)
      FreeVop(GetVopBvopTemp(tmp_vop));
    PutVopBvopTemp(NULL,tmp_vop);
/* 1197-3 */

    PutVopNextTemp(tmp_vop,curr_vop);

/* HYUNDAI 980507 : start */
    if(GetVopInterlaced(curr_vop))
         VopFieldPadding(GetVopNextTemp(curr_vop));
    else
         VopPadding(GetVopNextTemp(curr_vop));
/* HYUNDAI 980507 : end */

  }
  
/* 1197-3 SONY */  
  else if(!not_scalability&&!temporal_scalability) {
    
    if( GetVopBvopTemp(curr_vop) != NULL )
      FreeVop( GetVopBvopTemp(curr_vop) );
    PutVopBvopTemp(NULL,curr_vop);

    tmp_vop = CloneVop( curr_vop);
    if(GetVopPrevTemp(tmp_vop)!=NULL)
      FreeVop(GetVopPrevTemp(tmp_vop));
    PutVopPrevTemp(NULL,tmp_vop);
    if(GetVopNextTemp(tmp_vop)!=NULL)
      FreeVop(GetVopNextTemp(tmp_vop));
    PutVopNextTemp(NULL,tmp_vop);
    
    PutVopBvopTemp( tmp_vop, curr_vop);

/* HYUNDAI 980507 : start */
    if(GetVopInterlaced(curr_vop))
        VopFieldPadding( GetVopBvopTemp(curr_vop));
    else
        VopPadding( GetVopBvopTemp(curr_vop));
/* HYUNDAI 980507 : end */
    
  }
/* 1197-3 */



 
  
#ifdef _DEBUG_SPSC_
  PrintVopInfo(curr_vop,"currC_vop");
  PrintVopInfo(GetVopNextTemp(curr_vop),"currC_vop->N");
  PrintVopInfo(GetVopPrevTemp(curr_vop),"currC_vop->P");
  sprintf(name,"C_vop%d",count);
  WriteVopDebug(curr_vop,name,name,name,0);

  count++;
#endif
  if (short_video_header)
      H263BitstreamByteAlign(stream[vol_id]);
  else
  BitstreamByteAlign(stream[ vol_id ]); 

  return;
}





/***********************************************************CommentBegin******
 *
 * -- DecodeVopScalable -- Decodes the VOP information from the bitstream
 *			   in scalable mode
 *
 * Author :		
 *	Michael Wollborn (TUH)
 *
 * Created :		
 *	12-AUG-1996
 *
 * Purpose :		
 *	 Decode the vop information from the bitstream in scalable mode
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :		
 *	
 *
 ***********************************************************CommentEnd********/


/***********************************************************CommentBegin******
 *
 * -- DecodeVopNonScalable -- Decodes the VOP information from the bitstream
 *			      in non-scalable mode
 *
 * Author :		
 *	Michael Wollborn (TUH)
 *
 * Created :		
 *	12-AUG-1996
 *
 * Purpose :		
 *	 Decode the vop information from the bitstream.
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	At the start of the module, the previous VOP is the current VOP.
 *	It is then copied to the next temporal one, which is the forelast
 *	VOP and which is freed. Then the VOP to be decoded is written into
 *	the current VOP.
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 17-SEP-1996, M.Wollborn, introduced some changes for new
 *			   shape decoding		
 *	23.10.96 Robert Danielsen: Passing QP to lower functions for
 *			DCAC prediction
 *	27.12.96 Noel Brady: modifications to allow INTER shape decoding
 *			     in separate mode.
 *      22-Jan-97 Jan De Lameillieure : deallocate AB_first_MMR_values only 
 *		                        for separate mode ! 
 *	07-Feb-1997, M.Wollborn: changed in order to treat skipped MBs
 *			         correctly
 *      26.04.97 Luis Ducla Soares: Changes to allow error resilient
 *                                  decoding in the combined mode with
 *                                  data partitioning.
 *      14.10.97 Noel Brady: ChangeMBType2 is not called when the decoded VOP
 * 			     is a B-VOP
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 * 	16.08.99 Shigeru Fukunaga (Oki): modified modules for NEWPRED
 *
 ***********************************************************CommentEnd********/
Void DecodeVopNonScalable(Bitstream *stream, Int vo_id, Trace *trace, Vop *curr_vop,
                          Int short_video_header,
                          ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS],
			  NpConfig *np_cfg)	/** added for NEWPRED (Oki) 16-AUG-1999 **/ 
{
  Vop *tmp_vop;

  Image *MB_mot_decis,*B_mot_decis,*mot_x,*mot_y;

  Int MB_in_width,MB_in_height,B_in_width,B_in_height;
  Int *mb_type,*mvda;
  /*  Int vol_id = GetVopId(curr_vop);   :unused SB */

  Vop *sprite=NULL;
  
  if ( (GetVopSpriteUsage(curr_vop)!=SPRITE_NOT_USED)&&
       (GetVopPredictionType(curr_vop)==SPRITE_VOP) )
    sprite = GetVopSprite(curr_vop);

  /*****
   *
   *	Free image channels of VOP and allocate the image channels
   *	for the new sizes (only necessary if arbitrary shape)
   *
   *****/
  if (GetVopSpriteUsage(curr_vop)!=STATIC_SPRITE)
  if(GetVopShape(curr_vop))
    {
    FreeVopChannels(curr_vop);
    AllocVopChannels(curr_vop,GetVopWidth(curr_vop),GetVopHeight(curr_vop),GetVopAuxCompCount(curr_vop));
    }

  /*****
   *
   *	Set local parameters and allocate local memory
   *
   *****/
  tmp_vop = AllocVop(GetVopWidth(curr_vop),GetVopHeight(curr_vop),GetVopAuxCompCount(curr_vop));

  MB_in_width  = GetVopWidth(curr_vop)  / MB_SIZE;
  MB_in_height = GetVopHeight(curr_vop) / MB_SIZE;
  B_in_width   = 2 * MB_in_width;
  B_in_height  = 2 * MB_in_height;

  MB_mot_decis = AllocImage((UInt) MB_in_width,(UInt) MB_in_height,SHORT_TYPE);
  B_mot_decis = AllocImage ((UInt) B_in_width,(UInt) B_in_height,SHORT_TYPE);
  mot_x = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
  mot_y = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);

  mb_type = (Int *)calloc((MB_in_width*MB_in_height), sizeof(Int));
  mvda = (Int *)calloc((MB_in_width*MB_in_height), sizeof(Int));

  SetConstantImage(MB_mot_decis,MBM_TRANSPARENT);

  /*****
   *
   *	Do combined or separate decoding
   *
   *****/

  if (GetVopShape(curr_vop)==0) 
    {
      SetConstantImage(GetVopA(curr_vop),255);
      SetConstantImage(GetVopAuv(curr_vop),255);
    }
  if (GetVopErrorResDisable(curr_vop))
      DecodeVopCombMotionShapeTexture(stream,vo_id,trace,curr_vop,short_video_header,
	mot_x_P,mot_y_P,MB_decisions_P);
  else
      DecodeVopMotionShapeTextureErrRes(stream,vo_id,trace,curr_vop,
	mot_x_P,mot_y_P,MB_decisions_P,
        np_cfg);	/** added for NEWPRED (Oki) 16-AUG-1999 **/
  
  /*****
   *
   *	Free local memory and return
   *
   *****/
  free((Int *)mvda);
  free((Int *)mb_type);
  
  FreeVop(tmp_vop);
  
  FreeImage(MB_mot_decis);
  FreeImage(B_mot_decis);
  FreeImage(mot_x);
  FreeImage(mot_y);
  
  return;
}






/***********************************************************CommentBegin******
 *
 * -- PutVolHeaderToVop -- Puts the relevant information of the VOL to the VOP
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Put the relevant information of the VOL to the VOP structure, since
 *	the low level decoding modules need these informations
 *
 * 
 * Arguments in : 	
 *	Vol *curr_vol	current VOL
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop	current VOP
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :   23.12.96: F. Marques introducing the changes reported by
 * 		Noel Brady.
 *		23.04.97 Michael Wollborn: Changes for MPEG quant. stuff
 *              26.04.97 Luis Ducla-Soares: Changes for data partitioning mode.
 *              13.05.97 Minhua Zhou : added OBMCDisable	
 *              10.07.97 A. Sandvand: added VOId copying
 *              12.08.97 Minhua Zhou: added time_increment_resolution
 *              04.11.97 Minhua Zhou: added flags accroding to CD
 *              08.06.98 Marc Mongenet (EPFL): added Complexity Estimation syntax support
 *              15.02.99 U. Benzler : added quarter pel support
 *              14.06.99 K. Suehring (HHI): added VOL Control Parameters
 * 		16.08.99 Shigeru Fukunaga (Oki): added modules for NEWPRED
 *	        06.09.99 Eishi Morimatsu (Fujitsu Labs.): added DRC support
 *		09.29.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 *    26.10.99 Massimo Ravasi (EPFL): added Complexity Estimation syntax support
 *                                    Update version 2
 *
 ***********************************************************CommentEnd********/

Void PutVolHeaderToVop(Vol *vol, Vop *vop)
{
  Int i,*qmat_vol,*qmat_vop, aux;
  
  /*****
   *
   *	Copy all relevant info
   *
   *****/
  PutVopVOId(GetVolVOId(vol),vop);
  PutVopId(GetVolId(vol),vop);
  PutVopShape(i=GetVolShape(vol),vop);
 
  PutVopIsObjectLayerIdentifier(GetVolIsObjectLayerIdentifier(vol),vop);
  PutVopVisualObjectLayerVerid(GetVolVisualObjectLayerVerid(vol),vop); 
  PutVopVisualObjectLayerPriority(GetVolVisualObjectLayerPriority(vol),vop);
  PutVopVolControlParameters(GetVolVolControlParameters(vol),vop); 
  PutVopChromaFormat(GetVolChromaFormat(vol),vop);
  PutVopLowDelay(GetVolLowDelay(vol),vop);
  PutVopVbvParameters(GetVolVbvParameters(vol),vop);
  PutVopFirstHalfBitRate(GetVolFirstHalfBitRate(vol),vop);
  PutVopLatterHalfBitRate(GetVolLatterHalfBitRate(vol),vop);
  PutVopFirstHalfVbvBufferSize(GetVolFirstHalfVbvBufferSize(vol),vop);
  PutVopLatterHalfVbvBufferSize(GetVolLatterHalfVbvBufferSize(vol),vop);
  PutVopFirstHalfVbvOccupancy(GetVolFirstHalfVbvOccupancy(vol),vop);
  PutVopLatterHalfVbvOccupancy(GetVolLatterHalfVbvOccupancy(vol),vop);
  PutVopFixedVopRate(GetVolFixedVopRate(vol),vop); 
  PutVopQuantPrecision(GetVolQuantPrecision(vol),vop);
  PutVopBitsPerPixel(GetVolBitsPerPixel(vol),vop);
  
  /* added by Sven Brandau, related to N2339, Clause 2.1.9 */
  PutVopInterlaced(GetVolInterlaced(vol),vop);

  /* HYUNDAI (Grayscale) */ 
  if(GetVolShape(vol) == GREY_SCALE)
    PutVopDisableGrayQuantUpdate(GetVolDisableGrayQuantUpdate(vol),vop);
  PutVopAuxCompCount( GetVolAuxCompCount(vol), vop );  /* MAC (SB) 23-Nov-99 */


  /* START: Complexity Estimation syntax support - Marc Mongenet (EPFL) - 8 Jun 1998 */
  PutVopComplexityEstimationDisable(GetVolComplexityEstimationDisable(vol), vop);
  PutVopEstimationMethod(GetVolEstimationMethod(vol), vop);
  PutVopShapeComplexityEstimationDisable(GetVolShapeComplexityEstimationDisable(vol), vop);
  PutVopOpaque(GetVolOpaque(vol), vop);
  PutVopTransparent(GetVolTransparent(vol), vop);
  PutVopIntraCAE(GetVolIntraCAE(vol), vop);
  PutVopInterCAE(GetVolInterCAE(vol), vop);
  PutVopNoUpdate(GetVolNoUpdate(vol), vop);
  PutVopUpsampling(GetVolUpsampling(vol), vop);
  PutVopTextureComplexityEstimationSet1Disable(GetVolTextureComplexityEstimationSet1Disable(vol), vop);
  PutVopIntraBlocks(GetVolIntraBlocks(vol), vop);
  PutVopInterBlocks(GetVolInterBlocks(vol), vop);
  PutVopInter4vBlocks(GetVolInter4vBlocks(vol), vop);
  PutVopNotCodedBlocks(GetVolNotCodedBlocks(vol), vop);
  PutVopTextureComplexityEstimationSet2Disable(GetVolTextureComplexityEstimationSet2Disable(vol), vop);
  PutVopDCTCoefs(GetVolDCTCoefs(vol), vop);
  PutVopDCTLines(GetVolDCTLines(vol), vop);
  PutVopVLCSymbols(GetVolVLCSymbols(vol), vop);
  PutVopVLCBits(GetVolVLCBits(vol), vop);
  PutVopMotionCompensationComplexityDisable(GetVolMotionCompensationComplexityDisable(vol), vop);
  PutVopAPM(GetVolAPM(vol), vop);
  PutVopNPM(GetVolNPM(vol), vop);
  PutVopInterpolateMCQ(GetVolInterpolateMCQ(vol), vop);
  PutVopForwBackMCQ(GetVolForwBackMCQ(vol), vop);
  PutVopHalfpel2(GetVolHalfpel2(vol), vop);
  PutVopHalfpel4(GetVolHalfpel4(vol), vop);

	/* START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 26 Oct 1999 */
	PutVopVersion2ComplexityEstimationDisable(GetVolVersion2ComplexityEstimationDisable(vol), vop);
	PutVopSadct(GetVolSadct(vol), vop);
	PutVopQuarterpel(GetVolQuarterpel(vol), vop);
  /* END: Complexity Estimation syntax support - Update Version 2 */
  /* END: Complexity Estimation syntax support */

  PutVopRandomAccessibleVol(GetVolRandomAccessibleVol(vol),vop);


  if (i==2) PutVopBinaryShape(!BINARY_SHAPE,vop);
  else PutVopBinaryShape(BINARY_SHAPE,vop);

  
  if(GetVolShape(vol) == 0)
    {
      PutVopWidth(GetVolWidth(vol),vop);
      PutVopHeight(GetVolHeight(vol),vop);
    }

  /* 29.01.99 HHI Schueuer */
  PutVopSADCTDisable(GetVolSADCTDisable(vol),vop);
  /* end HHI */
  PutVopQuantType(GetVolQuantType(vol),vop);
  PutVopOBMCDisable(GetVolOBMCDisable(vol),vop);
  PutVopQuantPrecision(GetVolQuantPrecision(vol),vop);
  PutVopBitsPerPixel(GetVolBitsPerPixel(vol),vop);

  if(GetVolQuantType(vol))
    {
      PutVopLoadIntraQuantMat(GetVolLoadIntraQuantMat(vol),vop);
      
      qmat_vol = GetVolIntraQuantMat(vol);
      qmat_vop = GetVopIntraQuantMat(vop);

      for(i=0; i<64; i++)
        qmat_vop[i] = qmat_vol[i];

      PutVopLoadNonintraQuantMat(GetVolLoadNonintraQuantMat(vol),vop);
      
      qmat_vol = GetVolNonintraQuantMat(vol);
      qmat_vop = GetVopNonintraQuantMat(vop);

      for(i=0; i<64; i++)
        qmat_vop[i] = qmat_vol[i];

      if(GetVolShape(vol) == GREY_SCALE) /* Gray level shape */ /* HYUNDAI (Grayscale) */
	{
          for(aux=0;aux<GetVolAuxCompCount(vol);aux++) { /* MAC (SB) 17-Nov-99 */

            PutVopLoadGrayIntraQuantMat(GetVolLoadGrayIntraQuantMat(aux,vol),aux,vop);
      
            qmat_vol = GetVolGrayIntraQuantMat(aux,vol);
            qmat_vop = GetVopGrayIntraQuantMat(aux,vop);
	  
            for(i=0; i<64; i++)
              qmat_vop[i] = qmat_vol[i];

            PutVopLoadGrayNonintraQuantMat(GetVolLoadGrayNonintraQuantMat(aux,vol),aux,
                                           vop);
	  
            qmat_vol = GetVolGrayNonintraQuantMat(aux,vol);
            qmat_vop = GetVopGrayNonintraQuantMat(aux,vop);
	  
            for(i=0; i<64; i++)
              qmat_vop[i] = qmat_vol[i];
          }
	}
    }

  PutVopErrorResDisable(GetVolErrorResDisable(vol),vop);
  PutVopDataPartEnable(GetVolDataPartEnable(vol),vop);
  PutVopReverseVlc(GetVolReverseVlc(vol),vop);
  /** added for NEWPRED (Oki) 16-AUG-1999 **/
  PutVopNewpredEnable(GetVolNewpredEnable(vol),vop);
  PutVopNewpredSegmentType(GetVolNewpredSegmentType(vol),vop); /** Oki 26-AUG-1999 **/
  if(GetVolNewpredEnable(vol)){
    PutVopNewpredBackwardMessageType(GetVolNewpredBackwardMessageType(vol),vop);
  }
  /** end of NEWPRED (Oki) 16-AUG-1999 **/
/* >>> added for DRC by Fujitsu (top)    <<< */
  PutVopSwitchingFlag(GetVolReducedResolutionVopEnable(vol),vop);	
/* >>> added for DRC by Fujitsu (bottom)    <<< */
  PutVopIntraACDCPredDisable(GetVolACDCPredDisable(vol),vop);
  PutVopQuarterPel(GetVolQuarterPel(vol),vop);		/* MW QPEL 07-JUL-1998 */
  PutVopScalability(GetVolScalability(vol),vop);
  PutVopTimeIncrementResolution(GetVolTimeIncrementResolution(vol),vop);

/* begin : added for OBSS by Samsung AIT (1999-09-29)*/
  if(GetVolShape(vol) != BINARY_SHAPE_ONLY)
  {
/* end : added for OBSS by Samsung AIT (1999-09-29)*/
  if(GetVolScalability(vol))
    {
      PutVopHierarchyType(GetVolHierarchyType(vol),vop); /* added for OBSS by Samsung AIT (1999-09-29)*/
      PutVopRefId(GetVolRefId(vol),vop);
      PutVopRefSampDir(GetVolRefSampDir(vol),vop);
      PutVopHorSampN(GetVolHorSampN(vol),vop);
      PutVopHorSampM(GetVolHorSampM(vol),vop);
      PutVopVerSampN(GetVolVerSampN(vol),vop);
      PutVopVerSampM(GetVolVerSampM(vol),vop);
      PutVopEnhanceType(GetVolEnhanceType(vol),vop);
/* begin : added for OBSS by Samsung AIT (1999-09-29)*/
      if(GetVolShape(vol) && !GetVolHierarchyType(vol))
      {
        PutVopUseRefShape(GetVolUseRefShape(vol),vop);
        PutVopUseRefTexture(GetVolUseRefTexture(vol),vop);
        PutVopShapeHorSampN(GetVolShapeHorSampN(vol),vop);
        PutVopShapeHorSampM(GetVolShapeHorSampM(vol),vop);
        PutVopShapeVerSampN(GetVolShapeVerSampN(vol),vop);
        PutVopShapeVerSampM(GetVolShapeVerSampM(vol),vop);
      }
/* end : added for OBSS by Samsung AIT (1999-09-29)*/
    }
/* begin : added for OBSS by Samsung AIT (1999-09-29)*/
  } else
  {
        if(GetVolScalability(vol))
        {
      	 	if(GetVolVisualObjectLayerVerid(vol) != 1) { /* version ID == version 2 */
          PutVopShapeHorSampN(GetVolShapeHorSampN(vol),vop);
          PutVopShapeHorSampM(GetVolShapeHorSampM(vol),vop);
          PutVopShapeVerSampN(GetVolShapeVerSampN(vol),vop);
          PutVopShapeVerSampM(GetVolShapeVerSampM(vol),vop);
		} else {
	  fprintf(stderr,"Spatial scalable shape coding is not defined in version 1\n");
	  exit(1);
		}
        }
  }
/* end : added for OBSS by Samsung AIT (1999-09-29)*/

  PutVopSpriteUsage(GetVolSpriteUsage(vol),vop);
  if (GetVolSpriteUsage(vol) != SPRITE_NOT_USED) 
    {
      PutVopSpriteHdim(GetVolSpriteHdim(vol),vop);
      PutVopSpriteVdim(GetVolSpriteVdim(vol),vop);
      PutVopSpriteLeftEdge(GetVolSpriteLeftEdge(vol),vop);
      PutVopSpriteTopEdge(GetVolSpriteTopEdge(vol),vop);
      PutVopNoOfSpritePoints(GetVolNoOfSpritePoints(vol),vop);
      PutVopSprite(GetVolSprite(vol),vop);
      if(GetVolNoOfSpritePoints(vol) > 0) {
	PutVopRefPointCoord(GetVolRefPointCoord(vol),vop);
	PutVopRefPointCoord(GetVolRefPointCoord(vol),GetVopSprite(vop));
      }
      if(GetVolShape(vol) == 0)
	{
	  PutVopHorSpatRef(0,vop);
	  PutVopVerSpatRef(0,vop);
	}	
      PutVopBrightnessChangeInSprite(GetVolBrightnessChangeInSprite(vol),vop);
      PutVopLowLatencySpriteEnable(GetVolLowLatencySpriteEnable(vol),vop);
      PutVopWarpingAccuracy(GetVolWarpingAccuracy(vol),vop);
      PutVopSpriteTransmitMode(GetVolSpriteTransmitMode(vol),vop);
      PutVopSpriteTabTransmit(GetVolSpriteTabTransmit(vol),vop);
      PutVopSpriteTabAmbType(GetVolSpriteTabAmbType(vol),vop);
      PutVopSpriteTabMvda(GetVolSpriteTabMvda(vol),vop);
      PutVopSpriteTabQPStore(GetVolSpriteTabQPStore(vol),vop);
      PutVopSpriteTabDCStore(GetVolSpriteTabDCStore(vol),vop);
    }
  return;
}





/***********************************************************CommentBegin******
 *
 * -- WriteOutputImage -- writes all VOLs to an output image
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
  * Created :		
 *	15-AUG-1996
 *
 * Purpose :
 *	Writes the respective VOP for each active VOL to an output image.
 *	Therefore in each VOP of each VOL two VOPs can be accessed which
 *	are successive in time. Either the most recent VOP has exactly the
 *	same display time as the current local time, then this VOP is
 *	displayed. Otherwise, if one VOP has a later display time and the
 *	other an earlier display time, the earlier VOP is displayed. If the
 *	display time of both VOPs is earlier than the current local time, the
 *	respective VOP is not displayed at all, since then it is assumed that
 *	the VO has disappeared from the scene.
 *
 * 
 * Arguments in : 	
 *	Vol *vol_list[]  	  List of VOLs to be displayed
 *	Int local_time		  Local time base, i.e. time of actual display
 *  Int random_access_start_time_in
 *	Int image_counter	  Counter for the output image
 *	Char output_filename[][]  Name of the output file for the composed
 *				  image sequence
 *      Int post_filter_type  status of deblock filtering
 *	Int stop_decoding_vol[][] 	List with end info for all bitstreams
 *
 *
 * Arguments in/out :	
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	21.01.97 Robert Danielsen: Changed 127 to 128 for initialization
 *			of display_vop. Fix by Noel/Paulo.
 *      31.01.97 Aasmund Sandvand: Added deblock filtering
 *	12.02.97 M.Wollborn: Changed to avoid double writing of last VOP
 *      07.07.97 A. Sandvand: Modified to support Random Access
 *      24.07.97 Minhua Zhou: Modified to support B-VOPs
 *		 06.08.97 Noel Brady:  For VOPs, having only shape information, a default
 *											constant YUV colur is used i.e. 255,0,0
 *      09.10.97 Minhua Zhou: added Vop_coded to deal with empty VOPs
 *
 ***********************************************************CommentEnd********/

Void WriteOutputImage(Int number_of_vos,
		      Vol *vol_list[], Float local_time_in, 
		      Int random_access_start_time_in,Vop *display_vop, Int image_counter, 
		      Char output_filename[][300], Int post_filter_type,
		      Int stop_decoding_vol[][MAX_NUM_VOLS])
{
  Vop *local_vop, *filter_vop;
  Int vop_time,i;
  Int time_dis1,time_dis2,time_dis3,time_min,aux;
  Int local_time =0, random_access_start_time =0;
  Char g_files[MAX_MAC][100];


  /*****
   *
   *	Set the display VOP to grey (A=0, Y=0, U=V=128)
   *
   *****/
  SetConstantImage(GetVopA(display_vop),0);
  SetConstantImage(GetVopY(display_vop),0);
  SetConstantImage(GetVopU(display_vop),128);
  SetConstantImage(GetVopV(display_vop),128);
  
  /*****
   *
   *	The general policy for composing the VOs is the following:
   *
   *	Nor spatial neither temporal scalability is supported yet.
   *	The VOs are composed in an order derived by their appearance
   *	in the configuration file. The first VO appearing their has the
   *	highest priority, i.e. it is displaye in front of all other VOs
   *	and so forth. With respect to the local time base, the currently
   *	decoded VOP of a VOL is only displayed if its time pointer is lower
   *	or equal to the local time; otherwise the previously decoded VOP
   *	is displayed at this time instance.
   *
   *****/

  for(i=0; i<number_of_vos; i++)
  {
    /*****
     *
     *	  Display only VOs whose coding has not stopped yet
     *	  (implemented only for the base layer at the moment)
     *	  (M.Wollborn, 12.02.1997)
     *
     *****/
      
    if(stop_decoding_vol[i][0] <2)
    {
      /*****
       *
       *    Get he current VOP of the current VOL (only base VOL is processed
       *    in the current implementation) and check the time stamp
       *
       *****/
          
      if (stop_decoding_vol[i][0]==1) 
        stop_decoding_vol[i][0]=2;
      local_vop = GetVolVop(vol_list[i]);
      local_time = (Int)(0.001+local_time_in/1000.0*
                         (Double)GetVopTimeIncrementResolution(local_vop));
      random_access_start_time = (Int)(0.001+random_access_start_time_in/1000.0*
                                       (Double)GetVopTimeIncrementResolution(local_vop));
      vop_time = CalcVopDisplayTime(local_vop);
      time_dis1=abs(vop_time-local_time);
      vop_time = CalcVopDisplayTime(GetVopNextTemp(local_vop));
      time_dis2=abs(vop_time-local_time);
      if (GetVopScalability(local_vop)==0) {
        vop_time = CalcVopDisplayTime(GetVopPrevTemp(local_vop));
        time_dis3=abs(vop_time-local_time);
      } else time_dis3=time_dis1+1;
      time_min=time_dis1;
      if (time_min>time_dis3) time_min=time_dis3;
      if (time_min>time_dis2) time_min=time_dis2;
      if (time_min==time_dis2) 
        local_vop =GetVopNextTemp(local_vop);
      else if (time_min==time_dis3) 
        local_vop =GetVopPrevTemp(local_vop);
      
      
      
      
      
      /*****
       *
       *    For rectangular VOPs set the alph-plane again to "255"
       *    as a workaround, since it is modified by incorrect motion
       *    compensation (without padding)
       *
       *****/

      /* if(GetVopShape(local_vop) == 0) 
         SetConstantImage(GetVopA(local_vop),255);*/ 
      
      if (GetVopShape(local_vop) == BINARY_SHAPE_ONLY) /* BSO_NOEL */
      {
        SetConstantImage(GetVopY(local_vop),255);
        SetConstantImage(GetVopU(local_vop),0);
        SetConstantImage(GetVopV(local_vop),0);
      }
      
      /*****
       *
       *    If the VOP is empty (i.e. arbitrary shape and width=height=0
       *    then don't display it (this is not compliant with the VM yet,
       *    since it is not specified how to tell that a VOP has 
       *    disappeared) else create list of all VOPs to be displayed
       *
       *****/

      /* if( GetVopShape(local_vop)!=0 && */ /* commented out for testing vop_coded MW 30-NOV-1998 GetVopCoded(local_vop)==0) */
      /* Modified by Michael Wollborn */
      if (GetVopShape(local_vop)!=0 && GetVopCoded(local_vop)==0)
        /* Modified by K. Suehring */
        continue;
      else
      {
        if(post_filter_type==0)
          BlendVops(local_vop, display_vop);
        else
        {
          if (GetImageSizeX(GetVopQP(local_vop)) == GetVopWidth(local_vop)/16 ||
              GetVopPredictionType(local_vop) == B_VOP)
            filter_vop = CloneVop(local_vop);
          else
            filter_vop = CloneVop_TMP(local_vop); 
          PostFilter(filter_vop,post_filter_type);
          BlendVops(filter_vop, display_vop);
          FreeVop(filter_vop);
        }
        
      }
    }
  }
  
  /*****
   *
   *    Write composited display_vop to disk
   *
   *****/
  
  for(aux=0;aux<GetVolAuxCompCount(vol_list[0]);aux++) /* MAC (SB) 17-Nov-99 */
    sprintf(g_files[aux],"%s.%d.aux", output_filename[3], aux );


  if(local_time == random_access_start_time)
    WriteVopGeneric(display_vop,output_filename[0],output_filename[1],
                    output_filename[2],output_filename[3],
                    g_files,GetVopAuxCompCount(display_vop),
                    image_counter,
                    IO_FORMAT,IO_OVERWRITE,TRUE);
  else
    WriteVopGeneric(display_vop,output_filename[0],output_filename[1],
                    output_filename[2],output_filename[3],
                    g_files,GetVopAuxCompCount(display_vop),
                    image_counter,
                    IO_FORMAT,IO_APPEND,TRUE);
  
  return;
}





/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombMotionShapeTexture -- decode VOP in combined mode
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	16-AUG-1996
 *
 * Purpose :
 *	Decodes the VOP in the combined motion/shape/texture mode
 *
 * 
 * Arguments in : 
 *	Bitstream *stream	bitstream to decode
 *	Trace *trace		trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop		curent VOP to be decoded	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *     JDL introduced DecodeVopCombinedShapeTextureIntra (18-Jan-97)
 *     JDL added DecodeVopCombinedMotionShapeTextureInter (19-Jan-97)
 *     24.07.97 Minhua Zhou: added mot_x_p,mot_y_P to support B-VOPs
 *                           added B-VOP decoding
 *     11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *	
 *
 ***********************************************************CommentEnd********/
Void DecodeVopCombMotionShapeTexture(Bitstream *stream, Int vo_id,
                        Trace *trace, Vop *curr_vop, Int short_video_header,
                          ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])

{

  ImageF	*mot_x,
                *mot_y;
  Image *MB_decisions;
  Int vol_id,nvol_id;
  nvol_id = GetVopId( curr_vop );
  vol_id = bs_vol[vo_id][nvol_id];

  
  if(GetVopPredictionType(curr_vop) == INTRA_VOP)

       DecodeVopCombinedShapeTextureIntra(stream, vo_id, trace, curr_vop, short_video_header);
			     
  else if (GetVopPredictionType(curr_vop) != B_VOP)
  	{
	DecodeVopCombinedMotionShapeTextureInter(stream, vo_id, trace, curr_vop,&mot_x,&mot_y,
						 &MB_decisions,short_video_header);

	}
   else {
     Int nref_vol_id, ref_vol_id;
     ImageF *base_mot_x_P, *base_mot_y_P, *base_MB_decisions_P;

     if (GetVopScalability(curr_vop)==1) {
       nref_vol_id=GetVopRefId(curr_vop);
       ref_vol_id=bs_vol[vo_id][nref_vol_id];
       base_mot_x_P=AllocSameImage(mot_x_P[vo_id][ref_vol_id]);
       base_mot_y_P=AllocSameImage(mot_y_P[vo_id][ref_vol_id]);
       base_MB_decisions_P=AllocSameImage(MB_decisions_P[vo_id][ref_vol_id]);
       CopyImage(mot_x_P[vo_id][ref_vol_id],base_mot_x_P);
       CopyImage(mot_y_P[vo_id][ref_vol_id],base_mot_y_P);
       CopyImage(MB_decisions_P[vo_id][ref_vol_id],base_MB_decisions_P);
       B_DecodeVopCombinedMotionShapeTextureInter(stream,vo_id,trace,curr_vop,
         base_mot_x_P,base_mot_y_P,base_MB_decisions_P);
       FreeImage(base_mot_x_P);
       FreeImage(base_mot_y_P);
       FreeImage(base_MB_decisions_P);
     }
     else 
      B_DecodeVopCombinedMotionShapeTextureInter(stream,vo_id,trace,curr_vop,
      mot_x_P[vo_id][vol_id],mot_y_P[vo_id][vol_id],MB_decisions_P[vo_id][vol_id]);
     
   }

 /* Deallocate memory for motion vectors and MB modes */
	    if (GetVopPredictionType(curr_vop)!=B_VOP) {
	      if (mot_x_P[vo_id][vol_id]!=NULL) FreeImage(mot_x_P[vo_id][vol_id]);
	      if (mot_y_P[vo_id][vol_id]!=NULL) FreeImage(mot_y_P[vo_id][vol_id]);
              if (MB_decisions_P[vo_id][vol_id]!=NULL) FreeImage(MB_decisions_P[vo_id][vol_id]);     
              if (GetVopPredictionType(curr_vop)==I_VOP) {
                mot_x_P[vo_id][vol_id]=AllocImage(GetVopWidth (curr_vop)/8,GetVopHeight (curr_vop)/8,FLOAT_TYPE);
                mot_y_P[vo_id][vol_id]=AllocImage(GetVopWidth (curr_vop)/8,GetVopHeight (curr_vop)/8,FLOAT_TYPE);
                SetConstantImage (mot_x_P[vo_id][vol_id],+0.0);
                SetConstantImage (mot_y_P[vo_id][vol_id],+0.0);
                MB_decisions_P[vo_id][vol_id]=AllocImage(GetVopWidth (curr_vop)/16,GetVopHeight (curr_vop)/16,SHORT_TYPE);
                SetConstantImage (MB_decisions_P[vo_id][vol_id],0); /* MBM_INTRA */
                } else {
	           mot_x_P[vo_id][vol_id] = mot_x;
	           mot_y_P[vo_id][vol_id] = mot_y;
                   MB_decisions_P[vo_id][vol_id]=MB_decisions;
                  }
	    } 


  return;
}


/***********************************************************CommentBegin******
 *
 * -- DecodeVopMotionShapeTextureErrRes -- decode VOP in combined error
 *                                             resilient mode with data partitioning.
 *
 * Author :		
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	28.04.97
 *
 * Purpose :
 *	Decodes the VOP in the error resilient combined motion/shape/texture
 *      mode with data partitioning.
 *
 * 
 * Arguments in : 
 *	Bitstream *stream	bitstream to decode
 *	Trace *trace		trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop		curent VOP to be decoded	
 *	NpConfig *np_cfg	config structure for NEWPRED (Oki)
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :based on DecodeVopCombMotionShapeTexture().
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	09.12.97 Luis Ducla-Soares: changed the name of the function from
 *                                  DecodeVopCombMotionShapeTextureErrRes to
 *                                  DecodeVopMotionShapeTextureErrRes.
 *      09.12.97 Luis Ducla-Soares: modifications to allow B-VOPs.
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *	03.03.99 Seishi TAKAMURA (NTT): added GMC coding
 * 	16.08.99 Shigeru Fukunaga (Oki): modified modules for NEWPRED
 *
 ***********************************************************CommentEnd********/
Void DecodeVopMotionShapeTextureErrRes(Bitstream *stream, Int vo_id,
                                       Trace *trace, Vop *curr_vop,
                          ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS], 
			  NpConfig *np_cfg)	/** added for NEWPRED (Oki) 16-AUG-1999 **/ 
{

  ImageF	*mot_x, *mot_y;
  Image *MB_decisions;
  Int vol_id,nvol_id;
  nvol_id = GetVopId( curr_vop );
  vol_id = bs_vol[vo_id][nvol_id];
  
  if(GetVopPredictionType(curr_vop) == INTRA_VOP)
    {
      if (GetVopDataPartEnable(curr_vop) 
	  && (GetVopShape(curr_vop) != BINARY_SHAPE_ONLY) )
	{
	  /* this is the same function as in the separate mode */
	  DecodeVopDataPartitioningIntraErrRes(stream, vo_id,
					       trace, curr_vop,
					       np_cfg);	/** added for NEWPRED (Oki) 16-AUG-1999 **/
	}
      else
	DecodeVopCombinedShapeTextureIntraErrRes(stream,vo_id,
						 trace,curr_vop,
						 np_cfg);	/** added for NEWPRED (Oki) 16-AUG-1999 **/      
      
    }
/* modified by NTT for GMC coding : start
  else if (GetVopPredictionType(curr_vop) == P_VOP)
*/
  else if (GetVopPredictionType(curr_vop) == P_VOP || GetVopPredictionType(curr_vop) == SPRITE_VOP)
/* modified by NTT for GMC coding : end */
    {
      if (GetVopDataPartEnable(curr_vop))
	DecodeVopDataPartitioningInterErrRes(stream, vo_id,
					     trace, curr_vop,
					     &mot_x, &mot_y,
					     &MB_decisions,
					     np_cfg);	/** added for NEWPRED (Oki) 16-AUG-1999 **/
      else
	DecodeVopCombinedMotionShapeTextureInterErrRes(stream, vo_id, 
						       trace,
						       curr_vop,
						       &mot_x, &mot_y,
						       &MB_decisions,
						       np_cfg);	/** added for NEWPRED (Oki) 16-AUG-1999 **/      
    }
  else
    {
      B_DecodeVopCombinedMotionShapeTextureInterErrRes(stream,vo_id,trace,curr_vop,
						       mot_x_P[vo_id][vol_id],
						       mot_y_P[vo_id][vol_id],
						       MB_decisions_P[vo_id][vol_id]);
      
    }

 /* Deallocate memory for motion vectors and MB modes */
  if (GetVopPredictionType(curr_vop)!=B_VOP) {
    if (mot_x_P[vo_id][vol_id]!=NULL) FreeImage(mot_x_P[vo_id][vol_id]);
    if (mot_y_P[vo_id][vol_id]!=NULL) FreeImage(mot_y_P[vo_id][vol_id]);
    if (MB_decisions_P[vo_id][vol_id]!=NULL) FreeImage(MB_decisions_P[vo_id][vol_id]);     
    if (GetVopPredictionType(curr_vop)==I_VOP) {
      mot_x_P[vo_id][vol_id]=AllocImage(GetVopWidth (curr_vop)/8,GetVopHeight (curr_vop)/8,FLOAT_TYPE);
      mot_y_P[vo_id][vol_id]=AllocImage(GetVopWidth (curr_vop)/8,GetVopHeight (curr_vop)/8,FLOAT_TYPE);
      SetConstantImage (mot_x_P[vo_id][vol_id],+0.0);
      SetConstantImage (mot_y_P[vo_id][vol_id],+0.0);
      MB_decisions_P[vo_id][vol_id]=AllocImage(GetVopWidth (curr_vop)/16,GetVopHeight (curr_vop)/16,SHORT_TYPE);
      SetConstantImage (MB_decisions_P[vo_id][vol_id],0); /* MBM_INTRA */
    } else {
      mot_x_P[vo_id][vol_id] = mot_x;
      mot_y_P[vo_id][vol_id] = mot_y;
      MB_decisions_P[vo_id][vol_id]=MB_decisions;
    }
  } 
  
  return;
}


/***********************************************************CommentBegin******
 *
 * -- FindStartGOV -- 
 *
 * Author :  Aasmund Sandvand (Telenor)
 *
 * Created : 15.09.97	
 *
 * Purpose : To locate first GOV after random_access_start_time	
 * 
 *
 * Arguments in : 	
 *      Int number_of_vols   number_of_vols for this VO
 *      Vol *vol_list[]
 *
 * Arguments in/out :	
 *  Bitstream *stream[]      Array of bitstreams for this VO
 *	Int *readen_bits[]	pointer to value of up to now readen_bits
 *	Trace *trace[]		pointer to trace information
 *  Int random_access_start_time
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *
 *
 ***********************************************************CommentEnd********/

Void FindStartGOV(Int number_of_vols,Bitstream *stream[], 
				  Trace *trace[],Vol *vol_list, Int readen_bits[], Int random_access_start_time)
{
	Int time,broken_link,closed_gov;
	Int k,vo_id, vol_id;
	Vol *next_vol;
	Bitstream *tmp_stream = NULL;
	Bitstream *tmp_stream2 = NULL;
	Int OK = 0;
	Int offset,offset2;
	
	next_vol = vol_list;
	vo_id = GetVolVOId(next_vol);
	for(k=0; k<number_of_vols; k++){
		vol_id = GetVolId(next_vol);
		tmp_stream=clone_bitstream(stream[k]);
		tmp_stream2=clone_bitstream(stream[k]);
		GetBitstreamPosition(&offset, stream[k]);
		GetBitstreamPosition(&offset2, stream[k]);
		OK=0;
		while(!OK){
			/* search for next GROUP_START_CODE */
			while (BitstreamShowBits(tmp_stream, VOP_START_CODE_LENGTH) != GROUP_START_CODE) {
				BitstreamFlushBits(tmp_stream, 8);
			}
			GetBitstreamPosition(&offset2, tmp_stream);
			tmp_stream2=clone_bitstream(tmp_stream);
			ReadGOV(&time, &closed_gov, &broken_link, tmp_stream,trace[k],&readen_bits[k]);
			if(closed_gov!=1){
				printf("\n*** ERROR: VO_%i, VOL_%i: Random Access: GOV is not followed by I-VOP\n",vo_id,vol_id);
				exit(1);
			}
			if(time<= random_access_start_time/1000){
				stream[k]=clone_bitstream(tmp_stream2);
				offset=offset2;
			}
			else
				OK=1;
		}/*while(!OK)*/
		
		SeekBitstream(offset,stream[k]);
		
		next_vol = GetVolNext(next_vol);
	}/*for k..*/
	free(tmp_stream);
	free(tmp_stream2);
}



/***********************************************************CommentBegin******
 *
 * -- ReadGOV -- Read and return GOV information 
 *
 * Author :  Aasmund Sandvand (Telenor)
 *
 * Created : 07.07.97	
 *
 * Purpose :		
 * 
 *
 * Arguments in : Bitstream *stream
 *
 *
 * Arguments in/out :Int *time
 *                   Int *closed_gov
 *                   Int *broken_link 
 *                   Trace *trace
 *                   Int *readen_bits
 *		
 *
 *
 * Arguments out : 
 *
 * Return values :	
 *	
 *
 * Side effects :  
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :	25.03.98 M.Wollborn: Modified due to N2171 Cl. 2.1.9 (user data)
 *
 *
 ***********************************************************CommentEnd********/


Void ReadGOV(Int *time, Int *closed_gov, Int *broken_link,
	     Bitstream *stream, Trace *trace, Int *readen_bits)
{
   Int tmpvar,time_s; 
  tmpvar = (Int) BitstreamReadBits(stream,32,"ReadGOV: group_start_code",trace,CODE);
  if (tmpvar!= GROUP_START_CODE){
    printf("\n*** ERROR: ReadGOV: This is no GOV\n");
    exit(1);
  }
  tmpvar = (Int) BitstreamReadBits(stream,5, "ReadGOV: Hour",trace,NUM);
   time_s= tmpvar*3600;
   tmpvar = (Int) BitstreamReadBits(stream,6, "ReadGOV: Minute",trace,NUM);
   time_s += tmpvar*60;
   tmpvar = (Int) BitstreamReadBits(stream,1,"ReadGOV: Marker_bit",trace,NUM);
   tmpvar = (Int) BitstreamReadBits(stream,6,"ReadGOV: Second",trace,NUM);
   time_s += tmpvar;
   *time=time_s;
   tmpvar = (Int) BitstreamReadBits(stream,1, "ReadGOV: closed_gov",trace,NUM);
   *closed_gov=tmpvar;
   tmpvar = (Int) BitstreamReadBits(stream,1, "ReadGOV: broken_link",trace,NUM);
   *broken_link=tmpvar;
   *readen_bits += 52;
   BitstreamByteAlign(stream);

   /* Included due to N2171 Cl. 2.1.9 (user data) MW 25-MAR-1998 */
   tmpvar = (Int) BitstreamShowBits(stream,USER_DATA_START_CODE_LENGTH);
   if (tmpvar == USER_DATA_START_CODE) 
     *readen_bits += GetUserDataFromBitstream(stream,trace,NULL);

}

/***********************************************************CommentBegin******
 *
 * -- DecodeTimeControl
 *
 * Author :  Ryuji Kitaura (sharp)
 *
 * Modified: 11.09.98 Tomoko Aono (sharp), Guido Heising (HHI): 
 *                    temporal references for temporal scalability changed 
 *
 ********************************************************************/
Int DecodeTimeControl(Bitstream *stream,
		      Int vop_display_time,
		      Int prev_vop_display_time,
	              Int next_display_time,
		      Int stop_decoding_vol,
                      Vop *curr_vop,
		      Int random_access_start_time,
		      Float next_display_time_in,
		      Int *enh_decoding_flag,
		      Trace *trace)
{
  Int decode_start_flag=0;
  Int predic_type;
  Int time;
  static Int prev_predic_type;
  
  static Int base_resolution = 1,
             enh_resolution  = 1;
/*** 9/11 changed by sharp */
  static Int str_base,min_inc=0;
/* 9/11 changed by sharp ***/

  
  if (stop_decoding_vol == 0) {
    if(!temporal_scalability){
      predic_type = GetVopPredictionType(curr_vop);
      if(prev_vop_display_time < next_display_time
	 && (vop_display_time < next_display_time || predic_type != B_VOP))
	decode_start_flag = 1;
    }
    else{    
      if(CheckBaseVop(curr_vop)){ /* base_time */
	base_resolution = GetVopTimeIncrementResolution(curr_vop);
	if(last_base_decoding_time[0] < 0 || last_base_decoding_time[2] < 0)
/*** 9/11 changed by sharp */
	{
	  decode_start_flag = 1;
	  str_base=last_base_decoding_time[2]; /* first time_inc in Base */
	}
/* 9/11 changed by sharp ***/
	else {
	  predic_type=preview_vop_header(stream,trace);
	  if (predic_type== -1)
	    return(1);

/*** 9/11 changed by sharp */
/*          time = (last_enh_decoding_time + 1) * base_resolution / enh_resolution; */
          time = (last_enh_decoding_time + min_inc) * base_resolution / enh_resolution;
/* 9/11 changed by sharp ***/	 	  
        
	  if (last_base_decoding_time[1]<0) { /* no B_VOP */
	    if (predic_type==B_VOP) {
	      decode_start_flag = 1;
	      prev_predic_type=B_VOP;
	    }
	    else {
	      if(time==last_base_decoding_time[2])
		decode_start_flag = 1;
	      else decode_start_flag = 0;
	    }
	  }
	  
	  else { /* B_VOP+P_VOP */
	    if( last_enh_decoding_time < 0                                                        ||
	        (predic_type==B_VOP && prev_predic_type==P_VOP)                                   ||
	       ((predic_type==I_VOP || predic_type==P_VOP) && time == last_base_decoding_time[2]) ||
	        (predic_type==B_VOP                        && time == last_base_decoding_time[1])    ){ 
	      decode_start_flag = 1;
	      prev_predic_type  = predic_type;
	    }
	    else decode_start_flag = 0;
	  }
	}
      }
      else { /* enhance_time */
/*** 9/11 changed by sharp */
	if (last_enh_decoding_time!=0 && min_inc==0)
	  min_inc=last_enh_decoding_time-str_base; /* min frame increment */
/* 9/11 changed by sharp ***/
          enh_resolution = GetVopTimeIncrementResolution(curr_vop);
	  if((Int)(last_base_decoding_time[0] * enh_resolution / base_resolution ) == next_display_time ||
	     (Int)(last_base_decoding_time[1] * enh_resolution / base_resolution ) == next_display_time ||
	     (Int)(last_base_decoding_time[2] * enh_resolution / base_resolution ) == next_display_time )
	    decode_start_flag = 0;
	  else if(!*enh_decoding_flag){
	    decode_start_flag = 1;
	    *enh_decoding_flag = 1;
	  }
	  else decode_start_flag = 0; 
      }
    }
  }
  else decode_start_flag = 0;
  
  return(decode_start_flag);
}

Int IsSOLlayer(char *bitstream_filename)
{
  FILE *stream;
  Int  IsSOLflag=TRUE;
  Int  len = VOL_START_CODE_LENGTH;
  UInt StartCode=0;

  stream = fopen(bitstream_filename, "rb");

  /* skip the VO_START_CODE and VO_ID */
  if (VO_HEADER_LENGTH%8 != 0){
    fprintf(stderr, "\nError:VOL Start Code should be byte aligned");
    exit(-1);
  }
  fseek(stream, VO_HEADER_LENGTH/8, SEEK_SET);

  while(len>8){
    StartCode += getc(stream);
    StartCode <<= 8;
    len -= 8;
  }
  StartCode +=  getc(stream);
  StartCode >>= (8-len);

  if (StartCode == VOL_START_CODE)
    IsSOLflag = FALSE;

  fclose(stream);

  return IsSOLflag;
}







/***********************************************************CommentBegin******
 *
 * -- GetUserDataFromBitstream() -- writes user data from file into bitstream
 *
 * Author :             
 *      Michael Wollborn, University of Hannover
 *
 * Created :            
 *      23-03-98
 *
 * Purpose :
 *	This function reads the user data from a the bitstream to a given file.
 * 
 * Arguments in :       
 *	VolConfig *vol_config - VOL config information
 *
 * Arguments in/out :   
 *	Image *header_bitstream - intermediate bitstream for VOL header
 *
 * Arguments out :      
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      
 *
 * Modified :           
 *
 ***********************************************************CommentEnd********/

Int
GetUserDataFromBitstream(Bitstream *stream, Trace *trace, Char *filename)
{
  Int filesize=0, tmpvar;

  FILE *ud_file;
  Byte data;
  
  /* Create output file for user data */
  if(filename != NULL)
    {
      if((ud_file = fopen(filename,"wb")) == NULL)
	{
	  fprintf(stderr,"ERROR(GetUserDataToBitstream): unable to open %s\n",
		  filename);
	  exit(1);
	}
    }

  /* Read USER_DATA_START_CODE from bitstream */
  tmpvar = (Int)BitstreamReadBits(stream,USER_DATA_START_CODE_LENGTH,
				  "GetUserDataFromBitstream",trace,NUM);

  /* Read user data from bitstream until next start code */
  while((tmpvar = (Int)0x00ffffff & (Int)BitstreamShowBits(stream,START_CODE_PREFIX_LENGTH+8))
	  != START_CODE_PREFIX)
    {
      data = (Byte) BitstreamReadBits(stream,8,"User data",trace,NUM);

      if(filename != NULL)
        filesize += fwrite(&data,1,1,ud_file);
    }
  BitstreamByteAlign(stream);

  /* Close file and return number of bits written*/
  if(filename != NULL)
    fclose(ud_file);

  return(filesize*3);
}




/***********************************************************CommentBegin******
 *
 * -- BitstreamReadComplexityEstimationHeader() -- read from bitstream to vol
 *      define_VOP_complexity_estimation_header
 *
 * Author :
 *      Marc Mongenet, Swiss Federal Institute of Technology, Lausanne
 *
 * Created :
 *      05-06-98
 *
 * Purpose :
 *	This function reads the complexity estimation from the bitstream and
 *      update the VOL structure. exit() is called if the syntax is wrong,
 *      ie a Start Code emulation could happen because all sub-flags are
 *      disabled with the main flag enabled.
 *
 * Arguments in :
 *      Trace * trace        -
 *
 * Arguments in/out :
 *	Bitstream * stream   - the bitstream to read
 *      Vol * vol            - the VOL structure to update
 *
 * Arguments out :
 *
 * Return values :
 *      number of bits read.
 *
 * Side effects :
 *
 * Description :
 *
 * See also :
 *
 * Modified :
 *    26.10.99 Massimo Ravasi (EPFL): added Complexity Estimation syntax support
 *                                    Update version 2
 *
 ***********************************************************CommentEnd********/

Int
BitstreamReadComplexityEstimationHeader(Bitstream * stream,
					Vol * vol,
					Trace * trace)
{
  Int bits_read = 0;
  Int v;

  printf("Complexity Estimation is ENABLED\n");

  PutVolEstimationMethod((Int)BitstreamReadBits(stream, 2, "estimation_method", trace, NUM),
			 vol);
  bits_read += 2;
  if (GetVolEstimationMethod(vol) != 0 && GetVolEstimationMethod(vol) != 1) {
    fprintf(stderr, "\nERROR(BitstreamReadComplexityEstimationHeader): Unknown estimation method.\n");
    exit(1);
  }

  v = (Int)BitstreamReadBits(stream, 1, "shape_complexity_estimation_disable", trace, FLAG);
  bits_read += 1;
  PutVolShapeComplexityEstimationDisable(v, vol);
  if (!GetVolShapeComplexityEstimationDisable(vol)) {
    PutVolOpaque((Int)BitstreamReadBits(stream, 1, "opaque", trace, FLAG), vol);
    PutVolTransparent((Int)BitstreamReadBits(stream, 1, "transparent", trace, FLAG), vol);
    PutVolIntraCAE((Int)BitstreamReadBits(stream, 1, "intra_cae", trace, FLAG), vol);
    PutVolInterCAE((Int)BitstreamReadBits(stream, 1, "inter_cae", trace, FLAG), vol);
    PutVolNoUpdate((Int)BitstreamReadBits(stream, 1, "no_update", trace, FLAG), vol);
    PutVolUpsampling((Int)BitstreamReadBits(stream, 1, "upsampling", trace, FLAG), vol);
    bits_read += 6;
    if (GetVolOpaque(vol) == 0 &&
	GetVolTransparent(vol) == 0 &&
	GetVolIntraCAE(vol) == 0 &&
	GetVolInterCAE(vol) == 0 &&
	GetVolNoUpdate(vol) == 0 &&
	GetVolUpsampling(vol) == 0) {
      fprintf(stderr,
	      "ERROR(BitstreamReadComplexityEstimationHeader): "
	      "'shape_complexity_estimation_disable' is disabled but "
	      "all associated flags are disabled.\n");
      exit(1);
    }
  }

  v = (Int)BitstreamReadBits(stream, 1, "texture_complexity_estimation_set_1_disable", trace, FLAG);
  bits_read += 1;
  PutVolTextureComplexityEstimationSet1Disable(v, vol);
  if (!GetVolTextureComplexityEstimationSet1Disable(vol)) {
    PutVolIntraBlocks((Int)BitstreamReadBits(stream, 1, "intra_blocks", trace, FLAG), vol);
    PutVolInterBlocks((Int)BitstreamReadBits(stream, 1, "inter_blocks", trace, FLAG), vol);
    PutVolInter4vBlocks((Int)BitstreamReadBits(stream, 1, "inter4v_blocks", trace, FLAG), vol);
    PutVolNotCodedBlocks((Int)BitstreamReadBits(stream, 1, "not_coded_blocks", trace, FLAG), vol);
    bits_read += 4;
    if (GetVolIntraBlocks(vol) == 0 &&
	GetVolInterBlocks(vol) == 0 &&
	GetVolInter4vBlocks(vol) == 0 &&
	GetVolNotCodedBlocks(vol) == 0) {
      fprintf(stderr,
	      "ERROR(BitstreamReadComplexityEstimationHeader): "
	      "'texture_complexity_estimation_set_1_disable' is disabled but "
	      "all associated flags are disabled.\n");
      exit(1);
    }
  }


  /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.4 */
  BitstreamReadBits(stream, 1, "marker_bit", trace, FLAG);
  bits_read += 1;

  v = (Int)BitstreamReadBits(stream, 1, "texture_complexity_estimation_set_2_disable", trace, FLAG);
  bits_read += 1;
  PutVolTextureComplexityEstimationSet2Disable(v, vol);
  if (!GetVolTextureComplexityEstimationSet2Disable(vol)) {
    PutVolDCTCoefs((Int)BitstreamReadBits(stream, 1, "dct_coefs", trace, FLAG), vol);
    PutVolDCTLines((Int)BitstreamReadBits(stream, 1, "dct_lines", trace, FLAG), vol);
    PutVolVLCSymbols((Int)BitstreamReadBits(stream, 1, "vlc_symbols", trace, FLAG), vol);
    PutVolVLCBits((Int)BitstreamReadBits(stream, 1, "vlc_bits", trace, FLAG), vol);
    bits_read += 4;
    if (GetVolDCTCoefs(vol) == 0 &&
	GetVolDCTLines(vol) == 0 &&
	GetVolVLCSymbols(vol) == 0 &&
	GetVolVLCBits(vol) == 0) {
      fprintf(stderr,
	      "ERROR(BitstreamReadComplexityEstimationHeader): "
	      "'texture_complexity_estimation_set_2_disable' is disabled but "
	      "all associated flags are disabled.\n");
      exit(1);
    }
  }

  v = (Int)BitstreamReadBits(stream, 1, "motion_compensation_complexity_disable", trace, FLAG);
  bits_read += 1;
  PutVolMotionCompensationComplexityDisable(v, vol);
  if (!GetVolMotionCompensationComplexityDisable(vol)) {
    PutVolAPM((Int)BitstreamReadBits(stream, 1, "apm", trace, FLAG), vol);
    PutVolNPM((Int)BitstreamReadBits(stream, 1, "npm", trace, FLAG), vol);
    PutVolInterpolateMCQ((Int)BitstreamReadBits(stream, 1, "interpolate_mc_q", trace, FLAG), vol);
    PutVolForwBackMCQ((Int)BitstreamReadBits(stream, 1, "forw_back_mc_q", trace, FLAG), vol);
    PutVolHalfpel2((Int)BitstreamReadBits(stream, 1, "halfpel2", trace, FLAG), vol);
    PutVolHalfpel4((Int)BitstreamReadBits(stream, 1, "halfpel4", trace, FLAG), vol);
    bits_read += 6;
    if (GetVolAPM(vol) == 0 &&
	GetVolNPM(vol) == 0 &&
	GetVolInterpolateMCQ(vol) == 0 &&
	GetVolForwBackMCQ(vol) == 0 &&
	GetVolHalfpel2(vol) == 0 &&
	GetVolHalfpel4(vol) == 0) {
      fprintf(stderr,
	      "ERROR(BitstreamReadComplexityEstimationHeader): "
	      "'motion_compensation_complexity_disable' is disabled but "
	      "all associated flags are disabled.\n");
      exit(1);
    }
  }

  /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.4 */
  BitstreamReadBits(stream, 1, "marker_bit", trace, FLAG);
  bits_read += 1;

	/* START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 26 Oct 1999 */
  if (GetVolEstimationMethod(vol) == 1) {
		v = (Int)BitstreamReadBits(stream, 1, "version_2_complexity_estimation_disable", trace, FLAG);
		bits_read += 1;
		PutVolVersion2ComplexityEstimationDisable(v, vol);
		if (!GetVolVersion2ComplexityEstimationDisable(vol)) {
			PutVolSadct((Int)BitstreamReadBits(stream, 1, "sadct", trace, FLAG), vol);
			PutVolQuarterpel((Int)BitstreamReadBits(stream, 1, "quarterpel", trace, FLAG), vol);
			bits_read += 2;
			if (GetVolSadct(vol) == 0 &&
					GetVolQuarterpel(vol) == 0) {
				fprintf(stderr,
								"ERROR(BitstreamReadComplexityEstimationHeader): "
								"'version_2_complexity_estimation_disable' is disabled but "
								"all associated flags are disabled.\n");
				exit(1);
			}
		}
  }
  /* END: Complexity Estimation syntax support - Update Version 2 */

  
  return bits_read;
}




/***********************************************************CommentBegin******
 *
 * -- BitstreamReadDCECS() -- read from bitstream VOP_complexity_estimation_header
 *
 * Author :
 *      Marc Mongenet, Swiss Federal Institute of Technology, Lausanne
 *
 * Created :
 *      08-JUN-98
 *
 * Purpose :
 *	This function reads the complexity estimation from the bitstream and
 *      store it in the VOP structure. exit() is called if an illegal null
 *      is read from the bitstream.
 *
 * Arguments in :
 *	Trace * trace        -
 *
 * Arguments in/out :
 *      Bitstream * stream   - The bitstream to read from
 *	Vop * vop            - The VOP structure to be updated
 *
 * Arguments out :
 *
 * Return values :
 *      number of bits read.
 *
 * Side effects :
 *
 * Description :
 *
 * See also :
 *
 * Modified :
 *    26.10.99 Massimo Ravasi (EPFL): added Complexity Estimation syntax support
 *                                    Update version 2
 *
 ***********************************************************CommentEnd********/

Int BitstreamReadDCECS (Bitstream * stream,
			Vop * vop,
			Trace * trace)
{
  Int bits_read = 0;

  if (GetVopPredictionType(vop) == I_VOP ||
      GetVopPredictionType(vop) == P_VOP ||
      GetVopPredictionType(vop) == B_VOP) {

    if (GetVopOpaque(vop)) {
      bits_read += 8;
      PutVopDCECSOpaque((Int)BitstreamReadBits(stream, 8, "dcecs_opaque", trace, NUM), vop);
      if (GetVopDCECSOpaque(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_opaque' null value\n");
	exit(1);
      }
    }
    if (GetVopTransparent(vop)) {
      bits_read += 8;
      PutVopDCECSTransparent((Int)BitstreamReadBits(stream, 8, "dcecs_transparent", trace, NUM), vop);
      if (GetVopDCECSTransparent(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_transparent' null value\n");
	exit(1);
      }
    }
    if (GetVopIntraCAE(vop)) {
      bits_read += 8;
      PutVopDCECSIntraCAE((Int)BitstreamReadBits(stream, 8, "dcecs_intra_cae", trace, NUM), vop);
      if (GetVopDCECSIntraCAE(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_intra_cae' null value\n");
	exit(1);
      }
    }
    if (GetVopInterCAE(vop)) {
      bits_read += 8;
      PutVopDCECSInterCAE((Int)BitstreamReadBits(stream, 8, "dcecs_inter_cae", trace, NUM), vop);
      if (GetVopDCECSInterCAE(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_inter_cae' null value\n");
	exit(1);
      }
    }
    if (GetVopNoUpdate(vop)) {
      bits_read += 8;
      PutVopDCECSNoUpdate((Int)BitstreamReadBits(stream, 8, "dcecs_no_update", trace, NUM), vop);
      if (GetVopDCECSNoUpdate(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_no_update' null value\n");
	exit(1);
      }
    }
    if (GetVopUpsampling(vop)) {
      bits_read += 8;
      PutVopDCECSUpsampling((Int)BitstreamReadBits(stream, 8, "dcecs_upsampling", trace, NUM), vop);
      if (GetVopDCECSUpsampling(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_upsampling' null value\n");
	exit(1);
      }
    }
  }
  
  if (GetVopIntraBlocks(vop)) {
    bits_read += 8;
    PutVopDCECSIntraBlocks((Int)BitstreamReadBits(stream, 8, "dcecs_intra_blocks", trace, NUM), vop);
    if (GetVopDCECSIntraBlocks(vop) == 0) {
      fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_intra_blocks' null value\n");
      exit(1);
    }
  }
  if (GetVopNotCodedBlocks(vop)) {
    bits_read += 8;
    PutVopDCECSNotCodedBlocks((Int)BitstreamReadBits(stream, 8, "dcecs_not_coded_blocks", trace, NUM), vop);
    if (GetVopDCECSNotCodedBlocks(vop) == 0) {
      fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_not_coded_blocks' null value\n");
      exit(1);
    }
  }
  if (GetVopDCTCoefs(vop)) {
    bits_read += 8;
    PutVopDCECSDCTCoefs((Int)BitstreamReadBits(stream, 8, "dcecs_dct_coefs", trace, NUM), vop);
    if (GetVopDCECSDCTCoefs(vop) == 0) {
      fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_dct_coefs' null value\n");
      exit(1);
    }
  }
  if (GetVopDCTLines(vop)) {
    bits_read += 8;
    PutVopDCECSDCTLines((Int)BitstreamReadBits(stream, 8, "dcecs_dct_lines", trace, NUM), vop);
    if (GetVopDCECSDCTLines(vop) == 0) {
      fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_dct_lines' null value\n");
      exit(1);
    }
  }
  if (GetVopVLCSymbols(vop)) {
    bits_read += 8;
    PutVopDCECSVLCSymbols((Int)BitstreamReadBits(stream, 8, "dcecs_vlc_symbols", trace, NUM), vop);
    if (GetVopDCECSVLCSymbols(vop) == 0) {
      fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_vlc_symbols' null value\n");
      exit(1);
    }
  }
  if (GetVopVLCBits(vop)) {
    bits_read += 4;
    PutVopDCECSVLCBits((Int)BitstreamReadBits(stream, 4, "dcecs_vlc_bits", trace, NUM), vop);
    if (GetVopDCECSVLCBits(vop) == 0) {
      fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_vlc_bits' null value\n");
      exit(1);
    }
  }

  if (GetVopPredictionType(vop) == P_VOP ||
      GetVopPredictionType(vop) == B_VOP ||
      (GetVopPredictionType(vop) == SPRITE_VOP && GetVopSpriteUsage(vop) == STATIC_SPRITE) ) {
    if (GetVopInterBlocks(vop)) {
      bits_read += 8;
      PutVopDCECSInterBlocks((Int)BitstreamReadBits(stream, 8, "dcecs_inter_blocks", trace, NUM), vop);
      if (GetVopDCECSInterBlocks(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_inter_blocks' null value\n");
	exit(1);
      }
    }
    if (GetVopInter4vBlocks(vop)) {
      bits_read += 8;
      PutVopDCECSInter4vBlocks((Int)BitstreamReadBits(stream, 8, "dcecs_inter4v_blocks", trace, NUM), vop);
      if (GetVopDCECSInter4vBlocks(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_inter4v_blocks' null value\n");
	exit(1);
      }
    }
    if (GetVopAPM(vop)) {
      bits_read += 8;
      PutVopDCECSAPM((Int)BitstreamReadBits(stream, 8, "dcecs_apm", trace, NUM), vop);
      if (GetVopDCECSAPM(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_apm' null value\n");
	exit(1);
      }
    }
    if (GetVopNPM(vop)) {
      bits_read += 8;
      PutVopDCECSNPM((Int)BitstreamReadBits(stream, 8, "dcecs_npm", trace, NUM), vop);
      if (GetVopDCECSNPM(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_npm' null value\n");
	exit(1);
      }
    }
    if (GetVopForwBackMCQ(vop)) {
      bits_read += 8;
      PutVopDCECSForwBackMCQ((Int)BitstreamReadBits(stream, 8, "dcecs_forw_back_mc_q", trace, NUM), vop);
      if (GetVopDCECSForwBackMCQ(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_forw_back_mc_q' null value\n");
	exit(1);
      }
    }
    if (GetVopHalfpel2(vop)) {
      bits_read += 8;
      PutVopDCECSHalfpel2((Int)BitstreamReadBits(stream, 8, "dcecs_halfpel2", trace, NUM), vop);
      if (GetVopDCECSHalfpel2(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_halfpel2' null value\n");
	exit(1);
      }
    }
    if (GetVopHalfpel4(vop)) {
      bits_read += 8;
      PutVopDCECSHalfpel4((Int)BitstreamReadBits(stream, 8, "dcecs_halfpel4", trace, NUM), vop);
      if (GetVopDCECSHalfpel4(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_halfpel4' null value\n");
	exit(1);
      }
    }
  }

  if (GetVopPredictionType(vop) == B_VOP ||
      (GetVopPredictionType(vop) == SPRITE_VOP && GetVopSpriteUsage(vop) == STATIC_SPRITE) ) {
    if (GetVopInterpolateMCQ(vop)) {
      bits_read += 8;
      PutVopDCECSInterpolateMCQ((Int)BitstreamReadBits(stream, 8, "dcecs_interpolate_mc_q", trace, NUM), vop);
      if (GetVopDCECSInterpolateMCQ(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_interpolate_mc_q' null value\n");
	exit(1);
      }
    }
  }

	/* START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 26 Oct 1999 */
  if (GetVopPredictionType(vop) == I_VOP ||
      GetVopPredictionType(vop) == P_VOP ||
      GetVopPredictionType(vop) == B_VOP) {

    if (GetVopSadct(vop)) {
      bits_read += 8;
      PutVopDCECSSadct((Int)BitstreamReadBits(stream, 8, "dcecs_sadct", trace, NUM), vop);
      if (GetVopDCECSSadct(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_sadct' null value\n");
	exit(1);
      }
    }
	}

  if (GetVopPredictionType(vop) == P_VOP ||
      GetVopPredictionType(vop) == B_VOP ||
      (GetVopPredictionType(vop) == SPRITE_VOP && GetVopSpriteUsage(vop) == STATIC_SPRITE)) {

    if (GetVopQuarterpel(vop)) {
      bits_read += 8;
      PutVopDCECSQuarterpel((Int)BitstreamReadBits(stream, 8, "dcecs_quarterpel", trace, NUM), vop);
      if (GetVopDCECSQuarterpel(vop) == 0) {
	fprintf(stderr, "ERROR(BitstreamReadDCECS): Illegal 'dcecs_quarterpel' null value\n");
	exit(1);
      }
    }
	}
  /* END: Complexity Estimation syntax support - Update Version 2 */

  return bits_read;
}



/***********************************************************CommentBegin******
 *
 * -- InitShortVideoHeader -- Initializing of short_video_header Vol
 *
 * Authors :		
 *      Gunn K Klungsoeyr (Telenor R&D/ ACTS MoMuSys)
 *      Aasmund Sandvand (Telenor R&D/ ACTS MoMuSys)
 *
 * Created :		
 *	22-May-1998
 *
 * Purpose :		
 *
 * Arguments in : 	
 *
 * Arguments in/out :	
 *	Vol *curr_vol,		pointer on considered vol
 *
 * Arguments out :	
 *
 * Return values :	
 *
 * Side effects :	
 *
 * Description :	
 *
 * See also :
 *
 * Modified : 
 *	
 *
 ***********************************************************CommentEnd********/

Void InitShortVideoHeader(Vol *curr_vol)
{
	PutVolVOId(0,curr_vol);
	PutVolId(0,curr_vol);
	PutVolIsObjectLayerIdentifier(0,curr_vol);
	PutVolVolControlParameters(0,curr_vol);
	PutVolShape(0,curr_vol); /*rectangular*/
	PutVolTimeIncrementResolution(30000,curr_vol);
	PutVolFixedVopRate(0,curr_vol);
	PutVolWidth(2,curr_vol);	/*dummy*/
	PutVolHeight(2,curr_vol);	/*dummy*/
	PutVolOBMCDisable(1,curr_vol);	/*default*/
	PutVolSpriteUsage(0,curr_vol);
	PutVolBitsPerPixel(8,curr_vol);    
	PutVolQuantType(0,curr_vol);
	PutVolComplexityEstimationDisable(1,curr_vol);
	PutVolErrorResDisable(1,curr_vol);
	PutVolScalability(0,curr_vol);
	PutVolRandomAccessibleVol(0,curr_vol);
	PutVolACDCPredDisable(1,curr_vol);

	printf("** InitShortVideoHeader finished ..\n");

}

/***********************************************************CommentBegin******
 *
 * -- VideoPlaneWithShortHeader -- Decodes the short_video_header information from the bitstream
 *
 * Authors :		
 *      Gunn K Klungsoeyr (Telenor R&D/ ACTS MoMuSys)
 *      Aasmund Sandvand (Telenor R&D/ ACTS MoMuSys)
 *
 * Created :		
 *	22-May-1998
 *
 * Purpose :		
 *	 Decode the picture header information from an H.263-bitstream.
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *	Int *readen_bits	pointer to value of up to now readen_bits
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	
 *
 ***********************************************************CommentEnd********/
   
Int VideoPlaneWithShortHeader(Bitstream *stream, Int vo_id, Trace *trace, Vop *curr_vop, 
	       Int *readen_bits, Int *stop_decoding)
{
  Int tmpvar;
  Int time;
  /*  Int time_base=0;  :unused SB */
  Int pei;

  static Int not_first, last, count;

 
  /*****
   *
   *    Disable tracing for time preview 
   *
   *****/
  trace->trace = 0;
 
  /*****
   *
   *    Set trace level for H263 header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VOP_header_def;

  TraceBreakLine(trace);
  
  /*****
   *
   *	Read the short_video_header from the bitstream
   *
   *****/

  tmpvar = (Int) BitstreamShowBits(stream, SHORT_VIDEO_START_MARKER_LENGTH);

  if (tmpvar !=  SHORT_VIDEO_START_MARKER) {
      if (tmpvar == SHORT_VIDEO_END_MARKER) {
		  *stop_decoding = 1;
		  printf("End of sequence \n");
		  return(0);
      }
      *stop_decoding = 1;
      printf("END OF BITSTREAM or NO PICTURE-START-CODE \n");
      return(0);
  }

  BitstreamFlushBits(stream, SHORT_VIDEO_START_MARKER_LENGTH);
  *readen_bits += SHORT_VIDEO_START_MARKER_LENGTH;

/* Temporal reference. 
   Using vop_time_increment_resolution = 30000 */

  tmpvar = (Int) BitstreamReadBits(stream,8,
				   "h263_temporal_reference",trace,NUM);

  *readen_bits += 8;

  if (tmpvar < last) {
    count++;
  }
  last = tmpvar;

  time = (tmpvar + count*256);

  PutVopModTimeBase((1001*time)/curr_vop->time_increment_resolution, curr_vop );
  PutVopTimeInc((1001*time)%curr_vop->time_increment_resolution, curr_vop );

  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "h263_marker_bit",trace,FLAG);

  *readen_bits += 1;   

  if(!tmpvar)
    {
    printf ("This is not a legal H.263 bitstream\n");
    exit (1);
    }

  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "h263_bitstream_type",trace,FLAG);

  *readen_bits += 1;   

  if(tmpvar)
    {
    printf ("This is not a legal H.263 bitstream\n");
    exit (1);
    }

  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "h263_split_screen_indicator",trace,FLAG);

  *readen_bits += 1;   

  if(tmpvar)
    {
    printf ("Split screen indicator is ON\n");
    exit (1);
    }

  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "h263_document_freeze_camera",trace,FLAG);

  *readen_bits += 1;   

  if(tmpvar)
    {
    printf ("Document camera indicator is switched on\n");
    exit (1);
    }

  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "h263_freeze_picture_release",trace,FLAG);

  *readen_bits += 1;   

  if(tmpvar)
    {
    printf ("Freeze picture release is switched ON\n");
    exit (1);
    }

  tmpvar = (Int) BitstreamReadBits(stream,3,
				   "source_format", trace,CODE);

  *readen_bits += 3;   

  switch (tmpvar) {
    case 1:
      if (not_first == 0) {
        PutVopWidth(128,curr_vop); 
        PutVopHeight(96,curr_vop);
		PutVopNumMacroblocksInGob(8,curr_vop);
		PutVopNumGobsInVop(6,curr_vop);
        not_first = 1;
		FreeVopChannels(curr_vop);
		AllocVopChannels(curr_vop,
                                 GetVopWidth(curr_vop),
                                 GetVopHeight(curr_vop),
                                 GetVopAuxCompCount(curr_vop));
      }
      else {
        if (GetVopWidth(curr_vop) != 128 || GetVopHeight(curr_vop) != 96) {
          printf ("H.263 picture format changed during session\n");
          exit (1);
        }
      } 
      break;
    case 2:
      if (not_first == 0) {
        PutVopWidth(176,curr_vop); 
        PutVopHeight(144,curr_vop);
		PutVopNumMacroblocksInGob(11,curr_vop);
		PutVopNumGobsInVop(9,curr_vop);
        not_first = 1;
		FreeVopChannels(curr_vop);
		AllocVopChannels(curr_vop,
                                 GetVopWidth(curr_vop),
                                 GetVopHeight(curr_vop),
                                 GetVopAuxCompCount(curr_vop));
      }
      else {
        if (GetVopWidth(curr_vop) != 176 || GetVopHeight(curr_vop) != 144) {
          printf ("H.263 picture format changed during session\n");
          exit (1);
        }
      } 
      break;   
    case 3:
      if (not_first == 0) {
        PutVopWidth(352,curr_vop); 
        PutVopHeight(288,curr_vop);
		PutVopNumMacroblocksInGob(22,curr_vop);
		PutVopNumGobsInVop(18,curr_vop);
        not_first = 1;
		FreeVopChannels(curr_vop);
		AllocVopChannels(curr_vop,
                                 GetVopWidth(curr_vop),
                                 GetVopHeight(curr_vop),
                                 GetVopAuxCompCount(curr_vop));
      }
      else {
        if (GetVopWidth(curr_vop) != 352 || GetVopHeight(curr_vop) != 288) {
          printf ("H.263 picture format changed during session\n");
          exit (1);
        }
      } 
      break;
    case 4:
      if (not_first == 0) {
        PutVopWidth(704,curr_vop); 
        PutVopHeight(576,curr_vop);
		PutVopNumMacroblocksInGob(88,curr_vop);
		PutVopNumGobsInVop(18,curr_vop);
        not_first = 1;
		FreeVopChannels(curr_vop);
		AllocVopChannels(curr_vop,
                                 GetVopWidth(curr_vop),
                                 GetVopHeight(curr_vop),
                                 GetVopAuxCompCount(curr_vop));
      }
      else {
        if (GetVopWidth(curr_vop) != 704 || GetVopHeight(curr_vop) != 576) {
          printf ("H.263 picture format changed during session\n");
          exit (1);
        }
      } 
      break;
    case 5:
      if (not_first == 0) {
        PutVopWidth(1408,curr_vop); 
        PutVopHeight(1152,curr_vop);
		PutVopNumMacroblocksInGob(352,curr_vop);
		PutVopNumGobsInVop(18,curr_vop);
        not_first = 1;
		FreeVopChannels(curr_vop);
		AllocVopChannels(curr_vop,
                                 GetVopWidth(curr_vop),
                                 GetVopHeight(curr_vop),
                                 GetVopAuxCompCount(curr_vop));
      }
      else {
        if (GetVopWidth(curr_vop) != 1408 || GetVopHeight(curr_vop) != 1152) {
          printf ("H.263 picture format changed during session\n");
          exit (1);
        }
      } 
      break;
    default:
      printf ("H.263 source format not legal\n");
  }

  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "picture_coding_type",trace,NUM);

  *readen_bits += 1;

  PutVopPredictionType(tmpvar,curr_vop);

  tmpvar = (Int) BitstreamReadBits(stream,4,
				   "four_reserved_zero_bits",trace,FLAG);

  *readen_bits += 4;

  if(tmpvar)
    {
    printf ("four_reserved_zero_bits are not zero\n");
    exit (1);
    }


  tmpvar = (Int) BitstreamReadBits(stream,5,
				   "vop_quant",trace,NUM);

  *readen_bits += 5;

  PutVopQuantizer(tmpvar,curr_vop);

  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "zero_bit",trace,FLAG);

  *readen_bits += 1;

  if(tmpvar)
    {
    printf ("zero_bit is not zero\n");
    exit (1);
    }
 
  tmpvar = (Int) BitstreamReadBits(stream,1,
				   "pei",trace,FLAG);

  *readen_bits += 1;


  if(tmpvar)
    {
/* Changed Yoshinori Suzuki: 99/12/06 */
/* 
    printf ("Spare information expected (Not implemented yet)\n");
    exit (1);
*/
        *readen_bits += 8;
        pei = (Int) BitstreamReadBits(stream,8, "psupp",trace,CODE);
        while((tmpvar = 
                (Int) BitstreamReadBits(stream,1,"pei",trace,FLAG))==1){
                *readen_bits += 1;
                pei = (Int) BitstreamReadBits(stream,8,"psupp",trace,CODE);
                *readen_bits += 8;
        }
    }

/* Setting of other VOP-header parameters */
  PutVopQuantPrecision(5,curr_vop);
  PutVopGobNumber(0,curr_vop);
  PutVopCoded(1,curr_vop);
  PutVopRoundingType(0,curr_vop);
  PutVopIntraDCVlcThr(0,curr_vop); 
  PutVopInterlaced(0,curr_vop);
  if (GetVopPredictionType(curr_vop)!=I_VOP) 
    PutVopFCodeFor(1,curr_vop);

  printf ("** Video Plane with Short Header ..\n");

  return(1);
}

/***********************************************************CommentBegin******
 *
 * -- GobHeader --  Read gob header from bitstreams with short video header
 *
 * Authors :		
 *      Aasmund Sandvand (Telenor R&D/ ACTS MoMuSys)
 *      Gunn Klungsoeyr (Telenor R&D/ ACTS MoMuSys)
 *
 * Created :		
 *	9-Jun-1998
 *
 * Purpose :	Read gob headers from bitstreams with short video header (H.263)
 *              if they are present.
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 * Arguments in/out :
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())	
 *
 * Arguments out :	
 *
 * Return values :	
 *
 * Side effects :	
 *
 * Description :	
 *
 * See also :
 *
 * Modified : 
 *	
 *
 ***********************************************************CommentEnd********/

 Int GobHeader(Bitstream *stream, Trace *trace, Vop *curr_vop)
 {
	 Int tmpvar;
	 Int newgob;
	 Int gob_number;

	 gob_number = GetVopGobNumber(curr_vop);
	 tmpvar = (Int) BitstreamShowBits(stream, GOB_RESYNC_MARKER_LENGTH); 
	 if(gob_number != 0 && tmpvar == GOB_RESYNC_MARKER){
		 newgob = 1;
		 BitstreamFlushBits(stream, GOB_RESYNC_MARKER_LENGTH);
		 tmpvar = (Int) BitstreamReadBits(stream,5,"gob_number",trace,NUM);
		 PutVopGobNumber(tmpvar,curr_vop);
		 tmpvar = (Int) BitstreamReadBits(stream,2,"gob_frame_id",trace,NUM);
		 PutVopGobFrameId(tmpvar,curr_vop);
		 tmpvar = (Int) BitstreamReadBits(stream,GetVopQuantPrecision(curr_vop),"quant_scale",trace,NUM);
		 PutVopQuantizer(tmpvar,curr_vop);
	 }
	 else {
	         newgob = 0;
	 }
	 return(newgob);
 }

/* 01.04.01: Studio Profile */
/***********************************************************CommentBegin******
 *
 * -- InitVolList_Studio -- Initialize a list of video object layers for one VO 
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Initialize a list of layers for a specific video object. Within
 *	each VOL two VOPs are "salloced" in temporal list, one for the
 *	previous reconstructed VOP which is required for decoding of
 *	P-VOPs and one for the current VOP to be decoded.
 *
 * 
 * Arguments in : 	
 *	Int number_of_vols	number of layers for the video object
 *
 *
 * Arguments in/out :	
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :
 *	Vol *vol_list		pointer to list of layers for video object
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *	
 *
 ***********************************************************CommentEnd********/

Vol *InitVolList_Studio(Int number_of_vols)
{
  Vol *vol_list,*next_vol;
  Vop *vop1,*vop2;
  Int i;

  /*****
   *
   *	Initialize as many VOLs as necessary
   *
   *****/
  if(number_of_vols > 0)
    {
    vol_list = SallocVol();
    
    vop1 = SallocVop();
    PutVolVop(vop1,vol_list);
    
    vop2 = SallocVop();
    PutVopWidth_st(0,vop2,0);
    PutVopWidth_st(0,vop2,1);
    
    vop2->mod_time_base=-1;
    PutVopNextTemp(vop2,vop1);

    vop2 = SallocVop();
    PutVopWidth_st(0,vop2,0);
    PutVopWidth_st(0,vop2,1);
    vop2->mod_time_base=-1;
    PutVopPrevTemp(vop2,vop1);

    /* 1197-3 SONY */
    PutVopBvopTemp( NULL, vop1);
    /* 1197-3 */

    next_vol = vol_list;
    
    for(i=1; i<number_of_vols; i++)
      {
	PutVolNext(SallocVol(),next_vol);
	next_vol = GetVolNext(next_vol);
	
	vop1 = SallocVop();
	PutVolVop(vop1,next_vol);
	
	vop2 = SallocVop();
	vop2->mod_time_base=-1;
	PutVopWidth_st(0,vop2,0);
	PutVopWidth_st(0,vop2,1);
	
	PutVopNextTemp(vop2,vop1);
	
	vop2 = SallocVop();
	vop2->mod_time_base=-1;
	PutVopWidth_st(0,vop2,0);
	PutVopWidth_st(0,vop2,1);
	
	PutVopPrevTemp(vop2,vop1);
	
	/* 1197-3 SONY */
	PutVopBvopTemp( NULL, vop1);
	/* 1197-3 */
	
      }
    
    return(vol_list);
    }
  else
    return(NULL);
}

/***********************************************************CommentBegin******
 *
 * -- DecodeVolHeaderList_Studio -- Decode the headers for a list of VOLs for one VO 
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode the headers for a list of VOLs for a specific VO from given
 *	bitstreams. The bitstream for each layer is given in a separate file
 *
 * 
 * Arguments in : 	
 *	Int number_of_vols	number of layers for the video object
 *	Bitstream *stream[]	Pointer list to the bitstreams
 *	Trace *trace[]		Trace information for each layer
 *
 *
 * Arguments in/out :	
 *	Vol *vol_list	concat. list of layers for video object
 *	Int readen_bits[]	Readen bits for each layer
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :	25.03.98 M.Wollborn: N2171 user data
 *              11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *              01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *	
 *
 ***********************************************************CommentEnd********/
Void DecodeVolHeaderList_Studio(Int number_of_vols, Bitstream *stream[], 
				Trace *trace[], Vol *vol_list, Int readen_bits[],
				Int vo_id, Char vol_user_data_filename[][300],
				Int erd_flag[], Int *short_video_header, /* MW 14-MAY-1998 */
				ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
				ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
				Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])    

{
  Vol *next_vol;
  Int i;
  
  /*****
   *
   *	Decode the header for each VOL for the current VO
   *	(allocate first dummy VOP if non-rectangular shape)
   *
   *****/
  next_vol = vol_list;
  
  for(i=0; i<number_of_vols; i++)
    {
      DecodeVolHeader_Studio(stream[i],trace[i],next_vol,&readen_bits[i],vo_id,i,
			     vol_user_data_filename[i],erd_flag[i],short_video_header,
			     mot_x_P,mot_y_P,MB_decisions_P) ;
      /* N2171 user data */ /* erd_flag MW 14-MAY-1998 */

      next_vol = GetVolNext(next_vol);
    }
  
  return;
}

/***********************************************************CommentBegin******
 *
 * -- DecodeVolHeader_Studio -- Decode the header of a VOL
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode the header of a VOL
 *
 * 
 * Arguments in : 	
 *	Bitstream *stream	Pointer to the bitstream
 *	Trace *trace		Trace information for VOL
 *
 *
 * Arguments in/out :	
 *	Vol *next_vol		video object layer to be decoded
 *	Int *readen_bits	Readen bits for VOL
 *      Int *short_video_header Flag to signal short video header bitstreams (H.263)
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : Noel Brady 14-10-96
 *			Code added to read 
 *			error_resilient_disable,
 *			intra_acdc_disable,
 *			deblocking_filter_disable.
 *            14.01.97 Luis Ducla-Soares: added byte alignment for
 *			VOL_START_CODE
 *                      (call to function BitstreamByteAlign() )	
 *	28.01.97 Robert Danielsen: Small change to zigzag-scan
 *	23.04.97 Michael Wollborn: Added MPEG quantization stuff completely
 *      26.04.97 Luis Ducla Soares: Changes to allow error resilient
 *                                  decoding in the combined mode with
 *                                  data partitioning.
 * 12.05.97 Minhua Zhou: added "not_8_bit", "advanced_prediction_disable", etc.
 * 14.05.97 F. Jaureguizar: Modification of number of bits of fcodes, from
 *          2 bits to 3 bits (according to VM7.0).
 *    18.08.97 Minhua Zhou: added video_object_start_code
 * 18.06.97 M.Wollborn: Added correct trace mechanism
 * 09.07.97 A. Sandvand added storing of VO Id in VOL structure
 * 04.08.97 Minhua Zhou: removed f_codes in the VOL  
 * 06.08.97 Noel Brady: added mods for BINARY_SHAPE_ONLY mode
 * 07.08.97 Minhua Zhou: changed quant. matrix decoding
 * 08.09.97 Cecile Dufour: added 
 * 23.03.98 Michael Wollborn: moved "random_accessible_vol" due to N2171, 
 *                            Clause 2.1.7
 * 21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 * 05.06.98 Marc Mongenet (EPFL): added Complexity Estimation syntax support
 * 23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header 
 * 11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 * 01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/
Int DecodeVolHeader_Studio(Bitstream *stream, Trace *trace, Vol *curr_vol, 
                     Int *readen_bits, Int vo_id, Int file_vol_id,
                     Char vol_user_data_filename[], Int erd_flag,
		     Int *short_video_header,      
                     ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                     ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                     Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])
                             /* N2171 */ /* erd_flag MW 14-MAY-1998 */
{
  Int tmpvar,i,*qmat,j, res, bits;
  Vop	*sprite=NULL;
  TrajPoint *tmp_ref_point_coord=NULL;

  float frame_rate_value;

  Int prev_total_bits = *readen_bits;

  Int ii, jj;

  Int dec_vol_shape_ext[16] = {5,9,70,74,583,13,102,17,18,182,1463,166,1447,0,0,0};

  
  /*****
   *
   *    Set trace level for VO header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VO_header_def;

  TraceBreakLine(trace);

  /*****
   *
   *	Read the VOL header entries from the bitstream
   *
   *****/

  tmpvar = (Int) BitstreamShowBits(stream, 27);

  if (tmpvar == VO_START_CODE) 
    {
      *short_video_header = 0;

      /* video_object_start_code(32) */
      tmpvar = (Int) BitstreamReadBits(stream,27,
				       "vo_start_code",trace,CODE);
      tmpvar = (Int) BitstreamReadBits(stream,5,
				       "vo_id",trace,CODE);      
      *readen_bits +=32;
      PutVolVOId(tmpvar,curr_vol);
      
      /*****
       *
       *    Set trace level for VOL header
       *
       *****/
      if(trace->full_trace_period < 0)
	trace->trace = 1;
      else
	trace->trace = trace->VOL_header_def;
      TraceBreakLine(trace);

      /* video_object_layer_start_code(32) */
      tmpvar = (Int) BitstreamReadBits(stream,VOL_START_CODE_LENGTH,
				       "vol_start_code",trace,CODE);
      *readen_bits += VOL_START_CODE_LENGTH;
      if(tmpvar != VOL_START_CODE) {
	fprintf (stderr, "Syntax Error: No video_object_layer_start_code\n");
	exit (1);
      }
      tmpvar = (Int) BitstreamReadBits(stream,4,"vol_id",trace,CODE);
      *readen_bits += 4;
      PutVolId(tmpvar,curr_vol);
      
      /* for TPS */
      bs_vol[vo_id][tmpvar] = file_vol_id;
      
      /* random_accessible_vol(1) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"random_accessible_vol",trace,FLAG);
      *readen_bits += 1;
      PutVolRandomAccessibleVol(tmpvar,curr_vol);

      /* video_object_type_indication(8) */
      tmpvar = (Int) BitstreamReadBits(stream,8,"video_object_type_indication",trace,CODE);
      PutVolVideoObjectTypeIndication_st(tmpvar,curr_vol);
      *readen_bits += 8;
      /*
      if(tmpvar!=0x0e&&tmpvar!=0x0f)
	fprintf(stderr,"video_object_type_indication shall be either Simple or Core Studio Object Type for a Studio Profile\n");
	*/
      
      /* video_object_layer_verid(4) */
      tmpvar = (Int) BitstreamReadBits(stream,4,"visual_object_layer_verid",
				       trace,CODE);
      PutVolVisualObjectLayerVerid(tmpvar,curr_vol);
      *readen_bits +=4;
      /*
      if(tmpvar!=0x4)
	fprintf(stderr, "video_object_layer_verid shall be '0100'");
	*/
      
      /* video_object_layer_shape(2) */
      tmpvar = (Int) BitstreamReadBits(stream,2,"video_object_layer_shape",trace,CODE);
      *readen_bits += 2;
      PutVolShape(tmpvar,curr_vol);

      /* video_object_layer_shape_extension(4) */
      tmpvar = (Int) BitstreamReadBits(stream,4,"video_object_layer_shape_extension",trace,CODE);
      *readen_bits += 4;
      /*
      if(GetVolShape(curr_vol)!=3 && tmpvar!=15)
	fprintf(stderr, "video_object_layer_shape_extension shall be '1111' if not grayscale shape\n");
	*/
      
      PutVolAuxCompCount_st(dec_vol_shape_ext[tmpvar]&3, curr_vol);
      for(ii=0;ii<GetVolAuxCompCount_st(curr_vol);ii++)
	PutVolAuxCompType_st((dec_vol_shape_ext[tmpvar]>>(3*ii+2))&7, curr_vol, ii);
      for(jj=ii;jj<MAX_AUX_COMP_NUM;jj++)
	PutVolAuxCompType_st(AUX_COMP_NA, curr_vol, jj);
      
      /* progressive_sequence(1) */
      tmpvar = (Int) BitstreamReadBits(stream,1,"progressive_sequence",trace,FLAG);
      *readen_bits += 1;
      PutVolInterlaced(!tmpvar,curr_vol); 
      
      if(GetVolShape(curr_vol)!=BINARY_SHAPE_ONLY) {

	/* rgb_components (1)*/
	tmpvar = (Int) BitstreamReadBits(stream,1,"rgb_components",trace,CODE );
	*readen_bits += 1;
	PutVolRGBComponents_st(tmpvar,curr_vol);
	
	/* chroma_format (2bits) */
	tmpvar = (Int) BitstreamReadBits(stream,2,"chroma_format",trace,CODE );
	*readen_bits += 2;
	if(tmpvar == 0) {
	  fprintf(stderr, "Syntax Error: chroma_format '00' is reserved.\n");
	  exit(1);
	}
	else if(tmpvar == 1)
	  PutVolChromaFormat_st(CHROMA420,curr_vol);
	else if(tmpvar == 2)
	  PutVolChromaFormat_st(CHROMA422,curr_vol);
	else if(tmpvar == 3)
	  PutVolChromaFormat_st(CHROMA444,curr_vol);
	else {
	  fprintf(stderr, "wrong chroma_format\n");
	  exit(3);
	}

	/*
	if ( GetVolRGBComponents_st ( curr_vol ) && GetVolChromaFormat_st ( curr_vol ) != CHROMA444 )
	  fprintf ( stderr, "rgb_component==RGB, chroma_format!=4:4:4\n");
	  */
	
	/* bits_per_pixel (4bits) */
	tmpvar = (Int) BitstreamReadBits(stream,4,"bits_per_pixel",trace,CODE);
	*readen_bits += 4;
	if(tmpvar!=8 && tmpvar!=10 && tmpvar!=12) {
	  fprintf(stderr, "Syntax Error: bits_per_pixel shall be one of 8/10/12\n");
	  exit(1);
	}
	PutVolBitsPerPixel(tmpvar,curr_vol);
      }

      if(GetVolShape(curr_vol)==RECTANGULAR) {
	
	BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	*readen_bits += 1;
	
	/* vol_width (14 bits) */
	tmpvar = (Int) BitstreamReadBits(stream,14,"vol_width",trace,NUM);
	*readen_bits += 14;
	PutVolWidth(tmpvar,curr_vol);
	  
	BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	*readen_bits += 1;
	  
	/* vol_height (14 bits) */
	tmpvar = (Int) BitstreamReadBits(stream,14,"vol_height",trace,NUM);
	*readen_bits += 14;
	PutVolHeight(tmpvar,curr_vol);

	BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	*readen_bits += 1;	  
      }
      
      tmpvar = (Int) BitstreamReadBits( stream, 4, "aspect_ratio_info", trace, CODE );
      /*
      if(tmpvar==0)
	fprintf(stderr, "pixel_aspect_ratio '0' is forbidden\n");
	*/
      
      PutVolAspectRatioInfo( tmpvar, curr_vol );
      *readen_bits += 4;
      if (GetVolAspectRatioInfo(curr_vol)==EXTENDED_PAR) {
	tmpvar = (Int) BitstreamReadBits( stream, 8, "par_width", trace, CODE );
	/*
	if(tmpvar==0)
	  fprintf(stderr, "par_width '0' is forbidden\n");
	  */
	
	PutVolAspectRatioInfoParWidth( tmpvar, curr_vol );
	tmpvar = (Int) BitstreamReadBits( stream, 8, "par_height", trace, CODE );
	/*
	if(tmpvar==0)
	  fprintf(stderr, "par_height '0' is forbidden\n");
	  */
	
	PutVolAspectRatioInfoParHeight( tmpvar, curr_vol );
	*readen_bits += 16;
      }

      /* frame_rate_code (4bit) */
      tmpvar = (Int) BitstreamReadBits(stream,4,"frame_rate_code",trace,CODE );
      *readen_bits +=4;
      switch(tmpvar) {
      case 1:/* 23.98 */
	frame_rate_value = 23.98;
	PutVolTimeIncrementResolution(24,curr_vol);
	break;
      case 2:/* 24.00 */
	frame_rate_value = 24.00;
	PutVolTimeIncrementResolution(24,curr_vol);
	break;
      case 3:/* 25.00 */
	frame_rate_value = 25.00;
	PutVolTimeIncrementResolution(25,curr_vol);
	break;
      case 4:/* 29.97 */
	frame_rate_value = 30.00;
	PutVolTimeIncrementResolution(30,curr_vol);
	break;
      case 5:/* 30.00 */
	frame_rate_value = 29.97;
	PutVolTimeIncrementResolution(30,curr_vol);
	break;
      case 6:/* 50.00 */
	frame_rate_value = 50.00;
	PutVolTimeIncrementResolution(50,curr_vol);
	break;
      case 7:/* 59.94 */
	frame_rate_value = 59.94;
	PutVolTimeIncrementResolution(60,curr_vol);
	break;
      case 8:/* 60.00 */
	frame_rate_value = 60.00;
	PutVolTimeIncrementResolution(60,curr_vol);
	break;
      default:
	fprintf(stderr,"wrong frame_rate_code(VOL): %d\n", tmpvar);
	exit(3);
	break;
      }
      PutVolFixedVopRate(1,curr_vol);

      tmpvar = (Int) BitstreamReadBits( stream, 15, "first_half_bit_rate", trace, CODE );
      PutVolFirstHalfBitRate(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );
      tmpvar = (Int) BitstreamReadBits( stream, 15, "latter_half_bit_rate", trace, CODE );
      PutVolLatterHalfBitRate(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );
      tmpvar = (Int) BitstreamReadBits( stream, 15, "first_half_vbv_buffer_size", trace, CODE );
      PutVolFirstHalfVbvBufferSize(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );   
      tmpvar = (Int) BitstreamReadBits( stream,  3, "last_half_vbv_buffer_size", trace, CODE );
      PutVolLatterHalfVbvBufferSize(tmpvar,curr_vol);
      tmpvar = (Int) BitstreamReadBits( stream, 11, "first_half_vbv_occupancy", trace, CODE );
      PutVolFirstHalfVbvOccupancy(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );
      tmpvar = (Int) BitstreamReadBits( stream, 15, "last_half_vbv_occupancy", trace, CODE );
      PutVolLatterHalfVbvOccupancy(tmpvar,curr_vol);
      BitstreamReadBits( stream,  1, "marker_bit", trace, CODE );  
      *readen_bits += 15 + 1 + 15 + 1 + 15 + 1 + 3 + 11 + 1 + 15 + 1;

      /*
      if(GetVolFirstHalfBitRate(curr_vol)==0 && GetVolLatterHalfBitRate(curr_vol)==0)
	fprintf(stderr, "bit_rate '0' is forbidden\n");
      if(GetVolFirstHalfVbvBufferSize(curr_vol)==0 && GetVolLatterHalfVbvBufferSize(curr_vol)==0)
	fprintf(stderr, "vbv_buffer_size '0' is forbidden\n");
      
      printf("        Bit Rate       : %ld\n", ((LInt)GetVolFirstHalfBitRate(curr_vol)<<15)+GetVolLatterHalfBitRate(curr_vol));
      printf("        VBV Buffer Size: %ld\n", ((LInt)GetVolFirstHalfVbvBufferSize(curr_vol)<<3)+GetVolLatterHalfVbvBufferSize(curr_vol));
      printf("        VBV Occupancy  : %ld\n", ((LInt)GetVolFirstHalfVbvOccupancy(curr_vol)<<15)+GetVolLatterHalfVbvOccupancy(curr_vol));
      */
      
      /* low_delay (1bit) */
      BitstreamReadBits( stream,  1, "low_delay", trace, FLAG/*CODE*/ );
      *readen_bits += 1;

      /*mpeg2_stream*/
      tmpvar = (Int) BitstreamReadBits(stream,1,"mpeg2_stream",trace,CODE );
      PutVolMPEG2Stream_st(tmpvar,curr_vol);
      *readen_bits += 1;
      
      if(GetVolShape(curr_vol)==GREY_SCALE) {
	for(ii=0;ii<GetVolAuxCompCount_st(curr_vol);ii++) {

	  /* alpha_bits_per_pixel (4bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,4,"alpha_bits_per_pixel",trace,NUM);
	  if(tmpvar!=8 && tmpvar!=10 && tmpvar!=12) {
	    fprintf(stderr, "Syntax Error: alpha_bits_per_pixel shall be one of 8/10/12\n");
	    exit(1);
	  }
	  *readen_bits += 4;
	  PutVolGrayBitsPerPixel_st(tmpvar,curr_vol,ii);
	  
	  /* minimum_alpha_level (12bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,12,"minimum_alpha_level",trace,NUM);
	  *readen_bits += 12;
	  PutVolMinimumAlphaLevel_st(tmpvar,curr_vol,ii);

	  /* MoMuSys-2000FDAM1COR-020612_1 */
	  /* marker_bit */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  if ( !tmpvar ) {
	    fprintf ( stderr, "Shall be a marker_bit\n" );
	    exit(-3);
	  }
	  *readen_bits += 1;
	  /**/
	  
	  /* maximum_alpha_level (12bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,12,"maximum_alpha_level",trace,NUM);
	  *readen_bits += 12;
	  PutVolMaximumAlphaLevel_st(tmpvar,curr_vol,ii);

	  if ( GetVolMinimumAlphaLevel_st ( curr_vol, ii ) >= GetVolMaximumAlphaLevel_st ( curr_vol, ii ) ) {
	    fprintf ( stderr, "Syntax Error: minimum_alpha_level=%d, maximum_alpha_level=%d\n",
		     GetVolMinimumAlphaLevel_st ( curr_vol, ii ),  GetVolMaximumAlphaLevel_st ( curr_vol, ii ) );
	    exit ( -1 );
	  }
	}
	
	for(jj=ii;jj<MAX_AUX_COMP_NUM;jj++) {
	  PutVolGrayBitsPerPixel_st(8,curr_vol,jj);
	  PutVolMinimumAlphaLevel_st(0,curr_vol,jj);
	  PutVolMaximumAlphaLevel_st(255,curr_vol,jj);
	}	  
	
	/* composition_method (1bit) */
	BitstreamReadBits(stream,1,"composition_method",trace,CODE);
	*readen_bits += 1;
	
	/* linear_composition (1bit) */
	BitstreamReadBits(stream,1,"linear_composition",trace,CODE);
	*readen_bits += 1;
      }

      if(GetVolShape(curr_vol)!=BINARY_SHAPE_ONLY) {

	/* vol_sprite_usage (1 bits) */
	tmpvar = (Int) BitstreamReadBits(stream,1,"vol_sprite_usage",trace,CODE);
	*readen_bits += 1;

	PutVolSpriteUsage(tmpvar,curr_vol);
	
	if (GetVolSpriteUsage(curr_vol) != SPRITE_NOT_USED)
	  {
	    if( GetVolSpriteUsage(curr_vol) == STATIC_SPRITE)
	      {
		/* sprite_hdim (20 bits) */
		tmpvar = (Int) BitstreamReadBits(stream,20,"vol_sprite_hdim",
						 trace,NUM);
		*readen_bits += 20;
		PutVolSpriteHdim(tmpvar,curr_vol);
		
		BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
		*readen_bits += 1;
		
		/* sprite_vdim (20 bits) */
		tmpvar = (Int) BitstreamReadBits(stream,20,"vol_sprite_vdim",
						 trace,NUM);
		*readen_bits += 20;
		PutVolSpriteVdim(tmpvar,curr_vol);
		
		BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
		*readen_bits += 1;
		
		/* sprite_left_edge (20 bits) */
		tmpvar = (Int) BitstreamReadBits(stream,20,
						 "vol_sprite_left_edge",
						 trace,NUM);
		if(tmpvar%2) {
		  fprintf(stderr, "Syntax Error: sprite_left_coordinate shall be divisible by 2\n");
		  exit(1);
		}
		*readen_bits += 20;
		if (tmpvar>524287) tmpvar -= 1048576;
		PutVolSpriteLeftEdge(tmpvar,curr_vol);
		
		BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
		*readen_bits += 1;
		
		/* sprite_top_edge (20 bits) */
		tmpvar = (Int) BitstreamReadBits(stream,20,"vol_sprite_top_edge",
						 trace,NUM);
		if(tmpvar%2) {
		  fprintf(stderr, "Syntax Error: sprite_top_coordinate shall be divisible by 2\n");
		  exit(1);
		}
		*readen_bits += 20;
		if (tmpvar>524287) tmpvar -= 1048576;
		PutVolSpriteTopEdge(tmpvar,curr_vol);
		
		BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	      *readen_bits += 1;
	      }
	    
	    
	    /* no_of_sprite_points (6 bits) */
	    tmpvar = (Int) BitstreamReadBits(stream,6,"vol_no_of_sprite_points",
					   trace,NUM);
	    *readen_bits += 6;
	    PutVolNoOfSpritePoints(tmpvar,curr_vol);
	    
	    if (GetVolNoOfSpritePoints(curr_vol))
	      {
		tmp_ref_point_coord = (TrajPoint *) emalloc(GetVolNoOfSpritePoints(curr_vol)*sizeof(TrajPoint)); 
		PutVolRefPointCoord(tmp_ref_point_coord,curr_vol);
	      }
	    
	    /* warping accuracy (2 bit) */
	    tmpvar = (Int) BitstreamReadBits(stream,2,
					     "warping_accuracy_in_sprite",
					     trace,NUM);
	    *readen_bits += 2;
	    PutVolWarpingAccuracy(tmpvar,curr_vol);
	    
	    /* brightness_change_in_sprite (1 bit) */
	    tmpvar = (Int) BitstreamReadBits(stream,1,
					     "vol_brightness_change_in_sprite",
					   trace,FLAG);
	    *readen_bits += 1;
	    PutVolBrightnessChangeInSprite(tmpvar,curr_vol);

	    PutVolLowLatencySpriteEnable(0,curr_vol);
	    
	    /* sprite_defocusing (1 bit) */
	    tmpvar = (Int) BitstreamReadBits(stream,1,
					     "sprite_defocusing",
					     trace,FLAG);
	    *readen_bits += 1;
	    PutVolSpriteDefocusing_st(tmpvar, curr_vol);

	    /* lens distortion (1) */
	    tmpvar = (Int) BitstreamReadBits(stream,1,
					     "sprite_lens_distortion",
					     trace,FLAG);
	    *readen_bits += 1;
	    PutVolSpriteLensDistortion_st(tmpvar, curr_vol);
	    
	    res = GetVolSpriteHdim(curr_vol)%16;
	    if (res>0) PutVolSpriteHdim(GetVolSpriteHdim(curr_vol)+
					16-res,curr_vol);    
	    res = GetVolSpriteVdim(curr_vol)%16;
	    if (res>0) PutVolSpriteVdim(GetVolSpriteVdim(curr_vol)+
					16-res,curr_vol);

	    fprintf(stderr, "NOW allocationg SPRITE memory\n");
	    sprite = AllocFrameVop_Studio();
	    AllocVopChannels_Studio(sprite, FRAME_VOP, 0,
				    GetVolSpriteHdim(curr_vol),
				    GetVolSpriteVdim(curr_vol),
				    GetVolChromaFormat_st(curr_vol));
	    fprintf(stderr, "SPRITE CROMA = %d\n", GetVopChromaFormat_st(sprite));
	    PutVopAuxCompCount_st(GetVolAuxCompCount_st(curr_vol),sprite);
	    for(i=0;i<MAX_AUX_COMP_NUM;i++)
	      PutVopGrayBitsPerPixel_st(GetVolGrayBitsPerPixel_st(curr_vol,i), sprite, i);/* 01.04.01: Studio Profile: BUGFIX */
	      /*PutVopGrayBitsPerPixel_st(GetVopGrayBitsPerPixel_st(curr_vol,i), sprite, i);*/
	    /*PutVopShape(GREY_SCALE,display_vop);*/

	    PutVopShape(GetVolShape(curr_vol),sprite);
	    PutVopBitsPerPixel(GetVolBitsPerPixel(curr_vol), sprite);
	    
	    PutVolSprite(sprite,curr_vol);
	    PutVopSpriteUsage(GetVolSpriteUsage(curr_vol), sprite);
	    PutVopShape(GetVolShape(curr_vol),sprite);
	    for(i=0; i<2; i++) {
	      SetConstantImage(GetVopY_st(sprite,i),128); 
	      SetConstantImage(GetVopU_st(sprite,i),0); 
	      SetConstantImage(GetVopV_st(sprite,i),0);
	      for(ii=0;ii<MAX_AUX_COMP_NUM;ii++) {
		SetConstantImage(GetVopA_st(sprite,ii,i),0);
		if(GetVopShape(sprite)==GREY_SCALE)
		  SetConstantImage(GetVopG_st(sprite,ii,i),0);
	      }
	    }
	    PutVolSpriteTransmitMode(PIECE, curr_vol);
	    
	  } /* Sprite_usage != SPRITE_NOT_USED */
      }

      PutVolVolControlParameters(0,curr_vol);      
      PutVolOBMCDisable(1,curr_vol);
      PutVolQuantPrecision(5,curr_vol);

      PutVolScalability(0,curr_vol);
      PutVolDisableGrayQuantUpdate(1,curr_vol);
      PutVolQuantType(1,curr_vol);
      PutVolComplexityEstimationDisable(1, curr_vol);
      PutVolErrorResDisable(1,curr_vol);
      PutVolDataPartEnable(0,curr_vol);

      
      /* Default Quant Matrices */
      /* Intra Quant Matrix */
      PutVolLoadIntraQuantMat(0,curr_vol);
      qmat = GetVolIntraQuantMat(curr_vol);
      for(i=0; i<64; i++)
	qmat[i] = mpeg_iqmat_def[i];
      
      /* Nonintra Quant Matrix */
      PutVolLoadNonintraQuantMat(0,curr_vol);
      qmat = GetVolNonintraQuantMat(curr_vol);
      for(i=0; i<64; i++)
	qmat[i] = mpeg_nqmat_def[i];

      /* Chroma */
      /* Intra Quant Matrix */
      PutVolLoadChromaIntraQuantMat_st(0,curr_vol);
      qmat = GetVolChromaIntraQuantMat_st(curr_vol);
      for(i=0; i<64; i++)
	qmat[i] = mpeg_iqmat_def[i];
      
      /* Nonintra Quant Matrix */
      PutVolLoadChromaNonintraQuantMat_st(0,curr_vol);
      qmat = GetVolChromaNonintraQuantMat_st(curr_vol);
      for(i=0; i<64; i++)
	qmat[i] = mpeg_nqmat_def[i];

      if(GetVolShape(curr_vol) == GREY_SCALE) /* Quant Matrix for the Grayscale shape */
        {
	  for(ii=0;ii<MAX_AUX_COMP_NUM;ii++) {
	    /* Gray Intra Quant Matrix */
	    PutVolLoadGrayIntraQuantMat_st(0,curr_vol,ii);
	    qmat = GetVolGrayIntraQuantMat_st(curr_vol,ii);
	    for(i=0; i<64; i++)
	      qmat[i] = mpeg_giqmat_def[i];
	    /* Gray Nonintra Quant Matrix */
	    PutVolLoadGrayNonintraQuantMat_st(0,curr_vol,ii);    
	    qmat = GetVolGrayNonintraQuantMat_st(curr_vol,ii);
	    for(i=0; i<64; i++)
	      qmat[i] = mpeg_gnqmat_def[i];
	  }/*aux_comp*/
        } /* End of Quant Matrix for the Grayscale shape */

    
      /* next_start_code_studio() */
      *readen_bits += (Int) BitstreamNextStartCode_Studio(stream, trace, CODE);

      /* extension_and_user_data(2) */
      *readen_bits += (Int) DecodeExtensionAndUserData_Studio(2, curr_vol, NULL, stream, trace);

      /*
      tmpvar = (Int) BitstreamShowBits(stream,USER_DATA_START_CODE_LENGTH);
      if (tmpvar == USER_DATA_START_CODE) 
        *readen_bits += GetUserDataFromBitstream(stream,trace,
						 vol_user_data_filename);
      */
						 
      if( GetVolSpriteUsage(curr_vol) == STATIC_SPRITE )
  	{
	  Int *qmat_vol,*qmat_vop;
	  
	  
	  PutVolHeaderToVop(curr_vol, sprite);
	  PutVopWidth_st(GetVolSpriteHdim(curr_vol),sprite,0);
	  PutVopHeight_st(GetVolSpriteVdim(curr_vol),sprite,0);			
	  PutVopRGBComponents_st(GetVolRGBComponents_st(curr_vol),sprite);
	  PutVopChromaFormat_st(GetVolChromaFormat_st(curr_vol),sprite);
	  PutVopMPEG2Stream_st(GetVolMPEG2Stream_st(curr_vol),sprite);
	  
	  /* Chroma Mat */
	  PutVopLoadChromaIntraQuantMat_st(GetVolLoadChromaIntraQuantMat_st(curr_vol),sprite);
	  qmat_vol = GetVolChromaIntraQuantMat_st(curr_vol);
	  qmat_vop = GetVopChromaIntraQuantMat_st(sprite);
	  for(i=0; i<64; i++)
	    qmat_vop[i] = qmat_vol[i];
	  PutVopLoadChromaNonintraQuantMat_st(GetVolLoadChromaNonintraQuantMat_st(curr_vol),sprite);
	  qmat_vol = GetVolChromaNonintraQuantMat_st(curr_vol);
	  qmat_vop = GetVopChromaNonintraQuantMat_st(sprite);      
	  for(i=0; i<64; i++)
	    qmat_vop[i] = qmat_vol[i];
	  
	  PutVopAuxCompCount_st(GetVolAuxCompCount_st(curr_vol),sprite);
	  for(ii=0;ii<MAX_AUX_COMP_NUM;ii++) {
	    PutVopGrayBitsPerPixel_st(GetVolGrayBitsPerPixel_st(curr_vol,ii),sprite,ii);
	    PutVopMinimumAlphaLevel_st(GetVolMinimumAlphaLevel_st(curr_vol,ii),sprite,ii);
	    PutVopMaximumAlphaLevel_st(GetVolMaximumAlphaLevel_st(curr_vol,ii),sprite,ii);
	    /*gray quant mat*/
	    PutVopLoadGrayIntraQuantMat_st(GetVolLoadGrayIntraQuantMat_st(curr_vol,ii),sprite,ii);
	    qmat_vol = GetVolGrayIntraQuantMat_st(curr_vol,ii);
	    qmat_vop = GetVopGrayIntraQuantMat_st(sprite,ii);
	    for(i=0; i<64; i++)
	      qmat_vop[i] = qmat_vol[i];
	    PutVopLoadGrayNonintraQuantMat_st(GetVolLoadGrayNonintraQuantMat_st(curr_vol,ii),sprite,ii);
	    qmat_vol = GetVolGrayNonintraQuantMat_st(curr_vol,ii);
	    qmat_vop = GetVopGrayNonintraQuantMat_st(sprite,ii);      
	    for(i=0; i<64; i++)
	      qmat_vop[i] = qmat_vol[i];
	  }
	  PutVopSpriteDefocusing_st(GetVolSpriteDefocusing_st(curr_vol),sprite);
	  PutVopSpriteLensDistortion_st(GetVolSpriteLensDistortion_st(curr_vol),sprite);

	  if (GetVolLowLatencySpriteEnable(curr_vol)==0)
	    {
	      DecodeBasicSprite_Studio(stream, curr_vol, vo_id, trace, readen_bits,
				       mot_x_P,mot_y_P,MB_decisions_P); 		
	      PutVolSpriteTransmitMode(STOP, curr_vol);
	    }
	} /* CASE STATIC_SPRITE */
      
    } else {
      fprintf(stderr, "Syntax Error: No video_object_start_code\n");
      exit(1);
    }

  return (*readen_bits - prev_total_bits);
}

/***********************************************************CommentBegin******
 *
 * -- DecodeVolList_Studio -- Decode a list of video object layers for one VO 
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode a list of layers for a specific video object from given
 *	bitstreams. The bitstream for each layer is given in a separate file
 *
 * 
 * Arguments in : 	
 *	Int number_of_vols	number of layers for the video object
 *	Bitstream *stream[]	 Pointer list to the bitstreams
 *	Trace *trace[]		 Trace information for each layer
 *	Int next_display_time	next time a frame is displayed
 *  Int random_access_start_time
 *
 *
 * Arguments in/out :	
 *	Vol *vol_list	         concatenated list of layers for video object
 *	Int readen_bits[]	 Readen bits for each layer
 *	Int stop_decoding_vol[]  Flag to indicate end of all bitstreams
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 07.07.97: A. Sandvand: Added support for Random Access
 *            11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *            01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *	
 *
 ***********************************************************CommentEnd********/
/*In this function, Nothing are changed from the original one.*/
Void DecodeVolList_Studio(Int number_of_vols, Bitstream *stream[], Trace *trace[], 
                   Float next_display_time, Int random_access_start_time,
                   Vol *vol_list, Int readen_bits[], 
                   Int stop_decoding_vol[], Int vo_id, Int post_filter_type,
                   Char gov_user_data_filename[][300], Int short_video_header,
                   ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                   ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                   Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])   

{
  Vol *next_vol;
  Int k;

  /*****
   *
   *	Decode each layer for the current object until the time is
   *	come to display the next frame
   *
   *****/
  next_vol = vol_list;
  
  for(k=0; k<number_of_vols; k++)
    {
      if(stop_decoding_vol[k] == 0)
        DecodeVol_Studio(stream,trace[k],next_display_time,random_access_start_time,
                  next_vol,&readen_bits[k],
                  &stop_decoding_vol[k],vo_id,vol_list,post_filter_type,
                  gov_user_data_filename[k],short_video_header,
                  mot_x_P,mot_y_P,MB_decisions_P                  
                  );
#if 0
      if(temporal_scalability)
	  WriteOutputImage_TPS_for_base(base_output_vop,
					next_display_time,
					post_filter_type,
					!GetVolScalability(next_vol),
					stop_decoding_vol,
					number_of_vols);
#endif 0
      
      next_vol = GetVolNext(next_vol);
    }
}

/***********************************************************CommentBegin******
 *
 * -- DecodeVol_Studio -- Decode a video object layer
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	08-AUG-1996
 *
 * Purpose :
 *	Decode a video object layer
 *
 * 
 * Arguments in : 	
 *	Bitstream *stream	Pointer to the bitstream
 *	Trace *trace		Trace information for VOL
 *	Int next_display_time	next time a frame is displayed
 *      Int random_access_start_time
 *
 *
 * Arguments in/out :	
 *	Vol *next_vol		video object layer to be decoded
 *	Int *readen_bits	Readen bits for VOL
 *	Int *stop_decoding_vol	Flag to indicate end of all bitstreams
 *      Int short_video_header  Flag to signal short video header bitstreams (H.263)
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *  	14.01.97 Luis Ducla-Soares: added byte alignment for VOP_START_CODE 
 *	                   (call to function BitstreamByteAlign() )
 *
 *	10.02.97 M.Wollborn: changed code in order to avoid problems with
 *		             different VO and display frame rates
 *	09.05.97 Minhua Zhou: added "DecodeGOV"
 *      07.07.97: A. Sandvand: Added support for Random Access
 *      24.07.97 Minhua Zhou:  Modified decoding control due to  B-VOPs 
 *      15.9.97 A. Sandvand: Updated Random Access implementation
 *      25.03.98 M.Wollborn: Modifications due to N2171 Cl. 2.1.9 user data
 *      23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/
Void DecodeVol_Studio(Bitstream **stream, Trace *trace, Float next_display_time_in, 
               Int random_access_start_time,
               Vol *curr_vol, Int *readen_bits, Int *stop_decoding_vol,
               Int vo_id, Vol *vol_list, Int post_filter_type, 
               Char gov_user_data_filename[], Int short_video_header,
               ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])  
{
  Int tmpvar,vop_display_time;
  Int prev_vop_display_time;
  Int next_display_time;
  Vop *curr_vop;
  Int vol_id, nvol_id;
  Int enh_decoding_flag = 0;

  Int ii;
  Int i,*qmat_vol,*qmat_vop;
  
  nvol_id = GetVolId( curr_vol );
  vol_id  = bs_vol[vo_id][nvol_id];

  /*****
   *
   *	Decode all VOPs from the current VOL until the next frame
   *	has to be displayed
   *
   *****/
  curr_vop = GetVolVop(curr_vol);
  /*Copy Vol Syntax to Vop*/
  /*need checking*/
  PutVolHeaderToVop(curr_vol,curr_vop);/*NotChanged*/
  PutVopWidth(0,curr_vop);
  PutVopHeight(0,curr_vop);
  PutVopRGBComponents_st(GetVolRGBComponents_st(curr_vol),curr_vop);
  PutVopChromaFormat_st(GetVolChromaFormat_st(curr_vol),curr_vop);
  PutVopMPEG2Stream_st(GetVolMPEG2Stream_st(curr_vol),curr_vop);

  /* Chroma Mat */
  PutVopLoadChromaIntraQuantMat_st(GetVolLoadChromaIntraQuantMat_st(curr_vol),curr_vop);
  qmat_vol = GetVolChromaIntraQuantMat_st(curr_vol);
  qmat_vop = GetVopChromaIntraQuantMat_st(curr_vop);
  for(i=0; i<64; i++)
    qmat_vop[i] = qmat_vol[i];
  PutVopLoadChromaNonintraQuantMat_st(GetVolLoadChromaNonintraQuantMat_st(curr_vol),curr_vop);
  qmat_vol = GetVolChromaNonintraQuantMat_st(curr_vol);
  qmat_vop = GetVopChromaNonintraQuantMat_st(curr_vop);      
  for(i=0; i<64; i++)
    qmat_vop[i] = qmat_vol[i];
  
  /*AM2PDAM*/
  PutVopAuxCompCount_st(GetVolAuxCompCount_st(curr_vol),curr_vop);
  for(ii=0;ii<MAX_AUX_COMP_NUM;ii++) {
    PutVopGrayBitsPerPixel_st(GetVolGrayBitsPerPixel_st(curr_vol,ii),curr_vop,ii);
    PutVopMinimumAlphaLevel_st(GetVolMinimumAlphaLevel_st(curr_vol,ii),curr_vop,ii);
    PutVopMaximumAlphaLevel_st(GetVolMaximumAlphaLevel_st(curr_vol,ii),curr_vop,ii);
    /*gray quant mat*/
    PutVopLoadGrayIntraQuantMat_st(GetVolLoadGrayIntraQuantMat_st(curr_vol,ii),curr_vop,ii);
    qmat_vol = GetVolGrayIntraQuantMat_st(curr_vol,ii);
    qmat_vop = GetVopGrayIntraQuantMat_st(curr_vop,ii);
    for(i=0; i<64; i++)
      qmat_vop[i] = qmat_vol[i];
    PutVopLoadGrayNonintraQuantMat_st(GetVolLoadGrayNonintraQuantMat_st(curr_vol,ii),curr_vop,ii);
    qmat_vol = GetVolGrayNonintraQuantMat_st(curr_vol,ii);
    qmat_vop = GetVopGrayNonintraQuantMat_st(curr_vop,ii);      
    for(i=0; i<64; i++)
      qmat_vop[i] = qmat_vol[i];
  }
  /*000705FPDAM*/
  PutVopSpriteDefocusing_st(GetVolSpriteDefocusing_st(curr_vol),curr_vop);
  PutVopSpriteLensDistortion_st(GetVolSpriteLensDistortion_st(curr_vol),curr_vop);
  
  /* Studio Profile */
  /* Here, Gray matrix should be inserted. */
  
  next_display_time=(Int)(0.001+next_display_time_in/1000.0
                     *(Double)GetVopTimeIncrementResolution(curr_vop));
  if(next_display_time_in == random_access_start_time)
    vop_display_time = prev_vop_display_time=-1;
  else 
    {
      vop_display_time = CalcVopDisplayTime(curr_vop);
      prev_vop_display_time=CalcVopDisplayTime(GetVopPrevTemp(curr_vop));
    }
          
  while (DecodeTimeControl(stream[vol_id],
			   vop_display_time,
			   prev_vop_display_time,
			   next_display_time,
			   *stop_decoding_vol,
			   curr_vop,
			   random_access_start_time,
			   next_display_time_in,
			   &enh_decoding_flag,
			   trace))
    {
      printf("display_time=%d,prev_time=%d,vop_time=%d\n",next_display_time,
	     prev_vop_display_time,vop_display_time);

      /*need checking*/
      PutVopWidth_st (GetVolWidth (curr_vol), curr_vop, 0);
      PutVopHeight_st(GetVolHeight(curr_vol), curr_vop, 0);
      PutVopWidth_st (GetVolWidth (curr_vol), curr_vop, 1);
      PutVopHeight_st(GetVolHeight(curr_vol), curr_vop, 1);
      
	  if (short_video_header) 
		/* If decoding a short_video_header bitstream, DecodeVop() is to be called next */
		tmpvar = VOP_START_CODE;
	  else
      tmpvar = (Int) BitstreamShowBits(stream[ vol_id ],VOP_START_CODE_LENGTH);

      /* 01.04.01: Studio Profile: BUGFIX */
      if ( tmpvar == GROUP_START_CODE || tmpvar == VOP_START_CODE ) {
      /* 01.04.01: Studio Profile: BUGFIX */	
      
      if (tmpvar == GROUP_START_CODE) {
	fprintf(stderr, "Start Decoding Studio GOV header.\n");
        *readen_bits += DecodeGOV_Studio ( stream[ vol_id ], trace );
      }

      /* 01.04.01: Studio Profile: BUGFIX
      else if(tmpvar == VOP_START_CODE)
      */
	{
      
	  Int counter = -1;
	  do {  
	    counter++;
	    
	    PutVopVopCounter_st(counter,curr_vop); /* FrameVop:0, FieldVop:0or1 */
	    /* vop_counter is set only here and used in the following function */
	    DecodeSingleVop_Studio(stream,     vo_id,      trace,
				   curr_vop,   readen_bits,
				   vol_list,   next_display_time,
				   post_filter_type, short_video_header, stop_decoding_vol,
				   mot_x_P,mot_y_P,MB_decisions_P);
	    
	  } while (GetVopVopStructure_st(curr_vop) == FIELD_VOP && counter == 0);

	  if (GetVolSpriteUsage(curr_vol) == STATIC_SPRITE) 
	    PutVolSpriteTransmitMode(GetVopSpriteTransmitMode(curr_vop),
				     curr_vol);
	  
	  if(temporal_scalability && CheckBaseVop(curr_vop))
	    BASEImageInput(curr_vop, base_output_vop); /* Input base's Image to buffer */
	  
	  vop_display_time      = CalcVopDisplayTime(curr_vop);
	  prev_vop_display_time = CalcVopDisplayTime(GetVopPrevTemp(curr_vop));
	  
	}

      curr_vop->temporal_reference_st ++;

    /* 01.04.01: Studio Profile: BUGFIX */
    }
    /* 01.04.01: Studio Profile: BUGFIX */
      
      else
	*stop_decoding_vol = 1;
    }

  /* Update the quant matrices in curr_vol from curr_vop */
  PutVolLoadIntraQuantMat(GetVopLoadIntraQuantMat(curr_vop),curr_vol);
  qmat_vol = GetVolIntraQuantMat(curr_vol);
  qmat_vop = GetVopIntraQuantMat(curr_vop);
  for(i=0; i<64; i++)
    qmat_vol[i] = qmat_vop[i];
  PutVolLoadNonintraQuantMat(GetVopLoadNonintraQuantMat(curr_vop),curr_vol);
  qmat_vol = GetVolNonintraQuantMat(curr_vol);
  qmat_vop = GetVopNonintraQuantMat(curr_vop);
  for(i=0; i<64; i++)
    qmat_vol[i] = qmat_vop[i];
  PutVolLoadChromaIntraQuantMat_st(GetVopLoadChromaIntraQuantMat_st(curr_vop),curr_vol);
  qmat_vol = GetVolChromaIntraQuantMat_st(curr_vol);
  qmat_vop = GetVopChromaIntraQuantMat_st(curr_vop);
  for(i=0; i<64; i++)
    qmat_vol[i] = qmat_vop[i];
  PutVolLoadChromaNonintraQuantMat_st(GetVopLoadChromaNonintraQuantMat_st(curr_vop),curr_vol);
  qmat_vol = GetVolChromaNonintraQuantMat_st(curr_vol);
  qmat_vop = GetVopChromaNonintraQuantMat_st(curr_vop);
  for(i=0; i<64; i++)
    qmat_vol[i] = qmat_vop[i];

  for(ii=0;ii<MAX_AUX_COMP_NUM;ii++) {
    PutVolLoadGrayIntraQuantMat_st(GetVopLoadGrayIntraQuantMat_st(curr_vop,ii),curr_vol,ii);
    qmat_vol = GetVolGrayIntraQuantMat_st(curr_vol,ii);
    qmat_vop = GetVopGrayIntraQuantMat_st(curr_vop,ii);
    for(i=0; i<64; i++)
      qmat_vol[i] = qmat_vop[i];
    PutVolLoadGrayNonintraQuantMat_st(GetVopLoadGrayNonintraQuantMat_st(curr_vop,ii),curr_vol,ii);
    qmat_vol = GetVolGrayNonintraQuantMat_st(curr_vol,ii);
    qmat_vop = GetVopGrayNonintraQuantMat_st(curr_vop,ii);      
    for(i=0; i<64; i++)
      qmat_vol[i] = qmat_vop[i];
  }
  
  return;
}

/***********************************************************CommentBegin******
 *
 * -- DecodeGOV_Studio -- Decodes the Group of Studio VOPs from bitstream
 *
 * Author :		
 *	Osamu Sunohara (SONY)
 *
 * Created :		
 *	01.04.01
 *
 * Purpose :		
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *	Int *readen_bits	pointer to value of up to now readen_bits
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *
 ***********************************************************CommentEnd********/


Int DecodeGOV_Studio ( Bitstream *stream, Trace *trace )
{
  Int tmpvar;
  Int bits = 0;
  Int closed_gov, broken_link;
  
  /* group_vop_start_code(32) */
  tmpvar = (Int) BitstreamReadBits( stream, 32, "group_vop_start_code", trace, CODE );
  if( tmpvar != GROUP_START_CODE ) {
    printf ( "\nBitstream does not start with GROUP_START_CODE\n" );
    exit ( 1 );
  }
  bits += 32;

  /* time_code_SMPTE12M (64bits: dammy) */
  /* MoMuSys-2000FDAM1COR-020612_1 */
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_1",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_2",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_3",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_4",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream, 4,"reserved_bits",trace,FLAG);
  bits += 72;
  /**/
/*
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_1",trace,CODE);
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_2",trace,CODE);
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_3",trace,CODE);
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_4",trace,CODE);
  bits += 64;
*/

  closed_gov = (Int) BitstreamReadBits ( stream , 1, "closed_gov", trace, NUM);
  broken_link = (Int) BitstreamReadBits ( stream, 1, "broken_link", trace, NUM);
  bits += 2;

  bits += (Int) BitstreamNextStartCode_Studio( stream, trace, CODE );

  bits += (Int) DecodeExtensionAndUserData_Studio(3, NULL, NULL, stream, trace);
} 

/***********************************************************CommentBegin******
 *
 * -- DecodeVopHeader_Studio -- Decodes the VOPheader information from the bitstream
 *
 * Author :		
 *	Cecile Dufour (Philips LEP/ ACTS MoMuSys)
 *
 * Created :		
 *	14-Nov-1997
 *
 * Purpose :		
 *	 Decode the vop headerinformation from the bitstream.
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *	Int *readen_bits	pointer to value of up to now readen_bits
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	This routine has been created by the splitting of the previous routine 
 *	DecodeVop: this previous routine contained both the reading of the header
 *	for each vop, and the call for further decoding.
 *	DecodeVopHeader has be created becasue it is called in the routine
 *	DecodeBasicSprite used in the STATIC SPRITE case
 *
 *      28.03.98 M.Wollborn: Modifications due to N2171 Cl. 2.5.14
 *      21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 *	08.06.98 Marc Mongenet (EPFL): added Complexity Estimation syntax support
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/
   
Int DecodeVopHeader_Studio(Bitstream *stream, Int vo_id, Trace *trace, Vop *curr_vop, 
			   Int *readen_bits,
			   ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
			   ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
			   Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])    

{
  Vop *tmp_vop;
  Int tmpvar;
  Int i,bits;
  Int time_base0;
  Int time_base=0; 
  TrajPoint	*traj=NULL, *difftraj=NULL;
  Float	brightness_factor;
  /*  Int res;  :unused SB */
  TrajPoint	*tmp_refpoint=NULL;
  
#ifdef _DEBUG_SPSC_
  static Int count;
  Char name[81];
#endif
  
  Int prev_prediction_mode = -1;
  static Int prev_time_base=0;

  Int vop_structure, fip;
  Int tmp_prediction_type;
  Int tmp_vop_structure;
  Int tmp_top_field_first;  

  Int prev_total_bits = *readen_bits;

  Int ii, jj;

  /*****
   *
   *    Disable tracing for time preview 
   *
   *****/
  trace->trace = 0;
  
  /*****
   *
   *    Set trace level for VOP header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VOP_header_def;

  TraceBreakLine(trace);

  /*****
   *
   *	Free the forelast VOP and copy the current (which is now the previous
   *	VOP) into the right place of the structure
   *
   *****/

  /*****
   *
   *	Read the VOP header from the bitstream
   *
   *****/
  /* vop_start_code (VOP_START_CODE_LENGTH bits) */
  tmpvar = (Int) BitstreamReadBits(stream,VOP_START_CODE_LENGTH,"vop_start_code",trace,CODE);
  *readen_bits += VOP_START_CODE_LENGTH;
  if(tmpvar != VOP_START_CODE)
    {
    printf ("Bitstream does not start with VOP_START_CODE\n");
    exit (1);
   }

  prev_prediction_mode = GetVopPredictionType(curr_vop);/*never used*/
 
  /* time_code_SMPTE12M (64bits: dammy) */
  /* MoMuSys-2000FDAM1COR-020612_1 */
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_1",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_2",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_3",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_4",trace,CODE);
  tmpvar = (Int)BitstreamReadBits(stream, 1,"marker_bit",trace,FLAG);
  if ( !tmpvar ) {
    fprintf(stderr, "shall be marker_bit (DecodeGOV_Studio)\n");
    exit(-3);
  }
  BitstreamReadBits(stream, 4,"reserved_bits",trace,FLAG);
  readen_bits += 72;
  /**/
/*
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_1",trace,CODE);
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_2",trace,CODE);
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_3",trace,CODE);
  BitstreamReadBits(stream,16,"time_code_SMPTE12M_4",trace,CODE);
  *readen_bits += 64;
*/
  
  /* temporal_reference (10bits) */
  tmpvar = (Int) BitstreamReadBits(stream,10,"temporal_reference",trace,CODE);
  *readen_bits += 10;
  /* Put mod_time_base & time_increment from temporal_reference(0-1023 remeted!! and NO GOV!!) */
#if 1 /* 01.04.01: Studio Profile: BUGFIX */
  PutVopModTimeBase( curr_vop->temporal_reference_st/GetVopTimeIncrementResolution(curr_vop),curr_vop);
  PutVopTimeInc( curr_vop->temporal_reference_st%GetVopTimeIncrementResolution(curr_vop),curr_vop);
#elif
  PutVopModTimeBase( tmpvar/GetVopTimeIncrementResolution(curr_vop),curr_vop);
  PutVopTimeInc( tmpvar%GetVopTimeIncrementResolution(curr_vop),curr_vop);
#endif
  
  /* vop_structure (2bits) */
  tmpvar = (Int) BitstreamReadBits(stream,2,"vop_structure",trace,CODE);
  *readen_bits += 2;
  if(tmpvar==0) {
    fprintf(stderr,"reserved bits(vop_structure::VopHeader)\n");
    exit(3);
  }
  tmp_vop_structure = tmpvar;

  /* vop_prediction_type (2 bits) */
  tmpvar = (Int) BitstreamReadBits(stream,2,"vop_prediction_type",trace,CODE);
  *readen_bits += 2;

  tmp_prediction_type = tmpvar;

  if(tmp_vop_structure==1||tmp_vop_structure==2) {
    vop_structure = FIELD_VOP;
    if(tmp_vop_structure==1) {
      if(GetVopVopCounter_st(curr_vop)==0) {
	tmp_top_field_first = 1;
      } else {
	tmp_top_field_first = 0;
      }
      fip = 0;
    } else {
      if(GetVopVopCounter_st(curr_vop)==0) {
	tmp_top_field_first = 0;
      } else {
	tmp_top_field_first = 1;
      }
      fip = 1;
    }
  } else {/*FRAME_VOP*/
    vop_structure = FRAME_VOP;
    fip = 0;
  }
  PutVopVopStructure_st(vop_structure,curr_vop);
  PutVopPredictionType_st(tmp_prediction_type,curr_vop,fip);

  tmpvar = (Int) BitstreamReadBits(stream,1,"vop_coded",trace,NUM);

  if (tmpvar==0) 
    {
      /* 01.04.01: Studio Profile: BUGFIX */
      PutVopWidth_st(0,curr_vop,fip);
      PutVopHeight_st(0,curr_vop,fip);
      PutVopCoded_st(0,curr_vop,fip);
      /* search next_start_code */
      *readen_bits += (Int) BitstreamNextStartCode_Studio(stream, trace, CODE);      
      /* extension_and_user_data(4) */
      *readen_bits += (Int) DecodeExtensionAndUserData_Studio(4, NULL, curr_vop, stream, trace);

      printf("Empty Studio Vop decoded\n");

      
      if((GetVopVopStructure_st(curr_vop) == FRAME_VOP && GetVopVopCounter_st(curr_vop) == 0) ||
	 (GetVopVopStructure_st(curr_vop) == FIELD_VOP && GetVopVopCounter_st(curr_vop) == 1)) {
      
      if (GetVopSpriteUsage(curr_vop)!= STATIC_SPRITE||
	  GetVopPredictionType_st(curr_vop,fip)==SPRITE_VOP)
        if (GetVopPredictionType_st(curr_vop,fip)!=B_VOP) 
	  {
	    if (GetVopPrevTemp(curr_vop)!=NULL)
	      FreeVop_Studio(GetVopPrevTemp(curr_vop));
	    PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
	    PutVopNextTemp(NULL,curr_vop);

	    if (GetVopWidth_st(GetVopPrevTemp(curr_vop),0)!=0) 
	      {
		tmp_vop = CloneVop_Studio(GetVopPrevTemp(curr_vop),
					  GetVopVopStructure_st(GetVopPrevTemp(curr_vop)),0,0);
		if(GetVopVopStructure_st(tmp_vop)==FIELD_VOP) {
		  AllocVopChannels_Studio(tmp_vop,FIELD_VOP,1,
					  GetVopWidth_st(GetVopPrevTemp(curr_vop),1),
					  GetVopHeight_st(GetVopPrevTemp(curr_vop),1),
					  GetVopChromaFormat_st(tmp_vop));
		  CopyVop_Studio(GetVopPrevTemp(curr_vop),FIELD_VOP,1,tmp_vop,1);
		}
		  
		PutVopNextTemp(tmp_vop,curr_vop);
		PutVopModTimeBase(GetVopModTimeBase(curr_vop),GetVopNextTemp(curr_vop));
		PutVopTimeInc(GetVopTimeInc(curr_vop),GetVopNextTemp(curr_vop));
		PutVopCoded_st(GetVopCoded_st(curr_vop,0),GetVopNextTemp(curr_vop),0);
		PutVopCoded_st(GetVopCoded_st(curr_vop,1),GetVopNextTemp(curr_vop),1);
		
	      } 
	    else 
	      {
		tmp_vop = SallocVop(); 
		CopyVopNonImageField_Studio(curr_vop,GetVopVopStructure_st(curr_vop),0,tmp_vop,0);
		if(GetVopVopStructure_st(curr_vop)==FIELD_VOP)
		  CopyVopNonImageField_Studio(curr_vop,FIELD_VOP,1,tmp_vop,1);
		  
		PutVopNextTemp(tmp_vop,curr_vop);
	      }
	  }

    }/*frame/field*/

      
      return(0);
    }

  /* BugFix_011203 */
  PutVopCoded_st(1,curr_vop,fip);
  /*PutVopCoded(1,curr_vop);*/
  /* BugFix_011203 */

  /* set default value for shape_coding_type */
  if (GetVopPredictionType_st(curr_vop,fip) == I_VOP) 		
    PutVopShapeCodingType_st(0,curr_vop,fip);
  else 
    PutVopShapeCodingType_st(1,curr_vop,fip);

  PutVopRoundingType(0,curr_vop);
  
  if(GetVopShape(curr_vop) != RECTANGULAR)
    {
      if (!(GetVopSpriteUsage(curr_vop)==STATIC_SPRITE&&GetVopPredictionType_st(curr_vop,fip)==I_VOP))
	{
	  /* vop_binary_shape_coding_type (1bit) */

	  /* vop_width (14 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,14,"vop_width",trace,NUM);
	  *readen_bits += 14;
	  PutVopWidth_st(tmpvar,curr_vop,fip);
	  
	  /* marker_bit (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  *readen_bits += 1;
	  
	  /* vop_height (14 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,14,"vop_height",trace,NUM);
	  *readen_bits += 14;
	  PutVopHeight_st(tmpvar,curr_vop,fip);
	  
	  /* marker_bit (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  *readen_bits += 1;
	  
	  /* vop_horizontal_mc_spatial_ref (14 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,14,"vop_horizontal_mc_spatial_ref",
					   trace,NUM);
	  *readen_bits += 14;
	  
	  /* Check vhmsr */
	  if(GetVopChromaFormat_st(curr_vop)!=CHROMA444 && tmpvar%2) {
	    fprintf( stderr, "vop_horizontal_mc_spatial_ref is not divisible by 2!" );
	    exit(-1);
	  }
	  
	  /* Revised by C. Dufour 12/12/97 for low-latency*/
	  if (GetVopSpriteUsage(curr_vop)== STATIC_SPRITE &&
	      GetVopPredictionType_st(curr_vop,fip)==I_VOP)
	    {
	      if (tmpvar>4095) tmpvar -= 8192;
	      tmpvar -= GetVopSpriteLeftEdge(curr_vop);
	    }
	  /* End Revision */
	  
	  PutVopHorSpatRef_st(tmpvar,curr_vop,fip);
	  
	  /* marker_bit (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  *readen_bits += 1;
	  
	  /* vop_vertical_mc_spatial_ref (14 bits) */
	  tmpvar = (Int) BitstreamReadBits(stream,14,"vop_vertical_mc_spatial_ref",
					   trace,NUM);
	  *readen_bits += 14;
	  
	  /* Check vvmsr */
	  if (GetVopChromaFormat_st(curr_vop)==CHROMA420 &&
	      GetVopInterlaced(curr_vop) &&
	      vop_structure==FRAME_VOP &&
	      tmpvar%4) {
	    fprintf( stderr, "vop_vertical_mc_spatial_ref is not divisible by 4!" );
	    exit(-1);
	  } else if (((GetVopChromaFormat_st(curr_vop)!=CHROMA420 &&
		       GetVopInterlaced(curr_vop) &&
		       vop_structure==FRAME_VOP) ||
		      (GetVopChromaFormat_st(curr_vop)==CHROMA420 &&
		       ((GetVopInterlaced(curr_vop)&&vop_structure==FIELD_VOP) ||
			(GetVopInterlaced(curr_vop)==0)))) &&
		     tmpvar%2) {
	    fprintf( stderr, "vop_vertical_mc_spatial_ref is not divisible by 2!" );
	    exit(-1);
	  }
	  
	  /* Revised by C. Dufour 12/12/97 for low-latency*/
	  if (GetVopSpriteUsage(curr_vop)== STATIC_SPRITE &&
	      GetVopPredictionType_st(curr_vop,fip)==I_VOP)
	    {
	      if (tmpvar>4095) tmpvar -= 8192;
	      tmpvar -= GetVopSpriteTopEdge(curr_vop);
	    }
	  /* End Revision */
	  
	  PutVopVerSpatRef_st(tmpvar,curr_vop,fip);

	  /* MoMuSys-2000FDAM1COR-020612_1 */
	  /* marker_bit */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
	  if ( !tmpvar ) {
	    fprintf ( stderr, "Shall be a marker_bit\n" );
	    exit(-3);
	  }
	  *readen_bits += 1;
	  /**/
	}
      else
	{
	  PutVopWidth(GetVopSpriteHdim(curr_vop),curr_vop);
	  PutVopHeight(GetVopSpriteVdim(curr_vop),curr_vop);
	  PutVopHorSpatRef(0,curr_vop);
	  PutVopVerSpatRef(0,curr_vop);
	}/* of STATIC_SPRITE and first I_VOP */
      
      if(GetVopScalability(curr_vop)==1 && GetVopEnhanceType(curr_vop)==1)
	{
	  /* background_composition (1 bit) */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"background_composition",
					   trace,FLAG);
	  *readen_bits += 1;
	  PutVopBackComp(tmpvar,curr_vop);
	}
      else 
	PutVopBackComp(0,curr_vop);

      PutVopConstantAlpha(0,curr_vop);
      
    }
  else {
    /*For Rectangular VOP, vop size information is stored in curr_vol*/
    PutVopWidth_st(((GetVopWidth_st(curr_vop,fip)+15)/16)*16, curr_vop, fip);
    if(GetVopInterlaced(curr_vop)==0) /* progressive_sequence */
      PutVopHeight_st(((GetVopHeight_st(curr_vop,fip)+15)/16)*16, curr_vop, fip);
    else { /* interlaced_sequence */
      if(vop_structure==FRAME_VOP)
	PutVopHeight_st(((GetVopHeight_st(curr_vop,fip)+31)/32)*32, curr_vop, fip);
      else
	PutVopHeight_st(((GetVopHeight_st(curr_vop,fip)+31)/32)*16, curr_vop, fip);
    }
  }
  
  /* Complexity Estimation syntax support - Marc Mongenet (EPFL) - 8 Jun 1998 */
  if (!GetVopComplexityEstimationDisable(curr_vop))
    readen_bits += BitstreamReadDCECS(stream, curr_vop, trace);

  /* top_field_first */
  tmpvar = (Int) BitstreamReadBits(stream,1,"top_field_first",trace,CODE);
  *readen_bits += 1;
  if(vop_structure == FIELD_VOP && tmpvar != 0) {
    fprintf(stderr, "Syntax Error: top_field_first shall be 0 for FIELD_VOP.\n");
  } else if(vop_structure == FRAME_VOP)
    tmp_top_field_first = tmpvar;
  PutVopTopFieldFirst(tmp_top_field_first, curr_vop);

  /* repeat_first_field */
  tmpvar = (Int) BitstreamReadBits(stream,1,"repeat_first_field",trace,CODE);
  *readen_bits += 1;
  PutVopRepeatFirstField_st(tmpvar,curr_vop);
  if ( vop_structure == FIELD_VOP && tmpvar != 0 ) {
    fprintf(stderr, "Syntax Error: repeat_first_flag shall be 0 for FIELD_VOP.\n");
  }

  /* progressive_frame */
  BitstreamReadBits(stream,1,"progressive_frame",trace,CODE);
  *readen_bits += 1;

  /*intra_predictors_reset*/
  if(GetVopPredictionType_st(curr_vop,fip)==I_VOP) {
    tmpvar = (Int) BitstreamReadBits(stream,1,"intra_predictors_reset",trace,CODE);
    *readen_bits += 1;
    PutVopIntraPredictorsReset_st(tmpvar,curr_vop,fip);
  }
  
  
  if (GetVopShape(curr_vop) != BINARY_SHAPE_ONLY)  /* BSO_NOEL */
    {
      /* alternate_scan */
      tmpvar = (Int) BitstreamReadBits(stream,1,"alternate_scan",trace,CODE);
      *readen_bits += 1;
      PutVopAlternateScan_st(tmpvar,curr_vop,fip);
	
      tmpvar = (Int) BitstreamReadBits(stream,1,"frame_pred_frame_dct",trace,CODE);
      *readen_bits += 1;
      PutVopFramePredFrameDCT_st(tmpvar,curr_vop);
      /*
      if(GetVopInterlaced(curr_vop)==0 && GetVopFramePredFrameDCT_st(curr_vop)!=1) {
	fprintf(stderr, "frame_pred_frame_dct should be 1 for progressive_sequence.\n");
      } else if(vop_structure==FIELD_VOP && GetVopFramePredFrameDCT_st(curr_vop)!=0) {
	fprintf(stderr, "frame_pred_frame_dct should be 0 for field_vop.\n");
      }
      */
	    
      tmpvar = (Int) BitstreamReadBits(stream,2,"dct_precision",trace,CODE);
      *readen_bits += 2;
      PutVopDCTPrecision_st(tmpvar,curr_vop,fip);
      
      tmpvar = (Int) BitstreamReadBits(stream,2,"intra_dc_precision",trace,CODE);
      *readen_bits += 2;
      PutVopIntraDCPrecision_st(tmpvar,curr_vop,fip);

      tmpvar = (Int) BitstreamReadBits(stream,1,"q_scale_type",trace,CODE);
      *readen_bits += 1;
      PutVopQScaleType_st(tmpvar,curr_vop,fip);

      if(GetVopPredictionType_st(curr_vop,fip)!=I_VOP &&
	 !(GetVopSpriteUsage(curr_vop)==STATIC_SPRITE &&
	   GetVopPredictionType_st(curr_vop,fip)==SPRITE_VOP)) {

	tmpvar = (Int) BitstreamReadBits(stream,4,"vop_fcode_for_x",trace,CODE);
	*readen_bits += 4;
	PutVopFCodeForX_st(tmpvar,curr_vop,fip);
	
	tmpvar = (Int) BitstreamReadBits(stream,4,"vop_fcode_for_y",trace,CODE);
	*readen_bits += 4;
	PutVopFCodeForY_st(tmpvar,curr_vop,fip);
	
	tmpvar = (Int) BitstreamReadBits(stream,4,"vop_fcode_back_x",trace,CODE);
	*readen_bits += 4;
	
	tmpvar = (Int) BitstreamReadBits(stream,4,"vop_fcode_back_y",trace,CODE);
	*readen_bits += 4;

	/*dead_zone_disable*/
	tmpvar = (Int) BitstreamReadBits(stream,1,"dead_zone_disable",trace,CODE);
	*readen_bits += 1;
	PutVopDeadZoneDisable_st(tmpvar,curr_vop,fip);

      }

    } 
      
  /* HYUNDAI (Grayscale) */ 
  if(GetVopShape(curr_vop) == GREY_SCALE) {

    /* MoMuSys-2000FDAM1COR-020612_1 */
    /* marker_bit */
    tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
    if ( !tmpvar ) {
      fprintf ( stderr, "Shall be a marker_bit\n" );
      exit(-3);
    }
    *readen_bits += 1;
    /**/
    
	for(ii=0;ii<GetVopAuxCompCount_st(curr_vop);ii++) {
	  
	  tmpvar = (Int) BitstreamReadBits(stream,2,"gray_dct_precision",trace,CODE);
	  *readen_bits += 2;
	  PutVopGrayDCTPrecision_st(tmpvar,curr_vop,ii,fip);
	  
	  /* gray_intra_dc_precison */
	  tmpvar = (Int) BitstreamReadBits(stream,2,"gray_intra_dc_precision",trace,CODE);
	  *readen_bits += 2;
	  PutVopGrayIntraDCPrecision_st(tmpvar,curr_vop,ii,fip);
	  
	  /* gray_q_scale_type */
	  tmpvar = (Int) BitstreamReadBits(stream,1,"gray_q_scale_type",trace,CODE);
	  *readen_bits += 1;
	  PutVopGrayQScaleType_st(tmpvar,curr_vop,ii,fip);

	  /* gray_alternate_scan */
	  PutVopGrayAlternateScan_st(GetVopAlternateScan_st(curr_vop,fip),curr_vop,ii,fip);
	  
	  /* gray_frame_pred_frame_dct */
	  PutVopGrayFramePredFrameDCT_st(GetVopFramePredFrameDCT_st(curr_vop),curr_vop,ii);

	}
	for(jj=ii;jj<MAX_AUX_COMP_NUM;jj++) {
	  PutVopGrayIntraDCPrecision_st(3,curr_vop,jj,fip);
	  PutVopGrayQScaleType_st(1,curr_vop,jj,fip);
	  PutVopGrayAlternateScan_st(GetVopAlternateScan_st(curr_vop,fip),curr_vop,jj,fip);
	  PutVopGrayFramePredFrameDCT_st(GetVopFramePredFrameDCT_st(curr_vop),curr_vop,jj);
	}	  

      }/*gray_shape*/


  if(GetVopSpriteUsage(curr_vop)==STATIC_SPRITE &&
     GetVopPredictionType_st(curr_vop,fip)==SPRITE_VOP) {

    if(GetVopNoOfSpritePoints(curr_vop) > 0) 
	{
	  /* decode the differentially encoded trajectories */
	  difftraj = DecodeSpriteTraj(stream,trace, GetVopNoOfSpritePoints(curr_vop),
				      GetVopSpriteUsage(curr_vop)); 
	  traj = AddTraj( GetVopNoOfSpritePoints(curr_vop), difftraj);
	  PutVopTrajPointCoord(traj,curr_vop);
	  PutVopTrajPointCoord(traj,GetVopSprite(curr_vop));
	  
	  tmp_refpoint = GetVopRefPointCoord(curr_vop);
	  for (i=0 ; i<GetVopNoOfSpritePoints(curr_vop); i++)
	    {
	      tmp_refpoint[i].x = 
		GetVopHorSpatRef_st(curr_vop,0)+(i%2)*GetVopWidth_st(curr_vop,0);
	      tmp_refpoint[i].y = 
		GetVopVerSpatRef_st(curr_vop,0)+(i>>1)*GetVopHeight_st(curr_vop,0);
	    }
	  free(difftraj);
	}
      
      if(GetVopBrightnessChangeInSprite(curr_vop)) 
        brightness_factor = decode_brightness_change_factor(stream, trace, readen_bits);
      else
        brightness_factor = 1;

      PutVopBrightnessChangeFactor(brightness_factor, curr_vop);

    if(GetVopSpriteDefocusing_st(curr_vop)) {
      /* MoMuSys-2000FDAM1COR-020615_0 */
      tmpvar = (Int) BitstreamReadBits(stream,1,"marker_bit",trace,FLAG);
      if ( !tmpvar ) {
	fprintf ( stderr, "Shall be a marker_bit\n" );
	exit(-3);
      }
      *readen_bits += 1;
      /**/
      tmpvar = decode_defocusing_control_parameter_Studio(GetVopWarpingAccuracy(curr_vop),
							  stream,
							  trace,
							  readen_bits);
    }
    else
      tmpvar = 1;
    PutVopDefocusingControlParameter_st(tmpvar, curr_vop);
    
    
      if(GetVopSpriteUsage(curr_vop) == STATIC_SPRITE)
	{
	  printf("Start SPRITE VOP (STATIC OFF-LINE CASE) decoding\n");
	  if (GetVopSpriteTransmitMode(curr_vop)!=STOP)
	    do
	      {
		/* sprite transmit mode (2 bits) */
		tmpvar = (Int) BitstreamReadBits(stream,2,"sprite_transmit_mode",trace,CODE);
		*readen_bits += 2;
		PutVopSpriteTransmitMode(tmpvar, curr_vop);
		
		if (GetVopSpriteTransmitMode(curr_vop)==PIECE ||
		    GetVopSpriteTransmitMode(curr_vop)==UPDATE)
                DecodeSpritePiece(stream, curr_vop, vo_id, trace, readen_bits,
                                    mot_x_P,mot_y_P,MB_decisions_P); 
	      } while (GetVopSpriteTransmitMode(curr_vop)!=STOP &&
		       GetVopSpriteTransmitMode(curr_vop)!=PAUSE);
				    
	  FreeVopChannels_Studio(curr_vop,FRAME_VOP,0);

	  AllocVopChannels_Studio(curr_vop, FRAME_VOP, 0,
				  GetVopWidth_st(curr_vop,0), GetVopHeight_st(curr_vop,0),
				  GetVopChromaFormat_st(curr_vop));

	  Compute_SpriteVop_Studio(curr_vop);

	  if(GetVopSpriteDefocusing_st(curr_vop)) {
	    defocus_filter_Studio(curr_vop);
	  }
	  
	  
	  if(GetVopNoOfSpritePoints(curr_vop) > 0)
	    free(traj);
		
	  if (GetVopPrevTemp(curr_vop)!=NULL) 
	    FreeVop(GetVopPrevTemp(curr_vop));
		
	  PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
	  PutVopNextTemp(NULL,curr_vop);

	  tmp_vop =CloneVop_Studio(curr_vop,FRAME_VOP,0,0);
	  if (GetVopPrevTemp(tmp_vop)!=NULL)
	    FreeVop_Studio(GetVopPrevTemp(tmp_vop));
		
	  PutVopPrevTemp(NULL,tmp_vop);  
	  PutVopNextTemp(tmp_vop,curr_vop);
	  
	  BitstreamNextStartCode_Studio(stream, trace, CODE);
	  /*BitstreamByteAlign(stream); */
	  return(0);
	} /* END STATIC CASE */
    }  /* SPRITE_USAGE!=SPRITE_NOT_USED */
  
/* 01.04.01: Studio Profile: BUGFIX */
  /*curr_vop->binary_shape_coding_method_st = 1;*/

  /* composite_display_flag(1) */
  tmpvar = (Int) BitstreamReadBits(stream,1,"composite_display_flag",trace,CODE);
  *readen_bits += 1;
  if ( tmpvar ) {
    /* v_axis(1) */
    BitstreamReadBits(stream,1,"v_axis",trace,CODE);
    *readen_bits += 1;

    /* field_sequence(1) */
    BitstreamReadBits(stream,3,"field_sequence",trace,CODE);
    *readen_bits += 3;
    
    /* sub_carrier(1) */
    BitstreamReadBits(stream,1,"sub_carrier",trace,CODE);
    *readen_bits += 1;

    /* burst_amplitude(7) */
    BitstreamReadBits(stream,7,"burst_amplitude",trace,CODE);
    *readen_bits += 7;
    
    /* sub_carrier_phase(8) */
    BitstreamReadBits(stream,8,"sub_carrier_phase",trace,CODE);
    *readen_bits += 8;
  }

  while(BitstreamShowBits(stream, 1)) {
    /* extra_bit_pixture (1bit) */
    BitstreamReadBits(stream,1,"extra_bit_picture",trace,FLAG);
    *readen_bits += 1;

    /* extra_information_picture (8bit) */
    BitstreamReadBits(stream,8,"extra_information_picture",trace,CODE);
    *readen_bits += 8;
  }
  
  /* extra_bit_pixture (1bit) */
  BitstreamReadBits(stream,1,"extra_bit_picture",trace,FLAG);
  *readen_bits += 1;
  
  /* search next_start_code */
  *readen_bits += (Int) BitstreamNextStartCode_Studio(stream, trace, CODE);

  /* extension_and_user_data(4) */
  *readen_bits += (Int) DecodeExtensionAndUserData_Studio(4, NULL, curr_vop, stream, trace);
  
  return (*readen_bits - prev_total_bits);
  /*return(1);*/
}

/***********************************************************CommentBegin******
 *
 * -- DecodeSingleVop_Studio -- Decodes the VOP information from the bitstream
 *
 * Author :		
 *	Michael Wollborn (TUH)
 *
 * Created :		
 *	12-AUG-1996
 *
 * Purpose :		
 *	 Decode the vop information from the bitstream.
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *	Int *readen_bits	pointer to value of up to now readen_bits     
 *      Int short_video_header  Flag to signal short video header bitstreams (H.263)
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	At the start of the module, the previous VOP is the current VOP.
 *	It is then copied to the next temporal one, which is the forelast
 *	VOP and which is freed. Then the VOP to be decoded is written into
 *	the current VOP.
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 28.10.96 Aasmund.Sandvand@fou.telenor.no: added support for
 *                     deblock filter
 * 	12.02.97 Aasmund Sanvand: Removed deblocking filter. Now done
 *			as post-processing.       
 *      07.07.97 Noel Brady: Added decoding of VOP_CR and change_CR_disable
 *      15-05-97 Minhua Zhou: added GOV and syntax changes
 *      18.06.97 M.Wollborn: Added correct trace mechanism
 *      24.07.97 Minhua Zhou: decoded non-B-VOPs are first padded,
 *                            then stored.
 *      04.08.97 Minhua Zhou: added f_code in the VOP layer
 *			      added intra_dc_vlc_thr
 *      05.08.97 Minhua Zhou added vop_coded
 *			06.08.97 Noel Brady: added mods for BINARY_SHAPE_ONLY
 *      28.08.97 Osamu Sunohara: modified for spatial scalable coding
 *      09.10.97 Minhua Zhou: added Vop_coded to deal with empty VOPs
 *      04.05.98 Jong Deuk Kim (HYUNDAI): field-baced ME/MC padding for interlaced
 *      23.06.98 G. Klungsoeyr/A. Sandvand: added support for short video header
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/
Void DecodeSingleVop_Studio(Bitstream **stream, Int vo_id, Trace *trace, Vop *curr_vop, 
               Int *readen_bits, Vol *vol_list, Int next_display_time,
               Int post_filter_type, Int short_video_header,Int *stop_decoding,
               ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
               Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])
   
{
  Vop *tmp_vop;
  Int num_pixels;
  Int num_lines;
  Int MB_width;
  Int MB_height;
  TrajPoint	*traj=NULL;
  Int ok;

  Int i, j;
  
#ifdef _DEBUG_SPSC_
  static Int count;
  Char name[81];
#endif
 

  Int vol_id, nvol_id;
  Int vop_structure;
  Int fip;

  /*****
   *
   *    Disable tracing for time preview 
   *
   *****/
  trace->trace = 0;

  nvol_id = GetVopId( curr_vop );
  vol_id = bs_vol[vo_id][nvol_id];
 
  /*****
   *
   *    Set trace level for VOP header
   *
   *****/
  if(trace->full_trace_period < 0)
    trace->trace = 1;
  else
    trace->trace = trace->VOP_header_def;

  TraceBreakLine(trace);

  /*****
   *
   *	Free the forelast VOP and copy the current (which is now the previous
   *	VOP) into the right place of the structure
   *
   *****/
	fprintf(stderr, "Start Decoding Studio VOP header.\n");

  ok = DecodeVopHeader_Studio(stream[vol_id], vo_id, trace, curr_vop, readen_bits,
			      mot_x_P,mot_y_P,MB_decisions_P);
  /*The output of the above function is changed to decoded bits count. */
  
  if (ok==0) 
    {
    if(temporal_scalability && GetVopScalability(curr_vop)==1)
	SetBGCImage(curr_vop,post_filter_type);
    return;
    }

  vop_structure = GetVopVopStructure_st(curr_vop);
  fip = CalcFieldParity_Studio(vop_structure,
			 GetVopVopCounter_st(curr_vop),
			 GetVopTopFieldFirst(curr_vop));
  /*****
   *
   *	Set the trace levels to the required values (either full or
   *    user-defined, see README for details)
   *
   *****/
  if(trace->full_trace_period < 0)
    {
      SetTraceLevelsFull(trace);
    }
  else if(trace->full_trace_period > 0 &&
	  trace->counter>=trace->full_trace_start &&
	  trace->counter<=trace->full_trace_end)
    {
      if((trace->counter - trace->full_trace_start) % trace->full_trace_period)
        SetTraceLevelsDefault(trace);
      else
        SetTraceLevelsFull(trace);
    }
  else
    {
      SetTraceLevelsDefaultZero(trace);
    }
  trace->counter++;

  TraceBreakLine(trace);

#ifdef _DEBUG_SPSC_
  PrintVopInfo(curr_vop,"currA_vop");
  PrintVopInfo(GetVopNextTemp(curr_vop),"currA_vop->N");
  PrintVopInfo(GetVopPrevTemp(curr_vop),"currA_vop->P");
  sprintf(name,"A_vop%d",count);
  WriteVopDebug(curr_vop,name,name,name,0);
#endif

  /*****
   *
   *	Do scalable or non-scalable decoding of the current VOP
   *
   *****/
  if(GetVopScalability(curr_vop) == 0) 
    DecodeSingleVopNonScalable_Studio(stream[ vol_id ],vo_id,trace,curr_vop,short_video_header,
				mot_x_P,mot_y_P,MB_decisions_P);
  else 
    DecodeVopScalable(stream,vo_id,trace,curr_vop,vol_list,post_filter_type,
			mot_x_P,mot_y_P,MB_decisions_P);
  
  if(temporal_scalability && CheckBaseVop(curr_vop))
    fprintf(stderr,"[B]frame[%d]\n",CalcFrameNumber(curr_vop));

  if ( (GetVopSpriteUsage(curr_vop) != SPRITE_NOT_USED)&&
  			(GetVopPredictionType(curr_vop)==SPRITE_VOP) )
	{
	if(GetVopNoOfSpritePoints(curr_vop) > 0)
		free(traj);
	}

#ifdef _DEBUG_SPSC_
  PrintVopInfo(curr_vop,"currB_vop");
  PrintVopInfo(GetVopNextTemp(curr_vop),"currB_vop->N");
  PrintVopInfo(GetVopPrevTemp(curr_vop),"currB_vop->P");
  sprintf(name,"B_vop%d",count);
  WriteVopDebug(curr_vop,name,name,name,0);
#endif

/* added by Minhua Zhou */

	if(GetVopVopStructure_st(curr_vop)==FRAME_VOP) {
	  if(GetVopInterlaced(curr_vop)==0) {
	    MacroblockBasedPadding_Frame_Studio(curr_vop);
	  } else {
	    Vop *tmp_rec_pad;
	    /*Int tmp_width, tmp_height;*/
	    tmp_rec_pad = CloneVop_Studio(curr_vop, FRAME_VOP, 0, 0);
	    /*
	    tmp_width  = GetVopWidth_st(curr_vop, 0);
	    tmp_height = GetVopHeight_st(curr_vop, 0)/2;

	    PutVopVopStructure_st(FIELD_VOP,curr_vop);
	    */
	    ChangeRectFrameVOPtoFieldVOPs_Studio(tmp_rec_pad);
	      
	    for(i=0;i<2;i++) {
	      PutVopVopCounter_st(i, tmp_rec_pad);
	      MacroblockBasedPadding_Field_Studio2(tmp_rec_pad,1);
	    }

	    for(i=0;i<2;i++) {
	      FreeImage(curr_vop->y_chan_st[i]);
	      FreeImage(curr_vop->u_chan_st[i]);
	      FreeImage(curr_vop->v_chan_st[i]);
	      for(j=0;j<MAX_AUX_COMP_NUM;j++) {
		FreeImage(curr_vop->a_chan_st[j][i]);
		FreeImage(curr_vop->g_chan_st[j][i]);
		FreeImage(curr_vop->a_uv_chan_st[j][i]);
	      }
	    }

	    for(i=0;i<2;i++) {
	      curr_vop->y_chan_st[i] = tmp_rec_pad->y_chan_st[i];
	      tmp_rec_pad->y_chan_st[i] = NULL;
	      curr_vop->u_chan_st[i] = tmp_rec_pad->u_chan_st[i];
	      tmp_rec_pad->u_chan_st[i] = NULL;
	      curr_vop->v_chan_st[i] = tmp_rec_pad->v_chan_st[i];
	      tmp_rec_pad->v_chan_st[i] = NULL;
	      for(j=0;j<MAX_AUX_COMP_NUM;j++) {
		curr_vop->a_chan_st[j][i] = tmp_rec_pad->a_chan_st[j][i];
		tmp_rec_pad->a_chan_st[j][i] = NULL;
		curr_vop->g_chan_st[j][i] = tmp_rec_pad->g_chan_st[j][i];
		tmp_rec_pad->g_chan_st[j][i] = NULL;
		curr_vop->a_uv_chan_st[j][i] = tmp_rec_pad->a_uv_chan_st[j][i];
		tmp_rec_pad->a_uv_chan_st[j][i] = NULL;
	      }
	    }

	    FreeVop_Studio(tmp_rec_pad);
	  }
	} else {
	  MacroblockBasedPadding_Field_Studio2(curr_vop,0);
	}  

  
  if((GetVopVopStructure_st(curr_vop) == FRAME_VOP && GetVopVopCounter_st(curr_vop) == 0) ||
     (GetVopVopStructure_st(curr_vop) == FIELD_VOP && GetVopVopCounter_st(curr_vop) == 1)) {
  if (GetVopPredictionType_st(curr_vop,0)!=B_VOP
    ) { 
    if (GetVopPrevTemp(curr_vop)!=NULL) 
      FreeVop_Studio(GetVopPrevTemp(curr_vop));
    PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
    PutVopNextTemp(NULL,curr_vop);


    /* Clone FrameVOP */
    tmp_vop = CloneVop_Studio(curr_vop, vop_structure, fip, fip);
    if(GetVopVopStructure_st(curr_vop)==FIELD_VOP) {
      AllocVopChannels_Studio(tmp_vop, FIELD_VOP, (fip==0)?1:0, GetVopWidth_st(curr_vop,(fip==0)?1:0), GetVopHeight_st(curr_vop,(fip==0)?1:0), GetVopChromaFormat_st(curr_vop));
      CopyVop_Studio(curr_vop, FIELD_VOP, (fip==0)?1:0, tmp_vop, (fip==0)?1:0);
    }

    if (GetVopPrevTemp(tmp_vop)!=NULL)
      FreeVop_Studio(GetVopPrevTemp(tmp_vop));
    PutVopPrevTemp(NULL,tmp_vop);  

/* 1197-3 SONY */
    if(GetVopBvopTemp(tmp_vop)!=NULL)
      FreeVop_Studio(GetVopBvopTemp(tmp_vop));
    PutVopBvopTemp(NULL,tmp_vop);
/* 1197-3 */

    PutVopNextTemp(tmp_vop,curr_vop);

  }
}
 
  
#ifdef _DEBUG_SPSC_
  PrintVopInfo(curr_vop,"currC_vop");
  PrintVopInfo(GetVopNextTemp(curr_vop),"currC_vop->N");
  PrintVopInfo(GetVopPrevTemp(curr_vop),"currC_vop->P");
  sprintf(name,"C_vop%d",count);
  WriteVopDebug(curr_vop,name,name,name,0);

  count++;
#endif
  
  return;
}

/***********************************************************CommentBegin******
 *
 * -- DecodeSingleVopNonScalable_Studio -- Decodes the VOP information from the bitstream
 *			      in non-scalable mode
 *
 * Author :		
 *	Michael Wollborn (TUH)
 *
 * Created :		
 *	12-AUG-1996
 *
 * Purpose :		
 *	 Decode the vop information from the bitstream.
 * 
 *
 * Arguments in : 	
 *	Bitstream *stream,	pointer on the decoder bitstream structure,
 *				should be at the right position.
 *	Trace *trace		pointer to trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop,		pointer on considered vop - the previous VOP
 *				can be accessed by the structure of the current
 *				VOP (GetVopNextTemp())
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	At the start of the module, the previous VOP is the current VOP.
 *	It is then copied to the next temporal one, which is the forelast
 *	VOP and which is freed. Then the VOP to be decoded is written into
 *	the current VOP.
 *
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 17-SEP-1996, M.Wollborn, introduced some changes for new
 *			   shape decoding		
 *	23.10.96 Robert Danielsen: Passing QP to lower functions for
 *			DCAC prediction
 *	27.12.96 Noel Brady: modifications to allow INTER shape decoding
 *			     in separate mode.
 *      22-Jan-97 Jan De Lameillieure : deallocate AB_first_MMR_values only 
 *		                        for separate mode ! 
 *	07-Feb-1997, M.Wollborn: changed in order to treat skipped MBs
 *			         correctly
 *      26.04.97 Luis Ducla Soares: Changes to allow error resilient
 *                                  decoding in the combined mode with
 *                                  data partitioning.
 *      14.10.97 Noel Brady: ChangeMBType2 is not called when the decoded VOP
 * 			     is a B-VOP
 *      11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/
Void DecodeSingleVopNonScalable_Studio(Bitstream *stream, Int vo_id, Trace *trace, Vop *curr_vop,
				       Int short_video_header,
				       ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
				       ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
				       Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])

{

  Vop *sprite=NULL;

  Int vop_structure = GetVopVopStructure_st(curr_vop);
  Int fip = CalcFieldParity_Studio(vop_structure,
			     GetVopVopCounter_st(curr_vop),
			     GetVopTopFieldFirst(curr_vop));
  Int chroma_format = GetVopChromaFormat_st(curr_vop);

  Int i;
  Int ii;
  
  
  if ( (GetVopSpriteUsage(curr_vop)!=SPRITE_NOT_USED)&&
       (GetVopPredictionType(curr_vop)==SPRITE_VOP) )
    sprite = GetVopSprite(curr_vop);

  /*****
   *
   *	Free image channels of VOP and allocate the image channels
   *	for the new sizes (only necessary if arbitrary shape)
   *
   *****/

  if (GetVopSpriteUsage(curr_vop)!=STATIC_SPRITE)
    {    
      if(GetVopVopCounter_st(curr_vop) == 0) {
	/* modified at Aug 26, 99 */
	for(i=0;i<2;i++)
	  FreeVopChannels_Studio(curr_vop, FIELD_VOP, i);
      }


      AllocVopChannels_Studio(curr_vop, vop_structure, fip,
			      ((GetVopWidth_st (curr_vop,fip)+15)/16)*16,
			      ((GetVopHeight_st(curr_vop,fip)+15)/16)*16,
			      chroma_format);
    }

  /*****
   *
   *	Set local parameters and allocate local memory
   *
   *****/
  
  /*****
   *
   *	Do combined or separate decoding
   *
   *****/
  /* ??? Here, should unused memories set to the constant value ???  */
  for(ii=0;ii<MAX_AUX_COMP_NUM;ii++) {
    
    if (GetVopShape(curr_vop)==RECTANGULAR ||
	((GetVopShape(curr_vop)==BINARY||GetVopShape(curr_vop)==BINARY_SHAPE_ONLY)&&ii!=0) ||
	(GetVopShape(curr_vop)==GREY_SCALE&&ii>=GetVopAuxCompCount_st(curr_vop))) {
      
      SetConstantImage(GetVopA_st(curr_vop,ii,fip),255);
      SetConstantImage(GetVopAuv_st(curr_vop,ii,fip),255);
      if(GetVopVopStructure_st(curr_vop) == FRAME_VOP) {
	SetConstantImage(GetVopA_st(curr_vop,ii,1),255);
	SetConstantImage(GetVopAuv_st(curr_vop,ii,1),255);
      }
      
    }
  }
  
  DecodeSingleVopCombMotionShapeTexture_Studio(stream,vo_id,trace,curr_vop,short_video_header,
					       mot_x_P,mot_y_P,MB_decisions_P);

  /*****
   *
   *	Free local memory and return
   *
   *****/
  
  return;
}

/***********************************************************CommentBegin******
 *
 * -- WriteOutputImage_Studio -- writes all VOLs to an output image
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
  * Created :		
 *	15-AUG-1996
 *
 * Purpose :
 *	Writes the respective VOP for each active VOL to an output image.
 *	Therefore in each VOP of each VOL two VOPs can be accessed which
 *	are successive in time. Either the most recent VOP has exactly the
 *	same display time as the current local time, then this VOP is
 *	displayed. Otherwise, if one VOP has a later display time and the
 *	other an earlier display time, the earlier VOP is displayed. If the
 *	display time of both VOPs is earlier than the current local time, the
 *	respective VOP is not displayed at all, since then it is assumed that
 *	the VO has disappeared from the scene.
 *
 * 
 * Arguments in : 	
 *	Vol *vol_list[]  	  List of VOLs to be displayed
 *	Int local_time		  Local time base, i.e. time of actual display
 *  Int random_access_start_time_in
 *	Int image_counter	  Counter for the output image
 *	Char output_filename[][]  Name of the output file for the composed
 *				  image sequence
 *      Int post_filter_type  status of deblock filtering
 *	Int stop_decoding_vol[][] 	List with end info for all bitstreams
 *
 *
 * Arguments in/out :	
 *
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *	21.01.97 Robert Danielsen: Changed 127 to 128 for initialization
 *			of display_vop. Fix by Noel/Paulo.
 *      31.01.97 Aasmund Sandvand: Added deblock filtering
 *	12.02.97 M.Wollborn: Changed to avoid double writing of last VOP
 *      07.07.97 A. Sandvand: Modified to support Random Access
 *      24.07.97 Minhua Zhou: Modified to support B-VOPs
 *		 06.08.97 Noel Brady:  For VOPs, having only shape information, a default
 *											constant YUV colur is used i.e. 255,0,0
 *      09.10.97 Minhua Zhou: added Vop_coded to deal with empty VOPs
 *      01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *
 ***********************************************************CommentEnd********/

Void WriteOutputImage_Studio(Int number_of_vos,
		      Vol *vol_list[], Float local_time_in, 
		      Int random_access_start_time_in,Vop *display_vop, Int image_counter, 
		      Char output_filename[][300],
		      Char output_filename_aux_comp[][100],
		      Int post_filter_type,
		      Int stop_decoding_vol[][MAX_NUM_VOLS])
{
  Vop *local_vop, *filter_vop;
  Int vop_time,i;
  Int time_dis1,time_dis2,time_dis3,time_min;
  Int local_time =0, random_access_start_time =0 ;

  Int k;

  Int ii;
  
  /*****
   *
   *	Set the display VOP to grey (A=0, Y=0, U=V=128)
   *
   *****/

  for(k=0;k<2;k++) {
    for(ii=0;ii<MAX_AUX_COMP_NUM;ii++)
      SetConstantImage(GetVopA_st(display_vop,ii,k),0);
    SetConstantImage(GetVopY_st(display_vop,k),0);
    SetConstantImage(GetVopU_st(display_vop,k),GetVopMidGrey(display_vop)/*128*/);
    SetConstantImage(GetVopV_st(display_vop,k),GetVopMidGrey(display_vop)/*128*/);
  }
 
  /*****
   *
   *	The general policy for composing the VOs is the following:
   *
   *	Nor spatial neither temporal scalability is supported yet.
   *	The VOs are composed in an order derived by their appearance
   *	in the configuration file. The first VO appearing their has the
   *	highest priority, i.e. it is displaye in front of all other VOs
   *	and so forth. With respect to the local time base, the currently
   *	decoded VOP of a VOL is only displayed if its time pointer is lower
   *	or equal to the local time; otherwise the previously decoded VOP
   *	is displayed at this time instance.
   *
   *****/
  for(i=0; i<number_of_vos; i++)
    {
      /*****
       *
       *	  Display only VOs whose coding has not stopped yet
       *	  (implemented only for the base layer at the moment)
       *	  (M.Wollborn, 12.02.1997)
       *
       *****/
      
      if(stop_decoding_vol[i][0] <2)
	{
	  /*****
	   *
	   *    Get he current VOP of the current VOL (only base VOL is processed
	   *    in the current implementation) and check the time stamp
	   *
	   *****/

	    if (stop_decoding_vol[i][0]==1)
	      stop_decoding_vol[i][0]=2;
	    local_vop = GetVolVop(vol_list[i]);
	    local_time = (Int)(0.001+local_time_in/1000.0*
			       (Double)GetVopTimeIncrementResolution(local_vop));
	    random_access_start_time = (Int)(0.001+random_access_start_time_in/1000.0*
					     (Double)GetVopTimeIncrementResolution(local_vop));
	    vop_time = CalcVopDisplayTime(local_vop);
	    time_dis1=abs(vop_time-local_time);
	    vop_time = CalcVopDisplayTime(GetVopNextTemp(local_vop));
	    time_dis2=abs(vop_time-local_time);
	    if (GetVopScalability(local_vop)==0) {
	      vop_time = CalcVopDisplayTime(GetVopPrevTemp(local_vop));
	      time_dis3=abs(vop_time-local_time);
	    } else time_dis3=time_dis1+1;
	    time_min=time_dis1;
	    if (time_min>time_dis3) time_min=time_dis3;
	    if (time_min>time_dis2) time_min=time_dis2;
	    if (time_min==time_dis2) {
	      local_vop =GetVopNextTemp(local_vop);
	      fprintf(stderr,"NEXT1\n");
	    }
	    else if (time_min==time_dis3) {
	      local_vop =GetVopPrevTemp(local_vop);
	    }
	    
	    /*****
	     *
	     *    For rectangular VOPs set the alph-plane again to "255"
	     *    as a workaround, since it is modified by incorrect motion
	     *    compensation (without padding)
	     *
	     *****/
	    /* if(GetVopShape(local_vop) == 0) 
	       SetConstantImage(GetVopA(local_vop),255);*/ 
	    
	    if (GetVopShape(local_vop) == BINARY_SHAPE_ONLY) /* BSO_NOEL */
	      {
		for(k=0;k<2;k++) {
		  SetConstantImage(GetVopY_st(local_vop,k),255);
		  SetConstantImage(GetVopU_st(local_vop,k),0);
		  SetConstantImage(GetVopV_st(local_vop,k),0);
		}
	      }

	    /*****
	     *
	     *    If the VOP is empty (i.e. arbitrary shape and width=height=0
	     *    then don't display it (this is not compliant with the VM yet,
	     *    since it is not specified how to tell that a VOP has 
	     *    disappeared) else create list of all VOPs to be displayed
	     *
	     *****/
#if 0
	    if(/* GetVopShape(local_vop)!=0 && *//* commented out for testing vop_coded MW 30-NOV-1998 */GetVopCoded(local_vop)==0)
#endif 0
	    if (GetVopShape(local_vop)!=0 && GetVopCoded_st(local_vop,0)==0)
	      /* Modified by Michael Wollborn */
	      continue;
	    else
	      {
		Int iii,jjj;
		
		if(post_filter_type==0)
		  BlendVops_Studio(local_vop, display_vop);
		else
		  {
		    fprintf(stderr,"postfilter unsupported WriteOutputImage_Studio\n");
		    exit(3);
		      
                    if (GetImageSizeX(GetVopQP(local_vop)) == GetVopWidth(local_vop)/16 ||
		        GetVopPredictionType(local_vop) == B_VOP)
		         filter_vop = CloneVop(local_vop);
		    else
		         filter_vop = CloneVop_TMP(local_vop); 
		    PostFilter(filter_vop,post_filter_type);
		    BlendVops(filter_vop, display_vop);
		    FreeVop(filter_vop);
		  }
		
	      } /* vop coded */
	}
    }
  
  /*****
   *
   *    Write composited display_vop to disk
   *
   *****/

  if(local_time == random_access_start_time)
    WriteRectFrameVopGeneric_Studio(display_vop,output_filename[0],output_filename[1],
		    output_filename[2],output_filename[3],
		    output_filename_aux_comp,/*AM2PDAM*/
		    image_counter,
		    IO_FORMAT,IO_OVERWRITE,TRUE);
  else
    WriteRectFrameVopGeneric_Studio(display_vop,output_filename[0],output_filename[1],
		    output_filename[2],output_filename[3],
		    output_filename_aux_comp,/*AM2PDAM*/
		    image_counter,
		    IO_FORMAT,IO_APPEND,TRUE);

  return;
}

/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombMotionShapeTexture_Studio -- decode VOP in combined mode
 *
 * Author :		
 *      Michael Wollborn (TUH)
 *
 * Created :		
 *	16-AUG-1996
 *
 * Purpose :
 *	Decodes the VOP in the combined motion/shape/texture mode
 *
 * 
 * Arguments in : 
 *	Bitstream *stream	bitstream to decode
 *	Trace *trace		trace information
 *
 *
 * Arguments in/out :	
 *	Vop *curr_vop		curent VOP to be decoded	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified : 
 *     JDL introduced DecodeVopCombinedShapeTextureIntra (18-Jan-97)
 *     JDL added DecodeVopCombinedMotionShapeTextureInter (19-Jan-97)
 *     24.07.97 Minhua Zhou: added mot_x_p,mot_y_P to support B-VOPs
 *                           added B-VOP decoding
 *     11.09.98 Guido Heising (HHI): changes concerning error res. and scalability
 *     01.04.01 Osamu Sunohara (SONY): Function for Studio Profile
 *	
 *
 ***********************************************************CommentEnd********/
Void DecodeSingleVopCombMotionShapeTexture_Studio(Bitstream *stream, Int vo_id,
                        Trace *trace, Vop *curr_vop, Int short_video_header,
                          ImageF *mot_x_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          ImageF *mot_y_P[MAX_NUM_VOS][MAX_NUM_VOLS],
                          Image *MB_decisions_P[MAX_NUM_VOS][MAX_NUM_VOLS])

{

  ImageF	*mot_x,
                *mot_y;
  Image *MB_decisions;
  Int vol_id,nvol_id;

  Int vop_structure = GetVopVopStructure_st(curr_vop);
  Int fip = CalcFieldParity_Studio(vop_structure, GetVopVopCounter_st(curr_vop), GetVopTopFieldFirst(curr_vop));
  

  nvol_id = GetVopId( curr_vop );
  vol_id = bs_vol[vo_id][nvol_id];
  
  if(GetVopPredictionType_st(curr_vop,fip) == INTRA_VOP)

       DecodeSingleVopCombinedShapeTextureIntra_Studio(stream, vo_id, trace, curr_vop, short_video_header);
			     
  else if (GetVopPredictionType_st(curr_vop,fip) != B_VOP)
  	{
	DecodeVopCombinedMotionShapeTextureInter_Studio(stream, vo_id, trace, curr_vop,&mot_x,&mot_y,
						 &MB_decisions,short_video_header);

	}
   else {
     fprintf(stderr,"This software supports only I or P-VOP\n");
     exit(3);
   }

 /* Deallocate memory for motion vectors and MB modes */
	    if (GetVopPredictionType_st(curr_vop,fip)!=B_VOP) {
	      if (mot_x_P[vo_id][vol_id]!=NULL) FreeImage(mot_x_P[vo_id][vol_id]);
	      if (mot_y_P[vo_id][vol_id]!=NULL) FreeImage(mot_y_P[vo_id][vol_id]);
              if (MB_decisions_P[vo_id][vol_id]!=NULL) FreeImage(MB_decisions_P[vo_id][vol_id]);     
              if (GetVopPredictionType_st(curr_vop,fip)==I_VOP) {
                mot_x_P[vo_id][vol_id]=AllocImage(GetVopWidth_st (curr_vop,fip)/8,GetVopHeight_st (curr_vop,fip)/8,FLOAT_TYPE);
                mot_y_P[vo_id][vol_id]=AllocImage(GetVopWidth_st (curr_vop,fip)/8,GetVopHeight_st (curr_vop,fip)/8,FLOAT_TYPE);
                SetConstantImage (mot_x_P[vo_id][vol_id],+0.0);
                SetConstantImage (mot_y_P[vo_id][vol_id],+0.0);
                MB_decisions_P[vo_id][vol_id]=AllocImage(GetVopWidth_st (curr_vop,fip)/16,GetVopHeight_st (curr_vop,fip)/16,SHORT_TYPE);
                SetConstantImage (MB_decisions_P[vo_id][vol_id],0); /* MBM_INTRA */
                } else {
	           mot_x_P[vo_id][vol_id] = mot_x;
	           mot_y_P[vo_id][vol_id] = mot_y;
                   MB_decisions_P[vo_id][vol_id]=MB_decisions;
                  }
	    }


  return;
}

Int DecodeExtensionAndUserData_Studio(Int header_id, Vol *vol, Vop *vop, Bitstream *stream, Trace *trace) {

  Int tmpvar;
  Int bits = 0;

  tmpvar = (Int) BitstreamShowBits ( stream, 32 );
  while ( tmpvar == EXTENSION_START_CODE || tmpvar == USER_DATA_START_CODE ) {
    if ( tmpvar == EXTENSION_START_CODE ) {
      if ( header_id == 2 || header_id == 4 ) {
	bits += DecodeExtensionData_Studio ( header_id, vol, vop, stream, trace );
      } else {
	fprintf ( stderr, "Syntax Error: The header of header_id other than 2, 4 does not support extension_data()\n");
	exit(1);
      }
    }
    if ( tmpvar == USER_DATA_START_CODE )
      bits += DecodeUserData_Studio( stream, trace );
    tmpvar = (Int) BitstreamShowBits ( stream, 32 );
  }

  return bits;
}

Int DecodeExtensionData_Studio ( Int header_id, Vol *vol, Vop *vop, Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;

  tmpvar = (Int) BitstreamShowBits ( stream, 32 );
  while ( tmpvar == EXTENSION_START_CODE ) {

    BitstreamReadBits ( stream, 32, "extension_start_code", trace, CODE );
    bits += 32;
    
    if ( header_id == 2 ) { /* Called in StudioVideoObjectLayer() */

      tmpvar = (Int) BitstreamShowBits ( stream, 4 );
      if ( tmpvar == SEQUENCE_DISPLAY_EXTENSION_ID )
	bits += DecodeSequenceDisplayExtension_Studio ( stream, trace );
      else if ( tmpvar == QUANT_MATRIX_EXTENSION_ID )
	bits += DecodeVolQuantMatrixExtension_Studio ( vol, stream, trace );
      else if ( tmpvar == VLC_CODE_EXTENSION_ID )
	bits += DecodeVLCCodeExtension_Studio ( stream, trace );
      else {
	fprintf ( stderr, "Syntax Error: A Wrong extension_start_code_identifier [%d] for header_id [%d]\n",
		  tmpvar, header_id );
	exit ( 1 );
      }

    } else if ( header_id == 4 ) { /* Called in StudioVideoObjectPlane() */

      tmpvar = (Int) BitstreamShowBits ( stream, 4 );
      if ( tmpvar == QUANT_MATRIX_EXTENSION_ID )
	bits += DecodeVopQuantMatrixExtension_Studio ( vop, stream, trace );
      else if ( tmpvar == COPYRIGHT_EXTENSION_ID )
	bits += DecodeCopyrightExtension_Studio ( stream, trace );
      else if ( tmpvar == PICTURE_DISPLAY_EXTENSION_ID )
	bits += DecodePictureDisplayExtension_Studio ( stream, trace );
      else if ( tmpvar == CAMERA_PARAMETERS_EXTENSION_ID )
	bits += DecodeCameraParametersExtension_Studio ( stream, trace );
      else if ( tmpvar == ITUT_EXTENSION_ID )
	bits += DecodeITUTExtension_Studio ( stream, trace );
      else if ( tmpvar == VLC_CODE_EXTENSION_ID )
	bits += DecodeVLCCodeExtension_Studio ( stream, trace );
      else {
	fprintf ( stderr, "Syntax Error: A Wrong extension_start_code_identifier [%d] for header_id [%d]\n",
		  tmpvar, header_id );
	exit ( 1 );
      }
    } else {
      fprintf ( stderr, "Syntax Error: extension_data() does not support the header of header_id\n" );
      exit ( 0 );
    }

    tmpvar = (Int) BitstreamShowBits ( stream, 32);
  }

  return bits;
}

Int DecodeUserData_Studio ( Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;

  /* user_data_start_code(32) */
  BitstreamReadBits ( stream, 32, "user_data_start_code", trace, CODE);
  bits += 32;

  tmpvar = (Int) BitstreamShowBits ( stream, 24);
  while ( tmpvar != 1 ) {
    /* user_data(8) */
    BitstreamReadBits ( stream, 8, "user_data", trace, CODE);
    bits += 8;

    tmpvar = (Int) BitstreamShowBits ( stream, 24);
  }

  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}
      
      
Int DecodeVolQuantMatrixExtension_Studio ( Vol *vol, Bitstream *stream, Trace *trace ) {

  Int *qmat;
  Int tmpvar;
  Int bits = 0;
  Int i, aux;
  Int *qmat2;
  
  /* extension_start_code_identifier */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != QUANT_MATRIX_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: Quant Matrix Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  /* load_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVolLoadIntraQuantMat ( tmpvar, vol );

  /* intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVolIntraQuantMat ( vol );
    qmat2 = GetVolChromaIntraQuantMat_st ( vol );
    for ( i=0; i<64; i++ ) {
      qmat2[ *(zigzag_i+i) ] = qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  /* load_non_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_non_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVolLoadNonintraQuantMat ( tmpvar, vol );

  /* non_intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVolNonintraQuantMat ( vol );
    qmat2 = GetVolChromaNonintraQuantMat_st ( vol );
    for ( i=0; i<64; i++ ) {
      qmat2[ *(zigzag_i+i) ] = qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "non_intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  /* load_chroma_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_chroma_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVolLoadChromaIntraQuantMat_st ( tmpvar, vol );

  if ( GetVolRGBComponents_st(vol) && tmpvar ) {
    fprintf ( stderr, "Syntax Error: rgb_components==1, load_chroma_intra_quant_mat==1\n");
    exit(-1);
  }
  
  /* chroma_intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVolChromaIntraQuantMat_st ( vol );
    for ( i=0; i<64; i++ ) {
      qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "chroma_intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  /* load_chroma_non_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_chroma_non_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVolLoadChromaNonintraQuantMat_st ( tmpvar, vol );

  if ( GetVolRGBComponents_st(vol) && tmpvar ) {
    fprintf ( stderr, "Syntax Error: rgb_components==1, load_chroma_non_intra_quant_mat==1\n");
    exit(-1);
  }
  
  /* chroma_non_intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVolChromaNonintraQuantMat_st ( vol );
    for ( i=0; i<64; i++ ) {
      qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "chroma_non_intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  if ( GetVolShape ( vol ) == GREY_SCALE ) {
    for ( aux = 0; aux < GetVolAuxCompCount_st ( vol ); aux ++ ) {

      /* load_gray_intra_quantiser_matrix_grayscale */
      tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_gray_intra_quantiser_matrix_grayscale", trace, FLAG );
      bits ++;
      PutVolLoadGrayIntraQuantMat_st ( tmpvar, vol, aux );
      
      /* gray_intra_quantiser_matrix_grayscale[64] */
      if ( tmpvar ) {
	qmat = GetVolGrayIntraQuantMat_st ( vol, aux );
	for ( i=0; i<64; i++ ) {
	  qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "gray_intra_quant_mat_grayscale", trace, NUM );
	  bits += 8;
	}
      }
      
      /* load_gray_non_intra_quantiser_matrix_grayscale */
      tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_gray_nonintra_quantiser_matrix_grayscale", trace, FLAG );
      bits ++;
      PutVolLoadGrayNonintraQuantMat_st ( tmpvar, vol, aux );
      
      /* gray_nonintra_quantiser_matrix_grayscale[64] */
      if ( tmpvar ) {
	qmat = GetVolGrayNonintraQuantMat_st ( vol, aux );
	for ( i=0; i<64; i++ ) {
	  qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "gray_nonintra_quant_mat_grayscale", trace, NUM );
	  bits += 8;
	}
      }
    }
  }

  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}

Int DecodeVopQuantMatrixExtension_Studio ( Vop *vop, Bitstream *stream, Trace *trace ) {

  Int *qmat;
  Int tmpvar;
  Int bits = 0;
  Int i, aux;
  Int *qmat2;
  
  /* extension_start_code_identifier */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != QUANT_MATRIX_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: Quant Matrix Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  /* load_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVopLoadIntraQuantMat ( tmpvar, vop );

  /* intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVopIntraQuantMat ( vop );
    qmat2 = GetVopChromaIntraQuantMat_st ( vop );
    for ( i=0; i<64; i++ ) {
      qmat2[ *(zigzag_i+i) ] = qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  /* load_non_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_non_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVopLoadNonintraQuantMat ( tmpvar, vop );

  /* non_intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVopNonintraQuantMat ( vop );
    qmat2 = GetVopChromaNonintraQuantMat_st ( vop );
    for ( i=0; i<64; i++ ) {
      qmat2[ *(zigzag_i+i) ] = qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "non_intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  /* load_chroma_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_chroma_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVopLoadChromaIntraQuantMat_st ( tmpvar, vop );

  if ( GetVopRGBComponents_st(vop) && tmpvar ) {
    fprintf ( stderr, "Syntax Error: rgb_components==1, load_chroma_intra_quant_mat==1\n");
    exit(-1);
  }
  
  /* chroma_intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVopChromaIntraQuantMat_st ( vop );
    for ( i=0; i<64; i++ ) {
      qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "chroma_intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  /* load_chroma_non_intra_quantiser_matrix */
  tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_chroma_non_intra_quantiser_matrix", trace, FLAG );
  bits ++;
  PutVopLoadChromaNonintraQuantMat_st ( tmpvar, vop );

  if ( GetVopRGBComponents_st(vop) && tmpvar ) {
    fprintf ( stderr, "Syntax Error: rgb_components==1, load_chroma_non_intra_quant_mat==1\n");
    exit(-1);
  }
  
  /* chroma_non_intra_quantiser_matrix[64] */
  if ( tmpvar ) {
    qmat = GetVopChromaNonintraQuantMat_st ( vop );
    for ( i=0; i<64; i++ ) {
      qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "chroma_non_intra_quant_mat", trace, NUM );
      bits += 8;
    }
  }

  
  if ( GetVopShape ( vop ) == GREY_SCALE ) {
    for ( aux = 0; aux < GetVopAuxCompCount_st ( vop ); aux ++ ) {

      /* load_intra_quantiser_matrix_grayscale */
      tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_intra_quantiser_matrix_grayscale", trace, FLAG );
      bits ++;
      PutVopLoadGrayIntraQuantMat_st ( tmpvar, vop, aux );
      
      /* intra_quantiser_matrix_grayscale[64] */
      if ( tmpvar ) {
	qmat = GetVopGrayIntraQuantMat_st ( vop, aux );
	for ( i=0; i<64; i++ ) {
	  qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "intra_quant_mat_grayscale", trace, NUM );
	  bits += 8;
	}
      }
      
      /* load_non_intra_quantiser_matrix_grayscale */
      tmpvar = (Int) BitstreamReadBits ( stream, 1, "load_non_intra_quantiser_matrix_grayscale", trace, FLAG );
      bits ++;
      PutVopLoadGrayNonintraQuantMat_st ( tmpvar, vop, aux );
      
      /* non_intra_quantiser_matrix_grayscale[64] */
      if ( tmpvar ) {
	qmat = GetVopGrayNonintraQuantMat_st ( vop, aux );
	for ( i=0; i<64; i++ ) {
	  qmat[ *(zigzag_i+i) ] = (Int) BitstreamReadBits ( stream, 8, "non_intra_quant_mat_grayscale", trace, NUM );
	  bits += 8;
	}
      }
    }
  }

  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}
  
Int DecodeSequenceDisplayExtension_Studio ( Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;

  /* extension_start_code_identifier(3) */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != SEQUENCE_DISPLAY_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: Sequence Display Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  /* video_format(3) */
  BitstreamReadBits ( stream, 3, "video_format", trace, CODE);
  bits += 3;

  /* video_range(1) */
  BitstreamReadBits ( stream, 1, "video_range", trace, CODE);
  bits ++;
  
  /* colour_description(1) */
  tmpvar = BitstreamReadBits ( stream, 1, "colour_description", trace, CODE);
  bits ++;

  if(tmpvar) {
    /* colour_primaries(8) */
    BitstreamReadBits ( stream, 8, "colour_primaries", trace, CODE);
    bits += 8;
    
    /* transfer_characteristics(8) */
    BitstreamReadBits ( stream, 8, "transfer_characteristics", trace, CODE);
    bits += 8;
    
    /* matrix_coefficients(8) */
    BitstreamReadBits ( stream, 8, "matrix_coefficients", trace, CODE);
    bits += 8;
  }

  /* display_horizontal_size(14) */
  BitstreamReadBits ( stream, 14, "display_horizontal_size", trace, CODE);
  bits += 14;
  
  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE);
  bits ++;
    
  /* display_vertical_size(14) */
  BitstreamReadBits ( stream, 14, "display_vertical_size", trace, CODE);
  bits += 14;
    
  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}

Int DecodePictureDisplayExtension_Studio ( Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;
  Int number_of_frame_centre_offsets = 1; /* shall be changed */
  Int i;
  
  /* extension_start_code_identifier(3) */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != PICTURE_DISPLAY_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: Picture Display Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  for ( i=0; i<number_of_frame_centre_offsets; i++ ) {

    /* frame_centre_horizontal_offset(16) */
    BitstreamReadBits ( stream, 16, "frame_centre_horizontal_offset", trace, CODE);
    bits += 16;
    
    /* marker_bit(1) */
    BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE);
    bits ++;

    /* frame_centre_vertical_offset(16) */
    BitstreamReadBits ( stream, 16, "frame_centre_vertical_offset", trace, CODE);
    bits += 16;
    
    /* marker_bit(1) */
    BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE);
    bits ++;
  }

  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}

Int DecodeCopyrightExtension_Studio ( Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;

  /* extension_start_code_identifier(3) */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != COPYRIGHT_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: Copyright Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  /* copyright_flag(1) */
  BitstreamReadBits ( stream, 1, "copyright_flag", trace, CODE );
  bits ++;

  /* copyright_identifier(8) */
  BitstreamReadBits ( stream, 8, "copyright_identifier", trace, CODE);
  bits += 8;
  
  /* original_or_copy(1) */
  tmpvar = BitstreamReadBits ( stream, 1, "original_or_copy", trace, CODE);
  bits ++;

  /* reserved(7) */
  BitstreamReadBits ( stream, 7, "reserved", trace, CODE);
  bits += 7;
  
  /* marker_bit(1) */
  tmpvar = BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE);
  bits ++;

  /* copyright_number_1(20) */
  BitstreamReadBits ( stream, 20, "copyright_number_1", trace, CODE);
  bits += 20;
  
  /* marker_bit(1) */
  tmpvar = BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE);
  bits ++;

  /* copyright_number_2(22) */
  BitstreamReadBits ( stream, 22, "copyright_number_2", trace, CODE);
  bits += 22;
  
  /* marker_bit(1) */
  tmpvar = BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE);
  bits ++;

  /* copyright_number_3(22) */
  BitstreamReadBits ( stream, 22, "copyright_number_3", trace, CODE);
  bits += 22;
  
  /* marker_bit(1) */
  tmpvar = BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE);
  bits ++;

  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}

Int DecodeCameraParametersExtension_Studio ( Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;

  /* extension_start_code_identifier(3) */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != CAMERA_PARAMETERS_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: Camera Parameters Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  /* reserved(1) */
  BitstreamReadBits ( stream, 1, "reserved", trace, CODE );
  bits ++;

  /* camera_id(7) */
  BitstreamReadBits ( stream, 7, "camera_id", trace, CODE);
  bits += 7;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* height_of_image_device(22) */
  BitstreamReadBits ( stream, 22, "height_of_image_device", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* focal_length(22) */
  BitstreamReadBits ( stream, 22, "focal_length", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* f_number(22) */
  BitstreamReadBits ( stream, 22, "f_number", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* vertical_angle_of_view(22) */
  BitstreamReadBits ( stream, 22, "vertical_angle_of_view", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_position_x_upper(16) */
  BitstreamReadBits ( stream, 16, "camera_position_x_upper", trace, CODE);
  bits += 16;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_position_x_lower(16) */
  BitstreamReadBits ( stream, 16, "camera_position_x_lower", trace, CODE);
  bits += 16;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_position_y_upper(16) */
  BitstreamReadBits ( stream, 16, "camera_position_y_upper", trace, CODE);
  bits += 16;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_position_y_lower(16) */
  BitstreamReadBits ( stream, 16, "camera_position_y_lower", trace, CODE);
  bits += 16;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_position_z_upper(16) */
  BitstreamReadBits ( stream, 16, "camera_position_z_upper", trace, CODE);
  bits += 16;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_position_z_lower(16) */
  BitstreamReadBits ( stream, 16, "camera_position_z_lower", trace, CODE);
  bits += 16;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_direction_x(22) */
  BitstreamReadBits ( stream, 22, "camera_direction_x", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_direction_y(22) */
  BitstreamReadBits ( stream, 22, "camera_direction_y", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* camera_direction_z(22) */
  BitstreamReadBits ( stream, 22, "camera_direction_z", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* image_plane_vertical_x(22) */
  BitstreamReadBits ( stream, 22, "image_plane_vertical_x", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* image_plane_vertical_y(22) */
  BitstreamReadBits ( stream, 22, "image_plane_vertical_y", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* image_plane_vertical_z(22) */
  BitstreamReadBits ( stream, 22, "image_plane_vertical_z", trace, CODE);
  bits += 22;

  /* marker_bit(1) */
  BitstreamReadBits ( stream, 1, "marker_bit", trace, CODE );
  bits ++;

  /* reserved(32) */
  BitstreamReadBits ( stream, 32, "reserved", trace, CODE );
  bits += 32;

  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}

Int DecodeITUTExtension_Studio ( Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;

  /* extension_start_code_identifier(3) */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != ITUT_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: ITU-T Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  tmpvar = (Int) BitstreamShowBits ( stream, 24);
  while ( tmpvar != 1 ) {
    /* ITU-T_data(1) */
    BitstreamReadBits ( stream, 1, "ITU-T_data", trace, CODE);
    bits ++;

    tmpvar = (Int) BitstreamShowBits ( stream, 24);
  }

  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}

Int DecodeVLCCodeExtension_Studio ( Bitstream *stream, Trace *trace ) {

  Int tmpvar;
  Int bits = 0;

  Int i, j;
  
  Int load_intra_vlc_code = 0;
  Int load_inter_vlc_code = 0;

  Int code, length;
  
  Int conv_t[12] = { 0, 8, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25 };
  Int conv_g[22] = { 0, 8, 9, 10, 11, 12, 13, 1, 2, 3, 4, 5, 6, 17, 18, 19, 20, 21, 22, 23, 24, 25 };
  
  /* extension_start_code_identifier(3) */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "extension_start_code_identifier", trace, CODE );
  if ( tmpvar != VLC_CODE_EXTENSION_ID ) {
    fprintf ( stderr, "Syntax Error: VLC Code Extension ID shall come here\n" );
    exit ( 1 );
  }
  bits += 4;

  /* load_vlc_code(4) */
  tmpvar = (Int) BitstreamReadBits ( stream, 4, "load_vlc_code", trace, CODE);
  bits += 4;

  if ( ! tmpvar ) {
    fprintf ( stderr, "Syntax Error: load_vlc_code of '0000' is forbidden.\n" );
    exit ( 1 );
  } else if ( tmpvar == 1 ) {
    load_intra_vlc_code = 1;
    load_inter_vlc_code = 0;
  } else if ( tmpvar == 2 ) {
    load_intra_vlc_code = 0;
    load_inter_vlc_code = 1;
  } else if ( tmpvar == 3 ) {
    load_intra_vlc_code = 1;
    load_inter_vlc_code = 1;
  } else {
    fprintf ( stderr, "Syntax Error: load_vlc_code of a value over '0011' is forbidden.\n" );
    exit ( 1 );
  }

  if ( load_intra_vlc_code ) {
    for ( j=0; j<12; j++ ) {
      for ( i=0; i<22; i++) {
	/* intra_vlc_length(4) */
	length = (Int) BitstreamReadBits ( stream, 4, "intra_vlc_length", trace, CODE);
	bits += 4;
	/* intra_vlc_code(16) */
	code = (Int) BitstreamReadBits ( stream, 16, "intra_vlc_code", trace, CODE);
	bits += 16;
	
	length ++;
	code >>= 16 - length;
	PutStudioGroupVLCIntra_Studio ( code, length, conv_t[j], conv_g[i] );
      }
    }
  }
  
  if ( load_inter_vlc_code ) {
    for ( j=0; j<12; j++ ) {
      for ( i=0; i<22; i++) {
	/* inter_vlc_length(4) */
	length = (Int) BitstreamReadBits ( stream, 4, "inter_vlc_length", trace, CODE);
	bits += 4;
	/* inter_vlc_code(16) */
	code = (Int) BitstreamReadBits ( stream, 16, "inter_vlc_code", trace, CODE);
	bits += 16;
	
	length ++;
	code >>= 16 - length;
	PutStudioGroupVLCInter_Studio ( code, length, conv_t[j], conv_g[i] );
      }
    }
  }
  
  bits += BitstreamNextStartCode_Studio( stream, trace, CODE );

  return bits;
}





/* 01.04.01: Studio Profile */
