/******************************************************************************
 *                                                                          
 * This software module was originally developed by 
 *
 * Noel O'Connor (TELTEC IRELAND / ACTS-MoMuSys).                       
 *
 * and edited by  
 *
 * Aasmund Sandvand (Telenor / ACTS-MoMuSys).                           
 * J. Ignacio Ronda (UPM / ACTS-MoMuSys).                               
 * Cecile Dufour (LEP / ACTS-MoMuSys).                                  
 * Sylvie Jeannin (LEP / ACTS-MoMuSys).                                 
 * Minhua Zhou (HHI / ACTS-MoMuSys).                                    
 * Fernando Jaureguizar (UPM / ACTS-MoMuSys).                           
 * Luis Ducla-Soares (IST / ACTS-MoMuSys).                              
 * Martina Eckert (UPM / ACTS-MoMuSys).                                 
 * Noel Brady (TELTEC IRELAND / ACTS-MoMuSys).                          
 * Michael Wollborn (TUH / ACTS-MoMuSys).                               
 * Ulrike Pestel (TUH / ACTS-MoMuSys).
 * Bob Eifrig (NextLevel Systems)
 * Michael Frater (UNSW)
 * Hung-Ju Lee (Sarnoff Corporation)
 * Ji Heon Kweon (HYUNDAI)
 * Jong Deuk Kim (HYUNDAI)
 * Marc Mongenet (EPFL)
 * Sven Brandau, Guido Heising (HHI / ACTS-MoMuSys)
 * Seishi TAKAMURA (NTT)
 * Oki Electric Industry Co., Ltd. (contact: Shigeru Fukunaga)
 * Fujitsu Laboratories Ltd. (contact: Eishi Morimatsu)
 * Dae-Sung Cho (Samsung AIT)
 * Massimo Ravasi (EPFL)
 * Takefumi Nagumo (SONY corporation)
 * Osamu Sunohara (SONY)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4 
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free 
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC 
 * 14496-2) standard. 
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original 
 * developer of this software module and his/her company, the subsequent 
 * editors and their companies, and ISO/IEC have no liability for use of this 
 * software module or modifications thereof in an implementation. Copyright is 
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) standard conforming 
 * products. 
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own 
 * purpose, assign or donate the code to a third party and to inhibit third 
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works. 
 *
 * Copyright (c) 1997
 *
 *****************************************************************************/




/***********************************************************HeaderBegin*******
 *                                                                         
 * File:        vm_enc_main.c
 *
 * Author:      Noel O'Connor Teltec Irl.
 * Created:     11-03-96
 *                                                                         
 * Description:
 *
 * Notes:
 *
 * Modified:
 *     21.04.96 Robert Danielsen: Reformatted. New headers.
 *     08.05.96 Noel O'Connor: New data structures for control
 *              file and config file parameters
 *     13.05.96 Bruno Loret: PowerPC input arguments
 *     07.06.96 Robert Danielsen: Replaced old huffman stuff with new.
 *              The low-level bitstream file is now opened and closed here.
 *              The file pointer is an external variable.
 *     30.07.96 Noel O'Connor: Changed entire high level structure
 *              for compliance with VM 3.0. The encoder now runs on a single
 *              Video Object (VO) but for multiple Video Object Layers (VOLs)
 *              For the time being only BASE layer coding is supported.
 *     23.09.96 Cor Quist: Added the MPEG-2 dct transform (fdctref)
 *     23.09.96 Noel O'Connor: Changed entire high level structure to allow
 *              coding of multiple VOs.
 *     26.11.96 Noel O'Connor: Changed main coding loop to allow coding
 *              of multiple VOs with different frame rates (which may or
 *              may not be related).
 *     14.01.97 Luis Ducla-Soares: enabled byte alignment for VOL_START_CODE
 *              in function CloseBitstreams() by removing
 *              comment arount NextStartCode() function call
 *     02.02.97 Aasmund Sandvand: Moved deblocking filter out of loop
 *     07.02.97 J. Ignacio Ronda: Modifcations to implement VM5 rate control
 *     20.02.97 Cecile Dufour: including STATIC sprite
 *     14-MAR-97 Minhua Zhou         : added B-VOPs  
 *     21.03.97 Cecile Dufour: including ONLINE sprite
 *     20.03.97 Fernando Jaureguizar: Modifications in the main loop to 
 *              solve some problems with last skipped frames of the sequence.
 *     23.04.97 M.Wollborn: MPEG quantization completely added
 *     04.05.97 Luis Ducla-Soares: added data_partitioning flag to the bitstream.
 *     13.05.97 Minhua Zhou : added advanced_prediction_disable and
 *                            Group_of_VOPs (GOV) 
 *     14.05.97 F. Jaureguizar: Modification of number of bits of fcodes in
 *               BitstreamPutVOLs(). From 2 bits to 3 bits.
 *     04-JUL-97 Sylvie Jeannin : BUG VM7-970515-04 about TPS related memory problems
 *     07.07.97 Martina Eckert: Introduction of global rate control 
 *              functions and parameters
 *     18.07.97 Ulrike Pestel-Schiller: Spatial Scalability
 *     30.07.97 Fernando Jaureguizar: deleted some old_and_commented_out
 *              code concerning Rate Control (labelled as "RC2: comented out").
 *              Deleted RC_ExcludeIFrame() in CodeEnhVol().
 *              Added the Remove_Padding() for the U and V images also. (SpSc)
 *     04.08.97 Minhua Zhou: rename advanced_prediction_mode
 *     29.08.97 Osamu Sunohara(Sony): added new condition to select
 *              enhancement layer prediction type.
 *     05.09.97 Fernando Jaureguizar: added some FeeVop(). (SpSc-2)
 *     08.09.97 Cecile Dufour: static sprite_pieces encoded in the VOL header
 *     23.10.97 Minhua Zhou  : changed deblock_filter_disable to post_filter
 *                             _type
 *     04.01.98 Michael Frater: support for non-8-bit video
 *     16.01.97 Martina Eckert: Change reading of RC parameters from .ctl file.
 *                              Now they are extracted from one single string
 *
 *     20.01.98 Added the wavelet texture object layer code  
 *     07.05.98 Jong Deuk Kim (jdkim97@hei.co.kr, HYUNDAI) : modified GetVopBounded() 
 *                                        for field-based chrominance subsampling 
 *     03.06.98 Marc Mongenet (EPFL): added Complexity Estimation syntax support
 *     11.08.98 Sven Brandau: changes due to N2339
 *     15.02.99 U. Benzler : added quarter pel support
 *     03.03.99 Seishi TAKAMURA (NTT): added GMC coding, MPEG-4 version2 syntax
 *     16.08.99 Shigeru Fukunaga (Oki): added and modified modules for NEWPRED
 *     06.09.99 Eishi Morimatsu (Fujitsu Labs.): added DRC support
 *     26.10.99 Massimo Ravasi (EPFL): added Complexity Estimation syntax support
 *     11.11.99 Sven Brandau (HHI): added support for MAC
 *     11.11.99 U. Benzler (UoH): corrected output of SADCT flag to be done only for Version2
 *     02.02.00 Takefumi Nagumo (SONY): Bug fix for OBSS
 *     01.04.01 Osamu Sunohara (SONY): Functions for Studio Profile
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h> 
#include <sys/stat.h>
#include "vm_config.h"
#include "momusys.h"
#include "mom_vop.h"
#include "mom_bitstream_i.h"
#include "mom_access.h"
#include "io_generic.h"
#include "putvlc.h"
#include "text_code_mb.h"
#include "vm_vop_code.h"
#include "vm_compos.h"
#include "vm_stats.h"
#include "mom_putbits.h"
#include "newpred.h"	/** added for NEWPRED (Oki) 16-AUG-1999 **/
#include "vm_enc_main.h"
#include "vm_vop_bound.h"
#include "mom_vol.h"
#include "mom_vo.h"
#include "mot_util.h"
#include "zigzag.h"
#include "post_filter.h"
#include "rc.h"
#include "tm5rc.h"
#include "startcode.h"

#include "sait_rc.h"
#include "mot_padding.h"
#include "do_bgc.h"
#include "sprite_util.h"
#include "sprite_enc_piece.h"
/* for spatial scalability (UPS) */
#include "vm_midproc.h"
/* end of ssp (UPS) */

/* 01.02.99 HHI Schueuer */
#include "sadct.h"
#include "sadct_kaup.h"
#include "sadct_s_k.h"
/* end HHI */

#include "enhance_vop.h"
#include "bgc.h"

#include "alp_code_header.h"

#include "alp_code_mc.h" /* added for OBSS by Samsung AIT (1999-09-29) */

/* 1197-2 SONY */
#ifdef _DEBUG_SPSC3_
static Int up_cnt = 0;
#endif 
/* 1197-2 */

/* VTC Begin ------------------------------------------ */
#include "dataStruct.h"
#include "dwt.h"
#include "interface.h"
/* VTC End   ------------------------------------------ */

#include "meitca_wrapper.h"

#ifndef SEEK_END
#define SEEK_END 2
#endif 

/* inserted by Mitsubishi & SAMSUNG AIT */
#include "rc_mvo.h"
extern RC_MVO rc_mvo;
Int     FirstInterFrame = 1;
/* inserted by Mitsubishi & SAMSUNG AIT */


#if 0
Void
CodeEnhVol(Vol  *enh_layer,
           VolConfig *layer_config,
           Int  frame,
           Float time,
           Vol *prev_vol_store,
           Vol *rec_vol_store,
           Vol *base_vol_store,
           Vol *base_rec_vol_store,
           Vol *next_vol_store,
           Vol *next_rec_vol_store,
           Int TRB, Int TRD, 
           Int vo_id,    
           VOConfig *vo_config_list, /* UPM Global RC */
           Int rc_type,              /* UPM Global RC */
           Int rc_algorithm,              /* hjlee */
           Int *frame_type,          /* UPM Global RC */
           BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS]);

Void
BitstreamPutVOLs(VO *vo_list,BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
                        VOConfig *vo_config_list, Int rc_type);


Void
UpdateVolStore(Vop *update_vop,
                                        Int layer_id,
                                        Vol *vol_store);
VO *
InitVOStore(VO *list);

Vol *
InitVolStore(Vol *list);

Void
QuantAdjust (Int target_bitrate,
                                Int temp_ref,
                                Int end_time,
                                Int time_instant,
                                Int num_bits_used,
                                Int num_bits_prev,
                                Int *QP);

 
Void
InitBitstreams(VOConfig *list);

Void
CloseBitstreams(VOConfig *list);

Void
InitBitCounts(VOConfig *vo_cfg_list, 
                                Char *stats_file,
                                BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS]);

Void 
AverageBitCounts(BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
                                        VOConfig *vo_cfg_list);

Void
WriteCompositedImage(EncodeControl *enc_ctl,
                                                Int     frame,
                                                VO      *vo_store,
                                                Vop     *display_vop);


Float
GetNextCodedVO(VOConfig *list,
                        Int *vo_id,
                        Int *vol_id,
                        Int *finish);

#endif

#ifdef _STUDIO_CONF_5_
FILE *g_input, *g_output;
extern long g_pixel[1024], g_coeff[4096];
#endif _STUDIO_CONF_5_

#ifdef _STUDIO_CONF_6_
extern FILE *t6_fp_qcoef;
extern int  t6_v_qcoef[18000];
#endif _STUDIO_CONF_6_

/***********************************************************CommentBegin******
 *
 * -- main -- Main calling function of MoMuSys implementation of MPEG4 VM
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      11-03-96
 *
 * Purpose :            
 *      Main calling function of MoMuSys implementation of MPEG4 VM
 * 
 * Arguments in :       
 *      See file "README_ENCODER"
 *
 * Arguments in/out :   
 *      See file "README_ENCODER"
 *
 * Arguments out :      
 *      See file "README_ENCODER"
 *
 * Return values :      
 *      none
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      See file "README_ENCODER"
 *
 * See also :
 *      See file "README_ENCODER"
 *
 * Modified :
 *     07.06.96 Robert Danielsen: Replaced old huffman stuff with new.
 *              The low-level bitstream file is now opened and closed here.
 *              The file pointer is an external variable.
 *     30.07.96 Noel O'Connor : Changed entire high level structure
 *              for compliance with VM 3.0. The encoder now runs on a single
 *              Video Object (VO) but for multiple Video Object Layers (VOLs)
 *              For the time being only BASE layer coding is supported.
 *     23.09.96 Cor Quist: Added the MPEG-2 dct transform (fdctref)
 *     23.09.96 Noel O'Connor: Changed the control and configuration s/w
 *              to allow for coding of multiple VOs.
 *     07.02.97 J. Ignacio Ronda: Added VM5_RATE_CONTROL initialization.
 *              UpdateVolConfigNextCodingTime() with new parameters
 *     20.03.97 Fernando Jaureguizar: Modifications in the main loop to 
 *              solve some problems with last skipped frames of the sequence.
 *     08.09.97 Cecile Dufour: BitstreamPutVOLs and PrintSession place changed
 *     27.11.97 M. Eckert: Changes for independent frame type rate control
 *     14.06.99 K.Suehring (HHI): multi object bug fixed 
 *     16.08.99 Shigeru Fukunaga (Oki): added and modified modules for NEWPRED
 *     08.10.99 M. Eckert: Bug fix for RC Buffer update
 *
 ***********************************************************CommentEnd********/
 
Int
main(Int argc, Char *argv[])
{
  Char              control_file[100],
                    stats_file[100];
  EncodeControl     *enc_ctl;
  VOConfig          *vo_config_list,
                    *vo_config;
/* VTC Begin ------------------------------------------ */
  VTCConfig         *vtc_config_list,  /* VTC */
                    *vtc_config;
/* VTC End   ------------------------------------------ */

  VolConfig         *vol_config=NULL;
  BitCount          num_bits[MAX_NUM_VOS][MAX_NUM_VOLS];
  VO                *video_object_list = NULL,
                    *prev_vo_store,
                    *prev_rec_vo_store,
                    *next_vo_store,
                    *next_rec_vo_store,
/* 1197-2 SONY */
                    **B_org_vo_store=NULL,
/* 1197-2 */ 
                    **B_vo_store=NULL,
                    *vo,
                    *prev_vo,*next_vo,
                    *prev_rec_vo,*next_rec_vo;
  Vol               *vol,
                    *prev_vol,*next_vol,
                    *prev_rec_vol,*next_rec_vol,
                    *base_vol=NULL,*base_rec_vol=NULL; /*for spatial scal (UPS)*/   
  Vop               *display_vop;
 Float             time_instant,
                    start_time,
                    time_inc,
                    next_display_time;
  Int               finish = 0;
  Int               output_frame = 0,
                    num_out_frames,
                    frame,
                    vo_id,
                    vol_id; 
  Int               TRB,M=1,FIRST,current_frame_number=0;

  Float             end_time;           /* inserted by Mistubishi & SAMSUNG AIT */
  Int               tmp_frame, i;       /* inserted by Mistubishi & SAMSUNG AIT */
  Int               FirstIntraOrInter = 0;      /* inserted by Mistubishi & SAMSUNG AIT */

/* Global RC: */
  Int               rc_type, 
                    rc_algorithm, 
                    frame_type;
  Char             *rc_ctl_string;

  Float             old_time = 0.;
#ifndef NLSSTATS  
FILE              *fpstats_file;
#endif

  Float             ref_time_instant     = 0.0;
  Int               bgc_mode             = 0;
  Int               finish_base          = 0;
  Int               enhance_count        = 0;
  Int               temporal_scalability = 0;
  Int               enhancement_type     = 0;
  Int               FIRST_enhance        = 1;         
  Int               finish_enhance       = 0;
  Vop *base_vop_not_padding_for_bgc = NULL; /* base image for BGC which is not padding vop */
  Vop *enhance_vop[300];
  Vop *base_B_vop_TPS               = NULL;
  Vop *original_prev_base_vop       = NULL;
  
  /* for spatial scalability (UPS)*/
  Int               scalability;
  Int               spatial_scalability =0 ;    /*end of ssp (UPS)*/

  /** added for NEWPRED (Oki) 16-AUG-1999 **/
  RefVop	***ref_vop;			/* reference memory for NEWPRED */
  Int		np_vop_id = 0;			/* vop_id for NEWPRED */
  /** end of NEWPRED (Oki) 16-AUG-1999 **/

#ifdef _STUDIO_CONF_5_
  int iii;
  
  if ( (g_input = fopen("pixel_","w")) == NULL ) {
    fprintf(stderr, "cannot open\n");
    exit(-1);
  }
  if ( (g_output = fopen("coeff_","w")) == NULL ) {
    fprintf(stderr, "cannot open\n");
    exit(-1);
  }

  for ( iii=0; iii<1024; iii++)
    g_pixel[iii] = 0;
  for ( iii=0; iii<4096; iii++)
    g_coeff[iii] = 0;

#endif _STUDIO_CONF_5_

#ifdef _STUDIO_CONF_6_
  int iii;
  
  if ( (t6_fp_qcoef = fopen("6_qcoef_","w")) == NULL ) {
    fprintf(stderr, "cannot open\n");
    exit(-1);
  }
  for ( iii=0; iii<18000; iii++)
    t6_v_qcoef[iii] = 0;
#endif _STUDIO_CONF_6_
  
  /* Put text, if necessary, explaining how to run the encoder software */
#ifndef __POWERPC__
        if(argc < 2)
        {
                printf("usage : %s control file <statistics file (optional)>\n",argv[0]);
                exit(1);
        }

        /* Process input arguments */
        strcpy(control_file, argv[1]);
                if(argc == 3)
                        strcpy(stats_file, argv[2]);
                else
                        strcpy(stats_file, DEFAULT_STATS_FILE); 

#else

                strcpy(control_file, "weather.ctl");
                strcpy(stats_file,"weather_enc.dat");
#endif


        /* Read control file parameters */
  /* 01.04.01: Studio Profile */
        enc_ctl = ReadEncoderControlFile(control_file);
#if 0
        enc_ctl = ReadEncoderControlFile_Studio(control_file);
        /*enc_ctl = ReadEncoderControlFile(control_file);*/
#endif 0
  /* 01.04.01: Studio Profile */

        /* Open stats file file pointer */
        if((fpstats_file = fopen(stats_file,"w")) == NULL)
        {
                fprintf(stdout,"ERROR(main) : Cannot open %s\n",stats_file);
                exit(1);
        }
 

        /* Read config files for VOs */
  /* 01.04.01: Studio Profile */
#if 0
         if(GetEncodeControlStudioProfile_st(enc_ctl))
	  vo_config_list = ReadConfigFiles_Studio(enc_ctl);
        else
#endif 0
 /* 01.04.01: Studio Profile */
        vo_config_list = ReadConfigFiles(enc_ctl); 

/* VTC Begin ------------------------------------------ */
       /* Read VTC config files for VOs */
        vtc_config_list = ReadVTCConfigFiles(enc_ctl); 
        vtc_config = vtc_config_list;  
        while (vtc_config != NULL) {
          
          VTC_Encode(vtc_config);

          vtc_config = GetVTCConfigNext (vtc_config);
        }
/* VTC End ------------------------------------------ */
 

        if (vo_config_list== NULL)
          exit(0);
        if (vo_config_list->layers == NULL)
          exit(0);


        /* Calculate some control parameters based on data read */
        InitControlParams(vo_config_list,enc_ctl);
        /* Put some text for user (to display and stats file) */
 /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl)) {
	  PrintEncodeControl_Studio(enc_ctl,stdout);
	  PrintEncodeControl_Studio(enc_ctl,fpstats_file);
	} else {
 /* 01.04.01: Studio Profile */
        PrintEncodeControl(enc_ctl,stdout);
        PrintEncodeControl(enc_ctl,fpstats_file);
        }
        fprintf(stdout,"Bit statistic file - %s\n\n",stats_file);

        /* Put some text for user (to display and stats file) */
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl)) {
	  PrintVOConfigList_Studio(vo_config_list,stdout);
	  PrintVOConfigList_Studio(vo_config_list,fpstats_file);
	} else {
  /* 01.04.01: Studio Profile */
        PrintVOConfigList(vo_config_list,stdout);
        PrintVOConfigList(vo_config_list,fpstats_file); 
        }

      /* Set up list for input VOPs for each VO using control and config info. */               
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl))
	  video_object_list = SetUpVOList_Studio(vo_config_list);
        else
  /* 01.04.01: Studio Profile */
        video_object_list = SetUpVOList(vo_config_list);

        /* Close stats file (will write to it incrmentally from now on */
  /* 01.04.01: Studio Profile */
         if(GetEncodeControlStudioProfile_st(enc_ctl))/*AM2PDAM*/
	  ;
        else {
	  ;
  /* 01.04.01: Studio Profile */
#ifndef NLSSTATS
        fclose(fpstats_file);
#endif
	}
  
        /* Initialise the VO stores with the VO list. */
        /* A VO store has a node for each VO. These nodes
                 then contain pointers to stores for the various VOLs */
  /* 01.04.01: Studio Profile */
       if(GetEncodeControlStudioProfile_st(enc_ctl)) {
	 prev_vo_store = InitVOStore_Studio(video_object_list);
	 prev_rec_vo_store = InitVOStore_Studio(video_object_list);
	 
	 next_vo_store = InitVOStore_Studio(video_object_list);
	 next_rec_vo_store = InitVOStore_Studio(video_object_list);
       } else {
  /* 01.04.01: Studio Profile */
        prev_vo_store = InitVOStore(video_object_list);
        prev_rec_vo_store = InitVOStore(video_object_list);
        
       /* Minhua Zhou: store next original and reconstructed VOs for B-VOPs */
        next_vo_store = InitVOStore(video_object_list);
        next_rec_vo_store = InitVOStore(video_object_list);
      }
 
  /*
   *
   * Get ready to start a session 
   *
   */


	/* 01.02.99 HHI Schueuer */
	Initialisations_for_SADCT();
	Initialisations_for_SADCT_s_k();	
	Initialisations_for_SADCT_Kaup();
	/* end HHI */

        /* Initialize MPEG-2 DCT-transform (fdctref) */


        /* Allocate memory for the display vop */
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl)) {
	  display_vop = AllocFrameVop_Studio();
	  AllocVopChannels_Studio(display_vop, FRAME_VOP, 0,
				  GetEncodeControlDispSizeX(enc_ctl),
				  GetEncodeControlDispSizeY(enc_ctl),
				  GetEncodeControlDispChromaFormat_st(enc_ctl));
	} else
  /* 01.04.01: Studio Profile */
        display_vop = AllocVop(GetEncodeControlDispSizeX(enc_ctl),
                               GetEncodeControlDispSizeY(enc_ctl),
                               0);
        PutVopBitsPerPixel(GetEncodeControlDispBitsPerPixel(enc_ctl),display_vop);
        
        /* Open and initialise all bitstream files required */
        InitBitstreams(vo_config_list);
        
        /* Initialise bit counting structures */
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl))
	  InitBitCounts_Studio(vo_config_list,stats_file,num_bits);
	else 
  /* 01.04.01: Studio Profile */  
        InitBitCounts(vo_config_list,stats_file,num_bits);

        /* Set up coding loop indices */
        start_time = GetEncodeControlStartTime(enc_ctl);
        time_instant = start_time;
        next_display_time = start_time;
        time_inc = GetEncodeControlTimeInc(enc_ctl);

        /* Initialise VM5_RATE_CONTROL */
        rc_type = GetEncodeControlRCType(enc_ctl); 
        /* 16.01. Read all control parameters in one string */
        rc_ctl_string = GetEncodeControlRCParam(enc_ctl);

        if (rc_type == VM5_RATE_CONTROL) {
          RC_init_global(vo_config_list, rc_ctl_string, &rc_algorithm);
          if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED) {
              RCQ2_MVO_InitGlobal(vo_config_list, enc_ctl);
          }
        }
        else if (rc_type >= TM5_RATE_CONTROL) {
            for (vo_config = vo_config_list; vo_config != NULL; vo_config = vo_config->pnext) {
                for (vol_config = vo_config->layers; vol_config != NULL; vol_config = vol_config->pnext) {
                    vol_config->rcdata = tm5rc_init_seq(vol_config, rc_type);
                }
            }
        }

        scalability = CheckScalability(vo_config_list);
        if(scalability==1){
          if(CheckScalType(vo_config_list) == 0 || CheckScalType(vo_config_list) == 2)
            temporal_scalability = 3;
          else if(CheckScalType(vo_config_list) == 3)
            temporal_scalability = 1;
          else if(CheckScalType(vo_config_list) == 4)
            temporal_scalability = 5;
          else if(CheckScalType(vo_config_list) == 1 || CheckScalType(vo_config_list) == 2)
            spatial_scalability =1;
          
          if(temporal_scalability != 0){
            enhancement_type    = CheckEnhancementType(vo_config_list);
            if(enhancement_type != 2){
              fprintf(stderr,"BGC(ON)\n");
                bgc_mode = 1;
            }
            else{
              fprintf(stderr,"BGC(OFF)\n");
              enhancement_type = 1;
            }
          }
        }

        /* Write VOL headers to bitstream */
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl))
	  BitstreamPutVOLs_Studio(video_object_list,num_bits, vo_config_list, rc_type);
        else
  /* 01.04.01: Studio Profile */
        BitstreamPutVOLs(video_object_list,num_bits, vo_config_list, rc_type);

        /* Print Session start statistics to display and stats file */
  /* 01.04.01: Studio Profile */  
        if(GetEncodeControlStudioProfile_st(enc_ctl)) {/*AM2PDAM*/
	  DoPrintSessionStartStat_Studio(num_bits, video_object_list, fpstats_file);
	  DoPrintSessionStartStat_Studio(num_bits, video_object_list, stdout);
	} else {
  /* 01.04.01: Studio Profile */
#ifdef NLSSTATS
        DoPrintSessionStartStat(num_bits, video_object_list, fpstats_file);
        DoPrintSessionStartStat(num_bits, video_object_list, stdout);
#else
        PrintSessionStartStat(num_bits,video_object_list,STATS_FILE);
        PrintSessionStartStat(num_bits,video_object_list,DISPLAY);
#endif
      }
        FIRST = 1;
          if (temporal_scalability != 0){
              FILE *fp_TMP;
              fprintf(stderr,"temporal scalability ON\n");

              base_vop_not_padding_for_bgc = AllocVop(GetEncodeControlDispSizeX(enc_ctl),
                                                      GetEncodeControlDispSizeY(enc_ctl),
                                                      0);
              PutVopPrevTemp(NULL, base_vop_not_padding_for_bgc);
              PutVopNextTemp(NULL, base_vop_not_padding_for_bgc);
              {
                 Int i;
                 for(i = 0; i < 299; i++)
                     enhance_vop[i] = NULL;
              }
              
              if(NULL == (fp_TMP = fopen(GetEncodeControlDispYFile(enc_ctl),"w"))){
                fprintf(stderr,"cannot open %s\n",GetEncodeControlDispYFile(enc_ctl));
                exit(1);
              }
              fclose(fp_TMP);
              if(NULL == (fp_TMP = fopen(GetEncodeControlDispUFile(enc_ctl),"w"))){
                fprintf(stderr,"cannot open %s\n",GetEncodeControlDispUFile(enc_ctl));
                exit(1);
              }
              fclose(fp_TMP);
              if(NULL == (fp_TMP = fopen(GetEncodeControlDispVFile(enc_ctl),"w"))){
                fprintf(stderr,"cannot open %s\n",GetEncodeControlDispVFile(enc_ctl));
                exit(1);
              }
              fclose(fp_TMP);
              
   /***************************************
   <<<   temporal_scalability usage   >>>
   ***************************************
    temporal_scalability = 1  : Enhance vop is B_VOP_TYPE(1)
    temporal_scalability = 2  : Enhance vop is B_VOP_TYPE(2)
    temporal_scalability = 3  : Enhance vop is I_VOP or P_VOP
    temporal_scalability = 4  : Enhance vop is P_VOP in B_VOP_TYPE(1)
    temporal_scalability = 5  : Enhance vop is P_VOP in B_VOP_TYPE(2)
    
    ==================================================
    prediction_type = P_VOP(temporal_scalability = 3 )
    ==================================================
       temporal_scalability = 3,4 -> RefSelCode = 0 : forward (from E frame)
                                     RefSelCode = 1 : forward (from B frame) 
                                     RefSelCode = 2 : backward(from E frame)
    =======================
    prediction_type = B_VOP
    =======================
       temporal_scalability = 1   -> RefSelCode = 3 : B_TYPE(1)
       temporal_scalability = 2   -> RefSelCode = 1 : B_TYPE(2)
  **********************************************************************/
              
              fprintf(stderr,"temporal_scalability MODE : ");
              if(temporal_scalability == 3)
                  fprintf(stderr,"case0 : ");
              else if(temporal_scalability == 1)
                  fprintf(stderr,"case1 : ");
              else if(temporal_scalability == 5)
                  fprintf(stderr,"case2 : ");
              else{
                  fprintf(stderr,"\n");
                  fprintf(stderr,"B_MODE for temporal_scalability is\n");
                  fprintf(stderr,"       3 : Enhance vop is B_VOP_TYPE(1)\n"); 
                  fprintf(stderr,"       4 : Enhance vop is B_VOP_TYPE(2)\n"); 
                  fprintf(stderr,"       0 : Enhance vop is I_VOP or P_VOP \n");
                  exit(-1);
              } 
              if(enhancement_type == 1)
                  fprintf(stderr,"Type 1\n");
              else{
                  if(CheckArbitraryShape(vo_config_list))
                      fprintf(stderr,"Type 2\n");
                  else
                      fprintf(stderr,"Type 0\n");
              }
                  
              base_B_vop_TPS = NULL; 
        }
  
        /** added for NEWPRED (Oki) 16-AUG-1999 **/
        if(GetVolConfigNewpredEnable(vo_config_list->layers)){
          /* initialize NEWPRED mode */
          InitNewpred(vo_config_list, &ref_vop, GetEncodeControlNumVOs(enc_ctl));
        }
        /** end of NEWPRED (Oki) 16-AUG-1999 **/

       while(TRUE){
         
                 if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED) {
                    RCQ2_MVO_UpdateVOCount(vo_id);
                 }

                if(temporal_scalability == 0) 
                    time_instant = GetNextCodedVO(vo_config_list,&vo_id,&vol_id,&finish);
                 else
                    time_instant = GetNextCodedVO_TPS(vo_config_list,
                                                      &vo_id,  &vol_id,
                                                      &finish, &finish_base, &finish_enhance,
                                                      temporal_scalability);
                        

                /* if use P_VOP in B_VOP_TYPE(1) : TPS */
                if(temporal_scalability == 1 && finish_base == FINISHED)
                    temporal_scalability = 4; /* B_TYPE(1) : E frame's predictionmode change (B_VOP -> P_VOP) */

                 if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED) {
                   ;
                 }
                 else {
                   if (rc_type == VM5_RATE_CONTROL && 
                       rc_algorithm != RC_ALGORITHM_Q2)
                     
                     if(time_instant != old_time) 
                     {
                       Int skip_frame;
                       old_time = time_instant;
                       
                       /* Update Global RC Buffer: */
                       
                        if (!FIRST)
                          RC_UpdateBuffer(vo_config_list, num_bits);
#ifndef _RC_NO_SKIPPING_
                       do
                       {
                         skip_frame = RC_GlobalFrameSkipping(vo_config_list);
                         if (skip_frame)
                         {
                           time_instant = GetNextCodedVO(vo_config_list,&vo_id,&vol_id,&finish);
                         }
                       }while(skip_frame);
#endif          
                     }
                 }
                 
               /* Minhua Zhou: Compositor is changed for B-VOPs */
             
                if ((time_instant > next_display_time)||(finish==FINISHED))
                        {
                        num_out_frames = (Int)(RoundTime((time_instant - next_display_time)/
                                time_inc/(Float)M)+0.001);
                        if (!FIRST) 
                          for (TRB=1;TRB<M;TRB++)
                           for(frame = 0; frame < num_out_frames; frame++) 
                                {
                                if(temporal_scalability == 0)
				  /* 01.04.01: Studio Profile: BUGFIX */
				  if(! GetEncodeControlStudioProfile_st(enc_ctl))
				  WriteCompositedImage(enc_ctl,output_frame,B_vo_store[TRB-1],display_vop);
                                output_frame++;
                            }
                       for(frame = 0; frame < num_out_frames; frame++)
                                {
                                /* Put some text for user */
                                /* fprintf(stdout,
                                                "Writing composited frame for : %.3f <= time < %.3f\n",
                                                (float)next_display_time,
                                                RoundTime(next_display_time + time_inc)); */
                                if(temporal_scalability == 0)
				  /* 01.04.01: Studio Profile: BUGFIX */
				  if(! GetEncodeControlStudioProfile_st(enc_ctl))
				  WriteCompositedImage(enc_ctl,output_frame,next_rec_vo_store,display_vop);
                                output_frame++;
                             }
                 next_display_time = RoundTime(next_display_time + (Float)(M*num_out_frames)*time_inc);
                 /* put the next VOs to the previous VOs */
                    
                 if (FIRST) FIRST=0;
                }
                if(finish == FINISHED)
                        break;
                /* Put some text for user (display and stats file) */
  /* 01.04.01: Studio Profile */
		if(GetEncodeControlStudioProfile_st(enc_ctl)) {/*AM2PDAM*/
		  DoPrintTimeStartStat(time_instant,fpstats_file);
		  DoPrintTimeStartStat(time_instant,stdout);
		} else {
		  ;
  /* 01.04.01: Studio Profile */
#ifndef NLSSTATS
                PrintTimeStartStat(time_instant,stats_file,DISPLAY);
                PrintTimeStartStat(time_instant,stats_file,STATS_FILE);
#endif
		}

                vo = GetVOById(video_object_list,vo_id);
                prev_vo = GetVOById(prev_vo_store,vo_id);
                prev_rec_vo = GetVOById(prev_rec_vo_store,vo_id);
                
                /* Minhua Zhou: for B-VOPs */
                next_vo = GetVOById(next_vo_store,vo_id);
                next_rec_vo = GetVOById(next_rec_vo_store,vo_id);

                vo_config = GetVOConfigById(vo_config_list,vo_id);
                vol_config = GetVolConfigById(GetVOConfigLayers(vo_config),vol_id);

                /* Put some text for user */
  /* 01.04.01: Studio Profile */
		if(GetEncodeControlStudioProfile_st(enc_ctl)) {/*AM2PDAM*/
		  DoPrintVOTimeStartStat_Studio(GetVOId(vo),fpstats_file);
		  DoPrintVOTimeStartStat_Studio(GetVOId(vo),stdout);
		} else {
		  ;
  /* 01.04.01: Studio Profile */
#ifndef NLSSTATS
                PrintVOTimeStartStat(num_bits,vo,DISPLAY);
                PrintVOTimeStartStat(num_bits,vo,STATS_FILE);
#endif
		}

                vol = GetVolById(GetVOLayers(vo),vol_id);
                prev_vol = GetVolById(GetVOLayers(prev_vo),vol_id);
                prev_rec_vol = GetVolById(GetVOLayers(prev_rec_vo),vol_id);

                /* Minhua Zhou: for B-VOPs */
                next_vol = GetVolById(GetVOLayers(next_vo),vol_id);
                next_rec_vol = GetVolById(GetVOLayers(next_rec_vo),vol_id);
                

        /*  for spatial scalability (UPS)*/     
                
                /* modified 1197-2 SONY */
#ifdef _DEBUG_SPSC3_
                fprintf(stderr,"frame/skip = %d\n", vol_config->frame/GetVolConfigFrameSkip(vol_config));
#endif

                if( GetVolConfigScalability(vol_config)==1 && GetVolConfigScalType(vol_config)==1 )
                  {
                    int frame_skip_ratio = vol_config->frame / GetVolConfigFrameSkip(vol_config);
                    int ref_M = GetVolConfigM(GetVolConfigById(GetVOConfigLayers(vo_config),vol->ref_ident));
                    
                    if( (((frame_skip_ratio-1)%ref_M)==(ref_M-1))||(vol_config->frame==0))
                      {
#ifdef _DEBUG_SPSC3_
                        fprintf(stderr,"UP=next\n");
#endif

                        base_vol     = GetVolById(GetVOLayers(next_vo),vol->ref_ident);
                        base_rec_vol = GetVolById(GetVOLayers(next_rec_vo),vol->ref_ident);
                      }
                    else
                      {
#ifdef _DEBUG_SPSC3_
                        fprintf(stderr,"UP=B_vo");
                        fprintf(stderr, "num== %d\n",(frame_skip_ratio-1)%ref_M);
#endif


                        base_vol     = GetVolById(GetVOLayers( B_org_vo_store[(frame_skip_ratio-1)%ref_M]),vol->ref_ident);
                        base_rec_vol = GetVolById(GetVOLayers(     B_vo_store[(frame_skip_ratio-1)%ref_M]),vol->ref_ident);
                    }
                  }
/* 1197-2 */

                
                /*Minhua Zhou: replace the prevs_vol with the next_vol 
                  for P-VOP coding */
                
                if(temporal_scalability == 0 || !GetVolScalability(vol)){
  /* 01.04.01: Studio Profile */
		  if(GetEncodeControlStudioProfile_st(enc_ctl)) {
		    if(GetVolVop(next_vol)!=NULL && GetVopCoded(GetVolVop(next_vol))==1)
		      UpdateVolStore_Studio(GetVolVop(next_vol),     vol_id, prev_vol);
		    if(GetVolVop(next_rec_vol)!=NULL && GetVopCoded(GetVolVop(next_rec_vol))==1)
		      UpdateVolStore_Studio(GetVolVop(next_rec_vol), vol_id, prev_rec_vol);
		  } else {
  /* 01.04.01: Studio Profile */
                  /* MW 30-NOV-1998 */
                  if(GetVolVop(next_vol)!=NULL && GetVopCoded(GetVolVop(next_vol))==1)
                    UpdateVolStore(GetVolVop(next_vol),     vol_id, prev_vol);
                  /* MW 30-NOV-1998 */
                  if(GetVolVop(next_rec_vol)!=NULL && GetVopCoded(GetVolVop(next_rec_vol))==1)
                    UpdateVolStore(GetVolVop(next_rec_vol), vol_id, prev_rec_vol);
		  }
                    
                    /* if use B_VOP : TPS */
                    if(!(temporal_scalability == 0 || temporal_scalability == 3) &&
                        GetVolVop(prev_vol) != NULL){
                        if(original_prev_base_vop != NULL){
                            FreeVop(original_prev_base_vop);
                            original_prev_base_vop = NULL;
                        }
                        original_prev_base_vop = CloneVop_TMP(GetVolVop(prev_vol));
                    }
                }
                else{
                    if(temporal_scalability == 5 ||
                       temporal_scalability == 2 ||
                       !SkipVop(vol, vo_config, temporal_scalability)){
                        UpdateVolStore(GetVolVop(next_vol),     vol_id, prev_vol);
                        UpdateVolStore(GetVolVop(next_rec_vol), vol_id, prev_rec_vol);
                    }
                    else if(current_frame_number != 0){
                        NotUpdateVolStore(GetVolVop(next_vol),     vol_id, prev_vol);
                        NotUpdateVolStore(GetVolVop(next_rec_vol), vol_id, prev_rec_vol);
                    }
                }
                
                next_vol->pvop = NULL;
                next_rec_vol->pvop = NULL;
                
                current_frame_number=GetVolConfigFrame(vol_config);

                if((temporal_scalability == 5 || temporal_scalability == 2) &&
                   GetVolConfigScalability( vol_config ))
                    ChangeBaseImage_BGC(vo_config, base_vop_not_padding_for_bgc,
                                        &base_B_vop_TPS, vo_id, vol_id,
                                        current_frame_number,
                                        &original_prev_base_vop,
                                        finish_base);
                
                M = GetVolConfigM(vol_config);

/* modified 1197-2 SONY */
                if ((M-1)&&(B_vo_store==NULL)&&(B_org_vo_store==NULL)) {
                  B_org_vo_store = (VO**)calloc(M-1,sizeof(VO*));
                  B_vo_store     = (VO**)calloc(M-1,sizeof(VO*));
                  for (TRB=1;TRB<M;TRB++) {
  /* 01.04.01: Studio Profile */
		    if(GetEncodeControlStudioProfile_st(enc_ctl)) {
		      B_org_vo_store[TRB-1] = InitVOStore_Studio(video_object_list);
		      B_vo_store[TRB-1]     = InitVOStore_Studio(video_object_list);
		    } else {
  /* 01.04.01: Studio Profile */
 
                    B_org_vo_store[TRB-1] = InitVOStore(video_object_list);
                    B_vo_store[TRB-1]     = InitVOStore(video_object_list);
		    }
                  }
                }
/*              if ((M-1)&&(B_vo_store==NULL)) {
                  B_vo_store =(VO**)calloc(M-1,sizeof(VO*));
                  for (TRB=1;TRB<M;TRB++)
                    B_vo_store[TRB-1] = InitVOStore(video_object_list);
                    }
*/
/* 1197-2 */           

            
            if ( !GetVolConfigScalability( vol_config ) ){ 

                if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED) {

#if 1
                  if (RCQ2_MVO_CHECK() >= RCQ2_MVO_HVS) {
                    if (FirstIntraOrInter == 0) 
                      SetFirstIntraQuantizer(vol, vol_config, 
                                             current_frame_number, 0, rc_mvo.numVOs); 
                  }
#endif
                  FirstIntraOrInter = 1;
                  if (time_instant > (GetVolConfigTimeInc(vol_config))) 
                    FirstInterFrame = 0;        
  
                }

		/** added for NEWPRED (Oki) 16-AUG-1999 **/
		if(GetVolConfigNewpredEnable(vol_config)){
		  np_vop_id++;
		  /* set of reference vop */
		  SetRefVopNewpred(prev_vol->pvop,	/* previous original VOP */
				   prev_rec_vol->pvop,	/* previous reconstructed VOP */
				   ref_vop,		/* memories of reference VOPs */
				   vol_config,		/* config structure */
				   vo_id,		/* VO ID */
				   np_vop_id);		/* VOP ID */
		}
		/** end of NEWPRED (Oki) 16-AUG-1999 **/

  /* 01.04.01: Studio Profile */
		if(GetVolConfigStudioProfile_st(vol_config))
		  CodeBaseVol_Studio(vol,
				     vol_config,
				     current_frame_number, 
				     time_instant,
				     prev_vol,
				     prev_rec_vol,
				     next_vol,
				     next_rec_vol,
				     0,M,                                                
				     vo_id,
				     vo_config_list,
				     rc_type,
				     rc_algorithm,
				     &frame_type,
				     num_bits,      
				     NULL,
				     NULL,
				     base_vop_not_padding_for_bgc,
				     temporal_scalability,
				     enhancement_type,
				     NULL);
		else
  /* 01.04.01: Studio Profile */
                CodeBaseVol(vol,
                            vol_config,
                            current_frame_number, 
                            time_instant,
                            prev_vol,
                            prev_rec_vol,
                            next_vol,
                            next_rec_vol,
                            0,M,                                                
                            vo_id,
                            vo_config_list, /* UPM Global RC */
                            rc_type,        /* UPM Global RC */
                            rc_algorithm,        /* hjlee */
                            &frame_type,
                            num_bits,      
/* 1197-2 SONY */
                            NULL,
/* 1197-2 */                        
                            NULL,
                            base_vop_not_padding_for_bgc,
                            temporal_scalability,
                            enhancement_type,
                            NULL,
			    ref_vop,	/** added for NEWPRED (Oki) 16-AUG-1999 **/
			    np_vop_id);	/** added for NEWPRED (Oki) 16-AUG-1999 **/
                if(temporal_scalability != 0) ref_time_instant = time_instant;
            }
            /* if use I_VOP or P_VOP : TPS */
            else if(finish_enhance != FINISHED &&
                    (temporal_scalability == 3 ||
                     temporal_scalability == 4 ||
                     temporal_scalability == 5)){
               CodeEnhancementVol(vol,
                                  vo_config,     
                                  current_frame_number, 
                                  time_instant,
                                  prev_vo,       
                                  prev_rec_vo,        
                                  next_vo,       
                                  next_rec_vo,
                                  0,M,            /* M =2 */
                                  vo_id,
                                  vo_config_list, /* UPM Global RC */
                                  rc_type,        /* UPM Global RC */
                                  rc_algorithm,        /* hjlee */
                                  &frame_type,
                                  num_bits,
                                  NULL,
                                  enc_ctl,
                                  base_vop_not_padding_for_bgc,
                                  enhance_vop,
                                  &enhance_count,
                                  bgc_mode,
                                  temporal_scalability,
                                  original_prev_base_vop,
                                  finish_base);
              if(temporal_scalability == 5 && GetVolVop(next_rec_vol) != NULL)
                   FIRST_enhance = 0;
            }
            else if(spatial_scalability) /*for ssp (UPS)*/
              CodeEnhVol(vol,
                         vol_config,
                         current_frame_number, 
                         time_instant,
                         prev_vol,
                         prev_rec_vol,
                         base_vol,
                         base_rec_vol,
                         next_vol,
                         next_rec_vol,
                         0,1,                                                
                         vo_id,
                         vo_config_list, /* UPM Global RC */
                         rc_type,        /* UPM Global RC */
                         rc_algorithm,        /* hjlee */
                         &frame_type,
                         num_bits); /*end of ssp (UPS)*/           

                if (!FIRST)
                for (TRB=1;TRB<M;TRB++) {
#ifndef NLSSTATS
                /* Put some text for user (display and stats file) */
                PrintTimeStartStat(time_instant-(M-TRB)*GetVolConfigTimeInc(vol_config),
                                   stats_file,DISPLAY);
                PrintTimeStartStat(time_instant-(M-TRB)*GetVolConfigTimeInc(vol_config),
                                       stats_file,STATS_FILE);
                fprintf(stdout,"Coding of B-VOP\n"); 
#endif

                if ( !GetVolConfigScalability( vol_config ) ) {
    CodeBaseVol(vol,
                vol_config,
                current_frame_number-
                (M-TRB)*GetVolConfigFrameSkip(vol_config),                                     
                time_instant-(M-TRB)*GetVolConfigTimeInc(vol_config),
                prev_vol,
                prev_rec_vol,
                next_vol,                      
                next_rec_vol,
                TRB,M,                                                
                vo_id,
                vo_config_list, /* UPM Global RC */
                rc_type,        /* UPM Global RC */
                rc_algorithm,        /* hjlee */
                &frame_type,
                num_bits,  
/* 1197-2 SONY */
    /*            GetVolById(GetVOLayers(B_org_vo_store[TRB-1]),vol_id),  */
                GetVolById(GetVOLayers(GetVOById(B_org_vo_store[TRB-1],vo_id)),vol_id), /* 14.06.99 K.Suehring (HHI): multi object bug fixed */ 
/* 1197-2 */
   /*             GetVolById(GetVOLayers(B_vo_store[TRB-1]),vol_id),  */
                GetVolById(GetVOLayers(GetVOById(B_vo_store[TRB-1],vo_id)),vol_id), /* 14.06.99 K.Suehring (HHI): multi object bug fixed */ 
                base_vop_not_padding_for_bgc,
                temporal_scalability,
                enhancement_type,
                &base_B_vop_TPS,
		ref_vop,	/** added for NEWPRED (Oki) 16-AUG-1999 **/
		np_vop_id);	/** added for NEWPRED (Oki) 16-AUG-1999 **/
                        if(temporal_scalability)
                            ref_time_instant = time_instant-(M-TRB)*GetVolConfigTimeInc(vol_config);
                }
                else
                  fprintf(stdout,"\nERROR on .cfg with Scalabilty and ScalType settings\n");
            }
                
                /* if use B_VOP : TPS */
                if(finish_enhance != FINISHED &&
                   GetVolConfigScalability( vol_config ) &&
                   ( (temporal_scalability == 1 && finish_base != FINISHED) ||
                     temporal_scalability == 2 )
                ){
                    M = GetVolConfigFrameSkip(GetVolConfigById(GetVOConfigLayers(vo_config),
                        GetVolConfigRefId(vol_config))) / GetVolConfigFrameSkip( vol_config );
                    if(temporal_scalability == 2 && M != 2){
                        fprintf(stderr,"\n-Error- now TPS_B(case2)is M = %d\n",M);
                        exit(-1);
                    }
                    if ((M-1)&&(B_vo_store==NULL)) {
                        B_vo_store =(VO**)calloc(M-1,sizeof(VO*));
                        for (TRB=1;TRB<M;TRB++)
                            B_vo_store[TRB-1] = InitVOStore(video_object_list);
                    }
                    for (TRB = 1; TRB < M; TRB++){
                        CodeEnhancementVol(vol,
                                           vo_config,     
                                           current_frame_number ,
                                           time_instant,
                                           prev_vo,       
                                           prev_rec_vo,        
                                           next_vo,       
                                           next_rec_vo,
                                           TRB,M,
                                           vo_id,
                                           vo_config_list, 
                                           rc_type,
                                           rc_algorithm, /* hjlee */
                                           &frame_type,
                                           num_bits,
                                           GetVolById(GetVOLayers(B_vo_store[TRB-1]),vol_id),
                                           enc_ctl,
                                           base_vop_not_padding_for_bgc,
                                           enhance_vop,
                                           &enhance_count,
                                           bgc_mode,
                                           temporal_scalability,
                                           original_prev_base_vop,
                                           finish_base);
                        if(TRB < M - 1){
                            current_frame_number =  GetVolConfigFrame(vol_config);
                            time_instant         += GetVolConfigTimeInc(vol_config);
                            PutVolConfigNextCodingTime((Float)time_instant, vol_config);
                        }
                    }
                }/* B_VOP end : TPS */
                
                if(temporal_scalability == 5 && FIRST_enhance == 0)
                    temporal_scalability = 2;
                
                if (temporal_scalability != 0)
                    BGCProcess(time_instant, ref_time_instant, 
                               vo_id,        vol_id,
                               vo_config,
                               next_rec_vo,
                               prev_rec_vo_store,
                               enc_ctl,
                               base_vop_not_padding_for_bgc,
                               enhance_vop,
                               &enhance_count,
                               bgc_mode,
                               temporal_scalability,
                               base_B_vop_TPS,
                               finish_base);
                if (temporal_scalability != 0 && GetVolConfigScalability(vol_config))
                    UpdateVolConfigNextCodingTimeEnhancement(vo_config, vol_id, temporal_scalability);
                /* Update the next coding time for this VOL */

                if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED)
                {
                    end_time = GetVolConfigEndTime(vol_config);
                    if (((time_instant - start_time) != 0) && 
                        (time_instant < end_time)) 
                    {
                        /***********************/
                        /* POST-ENCODING STAGE */
                        /***********************/
                      /* bug fix: avetro 110399 */
                      /* if(vo_id == rc_mvo.numVOs-1) */
                      if(vo_id == rc_mvo.numVOs+rc_mvo.first_vo-1)
                      {
                        /* Determine second order complexities for each VO */
                        RCQ2_MVO_Update2OrderModel(num_bits, rc_mvo.numVOs);
                        RCQ2_MVO_SkipControl(1, rc_mvo.numVOs);
                        RCQ2_MVO_ResetBitCount(num_bits, rc_mvo.numVOs);
                        
                        /* execute any necessary frame skipping */
                        /* bug fix: avetro 110399 */
                        /* for(i=0; i<rc_mvo.numVOs; i++) */
                        for(i=rc_mvo.first_vo; i<rc_mvo.numVOs+rc_mvo.first_vo; i++)
                          while(vo_id==(rc_mvo.numVOs+rc_mvo.first_vo-1) && rc_mvo.skip[i])

                          {
                            vo_config = GetVOConfigById(vo_config_list,i);
                            vol_config = GetVolConfigById(GetVOConfigLayers(vo_config),0);
                            rc_mvo.skip[i]--;
                            rc_mvo.VBV_fullness -= rc_mvo.Bpp/rc_mvo.numVOs;
                            rc_mvo.numVOleft[i]--;
                            tmp_frame = GetVolConfigFrame(vol_config);
                            UpdateVolConfigNextCodingTime(vol_config, vo_id, vol_id, rc_type, rc_algorithm, frame_type);
                          }
                      }
                    }
                    /* bug fix: avetro 110399 */
                    /* else if((time_instant == start_time) && (vo_id == rc_mvo.numVOs-1)) */
                    else if((time_instant == start_time) && (vo_id == rc_mvo.numVOs+rc_mvo.first_vo-1))
                    {
                      RCQ2_MVO_ExcludeIFrame(num_bits, rc_mvo.numVOs);
                      RCQ2_MVO_ResetBitCount(num_bits, rc_mvo.numVOs);
                    }
                    {
                      Float time_inc = GetVolConfigTimeInc(vol_config);
                      Float new_time = vol_config->next_coding_time;
                      new_time += time_inc*(Float)GetVolConfigM(vol_config);
                      new_time = RoundTime(new_time);
                      vol_config->next_coding_time = new_time;
                    }
                }
                /* inserted by Mistubishi & SAMSUNG AIT */
                else
                 UpdateVolConfigNextCodingTime(vol_config, vo_id, vol_id, rc_type, rc_algorithm, frame_type);               


                
        } /* End of while(TRUE) */


        /* Calculate average bit statistics */
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl))
	  AverageBitCounts_Studio(num_bits,vo_config_list);
	else  
  /* 01.04.01: Studio Profile */
        AverageBitCounts(num_bits,vo_config_list);
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl)) {
	  DoPrintSessionEndStat_Studio(num_bits, video_object_list, stdout);
	  DoPrintSessionEndStat_Studio(num_bits, video_object_list, fpstats_file);
	} else {
  /* 01.04.01: Studio Profile */
#ifdef NLSSTATS
        DoPrintSessionEndStat(num_bits, video_object_list, stdout);
        DoPrintSessionEndStat(num_bits, video_object_list, fpstats_file);
#else
        PrintSessionEndStat(num_bits,video_object_list,DISPLAY);
        PrintSessionEndStat(num_bits,video_object_list,STATS_FILE);
#endif
        }

        if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED)
        {
#ifdef _RC_DEBUG_
            RCQ2_MVO_EndStats(rc_mvo.numVOs, num_bits);
#endif 
        }


  /* Close bit stream files */
        CloseBitstreams(vo_config_list);

  /*
   *
   * Deallocate all memory 
   *
   */
                
        if (rc_type == VM5_RATE_CONTROL)
          RC_Free(vo_config_list, rc_algorithm);


        if (temporal_scalability != 0){
            FreeEnhanceVop(enhance_vop, &enhance_count);
            if(base_vop_not_padding_for_bgc != NULL){
                if (GetVopPrevTemp(base_vop_not_padding_for_bgc) != NULL) 
                    FreeVop(GetVopPrevTemp(base_vop_not_padding_for_bgc));
                if (GetVopNextTemp(base_vop_not_padding_for_bgc) != NULL) 
                    FreeVop(GetVopNextTemp(base_vop_not_padding_for_bgc));
                FreeVop(base_vop_not_padding_for_bgc);

                /* if use B_VOP : TPS */
                if(temporal_scalability != 3){
                    if(original_prev_base_vop != NULL)
                        FreeVop(original_prev_base_vop);
                    if(base_B_vop_TPS != NULL){
                        if(GetVopNextTemp(base_B_vop_TPS) != NULL){
                            FreeVop(GetVopNextTemp(base_B_vop_TPS));
                            PutVopNextTemp(NULL, base_B_vop_TPS);
                        }
                        FreeVop(base_B_vop_TPS);
                        base_B_vop_TPS = NULL;
                   }     
                }
            }
         }

  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl)) {

	  /* VO i/p list */
	  FreeVOList_Studio(video_object_list);
	  
	  /* VO storage lists */
	  FreeVOList_Studio(prev_vo_store);
	  FreeVOList_Studio(prev_rec_vo_store);
	  
	  FreeVOList_Studio(next_vo_store);
	  FreeVOList_Studio(next_rec_vo_store);
	  
	  /* 1197-2 SONY */
	  /*** 12/10 SHARP */
	  if (M-1) {
	    for (TRB=1;TRB<M;TRB++) 
	      FreeVOList_Studio(B_vo_store[TRB-1]);
	    free(B_vo_store);
	    
	    if(B_org_vo_store!=NULL){
	      for (TRB=1;TRB<M;TRB++) 
		FreeVOList_Studio(B_org_vo_store[TRB-1]);
	      free(B_org_vo_store);
	    }
	  }
	  /* 12/10 SHARP ***/
	  /* 1197-2 */
	} else {
  /* 01.04.01: Studio Profile */
  
        /* VO i/p list */
        FreeVOList(video_object_list);

        /* VO storage lists */
        FreeVOList(prev_vo_store);
        FreeVOList(prev_rec_vo_store);

        FreeVOList(next_vo_store);
        FreeVOList(next_rec_vo_store);

/* 1197-2 SONY */
 /*** 12/10 SHARP */
  if (M-1) {
    for (TRB=1;TRB<M;TRB++) 
      FreeVOList(B_vo_store[TRB-1]);
    free(B_vo_store);
      
    if(B_org_vo_store!=NULL){
      for (TRB=1;TRB<M;TRB++) 
        FreeVOList(B_org_vo_store[TRB-1]);
      free(B_org_vo_store);
    }
  }
 /* 12/10 SHARP ***/
/* 1197-2 */

      }
  
        /* Control Data Structure */
        FreeEncodeControl(enc_ctl);

        /* VO config data structures */
        FreeVOConfigList(vo_config_list);

        /* Display Vop */
  /* 01.04.01: Studio Profile */
        if(GetEncodeControlStudioProfile_st(enc_ctl))
	 FreeVop_Studio(display_vop);
       else
 /* 01.04.01: Studio Profile */
        FreeVop(display_vop);

	/* 01.02.99 HHI Schueuer */
	Free_SADCT_Kaup();
        Free_SADCT(); 
	Free_SADCT_s_k();
	/* end HHI */   

#ifdef _STUDIO_CONF_5_

  for ( iii=0; iii<1024; iii++)
    fprintf(g_input, "%d\t%10d\n", iii-512, g_pixel[iii]);
  for ( iii=0; iii<4096; iii++)
    fprintf(g_output, "%d\t%10d\n", iii-2048, g_coeff[iii]);
  fclose(g_input);
  fclose(g_output);
#endif _STUDIO_CONF_5_

#ifdef _STUDIO_CONF_6_
  for ( iii=0; iii<18000; iii++)
    fprintf(t6_fp_qcoef, "%d\t%10d\n", iii-9000, t6_v_qcoef[iii]);

  fclose(t6_fp_qcoef);
#endif _STUDIO_CONF_6_

        exit(0);
}
      

/***********************************************************CommentBegin******
 *
 * -- CodeBaseVol -- High level coding function for BASE layer
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      30-06-96
 *
 * Purpose :            
 *      This function reads in the next Vop to be coded for the BASE
 *      layer, sets up pointers to the required previous and 
 *      reconstructed Vops, calls the coding function for the Vop
 *      updates the layer store, and writes the coded Vop to
 *      disk if required.
 *
 * Arguments in :       
 *      Vol *base_layer - pointer to base layer Vol structure
 *      VolConfig *layer_config - pointer to config structure for BASE
 *      Int     frame - frame containing Vop to be read from disk
 *      Int time - current coding time instant
 *      Int vo_id - current VO Id
 *      Int np_vop_id - vop_id for NEWPRED (Oki)
 *
 * Arguments in/out :   
 *      Vol *prev_vol_store - store of previous original layers
 *      Vol *rec_vol_store - store of previous reconstructed layers
 *      BitCount num_bits[][] - array of BitCount structures 
 *      RefVop ***ref_vop - reference memory for NEWPRED (Oki)
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *      23.09.96 Noel O'Connor : small changes to allow coding of
 *               multiplt VOs.
 *      09.12.96 F. Marques: to introduce the change proposed by C. Billotet
 *          with respect to the computation of "output_frame" 
 *      15.01.97 Robert Danielsen: Updating with NULL Vop for disappearing
 *          Vop's. Fixed by Noel O'Connor.
 *      21.01.97 Added fix for bug VM4-970110-03 by Noel O'Connor.
 * 04.02.97 Noel O'Connor: cmods. for non-unique VOL Ids
 * 07.02.97 J. Ignacio Ronda: Added VM5_RATE_CONTROL 
 * 11.03.97 Minhua Zhou     : Added B_VOPs  
 * 15.05.97 Minhua Zhou     : Solved BUG VM7-970515-01 concerning VM5 Rate control
 *                            in the case of empty VOPs  
 *     04-JUL-97 Sylvie Jeannin : BUG VM7-970515-04 about TPS related memory problems
 * 08.09.97 Cecile Dufour: remove the I_VOP encoding of the static sprite
 * 04.02.98 M.Eckert: Solved bug concerning write empty VOPs, MoMuSys-VCD-V09-980126
 * 07.05.98 Jong Deuk Kim (HYUNDAI) : modified GetVopBounded() 
 *                                    for field-based chrominance subsampling 
 * 28.01.99 M. Eckert: RC_DEBUG outputs for results
 * 16.08.99 Shigeru Fukunaga (Oki): modified for NEWPRED
 *
 ***********************************************************CommentEnd********/

Void
CodeBaseVol(Vol *base_layer,
            VolConfig *layer_config,
            Int frame,
            Float time,
            Vol *prev_vol_store,
            Vol *rec_vol_store,
            Vol *next_vol_store,
            Vol *next_rec_vol_store,
            Int TRB, Int TRD, 
            Int vo_id,    
            VOConfig *vo_config_list, /* UPM Global RC */
            Int rc_type,              /* UPM Global RC */
            Int rc_algorithm,              /* hjlee  */
            Int *frame_type,         /* UPM Global RC */
            BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
            /* 1197-2 SONY */
            Vol *B_org_vol_store,
            /* 1197-2 */        
            Vol *B_vol_store,
            Vop *base_vop_not_padding_for_bgc,
            Int temporal_scalability,
            Int enhancement_type,
            Vop **base_B_vop_TPS,
	    RefVop ***ref_vop,	/** added for NEWPRED (Oki) 16-AUG-1999 **/
	    Int np_vop_id)	/** added for NEWPRED (Oki) 16-AUG-1999 **/
{
        Vol             *prev_base,
                                *rec_base;

        Vop             *curr_vop_in,
                                *curr_vop_bb,
                                *prev_vop,
                                *rec_vop,
                                *rec_curr=NULL;

        Int             shape,
                                vop_has_content,
                                vol_id,
                                vop_type,
                                output_frame;
        Vop       *no_padding_vop    = NULL,
                  *no_padding_sprite = NULL;
        /* Variables required for rate control */
        Int             vop_quantizer,
                                temp_ref,
                                target_bitrate,
                                end_time;

	Int np_seg_id;	/** added for NEWPRED (Oki) 16-AUG-1999 **/


#ifdef _RC_DEBUG_
        Char bit_file_name[15], psnr_file_name[15], qp_file_name[15];
        Char Ibit_file_name[15], Ipsnr_file_name[15], Iqp_file_name[15];
        Char Bbit_file_name[15], Bpsnr_file_name[15], Bqp_file_name[15];
        Char Pbit_file_name[15], Ppsnr_file_name[15], Pqp_file_name[15];
#endif

        vol_id = GetVolId(base_layer);

#ifdef _RC_DEBUG_
   /* File names for rate, psnr and qp data files for every frame type */
        sprintf(bit_file_name,"bits_%d_%d.b",vo_id,vol_id);
        sprintf(psnr_file_name,"psnr_%d_%d.b",vo_id,vol_id);
        sprintf(qp_file_name,"qp_%d_%d.b",vo_id,vol_id);
        sprintf(Ibit_file_name,"I-bits_%d_%d.b",vo_id,vol_id);
        sprintf(Ipsnr_file_name,"I-psnr_%d_%d.b",vo_id,vol_id);
        sprintf(Iqp_file_name,"I-qp_%d_%d.b",vo_id,vol_id);
        sprintf(Bbit_file_name,"B-bits_%d_%d.b",vo_id,vol_id);
        sprintf(Bpsnr_file_name,"B-psnr_%d_%d.b",vo_id,vol_id);
        sprintf(Bqp_file_name,"B-qp_%d_%d.b",vo_id,vol_id);
        sprintf(Pbit_file_name,"P-bits_%d_%d.b",vo_id,vol_id);
        sprintf(Ppsnr_file_name,"P-psnr_%d_%d.b",vo_id,vol_id);
        sprintf(Pqp_file_name,"P-qp_%d_%d.b",vo_id,vol_id);
#endif 

        if  ( (GetVolConfigSpriteUsage(layer_config)==STATIC_SPRITE)&&
                        ( GetVolConfigWarpParamCounter(layer_config)==-1))
                {
                PutVolConfigWarpParamCounter(0,layer_config);
                }  /* CASE I_VOP OF SPRITE STATIC */

#ifndef NLSSTATS
        /* Put some text for user (DISPLAY and STATS FILE) */
        PrintVolTimeStartStat(num_bits,vo_id,vol_id,frame,DISPLAY);
        PrintVolTimeStartStat(num_bits,vo_id,vol_id,frame,STATS_FILE);
#endif

        /* BASE VOLs are always first in the list */
        prev_base = prev_vol_store;
        rec_base = rec_vol_store;

        /* Get Previous and Reconstructed previous VOPs if they exist */
        prev_vop = GetVolVop(prev_base);
        rec_vop = GetVolVop(rec_base);  

        curr_vop_in = GetVolVop(base_layer);
        

        ReadVopGeneric(GetVolConfigY(layer_config),
                        GetVolConfigU(layer_config),
                        GetVolConfigV(layer_config),
                        GetVolConfigA(layer_config),
                        layer_config->disk_g_file,
			GetVolConfigAuxCompCount(layer_config),
                        frame,
                        IO_FORMAT,
                        curr_vop_in);
        
        /* Disable shape coding if required */
        shape = GetVolConfigShape(layer_config);

        PutVopShape(shape,curr_vop_in);
        if (shape == RECTANGULAR)
                SetConstantImage(GetVopA(curr_vop_in),BINARY_ALPHA);    
        else
                if(shape == GREY_SCALE)
                        PutVopBinaryShape(!BINARY,curr_vop_in);
                else
                        PutVopBinaryShape(BINARY,curr_vop_in);


        
  /* Need to extract bounded vop from vop read in from disk */
  /* Calc. width, height, horizontal/vertical spatial
     reference, shape of vop (REACTANGLE or ARBITRARY),
     and the type of alpha map (BINARY or NON-BINARY).
     Also extract relevant image data */

        curr_vop_bb = GetVopBounded(curr_vop_in,1,1,&vop_has_content,layer_config /* HYUNDAI 980507 */);

        curr_vop_bb->frame = frame;

        /* Get RateControl type */
        /* Changed to GetEncodeControlRCType(enc_ctl) in Main: */
        /* rc_type = GetVolConfigRateControl(layer_config); */

        /* Get RateControl type */
        /* Changed to GetEncodeControlRCType(enc_ctl) in Main: */
        /* rc_type = GetVolConfigRateControl(layer_config); */

        if(temporal_scalability != 0 && vop_has_content)
          no_padding_vop = AllocVop(GetVopWidth(curr_vop_bb),GetVopHeight(curr_vop_bb),GetVopAuxCompCount(curr_vop_bb));
        PutVopRefSelCode(-1,curr_vop_bb);  

	/** added for NEWPRED (Oki) 16-AUG-1999 **/
	if(GetVolConfigNewpredEnable(layer_config)){
	  /* set vop_id for NEWPRED */
	  PutVopNewpredVopId(np_vop_id, curr_vop_bb);
	  /* set number of NEWPRED segment */
	  PutVopNewpredNumVps(layer_config->num_segments, curr_vop_bb);
	  /* set MBA of each NEWPRED segment */
	  for(np_seg_id = 0; np_seg_id <= layer_config->num_segments; np_seg_id++)
	    PutVopNewpredMbaVp(layer_config->mba_segment[np_seg_id], curr_vop_bb, np_seg_id);
	}
	/** end of NEWPRED (Oki) 16-AUG-1999 **/

        rec_curr = VopProcess(curr_vop_bb,
                              prev_vop,
                              rec_vop,
                              GetVolVop(next_vol_store), 
                              GetVolVop(next_rec_vol_store), 
                              time,
                              layer_config,
                              vop_has_content,
                              vo_id,TRB,TRD, 
                              vo_config_list, /* UPM Global RC */
                              rc_type,        /* UPM Global RC */
                              rc_algorithm,  /* hjlee */
                              num_bits,
                              no_padding_vop,
                              temporal_scalability);

/**/

	/** added for NEWPRED (Oki) 16-AUG-1999 **/
	if(GetVolConfigNewpredEnable(layer_config)){
	  /* store coded VOP to memory of reference VOPs */
	  StoreRefVopNewpred(rec_curr,		/* reconstructed current VOP */
			     ref_vop,		/* memories of reference VOPs */
			     layer_config,	/* config structure */
			     vo_id,		/* VO ID */
			     np_vop_id);	/* VOP ID */
	}
	/** end of NEWPRED (Oki) 16-AUG-1999 **/

        if(temporal_scalability != 0 && vop_has_content){
            SaveBaseImage_BGC(layer_config,
                              no_padding_vop,
                              base_vop_not_padding_for_bgc,
                              rec_vop,
                              enhancement_type,
                              GetVopPredictionType(curr_vop_bb),
                              curr_vop_bb,
                              &(*base_B_vop_TPS),
                              temporal_scalability);/* 4/19 changed by SHARP */
             PrintEncodeProcess_TPS(GetVopScalability(curr_vop_bb),
                                    GetVopPredictionType(curr_vop_bb),
                                    GetVolConfigFrameBack(layer_config),
                                    GetVolConfigFrameSkip(layer_config));
        }
        
        *frame_type = GetVopPredictionType(curr_vop_bb);
        
        if (vop_has_content) {  /* added Minhua Zhou 15-05-97 */

        if (rc_type == VM5_RATE_CONTROL)
        {
#if 0
           if (( (prev_vop==NULL)||(GetVopWidth(prev_vop)==0) || 
                 (GetVopPredictionType(curr_vop_bb) == I_VOP) ) && 
               (rc_algorithm != RC_ALGORITHM_Q2))

              RC_ExcludeIFrame(vo_id, vol_id, num_bits[vo_id][vol_id].vop);

/* UPM Global RC */
            else
#endif
            {
               Int num_bits_header;            /* ME - 26.09.97 */

               if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED)
                 ;
               else {
                 /* 06-07-99 Jae-won Chung */
                 if (GetVopPredictionType(curr_vop_bb)==I_VOP) 
                   num_bits_header = num_bits[vo_id][vol_id].syntax;
                 else
                   num_bits_header = num_bits[vo_id][vol_id].vop
                     - num_bits[vo_id][vol_id].mot_shape_text;
                 
                 RC_UpdateModel(vo_id, vol_id,
                                num_bits[vo_id][vol_id].vop,
                                num_bits_header,
                                num_bits[vo_id][vol_id].psnr_y,
                                GetVopPredictionType(curr_vop_bb));     
               }
            }

#if 0
            /* RC2: adding combined mode rate control */
            DSRC_Update2OrderModel(vo_id, vol_id,
                                       num_bits[vol_id][vol_id].vop, 0, 1);
#endif
        }
        }

        /* Update the VOL stores with coded vop and the original vop */
        
        vop_type = GetVopPredictionType(curr_vop_bb);
 
#ifdef _RC_DEBUG_
        if (vop_type == I_VOP){
          WRITE_INT_INT(qp_file_name, frame, GetVopIntraQuantizer(curr_vop_bb));
          WRITE_INT_INT(Iqp_file_name, frame, GetVopIntraQuantizer(curr_vop_bb));
        }
        else if (vop_type == B_VOP){
          WRITE_INT_INT(qp_file_name, frame, GetVopBQuantizer(curr_vop_bb));
          WRITE_INT_INT(Bqp_file_name, frame, GetVopBQuantizer(curr_vop_bb));
        }
        else if (vop_type == P_VOP){
          WRITE_INT_INT(qp_file_name, frame, GetVopQuantizer(curr_vop_bb));
          WRITE_INT_INT(Pqp_file_name, frame, GetVopQuantizer(curr_vop_bb));
        }
        else 
           error_exit("ERROR CodeBaseVol: Unknown frame type!\n");
#endif

        if (vop_has_content)
          {       
/* 1197-2 SONY */
            if (B_vol_store==NULL&&B_org_vol_store==NULL) {
              UpdateVolStore(curr_vop_bb,vol_id,next_vol_store);
              UpdateVolStore(rec_curr,vol_id,next_rec_vol_store);
            }
            else {
              UpdateVolStore(curr_vop_bb,vol_id,B_org_vol_store);
              UpdateVolStore(rec_curr,vol_id,B_vol_store);
            }
/*                    else {
                             UpdateVolStore(rec_curr,vol_id,B_vol_store);
                             FreeVop(curr_vop_bb);
                            }
*/
/* 1197-2 */


             }

        /* If the VOP is empty but the previous VOP exists, then the VO
            has disappeared. In this case the VOL stores are updated with
            a NULL VOP so that the disappearing VO is not composited
            onto the ouput display image - NOC 14/01/97 */
        else if((!vop_has_content) && (prev_vop != NULL))
                {
                  /* MW 30-NOV-1998 */
                  /**/
                  /* VOP composition depends on width==0 up to now, so */
                  /* VOL store should be updated for empty VOPs with */
                  /* the empty VOP; otherwise, during prediction it can */
                  /* not be recognized that the target VOP is empty if it */
                  /* is set to NULL in the VOL store */
                  /**/
                  /*   if (B_vol_store==NULL) {*/
                  /*         UpdateVolStore(NULL,vol_id,next_vol_store);*/
                  /*         UpdateVolStore(NULL,vol_id,next_rec_vol_store);*/
                  /*        } else UpdateVolStore(NULL,vol_id,B_vol_store);*/
                  if (B_vol_store==NULL) {
                    UpdateVolStore(curr_vop_bb,vol_id,next_vol_store);
                    UpdateVolStore(curr_vop_bb,vol_id,next_rec_vol_store);
                  } else UpdateVolStore(curr_vop_bb,vol_id,B_vol_store);
          }

        output_frame = (Int) (time*GetVolConfigFrameRate(layer_config)/1000. + .5);

        if ((rc_type == VM4_RATE_CONTROL)&&(vop_has_content)) /* modified by Minhua Zhou 15-05-97 */    
                {
                end_time = GetVolConfigEndTime(layer_config);
        
                if (((time - GetVolConfigStartTime(layer_config)) != 0) &&
                        (time < end_time))
                        {

                        vop_quantizer = GetVolConfigQuantizer(layer_config);
                        target_bitrate = GetVolConfigBitrate(layer_config);
                        temp_ref = 1000 / GetVolConfigFrameRate(layer_config);
          
                        QuantAdjust (target_bitrate,
                                        temp_ref,
                                        end_time,
                                        time,
                                        num_bits[vo_id][vol_id].average,
                                        num_bits[vo_id][vol_id].vop,
                                        &vop_quantizer);
                        fprintf(stdout, ">>>>> New quantizer= %d\n", (int)vop_quantizer);
          
                        PutVolConfigQuantizer(vop_quantizer,layer_config);
                        }
                }

        /* Write the coded vop to disk if required */
        if (GetVolConfigWriteCoded(layer_config)&&(vop_has_content)) {/* modified by M.Eckert, 04.02.98 */
           if (((TRD>1)&&TRB)|| (TRD==1) ||(prev_vop==NULL))
                        WriteCodedVopToDisk(rec_curr,
                                GetVolConfigCodedY(layer_config),
                                GetVolConfigCodedU(layer_config),
                                GetVolConfigCodedV(layer_config),
                                GetVolConfigCodedA(layer_config),
                                layer_config->coded_g_file,
                                output_frame,
                                GetVolConfigDiskSeqX(layer_config),
                                GetVolConfigDiskSeqY(layer_config),
                                GetVolConfigShape(layer_config),
                                vop_has_content);
             if ((TRD>1)&&(TRB==TRD-1))    /* next P-VOP */
                        WriteCodedVopToDisk(GetVolVop(next_rec_vol_store),
                                GetVolConfigCodedY(layer_config),
                                GetVolConfigCodedU(layer_config),
                                GetVolConfigCodedV(layer_config),
                                GetVolConfigCodedA(layer_config),
                                layer_config->coded_g_file,
                                output_frame,
                                GetVolConfigDiskSeqX(layer_config),
                                GetVolConfigDiskSeqY(layer_config),
                                GetVolConfigShape(layer_config),
                                vop_has_content);
    
             }
        /* Put some text for user */
#ifdef NLSSTATS
       if (vop_has_content) /* 16.07.99 K. Suehring (HHI) */
       { 
        NLSPrintVolTimeEndStat(&num_bits[vo_id][vol_id], rec_curr, time, stdout);
        NLSPrintVolTimeEndStat(&num_bits[vo_id][vol_id], rec_curr, time, fpstats_file);
      } 
#else
        PrintVolTimeEndStat(num_bits,vo_id,vol_id,vop_type,DISPLAY,GetVolConfigAuxCompCount(layer_config));
        PrintVolTimeEndStat(num_bits,vo_id,vol_id,vop_type,STATS_FILE,GetVolConfigAuxCompCount(layer_config));
#endif

#ifdef _RC_DEBUG_
        WRITE_INT_INT(bit_file_name, frame, num_bits[vo_id][vol_id].vop);
        WRITE_INT_FLOAT(psnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);

        if (vop_type == I_VOP){
          WRITE_INT_INT(Ibit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Ipsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else if (vop_type == B_VOP){
          WRITE_INT_INT(Bbit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Bpsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else if (vop_type == P_VOP){
          WRITE_INT_INT(Pbit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Ppsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else 
           error_exit("ERROR CodeBaseVol: Unknown frame type!\n");
#endif


        if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED) 
          ;
        else {
          /* Reset bit counts for the next time this VOL is coded */
          num_bits[vo_id][vol_id].syntax = 0;
          num_bits[vo_id][vol_id].shape = 0;
          num_bits[vo_id][vol_id].texture = 0;
          num_bits[vo_id][vol_id].sprite_piece = 0;
          num_bits[vo_id][vol_id].motion = 0;
          num_bits[vo_id][vol_id].mot_shape_text = 0;
          /* num_bits[vo_id][vol_id].vop = 0; -> set 0 in RC_UpdateBuffer! */
        }

        /* Increment occurrence counter for this VOL */
        num_bits[vo_id][vol_id].occurrence++;
      
      if(temporal_scalability != 0){
        if(no_padding_vop != NULL)
                 FreeVop(no_padding_vop);
        if(no_padding_sprite != NULL)
                 FreeVop(no_padding_sprite);
      }

        return;
}

/***********************************************************CommentBegin******
 *
 * -- CodeEnhVol -- High level coding function for ENHANCEMENT layer
 *
 * Author :             
 *      Ulrike Pestel-Schiller based on CodeBaseVol (970714)
 *
 * Created :            
 *      16-07-97
 *
 * Purpose :            
 *      This function reads in the next Vop to be coded for the ENHANCEMENT
 *      layer, sets up pointers to the required previous and 
 *      reconstructed Vops, calls the coding function for the Vop
 * updates the layer store, and writes the coded Vop to
 *      disk if required.
 *
 * Arguments in :       
 *      Vol *enh_layer - pointer to enh layer Vol structure
 *      VolConfig *layer_config - pointer to config structure for BASE
 *      Int     frame - frame containing Vop to be read from disk
 *      Int time - current coding time instant
 * Int vo_id - current VO Id
 *
 * Arguments in/out :   
 *      Vol *prev_vol_store - store of previous original layers
 *      Vol *rec_vol_store - store of previous reconstructed layers
 *      BitCount num_bits[][] - array of BitCount structures 
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *   30.07.97 Fernando Jaureguizar: commented by #ifdef 0 the code concerning
 *            to RC_ExcludeIFrame(), because this is the enhancement layer
 *   29.08.97 Osamu Sunohara: added new condition to select enhancement layer
 *                            prediction type
 *   05.09.97 Fernando Jaureguizar: added some FeeVop(). (SpSc-2)
 *   07.05.98 Jong Deuk Kim (HYUNDAI) : modified GetVopBounded() 
 *   28.01.99 M. Eckert: RC_DEBUG outputs for results
 *   09.29.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 *   10.15.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 *
 ***********************************************************CommentEnd********/ 
Void
CodeEnhVol(Vol  *enh_layer,
           VolConfig *layer_config,
           Int  frame,
           Float time,
           Vol *prev_vol_store,
           Vol *rec_vol_store,
           Vol *base_vol_store,
           Vol *base_rec_vol_store,
           Vol *next_vol_store,
           Vol *next_rec_vol_store,
           Int TRB, Int TRD, 
           Int vo_id,    
           VOConfig *vo_config_list, /* UPM Global RC */
           Int rc_type,              /* UPM Global RC */
           Int rc_algorithm,              /* hjlee */
           Int *frame_type,          /* UPM Global RC */
           BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS])
{
  Vol   *prev_vol,*prev_rec_vol,*base_vol,*base_rec_vol,*enh_vol;

  Vop   *curr_vop_in=NULL,
        *curr_vop_bb=NULL,
        *prev_vop=NULL,
        *prev_rec_vop=NULL,
        *base_vop=NULL,
        *base_rec_vop=NULL,
        *base_vop_padd=NULL,
        *base_rec_vop_padd=NULL,
        *up_vop=NULL,
        *up_rec_vop=NULL,
        *rec_curr=NULL,
        *no_padding_vop=NULL;

  Vop   *bg_composed_vop; /* added for OBSS by Samsung AIT (1999-10-15) */
  
  Int   shape,
        vop_has_content,
        vol_id,
        vop_type,
        output_frame;
  Int   aux_width,
    aux_height,
    aux_hor_spat_ref ,
    aux_ver_spat_ref ; 

 
        /* Variables required for rate control */
  Int           
          vop_quantizer,
          temp_ref,
          target_bitrate,
          end_time;
        
#ifdef _RC_DEBUG_
  Char bit_file_name[15], psnr_file_name[15];
  Char Ibit_file_name[15], Ipsnr_file_name[15];
  Char Bbit_file_name[15], Bpsnr_file_name[15];
  Char Pbit_file_name[15], Ppsnr_file_name[15];
#endif 
        
  vol_id = GetVolId(enh_layer);
        
#ifdef _RC_DEBUG_
  sprintf(bit_file_name,"bits_%d_%d.b",vo_id,vol_id);
  sprintf(psnr_file_name,"psnr_%d_%d.b",vo_id,vol_id);
  sprintf(Ibit_file_name,"I-bits_%d_%d.b",vo_id,vol_id);
  sprintf(Ipsnr_file_name,"I-psnr_%d_%d.b",vo_id,vol_id);
  sprintf(Bbit_file_name,"B-bits_%d_%d.b",vo_id,vol_id);
  sprintf(Bpsnr_file_name,"B-psnr_%d_%d.b",vo_id,vol_id);
  sprintf(Pbit_file_name,"P-bits_%d_%d.b",vo_id,vol_id);
  sprintf(Ppsnr_file_name,"P-psnr_%d_%d.b",vo_id,vol_id);
#endif 
        

  /* Put some text for user (DISPLAY and STATS FILE) */
#ifndef NLSSTATS
  PrintVolTimeStartStat(num_bits,vo_id,vol_id,frame,DISPLAY);
  PrintVolTimeStartStat(num_bits,vo_id,vol_id,frame,STATS_FILE);
#endif 
        
  prev_vol     = prev_vol_store;
  prev_rec_vol = rec_vol_store;
  base_vol     = base_vol_store;
  base_rec_vol = base_rec_vol_store;
  enh_vol      = enh_layer;

  /* Get Previous and Reconstructed previous VOPs if they exist */
  prev_vop       = GetVolVop(prev_vol);
  prev_rec_vop   = GetVolVop(prev_rec_vol);
  base_vop_padd       = GetVolVop(base_vol);
  base_rec_vop_padd   = GetVolVop(base_rec_vol);                  
  curr_vop_in = GetVolVop(enh_vol);
  curr_vop_in->scalability = enh_layer->scalability;
        
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  PutVopHierarchyType(GetVolHierarchyType(enh_layer),curr_vop_in);
  PutVopRefId(GetVolRefId(enh_layer),curr_vop_in);
  PutVopRefSampDir(GetVolRefSampDir(enh_layer),curr_vop_in);
  PutVopHorSampN(GetVolHorSampN(enh_layer),curr_vop_in);
  PutVopHorSampM(GetVolHorSampM(enh_layer),curr_vop_in);
  PutVopVerSampN(GetVolVerSampN(enh_layer),curr_vop_in);
  PutVopVerSampM(GetVolVerSampM(enh_layer),curr_vop_in);
  PutVopEnhanceType(GetVolEnhanceType(enh_layer),curr_vop_in);
  if(GetVolShape(enh_layer)==1 && GetVolHierarchyType(enh_layer)==0) {
  	PutVopUseRefShape(GetVolUseRefShape(enh_layer),curr_vop_in);
  	PutVopUseRefTexture(GetVolUseRefTexture(enh_layer),curr_vop_in);
  }
  if(GetVolShape(enh_layer)!=0) {
   	PutVopShapeHorSampN(GetVolShapeHorSampN(enh_layer),curr_vop_in);
  	PutVopShapeHorSampM(GetVolShapeHorSampM(enh_layer),curr_vop_in);
  	PutVopShapeVerSampN(GetVolShapeVerSampN(enh_layer),curr_vop_in);
  	PutVopShapeVerSampM(GetVolShapeVerSampM(enh_layer),curr_vop_in);
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  ReadVopGeneric(GetVolConfigY(layer_config),
                 GetVolConfigU(layer_config),
                 GetVolConfigV(layer_config),
                 GetVolConfigA(layer_config),
                 layer_config->disk_g_file,
                 GetVolConfigAuxCompCount(layer_config),
                 frame,
                 IO_FORMAT,
                 curr_vop_in);
        
  /* Disable shape coding if required */
  shape = GetVolConfigShape(layer_config);
        
  PutVopShape(shape,curr_vop_in);
  if (shape == RECTANGULAR)
    SetConstantImage(GetVopA(curr_vop_in),BINARY_ALPHA);        
  else
    if(shape == GREY_SCALE)
      PutVopBinaryShape(!BINARY,curr_vop_in);
    else
      PutVopBinaryShape(BINARY,curr_vop_in);
        
        
        
  /* Need to extract bounded vop from vop read in from disk */
  /* Calc. width, height, horizontal/vertical spatial
     reference, shape of vop (REACTANGLE or ARBITRARY),
     and the type of alpha map (BINARY or NON-BINARY).
     Also extract relevant image data */
  curr_vop_bb = GetVopBounded(curr_vop_in,1,1,&vop_has_content, layer_config /* HYUNDAI 980507 */);

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  PutVopHierarchyType(GetVopHierarchyType(curr_vop_in),curr_vop_bb);
  PutVopRefId(GetVopRefId(curr_vop_in),curr_vop_bb);
  PutVopRefSampDir(GetVopRefSampDir(curr_vop_in),curr_vop_bb);
  PutVopHorSampN(GetVopHorSampN(curr_vop_in),curr_vop_bb);
  PutVopHorSampM(GetVopHorSampM(curr_vop_in),curr_vop_bb);
  PutVopVerSampN(GetVopVerSampN(curr_vop_in),curr_vop_bb);
  PutVopVerSampM(GetVopVerSampM(curr_vop_in),curr_vop_bb);
  PutVopEnhanceType(GetVopEnhanceType(curr_vop_in),curr_vop_bb);
  if(GetVopShape(curr_vop_bb)==1 && GetVopHierarchyType(curr_vop_bb)==0) {
/*OBSSFIX_MODE3-start-*/    
    PutVopUseRefShape(GetVopUseRefShape(curr_vop_in),curr_vop_bb);
    PutVopUseRefTexture(GetVopUseRefTexture(curr_vop_in),curr_vop_bb);
/*  PutVopUseRefShape(GetVolUseRefShape(curr_vop_in),curr_vop_bb);*/
/*  PutVopUseRefTexture(GetVolUseRefTexture(curr_vop_in),curr_vop_bb);*/
/*OBSSFIX_MODE3-end-*/    
  }

/*OBSSFIX_MODE3-start-*/
  if(GetVopShape(curr_vop_bb)!=0) {
/*  if(GetVolShape(curr_vop_bb)!=0) {*/
/*OBSSFIX_MODE3-end-*/
   	PutVopShapeHorSampN(GetVopShapeHorSampN(curr_vop_in),curr_vop_bb);
  	PutVopShapeHorSampM(GetVopShapeHorSampM(curr_vop_in),curr_vop_bb);
  	PutVopShapeVerSampN(GetVopShapeVerSampN(curr_vop_in),curr_vop_bb);
  	PutVopShapeVerSampM(GetVopShapeVerSampM(curr_vop_in),curr_vop_bb);
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

  /* Get RateControl type */
  /* Changed to GetEncodeControlRCType(enc_ctl) in Main: */
  /* rc_type = GetVolConfigRateControl(layer_config); */
        
  /*Unpadding of base layer(UPS)*/

  /*   shape rectangular in this VM */

 /* changed by Minhua Zhou on 24.09.97 */
  /* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if(GetVolVisualObjectLayerVerid(enh_vol)!=1) {
    base_rec_vop = CloneVop_TMP(base_rec_vop_padd);
    base_vop = CloneVop_TMP(base_vop_padd);
  } else {
    /* end: added for OBSS by Samsung AIT (1999-09-29) */
    if(GetVopPredictionType(base_rec_vop_padd) != B_VOP) {
      base_rec_vop = CloneVop_TMP(base_rec_vop_padd);
      base_vop = CloneVop_TMP(base_vop_padd);
    } else {
      base_rec_vop = CloneVop(base_rec_vop_padd);
      base_vop = CloneVop(base_vop_padd);
    }
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

/* 1197-2 SONY */
#ifdef _DEBUG_SPSC3_
  if(up_cnt==0) {
    WriteVopGeneric( base_rec_vop, "E_base_rec_vop", "E_base_rec_vop", "E_base_rec_vop",
                    "tmpppu", 0, IO_FORMAT, IO_OVERWRITE, 0);
  } else {
    WriteVopGeneric( base_rec_vop, "E_base_rec_vop", "E_base_rec_vop", "E_base_rec_vop",
                    "tmpppu", up_cnt, IO_FORMAT, IO_APPEND, 0);
  }
  up_cnt++;
#endif
/* 1197-2 */  

  /*Midprocessor for spatial scal (UPS)   */
  /*1197-3 SONY*/
/* begin: modified for OBSS by Samsung AIT (1999-09-29) */
  /* up_rec_vop=Midproc(base_rec_vop,enh_layer); 
  up_vop=Midproc(base_vop,enh_layer); */
  up_rec_vop = Midproc(base_rec_vop, curr_vop_bb, enh_layer);
  up_vop = Midproc(base_vop, curr_vop_bb, enh_layer);
/* end: modified for OBSS by Samsung AIT (1999-09-29) */
  /*1197-3 SONY*/
    aux_width = GetVopWidth(up_vop);
    aux_height = GetVopHeight(up_vop);
    aux_hor_spat_ref = GetVopHorSpatRef(up_vop);
    aux_ver_spat_ref = GetVopVerSpatRef(up_vop);
    CopyVopNonImageField(base_vop_padd,up_vop);
    PutVopWidth(aux_width, up_vop);
    PutVopHeight(aux_height,up_vop);
    PutVopHorSpatRef(aux_hor_spat_ref,up_vop);
    PutVopVerSpatRef(aux_ver_spat_ref,up_vop);
    CopyVopNonImageField(base_rec_vop_padd,up_rec_vop);
    PutVopWidth(aux_width, up_rec_vop);
    PutVopHeight(aux_height,up_rec_vop);
    PutVopHorSpatRef(aux_hor_spat_ref,up_rec_vop);
    PutVopVerSpatRef(aux_ver_spat_ref,up_rec_vop);

  FreeVop(base_rec_vop); /*SpSc-2*/
  FreeVop(base_vop); /*SpSc-2*/

/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  if(GetVolVisualObjectLayerVerid(enh_vol)!=1) {
    VopExtendByPadding(up_rec_vop);
    VopExtendByPadding(up_vop);
  } else {
/* end: added for OBSS by Samsung AIT (1999-09-29) */
    VopPadding(up_rec_vop);
    VopPadding(up_vop); 
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
  }
/* end: added for OBSS by Samsung AIT (1999-09-29) */
  /*statistics of upsampled data*/
 
  /* modified by Sony 290897 */
/*OBSSFIX_MODE3-start-*/  
/*  if ( ( (prev_vop != 0) &&
      ((layer_config->frame - layer_config->start_frame - layer_config->frame_skip)
       %(layer_config->frame_skip * layer_config->intra_period)!=0) ))*/
  if ( ( (prev_vop != 0) &&
      ((layer_config->frame - layer_config->start_frame - layer_config->frame_skip)
       %(layer_config->frame_skip * layer_config->intra_period)!=0) )
      && (GetVolConfigEnhPredType(layer_config)!=1)/* <-added by Sony 290897*/ )
/*OBSSFIX_MODE3-end-*/    
    /* 290897 */
    {
      /* B-VOP Spatial scalability*/
      
      PutVopPredictionType(B_VOP,curr_vop_bb);
      PutVopRefSelCode(0,curr_vop_bb);          
      PutVopForTempRef(-1,curr_vop_bb);
      PutVopBackTempRef(-1,curr_vop_bb);
      rec_curr = VopProcess(curr_vop_bb,
                            prev_vop,
                            prev_rec_vop,
                            up_vop,
                            up_rec_vop,
                            time,
                            layer_config,
                            vop_has_content,
                            vo_id,TRB,TRD,
                            vo_config_list,
                            rc_type, 
                            rc_algorithm, /* hjlee */
                            num_bits,
                            no_padding_vop,
                            0);
    }
  else
    {
      /* P-VOP   Spatial Scalability*/

      PutVopPredictionType(P_VOP,curr_vop_bb);
      PutVopRefSelCode(3,curr_vop_bb);  
      PutVopForTempRef(-1,curr_vop_bb);
      rec_curr = VopProcess(curr_vop_bb,
                            up_vop,
                            up_rec_vop,
                            GetVolVop(next_vol_store), 
                            GetVolVop(next_rec_vol_store), 
                            time,
                            layer_config,
                            vop_has_content,
                            vo_id,TRB,TRD, 
                            vo_config_list,
                            rc_type,  
                            rc_algorithm, /* hjlee */
                            num_bits,
                            no_padding_vop,
                            0); 
    }
  FreeVop(up_rec_vop);
  FreeVop(up_vop); /*SpSc-2*/
  
  *frame_type = GetVopPredictionType(curr_vop_bb);
  
  if (vop_has_content) {  /* added Minhua Zhou 15-05-97 */
    
    if (rc_type == VM5_RATE_CONTROL)
      {
#if 0
        if ((prev_vop==NULL)||(GetVopWidth(prev_vop)==0)) /* Minhua Zhou 15-05-97 */
          RC_ExcludeIFrame(vo_id, vol_id, num_bits[vo_id][vol_id].vop);
        
        /* UPM Global RC */
        else
#endif
        {
           Int num_bits_header;        /* ME - 26.09.97 */


              num_bits_header = num_bits[vo_id][vol_id].vop
                 - num_bits[vo_id][vol_id].mot_shape_text;
           
           RC_UpdateModel(vo_id, vol_id,
                          num_bits[vo_id][vol_id].vop,
                          num_bits_header,
                          num_bits[vo_id][vol_id].psnr_y,
                          GetVopPredictionType(curr_vop_bb));


        }       
#if 0
        /* RC2: adding combined mode rate control */
                DSRC_Update2OrderModel(vo_id, vol_id,
                                       num_bits[vol_id][vol_id].vop, 0, 1);
#endif
      }
  }
  
  /* Update the VOL stores with coded vop and the original vop */
        
  if (vop_has_content)
    {   
      UpdateVolStore(curr_vop_bb,vol_id,next_vol_store);
      UpdateVolStore(rec_curr,vol_id,next_rec_vol_store);
    }
        
  /* If the VOP is empty but the previous VOP exists, then the VO
     has disappeared. In this case the VOL stores are updated with
     a NULL VOP so that the disappearing VO is not composited
     onto the ouput display image - NOC 14/01/97 */
  else if((!vop_has_content) && (prev_vop != NULL))
          {
            /* MW 30-NOV-1998 */
            /**/
            /* VOP composition depends on width==0 up to now, so */
            /* VOL store should be updated for empty VOPs with */
            /* the empty VOP; otherwise, during prediction it can */
            /* not be recognized that the target VOP is empty if it */
            /* is set to NULL in the VOL store */
            /**/
            /* UpdateVolStore(NULL,vol_id,next_vol_store); */
            /* UpdateVolStore(NULL,vol_id,next_rec_vol_store); */
            UpdateVolStore(curr_vop_bb,vol_id,next_vol_store);
            UpdateVolStore(rec_curr,vol_id,next_rec_vol_store);
          }
  
  output_frame = (Int) (time*GetVolConfigFrameRate(layer_config)/1000. + .5);
  
  if ((rc_type == VM4_RATE_CONTROL)&&(vop_has_content)) /* modified by Minhua Zhou 15-05-97 */  
    {
      end_time = GetVolConfigEndTime(layer_config);
      
      if (((time - GetVolConfigStartTime(layer_config)) != 0) &&
          (time < end_time))
        {
          
          vop_quantizer = GetVolConfigQuantizer(layer_config);
          target_bitrate = GetVolConfigBitrate(layer_config);
          temp_ref = 1000 / GetVolConfigFrameRate(layer_config);
          
          QuantAdjust (target_bitrate,
                       temp_ref,
                       end_time,
                       time,
                       num_bits[vo_id][vol_id].average,
                       num_bits[vo_id][vol_id].vop,
                       &vop_quantizer);
          fprintf(stdout, ">>>>> New quantizer= %d\n", (int)vop_quantizer);
          
          PutVolConfigQuantizer(vop_quantizer,layer_config);
        }
    }
        
/* begin: added for OBSS by Samsung AIT (1999-10-15) */
      if( GetVopShape(curr_vop_bb) &&
	GetVopEnhanceType(curr_vop_bb) && GetVopBackComp(curr_vop_bb)) {	

        /* Write the coded vop to disk if required */
        if (GetVolConfigWriteCoded(layer_config)) {

  	  base_rec_vop = CloneVop_TMP(base_rec_vop_padd);

          if (((TRD>1)&&TRB)|| (TRD==1) ||(prev_vop==NULL)) 
	  {
  		bg_composed_vop= do_bgc_OBSS(base_rec_vop, 
					CloneVop_TMP(rec_curr),
                                GetVolConfigDiskSeqX(layer_config),
                                GetVolConfigDiskSeqY(layer_config),
				enh_layer);
          } else if ((TRD>1)&&(TRB==TRD-1))    /* next P-VOP */
	  {
  		bg_composed_vop= do_bgc_OBSS(base_rec_vop, 
					CloneVop_TMP(GetVolVop(next_rec_vol_store)),
                                GetVolConfigDiskSeqX(layer_config),
                                GetVolConfigDiskSeqY(layer_config),
				enh_layer);
	  }
          WriteCodedVopToDisk(bg_composed_vop,
                                GetVolConfigCodedY(layer_config),
                                GetVolConfigCodedU(layer_config),
                                GetVolConfigCodedV(layer_config),
                                GetVolConfigCodedA(layer_config),
                                layer_config->coded_g_file,
                                output_frame,
                                GetVolConfigDiskSeqX(layer_config),
                                GetVolConfigDiskSeqY(layer_config),
                                GetVolConfigShape(layer_config),
                                vop_has_content);
	  FreeVop(base_rec_vop);
	  FreeVop(bg_composed_vop);
        }
      } else {	
/* end: added for OBSS by Samsung AIT (1999-10-15) */
        /* Write the coded vop to disk if required */
        if (GetVolConfigWriteCoded(layer_config)) {
          if (((TRD>1)&&TRB)|| (TRD==1) ||(prev_vop==NULL))
            WriteCodedVopToDisk(rec_curr,
                                GetVolConfigCodedY(layer_config),
                                GetVolConfigCodedU(layer_config),
                                GetVolConfigCodedV(layer_config),
                                GetVolConfigCodedA(layer_config),
                                layer_config->coded_g_file,
                                output_frame,
                                GetVolConfigDiskSeqX(layer_config),
                                GetVolConfigDiskSeqY(layer_config),
                                GetVolConfigShape(layer_config),
                                vop_has_content);
          if ((TRD>1)&&(TRB==TRD-1))    /* next P-VOP */
            WriteCodedVopToDisk(GetVolVop(next_rec_vol_store),
                                GetVolConfigCodedY(layer_config),
                                GetVolConfigCodedU(layer_config),
                                GetVolConfigCodedV(layer_config),
                                GetVolConfigCodedA(layer_config),
                                layer_config->coded_g_file,
                                output_frame,
                                GetVolConfigDiskSeqX(layer_config),
                                GetVolConfigDiskSeqY(layer_config),
                                GetVolConfigShape(layer_config),
                                vop_has_content);
          
        }
/* begin: added for OBSS by Samsung AIT (1999-10-15) */
      }
/* end: added for OBSS by Samsung AIT (1999-10-15) */
        /* Put some text for user */
        vop_type = GetVopPredictionType(curr_vop_bb);

#ifdef NLSSTATS
      if (vop_has_content) /* 16.07.99 K. Suehring (HHI) */
      {
        NLSPrintVolTimeEndStat(&num_bits[vo_id][vol_id], rec_curr, time, stdout);
        NLSPrintVolTimeEndStat(&num_bits[vo_id][vol_id], rec_curr, time, fpstats_file);
      }
#else
        PrintVolTimeEndStat(num_bits,vo_id,vol_id,vop_type,DISPLAY,GetVolConfigAuxCompCount(layer_config));
        PrintVolTimeEndStat(num_bits,vo_id,vol_id,vop_type,STATS_FILE,GetVolConfigAuxCompCount(layer_config));
#endif
        
#ifdef _RC_DEBUG_
        WRITE_INT_INT(bit_file_name, frame, num_bits[vo_id][vol_id].vop);
        WRITE_INT_FLOAT(bit_file_name, frame, num_bits[vo_id][vol_id].psnr_y);

        if (vop_type == I_VOP){
          WRITE_INT_INT(Ibit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Ipsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else if (vop_type == B_VOP){
          WRITE_INT_INT(Bbit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Bpsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else if (vop_type == P_VOP){
          WRITE_INT_INT(Pbit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Ppsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else 
           error_exit("ERROR CodeBaseVol: Unknown frame type!\n");
#endif
        
        /* Reset bit counts for the next time this VOL is coded */
           num_bits[vo_id][vol_id].syntax = 0;
           num_bits[vo_id][vol_id].shape = 0;
           num_bits[vo_id][vol_id].texture = 0;
           num_bits[vo_id][vol_id].motion = 0;
           num_bits[vo_id][vol_id].mot_shape_text = 0;
        
        /* Increment occurrence counter for this VOL */
        num_bits[vo_id][vol_id].occurrence++;   
        
        return;
}

/***********************************************************CommentBegin******
 *
 * -- BitstreamPutVols -- Write VOL bitstreams to disk
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      30-06-96
 *
 * Purpose :            
 *      Writes the VOLs to disk for all layers in the coding process.
 * 
 * Arguments in :       
 *      VO *list - Pointer to linked list of VOs which contains the VOLs
 *      whose syntax fields are to be written to disk
 *      
 * Arguments in/out :   
 *      BitCount num_bits - structure for the number of bits written to disk
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none 
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *      21.01.97 Added bytealignment fix by Luis
 *      28.01.97 Robert Danielsen: Small change in zigzag-scan
 * 26.02.97 Noel O'Connor: added shape_effects to bitstream
 *     23.04.97 M.Wollborn: MPEG quantization completely added
 *     26.04.97 Luis Ducla-Soares: Added data_partitioning flag to the bitstream.
 *     12.05.97 Minhua Zhou: added "not_8_bit", "advanced_prediction_disable", etc. 
 * 14.05.97 F. Jaureguizar: Modification of BitstreamPutBits() calls for
 *    FCode's . From 2 bits to 3 bits (VM7.0).
 *    18.07.97 Minhua Zhou: added video_object_start_code
 *    04.08.97 Minhua Zhou: removed f_codes from VOL 
 *    06.08.97 Noel Brady: Binary Shape Only switches out several irrelevant
 *              fields in the VOL header
 *    11.08.97 Minhua Zhou: changed quant. matrix coding
 *    12.08.97 Minhua Zhou: added time_increment_resolution
 *    08.09.97 Cecile Dufour: added input arguments: vo_config_list and rc_type
 *                              to encode sprite_pieces
 *    05.11.97 Minhua Zhou:   added flags according to CD
 *    23.03.98 Michael Wollborn: moved "random_accessible_vol" due to N2171, Clause 2.1.7
 *                               included user_data() due to N2171, Clause 2.1.9
 *    26.03.98 Michael Wollborn: changes due to N2171, Clause 2.5.1/14
 *    21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 *    18.01.99 Takefumi Nagumo: added hierarchy_type.
 *    15.02.99 U. Benzler : added quarter pel support
 *    03.03.99 Seishi TAKAMURA (NTT): added GMC coding, MPEG-4 version2 syntax
 *    16.08.99 Shigeru Fukunaga (Oki): added modules for NEWPRED
 *    06.09.99 Eishi Morimatsu (Fujitsu Labs.): added DRC support
 *    09.29.99 Dae-Sung Cho (Samsung AIT): added support for OBSS
 *    11.11.99 U. Benzler (UoH): corrected output of SADCT flag to be done only for Version2
 *    11.11.99 U. Benzler (UoH) - added visual_object_type_indication to be able to create conformance bitstreams
 *
 ***********************************************************CommentEnd********/
 
Void
BitstreamPutVOLs(VO *vo_list,BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
                 VOConfig *vo_config_list, Int rc_type)
{
  VO                    *vo;
  
  Vol           *layer;
  
  Image         *header_bitstream;
  
  Int           i, j,vol_id, vo_id, tmp_val, n;
  VOConfig      *vo_config;
  VolConfig     *vol_config;
  Int *qmat;
  
  vo = vo_list;
  
  while(vo != NULL)
    {
      vo_id = GetVOId(vo);
      vo_config = GetVOConfigById(vo_config_list,vo_id);
      
      layer = GetVOLayers(vo);
      
      while(layer != NULL)
        {
          /* Get VOL Id as index for accessing bitstreams and bit counts */
          vol_id = GetVolId(layer);
          vol_config = GetVolConfigById(GetVOConfigLayers(vo_config),vol_id);
          
          /* Set up intermediate integer level data structure */
          
          header_bitstream = BitstreamInit();
          
          /* 
           *
           * Write all syntax fields in VOL header to data structure
           *
           */

          BitstreamPutBits(header_bitstream,VO_START_CODE,
                           27);
          BitstreamPutBits(header_bitstream,vo_id,
                           5);


          BitstreamPutBits(header_bitstream,VOL_START_CODE,
                           VOL_START_CODE_LENGTH);
          BitstreamPutBits(header_bitstream,GetVolId(layer),4);

          /* Included due to N2171, Clause 2.1.7 MW 23-MAR-1998 */
          PutVolRandomAccessibleVol(0,layer);
          BitstreamPutBits(header_bitstream,GetVolRandomAccessibleVol(layer),1);


          /* 15.12.98 Sven Brandau: add video_object_type_indication, see N2502(a), Clause 6.2.4 - 
             11.11.99 U.Benzler (UoH): added visual_object_type_indication to be able to 
                                       create conformance bitstreams*/
          BitstreamPutBits(header_bitstream,GetVolVisualObjectTypeIndication(layer),8);


/* modified by NTT for GMC coding : start
          PutVolIsObjectLayerIdentifier(0,layer);
 modified by NTT for GMC coding : end */

          BitstreamPutBits(header_bitstream,GetVolIsObjectLayerIdentifier(layer),1);
          if (GetVolIsObjectLayerIdentifier(layer)) {
            BitstreamPutBits(header_bitstream, GetVolVisualObjectLayerVerid(layer), 4);
            BitstreamPutBits(header_bitstream, GetVolVisualObjectLayerPriority(layer), 3);
          } 
          
          /* 11.08.98 Sven Brandau:  "define aspect_ratio_info clearly" due to N2339, Clause 2.1.17 */
          PutVolAspectRatioInfo( 1, layer ); /* -->const. value only for the test phase */
          BitstreamPutBits( header_bitstream, GetVolAspectRatioInfo(layer), 4 );
          if (GetVolAspectRatioInfo(layer)==EXTENDED_PAR) {
            BitstreamPutBits( header_bitstream, GetVolAspectRatioInfoParWidth(layer),  8 );
            BitstreamPutBits( header_bitstream, GetVolAspectRatioInfoParHeight(layer), 8 );         
          }


          /* PutVolVolControlParameters(0,layer);*/ /* always disabled */
          BitstreamPutBits(header_bitstream,GetVolVolControlParameters(layer),1);
	  if (GetVolVolControlParameters(layer))
	  {
		BitstreamPutBits(header_bitstream,GetVolChromaFormat(layer),2);
		BitstreamPutBits(header_bitstream,GetVolLowDelay(layer),1);
		BitstreamPutBits(header_bitstream,GetVolVbvParameters(layer),1);
		if (GetVolVbvParameters(layer))
		{
		  BitstreamPutBits(header_bitstream, 
		                   GetVolFirstHalfBitRate(layer), 15);
		  BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
		  BitstreamPutBits(header_bitstream, 
		                   GetVolLatterHalfBitRate(layer), 15);
		  BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
		  BitstreamPutBits(header_bitstream, 
		                   GetVolFirstHalfVbvBufferSize(layer), 15);
		  BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
		  BitstreamPutBits(header_bitstream, 
		                   GetVolLatterHalfVbvBufferSize(layer), 3);
		  BitstreamPutBits(header_bitstream, 
		                   GetVolFirstHalfVbvOccupancy(layer), 11);
		  BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
		  BitstreamPutBits(header_bitstream, 
		                   GetVolLatterHalfVbvOccupancy(layer), 15);
		  BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
		
		}
	  }

          if (GetVolShape(layer) == BINARY_SHAPE_ONLY) 
            BitstreamPutBits(header_bitstream,2,2);
          else
            {
              /* HYUNDAI (Grayscale) */
              if (GetVolShape(layer) == GREY_SCALE) {
                BitstreamPutBits(header_bitstream,3,2);
                if (GetVolVisualObjectLayerVerid(layer)!=1)
                  BitstreamPutBits(header_bitstream,GetVolShapeExtension(layer),4); /* MAC (SB) 11-Nov-99 */
              } else
                BitstreamPutBits(header_bitstream,GetVolShape(layer),2);
            }
  
          /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.5 */
          BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
       
          /* 10.08.98 Sven Brandau:  "change VOP_time_increment_resolution to 16 Bit" due to N2339, Clause 2.1.2 */
          BitstreamPutBits(header_bitstream,GetVolTimeIncrementResolution(layer),16);

          /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.5 */
          BitstreamPutBits(header_bitstream, MARKER_BIT, 1);

          PutVolFixedVopRate(0,layer);
          BitstreamPutBits(header_bitstream, GetVolFixedVopRate(layer), 1);
          
          if (GetVolShape(layer) != BINARY_SHAPE_ONLY) {/* BSO_NOEL */
            
            if(GetVolShape(layer) == RECTANGULAR)
              {
                BitstreamPutBits(header_bitstream,1,1);
                BitstreamPutBits(header_bitstream,GetVolWidth(layer),13);
                BitstreamPutBits(header_bitstream,1,1);
                BitstreamPutBits(header_bitstream,GetVolHeight(layer),13);
                /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.6 */
                BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
              }
            
            /* 19.08.98 Sven Brandau:  "move interlaced bit from VOP to VOL" due to N2339, Clause 2.1.9 */
            BitstreamPutBits(header_bitstream,GetVolInterlaced(layer), 1);

            BitstreamPutBits(header_bitstream,GetVolOBMCDisable(layer),1);

/* modified by NTT for GMC coding : start
            BitstreamPutBits(header_bitstream,GetVolSpriteUsage(layer),1);
*/
	    if (GetVolVisualObjectLayerVerid(layer) == 1) {
		BitstreamPutBits(header_bitstream,GetVolSpriteUsage(layer),1);
	    } else {
		BitstreamPutBits(header_bitstream,GetVolSpriteUsage(layer),2);
	    }
/* modified by NTT for GMC coding : end */

            if( GetVolSpriteUsage(layer) ==  STATIC_SPRITE )
              {
                    BitstreamPutBits(header_bitstream, GetVolSpriteHdim(layer), 13);
                    BitstreamPutBits(header_bitstream,1,1);
                    BitstreamPutBits(header_bitstream, GetVolSpriteVdim(layer), 13);
                    BitstreamPutBits(header_bitstream,1,1);
                    tmp_val = GetVolSpriteLeftEdge(layer);
                    if (tmp_val<0) 
                      tmp_val +=8192; /* 2 complemented for signed int */
                    BitstreamPutBits(header_bitstream, tmp_val, 13);
                    BitstreamPutBits(header_bitstream,1,1);
                    tmp_val = GetVolSpriteTopEdge(layer);
                    if (tmp_val<0) 
                      tmp_val +=8192; /* 2 complemented for signed int */
                    BitstreamPutBits(header_bitstream, tmp_val, 13);
                    BitstreamPutBits(header_bitstream,1,1);
                
                BitstreamPutBits(header_bitstream, GetVolNoOfSpritePoints(layer), 6);
                BitstreamPutBits(header_bitstream, GetVolWarpingAccuracy(layer), 2);
                BitstreamPutBits(header_bitstream, GetVolBrightnessChangeInSprite(layer), 1);
                BitstreamPutBits(header_bitstream, GetVolLowLatencySpriteEnable(layer), 1);
                
              }
/* modified by NTT for GMC coding : start */
	    else if ( GetVolSpriteUsage(layer) ==  GMC_SPRITE) {
		BitstreamPutBits(header_bitstream, GetVolNoOfSpritePoints(layer), 6);
		BitstreamPutBits(header_bitstream, GetVolWarpingAccuracy(layer), 2);
		BitstreamPutBits(header_bitstream, GetVolBrightnessChangeInSprite(layer), 1);
	    }
/* modified by NTT for GMC coding : end */

            /* 01.02.99 HHI Schueuer - U. Benzler UoH 991111 added 
              (GetVolVisualObjectLayerVerid(layer) != 1)*/
            if ((GetVolShape(layer)!=0) && (GetVolVisualObjectLayerVerid(layer) != 1))
	      BitstreamPutBits(header_bitstream,GetVolSADCTDisable(layer),1);
	    /* end HHI */

            BitstreamPutBits(header_bitstream,GetVolNot8Bit(layer),1);
            if (GetVolNot8Bit(layer)) {
              BitstreamPutBits(header_bitstream,GetVolQuantPrecision(layer),4);
              BitstreamPutBits(header_bitstream,GetVolBitsPerPixel(layer),4);
            } 
            
            /* HYUNDAI (Grayscale) */
            if(GetVolShape(layer) == GREY_SCALE) {
              BitstreamPutBits(header_bitstream,
                               GetVolDisableGrayQuantUpdate(layer),1);

              BitstreamPutBits(header_bitstream,1,1); /* composition_method : Not Supported */
              BitstreamPutBits(header_bitstream,1,1); /* linear_composition : Not Supported */
       
            }

            BitstreamPutBits(header_bitstream,GetVolQuantType(layer),1);
            
            /* Changed stuff for MPEG quantization (23-APR-1997, MW) */
            if (GetVolQuantType(layer))
              {
                BitstreamPutBits(header_bitstream,
                                 GetVolLoadIntraQuantMat(layer),1);
                if(GetVolLoadIntraQuantMat(layer)) {
                  qmat = GetVolIntraQuantMat(layer);
                  for (j=63;j>=1;j--) 
                  if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
                    break;
                  if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
                    j=0;   
                  for(i=0; i<j+1; i++)                                  
                    BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
                  if (j<63) 
                    BitstreamPutBits(header_bitstream,0,8);
                }

                BitstreamPutBits(header_bitstream,
                                 GetVolLoadNonintraQuantMat(layer),1);
                if(GetVolLoadNonintraQuantMat(layer)) {
                  qmat = GetVolNonintraQuantMat(layer); 
                  for (j=63;j>=1;j--) 
                  if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
                    break;
                  if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
                    j=0;   
                  for(i=0; i<j+1; i++)                                  
                    BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
                  if (j<63) 
                    BitstreamPutBits(header_bitstream,0,8);
                }
                

                /* HYUNDAI (Grayscale) */
                if(GetVolShape(layer) == GREY_SCALE)
                  {

                    for (n=0; n<GetVolAuxCompCount(layer); n++ ) { /* MAC (SB) 11-Nov-99 */

                      BitstreamPutBits(header_bitstream,
                                       GetVolLoadGrayIntraQuantMat(n,layer),1);
                      if(GetVolLoadGrayIntraQuantMat(n,layer)) {
                        qmat = GetVolGrayIntraQuantMat(n,layer);
                        for (j=63;j>=1;j--) 
                          if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
                            break;
                        if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
                          j=0;   
                        for(i=0; i<j+1; i++)                                      
                          BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
                        if (j<63) BitstreamPutBits(header_bitstream,0,8);
                      }
                      
                      BitstreamPutBits(header_bitstream,
                                       GetVolLoadGrayNonintraQuantMat(n,layer),1);
                      if(GetVolLoadGrayNonintraQuantMat(n,layer)) {
                        qmat = GetVolGrayNonintraQuantMat(n,layer);
                        for (j=63;j>=1;j--) 
                          if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
                            break;
                        if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
                          j=0;   
                        for(i=0; i<j+1; i++)                                      
                          BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
                        if (j<63) 
                          BitstreamPutBits(header_bitstream,0,8);
                      }
                    }
                  }
              }

	    if (GetVolVisualObjectLayerVerid(layer) != 1)    /* NTT for version2 syntax */
	    /* MW QPEL 07-JUL-1998 */
	    BitstreamPutBits(header_bitstream,GetVolQuarterPel(layer),1);

            /* added by Minhua Zhou,complexity_estimation_disable */
            /* Complexity Estimation syntax support - Marc Mongenet (EPFL) - 3 Jun 1998 */
            BitstreamPutBits(header_bitstream, GetVolComplexityEstimationDisable(layer), 1);
            if (!GetVolComplexityEstimationDisable(layer))
              PutDefineVOPComplexityEstimationHeaderToBitstream(layer,
                                                                header_bitstream);

            /* Modified due to N2171 Cl. 2.5.1/14 MW 26-MAR-1998 */
            /* BitstreamPutBits(header_bitstream,GetVolErrorResDisable(layer),1); */
            /* if(!GetVolErrorResDisable(layer)) */
            /*   { */
            /*     BitstreamPutBits(header_bitstream,GetVolDataPartEnable(layer),1); */
            /*     BitstreamPutBits(header_bitstream,GetVolReverseVlc(layer),1); */
            /*   } */


            /* 19.08.98 Sven Brandau: "insert resync_marker_disable" due to N2339, Clause 3.5.1 */
            /* GetVolErrorResDisable have to be changed to GetVolErrorResDisable in a later time */
            BitstreamPutBits(header_bitstream, GetVolErrorResDisable(layer), 1);

            BitstreamPutBits(header_bitstream,GetVolDataPartEnable(layer),1);
            
            if(GetVolDataPartEnable(layer))
              BitstreamPutBits(header_bitstream,GetVolReverseVlc(layer),1);
            else if(GetVolReverseVlc(layer))
              {
                printf("WARNING: RVLCs can only be used if data partitioning is enabled\n");
                printf("WARNING: RVLCs will not be used!!!\n");
                PutVolReverseVlc(0,layer);
              }

            if(GetVolVisualObjectLayerVerid(layer) != 1) { /** Oki 20-AUG-1999 **/
	      /** added for NEWPRED (Oki) 16-AUG-1999 **/
	      /* NEWPRED flag */
	      BitstreamPutBits(header_bitstream,GetVolNewpredEnable(layer),1);
	      if(GetVolNewpredEnable(layer)){
	        /* backward message type & segment type for NEWPRED */
	        BitstreamPutBits(header_bitstream,GetVolNewpredBackwardMessageType(layer),2);
	        BitstreamPutBits(header_bitstream,GetVolNewpredSegmentType(layer),1);
	      }
	      /** end of NEWPRED (Oki) 16-AUG-1999 **/
/* >>> added for DRC by Fujitsu (top)    <<< */
      	      BitstreamPutBits(header_bitstream,GetVolReducedResolutionVopEnable(layer),1);
/* >>> added for DRC by Fujitsu (bottom) <<< */
	    } /** Oki 20-AUG-1999 **/

            PutVolACDCPredDisable(0,layer); /* AC/DC prediction is always enabled */

            BitstreamPutBits(header_bitstream,GetVolScalability(layer),1);
            
            if(GetVolScalability(layer))
              {
                BitstreamPutBits(header_bitstream,GetVolHierarchyType(layer),1);
                BitstreamPutBits(header_bitstream,GetVolRefId(layer),4);
                BitstreamPutBits(header_bitstream,GetVolRefSampDir(layer),1);
                BitstreamPutBits(header_bitstream,GetVolHorSampN(layer),5);
                BitstreamPutBits(header_bitstream,GetVolHorSampM(layer),5);
                BitstreamPutBits(header_bitstream,GetVolVerSampN(layer),5);
                BitstreamPutBits(header_bitstream,GetVolVerSampM(layer),5);
                if(GetVolEnhanceType(layer)==2)
                  PutVolEnhanceType(1,layer);
                BitstreamPutBits(header_bitstream,GetVolEnhanceType(layer),1);
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
                if(GetVolShape(layer) && !GetVolHierarchyType(layer)){
                        BitstreamPutBits(header_bitstream,GetVolUseRefShape(layer),1);
                        BitstreamPutBits(header_bitstream,GetVolUseRefTexture(layer),1);
                        BitstreamPutBits(header_bitstream,GetVolShapeHorSampN(layer),5);
                        BitstreamPutBits(header_bitstream,GetVolShapeHorSampM(layer),5);
                        BitstreamPutBits(header_bitstream,GetVolShapeVerSampN(layer),5);
                        BitstreamPutBits(header_bitstream,GetVolShapeVerSampM(layer),5);
                }
/* end: added for OBSS by Samsung AIT (1999-09-29) */
              }

            /* Removed due to N2171, Clause 2.1.7 MW 23-MAR-1998 */
            /* PutVolRandomAccessibleVol(0,layer); */
            /* BitstreamPutBits(header_bitstream,GetVolRandomAccessibleVol(layer),1); */
            
            /*
             *
             * Write intermediate bitstream data structure to disk
             *
             */

            num_bits[vo_id][vol_id].vol += BitstreamPut(header_bitstream,vo_id,vol_id);
            /* No longer require the intermediate data structure */
            BitstreamFree(header_bitstream);
            /* should this also be counted ??? -> YES/MW 23-MAR-1998 */
            num_bits[vo_id][vol_id].vol += NextStartCode(vo_id,vol_id);

            /* Included for user data due to N2171, Cl. 2.1.9 MW 23-MAR-1998 */
            if(GetVolConfigIsUserDataInVol(vol_config))
              num_bits[vo_id][vol_id].vol += PutUserDataToBitstream(
                GetVolConfigUserDataInVolFile(vol_config),vo_id,vol_id);

            if( GetVolSpriteUsage(layer) == STATIC_SPRITE&&
                                GetVolLowLatencySpriteEnable(layer)==0)
              {
                EncodeBasicSprite(vol_config,
                                      vo_id, vol_id,
                                      num_bits,
                                      vo_config_list,
                                      rc_type);
                num_bits[vo_id][vol_id].vol += num_bits[vo_id][vol_id].sprite_piece;
                PutVolConfigSpriteTransmitMode(STOP, vol_config);
              }
          } /* BSO_NOEL */

          else /* BINARY SHAPE ONLY */
            {
/* begin: added for OBSS by Samsung AIT (1999-09-29) */
	      if(GetVolVisualObjectLayerVerid(layer) != 1) {
                BitstreamPutBits(header_bitstream,GetVolScalability(layer),1);

                if(GetVolScalability(layer))
                {
                        BitstreamPutBits(header_bitstream,GetVolShapeHorSampN(layer),5);
                        BitstreamPutBits(header_bitstream,GetVolShapeHorSampM(layer),5);
                        BitstreamPutBits(header_bitstream,GetVolShapeVerSampN(layer),5);
                        BitstreamPutBits(header_bitstream,GetVolShapeVerSampM(layer),5);
                }
	      } else {
		fprintf(stderr,"Spatial scalable shape coding is not defined in version 1\n");
		exit(1);
	      }
/* end: added for OBSS by Samsung AIT (1999-09-29) */

              /* 19.08.98 Sven Brandau: "change error_resilient_disable to resync_marker_disable"
                 due to N2339, Clause 3.5.1 */
              /* GetVolErrorResDisable have to be changed to GetVolErrorResDisable in a later time */
              BitstreamPutBits(header_bitstream,GetVolErrorResDisable(layer),1);

              num_bits[vo_id][vol_id].vol += BitstreamPut(header_bitstream,vo_id,vol_id);
              /* No longer require the intermediate data structure */
              BitstreamFree(header_bitstream);
              NextStartCode(vo_id,vol_id);
              
              /* Included for user data due to N2171, Cl. 2.1.9 MW 23-MAR-1998 */
              if(GetVolConfigIsUserDataInVol(vol_config))
                num_bits[vo_id][vol_id].vol += PutUserDataToBitstream(
                  GetVolConfigUserDataInVolFile(vol_config),vo_id,vol_id);
            } 
          
          layer = GetVolNext(layer);
        }
      
      vo = GetVONext(vo);
    }
  
  return;
}


/***********************************************************CommentBegin******
 *
 * -- UpdateVolStore -- Updates the store of Vols
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      30-06-96
 *
 * Purpose :            
 * Given the most recent occurrence of a Vop this function
 *      updates the relevant layer in the VOL store with the
 *      new VOP. The old VOP in the store is freed.
 *
 * Arguments in :       
 *      Vop *update_vop - vop to be placed in store
 *      Int layer_id - Id of layer to be updated with i/p Vop
 *      
 * Arguments in/out :   
 *      Vol     *vol_store - linked list of Vols which acts a memory.
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *
 *      
 *
 ***********************************************************CommentEnd********/
Void
UpdateVolStore(Vop *update_vop,
                                        Int layer_id,
                                        Vol *vol_store)
{
        Vol             *replace_vol;
        
        Vop             *replace_vop;

        /* Get VOL whose VOP is to be updated */
        replace_vol = GetVolById(vol_store,layer_id);

        /* Get VOP to be replaced */
        replace_vop = GetVolVop(replace_vol);

        /* Check if VOP to be replaced exists or not */
        if(replace_vop == NULL)
                PutVolVop(update_vop,replace_vol);

        else
                {
                FreeVop(replace_vop);
                PutVolVop(update_vop,replace_vol);
                }

        return;
}
                


/***********************************************************CommentBegin******
 *
 * -- InitVOStore -- Initialises the store of VOs
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      23-09-96
 *
 * Purpose :            
 *      Initialises a store of VOs by setting up a linked list which is
 *      exactly the same as the input list of VOs (this is the list that
 *      i/p Vops are read into) except that the Vops in the Vols of the
 *      store are not allocated
 *
 * Arguments in :       
 *      VO *list - list of VOs. This is usually a list of VOs in which
 *      the Vops are allocated with the dimensions of the source seq. on disk.
 *      It is used as an input buffer for the Vops from disk. The store is
 *      initialised by simply copying this linked list apart from the 
 *      Vop field.
 *
 * Arguments in/out :   
 *  none
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      VO *store - initialised VO store.
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *      -
 *      
 *
 ***********************************************************CommentEnd********/
VO *
InitVOStore(VO *list)
{
        VO      *store = NULL,
                        *vo,
                        *node;

        Vol     *vol_list,
                        *node_vol_list;

        vo = list;

        while(vo != NULL)
                {
                node = SallocVO();

                PutVOId(GetVOId(vo),node);

                vol_list = GetVOLayers(vo);

                node_vol_list = InitVolStore(vol_list);

                PutVOLayers(node_vol_list,node);

                if(store == NULL)
                        store = node;
                else
                        AddVOToList(node,store);

                vo = GetVONext(vo);
                }

        return(store);
}



/***********************************************************CommentBegin******
 *
 * -- InitVolStore -- Initialises the store of Vols
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      30-06-96
 *
 * Purpose :            
 *      Initialises a store of Vols by setting up a linked list which is
 *      exactly the same as the input list of Vols (this is the list that
 *      i/p Vops are read into) except that the Vops in the Vol store are
 *      not allocated
 *
 * Arguments in :       
 *      Vol *list - list of Vols. This is usually a list of Vols in which
 *      the Vops are allocated with the dimensions of the source seq. on disk.
 *      It is used as an input buffer for the Vops from disk. The store is
 *      initialised by simply copying this linked list apart from the 
 *      Vop field.
 *
 * Arguments in/out :   
 * none
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      Vol *store - initialised VOL store.
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *      -
 *      
 *
 ***********************************************************CommentEnd********/
Vol *
InitVolStore(Vol *list)
{
        Vol     *store = NULL,
                        *layer,
                        *node;

        layer = list;

        while(layer != NULL)
                {
                node = SallocVol();

                CopyVolSyntax(layer,node);

                if(store == NULL)
                        store = node;
                else
                        AddVolToList(node,store);

                layer = GetVolNext(layer);
                }

        return(store);
}


/***********************************************************CommentBegin******
 *
 * -- QuantAdjust -- desc
 *
 * Author :             
 *      Robert Danielsen
 *
 * Created :            
 *      29.04.96
 *
 * Purpose :            
 *      Adjust vop quantization parameter, according to method
 *      described in document Momusys/WG2-0040.
 * 
 * Arguments in :       
 *      Int target_bitrate      Target bitrate from config file
 *      Int temp_ref    Vop temp ref
 *      Int end_time    Vop end time
 *      Int time_instant        Current vop time instant
 *      Int num_bits_used       Total bits used so far for vop
 *  Int num_bits_prev   Bits used for last time instant
 *
 * Arguments in/out :   
 *      Int *QP                 Quantization parameter (Inter) to be adjusted
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none
 *
 * Side effects :       
 *      Adjusts QP
 *
 * Description :        
 *      Adjusts QP according to the following rule: Calculate the number
 *      of bits available per frame for the rest of the sequence. Check
 *      if the last coded frame was more than 15% of this average. If it
 *      was, adjust QP.
 *
 * See also :
 *      -
 *
 * Modified :
 *     20-SEP-1996, M.Wollborn, Changed the code wtih respect to
 *                         the new main routines of VM3.1. The times are now
 *                         all given in milliseconds, not longer in absolute
 *                         frame numbers as in VM2.2 software. Thus this module
 *                         is not compatible with the old software!!!
 *
 ***********************************************************CommentEnd********/

Void
QuantAdjust (Int target_bitrate,
                                Int temp_ref,
                                Int end_time,
                                Int time_instant,
                                Int num_bits_used,
                                Int num_bits_prev,
                                Int *QP)
{
  Float         average;                /* Average bits/frame for remaining frames */
  Int                   dQP;                    /* Step size for change */
  Float         total_bits;     /* Could be calculated before this function */

  total_bits = 1.0 * target_bitrate * (end_time + temp_ref) / 1000;
  average    = 1.0 * (total_bits - num_bits_used)
                   / (end_time - time_instant) * temp_ref;

  fprintf(stdout,"\nRate control:");
  fprintf(stdout,"\n-------------\n");
  
  fprintf(stdout,"\nframe length: %d ms/frame", (int)temp_ref);
  fprintf(stdout,"\nend time:     %d ms", (int)end_time);
  fprintf(stdout,"\ncurrent time: %d ms\n", (int)time_instant);

  fprintf(stdout,"\ntarget bitrate:         %d bit/s", (int)target_bitrate);
  fprintf(stdout,"\ntotal bits available:   %.2f bit",total_bits);
  fprintf(stdout,"\ntotal bits used:        %d bit", (int)num_bits_used);
  fprintf(stdout,"\nbits for prev. frame:   %d bit/frame\n",
          (int)num_bits_prev);
  fprintf(stdout,"\naverage bits available: %.2f bit/frame\n",average);

  fprintf(stdout,"\nold QP: %d", (int)(*QP));

  dQP = MAX (1, *QP * 0.1);     /* Larger steps than 1 possible */

  if (num_bits_prev > (Int)(average * 1.15))
    {
    *QP = MIN (31, *QP + dQP);
    }
  if (num_bits_prev < (Int)(average / 1.15))
    {
    *QP = MAX (1, *QP - dQP);
    }

  fprintf(stdout,"\nnew QP: %d\n", (int)(*QP));
  fprintf(stdout,"\nEnd of rate control\n\n\n");
  
  return;
}

/***********************************************************CommentBegin******
 *
 * -- InitBitstreams() -- Open and initialise bitstream files
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      28-08-96
 *
 * Purpose :            
 *      Open and initialises all bitstream files required in the encoding
 *      process. The names of these files have been read in from the video object
 *      config files on disk. They are stored in a linked list of
 *      VolConfig structures, for each VO. Each VO to be coded has
 *      a linked list associated with it. The VO config information itself
 *      is stored in a linked list of VOConfig structures. The bitstream file 
 *      pointers are stored in an array
 *      (size = MAX_NUM_VOLS) and initialised in this function by VOL id.
 *      Thus from now on in the software, to write bits for a particular layer,
 *      one simply has to pass the vol_id as an argument to the bit writing
 *      function (BitstreamPutBits()).
 * 
 * Arguments in :       
 *      VOConfig *list - linked list of VOConfig structures
 *      
 * Arguments in/out :   
 *      none
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none 
 *
 * Side effects :       
 *      none
 *
 * Description :
 *      -
 *
 * See also :
 *      InitBits(), CloseBitstreams(), BitstreamPutBits(), PutBits()
 *
 * Modified :           
 *     04.02.97 Noel O'Connor: mods for non unique VOl Ids 
 *      
 *
 ***********************************************************CommentEnd********/
 
Void
InitBitstreams(VOConfig *list)
{
        VOConfig                *vo_config;

        VolConfig       *vol_config;

        Int                     vo_id,vol_id;

        Char                    *bitstream_file_name;

        vo_config = list;

        while(vo_config != NULL)
                {
                vo_id = GetVOConfigId(vo_config);

                vol_config = GetVOConfigLayers(vo_config);

                while(vol_config != NULL)
                        {
                        vol_id = GetVolConfigId(vol_config);

                        bitstream_file_name = GetVolConfigBitstream(vol_config);

                        bitstream[vo_id][vol_id] = fopen (bitstream_file_name,"wb");
                        if(bitstream[vo_id][vol_id] == NULL)
                                {
                                fprintf(stderr,"Unable to open %s\n",bitstream_file_name);
                                exit(-1);
                                }

                        InitBits(vo_id,vol_id);

                        vol_config = GetVolConfigNext(vol_config);
                        }

                vo_config = GetVOConfigNext(vo_config);
                }

        return;
}

/***********************************************************CommentBegin******
 *
 * -- CloseBitstreams() -- Closes all bitstream files
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      28-08-96
 *
 * Purpose :            
 *      This function  closes any open bitstream files (incl. padding with
 *      1's to byte boundary).
 * 
 * Arguments in :       
 *      VOConfig *list - linked list of VO1Config structures (incl. file names
 *      and VOL ids).
 *      
 * Arguments in/out :   
 *      none
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none 
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      NextStartCode(), InitBitstreams()
 *
 * Modified :           
 *     14.01.97 Luis Ducla-Soares: enabled byte alignment for VOL_START_CODE    
 *              by removing comment around NextStartCode() function call.
 *          21.01.97 Removed the NextStartCode line. Done elsewhere. Fix by Luis.
 *     04.02.97 Noel O'Connor: mods for non unique VOl Ids 
 *
 ***********************************************************CommentEnd********/

Void
CloseBitstreams(VOConfig *list)
{
        VOConfig                *vo_config;

        VolConfig       *vol_config;

        Int                     vo_id,vol_id;

        vo_config = list;

        while(vo_config != NULL)
                {
                vo_id = GetVOConfigId(vo_config);

                vol_config = GetVOConfigLayers(vo_config);

                while(vol_config != NULL)
                        {
                        vol_id = GetVolConfigId(vol_config);
                        
                        CloseBits(vo_id,vol_id); 

                        fclose(bitstream[vo_id][vol_id]);

                        vol_config = GetVolConfigNext(vol_config);
                        }

                vo_config = GetVOConfigNext(vo_config);
                }

        return;
}

/***********************************************************CommentBegin******
 *
 * -- InitBitCounts -- Initialise all required BitCount structures
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      23-09-96
 *
 * Purpose :            
 *      This function initialises all required BitCount structures (i.e. sets
 *      all fields to zero and copies in the name of the stats file). The
 *      structures are stored in an array and accessed by VOL Id.
 * 
 * Arguments in :       
 *      VOConfig *vo_cfg_list - linked list of VOConfig structures (incl. VOL Ids)
 *      Char *stats_file - name of statistics file on disk
 *      
 * Arguments in/out :   
 *      none
 *
 * Arguments out :      
 *      BitCount num_bits[] - Initialised array of BitCount structures. 
 *
 * Return values :      
 *      none 
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      
 *
 * Modified :           
 *     04.02.97 Noel O'Connor : modified to deal with unique VOL Ids
 *     08.09.97 Cecile Dufour: added sprite_piece=0
 *     21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 *      
 *
 ***********************************************************CommentEnd********/
Void
InitBitCounts(VOConfig *vo_cfg_list, 
                                Char *stats_file,
                                BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS])
{
        VOConfig                *vo_cfg;

        VolConfig       *vol_cfg;

        Int             vo_id,
                        vol_id,
                        sep_mot_shape_text,
			aux;

        vo_cfg = vo_cfg_list;

        while(vo_cfg != NULL)
                {
                vo_id = GetVOConfigId(vo_cfg);

                vol_cfg = GetVOConfigLayers(vo_cfg);
                
                while(vol_cfg != NULL)
                        {
                        vol_id = GetVolConfigId(vol_cfg);
                        sep_mot_shape_text = 0;
#ifndef NLSSTATS
                        strcpy(num_bits[vo_id][vol_id].stats_file,stats_file);
#endif
                        num_bits[vo_id][vol_id].sep_mot_shape_text = sep_mot_shape_text;
                        num_bits[vo_id][vol_id].vol = 0;
                        num_bits[vo_id][vol_id].time = 0;
                        num_bits[vo_id][vol_id].vop = 0;
                        num_bits[vo_id][vol_id].syntax = 0;
                        num_bits[vo_id][vol_id].shape = 0;
                        num_bits[vo_id][vol_id].texture = 0;
                        num_bits[vo_id][vol_id].sprite_piece = 0;
                        num_bits[vo_id][vol_id].motion = 0;
                        num_bits[vo_id][vol_id].mot_shape_text = 0;
                        num_bits[vo_id][vol_id].psnr_y = 0;
                        for(aux=0;aux<GetVolConfigAuxCompCount(vol_cfg);aux++) {/* MAC (SB) 16-Nov-99 */
                          num_bits[vo_id][vol_id].psnr_g[aux]     = 0;     	/* HYUNDAI (Grayscale) */
                          num_bits[vo_id][vol_id].psnr_g_ave[aux] = 0; 		/* HYUNDAI (Grayscale) */
                        }
                        num_bits[vo_id][vol_id].psnr_u = 0;
                        num_bits[vo_id][vol_id].psnr_v = 0;
                        num_bits[vo_id][vol_id].psnr_y_ave = 0;
                       
                        num_bits[vo_id][vol_id].psnr_u_ave = 0;
                        num_bits[vo_id][vol_id].psnr_v_ave = 0;
                        num_bits[vo_id][vol_id].average = 0; 
                        num_bits[vo_id][vol_id].average_shape = 0; 
                        num_bits[vo_id][vol_id].occurrence = 0;

                        vol_cfg = GetVolConfigNext(vol_cfg);
                        }

                vo_cfg = GetVOConfigNext(vo_cfg);       
                }

        return;
}


/***********************************************************CommentBegin******
 *
 * -- AverageBitCounts -- Calculate averages for all relevant BitCount
 *                      steuctures
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      23-09-96
 *
 * Purpose :            
 *      This function calculates average bit statistics (and average PSNRs)
 *      for each VOL's BitCount structure. The BitCount structures are stored
 *      in an array and accessed by VOL Id.
 * 
 * Arguments in :       
 *      num_bits[] - array of BitCount structures
 *      VOConfig  *vo_cfg_list - linked list of VOConfig structures (incl. 
 *                      VOL Ids.
 *
 * Arguments in/out :   
 *      none
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none 
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *     21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 *
 *      
 *
 ***********************************************************CommentEnd********/

Void 
AverageBitCounts(BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
                                        VOConfig *vo_cfg_list)
{
        VOConfig                *vo_cfg;

        VolConfig       *vol_cfg;

        Int                     vo_id,vol_id,aux;

        vo_cfg = vo_cfg_list;

        while(vo_cfg != NULL)
                {
                vo_id = GetVOConfigId(vo_cfg);

                vol_cfg = GetVOConfigLayers(vo_cfg);

                while(vol_cfg != NULL)
                        {
                        vol_id = GetVolConfigId(vol_cfg);

                        num_bits[vo_id][vol_id].average = num_bits[vo_id][vol_id].average/num_bits[vo_id][vol_id].occurrence;

                        num_bits[vo_id][vol_id].average_shape = num_bits[vo_id][vol_id].average_shape/num_bits[vo_id][vol_id].occurrence;

                        num_bits[vo_id][vol_id].psnr_y_ave = num_bits[vo_id][vol_id].psnr_y_ave/num_bits[vo_id][vol_id].occurrence;
                        for(aux=0;aux<GetVolConfigAuxCompCount(vol_cfg);aux++) /* MAC (SB) 16-Nov-99 */
                          num_bits[vo_id][vol_id].psnr_g_ave[aux] = num_bits[vo_id][vol_id].psnr_g_ave[aux]
                            /num_bits[vo_id][vol_id].occurrence;     /* HYUNDAI (Grayscale) */
                        num_bits[vo_id][vol_id].psnr_u_ave = num_bits[vo_id][vol_id].psnr_u_ave/num_bits[vo_id][vol_id].occurrence;
                        num_bits[vo_id][vol_id].psnr_v_ave = num_bits[vo_id][vol_id].psnr_v_ave/num_bits[vo_id][vol_id].occurrence;

                        vol_cfg = GetVolConfigNext(vol_cfg);
                        }

                vo_cfg = GetVOConfigNext(vo_cfg);
                }

        return;
}

/***********************************************************CommentBegin******
 *
 * -- WriteCompositedImage -- Writes composited image to disk
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      23-09-96
 *
 * Purpose :            
 *      This is a "psuedo" compositor. It simply takes each coded VOP in the
 *      VO store and blends them recursively onto a display VOP.
 * 
 * Arguments in :       
 *      EncodeControl *enc_ctl - pointer ot encoder control structure (incl.
 *                               display file name s on disk).
 *      Int frame - output frame number
 *      VO *vo_store - linked list of VOs (incl. most recently coded VOPs). 
 *      
 * Arguments in/out :   
 *      Vop     *display_vop - pointer to the (allocated) display vop.
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none 
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      mini *
 * Modified : 
 *     31.01.97 Aasmund Sandvand: Added deblock filtering
 *               06.08.97 Noel Brady:  For VOPs, having only shape information, a default
 *               constant YUV colur is used i.e. 255,0,0
 *
 *      
 *
 ***********************************************************CommentEnd********/
Void
WriteCompositedImage(EncodeControl *enc_ctl,
                     Int     frame,
                     VO      *vo_store,
                     Vop     *display_vop)
  {
    VO              *vo;
    
    Vol     *base_layer;
    
    Vop     *vop, *filter_vop;
    
    /* Display VOP to grey */
    SetConstantImage(GetVopA(display_vop),0);
    SetConstantImage(GetVopY(display_vop),0);
    SetConstantImage(GetVopU(display_vop),128);
    SetConstantImage(GetVopV(display_vop),128);
    
    vo = vo_store;
    
    while(vo != NULL)
      {
        /* Only composite BASE layer */
        base_layer = GetVOLayers(vo);
        
        /* Get VOP to blend */
        vop = GetVolVop(base_layer);
        
        /* Check that this Vop actually exists. It may not if the
           Vol is not active yet */
        if(vop != NULL)
          {
            
            /* "Borrowed" the following from Michael Wolburn's complementary 
               function in the decoder */
            
            /*****
             *
             *    For rectangular VOPs set the alph-plane again to "255"
             *    as a workaround, since it is modified by incorrect motion
             *    compensation (without padding)
             *
             *****/
            if(GetVopShape(vop) == 0)
            SetConstantImage(GetVopA(vop),255);
            
            if (GetVopShape(vop) == BINARY_SHAPE_ONLY) /* BSO_NOEL */
              {
                SetConstantImage(GetVopY(vop),255);
                SetConstantImage(GetVopU(vop),0);
                SetConstantImage(GetVopV(vop),0);
              }
            
            /*****                                               
             *                                                                 
             *    If the VOP is empty (i.e. arbitrary shape and width=height=0
             *    then don't display it (this is not compliant with the VM yet,
             *    since it is not specified how to tell that a VOP has
             *     disappeared)
             *    else create list of all VOPs to be displayed
             *
             *****/
            if(GetVopShape(vop)!=0 && GetVopWidth(vop)==0)
              {
                vo = GetVONext(vo); /* MW 30-NOV-1998 */
                continue;
              }
            else
              {
                if(GetEncodeControlPostFilterType(enc_ctl)==0)
                  BlendVops(vop, display_vop);
                else
                  {
                    if (GetImageSizeX(GetVopQP(vop))==GetVopWidth(vop)/16)
                      filter_vop=CloneVop(vop);
                    else 
                      filter_vop =CloneVop_TMP(vop); 

                    PostFilter(filter_vop,
                               GetEncodeControlPostFilterType(enc_ctl));
                    BlendVops(filter_vop, display_vop);
                    FreeVop(filter_vop);
                  }
              }
          }
        vo = GetVONext(vo);
      }
    
    if(frame == 0)
      WriteVopGeneric(display_vop,
                      GetEncodeControlDispYFile(enc_ctl),
                      GetEncodeControlDispUFile(enc_ctl),
                      GetEncodeControlDispVFile(enc_ctl),
                      GetEncodeControlDispYFile(enc_ctl),
                      NULL, 
                      0,
                      frame,
                      IO_FORMAT,
                      IO_OVERWRITE,
                      FALSE);
    else
      WriteVopGeneric(display_vop,
                      GetEncodeControlDispYFile(enc_ctl),
                      GetEncodeControlDispUFile(enc_ctl),
                      GetEncodeControlDispVFile(enc_ctl),
                      GetEncodeControlDispYFile(enc_ctl),
                      NULL,
                      0,
                      frame,
                      IO_FORMAT,
                      IO_APPEND,
                      FALSE);
    
    return;
  }

/***********************************************************CommentBegin******
 *
 * -- GetNextCodedVO() -- Returns info on next VO to be coded 
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      20-11-96
 *
 * Purpose :            
 *      Given a linked list of configuration information for the VOs in a session
 * this function decides what is the next VO to be coded. It does this by
 * searching for the VO with the smallest "next_coding_time" (this is a field
 * in each VOL's configuration inofrmation which is updated by the VOL's
 * time_inc/temporal reference each an instance of the VOL is coded). 
 * The time at which the next VOL should be
 * coded is returned along with the VOL and VO Id.
 * 
 * Arguments in :       
 *       VOConfig *list - linked list of VOConfig information 
 *      
 * Arguments in/out :   
 *       
 *
 * Arguments out :      
 *       Int    *vo_id - id of required VO
 *       Int  *vol_id - id of required VOL
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      
 *
 * Modified :           
 *     20.03.97 Fernando Jaureguizar: New finish parameter to return the
 *              finish flag here instead of using the function return which
 *              gives the time_instant.
 *     27.05.97 Paulo Nunes: modified first_pass
 *
 ***********************************************************CommentEnd********/

Float
GetNextCodedVO(VOConfig *list,
                        Int *vo_id,
                        Int *vol_id,
                        Int *finish)
{
        VOConfig        *curr_vo;

        VolConfig       *curr_vol;
/* 1197-2 SONY */
        VolConfig       *curr_vol_list;
/* 1197-2 */

        Float           min_time=0,
                                next_coding_time,
                                time_to_end,
                                end_time,
                                max_end_time = -1;

        Int             first_pass = 1;
        
        curr_vo = list;

    while( curr_vo != NULL )
    {
        curr_vol_list =   /* 1197 SONY */
        curr_vol = GetVOConfigLayers(curr_vo);

        while(curr_vol != NULL)
        {
            next_coding_time = GetVolConfigNextCodingTime(curr_vol);
            end_time = GetVolConfigEndTime(curr_vol);
             if (first_pass)
            {
                *vo_id = GetVOConfigId(curr_vo);         
                *vol_id = GetVolConfigId(curr_vol);      
                min_time = next_coding_time;
                if(next_coding_time <= end_time)
                  first_pass = 0;
            }
                        
           if(max_end_time < end_time)
                max_end_time = end_time;
                        
            if(next_coding_time < min_time)
            {
                if(next_coding_time <= end_time)
                {
/* 1197-2 SONY */
                  if(  GetVolConfigScalability(curr_vol)==0 ||
                     ( GetVolConfigScalability(curr_vol)==1 && GetVolConfigScalType(curr_vol)==0 ) ||
                     (
                       GetVolConfigScalability(curr_vol)==1 && GetVolConfigScalType(curr_vol)==1 &&
/* SONY 070498 - start */
                      ( ((GetVolConfigById(curr_vol_list,GetVolConfigRefId(curr_vol)))->frame)
                       !=(curr_vol->frame+(GetVolConfigM( GetVolConfigById( curr_vol_list, GetVolConfigRefId( curr_vol ) ))-1)*GetVolConfigFrameSkip(curr_vol)) )
/* SONY 070498 - end */
                     )
                    )   
                    {
                      *vo_id = GetVOConfigId(curr_vo);
                      *vol_id = GetVolConfigId(curr_vol);
                      min_time = next_coding_time;
                    }
/*
                    *vo_id = GetVOConfigId(curr_vo);
                    *vol_id = GetVolConfigId(curr_vol);
                    min_time = next_coding_time;
*/
/* 1197-2 */
                }
                else
                {
                    /* Special case for last vop */
                    time_to_end = next_coding_time - end_time;
                    if(time_to_end < 0.001)
                    {
/* 1197-2 SONY */
                  if(  GetVolConfigScalability(curr_vol)==0 ||
                     ( GetVolConfigScalability(curr_vol)==1 && GetVolConfigScalType(curr_vol)==0 ) ||
                     (
                       GetVolConfigScalability(curr_vol)==1 && GetVolConfigScalType(curr_vol)==1 &&
/* SONY 070498 - start */
                      ( ((GetVolConfigById(curr_vol_list,GetVolConfigRefId(curr_vol)))->frame)
                       !=(curr_vol->frame+(GetVolConfigM( GetVolConfigById( curr_vol_list, GetVolConfigRefId( curr_vol ) ))-1)*GetVolConfigFrameSkip(curr_vol)) )
/* SONY 070498 - end */
                     )
                    )   
                    {
                      *vo_id = GetVOConfigId(curr_vo);
                      *vol_id = GetVolConfigId(curr_vol);
                      min_time = next_coding_time;
                    }
/*
                        *vo_id = GetVOConfigId(curr_vo);
                        *vol_id = GetVolConfigId(curr_vol);
                        min_time = next_coding_time;
*/
/* 1197-2 */
                    }
                }
            }
            curr_vol = GetVolConfigNext(curr_vol);
        }

        curr_vo = GetVOConfigNext(curr_vo);
    }

    if(min_time > max_end_time)
        *finish = FINISHED;
    else
        *finish = 0;
      return(min_time);

}






/***********************************************************CommentBegin******
 *
 * -- PutUserDataToBitstream() -- writes user data from file into bitstream
 *
 * Author :             
 *      Michael Wollborn, University of Hannover
 *
 * Created :            
 *      23-03-98
 *
 * Purpose :
 *      This function writes the user data from a file on disk into the bitstream.
 * 
 * Arguments in :       
 *      VolConfig *vol_config - VOL config information
 *
 * Arguments in/out :   
 *      Image *header_bitstream - intermediate bitstream for VOL header
 *
 * Arguments out :      
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      
 *
 * Modified :           
 *
 ***********************************************************CommentEnd********/

Int
PutUserDataToBitstream(Char *filename, Int vo_id, Int vol_id)
{
  Int filesize=0;

  Image *header_bitstream;
  FILE *ud_file;
  Byte *data;

  Int i;

  /* Check if file is there */
  if((ud_file = fopen(filename,"rb")) == NULL)
    {
      fprintf(stderr,"ERROR(PutUserDataToBitstream): unable to open %s\n",
              filename);
      exit(1);
    }

  /* Init intermediate data structure */
  header_bitstream = BitstreamInit();

  /* Check size of file (ftell() gives size in Bytes!) */
  fseek(ud_file,0L,SEEK_END);
  filesize = ftell(ud_file);
  rewind(ud_file);

  /* Allocate sufficient memory */
  data = (Byte *)malloc((filesize) * sizeof(Byte));

  /* Load user data */
  fread(data,1,(filesize),ud_file);

  /* Write user_data_start_code and user data */
  BitstreamPutBits(header_bitstream,USER_DATA_START_CODE,USER_DATA_START_CODE_LENGTH);

  for(i=0; i<filesize; i++)
    BitstreamPutBits(header_bitstream, data[i], 8);

  /* Write to real bitstream plus next_start_code() */
  BitstreamPut(header_bitstream,vo_id,vol_id);
  filesize += NextStartCode(vo_id,vol_id);

  /* Free memory, close file and return number of bits written*/
  BitstreamFree(header_bitstream);
  free((Char *) data);
  fclose(ud_file);
  return(filesize);
}




/***********************************************************CommentBegin******
 *
 * -- PutDefineVOPComplexityEstimationHeaderToBitstream -- writes into bitstream
 *      the define_VOP_complexity_estimation_header.
 *
 * Author :
 *      Marc Mongenet, Swiss Federal Institute of Technology, Lausanne
 *
 * Created :
 *      03-06-98
 *
 * Purpose :
 *      This function writes the define_VOP_complexity_estimation_header from
 *      the Vol structure into the bitstream VOL header.
 *
 * Arguments in :
 *      Vol * vol   - vol structure with complexity estimation header to write
 *
 * Arguments in/out :
 *      Image * bitstream - bitstream for VOL header.
 *
 * Arguments out :
 *
 * Return values :
 *
 * Side effects :
 *
 * Description :
 *      The actual value of flags is not verified, only one bit is written
 *      => better use 0 as false and 1 as true. Only estimation method 0 is
 *      defined.
 *
 * See also :
 *
 * Modified :
 *      26.10.99 Massimo Ravasi (EPFL): added Complexity Estimation syntax support
 *                                      Update version 2
 *
 ***********************************************************CommentEnd********/

Void PutDefineVOPComplexityEstimationHeaderToBitstream(Vol * vol,
                                                       Image * bitstream)
{
  if (!GetVolComplexityEstimationDisable(vol))
  {
    Int estimation_method = GetVolEstimationMethod(vol);
    Int shape_complexity_estimation_disable = GetVolShapeComplexityEstimationDisable(vol);
    Int texture_complexity_estimation_set_1_disable = GetVolTextureComplexityEstimationSet1Disable(vol);
    Int texture_complexity_estimation_set_2_disable = GetVolTextureComplexityEstimationSet2Disable(vol);
    Int motion_compensation_complexity_disable = GetVolMotionCompensationComplexityDisable(vol);
		Int version2_complexity_estimation_disable = GetVolVersion2ComplexityEstimationDisable(vol);

    if (estimation_method != 0 && estimation_method != 1) {
      fprintf(stderr,
              "ERROR(PutDefineVOPComplexityEstimationHeaderToBitstream): "
              "Estimation method %d not supported\n",
              estimation_method);
      exit(1);
    }

    BitstreamPutBits(bitstream, estimation_method, 2);

    BitstreamPutBits(bitstream, shape_complexity_estimation_disable, 1);
    if (!shape_complexity_estimation_disable) {
      BitstreamPutBits(bitstream, GetVolOpaque(vol), 1);
      BitstreamPutBits(bitstream, GetVolTransparent(vol), 1);
      BitstreamPutBits(bitstream, GetVolIntraCAE(vol), 1);
      BitstreamPutBits(bitstream, GetVolInterCAE(vol), 1);
      BitstreamPutBits(bitstream, GetVolNoUpdate(vol), 1);
      BitstreamPutBits(bitstream, GetVolUpsampling(vol), 1);
    }

    BitstreamPutBits(bitstream, texture_complexity_estimation_set_1_disable, 1);
    if (!texture_complexity_estimation_set_1_disable) {
      BitstreamPutBits(bitstream, GetVolIntraBlocks(vol), 1);
      BitstreamPutBits(bitstream, GetVolInterBlocks(vol), 1);
      BitstreamPutBits(bitstream, GetVolInter4vBlocks(vol), 1);
      BitstreamPutBits(bitstream, GetVolNotCodedBlocks(vol), 1);
    }

    /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.4 */
    BitstreamPutBits(bitstream, MARKER_BIT, 1 );

    BitstreamPutBits(bitstream, texture_complexity_estimation_set_2_disable, 1);
    if (!texture_complexity_estimation_set_2_disable) {
      BitstreamPutBits(bitstream, GetVolDCTCoefs(vol), 1);
      BitstreamPutBits(bitstream, GetVolDCTLines(vol), 1);
      BitstreamPutBits(bitstream, GetVolVLCSymbols(vol), 1);
      BitstreamPutBits(bitstream, GetVolVLCBits(vol), 1);
    }

    BitstreamPutBits(bitstream, motion_compensation_complexity_disable, 1);
    if (!motion_compensation_complexity_disable) {
      BitstreamPutBits(bitstream, GetVolAPM(vol), 1);
      BitstreamPutBits(bitstream, GetVolNPM(vol), 1);
      BitstreamPutBits(bitstream, GetVolInterpolateMCQ(vol), 1);
      BitstreamPutBits(bitstream, GetVolForwBackMCQ(vol), 1);
      BitstreamPutBits(bitstream, GetVolHalfpel2(vol), 1);
      BitstreamPutBits(bitstream, GetVolHalfpel4(vol), 1);
    }

    /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.4 */
    BitstreamPutBits(bitstream, MARKER_BIT, 1 );


  	/* START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 26 Oct 1999 */
    if (estimation_method == 1) {
			BitstreamPutBits(bitstream, version2_complexity_estimation_disable, 1);
			if (!version2_complexity_estimation_disable) {
				BitstreamPutBits(bitstream, GetVolSadct(vol), 1);
				BitstreamPutBits(bitstream, GetVolQuarterpel(vol), 1);
			}
		}
    /* END: Complexity Estimation syntax support - Update Version 2 */
  }
}

/* 01.04.01: Studio Profile */
/***********************************************************CommentBegin******
 *
 * -- CodeBaseVol_Studio -- High level coding function for BASE layer
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      30-06-96
 *
 * Purpose :            
 *      This function reads in the next Vop to be coded for the BASE
 *      layer, sets up pointers to the required previous and 
 *      reconstructed Vops, calls the coding function for the Vop
 * updates the layer store, and writes the coded Vop to
 *      disk if required.
 *
 * Arguments in :       
 *      Vol *base_layer - pointer to base layer Vol structure
 *      VolConfig *layer_config - pointer to config structure for BASE
 *      Int     frame - frame containing Vop to be read from disk
 *      Int time - current coding time instant
 * Int vo_id - current VO Id
 *
 * Arguments in/out :   
 *      Vol *prev_vol_store - store of previous original layers
 *      Vol *rec_vol_store - store of previous reconstructed layers
 *      BitCount num_bits[][] - array of BitCount structures 
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *      23.09.96 Noel O'Connor : small changes to allow coding of
 *               multiplt VOs.
 *      09.12.96 F. Marques: to introduce the change proposed by C. Billotet
 *          with respect to the computation of "output_frame" 
 *      15.01.97 Robert Danielsen: Updating with NULL Vop for disappearing
 *          Vop's. Fixed by Noel O'Connor.
 *      21.01.97 Added fix for bug VM4-970110-03 by Noel O'Connor.
 * 04.02.97 Noel O'Connor: cmods. for non-unique VOL Ids
 * 07.02.97 J. Ignacio Ronda: Added VM5_RATE_CONTROL 
 * 11.03.97 Minhua Zhou     : Added B_VOPs  
 * 15.05.97 Minhua Zhou     : Solved BUG VM7-970515-01 concerning VM5 Rate control
 *                            in the case of empty VOPs  
 *     04-JUL-97 Sylvie Jeannin : BUG VM7-970515-04 about TPS related memory problems
 * 08.09.97 Cecile Dufour: remove the I_VOP encoding of the static sprite
 * 04.02.98 M.Eckert: Solved bug concerning write empty VOPs, MoMuSys-VCD-V09-980126
 * 07.05.98 Jong Deuk Kim (HYUNDAI) : modified GetVopBounded() 
 *                                    for field-based chrominance subsampling 
 * 28.01.99 M. Eckert: RC_DEBUG outputs for results
 * 26.07.99 Osamu Sunohara (SONY): For Studio Profile
 *
 ***********************************************************CommentEnd********/

Void
CodeBaseVol_Studio(Vol *base_layer,
		   VolConfig *layer_config,
		   Int frame,
		   Float time,
		   Vol *prev_vol_store,
		   Vol *rec_vol_store,
		   Vol *next_vol_store,
		   Vol *next_rec_vol_store,
		   Int TRB, Int TRD, 
		   Int vo_id,    
		   VOConfig *vo_config_list, /* UPM Global RC */
		   Int rc_type,              /* UPM Global RC */
		   Int rc_algorithm,              /* hjlee  */
		   Int *frame_type,         /* UPM Global RC */
		   BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
		   /* 1197-2 SONY */
		   Vol *B_org_vol_store,
		   /* 1197-2 */        
		   Vol *B_vol_store,
		   Vop *base_vop_not_padding_for_bgc,
		   Int temporal_scalability,
		   Int enhancement_type,
		   Vop **base_B_vop_TPS)
{
        Vol             *prev_base,
                                *rec_base;

        Vop             *curr_vop_in,
                                *curr_vop_bb,
                                *prev_vop,
                                *rec_vop,
                                *rec_curr=NULL;

        Int             shape,
                                vop_has_content,
                                vol_id,
                                vop_type,
                                output_frame;
        Vop       *no_padding_vop    = NULL,
                  *no_padding_sprite = NULL;
        /* Variables required for rate control */
        Int             vop_quantizer,
                                temp_ref,
                                target_bitrate,
                                end_time;

#ifdef _RC_DEBUG_
        Char bit_file_name[15], psnr_file_name[15], qp_file_name[15];
        Char Ibit_file_name[15], Ipsnr_file_name[15], Iqp_file_name[15];
        Char Bbit_file_name[15], Bpsnr_file_name[15], Bqp_file_name[15];
        Char Pbit_file_name[15], Ppsnr_file_name[15], Pqp_file_name[15];
#endif

	Int curr_vop_structure;
	Int i, vop_type2[2];
	Int kk;
	
        vol_id = GetVolId(base_layer);

#ifdef _RC_DEBUG_
   /* File names for rate, psnr and qp data files for every frame type */
        sprintf(bit_file_name,"bits_%d_%d.b",vo_id,vol_id);
        sprintf(psnr_file_name,"psnr_%d_%d.b",vo_id,vol_id);
        sprintf(qp_file_name,"qp_%d_%d.b",vo_id,vol_id);
        sprintf(Ibit_file_name,"I-bits_%d_%d.b",vo_id,vol_id);
        sprintf(Ipsnr_file_name,"I-psnr_%d_%d.b",vo_id,vol_id);
        sprintf(Iqp_file_name,"I-qp_%d_%d.b",vo_id,vol_id);
        sprintf(Bbit_file_name,"B-bits_%d_%d.b",vo_id,vol_id);
        sprintf(Bpsnr_file_name,"B-psnr_%d_%d.b",vo_id,vol_id);
        sprintf(Bqp_file_name,"B-qp_%d_%d.b",vo_id,vol_id);
        sprintf(Pbit_file_name,"P-bits_%d_%d.b",vo_id,vol_id);
        sprintf(Ppsnr_file_name,"P-psnr_%d_%d.b",vo_id,vol_id);
        sprintf(Pqp_file_name,"P-qp_%d_%d.b",vo_id,vol_id);
#endif 

        if  ( (GetVolConfigSpriteUsage(layer_config)==STATIC_SPRITE)&&
                        ( GetVolConfigWarpParamCounter(layer_config)==-1))
                {
                PutVolConfigWarpParamCounter(0,layer_config);
                }  /* CASE I_VOP OF SPRITE STATIC */

#ifndef NLSSTATS
        /* Put some text for user (DISPLAY and STATS FILE) */
        PrintVolTimeStartStat(num_bits,vo_id,vol_id,frame,DISPLAY);
        PrintVolTimeStartStat(num_bits,vo_id,vol_id,frame,STATS_FILE);
#endif

        /* BASE VOLs are always first in the list */
        prev_base = prev_vol_store;
        rec_base = rec_vol_store;

        /* Get Previous and Reconstructed previous VOPs if they exist */
        prev_vop = GetVolVop(prev_base);
        rec_vop = GetVolVop(rec_base);  

        curr_vop_in = GetVolVop(base_layer);

	/*moved*//*AM2PDAM*/
	shape = GetVolConfigShape(layer_config);
	PutVopShape(shape,curr_vop_in);
	
        if(GetVopVopStructure_st(curr_vop_in) == FIELD_VOP)
	  ChangeRectFieldVOPstoFrameVOP_Studio(curr_vop_in);/*curr_vop_in is rectanglar*/
        ReadRectFrameVopGeneric_Studio(GetVolConfigY(layer_config),
				       GetVolConfigU(layer_config),
				       GetVolConfigV(layer_config),
				       GetVolConfigA(layer_config),
				       GetVolConfigG2_st(layer_config),/*AM2PDAM*/
				       frame,
				       IO_FORMAT,
				       curr_vop_in);
	/* This function needs curr_vop_in->shape to select the input file for alpha(s) */

#if 0

	WriteRectFrameVopGeneric_Studio(curr_vop_in,
					"tmpin",
					"tmpin",
					"tmpin",
					"tmpain",
					0,
					IO_FORMAT,
					IO_OVERWRITE,
					0);

#endif 1
	
        /* Disable shape coding if required */
/*AM2PDAM
   shape = GetVolConfigShape(layer_config);
   PutVopShape(shape,curr_vop_in);
*/
        if (shape == RECTANGULAR) {
	  for(i=0;i<MAX_AUX_COMP_NUM;i++) {
	    SetConstantImage(GetVopA_st(curr_vop_in,i,0),BINARY_ALPHA);
	    SetConstantImage(GetVopA_st(curr_vop_in,i,1),BINARY_ALPHA);
	  }
	}
        else
	  if(shape == GREY_SCALE /*|| shape == GRAY_SCALE_ONLY*//*AM2PDAM*/)
	    PutVopBinaryShape(!BINARY,curr_vop_in);     /* perhaps for Sprite Coding */
	  else
	    PutVopBinaryShape(BINARY,curr_vop_in);
	
	if ( shape == GREY_SCALE ) { /*AM2PDAM*/
	  for(i=GetVopAuxCompCount_st(curr_vop_in);i<MAX_AUX_COMP_NUM;i++) {
	    SetConstantImage(GetVopA_st(curr_vop_in,i,0),BINARY_ALPHA);
	    SetConstantImage(GetVopA_st(curr_vop_in,i,1),BINARY_ALPHA);
	  }
	} else if ( shape == BINARY || shape == BINARY_SHAPE_ONLY ) {
	  for(i=1;i<MAX_AUX_COMP_NUM;i++) {
	    SetConstantImage(GetVopA_st(curr_vop_in,i,0),BINARY_ALPHA);
	    SetConstantImage(GetVopA_st(curr_vop_in,i,1),BINARY_ALPHA);
	  }
	}	

/** for the time being removed to check the program, 990830
	if(shape == GRAY_SCALE_ONLY) {
	  SetConstantImage(GetVopY_st(curr_vop_in,0),0);
	  SetConstantImage(GetVopY_st(curr_vop_in,1),0);
	  SetConstantImage(GetVopU_st(curr_vop_in,0),0);
	  SetConstantImage(GetVopU_st(curr_vop_in,1),0);
	  SetConstantImage(GetVopV_st(curr_vop_in,0),0);
	  SetConstantImage(GetVopV_st(curr_vop_in,1),0);
	}
**/
	
	/* Need to extract bounded vop from vop read in from disk */
	/* Calc. width, height, horizontal/vertical spatial
	   reference, shape of vop (REACTANGLE or ARBITRARY),
	   and the type of alpha map (BINARY or NON-BINARY).
	   Also extract relevant image data */

	/* Frame or Field Picture selection */
	if(GetVolConfigInterlaced(layer_config) == 1 &&
	   (GetVolConfigVopStructure_st(layer_config) == 1/*ALL_FIELD_VOP*/ ||
	    (GetVolConfigVopStructure_st(layer_config) == 2/*RANDOM_VOP*/ && GetRandomValue_Studio(0,1) == 1)))
	  curr_vop_structure = FIELD_VOP;
	else
	  curr_vop_structure = FRAME_VOP;

	if(curr_vop_structure == FIELD_VOP)
	  ChangeRectFrameVOPtoFieldVOPs_Studio(curr_vop_in);

	PutVopVopStructure_st(curr_vop_structure, curr_vop_in);

        curr_vop_bb = GetVopsBounded_Studio(curr_vop_in,1,1,&vop_has_content,layer_config /* HYUNDAI 980507 */);

	for(kk=0;kk<2;kk++)
	  fprintf(stderr,"width = %d, height = %d, hor_spat = %d, ver_spat = %d\n",
		  GetVopWidth_st (curr_vop_bb,kk),
		  GetVopHeight_st(curr_vop_bb,kk),
		  GetVopHorSpatRef_st(curr_vop_bb,kk),
		  GetVopVerSpatRef_st(curr_vop_bb,kk));
	
        curr_vop_bb->frame = frame;

        /* Get RateControl type */
        /* Changed to GetEncodeControlRCType(enc_ctl) in Main: */
        /* rc_type = GetVolConfigRateControl(layer_config); */

        /* Get RateControl type */
        /* Changed to GetEncodeControlRCType(enc_ctl) in Main: */
        /* rc_type = GetVolConfigRateControl(layer_config); */

	/* 01.04.01: Studio Profile: BUGFIX */
	/*
        if(temporal_scalability != 0 && vop_has_content)
          no_padding_vop = AllocVop(GetVopWidth(curr_vop_bb),GetVopHeight(curr_vop_bb));
	*/
        PutVopRefSelCode(-1,curr_vop_bb);  
        rec_curr = VopProcess_Studio(curr_vop_bb,
                              prev_vop,
                              rec_vop,
                              GetVolVop(next_vol_store), 
                              GetVolVop(next_rec_vol_store), 
                              time,
                              layer_config,
                              vop_has_content,
                              vo_id,TRB,TRD, 
                              vo_config_list, /* UPM Global RC */
                              rc_type,        /* UPM Global RC */
                              rc_algorithm,  /* hjlee */
                              num_bits,
                              no_padding_vop,
                              temporal_scalability);

        if(temporal_scalability != 0 && vop_has_content){
            SaveBaseImage_BGC(layer_config,
                              no_padding_vop,
                              base_vop_not_padding_for_bgc,
                              rec_vop,
                              enhancement_type,
                              GetVopPredictionType(curr_vop_bb),
                              curr_vop_bb,
                              &(*base_B_vop_TPS),
                              temporal_scalability);/* 4/19 changed by SHARP */
             PrintEncodeProcess_TPS(GetVopScalability(curr_vop_bb),
                                    GetVopPredictionType(curr_vop_bb),
                                    GetVolConfigFrameBack(layer_config),
                                    GetVolConfigFrameSkip(layer_config));
        }
        
        *frame_type = GetVopPredictionType(curr_vop_bb);
        
        if (vop_has_content) {  /* added Minhua Zhou 15-05-97 */

        if (rc_type == VM5_RATE_CONTROL)
        {
#if 0
           if (( (prev_vop==NULL)||(GetVopWidth(prev_vop)==0) || 
                 (GetVopPredictionType(curr_vop_bb) == I_VOP) ) && 
               (rc_algorithm != RC_ALGORITHM_Q2))

              RC_ExcludeIFrame(vo_id, vol_id, num_bits[vo_id][vol_id].vop);

/* UPM Global RC */
            else
#endif
            {
               Int num_bits_header;            /* ME - 26.09.97 */

               if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED)
                 ;
               else {
                 num_bits_header = num_bits[vo_id][vol_id].vop
                   - num_bits[vo_id][vol_id].mot_shape_text;
                 
                 RC_UpdateModel(vo_id, vol_id,
                                num_bits[vo_id][vol_id].vop,
                                num_bits_header,
                                num_bits[vo_id][vol_id].psnr_y,
                                GetVopPredictionType(curr_vop_bb));     
               }
            }

#if 0
            /* RC2: adding combined mode rate control */
            DSRC_Update2OrderModel(vo_id, vol_id,
                                       num_bits[vol_id][vol_id].vop, 0, 1);
#endif
	 }
      }

        /* Update the VOL stores with coded vop and the original vop */
        
        vop_type = GetVopPredictionType(curr_vop_bb);
	for(i=0;i<2;i++)
	  vop_type2[i] = GetVopPredictionType_st(curr_vop_bb,i);
	
	
#ifdef _RC_DEBUG_
        if (vop_type == I_VOP){
          WRITE_INT_INT(qp_file_name, frame, GetVopIntraQuantizer(curr_vop_bb));
          WRITE_INT_INT(Iqp_file_name, frame, GetVopIntraQuantizer(curr_vop_bb));
        }
        else if (vop_type == B_VOP){
          WRITE_INT_INT(qp_file_name, frame, GetVopBQuantizer(curr_vop_bb));
          WRITE_INT_INT(Bqp_file_name, frame, GetVopBQuantizer(curr_vop_bb));
        }
        else if (vop_type == P_VOP){
          WRITE_INT_INT(qp_file_name, frame, GetVopQuantizer(curr_vop_bb));
          WRITE_INT_INT(Pqp_file_name, frame, GetVopQuantizer(curr_vop_bb));
        }
        else 
           error_exit("ERROR CodeBaseVol: Unknown frame type!\n");
#endif

        if (vop_has_content)
          {       
/* 1197-2 SONY */
            if (B_vol_store==NULL&&B_org_vol_store==NULL) {
              UpdateVolStore_Studio(curr_vop_bb,vol_id,next_vol_store);
              UpdateVolStore_Studio(rec_curr,vol_id,next_rec_vol_store);
            }
            else {
              UpdateVolStore_Studio(curr_vop_bb,vol_id,B_org_vol_store);
              UpdateVolStore_Studio(rec_curr,vol_id,B_vol_store);
            }
/*                    else {
                             UpdateVolStore(rec_curr,vol_id,B_vol_store);
                             FreeVop(curr_vop_bb);
                            }
*/
/* 1197-2 */


	  }

        /* If the VOP is empty but the previous VOP exists, then the VO
            has disappeared. In this case the VOL stores are updated with
            a NULL VOP so that the disappearing VO is not composited
            onto the ouput display image - NOC 14/01/97 */
        else if((!vop_has_content) && (prev_vop != NULL))
                {
                  /* MW 30-NOV-1998 */
                  /**/
                  /* VOP composition depends on width==0 up to now, so */
                  /* VOL store should be updated for empty VOPs with */
                  /* the empty VOP; otherwise, during prediction it can */
                  /* not be recognized that the target VOP is empty if it */
                  /* is set to NULL in the VOL store */
                  /**/
                  /*   if (B_vol_store==NULL) {*/
                  /*         UpdateVolStore(NULL,vol_id,next_vol_store);*/
                  /*         UpdateVolStore(NULL,vol_id,next_rec_vol_store);*/
                  /*        } else UpdateVolStore(NULL,vol_id,B_vol_store);*/
                  if (B_vol_store==NULL) {
                    UpdateVolStore_Studio(curr_vop_bb,vol_id,next_vol_store);
                    UpdateVolStore_Studio(curr_vop_bb,vol_id,next_rec_vol_store);
                  } else UpdateVolStore_Studio(curr_vop_bb,vol_id,B_vol_store);
		}

        output_frame = (Int) (time*GetVolConfigFrameRate(layer_config)/1000. + .5);

        if ((rc_type == VM4_RATE_CONTROL)&&(vop_has_content)) /* modified by Minhua Zhou 15-05-97 */    
                {
                end_time = GetVolConfigEndTime(layer_config);
        
                if (((time - GetVolConfigStartTime(layer_config)) != 0) &&
                        (time < end_time))
                        {

                        vop_quantizer = GetVolConfigQuantizer(layer_config);
                        target_bitrate = GetVolConfigBitrate(layer_config);
                        temp_ref = 1000 / GetVolConfigFrameRate(layer_config);
          
                        QuantAdjust (target_bitrate,
                                        temp_ref,
                                        end_time,
                                        time,
                                        num_bits[vo_id][vol_id].average,
                                        num_bits[vo_id][vol_id].vop,
                                        &vop_quantizer);
                        fprintf(stdout, ">>>>> New quantizer= %d\n", (int)vop_quantizer);
          
                        PutVolConfigQuantizer(vop_quantizer,layer_config);
                        }
	      }

        /* Write the coded vop to disk if required */
        if (GetVolConfigWriteCoded(layer_config)&&(vop_has_content)) {/* modified by M.Eckert, 04.02.98 */
#if 0
	  {
		  char sprite_g[3][100] = { "just_g0", "just_g1", "just_g2"};
/*
		  CopyImage(GetVopG_st(rec_curr,1,0),GetVopA_st(rec_curr,1,0));
		  CopyImage(GetVopG_st(rec_curr,1,1),GetVopA_st(rec_curr,1,1));
*/		  
		  WriteRectFrameVopGeneric_Studio(rec_curr,
						"justsprite",
						"justsprite",
						"justsprite",
						"just_a",
						sprite_g,
						0,
						" ",
						"OVERWRITE",
						1);
		}
#endif 0
	  
	  if (((TRD>1)&&TRB)|| (TRD==1) ||(prev_vop==NULL))
	    WriteCodedVopToDisk_Studio(rec_curr,
				       GetVolConfigCodedY(layer_config),
				       GetVolConfigCodedU(layer_config),
				       GetVolConfigCodedV(layer_config),
				       GetVolConfigCodedA(layer_config),
				       GetVolConfigCodedG2_st(layer_config),/*AM2PDAM*/
				       output_frame,
				       GetVolConfigDiskSeqX(layer_config),
				       GetVolConfigDiskSeqY(layer_config),
				       GetVolConfigShape(layer_config),
				       vop_has_content);
	  if ((TRD>1)&&(TRB==TRD-1))    /* next P-VOP */
	    WriteCodedVopToDisk_Studio(GetVolVop(next_rec_vol_store),
				       GetVolConfigCodedY(layer_config),
				       GetVolConfigCodedU(layer_config),
				       GetVolConfigCodedV(layer_config),
				       GetVolConfigCodedA(layer_config),
				       GetVolConfigCodedG2_st(layer_config),/*AM2PDAM*/
				       output_frame,
				       GetVolConfigDiskSeqX(layer_config),
				       GetVolConfigDiskSeqY(layer_config),
				       GetVolConfigShape(layer_config),
				       vop_has_content);
	  
	}
	
        /* Put some text for user */
        PrintVolTimeEndStat_Studio(num_bits,vo_id,vol_id,rec_curr,vop_type2[0],DISPLAY);
        PrintVolTimeEndStat_Studio(num_bits,vo_id,vol_id,rec_curr,vop_type2[0],STATS_FILE);
	
#if 0	
#ifdef NLSSTATS
        NLSPrintVolTimeEndStat(&num_bits[vo_id][vol_id], rec_curr, time, stdout);
        NLSPrintVolTimeEndStat(&num_bits[vo_id][vol_id], rec_curr, time, fpstats_file);
#else
        PrintVolTimeEndStat_Studio(num_bits,vo_id,vol_id,rec_curr,vop_type2[0],DISPLAY);
        PrintVolTimeEndStat_Studio(num_bits,vo_id,vol_id,rec_curr,vop_type2[0],STATS_FILE);
#endif
#endif 0

#ifdef _RC_DEBUG_
        WRITE_INT_INT(bit_file_name, frame, num_bits[vo_id][vol_id].vop);
        WRITE_INT_FLOAT(psnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);

        if (vop_type == I_VOP){
          WRITE_INT_INT(Ibit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Ipsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else if (vop_type == B_VOP){
          WRITE_INT_INT(Bbit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Bpsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else if (vop_type == P_VOP){
          WRITE_INT_INT(Pbit_file_name, frame, num_bits[vo_id][vol_id].vop);
          WRITE_INT_FLOAT(Ppsnr_file_name, frame, num_bits[vo_id][vol_id].psnr_y);
        }
        else 
           error_exit("ERROR CodeBaseVol: Unknown frame type!\n");
#endif


        if (RCQ2_MVO_CHECK() >= RCQ2_MVO_ENABLED) 
          ;
        else {
          /* Reset bit counts for the next time this VOL is coded */
          num_bits[vo_id][vol_id].syntax = 0;
          num_bits[vo_id][vol_id].shape = 0;
          num_bits[vo_id][vol_id].texture = 0;
/*	  for(i=0;i<MAX_NUM_SLICE_LOG;i++)
	    num_bits[vo_id][vol_id].slice[i] = 0;*//*AM2PDAM*/
          num_bits[vo_id][vol_id].sprite_piece = 0;
          num_bits[vo_id][vol_id].motion = 0;
          num_bits[vo_id][vol_id].mot_shape_text = 0;
          /* num_bits[vo_id][vol_id].vop = 0; -> set 0 in RC_UpdateBuffer! */
        }

        /* Increment occurrence counter for this VOL */
        num_bits[vo_id][vol_id].occurrence++;
      
      if(temporal_scalability != 0){
        if(no_padding_vop != NULL)
                 FreeVop(no_padding_vop);
        if(no_padding_sprite != NULL)
                 FreeVop(no_padding_sprite);
      }

        return;
}

/***********************************************************CommentBegin******
 *
 * -- BitstreamPutVols_Studio -- Write VOL bitstreams to disk
 *
 * Author :             
 *      Noel O'Connor Teltec Irl.
 *
 * Created :            
 *      30-06-96
 *
 * Purpose :            
 *      Writes the VOLs to disk for all layers in the coding process.
 * 
 * Arguments in :       
 *      VO *list - Pointer to linked list of VOs which contains the VOLs
 *      whose syntax fields are to be written to disk
 *      
 * Arguments in/out :   
 *      BitCount num_bits - structure for the number of bits written to disk
 *
 * Arguments out :      
 *      none
 *
 * Return values :      
 *      none 
 *
 * Side effects :       
 *      none
 *
 * Description :        
 *      -
 *
 * See also :
 *      -
 *
 * Modified :           
 *      21.01.97 Added bytealignment fix by Luis
 *      28.01.97 Robert Danielsen: Small change in zigzag-scan
 * 26.02.97 Noel O'Connor: added shape_effects to bitstream
 *     23.04.97 M.Wollborn: MPEG quantization completely added
 *     26.04.97 Luis Ducla-Soares: Added data_partitioning flag to the bitstream.
 *     12.05.97 Minhua Zhou: added "not_8_bit", "advanced_prediction_disable", etc. 
 * 14.05.97 F. Jaureguizar: Modification of BitstreamPutBits() calls for
 *    FCode's . From 2 bits to 3 bits (VM7.0).
 *    18.07.97 Minhua Zhou: added video_object_start_code
 *    04.08.97 Minhua Zhou: removed f_codes from VOL 
 *    06.08.97 Noel Brady: Binary Shape Only switches out several irrelevant
 *              fields in the VOL header
 *    11.08.97 Minhua Zhou: changed quant. matrix coding
 *    12.08.97 Minhua Zhou: added time_increment_resolution
 *    08.09.97 Cecile Dufour: added input arguments: vo_config_list and rc_type
 *                              to encode sprite_pieces
 *    05.11.97 Minhua Zhou:   added flags according to CD
 *    23.03.98 Michael Wollborn: moved "random_accessible_vol" due to N2171, Clause 2.1.7
 *                               included user_data() due to N2171, Clause 2.1.9
 *    26.03.98 Michael Wollborn: changes due to N2171, Clause 2.5.1/14
 *    21.05.98 Ji Heon Kweon (HYUNDAI) : support for grayscale coding 
 *    18.01.99 Takefumi Nagumo: added hierarchy_type.
 *    26.07.99 Osamu Sunohara (SONY): For Studio Profile
 *
 ***********************************************************CommentEnd********/
 
Void
BitstreamPutVOLs_Studio(VO *vo_list,BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
                 VOConfig *vo_config_list, Int rc_type)
{
  VO                    *vo;
  
  Vol           *layer;
  
  Image         *header_bitstream;
  
  Int           i, j,vol_id, vo_id, tmp_val;
  VOConfig      *vo_config;
  VolConfig     *vol_config;
  Int *qmat;

  Int vol_shape_extension;

  Int code_vol_shape_ext[6][6][6];
  Int ii, jj, kk;
  /* initialize code_vol_shape_ext[][][] */
  for(kk=0;kk<6;kk++)
    for(jj=0;jj<6;jj++)
      for(ii=0;ii<6;ii++)
	code_vol_shape_ext[kk][jj][ii] = -1;
  code_vol_shape_ext[1][0][0] = 0;
  code_vol_shape_ext[2][0][0] = 1;
  code_vol_shape_ext[1][2][0] = 2;
  code_vol_shape_ext[2][2][0] = 3;
  code_vol_shape_ext[1][2][2] = 4;
  code_vol_shape_ext[3][0][0] = 5;
  code_vol_shape_ext[1][3][0] = 6;
  code_vol_shape_ext[4][0][0] = 7;
  code_vol_shape_ext[5][0][0] = 8;
  code_vol_shape_ext[5][5][0] = 9;
  code_vol_shape_ext[5][5][5] = 10;
  code_vol_shape_ext[1][5][0] = 11;
  code_vol_shape_ext[1][5][5] = 12;
  code_vol_shape_ext[0][0][0] = 15;
  
  vo = vo_list;
  
  while(vo != NULL)
    {
      vo_id = GetVOId(vo);
      vo_config = GetVOConfigById(vo_config_list,vo_id);
      
      layer = GetVOLayers(vo);
      
      while(layer != NULL)
        {
          /* Get VOL Id as index for accessing bitstreams and bit counts */
          vol_id = GetVolId(layer);
          vol_config = GetVolConfigById(GetVOConfigLayers(vo_config),vol_id);
          
          /* Set up intermediate integer level data structure */
          
          header_bitstream = BitstreamInit();
          
          /* 
           *
           * Write all syntax fields in VOL header to data structure
           *
           */

          BitstreamPutBits(header_bitstream,VO_START_CODE,27);
          BitstreamPutBits(header_bitstream,vo_id,5);

          BitstreamPutBits(header_bitstream,VOL_START_CODE,VOL_START_CODE_LENGTH);
          BitstreamPutBits(header_bitstream,GetVolId(layer),4);

          /* Included due to N2171, Clause 2.1.7 MW 23-MAR-1998 */
          PutVolRandomAccessibleVol(0,layer);
          BitstreamPutBits(header_bitstream,GetVolRandomAccessibleVol(layer),1);

          /* 15.12.98 Sven Brandau: add video_object_type_indication, see N2502(a), Clause 6.2.4 */
	  /* Syntax_011203 */
	  /* Encode as the Core Studio Object Type */
	  BitstreamPutBits(header_bitstream,15/*0*/,8);

/*010320*/	  
	  BitstreamPutBits(header_bitstream, 4, 4);
/*	  
          PutVolIsObjectLayerIdentifier(0,layer);
          BitstreamPutBits(header_bitstream,GetVolIsObjectLayerIdentifier(layer),1);
          if (GetVolIsObjectLayerIdentifier(layer)) {
            BitstreamPutBits(header_bitstream, GetVolVisualObjectLayerVerid(layer), 4);
            BitstreamPutBits(header_bitstream, GetVolVisualObjectLayerPriority(layer), 3);
          }
*//*010320*/
          
          /* video_object_layer_shape */
          if (GetVolShape(layer) == RECTANGULAR)
            BitstreamPutBits(header_bitstream,0,2);
          else if (GetVolShape(layer) == BINARY)
	    BitstreamPutBits(header_bitstream,1,2);
          else if (GetVolShape(layer) == BINARY_SHAPE_ONLY)
	    BitstreamPutBits(header_bitstream,2,2);
          else if (GetVolShape(layer) == GREY_SCALE)
	    BitstreamPutBits(header_bitstream,3,2);
          else
	    exit(3);

	  /* video_object_layer_shape_extension *//*AM2PDAM*/
	  fprintf(stdout, "aux_comp_type : [%d][%d][%d]\n",
		  GetVolAuxCompType_st(layer,0),
		  GetVolAuxCompType_st(layer,1),
		  GetVolAuxCompType_st(layer,2));
	  vol_shape_extension =
	    code_vol_shape_ext[GetVolAuxCompType_st(layer,0)][GetVolAuxCompType_st(layer,1)][GetVolAuxCompType_st(layer,2)];
	  if(vol_shape_extension == -1) {
	    fprintf(stderr, "Invalid aux_comp_type assignment\n");
	    exit(3);
	  }
	  BitstreamPutBits(header_bitstream,vol_shape_extension,4);
fprintf(stdout,"vol_shape_extension = %d\n", vol_shape_extension);
          /* progressive_sequence */
          BitstreamPutBits(header_bitstream,!GetVolInterlaced(layer), 1);/*Put Progressive_sequence*/

          if (GetVolShape(layer) != BINARY_SHAPE_ONLY) {

	    /*v001226_0 syntax*/
	    /*rgb_components*/
	    BitstreamPutBits(header_bitstream, 0, 1);
	    
	    /* chroma_format */
	    if(GetVolChromaFormat_st(layer) == CHROMA420)
	      BitstreamPutBits(header_bitstream, 1, 2);
	    else if(GetVolChromaFormat_st(layer) == CHROMA422)
	      BitstreamPutBits(header_bitstream, 2, 2);
	    else if(GetVolChromaFormat_st(layer) == CHROMA444)
	      BitstreamPutBits(header_bitstream, 3, 2);
	    else
	      exit(3);
	    
	    /* bit_per_pixel */
	    BitstreamPutBits(header_bitstream,GetVolBitsPerPixel(layer),4);
	  }

	  if(GetVolShape(layer) == RECTANGULAR)
	    {
	      /* vol_width and vol_height */
	      BitstreamPutBits(header_bitstream,1,1);
	      BitstreamPutBits(header_bitstream,GetVolWidth(layer),14);/*STP*/
	      BitstreamPutBits(header_bitstream,1,1);
	      BitstreamPutBits(header_bitstream,GetVolHeight(layer),14);/*STP*/
	      BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
	    }	  
	  
          /* 11.08.98 Sven Brandau:  "define aspect_ratio_info clearly" due to N2339, Clause 2.1.17 */
          PutVolAspectRatioInfo( 1, layer ); /* -->const. value only for the test phase */
          BitstreamPutBits( header_bitstream, GetVolAspectRatioInfo(layer), 4 );
          if (GetVolAspectRatioInfo(layer)==EXTENDED_PAR) {
            BitstreamPutBits( header_bitstream, GetVolAspectRatioInfoParWidth(layer),  8 );
            BitstreamPutBits( header_bitstream, GetVolAspectRatioInfoParHeight(layer), 8 );         
          }

          /* frame_rate_code (gotten from 'vol_config') */
          BitstreamPutBits(header_bitstream, GetVolConfigFrameRateCode_st(vol_config), 4);	    

	  /* first_half_bit_rate */
	  BitstreamPutBits(header_bitstream,0,15);
	  BitstreamPutBits(header_bitstream,1,1);
	  /* latter_half_bit_rate */
	  BitstreamPutBits(header_bitstream,0,15);
	  BitstreamPutBits(header_bitstream,1,1);
	  /* first_half_vbv_buffer_size */
	  BitstreamPutBits(header_bitstream,0,15);
	  BitstreamPutBits(header_bitstream,1,1);
	  /* latter_half_vbv_buffer_size */
	  BitstreamPutBits(header_bitstream,0,3);
	  /* first_half_vbv_occupancy */
	  BitstreamPutBits(header_bitstream,0,11);
	  BitstreamPutBits(header_bitstream,1,1);
	  /* latter_half_vbv_occupancy */
	  BitstreamPutBits(header_bitstream,0,15);
	  BitstreamPutBits(header_bitstream,1,1);

	  /* low_delay */
	  /* Syntax_011203 */
	  BitstreamPutBits(header_bitstream,1/*0*/,1);

	  /*v001226_0 syntax*/
	  /*mpeg2_stream*/
#ifdef _STUDIO_CONF_5_
          BitstreamPutBits(header_bitstream,1,1);
#else
	  BitstreamPutBits(header_bitstream,0,1);
#endif _STUDIO_CONF_5_
	  
	  /* auxiliary components related */
	  if(GetVolShape(layer) == GREY_SCALE) {
	    for(i=0; i<GetVolAuxCompCount_st(layer); i++) {
	      /* alpha_bits_per_pixel[i] */
	      BitstreamPutBits(header_bitstream,GetVolGrayBitsPerPixel_st(layer,i),4);
	      /* minimum_alpha_level[i] */
	      BitstreamPutBits(header_bitstream,GetVolMinimumAlphaLevel_st(layer,i),12);
	      /* MoMuSys-2000FDAM1COR-020612 *//*marker_bit*/
	      BitstreamPutBits(header_bitstream,1,1);
	      /* maximum_alpha_level[i] */
	      BitstreamPutBits(header_bitstream,GetVolMaximumAlphaLevel_st(layer,i),12);
	    }
	    /* composition_method : Not Supported */
	    BitstreamPutBits(header_bitstream,1,1);
	    /* linear_composition : Not Supported */
	    BitstreamPutBits(header_bitstream,1,1);
	  }
	  
	  if(GetVolShape(layer) != BINARY_SHAPE_ONLY) {
	    /* sprite_enable */
	    BitstreamPutBits(header_bitstream,GetVolSpriteUsage(layer),1);
fprintf(stdout, "SPRITE USAGE = %d\n", GetVolSpriteUsage(layer));
            if( GetVolSpriteUsage(layer) ==  STATIC_SPRITE ) {
	      BitstreamPutBits(header_bitstream, GetVolSpriteHdim(layer), 20);
	      BitstreamPutBits(header_bitstream,1,1);
	      BitstreamPutBits(header_bitstream, GetVolSpriteVdim(layer), 20);/*long int*/
	      BitstreamPutBits(header_bitstream,1,1);

	      tmp_val = GetVolSpriteLeftEdge(layer);
	      if (tmp_val<0) 
		tmp_val += 1048576; /* 2 complemented for signed int */

	      BitstreamPutBits(header_bitstream, GetVolSpriteLeftEdge(layer)/*tmp_val*/, 20);
	      BitstreamPutBits(header_bitstream,1,1);

	      tmp_val = GetVolSpriteTopEdge(layer);
	      if (tmp_val<0) 
		tmp_val += 1048576; /* 2 complemented for signed int */

	      BitstreamPutBits(header_bitstream, GetVolSpriteTopEdge(layer)/*tmp_val*/, 20);
	      BitstreamPutBits(header_bitstream,1,1);
	      
	      BitstreamPutBits(header_bitstream, GetVolNoOfSpritePoints(layer), 6);
	      BitstreamPutBits(header_bitstream, GetVolWarpingAccuracy(layer), 2);
	      BitstreamPutBits(header_bitstream, GetVolBrightnessChangeInSprite(layer), 1);
	      /*000705FPDAM*/
	      BitstreamPutBits(header_bitstream, GetVolSpriteDefocusing_st(layer), 1);
	      /*BitstreamPutBits(header_bitstream, GetVolLowLatencySpriteEnable(layer), 1);*/
	      /*000705FPDAM*/
	      
	    }
	    

#if 0 /*000624FPDAM*/
	    /* scalability */
	    BitstreamPutBits(header_bitstream,GetVolScalability(layer),1);
#endif 0 /*000624FPDAM*/
	  }


#if 0 /*AM2PDAM*/
          PutVolVolControlParameters(0,layer); /* always disabled */
          BitstreamPutBits(header_bitstream,GetVolVolControlParameters(layer),1);

          /* frame_rate_code (gotten from 'vol_config') */
          BitstreamPutBits(header_bitstream, GetVolConfigFrameRateCode_st(vol_config), 4);	    
	  BitstreamPutBits(header_bitstream, 0, 2);/*frame_rate_extension_n*/	  
          BitstreamPutBits(header_bitstream, 0, 5);/*frame_rate_extension_d*/
	  
          if (GetVolShape(layer) != BINARY_SHAPE_ONLY) {/* BSO_NOEL */
            
            if(GetVolShape(layer) == RECTANGULAR)
              {
                BitstreamPutBits(header_bitstream,1,1);
                BitstreamPutBits(header_bitstream,GetVolWidth(layer),14);/*STP*/
                BitstreamPutBits(header_bitstream,1,1);
                BitstreamPutBits(header_bitstream,GetVolHeight(layer),14);/*STP*/
                /* 10.08.98 Sven Brandau:  "insert marker_bit" due to N2339, Clause 2.1.6 */
                BitstreamPutBits(header_bitstream, MARKER_BIT, 1);
              }
	    
/*            BitstreamPutBits(header_bitstream,GetVolOBMCDisable(layer),1);*/
/*            BitstreamPutBits(header_bitstream,GetVolSpriteUsage(layer),1);*/
#if 0
            if( GetVolSpriteUsage(layer) ==  STATIC_SPRITE )
              {
                    BitstreamPutBits(header_bitstream, GetVolSpriteHdim(layer), 13);
                    BitstreamPutBits(header_bitstream,1,1);
                    BitstreamPutBits(header_bitstream, GetVolSpriteVdim(layer), 13);
                    BitstreamPutBits(header_bitstream,1,1);
                    tmp_val = GetVolSpriteLeftEdge(layer);
                    if (tmp_val<0) 
                      tmp_val +=8192; /* 2 complemented for signed int */
                    BitstreamPutBits(header_bitstream, tmp_val, 13);
                    BitstreamPutBits(header_bitstream,1,1);
                    tmp_val = GetVolSpriteTopEdge(layer);
                    if (tmp_val<0) 
                      tmp_val +=8192; /* 2 complemented for signed int */
                    BitstreamPutBits(header_bitstream, tmp_val, 13);
                    BitstreamPutBits(header_bitstream,1,1);
                
                BitstreamPutBits(header_bitstream, GetVolNoOfSpritePoints(layer), 6);
                BitstreamPutBits(header_bitstream, GetVolWarpingAccuracy(layer), 2);
                BitstreamPutBits(header_bitstream, GetVolBrightnessChangeInSprite(layer), 1);
                BitstreamPutBits(header_bitstream, GetVolLowLatencySpriteEnable(layer), 1);
                
	       }
#endif 0

	    /* Syntax elements only for Texture */
	    if(GetVolShape(layer) != GRAY_SCALE_ONLY) {
	      /* chroma_format */
	      if(GetVolChromaFormat_st(layer) == CHROMA420)
		BitstreamPutBits(header_bitstream, 1, 2);
	      else if(GetVolChromaFormat_st(layer) == CHROMA422)
		BitstreamPutBits(header_bitstream, 2, 2);
	      else if(GetVolChromaFormat_st(layer) == CHROMA444)
		BitstreamPutBits(header_bitstream, 3, 2);
	      else
		exit(3);
	      
#if 0 /* remove WD1.3 */
	      /* sprite_enable */
	      BitstreamPutBits(header_bitstream,GetVolSpriteUsage(layer),1);
#endif 0
	      /* bit_per_pixel */
              BitstreamPutBits(header_bitstream,GetVolBitsPerPixel(layer),4);

#if 0 /* remove WD1.3 */
	      /* scalability */
	      BitstreamPutBits(header_bitstream,GetVolScalability(layer),1);
#endif 0
	    }

	    /* Syntax element only for grayscale shape */
	    if(GetVolShape(layer) == GREY_SCALE || GetVolShape(layer) == GRAY_SCALE_ONLY) {
	      /* gray_bits_per_pixel */
	      BitstreamPutBits(header_bitstream,GetVolGrayBitsPerPixel_st(layer),4);

	      /* minimum/maximum_alpha_level */
	      BitstreamPutBits(header_bitstream,4095,12);
	      BitstreamPutBits(header_bitstream,4095,12);

              BitstreamPutBits(header_bitstream,1,1); /* composition_method : Not Supported */
              BitstreamPutBits(header_bitstream,1,1); /* linear_composition : Not Supported */
	    }

#if 0 /*WD1.3*/  
	    /* the following flags are necessary ?? for Studio Profile */
            /* HYUNDAI (Grayscale) */
            if(GetVolShape(layer) == GREY_SCALE) {
              BitstreamPutBits(header_bitstream,
                               GetVolDisableGrayQuantUpdate(layer),1);

              BitstreamPutBits(header_bitstream,1,1); /* composition_method : Not Supported */
              BitstreamPutBits(header_bitstream,1,1); /* linear_composition : Not Supported */
            }
#endif

/*            BitstreamPutBits(header_bitstream,GetVolQuantType(layer),1);*/

#if 0 /* WD1.3 */
            /* Changed stuff for MPEG quantization (23-APR-1997, MW) */
            if (1/*GetVolQuantType(layer)*/)
              {

		/* Quant Matrices only for Texture */
		if(GetVolShape(layer) != GRAY_SCALE_ONLY) {

		  /* Intra Quant Matrix */
		  BitstreamPutBits(header_bitstream,GetVolLoadIntraQuantMat(layer),1);
		  if(GetVolLoadIntraQuantMat(layer)) {
		    qmat = GetVolIntraQuantMat(layer);
		    for (j=63;j>=1;j--) 
		      if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
			break;
		    if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
		      j=0;   
		    for(i=0; i<j+1; i++)                                  
		      BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
		    if (j<63) 
		      BitstreamPutBits(header_bitstream,0,8);
		  }

		  /* Nonintra Quant Matrix */
		  BitstreamPutBits(header_bitstream,GetVolLoadNonintraQuantMat(layer),1);
		  if(GetVolLoadNonintraQuantMat(layer)) {
		    qmat = GetVolNonintraQuantMat(layer); 
		    for (j=63;j>=1;j--) 
		      if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
			break;
		    if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
		      j=0;   
		    for(i=0; i<j+1; i++)                                  
		      BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
		    if (j<63) 
		      BitstreamPutBits(header_bitstream,0,8);
		  }
		}
	      
		/* Quant Matrices only for Grayscale shape */
                if(GetVolShape(layer) == GREY_SCALE || GetVolShape(layer) == GRAY_SCALE_ONLY)
                  {
		    /* Grayscale Intra Quant Matrix */
                    BitstreamPutBits(header_bitstream,GetVolLoadGrayIntraQuantMat(layer),1);
                    if(GetVolLoadGrayIntraQuantMat(layer)) {
                      qmat = GetVolGrayIntraQuantMat(layer);
                      for (j=63;j>=1;j--) 
                       if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
                         break;
                      if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
                        j=0;   
                      for(i=0; i<j+1; i++)                                      
                        BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
                      if (j<63) BitstreamPutBits(header_bitstream,0,8);
                    }

		    /* Grayscale Nonintra Quant Matrix */
                    BitstreamPutBits(header_bitstream,GetVolLoadGrayNonintraQuantMat(layer),1);
                    if(GetVolLoadGrayNonintraQuantMat(layer)) {
                      qmat = GetVolGrayNonintraQuantMat(layer);
                      for (j=63;j>=1;j--) 
                        if (qmat[*(zigzag_i+j)]!=qmat[*(zigzag_i+j-1)]) 
                          break;
                      if ((j==1)&&(qmat[*(zigzag_i+j)]==qmat[*(zigzag_i+j-1)]))
                        j=0;   
                      for(i=0; i<j+1; i++)                                      
                        BitstreamPutBits(header_bitstream,qmat[*(zigzag_i+i)],8);
                      if (j<63) 
                        BitstreamPutBits(header_bitstream,0,8);
                    }
                  }
              }
#endif 0

#if 0
            /* added by Minhua Zhou,complexity_estimation_disable */
            /* Complexity Estimation syntax support - Marc Mongenet (EPFL) - 3 Jun 1998 */
            BitstreamPutBits(header_bitstream, GetVolComplexityEstimationDisable(layer), 1);
            if (!GetVolComplexityEstimationDisable(layer))
              PutDefineVOPComplexityEstimationHeaderToBitstream(layer,
                                                                header_bitstream);

            /* Modified due to N2171 Cl. 2.5.1/14 MW 26-MAR-1998 */
            /* BitstreamPutBits(header_bitstream,GetVolErrorResDisable(layer),1); */
            /* if(!GetVolErrorResDisable(layer)) */
            /*   { */
            /*     BitstreamPutBits(header_bitstream,GetVolDataPartEnable(layer),1); */
            /*     BitstreamPutBits(header_bitstream,GetVolReverseVlc(layer),1); */
            /*   } */


            /* 19.08.98 Sven Brandau: "insert resync_marker_disable" due to N2339, Clause 3.5.1 */
            /* GetVolErrorResDisable have to be changed to GetVolErrorResDisable in a later time */
            BitstreamPutBits(header_bitstream, GetVolErrorResDisable(layer), 1);

            BitstreamPutBits(header_bitstream,GetVolDataPartEnable(layer),1);
            
            if(GetVolDataPartEnable(layer))
              BitstreamPutBits(header_bitstream,GetVolReverseVlc(layer),1);
            else if(GetVolReverseVlc(layer))
              {
                printf("WARNING: RVLCs can only be used if data partitioning is enabled\n");
                printf("WARNING: RVLCs will not be used!!!\n");
                PutVolReverseVlc(0,layer);
              }

            PutVolACDCPredDisable(0,layer); /* AC/DC prediction is always enabled */

            BitstreamPutBits(header_bitstream,GetVolScalability(layer),1);
            
            if(GetVolScalability(layer))
              {
                BitstreamPutBits(header_bitstream,GetVolHierarchyType(layer),1);
                BitstreamPutBits(header_bitstream,GetVolRefId(layer),4);
                BitstreamPutBits(header_bitstream,GetVolRefSampDir(layer),1);
                BitstreamPutBits(header_bitstream,GetVolHorSampN(layer),5);
                BitstreamPutBits(header_bitstream,GetVolHorSampM(layer),5);
                BitstreamPutBits(header_bitstream,GetVolVerSampN(layer),5);
                BitstreamPutBits(header_bitstream,GetVolVerSampM(layer),5);
                if(GetVolEnhanceType(layer)==2)
                  PutVolEnhanceType(1,layer);
                BitstreamPutBits(header_bitstream,GetVolEnhanceType(layer),1);
              }

            /* Removed due to N2171, Clause 2.1.7 MW 23-MAR-1998 */
            /* PutVolRandomAccessibleVol(0,layer); */
            /* BitstreamPutBits(header_bitstream,GetVolRandomAccessibleVol(layer),1); */
#endif 0
#endif 0 /*AM2PDAM*/            
            /*
             *
             * Write intermediate bitstream data structure to disk
             *
             */

            num_bits[vo_id][vol_id].vol += BitstreamPut(header_bitstream,vo_id,vol_id);
            /* No longer require the intermediate data structure */
            BitstreamFree(header_bitstream);
            /* should this also be counted ??? -> YES/MW 23-MAR-1998 */
            num_bits[vo_id][vol_id].vol += NextStartCode_Studio(vo_id,vol_id, GetRandomValue_Studio(0,5));

	    BitstreamPutExtensionsInVOL_Studio(layer, vo_id, vol_id);



	    
            /* Included for user data due to N2171, Cl. 2.1.9 MW 23-MAR-1998 */
            if(GetVolConfigIsUserDataInVol(vol_config))
              num_bits[vo_id][vol_id].vol += PutUserDataToBitstream(
                GetVolConfigUserDataInVolFile(vol_config),vo_id,vol_id);

            if( GetVolSpriteUsage(layer) == STATIC_SPRITE&&
                                GetVolLowLatencySpriteEnable(layer)==0)
              {
		fprintf(stderr, "Before EncodeBasicSprite_Studio\n");
		fprintf(stderr, "AuxCompCount = %d\n", GetVolConfigAuxCompCount_st(vol_config));
                EncodeBasicSprite_Studio(vol_config,
					 vo_id, vol_id,
					 num_bits,
					 vo_config_list,
					 rc_type);
		fprintf(stderr, "After EncodeBasicSprite_Studio\n");
                num_bits[vo_id][vol_id].vol += num_bits[vo_id][vol_id].sprite_piece;
                PutVolConfigSpriteTransmitMode(STOP, vol_config);

		WriteRectFrameVopGeneric_Studio(vol_config->p_orisprite,
						"orisprite",
						"orisprite",
						"orisprite",
						"tmpain",
						"tmpgin",
						0,
						IO_FORMAT,
						IO_OVERWRITE,
						0);
		{
		  char sprite_g[3][100] = { "sprite_g0", "sprite_g1", "sprite_g2"};

		  WriteRectFrameVopGeneric_Studio(vol_config->psprite,
						"decsprite",
						"decsprite",
						"decsprite",
						"sprite_a",
						sprite_g,
						0,
						IO_FORMAT,
						IO_OVERWRITE,
						1);
	      }
              }
#if 0 /*AM2PDAM*/
	  } /* BSO_NOEL */
#endif 0 /*AM2PDAM*/
#if 0
          else /* BINARY SHAPE ONLY */
            {
#if 0
              /* 19.08.98 Sven Brandau: "change error_resilient_disable to resync_marker_disable"
                 due to N2339, Clause 3.5.1 */
              /* GetVolErrorResDisable have to be changed to GetVolErrorResDisable in a later time */
              BitstreamPutBits(header_bitstream,GetVolErrorResDisable(layer),1);
#endif 0
              num_bits[vo_id][vol_id].vol += BitstreamPut(header_bitstream,vo_id,vol_id);
              /* No longer require the intermediate data structure */
              BitstreamFree(header_bitstream);
              NextStartCode_Studio(vo_id,vol_id,GetRandomValue_Studio(0,5));
              
              /* Included for user data due to N2171, Cl. 2.1.9 MW 23-MAR-1998 */
              if(GetVolConfigIsUserDataInVol(vol_config))
                num_bits[vo_id][vol_id].vol += PutUserDataToBitstream(
                  GetVolConfigUserDataInVolFile(vol_config),vo_id,vol_id);
            }
#endif 0     
          layer = GetVolNext(layer);
        }
      
      vo = GetVONext(vo);
    }
  
  return;
}

Void
UpdateVolStore_Studio(Vop *update_vop,
		      Int layer_id,
		      Vol *vol_store)
{
        Vol             *replace_vol;
        
        Vop             *replace_vop;

        /* Get VOL whose VOP is to be updated */
        replace_vol = GetVolById(vol_store,layer_id);

        /* Get VOP to be replaced */
        replace_vop = GetVolVop(replace_vol);

        /* Check if VOP to be replaced exists or not */
        if(replace_vop == NULL)
                PutVolVop(update_vop,replace_vol);

        else
                {
                FreeVop_Studio(replace_vop);/*AM2PDAM*/
                PutVolVop(update_vop,replace_vol);
                }

        return;
}

VO *
InitVOStore_Studio(VO *list)
{
        VO      *store = NULL,
                        *vo,
                        *node;

        Vol     *vol_list,
                        *node_vol_list;

        vo = list;

        while(vo != NULL)
                {
                node = SallocVO();

                PutVOId(GetVOId(vo),node);

                vol_list = GetVOLayers(vo);

                node_vol_list = InitVolStore_Studio(vol_list);

                PutVOLayers(node_vol_list,node);

                if(store == NULL)
                        store = node;
                else
                        AddVOToList(node,store);

                vo = GetVONext(vo);
                }

        return(store);
}

Vol *
InitVolStore_Studio(Vol *list)
{
        Vol     *store = NULL,
                        *layer,
                        *node;

        layer = list;

        while(layer != NULL)
                {
                node = SallocVol();

                CopyVolSyntax_Studio(layer,node);

                if(store == NULL)
                        store = node;
                else
                        AddVolToList(node,store);

                layer = GetVolNext(layer);
                }

        return(store);
}

Void
InitBitCounts_Studio(VOConfig *vo_cfg_list, 
		     Char *stats_file,
		     BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS])
{
        VOConfig                *vo_cfg;

        VolConfig       *vol_cfg;

        Int                     vo_id,
                                        vol_id,
                                        sep_mot_shape_text;
/*	Int i;*//*AM2PDAM*//*Studio Profile*/

	Int ii;
	
        vo_cfg = vo_cfg_list;

        while(vo_cfg != NULL)
                {
                vo_id = GetVOConfigId(vo_cfg);

                vol_cfg = GetVOConfigLayers(vo_cfg);
                
                while(vol_cfg != NULL)
                        {
                        vol_id = GetVolConfigId(vol_cfg);
                        sep_mot_shape_text = 0;
#ifndef NLSSTATS
                        strcpy(num_bits[vo_id][vol_id].stats_file,stats_file);
#endif
                        num_bits[vo_id][vol_id].sep_mot_shape_text = sep_mot_shape_text;
                        num_bits[vo_id][vol_id].vol = 0;
                        num_bits[vo_id][vol_id].time = 0;
                        num_bits[vo_id][vol_id].vop = 0;
                        num_bits[vo_id][vol_id].syntax = 0;
                        num_bits[vo_id][vol_id].shape = 0;
                        num_bits[vo_id][vol_id].texture = 0;
			/*AM2PDAM
			for(i=0;i<MAX_NUM_SLICE_LOG;i++)
			num_bits[vo_id][vol_id].slice[i] = 0; Studio Proifle
			num_bits[vo_id][vol_id].macroblock = 0;
			*/
                        num_bits[vo_id][vol_id].sprite_piece = 0;
                        num_bits[vo_id][vol_id].motion = 0;
                        num_bits[vo_id][vol_id].mot_shape_text = 0;
                        num_bits[vo_id][vol_id].psnr_y = 0;
			for(ii=0;ii<MAX_AUX_COMP_NUM;ii++)
			  num_bits[vo_id][vol_id].psnr_g_st[ii] = 0;
                        num_bits[vo_id][vol_id].psnr_u = 0;
                        num_bits[vo_id][vol_id].psnr_v = 0;
                        num_bits[vo_id][vol_id].psnr_y_ave = 0;
			for(ii=0;ii<MAX_AUX_COMP_NUM;ii++)
			  num_bits[vo_id][vol_id].psnr_g_ave_st[ii] = 0;
                        num_bits[vo_id][vol_id].psnr_u_ave = 0;
                        num_bits[vo_id][vol_id].psnr_v_ave = 0;
                        num_bits[vo_id][vol_id].average = 0; 
                        num_bits[vo_id][vol_id].average_shape = 0; 
                        num_bits[vo_id][vol_id].occurrence = 0;

                        vol_cfg = GetVolConfigNext(vol_cfg);
                        }

                vo_cfg = GetVOConfigNext(vo_cfg);       
                }

        return;
}

Void 
AverageBitCounts_Studio(BitCount num_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
                                        VOConfig *vo_cfg_list)
{
        VOConfig                *vo_cfg;

        VolConfig       *vol_cfg;

        Int                     vo_id,vol_id;

	Int ii;

        vo_cfg = vo_cfg_list;

        while(vo_cfg != NULL)
                {
                vo_id = GetVOConfigId(vo_cfg);

                vol_cfg = GetVOConfigLayers(vo_cfg);

                while(vol_cfg != NULL)
                        {
                        vol_id = GetVolConfigId(vol_cfg);

                        num_bits[vo_id][vol_id].average = num_bits[vo_id][vol_id].average/num_bits[vo_id][vol_id].occurrence;

                        num_bits[vo_id][vol_id].average_shape = num_bits[vo_id][vol_id].average_shape/num_bits[vo_id][vol_id].occurrence;

                        num_bits[vo_id][vol_id].psnr_y_ave = num_bits[vo_id][vol_id].psnr_y_ave/num_bits[vo_id][vol_id].occurrence;
			for(ii=0;ii<MAX_AUX_COMP_NUM;ii++)/*AM2PDAM*/
			  num_bits[vo_id][vol_id].psnr_g_ave_st[ii] = num_bits[vo_id][vol_id].psnr_g_ave_st[ii]/num_bits[vo_id][vol_id].occurrence;
                        num_bits[vo_id][vol_id].psnr_u_ave = num_bits[vo_id][vol_id].psnr_u_ave/num_bits[vo_id][vol_id].occurrence;
                        num_bits[vo_id][vol_id].psnr_v_ave = num_bits[vo_id][vol_id].psnr_v_ave/num_bits[vo_id][vol_id].occurrence;

                        vol_cfg = GetVolConfigNext(vol_cfg);
                        }

                vo_cfg = GetVOConfigNext(vo_cfg);
                }

        return;
}

Int
BitstreamPutExtensionsInVOL_Studio(Vol *vol, Int vo_id, Int vol_id)
{
  Image *extension_bitstream;
  Int send_qmat=0;
  Int aux;
  Int *qmat;
  Int bits=0;
  Int i;

  /* Quant Matrix Extension */
  
  if(GetVolLoadIntraQuantMat(vol))
    send_qmat = 1;
  if(GetVolLoadNonintraQuantMat(vol))
    send_qmat = 1;
  if(GetVolLoadChromaIntraQuantMat_st(vol))
    send_qmat = 1;
  if(GetVolLoadChromaNonintraQuantMat_st(vol))
    send_qmat = 1;
  if(GetVolShape(vol)==GREY_SCALE) {
    for(aux=0;aux<GetVolAuxCompCount_st(vol);aux++) {
      if(GetVolLoadGrayIntraQuantMat_st(vol,aux))
	send_qmat = 1;
      if(GetVolLoadGrayNonintraQuantMat_st(vol,aux))
	send_qmat = 1;
    }
  }

  if(send_qmat) {

    extension_bitstream = BitstreamInit();
    
    /* extension_start_code */
    BitstreamPutBits(extension_bitstream,EXTENSION_START_CODE,32);

    /* extension_start_code_identifier */
    BitstreamPutBits(extension_bitstream,QUANT_MATRIX_EXTENSION_ID,4);

    /* load_intra_quantiser_matrix */
    BitstreamPutBits(extension_bitstream,GetVolLoadIntraQuantMat(vol),1);

    /* intra_quantiser_matrix */
    if(GetVolLoadIntraQuantMat(vol)) {
      qmat = GetVolIntraQuantMat(vol);
      for(i=0;i<64;i++)
	BitstreamPutBits(extension_bitstream,qmat[*(zigzag_i+i)],8);
    }

    /* load_nonintra_quantiser_matrix */
    BitstreamPutBits(extension_bitstream,GetVolLoadNonintraQuantMat(vol),1);

    /* nonintra_quantiser_matrix */
    if(GetVolLoadNonintraQuantMat(vol)) {
      qmat = GetVolNonintraQuantMat(vol);
      for(i=0;i<64;i++)
	BitstreamPutBits(extension_bitstream,qmat[*(zigzag_i+i)],8);
    }
    
    /* load_chroma_intra_quantiser_matrix */
    BitstreamPutBits(extension_bitstream,GetVolLoadChromaIntraQuantMat_st(vol),1);

    /* chroma_intra_quantiser_matrix */
    if(GetVolLoadChromaIntraQuantMat_st(vol)) {
      qmat = GetVolChromaIntraQuantMat_st(vol);
      for(i=0;i<64;i++)
	BitstreamPutBits(extension_bitstream,qmat[*(zigzag_i+i)],8);
    }

    /* load_chroma_nonintra_quantiser_matrix */
    BitstreamPutBits(extension_bitstream,GetVolLoadChromaNonintraQuantMat_st(vol),1);

    /* chroma_nonintra_quantiser_matrix */
    if(GetVolLoadChromaNonintraQuantMat_st(vol)) {
      qmat = GetVolChromaNonintraQuantMat_st(vol);
      for(i=0;i<64;i++)
	BitstreamPutBits(extension_bitstream,qmat[*(zigzag_i+i)],8);
    }

    if(GetVolShape(vol)==GREY_SCALE) {
      for(aux=0;aux<GetVolAuxCompCount_st(vol);aux++) {
	/* load_gray_intra_quantiser_matrix */
	BitstreamPutBits(extension_bitstream,GetVolLoadGrayIntraQuantMat_st(vol,aux),1);
	
	/* gray_intra_quantiser_matrix */
	if(GetVolLoadGrayIntraQuantMat_st(vol,aux)) {
	  qmat = GetVolGrayIntraQuantMat_st(vol,aux);
	  for(i=0;i<64;i++)
	    BitstreamPutBits(extension_bitstream,qmat[*(zigzag_i+i)],8);
	}
	
	/* load_gray_nonintra_quantiser_matrix */
	BitstreamPutBits(extension_bitstream,GetVolLoadGrayNonintraQuantMat_st(vol,aux),1);
	
	/* gray_nonintra_quantiser_matrix */
	if(GetVolLoadGrayNonintraQuantMat_st(vol,aux)) {
	  qmat = GetVolGrayNonintraQuantMat_st(vol,aux);
	  for(i=0;i<64;i++)
	    BitstreamPutBits(extension_bitstream,qmat[*(zigzag_i+i)],8);
	}
      }
    }

    bits += BitstreamPut(extension_bitstream,vo_id,vol_id);
    BitstreamFree(extension_bitstream);
    bits += NextStartCode_Studio(vo_id,vol_id,GetRandomValue_Studio(0,5));
    
  }/* Encode QMatrix */

}

/* 01.04.01: Studio Profile */
